#!/usr/bin/env python
"""
Top-level administrative script for the PRAMS_common package.

This program requires either Python version 2.6+ or 3.2+, and is importable as
a python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import sys
import os
import subprocess
import getopt
import shlex
import re

app_label = "PRAMS_common"

#------------------------------------------------------------------------------
def main(action):
    """Administrative task driver."""

    #=== Define some useful parameters:

    beg_cwd = os.getcwd()  # Record the current working directory

      # Determine fully-qualified filesystem location of this script:
    anchor_path = os.path.abspath(os.path.dirname(sys.argv[0]))
    os.chdir(anchor_path)   # Start from a "known" location

      # Frequently referenced parameters:

    local_infra_path = anchor_path
    version_fpath = os.path.join(local_infra_path, "etc", "version_info")

      # Further argument processing:

    arg_check_action = ["version", "clean", "build", "install", "query_build"]
    arg_check_clean = ["ALL"]
    arg_check_build = ["serial", "DM_only", "SM_only", "hybrid", "debug"]
    arg_check_unary = ["version", "query_build"]
    if action[0] in arg_check_action:   # Is the specified action valid?
        this_action = action[0]
        len_args_action_list = len(action)
        if this_action in arg_check_unary:   # Should not have extraneous args
            if len_args_action_list > 1:   
                usage()   # Disallow extraneous arguments
                sys.exit(1)
        elif this_action == "install":   # Needs one extra argument
            if len_args_action_list == 2:
                install_image_fpath = os.path.join(beg_cwd, action[1])
            else:
                usage()
                sys.exit(1)
        elif this_action == "clean":   # May have 0 or 1 extra argument
            clean_ALL = False   # Default
            for arg_item in action[1:]:
                if arg_item in arg_check_clean:  # Is the specified token valid?
                    clean_ALL = True
                else:                      # The specified token is invalid
                    usage()
                    sys.exit(1)
        elif action == "build":   # May have 0, 1, or 2 extra arguments
            if len_args_action_list > 1:
                for arg_item in action[1:]:
                    if arg_item in arg_check_build:  # Is the token valid?
                        pass
                    else:                      # The specified token is invalid
                        usage()
                        sys.exit(1)
    else:                         # The specified action is invalid
        usage
        sys.exit(1)

    #=== Perform specified action(s):

    if this_action == "version":
          # Read the application information file, parse its contents, and
          #  display the result(s):
        vid = {}
        with open(version_fpath, "rt") as version_f:
            version_f_raw = version_f.read()
        for pair in shlex.split(version_f_raw, True, True):
            split_pair = (pair.replace(';', ' ')).split('=', 1)
            vid[split_pair[0]] = split_pair[1]

        print(app_label+": "+vid["version"].strip())

    elif this_action == "clean":
          # Clean any build detritus that may be present:
        print("Cleaning "+app_label+" debris ...")

    elif this_action == "query_build":
        print(' ')     # Currently does nothing

    elif this_action == "build":
        print(' ')     # Currently does nothing

    elif this_action == "install":
          # Install the specified archive image:

        print("Installing "+app_label+" archive image ...")

        if not (os.path.isfile(install_image_fpath) or \
                os.path.islink(install_image_fpath)):
            print(app_label+" ERROR: Archive image file ("+ \
                  install_image_fpath+" was not found.")
            sys.exit(1)

        os.chdir(os.path.join(anchor_path, "..", ".."))  # Change to install dir

        install_image_bn = os.path.basename(install_image_fpath)
        tail_match = re.search("(\.[^s]*){1,2}$", install_image_bn)
        if tail_match:
            if tail_match.group(0) == ".tar.bz2":
                p1 = subprocess.Popen(["bzip2", "-dc", install_image_fpath], 
                                      stdout=subprocess.PIPE,
                                      universal_newlines=True)
                p2 = subprocess.Popen(["tar", "-xvf", '-'], stdin=p1.stdout,
                                      universal_newlines=True)
                p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
                output, err = p2.communicate()
            elif tail_match.group(0) == ".tar.gz":
                p1 = subprocess.Popen(["gzip", "-dc", install_image_fpath], 
                                  stdout=subprocess.PIPE,
                                  universal_newlines=True)
                p2 = subprocess.Popen(["tar", "-xvf", '-'], stdin=p1.stdout,
                                      universal_newlines=True)
                p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
                output, err = p2.communicate()
            elif tail_match.group(0) == ".tar":
                subprocess.check_call(["tar", "-xvf", install_image_fpath])
            elif tail_match.group(0) == ".zip":
                subprocess.check_call(["unzip", install_image_fpath])
        else:
            print(app_label+" ERROR: Archive image file ("+ \
                  install_image_fpath+" has an unsupported format.")
            sys.exit(1)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    usage_msg = """
usage: admin_script.py [-h]  action

Top-level administrative script for this package.  This program requires either
Python version 2.6+ or 3.2+, and is importable as a python module.

mandatory argument:
  action -- One of the following:
      version  :  output the version ID of this code package
      clean  :  (currently does nothing) Clean up local non-configure build
                process debris
      clean ALL  :  (currently does nothing) Clean up all known build process
                    debris, including that of configure and other linked
                    packages
      build [debug]  :  (currently does nothing) Build package
       " "  serial [debug]  :  (DEFAULT) all parallel features disabled
       " "  DM_only [debug]  :  distributed-memory parallel (MPI) only
       " "  SM_only [debug]  :  shared-memory parallel (OpenMP) only
       " "  hybrid [debug]  :  hybrid parallel (MPI + OpenMP)
      install FILE  :  Install a specified code package update
      query_build  :  Usage/linking info for the built package

optional arguments:
  -h, --help   show this help message and exit
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'h', ["help"])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    # NONE
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 3 or len(args) < 1):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)
    action = args

    main(action)
