#!/usr/bin/env python
"""
Provides a mechanism for creating a package image/archive for distribution.

This program requires either Python version 2.6+ or 3.2+, and is importable as a
python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import sys
import os
import subprocess
import getopt

app_label = "create_image"
associated_app = "PRAMS_common"

#------------------------------------------------------------------------------
# Emulates 'rm -f' functionality:
def rm_f(fpath):
    """
    Emulates 'rm -f' functionality.
    """

    if os.path.isfile(fpath) or os.path.islink(fpath):
        os.remove(fpath)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def main(anchor_path, version_ID, image_type):
    """Creates a package image/archive for distribution."""

    os.chdir(anchor_path)   # Start from a "known" location

    # Define some useful parameters:

    apptop_path = os.path.abspath(os.path.join(anchor_path, "..", ".."))
    i_tarlist_path = os.path.join(anchor_path, "image_tarlist")

    content_type_dict = dict( fs = ["source_code", "full"],
                              us = ["source_code", "update"],
                              fd = ["data", "full"],
                              ud = ["data", "update"] )

    # Determine important file pathnames:

    i_content = content_type_dict[image_type][0]
    i_type = content_type_dict[image_type][1]

    image_tarlist_fpath = os.path.join(i_tarlist_path,
                                "create_"+i_type+'-'+i_content+".image_tarlist")
    image_fpath = os.path.join(anchor_path, associated_app+'-'+ \
                          version_ID.strip()+'-'+image_type.strip()+"_dist.tar")

    # Make sure app_image file(s) do not already exist:
    if os.path.isfile(image_fpath) or os.path.islink(image_fpath):
        print(app_label+" ERROR: File ("+image_fpath+") already exists.")
        sys.exit(1)
    tmp_fpath = image_fpath+".bz2"
    if os.path.isfile(tmp_fpath) or os.path.islink(tmp_fpath):
        print(app_label+" ERROR: File ("+tmp_fpath+") already exists.")
        sys.exit(1)

    # Create app_image_info.include (using text encoding of current "locale"):

    image_info_fpath = os.path.join(apptop_path, "app_image_info.include")

    text_to_write = \
         "SPECIFIC_APPNAME="+associated_app+"\n\n"+ \
         "# Application version ID (that contained in this image):\n"+ \
         "IMAGE_APPVERSION="+version_ID+"\n\n"+ \
         "# Possible values: source_code, data\n"+ \
         "IMAGE_CONTENT="+i_content+"\n\n"+ \
         "# Possible values: full, update\n"+ \
         "IMAGE_TYPE="+i_type+"\n"

    with open (image_info_fpath, "wt") as i_info_f:
        i_info_f.write(text_to_write)

    # Create tmp.image_tarlist (using text encoding of current "locale"):
    image_tarltmp_fpath = os.path.join(anchor_path, "tmp.image_tarlist")
    with open (image_tarltmp_fpath, "wt") as i_info_f:
        with open (image_tarlist_fpath, "rt") as input_f:
            input_f_line_list = input_f.readlines()
        for line in input_f_line_list:
            trimmed_line = line.strip()
            text_to_write = os.path.join(os.path.basename(apptop_path), 
                                         trimmed_line)+"\n"
            i_info_f.write(text_to_write)

    # Create image archive:

    beg_cwd = os.getcwd()  # Record the current working directory
    os.chdir(os.path.abspath(os.path.join(apptop_path, "..")))

    try:
        subprocess.check_call(["tar", "cvTf", image_tarltmp_fpath, image_fpath])
        print("Compressing archive ...")
        subprocess.check_call(["bzip2", image_fpath])
    except:
        print(app_label+" ERROR: tar or bzip2 operation failed.")
        rm_f(image_info_fpath)   # Clean up
        rm_f(image_tarltmp_fpath)
        sys.exit(1)   # Abnormal termination
    else:      # Action finished, clean up
        rm_f(image_info_fpath)
        rm_f(image_tarltmp_fpath)

    # Return to the initial directory:
    os.chdir(beg_cwd)

    print(app_label+ \
           ": Finished application image archive creation successfully.")
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    usage_msg = """
usage: create_image.py [-h]  version_ID  image_type

Create a package image/archive for distribution.

mandatory arguments:
  version_ID   The version ID of the image/archive (e.g., 1.0_r3).
  image_type   The image/archive type:
                  fs = full source code
                  us = update source code (a subset of source files)
                  fd = full data
                  ud = update data (a subset of data files)

optional arguments:
  -h, --help   show this help message and exit
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Determine fully-qualified filesystem location of this script:
    anchor_path = os.path.abspath(os.path.dirname(sys.argv[0]))

    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'h', ["help"])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    # NONE
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 2 or len(args) < 2):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)
    version_ID = args[0]
    image_type = args[1]

    # Further argument processing:
    arg_check_image_type = ["fs", "us", "fd", "ud"]
    if image_type not in arg_check_image_type:  # Is the specified token valid?
        usage()
        sys.exit(1)

    # Run driver:
    main(anchor_path, version_ID, image_type)
