!```````````````````````````````````````````````````````````````````
module PIPP_format_mod

!+ Defines and deals with the PIPP datafile format
!
!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

use univ_kind_defs_mod, only: f4

implicit none

private

! Normally only write one PIPP file at a time (nf=1), but is sometimes useful
!  to set nf=n for reading:

integer, public :: nf    

! nf .... Number of PIPP files to be read/written
!
! nt ... array(nf); number of times in file
! nx0 ... array(nf); grid points in x-dir
! ny0 ... array(nf); grid points in y-dir
! nv ... array(nf); number of variables per time
!
! dstring ...array(nf); data description string 
! lsubs_ts ... array(nt,nf); values of Ls for each time in data
! utc_ts ... array(nt,nf); UTC times (Mars-hours)
! sol_ts ... array(nt,nf); sol # (starts at 1 for the first timestep)
! nz0 ... array(nv,nf); vertical levels of each variable
!
! vtype ... array(nv,nf); type of variable
!                      1 = 3D atm variable
!                      2 = 2D atm variable
!                      3 = 3D sfc variable
!                      4 = 2D sfc variable
!
! lat ... array(nx0,ny0,nf); latitude at each grid point
! lon ... array(nx0,ny0,nf); longitude at each grid point
! topo ... array(nx0,ny0,nf); MGCM topography at each grid point

integer, public, allocatable, dimension(:) :: nt,nx0,ny0,nv
character(len=32), public, allocatable, dimension(:) :: dstring
integer, public, allocatable, dimension(:,:) :: sol_ts,nz0,vtype
real(f4), public, allocatable, dimension(:,:) :: lsubs_ts,utc_ts
real(f4), public, allocatable, dimension(:,:,:) :: lat,lon,topo

! PIPP_data ... array(variable size); holds a 4D data cube (nx0,ny0,nz0,nv)
! PIPP_idx ... array(nv,nf); array indices for PIPP_data

real(f4), public, allocatable, dimension(:) :: PIPP_data
integer, public, allocatable, dimension(:,:) :: PIPP_idx

! dsize4d ... array(nf); max total size of a timestep's data (PIPP_data dim)
! rec_size ... array(nf); direct access record size (4-byte words)

integer, public, allocatable, dimension(:) :: dsize4d,rec_size

interface PIPPf_w_fill
   module procedure PIPPf_w_fill_1d
   module procedure PIPPf_w_fill_2d
   module procedure PIPPf_w_fill_3d
end interface

interface PIPPf_r_fill
   module procedure PIPPf_r_fill_1d
   module procedure PIPPf_r_fill_2d
   module procedure PIPPf_r_fill_3d
end interface

public :: PIPPf_Nmalloc,PIPPf_Omalloc,PIPPf_dealloc,PIPPf_w_head,PIPPf_r_head  &
     ,PIPPf_w_fill,PIPPf_r_fill,PIPPf_w_data,PIPPf_r_data

contains


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_Nmalloc(nti,nxi,nyi,nvi,nzi)

!+

implicit none

!=== Argument declarations:

integer, intent(IN) :: nti,nxi,nyi,nvi
integer, dimension(nvi), intent(IN) :: nzi

!== Local declarations:

integer :: len2d,iv,sum4d

!=== Executable statements:

allocate(nz0(nvi,1),vtype(nvi,1),  &
     lsubs_ts(nti,1),utc_ts(nti,1),sol_ts(nti,1),  &
     lat(nxi,nyi,1),lon(nxi,nyi,1),  &
     topo(nxi,nyi,1),PIPP_idx(nvi,1))

allocate(nt(1),nx0(1),ny0(1),nv(1),dstring(1)  & ! Only need 1 for file write
     ,rec_size(1),dsize4d(1))

len2d = nxi*nyi
rec_size(1) = len2d

sum4d = 0
do iv=1,nvi
   PIPP_idx(iv,1) = sum4d+1    ! Possible prec_reduct here
   sum4d = sum4d+len2d*nzi(iv)
   nz0(iv,1) = nzi(iv)    ! Possible prec_reduct here
enddo

allocate(PIPP_data(sum4d))
dsize4d(1) = sum4d

nx0(1) = nxi    ! Possible prec_reduct here
ny0(1) = nyi    ! Possible prec_reduct here
nt(1) = nti     ! Possible prec_reduct here
nv(1) = nvi     ! Possible prec_reduct here

end subroutine PIPPf_Nmalloc


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_Omalloc(pass,tnf)

!+

implicit none

!=== Argument declarations:

integer, intent(IN) :: pass,tnf

!== Local declarations:

integer :: it,iv,sum,itmp,jtmp,max4d

!=== Executable statements:

select case (pass)

case (0)

   nf = tnf

   allocate(nt(tnf),nx0(tnf),ny0(tnf),nv(tnf),dstring(tnf)  &
        ,rec_size(tnf),dsize4d(tnf))

case (1)
        
   itmp = maxval(nv)   ! Max nv in any of the given PIPP files
   allocate(nz0(itmp,tnf),vtype(itmp,tnf),PIPP_idx(itmp,nf))
   
   itmp = maxval(nt)   ! Max nt in any of the given PIPP files
   allocate(lsubs_ts(itmp,tnf),utc_ts(itmp,tnf),sol_ts(itmp,tnf))
   
   itmp = maxval(nx0)   ! Max nx in any of the given PIPP files
   jtmp = maxval(ny0)   ! Max ny in any of the given PIPP files
   allocate(lat(itmp,jtmp,tnf),lon(itmp,jtmp,tnf),topo(itmp,jtmp,tnf))
   
   do it=1,tnf
      rec_size(it) = nx0(it)*ny0(it)    ! Possible prec_increase here
   enddo
   
case (2)
   
   max4d = 0
   
   do it=1,tnf
      sum = 0
      do iv=1,nv(it)
         sum = sum+rec_size(it)*nz0(iv,it)
      enddo
      max4d = max(max4d,sum)     ! Max 4D memory needed by any file
      dsize4d(it) = sum   ! 4D memory needed by THIS file
   enddo

   allocate(PIPP_data(max4d))   ! Allocate memory to hold PIPP data

end select
  
end subroutine PIPPf_Omalloc


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_dealloc

!+

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

deallocate(nz0,vtype,lsubs_ts,utc_ts,sol_ts,lat,lon,topo,PIPP_idx  &
     ,PIPP_data,nt,nx0,ny0,nv,dstring,rec_size,dsize4d)

end subroutine PIPPf_dealloc


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_w_head(hdrfn)

!+

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: hdrfn

!== Local declarations:

! NONE

!=== Executable statements:

open(73,file=hdrfn(1:len_trim(hdrfn)),status='unknown',form='unformatted')

write (73) dstring(1)               ! Description
write (73) nx0(1),ny0(1),nv(1),nt(1)     ! Dimensions
write (73) nz0,vtype,PIPP_idx     ! Arrays(nv)
write (73) lsubs_ts,utc_ts,sol_ts      ! Arrays(nt)
write (73) lat,lon,topo          ! Arrays(nx0,ny0)

close(73)  ! Close header file

end subroutine PIPPf_w_head


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_r_head(hdrfn,tnfiles)

!+

implicit none

!=== Argument declarations:

integer, intent(IN) :: tnfiles    ! Total number of PIPP files to be read in
character(len=*), dimension(tnfiles), intent(IN) :: hdrfn  ! PIPP header filename(s)

!== Local declarations:

integer :: it

!=== Executable statements:

call PIPPf_Omalloc(0,tnfiles)   ! Allocate crucial arrays

do it=1,tnfiles   ! Loop over number of files to be read

   open(73,file=hdrfn(it)(1:len_trim(hdrfn(it))),status='old'  &
        ,form='unformatted')
   read (73) dstring(it)               ! Description
   read (73) nx0(it),ny0(it),nv(it),nt(it)     ! Dimensions
   close(73)

enddo

call PIPPf_Omalloc(1,tnfiles)   ! Allocate simple arrays

do it=1,tnfiles

   open(73,file=hdrfn(it)(1:len_trim(hdrfn(it))),status='old'  &
        ,form='unformatted')

   read (73) dstring(it)               ! Description          ! Read over
   read (73) nx0(it),ny0(it),nv(it),nt(it)     ! Dimensions   !  again
     
   read (73) nz0(1:nv(it),it),vtype(1:nv(it),it),PIPP_idx(1:nv(it),it)
   read (73) lsubs_ts(1:nt(it),it),utc_ts(1:nt(it),it),sol_ts(1:nt(it),it)
   read (73) lat(1:nx0(it),1:ny0(it),it),lon(1:nx0(it),1:ny0(it),it)  &
        ,topo(1:nx0(it),1:ny0(it),it)
 
   close(73)  ! Close header file

enddo

call PIPPf_Omalloc(2,tnfiles)   ! Allocate 4D array

end subroutine PIPPf_r_head


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_w_fill_1d(fill,iv)

!+

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: iv
real(f4), dimension(:), intent(IN) :: fill

!== Local declarations:

integer :: np,start

!=== Executable statements:

start = PIPP_idx(iv,1)-1    ! Possible prec_increase here
np = nx0(1)*ny0(1)*nz0(iv,1)

PIPP_data(1+start:np+start) = fill(1:np)    ! Possible prec_reduct here

end subroutine PIPPf_w_fill_1d


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_r_fill_1d(fill,iv,fnum)

!+

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: iv, fnum

real(f4), dimension(:), intent(INOUT) :: fill

!== Local declarations:
  
integer :: np, start

!=== Executable statements:

start = PIPP_idx(iv,fnum)-1   ! Possible prec_increase here
np = nx0(fnum)*ny0(fnum)*nz0(iv,fnum)

fill(1:np) = PIPP_data(1+start:np+start)    ! Possible prec_increase here

end subroutine PIPPf_r_fill_1d


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_w_fill_2d(fill,iv)

!+

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: iv
real(f4), dimension(:,:), intent(IN) :: fill

!== Local declarations:

integer :: i, j, k, start, iq

!=== Executable statements:

start = PIPP_idx(iv,1)-1    ! Possible prec_increase here

iq = 0
!do k=1,nz0(iv,1)
   do j=1,ny0(1)
      do i=1,nx0(1)
         iq = iq+1
         PIPP_data(iq+start) = fill(i,j)     ! Possible prec_reduct here
      end do
   end do
!end do

end subroutine PIPPf_w_fill_2d


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_r_fill_2d(fill,iv,fnum)

!+

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: iv, fnum

real(f4), dimension(:,:), intent(INOUT) :: fill

!== Local declarations:
  
integer :: i, j, k, start, iq

!=== Executable statements:

start = PIPP_idx(iv,fnum)-1   ! Possible prec_increase here

iq = 0
!do k=1,nz0(iv,fnum)
   do j=1,ny0(fnum)
      do i=1,nx0(fnum)
         iq = iq+1
         fill(i,j) = PIPP_data(iq+start)    ! Possible prec_increase here
      end do
   end do
!end do

end subroutine PIPPf_r_fill_2d


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_w_fill_3d(fill,iv)

!+

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: iv
real(f4), dimension(:,:,:), intent(IN) :: fill

!== Local declarations:

integer :: i, j, k, start, iq

!=== Executable statements:

start = PIPP_idx(iv,1)-1    ! Possible prec_increase here

iq = 0
do k=1,nz0(iv,1)
   do j=1,ny0(1)
      do i=1,nx0(1)
         iq = iq+1
         PIPP_data(iq+start) = fill(i,j,k)     ! Possible prec_reduct here
      end do
   end do
end do

end subroutine PIPPf_w_fill_3d


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_r_fill_3d(fill,iv,fnum)

!+

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: iv, fnum

real(f4), dimension(:,:,:), intent(INOUT) :: fill

!== Local declarations:
  
integer :: i, j, k, start, iq

!=== Executable statements:

start = PIPP_idx(iv,fnum)-1   ! Possible prec_increase here

iq = 0
do k=1,nz0(iv,fnum)
   do j=1,ny0(fnum)
      do i=1,nx0(fnum)
         iq = iq+1
         fill(i,j,k) = PIPP_data(iq+start)    ! Possible prec_increase here
      end do
   end do
end do

end subroutine PIPPf_r_fill_3d


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_w_data(dunit,it)

!+

implicit none

!=== Argument declarations:

integer, intent(IN) :: dunit,it

!== Local declarations:

integer :: rec_start,rec_cube,ir,ird

!=== Executable statements:

rec_cube = dsize4d(1)/rec_size(1)
rec_start = (it-1)*rec_cube+1

ird = 1
do ir=rec_start,rec_start+rec_cube-1
   write (dunit,rec=ir) PIPP_data(ird:ird+rec_size(1)-1)
   ird = ird+rec_size(1)
enddo

end subroutine PIPPf_w_data


!```````````````````````````````````````````````````````````````````
subroutine PIPPf_r_data(dunit,it,fnum)

!+

implicit none

!=== Argument declarations:

integer, intent(IN) :: dunit,it,fnum

!== Local declarations:

integer :: rec_start,rec_cube,ir,ird

!=== Executable statements:

rec_cube = dsize4d(fnum)/rec_size(fnum)
rec_start = (it-1)*rec_cube+1

ird = 1
do ir=rec_start,rec_start+rec_cube-1
   read (dunit,rec=ir) PIPP_data(ird:ird+rec_size(fnum)-1)
   ird = ird+rec_size(fnum)
enddo

end subroutine PIPPf_r_data


end module PIPP_format_mod
