!```````````````````````````````````````````````````````````````````
module PIPP_in_mod

!+ Contains routines that extract information from input datafile(s)
!
!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine ingest_ts(iunit, tstep, fnum, nd1, nd2, nd3, nd4, nd3_SBS, gcm,  &
     gt, gt_SBS, topoMG, latMG, lonMG, wet_GCM)

!+ Read PIPP format GCM data (produced by PIPP_prep routines)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_convert_units_mod, only: kg_m3_to_g_cm3_f4, um_to_m_f4
use PIPP_format_mod, only: topo,lat,lon,PIPPf_r_data,PIPPf_r_fill
use main_memory_bspec_mod, only: H2Ov_tog, dust_tog, dust_nb, dust_igd
use aermic_rad_mem_mod, only: aer_mdist_bm2n,aer_mdist_bfrac
use aemic_drvint_mod, only: aer_init_mdist_logn

implicit none

!=== Argument declarations:

integer, intent(IN) :: nd1,nd2,nd3,nd4,iunit,tstep,fnum,nd3_SBS
logical, intent(IN) :: wet_GCM

real(f4), dimension(nd1+1,nd2,nd3,nd4), intent(OUT) :: gcm
real(f4), dimension(nd1+1,nd2), intent(OUT) :: gt,topoMG,latMG,lonMG
real(f4), dimension(nd1+1,nd2,nd3_SBS), intent(OUT) :: gt_SBS

!== Local declarations:

real(f4), allocatable, dimension(:,:,:) :: dust_mc, H2Oi_mc

integer :: ipu,ipv,ipt,ipz,ipr,ipp,i,j,k,ib
real(f4) :: aer_numconc, aer_massconc, dust_geom_mnrad0, z_abs

!=== Executable statements:

ipu = 1            !
ipv = 2            !
ipt = 3            ! Set indices to various variables within `gcm` array
ipz = 4            !
ipr = 5            !
ipp = 6            !

! Initialize storage array with zeroes

gcm(:,:,:,:) = ZERO_f4

! Fill data arrays from the chosen GCM dataset:

call PIPPf_r_data(iunit,tstep,fnum)   ! Read in current timestep's 4D data cube

call PIPPf_r_fill(gcm(1:nd1,:,:,ipt),1,fnum)  ! Temp [K]
call PIPPf_r_fill(gcm(1:nd1,:,:,ipp),2,fnum)   ! Pressure [Pa]
call PIPPf_r_fill(gcm(1:nd1,:,:,ipu),3,fnum)  ! U-wind [m/s]
call PIPPf_r_fill(gcm(1:nd1,:,:,ipv),4,fnum)   ! V-wind [M/s]

! Allocate some temporary memory:

if (wet_GCM) then
   call PIPPf_r_fill(gcm(1:nd1,:,:,ipz),5,fnum)  ! Bodypotential height [m]
   call PIPPf_r_fill(gt(1:nd1,:),11,fnum)  ! Sfc T [K] or ice mass [-kg/m^2]
   call PIPPf_r_fill(gt_SBS(1:nd1,:,:),10,fnum)  ! SBS temp [K]

   if(H2Ov_tog > 0) then
      allocate(H2Oi_mc(nd1,nd2,nd3))

      call PIPPf_r_fill(H2Oi_mc,8,fnum)   ! H2O_ice mass conc [kg/m^3]
      call PIPPf_r_fill(gcm(1:nd1,:,:,ipr),6,fnum) ! H2Ovap mconc [kg/m^3]
      
      ! Add water ice aerosol mass to water vapor mass; convert from 
      !  [kg/m^3] -> [g/cm^3]; bound values: 
      gcm(1:nd1,:,:,ipr) = (max(ZERO_f4,gcm(1:nd1,:,:,ipr))+  &
           max(ZERO_f4,H2Oi_mc(:,:,:)))*kg_m3_to_g_cm3_f4

      deallocate(H2Oi_mc)
   endif

   if(dust_tog > 0) then

      allocate(dust_mc(nd1,nd2,nd3))

      call PIPPf_r_fill(dust_mc,7,fnum)     ! Dust mass conc [kg/m^3]

      ! Convert from [kg/m^3] -> [g/cm^3]: 
      dust_mc = dust_mc*kg_m3_to_g_cm3_f4  ! Entire array at once

      do k=1,nd3
         do j=1,nd2
            do i=1,nd1

               dust_mc(i,j,k) = max(ZERO_f4,dust_mc(i,j,k))

               z_abs = gcm(i,j,k,ipz)+topo(i,j,fnum)
               if(z_abs > 49000._f4) then                  !
                  dust_geom_mnrad0 = 0.3_f4                !
               elseif(z_abs < ZERO_f4) then                  ! Bound values
                  dust_geom_mnrad0 = 3.3_f4                !
               else
                  dust_geom_mnrad0 = 2._f4-sin((z_abs-18000._f4)/20000._f4)*  &
                       1.7_f4
               endif                                    !
               dust_geom_mnrad0 = dust_geom_mnrad0*um_to_m_f4  ! [um -> m]

               call aer_init_mdist_logn(dust_geom_mnrad0  &
                    ,dust_igd,1,dust_nb,1) ! Use namelist dust distro stddev

               do ib=1,dust_nb
                  aer_massconc = aer_mdist_bfrac(1,ib)*dust_mc(i,j,k)
                  aer_numconc = aer_massconc*aer_mdist_bm2n(1,ib)
                  gcm(i,j,k,ib+6) = aer_numconc     ! [#/cm^3]

               enddo

            enddo
         enddo
      enddo

      deallocate(dust_mc)

   end if

else          ! "Dry" GCM run
   call PIPPf_r_fill(gcm(1:nd1,:,:,ipz),5,fnum)  ! Geopotential height [m]
   call PIPPf_r_fill(gt(1:nd1,:),6,fnum)      ! Sfc temp [k] or ice mass [-kg]
   call PIPPf_r_fill(gt_SBS(1:nd1,:,:),7,fnum)  ! SBS temp [K]
!%   gcm(:,:,:,ipr) = 0.            ! Water mixing ratio (NO water currently)
endif

gcm(nd1+1,:,:,:) = gcm(1,:,:,:)   ! Wrap GCM data around in longitude
gt(nd1+1,:) = gt(1,:)               !
gt_SBS(nd1+1,:,:) = gt_SBS(1,:,:)   !
             
! Fill GCM topography array from datafile values:

topoMG(1:nd1,:) = topo(:,:,fnum)
topoMG(nd1+1,:) = topoMG(1,:)   ! Wrap in longitude

! Fill GCM lat/lon arrays from datafile values:

latMG(1:nd1,:) = lat(:,:,fnum)
lonMG(1:nd1,:) = lon(:,:,fnum)
latMG(nd1+1,:) = latMG(1,:)   ! Wrap in longitude
lonMG(nd1+1,:) = -lonMG(1,:)   !

end subroutine ingest_ts


end module PIPP_in_mod
