!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
module PIPP_intrp_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine vinterp(m1, m2, m3, nd1, nd2, nd3, nd4, topoMGr, topt, dati, dato,  &
     dattmp, rtgt, igd, wet_GCM)

!+ Vertical interpolation driver

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: gd
use main_memory_bspec_mod, only: dust_nb, dust_tog

implicit none

!=== Argument declarations:

logical, intent(IN) :: wet_GCM
integer, intent(IN) :: m1, m2, m3, nd1, nd2, nd3, nd4, igd
real(f4), dimension(m1,m2), intent(IN) :: topt, rtgt

real(f4), dimension(m1,m2), intent(INOUT) :: topoMGr
real(f4), dimension(m1,m2,nd3,nd4), intent(INOUT) :: dati, dattmp

real(f4), dimension(m1,m2,m3,nd4), intent(OUT) :: dato

!== Local declarations:

integer, parameter :: cmplxsw = 1, cutoff = 9

real(f4) :: topoq1, topoq2, zfirst
integer :: kpu, kpv, kpt, kpz, kpr, kps, jtgt, itgt, isclr_b,isclr_e, naer_sclr

!=== Executable statements:

! *** Set variable indices (nvar value of arrays):

kpu = 1
kpv = 2
kpt = 3
kpz = 4
kpr = 5
kps = 6           

!*** If PRAMS topography is zero everywhere (billiard ball), make GCM
!    topography flat as well:

topoq1 = minval(topt(:,:))
topoq2 = maxval(topt(:,:))
if (topoq1 <= 1.e-5_f4 .and. topoq1 >= -1.e-5_f4 .and.  &
     topoq2 <= 1.e-5_f4 .and. topoq2 >= -1.e-5_f4) then
   topoMGr(:,:) = ZERO_f4
end if

!*** Vertically interpolate each data column (on PRAMS grid) separately:

if (wet_GCM) then
   if (dust_tog > 0) then
      isclr_b = 7
      isclr_e = dust_nb+6
      naer_sclr = dust_nb
   else
      isclr_b = kpr
      isclr_e = kpr
      naer_sclr = 1
   end if
else   ! "Dry" GCM
   isclr_b = kpr
   isclr_e = kpr
   naer_sclr = 1
end if

do jtgt=1,m2
   do itgt=1,m1

      !*** Now intelligently adjust T, scalars, and gpZ vertically:

      call tz_vadj(cmplxsw, itgt, jtgt, m1, m2, nd3, topoMGr,  &
           dati(:,:,:,kpt), dati(:,:,:,kpz), dati(:,:,:,kpr),  & 
           dati(:,:,:,isclr_b:isclr_e), dattmp(:,:,:,kpt), dattmp(:,:,:,kpz),  &
           dattmp(:,:,:,kpr), dattmp(:,:,:,isclr_b:isclr_e), topt, cutoff,  &
           zfirst, wet_GCM, naer_sclr)

      !*** Intelligently adjust u and v in the vertical:

      call uv_vadj1(cmplxsw, itgt, jtgt, m1, m2, nd3, topoMGr,  &
           dati(:,:,:,kpu), dati(:,:,:,kpv), dattmp(:,:,:,kpu),  &
           dattmp(:,:,:,kpv), topt)

      !*** Interpolate all variables to sigmaZ levels:

      call vadj2mr(m1, m2, m3, nd3, itgt, jtgt, dattmp(:,:,:,kpu),  &
           dattmp(:,:,:,kpv), dattmp(:,:,:,kpt), dattmp(:,:,:,kpz),  &
           dattmp(:,:,:,kpr), dattmp(:,:,:,isclr_b:isclr_e), dato(:,:,:,kpu),  &
           dato(:,:,:,kpv), dato(:,:,:,kpt), dato(:,:,:,kpz),  &
           dato(:,:,:,kpr), dato(:,:,:,isclr_b:isclr_e), topt, rtgt,  &
           gd(igd)%zt, wet_GCM, naer_sclr)

      !*** Adjust near-surface profiles of u and v:

      call uv_vadj2(cmplxsw, m1, m2, m3, itgt, jtgt, dato(:,:,:,kpu),  &
           dato(:,:,:,kpv), dato(:,:,:,kpz), topt, zfirst)

      !*** Integrate p hydrostatically to sigmaZ levels:

      call p_vadj(m1, m2, m3, nd3, itgt, jtgt, dati(:,:,:,kpt),  &
           dati(:,:,:,kpz), dati(:,:,:,kps), dato(:,:,:,kpt),  &
           dato(:,:,:,kpz), dato(:,:,:,kps), gd(igd)%dzm, rtgt)

   end do
end do

end subroutine vinterp


!```````````````````````````````````````````````````````````````````
subroutine tz_vadj(meth, itgt, jtgt, m1, m2, n3, topoMGr, t_i, z_i, rv_i,  &
     aer_sclr_i, t_o, z_o, rv_o, aer_sclr_o, topt, cutoff, zfirst, wet_GCM,  &
     naer_sclr)

!+ Adjusts temperature and geopotential height (in the vertical) 

use univ_kind_defs_mod, only: f4
use body_specific_constants_mod, only: radius
use main_memory_bspec_mod, only: dust_tog, H2Ov_tog

implicit none

!=== Argument declarations:

logical, intent(IN) :: wet_GCM
integer, intent(IN) :: meth, itgt, jtgt, m1, m2, n3, cutoff, naer_sclr
real(f4), dimension(m1,m2,n3), intent(IN) :: t_i, rv_i
real(f4), dimension(m1,m2,n3,naer_sclr), intent(IN) :: aer_sclr_i
real(f4), dimension(m1,m2), intent(IN) :: topoMGr, topt

real(f4), dimension(m1,m2,n3), intent(INOUT) :: z_i

real(f4), intent(OUT) :: zfirst
real(f4), dimension(m1,m2,n3), intent(OUT) :: t_o, z_o, rv_o
real(f4), dimension(m1,m2,n3,naer_sclr), intent(OUT) :: aer_sclr_o

!== Local declarations:

real(f4) :: zadj, zwarn, lapse_t, lapse_rv, tadj, dz
integer :: k, cnt, kstar, ib
real(f4), dimension(naer_sclr) :: lapse_aer

!=== Executable statements:

!*** If meth=0, vertically adjust entire profile;
!    meth=1, vertically adjust ONLY profile below level `cutoff+1`, except
!            where terrain difference is extreme (adjusted differently)

interpolation_method: select case(meth)

case(0)       ! Simply adjust entire profile - will work always 

   zfirst = z_i(itgt,jtgt,1)  ! Thickness of first layer (in ~z)

   ! First add surface geopotential value to values:

   do k=1,n3
      zadj = z_i(itgt,jtgt,k)+topoMGr(itgt,jtgt) ! Full geopotential height
!      z_i(itgt,jtgt,k) = radius*zadj/(radius-zadj)  ! Convert to geometric height
      z_i(itgt,jtgt,k) = zadj
   end do

!*** zadj = (PRAMS topo + GCM sfc layer depth) - GCM gz at lowest level
!      (really just overkill -- could be simpler)
   zadj = topt(itgt,jtgt)+zfirst-z_i(itgt,jtgt,1)

!   do k=1,n3
!      z_o(itgt,jtgt,k) = z_i(itgt,jtgt,k)+zadj    ! F90 version below
!      t_o(itgt,jtgt,k) = t_i(itgt,jtgt,k)
!   enddo
   z_o(itgt,jtgt,:) = z_i(itgt,jtgt,:)+zadj
   t_o(itgt,jtgt,:) = t_i(itgt,jtgt,:)
   if (wet_GCM) then
      if (H2Ov_tog > 0) then
         rv_o(itgt,jtgt,:) = rv_i(itgt,jtgt,:)
      end if
      if (dust_tog > 0) then
         do ib=1,naer_sclr
            aer_sclr_o(itgt,jtgt,:,ib) = aer_sclr_i(itgt,jtgt,:,ib)
         end do
      end if
   end if

case(1)       ! More complex adjustment scheme - preserves upper air levels

   zfirst = z_i(itgt,jtgt,1)  ! Thickness of first layer (in ~z)

   ! First add surface geopotential value to values:

   do k=1,n3
      zadj = z_i(itgt,jtgt,k)+topoMGr(itgt,jtgt) ! Full geopotential height
!      z_i(itgt,jtgt,k) = radius*zadj/(radius-zadj)  ! Convert to geometric height
      z_i(itgt,jtgt,k) = zadj
   end do

!*** zwarn = dz between GCM levels `cutoff+1` and `cutoff`
   zwarn = z_i(itgt,jtgt,cutoff+1)-z_i(itgt,jtgt,cutoff)

!*** zadj = (PRAMS topo + GCM sfc layer depth) - GCM gz at lowest level
!      (really just overkill -- could be simpler)
   zadj = topt(itgt,jtgt)+zfirst-z_i(itgt,jtgt,1)

!*** Fill output arrays for upper levels (with no adjustment):

!   do k=n3,cutoff+1,-1
!      z_o(itgt,jtgt,k) = z_i(itgt,jtgt,k)    ! F90 version below
!      t_o(itgt,jtgt,k) = t_i(itgt,jtgt,k)
!   enddo
   z_o(itgt,jtgt,n3:cutoff+1:-1) = z_i(itgt,jtgt,n3:cutoff+1:-1)
   t_o(itgt,jtgt,n3:cutoff+1:-1) = t_i(itgt,jtgt,n3:cutoff+1:-1)
   if (wet_GCM) then
      if (H2Ov_tog > 0) then      
         rv_o(itgt,jtgt,n3:cutoff+1:-1) = rv_i(itgt,jtgt,n3:cutoff+1:-1)
      end if
      if (dust_tog > 0) then
         do ib=1,naer_sclr
            aer_sclr_o(itgt,jtgt,n3:cutoff+1:-1,ib) =  &
                 aer_sclr_i(itgt,jtgt,n3:cutoff+1:-1,ib)
         end do
      end if
   end if

!*** Fill output z array for lower levels (with simple adjustment):

!   do k=cutoff,1,-1
!      z_o(itgt,jtgt,k) = z_i(itgt,jtgt,k) + zadj   ! F90 version below
!   enddo
   z_o(itgt,jtgt,1:cutoff:1) = z_i(itgt,jtgt,1:cutoff:1)+zadj

!*** Mission now complete if terrain difference not too large; otherwise:

!$   if(abs(zadj) >= (0.90*zwarn)) then
!$      write (*,'(a,i3,3x,i3,1x,f13.6)')  &    ! Warn user
!$          'WARNING: Large vertical adjustment being made --',itgt,jtgt,zadj

   !*** Find first GCM level above adjusted `cutoff` level height:

   do k=cutoff+1,n3     ! Go upwards through GCM levels
      if (z_i(itgt,jtgt,k) > z_o(itgt,jtgt,cutoff)) then
         kstar = k
         exit
      end if
   end do

   !*** Lapse rate in original GCM profile (layer top is level `kstar`)

   dz = z_i(itgt,jtgt,kstar)-z_i(itgt,jtgt,kstar-1)
   lapse_t = (t_i(itgt,jtgt,kstar)-t_i(itgt,jtgt,kstar-1))/dz
   if (wet_GCM) then
      if (H2Ov_tog > 0) then   
         lapse_rv = (rv_i(itgt,jtgt,kstar)-rv_i(itgt,jtgt,kstar-1))/dz
      end if
      if (dust_tog > 0) then
         do ib=1,naer_sclr
            lapse_aer(ib) = (aer_sclr_i(itgt,jtgt,kstar,ib)-  &
                 aer_sclr_i(itgt,jtgt,kstar-1,ib))/dz
         end do
      end if
   end if

   !*** Interpolate GCM temperature at new level `cutoff`

   t_o(itgt,jtgt,cutoff) = (z_o(itgt,jtgt,cutoff)-  &
        z_i(itgt,jtgt,kstar-1))*lapse_t+t_i(itgt,jtgt,kstar-1)
   if (wet_GCM) then
      if (H2Ov_tog > 0) then   
         rv_o(itgt,jtgt,cutoff) = (z_o(itgt,jtgt,cutoff)-  &
              z_i(itgt,jtgt,kstar-1))*lapse_rv+rv_i(itgt,jtgt,kstar-1)
      end if
      if (dust_tog > 0) then
         do ib=1,naer_sclr
            aer_sclr_o(itgt,jtgt,cutoff,ib) = (z_o(itgt,jtgt,cutoff)-  &
                 z_i(itgt,jtgt,kstar-1))*lapse_aer(ib)+  &
                 aer_sclr_i(itgt,jtgt,kstar-1,ib)
         end do
      end if
   end if
   
   !*** Adjust temperatures (in lower levels) by temperature difference at
   !    new and old `cutoff` levels:

   tadj = t_i(itgt,jtgt,cutoff)-t_o(itgt,jtgt,cutoff)
!      do k=cutoff-1,1,-1
!         t_o(itgt,jtgt,k) = t_i(itgt,jtgt,k) - tadj   ! F90 version below
!      enddo
   t_o(itgt,jtgt,1:cutoff-1:1) = t_i(itgt,jtgt,1:cutoff-1:1)-tadj
   if (wet_GCM) then
      if (H2Ov_tog > 0) then   
         tadj = rv_i(itgt,jtgt,cutoff)-rv_o(itgt,jtgt,cutoff)
         rv_o(itgt,jtgt,1:cutoff-1:1) = rv_i(itgt,jtgt,1:cutoff-1:1)-tadj
      end if
      if (dust_tog > 0) then
         do ib=1,naer_sclr
            tadj = aer_sclr_i(itgt,jtgt,cutoff,ib)-  &
                 aer_sclr_o(itgt,jtgt,cutoff,ib)
            aer_sclr_o(itgt,jtgt,1:cutoff-1:1,ib) =   &
                 aer_sclr_i(itgt,jtgt,1:cutoff-1:1,ib)-tadj
         end do
      end if
   endif

   !*** Interpolate new heights and temperatures for GCM levels below
   !    new `cutoff` level:

   dz = (z_i(itgt,jtgt,kstar)-z_o(itgt,jtgt,cutoff))/(kstar-cutoff)
   cnt = 1
   do k=cutoff+1,kstar-1
      z_o(itgt,jtgt,k) = cnt*dz+z_o(itgt,jtgt,cutoff)
      t_o(itgt,jtgt,k) = cnt*dz*lapse_t+t_o(itgt,jtgt,cutoff)
      cnt = cnt+1
   end do
   if (wet_GCM) then
      if (H2Ov_tog > 0) then   
         cnt = 1
         do k=cutoff+1,kstar-1
            rv_o(itgt,jtgt,k) = cnt*dz*lapse_rv+rv_o(itgt,jtgt,cutoff)     
            cnt = cnt+1
         end do
      end if
      if (dust_tog > 0) then
         do ib=1,naer_sclr
            aer_sclr_o(itgt,jtgt,k,ib) = cnt*dz*lapse_aer(ib)+  &
                 aer_sclr_o(itgt,jtgt,cutoff,ib)
         end do
      end if      
   end if

!$   else     ! If terrain difference "nice"

!      do k=cutoff,1,-1
!         t_o(itgt,jtgt,k) = t_i(itgt,jtgt,k)     ! F90 version below
!      enddo
!$      t_o(itgt,jtgt,1:cutoff:1) = t_i(itgt,jtgt,1:cutoff:1)

!$   endif

!^
!{ 1 } Write out temperature/height profiles for inspection:
!open(23,file='tz.dat',status='unknown')
!write (*,*) itgt,jtgt
!do k=1,n3
!   write (23,*) t_o(itgt,jtgt,k),z_o(itgt,jtgt,k),t_i(itgt,jtgt,k)  &
!        ,z_i(itgt,jtgt,k)
!enddo
!close(23)
!pause
!^

end select interpolation_method

end subroutine tz_vadj


!```````````````````````````````````````````````````````````````````
subroutine uv_vadj1(meth, itgt, jtgt, m1, m2, n3, topoMGr, u_i, v_i, u_o,  &
     v_o, topt)

!+ Adjusts wind velocity components (in the vertical, on GCM levels) 

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: meth, itgt, jtgt, m1, m2, n3
real(f4), dimension(m1,m2,n3), intent(IN) :: u_i, v_i
real(f4), dimension(m1,m2), intent(IN) :: topoMGr, topt

real(f4), dimension(m1,m2,n3), intent(OUT) :: u_o, v_o

!== Local declarations:

! NONE

!=== Executable statements:

!do l=1,n3
!   u_o(itgt,jtgt,l) = u_i(itgt,jtgt,l)     ! F90 version below
!   v_o(itgt,jtgt,l) = v_i(itgt,jtgt,l)
!enddo
u_o(itgt,jtgt,:) = u_i(itgt,jtgt,:)
v_o(itgt,jtgt,:) = v_i(itgt,jtgt,:)

end subroutine uv_vadj1


!```````````````````````````````````````````````````````````````````
subroutine uv_vadj2(meth, m1, m2, m3, itgt, jtgt, u, v, z, topt, zfirst)
 
!+ Adjusts wind velocity components (for near-surface PRAMS levels) 

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use search_mod, only: jsrchgt

implicit none

!=== Argument declarations:

integer, intent(IN) :: meth, itgt, jtgt, m1, m2, m3
real(f4), dimension(m1,m2,m3), intent(IN) :: z
real(f4), dimension(m1,m2), intent(IN) :: topt
real(f4), intent(IN) :: zfirst

real(f4), dimension(m1,m2,m3), intent(INOUT) :: u, v

!== Local declarations:

integer :: l, cutoff
!integer, save :: cntu,cntv
logical :: sign_u, sign_v, sign_e

!data cntu/0/,cntv/0/

!=== Executable statements:

cutoff = jsrchgt(m3,z(itgt,jtgt,:),1,zfirst+topt(itgt,jtgt))

sign_u = u(itgt,jtgt,cutoff) >= ZERO_f4
sign_v = v(itgt,jtgt,cutoff) >= ZERO_f4

do l=cutoff-1,1,-1
   sign_e = u(itgt,jtgt,l) >= ZERO_f4
   if (sign_e .neqv. sign_u) then
      u(itgt,jtgt,l) = ZERO_f4
!      print *,'u',itgt,jtgt,l,sign_e,sign_u,cutoff
!      cntu = cntu+1
!   else
!      print *,'u',itgt,jtgt,l,sign_e,sign_u,cutoff
   end if
   sign_e = v(itgt,jtgt,l) >= ZERO_f4
   if (sign_e .neqv. sign_v) then
      v(itgt,jtgt,l) = ZERO_f4
!      print *,'v',itgt,jtgt,l,sign_e,sign_v,cutoff
!      cntv = cntv+1
!   else
!      print *,'v',itgt,jtgt,l,sign_e,sign_v,cutoff
   end if
end do
         
!if(itgt == m1 .and. jtgt == m2) then
!   print *,'cnt',cntu,cntv
!   stop
!endif

end subroutine uv_vadj2


!```````````````````````````````````````````````````````````````````
subroutine p_vadj(m1, m2, m3, nd3, itgt, jtgt, t_i, z_i, p_i, t_o, z_o, p_o,  &
     dzm, rtgt)

!+ Adjusts pressure via hydrostatic integration (in the vertical) 

use univ_kind_defs_mod, only: f4
use body_specific_constants_mod, only: g_Xoid, R_air, rocp, cp, p00

implicit none

!=== Argument declarations:

integer, intent(IN) :: itgt, jtgt, m1, m2, m3, nd3
real(f4), dimension(m1,m2,nd3), intent(IN) :: t_i, z_i, p_i
real(f4), dimension(m1,m2,m3), intent(IN) :: z_o
real(f4), dimension(m1,m2), intent(IN) :: rtgt
real(f4), dimension(m3), intent(IN) :: dzm
real(f4), dimension(m1,m2,m3), intent(INOUT) :: t_o

real(f4), dimension(m1,m2,m3), intent(OUT) :: p_o

!== Local declarations:

real(f4) :: deltz, avgt, z_bot, t_bot, p_bot, p1, p2, z1, z2
integer :: k

!=== Executable statements:

! Extrapolate downward or interpolate to obtain "surface" pressure:

z_bot = z_o(itgt,jtgt,1)
t_bot = t_o(itgt,jtgt,1)

k = 2
do while (z_i(itgt,jtgt,k) <= z_bot .and. k < nd3)
   k = k+1
end do
p1 = log(p_i(itgt,jtgt,k-1))
p2 = log(p_i(itgt,jtgt,k))
z1 = z_i(itgt,jtgt,k-1)
z2 = z_i(itgt,jtgt,k)
p_bot = exp((p2-p1)/(z2-z1)*(z_bot-z1)+p1)

! Hydrostatic integration to determine pressure of each layer (bottom->top):

k = 1
p_o(itgt,jtgt,k) = p_bot
do k=2,m3
   deltz = z_o(itgt,jtgt,k)-z_bot
   avgt = (t_bot+t_o(itgt,jtgt,k))*0.5_f4
   p_o(itgt,jtgt,k) = p_bot/(exp((g_Xoid*deltz)/(R_air*avgt)))
   p_bot = p_o(itgt,jtgt,k)
   z_bot = z_o(itgt,jtgt,k)
   t_bot = t_o(itgt,jtgt,k)
end do

do k=1,m3   ! Convert tempK to potential temperature
   t_o(itgt,jtgt,k) = t_o(itgt,jtgt,k)*(p00/p_o(itgt,jtgt,k))**rocp
end do

p_o(itgt,jtgt,m3) = cp*(p_o(itgt,jtgt,m3)/p00)**rocp  ! Convert to pi
do k=m3-1,1,-1    ! Redo hydrostatic integration (now cast in theta and pi)
   deltz = rtgt(itgt,jtgt)/dzm(k)
   avgt = 0.5_f4*(t_o(itgt,jtgt,k)+t_o(itgt,jtgt,k+1))  
   p_o(itgt,jtgt,k) = p_o(itgt,jtgt,k+1)+g_Xoid*deltz/avgt  ! Get new pi profile
end do

end subroutine p_vadj


!```````````````````````````````````````````````````````````````````
subroutine vadj2mr(m1, m2, m3, nd3, itgt, jtgt, u_i, v_i, t_i, z_i, rv_i,  &
     aer_sclr_i, u_o, v_o, t_o, z_o, rv_o, aer_sclr_o, topt, rtgt, zt,  &
     wet_GCM, naer_sclr)

!+ Vertically interpolates to PRAMS levels 

use univ_kind_defs_mod, only: f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_MESSAGE_LEN
use univ_exception_mod, only: exception, WARN
use main_memory_bspec_mod, only: dust_tog, H2Ov_tog

implicit none

!=== Argument declarations:

logical, intent(IN) :: wet_GCM
integer, intent(IN) :: itgt, jtgt, m1, m2, m3, nd3, naer_sclr
real(f4), dimension(m1,m2,nd3), intent(IN) :: u_i, v_i, t_i, z_i, rv_i
real(f4), dimension(m1,m2,nd3,naer_sclr), intent(IN) :: aer_sclr_i
real(f4), dimension(m1,m2), intent(IN) :: topt, rtgt
real(f4), dimension(m3), intent(IN) :: zt

real(f4), dimension(m1,m2,m3), intent(OUT) :: u_o, v_o, t_o, z_o, rv_o
real(f4), dimension(m1,m2,m3,naer_sclr), intent(OUT) :: aer_sclr_o

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'vadj2mr'

real(f4), allocatable, dimension(:) :: vectr01, vectr02, vectr03, vectr04,  &
     vectr05
character(len=MAX_MESSAGE_LEN) :: msg

integer :: maxl, k, e, ib

!=== Executable statements:

! Allocate work arrays:
maxl = max(nd3, m3)
allocate(vectr01(maxl), vectr02(maxl), vectr03(maxl), vectr04(maxl),  &
     vectr05(maxl), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'vadj2mr_01')

! Create height profile from sigmaZ values:
vectr01(1:m3) = zt(1:m3)*rtgt(itgt,jtgt)+topt(itgt,jtgt)

! Fill vectors with data to interpolate:
vectr02(1:nd3) = z_i(itgt,jtgt,:)
vectr03(1:nd3) = u_i(itgt,jtgt,:)
vectr04(1:nd3) = v_i(itgt,jtgt,:)
vectr05(1:nd3) = t_i(itgt,jtgt,:)

! Check for any data-range overrun; if so, warn user:
ib = 0
do k=m3,1,-1
   if (vectr01(k) <= vectr02(nd3)) then
      ib = k; exit  ! Found first valid interpolation
   end if
end do
if (ib < m3) then
   write (msg, '(a, i5, a, f10.2, a, f10.2, a)')  &
        'Interpolation of ', m3-ib, ' point(s) in this vertical column is '//  &
        'not possible, as it/they are outside the vertical range of the '//  &
        'data (data_z_max=', vectr02(nd3), 'm, versus model_target_z_max=',  &
        vectr01(m3), 'm); filled by zero-gradient extrapolation.'
   call exception(procedure_name, WARN, msg)
end if

! Interpolate data and output to ()_o arrays:

do k=1,m3
   call qintrp(vectr03, u_o(itgt,jtgt,k), vectr01(k), vectr02, nd3)
   call qintrp(vectr04, v_o(itgt,jtgt,k), vectr01(k), vectr02, nd3)
   call qintrp(vectr05, t_o(itgt,jtgt,k), vectr01(k), vectr02, nd3)
   z_o(itgt,jtgt,k) = vectr01(k)
end do

if (wet_GCM) then
   if (H2Ov_tog > 0) then
      vectr03(1:nd3) = rv_i(itgt,jtgt,:)
      do k=1,m3
         call qintrp(vectr03, rv_o(itgt,jtgt,k), vectr01(k), vectr02, nd3)
      end do
   end if
   if (dust_tog > 0) then
      do ib=1,naer_sclr
         vectr04(1:nd3) = aer_sclr_i(itgt,jtgt,:,ib)
         do k=1,m3
            call qintrp(vectr04, aer_sclr_o(itgt,jtgt,k,ib), vectr01(k),  &
                 vectr02, nd3)
         end do
      end do
   end if
end if

! Deallocate work arrays:
deallocate(vectr01, vectr02, vectr03, vectr04, vectr05, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'vadj2mr_01')

end subroutine vadj2mr


!```````````````````````````````````````````````````````````````````
subroutine qintrp(a, b, zval, z, nzp)

!+ Interpolates linearly in the vertical f(z); where this is not possible,
!   if zval < z(k) a constant-gradient extrapolation is performed
!   if zval >= z(nzp) a zero-gradient extrapolation is performed

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

real(f4), dimension(:), intent(IN) :: a, z
real(f4), intent(IN) :: zval
integer, intent(IN) :: nzp

real(f4), intent(OUT) :: b

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'qintrp'

integer :: k, kabv

!=== Executable statements:

if (z(nzp) > zval) then

   kabv = 0
   do k=1,nzp
      if (zval < z(k)) then
         if (k /= 1) then
            kabv = k
         else
            kabv = 2
         end if
         exit
      end if
   end do

   b = (a(kabv)*(zval-z(kabv-1))+a(kabv-1)*(z(kabv)-zval))/(z(kabv)-z(kabv-1))

else

   b = a(nzp)  ! Zero-gradient extrapolation

end if

end subroutine qintrp


!```````````````````````````````````````````````````````````````````
subroutine varuv(n1, n2, n3, igd, up, vp)

!+ Rotates wind vectors to polar stereographic; averages to staggered grid

use univ_kind_defs_mod, only: f4
use univ_OPS_proj_mod, only: xy_ll_OPS
use main_memory_mod, only: gd
use body_specific_constants_mod, only: radius
use polarst_mod, only: uevetouv

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, igd

real(f4), dimension(n1,n2,n3), intent(INOUT) :: up, vp

!== Local declarations:

integer :: i, j, k
real(f4) :: qlat, qlon, up1, vp1

!=== Executable statements:

do j=1,n3
   do i=1,n2
      do k=1,n1
         call xy_ll_OPS(qlat, qlon, gd(igd)%plat, gd(igd)%plon,  & ! Obtain lat/lon of
              gd(igd)%xt(i), gd(igd)%yt(j), radius)              !  current OPS point
         call uevetouv(up1, vp1, up(k,i,j), vp(k,i,j), qlat, qlon,  &
              gd(igd)%plat, gd(igd)%plon) ! "Rotate" winds from lat/lon to OPS coord system
         up(k,i,j) = up1
         vp(k,i,j) = vp1
      end do
   end do
end do

! Average velocities to staggered PRAMS C-grid (from A-grid):

do j=1,n3
   do i=1,gd(igd)%nx
      do k=1,n1
         up(k,i,j) = (up(k,i,j)+up(k,i+1,j))*0.5_f4
      end do
   end do
end do

do j=1,gd(igd)%ny
   do i=1,n2
      do k=1,n1
         vp(k,i,j) = (vp(k,i,j)+vp(k,i,j+1))*0.5_f4
      end do
   end do
end do

end subroutine varuv


!```````````````````````````````````````````````````````````````````
subroutine hinterpg(m1, m2, q1, q2, nd1, nd2, nd3, nd4, nd3_SBS, dati, dato,  &
     topoMG, topoMGr, gt, gt_SBS, gir, gtMGhi, glon, glat, latMG, lonMG)

!+ Horizontally interpolate GCM fields to PRAMS grid

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_interpolation_mod, only: intrp_2D_pnt_type02g
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use PRAMS_SBPS_mod, only: tsat_co2ice

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, nd1, nd2, nd3, nd4, q1, q2, nd3_SBS
real(f4), dimension(nd1+1,nd2,nd3,nd4), intent(IN) :: dati
real(f4), dimension(m1,m2) , intent(IN):: glon, glat
real(f4), dimension(nd1+1,nd2), intent(IN) :: topoMG, gt, latMG, lonMG
real(f4), dimension(nd1+1,nd2,nd3_SBS), intent(IN) :: gt_SBS 

real(f4), dimension(m1,m2,nd3,nd4), intent(OUT) :: dato
real(f4), dimension(q1,q2), intent(OUT) :: gir
real(f4), dimension(m1,m2), intent(OUT) :: topoMGr
real(f4), dimension(q1,q2,nd3_SBS+1), intent(OUT) :: gtMGhi

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'hinterpg'

real(f4), allocatable, dimension(:,:) :: gttmp, gitmp
real(f4), dimension(nd1+3,nd2) :: intrp_tmp
real(f4) :: grx, gry, gglon, gglat, gsx, gsy, eps
integer :: k, nv, i, j, igcm, jgcm, a_err

!=== Executable statements:

! Preprocess GCM `gt` variable (split out sfc_temp and ice_mass fields):

allocate(gttmp(nd1+1,nd2), gitmp(nd1+1,nd2), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'hinterpg_01')

do j=1,nd2
   do i=1,nd1+1
      if (gt(i,j) < ZERO_f4) then    ! If ice on ground in GCM
         gttmp(i,j) = Tsat_CO2ice(dati(i,j,1,6))   ! Peg at Tsat
         gitmp(i,j) = -gt(i,j)      ! CO2 ice mass (kg)
      else        ! Bare ground in GCM
         gttmp(i,j) = gt(i,j)   ! GCM sfc temperature
         gitmp(i,j) = ZERO_f4    ! Zero kg of CO2 ice
      end if
   end do
end do

! Interpolate GCM fields horizontally to PRAMS grid:

do j=1,m2
   do i=1,m1

      gglon = glon(i,j)
      gglat = glat(i,j)

      ! Find correct location of PRAMS point in GCM array:

      igcm = 1  ! Start at 180.W,90.S
      jgcm = 1  !

      do while (gglon > lonMG(igcm,jgcm))  ! Search for correct longitude
         igcm = igcm+1
      end do
      do while (gglat > latMG(igcm,jgcm))  ! Search for correct latitude
         jgcm = jgcm+1
      end do

      gsx = (gglon-lonMG(igcm-1,jgcm))/  &
           (lonMG(igcm,jgcm)-lonMG(igcm-1,jgcm))  ! d?/dlon
      gsy = (gglat-latMG(igcm,jgcm-1))/  &
           (latMG(igcm,jgcm)-latMG(igcm,jgcm-1))  ! d?/dlat

      ! Fractional locations in GCM (i,j) grid of PRAMS grid point:
      !  (Ensure that values are not out of bounds)

      eps = real(nd1+3, kind=f4)*epsilon(grx)
      grx = max(min(real(igcm, kind=f4)+gsx,  &     ! igcm-1+1
           real(nd1+3, kind=f4)-eps), ONE_f4+eps) 
      eps = real(nd2, kind=f4)*epsilon(gry)
      gry = max(min(real(jgcm-1, kind=f4)+gsy,real(nd2, kind=f4)-eps),  &
           ONE_f4+eps)

      !*** Horizontally interpolate variables:
           
      do nv=1,nd4
    
         do k=1,nd3
          
!            if(nv /= 5) then
            intrp_tmp(2:nd1+2,:) = dati(1:nd1+1,:,k,nv)
            intrp_tmp(1,:) = dati(nd1+1,:,k,nv)
            intrp_tmp(nd1+3,:) = dati(3,:,k,nv)
            dato(i,j,k,nv) = intrp_2D_pnt_type02g(intrp_tmp, grx, gry)
!            else
!               dato(i,j,:,nv) = 0.     ! Water mixing ratio = zero
!            endif

         end do
         
      end do
      
      ! Topography:
      intrp_tmp(2:nd1+2,:) = topoMG(1:nd1+1,:)
      intrp_tmp(1,:) = topoMG(nd1+1,:); intrp_tmp(nd1+3,:) = topoMG(3,:)
      topoMGr(i,j) = intrp_2D_pnt_type02g(intrp_tmp, grx, gry)

      ! Surface temperature:
      intrp_tmp(2:nd1+2,:) = gttmp(1:nd1+1,:)
      intrp_tmp(1,:) = gttmp(nd1+1,:); intrp_tmp(nd1+3,:) = gttmp(3,:)
      gtMGhi(i,j,nd3_SBS+1) = intrp_2D_pnt_type02g(intrp_tmp, grx, gry)

      ! Surface CO2 ice mass:
      intrp_tmp(2:nd1+2,:) = gitmp(1:nd1+1,:)
      intrp_tmp(1,:) = gitmp(nd1+1,:); intrp_tmp(nd1+3,:) = gitmp(3,:)
      gir(i,j) = intrp_2D_pnt_type02g(intrp_tmp, grx, gry)

      ! SBS temperature:
      do k=1,nd3_SBS
         intrp_tmp(2:nd1+2,:) = gt_SBS(1:nd1+1,:,k)
         intrp_tmp(1,:) = gt_SBS(nd1+1,:,k); intrp_tmp(nd1+3,:) = gt_SBS(3,:,k)
         gtMGhi(i,j,k) = intrp_2D_pnt_type02g(intrp_tmp,grx, gry)
      end do

   end do
end do

deallocate(gttmp, gitmp, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'hinterpg_01')

end subroutine hinterpg


!```````````````````````````````````````````````````````````````````
subroutine swtch2otp(m1, m2, m3, nd4, q, u_o, v_o, pi_o, th_o, rv_o,  &
     dustncXX, wet_GCM, dust_nb)

!+ Fill standard PRAMS arrays with output quantities in correct order

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use body_specific_constants_mod, only: cp, p00i, rocp, p00
use main_memory_bspec_mod, only: H2Ov_tog, dust_tog
  
implicit none

!=== Argument declarations:

logical, intent(IN) :: wet_GCM
integer, intent(IN) :: m1, m2, m3, nd4, dust_nb
real(f4), dimension(m1,m2,m3,nd4), intent(IN) :: q

real(f4), dimension(m3,m1,m2), intent(OUT) :: u_o, v_o, pi_o, th_o, rv_o
real(f4), dimension(m3,m1,m2,dust_nb), intent(OUT) :: dustncXX

!== Local declarations:

integer :: i, j, ib

!=== Executable statements:

!*** Switch ordering; calculate output quantities:

do j=1,m2
   do i=1,m1

      u_o(:,i,j) = q(i,j,:,1)
      v_o(:,i,j) = q(i,j,:,2)
!@      pi_o(:,i,j) = cp*((q(i,j,:,6)*p00i)**rocp)
      pi_o(:,i,j) = q(i,j,:,6)
!@      th_o(:,i,j) = q(i,j,:,3)*((p00/q(i,j,:,6))**rocp)
      th_o(:,i,j) = q(i,j,:,3)

      ! Need to force PRAMS-type BC upon all fields (except PI):

!      th_o(1,i,j) = th_o(2,i,j)
!      u_o(1,i,j) = u_o(2,i,j)
!      v_o(1,i,j) = v_o(2,i,j)

!      if(j>=14 .and. j <=17 .and.i==32) then! .and. i>=29 .and. i<=32) then
!         print *,i,j
!         print *,'u',u_o(1:5,i,j)
!         print *,'v',v_o(1:5,i,j)
!         print *,'p',pi_o(1:5,i,j)
!         print *,'t',th_o(1:5,i,j)
!         pause
!      endif

   end do
end do

if (wet_GCM) then

   if (H2Ov_tog > 0) then
      do j=1,m2
         do i=1,m1
            rv_o(:,i,j) = q(i,j,:,5)
         end do
      end do
      rv_o = merge(rv_o, ZERO_f4,rv_o >= ZERO_f4)
   end if
   
   if (dust_tog > 0) then
      do ib=1,dust_nb
         do j=1,m2
            do i=1,m1
               dustncXX(:,i,j,ib) = q(i,j,:,ib+6)
            end do
         end do
         dustncXX(:,:,:,ib) = merge(dustncXX(:,:,:,ib), ZERO_f4,  &
              dustncXX(:,:,:,ib) >= ZERO_f4)
      end do
   end if

else
   if (H2Ov_tog > 0) then
      rv_o(1:m3,1:m1,1:m2) = ZERO_f4      ! No moisture
   end if
end if

end subroutine swtch2otp


!```````````````````````````````````````````````````````````````````
subroutine SBS_temp(m1, m2, q1, q2, m6, nd3_SBS, zlevSBS, gtMGhi, gtr, LST)

!+ Generate sub-surface temperature profiles based on time of sol, etc.

use univ_kind_defs_mod, only: f4
use univ_interpolation_mod, only: intrp_1D_arr_type01a
use main_memory_mod, only: vctr1, SBPS_Tlev
use surface_init_mod, only: calc_SBPS_Tlev

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m6, q1, q2, nd3_SBS
real(f4), dimension(m1,m2), intent(IN) :: LST
real(f4), dimension(q1,q2,nd3_SBS+1), intent(IN) :: gtMGhi
real(f4), dimension(nd3_SBS+1), intent(IN) :: zlevSBS

real(f4), dimension(q1,q2,m6), intent(OUT) :: gtr

!== Local declarations:

!#real(f4), dimension(11,4) :: gtadj      ! *** HARDWIRED ***

real(f4), dimension(nd3_SBS+1) :: vctrQ

integer :: i, j, k

!=== Executable statements:

! Arbitrary temp differences between SBS levels (from an early Gusev run):

!#data gtadj(11:1:-1,1) / -2.144,-4.017,-6.617,-9.306,-2.838,1.412,1.480  &
!#     ,0.0383,-1.262,-1.937,0. /     ! Midnight
!#data gtadj(11:1:-1,2) / -1.460,-2.815,-4.910,-8.160,-6.225,-3.100,-1.950  &
!#     ,-2.140,-2.778,-3.321,0. /     ! 6am
!#data gtadj(11:1:-1,3) / 6.545,12.101,19.109,23.410,6.167,-1.548,-2.780  &
!#     ,-2.752,-2.775,-2.867,0. /     ! Noon
!#data gtadj(11:1:-1,4) / -3.950,-5.467,-4.165,6.062,12.559,6.301,1.094,-1.458  &
!#     ,-2.416,-2.701,0. /            !6pm


!#adjtd = 24./4.       ! LST interval between representative profiles

!#gtr(:,:,1) = 170.      ! "Deep" regolith temperature

call calc_SBPS_Tlev(m6)

do j=1,m2
   do i=1,m1

      ! Interpolate PRAMS SBS temperatures linearly from GCM values:

      do k=1,nd3_SBS+1
         vctrQ(k) = gtMGhi(i,j,k)
      end do
    
!      call htint(nd3_SBS+1,vctrQ,zlevSBS,m6,vctr1,SBPS_Tlev)
      vctr1(1:m6) = intrp_1D_arr_type01a(vctrQ, zlevSBS, SBPS_Tlev(1:m6))

      do k=2,m6
         gtr(i,j,k) = vctr1(k)
      end do
      gtr(i,j,1) = 170._f4  ! Always this temperature

!      iq = nd3_SBS+1
!      do k=m6,1,-1
!         do while(SBPS_Tlev(k) > zlevSBS(iq+1) .or.  &
!              SBPS_Tlev(k) < zlevSBS(iq))
!            tidx1 = tidx1+1
!         enddo
         
!      enddo
!#      tidx = int(LST(i,j)/adjtd)+1
!#      if(tidx <= 3) then
!#         tidx1 = tidx+1 
!#      else
!#         tidx1 = 1
!#      endif
!#      tint = (LST(i,j)/adjtd)-real(tidx-1)

!#      do iq=m6-1,2,-1
!#         gtr(i,j,iq) = gtr(i,j,iq+1)-((1.-tint)*gtadj(iq+1,tidx)+  &
!#              tint*gtadj(iq+1,tidx1))    ! Interpolate between representative
                                         !   SBS temperature profiles
!#      enddo

   end do
end do

end subroutine SBS_temp


!```````````````````````````````````````````````````````````````````
subroutine PRAMS_CartpreM_interp(n1, n2, n3, ntgt, vari, varo, v1, v2, v3,  &
     ztgt, topo, ztop, zt)

!+ Interpolates data on PRAMS sigma-z levels to Cartesian levels

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use interpolation_mod, only: zV_interp
use main_memory_mod, only: undef_value_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, ntgt
real(f4), dimension(n1,n2,n3), intent(IN) :: vari
real(f4), dimension(ntgt), intent(IN) :: ztgt
real(f4), dimension(n2,n3), intent(IN) :: topo
real(f4), dimension(n1), intent(IN) :: zt
real(f4), intent(IN) :: ztop

real(f4), dimension(ntgt), intent(OUT) :: v1, v2, v3
real(f4), dimension(n2,n3,ntgt), intent(OUT) :: varo

!== Local declarations:

integer :: i, j, k, idx_mask_begin

!=== Executable statements:

do j=1,n3
   do i=1,n2

      do k=1,n1
         v1(k) = vari(k,i,j)   ! Vector to be interpolated
         v2(k) = topo(i,j)+zt(k)*(ONE_f4-topo(i,j)/ztop)  ! Z levs of the above
      end do

      do k=ntgt,1,-1
         if (ztgt(k) <= v2(n1)) exit
      end do
      idx_mask_begin = k+1

      call zV_interp(n1, v1, v2, ntgt, v3, ztgt, topo(i,j))

      do k=1,idx_mask_begin-1
         varo(i,j,k) = v3(k)   ! Interpolated values
      end do
      do k=idx_mask_begin,ntgt
         varo(i,j,k) = undef_value_f4   ! Above top of data
      end do

   end do
end do

end subroutine PRAMS_CartpreM_interp


!```````````````````````````````````````````````````````````````````
subroutine PRAMS_CartpreG_interp(n2, n3, n1, n4, ntgt ,vari, varo, v1, v2,  &
     v3, ztgt, topo, gzlevs)

!+ Interpolates data on GCM sigma levels to Cartesian levels

use univ_kind_defs_mod, only: f4
use interpolation_mod, only: zV_interp
use main_memory_mod, only: undef_value_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, n4, ntgt
real(f4), dimension(n2,n3,n1,n4), intent(IN) :: vari
real(f4), dimension(ntgt), intent(IN) :: ztgt
real(f4), dimension(n2,n3), intent(IN) :: topo
real(f4), dimension(n2,n3,n1), intent(IN) :: gzlevs

real(f4), dimension(ntgt), intent(INOUT) :: v1, v2, v3   ! Scratch space

real(f4), dimension(n2,n3,ntgt,n4), intent(OUT) :: varo

!== Local declarations:

integer :: i, j, k, iq, idx_mask_begin

!=== Executable statements:

do j=1,n3
   do i=1,n2

      do k=1,n1
         v2(k) = gzlevs(i,j,k)  ! Z levs of above vector
      end do

      do k=ntgt,1,-1
         if (ztgt(k) <= v2(n1)) exit
      end do
      idx_mask_begin = k+1

      do iq=1,n4   ! Loop over all variables       
         do k=1,n1
            v1(k) = vari(i,j,k,iq)   ! Vector to be interpolated
         end do

         call zV_interp(n1, v1, v2, ntgt, v3, ztgt, topo(i,j))

         do k=1,idx_mask_begin-1
            varo(i,j,k,iq) = v3(k)   ! Interpolated values
         end do
         do k=idx_mask_begin,ntgt
            varo(i,j,k,iq) = undef_value_f4   ! Above top of data
         end do
      end do

   end do
end do

end subroutine PRAMS_CartpreG_interp


!```````````````````````````````````````````````````````````````````
subroutine prep_stdlevs(n2, n3, n1, zmax, ztgt, topo1, topo2)

!+ Prepares a "standard" column of geopotential height level values to
!   interpolate to

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3
real(f4), dimension(n2,n3), intent(IN) :: topo1, topo2
real(f4), intent(IN) :: zmax

real(f4), dimension(n1), intent(OUT) :: ztgt

!== Local declarations:

integer :: k, klast
real(f4) :: rtopomax, rtopomin, ztest, zinc

real(f4), parameter :: blyr_depth = 5000._f4,  &  ! Depth of avg PBL
              blyr_dz = 200._f4  ! delta-z [at >rtopomin and <('max+blyr_depth)]

!=== Executable statements:

! Find min/max topography values:

rtopomin = minval(topo1)
rtopomin = min(rtopomin,minval(topo2))
rtopomax = maxval(topo1)
rtopomax = max(rtopomax,maxval(topo2))

ztest = rtopomax+blyr_depth  ! "Boundary layer" is below this height

! "Boundary layer" levels:

k = 2
ztgt(1) = rtopomin
do while (ztgt(k-1) < ztest)
   ztgt(k) = ztgt(k-1)+blyr_dz    ! "standard" height values to interpolate to
   if (k < n1) then
      k = k+1
   else
      write (*, '(/,a,/)') 'ERROR: # of gz-interp levels too small'
   end if
end do
klast = k-1

! Remainder of atmosphere:

zinc = (zmax-ztgt(k))/real(n1-klast, kind=f4)  ! delta-z for remainder of atmosphere
do k=klast,n1
   ztgt(k) = ztgt(k-1)+zinc    ! "standard" height values to interpolate to
end do

end subroutine prep_stdlevs


!```````````````````````````````````````````````````````````````````
subroutine varuv2(n2, n3, n1, igd, up, vp)

!+ Rotates wind vectors to polar stereographic; averages to staggered grid
!   (for i,j,k ordered arrays)

use univ_kind_defs_mod, only: f4
use univ_OPS_proj_mod, only: xy_ll_OPS
use main_memory_mod, only: gd
use body_specific_constants_mod, only: radius
use polarst_mod, only: uevetouv

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, igd

real(f4), dimension(n2,n3,n1), intent(INOUT) :: up, vp

!== Local declarations:

integer :: i, j, k
real(f4) :: qlat, qlon, up1, vp1

!=== Executable statements:

DO K=1,n1
   DO J=1,n3
      DO I=1,n2
         call xy_ll_OPS(qlat, qlon, gd(igd)%plat, gd(igd)%plon,  &
              gd(igd)%xt(i), gd(igd)%yt(j), radius)
         call uevetouv(up1, vp1, up(i,j,k), vp(i,j,k), qlat, qlon,  &
              gd(igd)%plat, gd(igd)%plon)
         UP(I,J,K) = UP1
         VP(I,J,K) = VP1
      END DO
   END DO
END DO

! Average velocities to staggered PRAMS C-grid (from A-grid):

DO K=1,n1
   DO J=1,n3
      DO I=1,gd(igd)%nx
         UP(I,J,K) = (UP(I,J,K)+UP(I+1,J,K))*0.5_f4
      END DO
   END DO
END DO

DO K=1,n1
   DO J=1,gd(igd)%ny
      DO I=1,n2
         VP(I,J,K) = (VP(I,J,K)+VP(I,J+1,K))*0.5_f4
      END DO
   END DO
END DO

end subroutine varuv2


!```````````````````````````````````````````````````````````````````
subroutine convert_MGcmp(n2, n3, n1, T, P)

!+ Convert pressure/tempk to pi/theta for output

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: check_undef_val_f4
use body_specific_constants_mod, only: cp, p00i, rocp

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3

real(f4), dimension(n2,n3,n1), intent(INOUT) :: T, P

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

do k=1,n1
   do j=1,n3
      do i=1,n2
         if (P(i,j,k) < check_undef_val_f4) then
            P(i,j,k) = cp*((P(i,j,k)*p00i)**rocp)   ! PI
            T(i,j,k) = T(i,j,k)*cp/P(i,j,k)    ! theta
         end if
      end do
   end do
end do

end subroutine convert_MGcmp


end module PIPP_intrp_mod


!```````````````````````````````````````````````````````````````````
subroutine mvarfa(igd, nd1, nd2, nd3, nd4, nd3_SBS, gcm, topoMG, gt, gt_SBS,  &
     gtr, gir, gtMGhi, mg, curr_utc, latMG, lonMG, zlevSBS, wet_GCM)

!+ Interpolation driver (GCM data --> PRAMS grid) 

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_GrADS_format_mod, only: write_GrADS_dfile
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: a, ztop, maxnxp, maxnyp, nzg, gd,  &
     gen_vf_input_comparison
use main_memory_bspec_mod, only: dust_nb, dust_tog, H2Ov_tog
use vtables_mod, only: iglonn, iglatn, itoptn, irtgtn, iupn, ivpn, ippn,  &
     ithpn, irvn, ivt3da, ivt3db, ivt3dc, ivt3dp, idustncn
use PIPP_intrp_mod, only: hinterpg, vinterp, prep_stdlevs, convert_MGcmp,  &
     PRAMS_CartpreG_interp, varuv2, sbs_temp, swtch2otp, varuv
use PIPP_time_mod, only: PIPP_LSTcalc

implicit none

!=== Argument declarations:

logical, intent(IN) :: wet_GCM
integer, intent(IN) :: igd, nd1, nd2, nd3, nd4, mg, nd3_SBS
real(f4), dimension(nd1+1,nd2,nd3,nd4), intent(IN) :: gcm
real(f4), dimension(nd1+1,nd2), intent(IN) :: topoMG, gt, latMG, lonMG
real(f4), dimension(nd1+1,nd2,nd3_SBS), intent(IN) :: gt_SBS
real(f4), intent(IN) :: curr_utc
real(f4), dimension(nd3_SBS+1), intent(IN) :: zlevSBS

real(f4), dimension(maxnxp,maxnyp,nd3_SBS+1), intent(OUT) :: gtMGhi
real(f4), dimension(maxnxp,maxnyp,mg), intent(OUT) :: gtr
real(f4), dimension(maxnxp,maxnyp), intent(OUT) :: gir

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'mvarfa'

real(f4), allocatable, dimension(:,:,:,:) :: hItmp, vItmp, dattmp, datintrp
real(f4), allocatable, dimension(:,:) :: topoMGr, lmtime
integer :: a_err, ntgt, idx_r, idx_d, d_nb

!=== Executable statements:

!*** Allocate work arrays:

allocate(hItmp(gd(igd)%nxp,gd(igd)%nyp,nd3,nd4), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'mvarfa_01')
allocate(dattmp(gd(igd)%nxp,gd(igd)%nyp,nd3,nd4), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'mvarfa_02')
allocate(vItmp(gd(igd)%nxp,gd(igd)%nyp,gd(igd)%nzp,nd4), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'mvarfa_03')
allocate(topoMGr(gd(igd)%nxp,gd(igd)%nyp), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'mvarfa_04')
allocate(lmtime(gd(igd)%nxp,gd(igd)%nyp), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'mvarfa_05')

!*** Calculate Mars LST for every PRAMS grid point:

call PIPP_LSTcalc(gd(igd)%nxp, gd(igd)%nyp, a(iglonn(igd)), lmtime, curr_utc)

!*** Horizontally interpolate semi-raw GCM data to PRAMS grid:

call hinterpg(gd(igd)%nxp, gd(igd)%nyp, maxnxp, maxnyp, nd1, nd2, nd3, nd4,  &
     nd3_SBS, gcm, hItmp, topoMG, topoMGr, gt, gt_SBS, gir, gtMGhi,  &
     a(iglonn(igd)), a(iglatn(igd)), latMG, lonMG)

!*** Vertically interpolate semi-raw GCM data to sigma-z levels:

call vinterp(gd(igd)%nxp, gd(igd)%nyp, gd(igd)%nzp, nd1, nd2, nd3, nd4,  &
     topoMGr, a(itoptn(igd)), hItmp, vItmp, dattmp, a(irtgtn(igd)), igd,  &
     wet_GCM)


!*** Interpolate GCMdata (already horizontally interpolated to PRAMS grid)
!     to "standard" Cartesian vertical levels; output to GrADS format file:

if (gen_vf_input_comparison) then    ! PRAMSinit/GCM comparison
   
   if (igd == 1) then   ! Only perform for coarsest grid

      ntgt = (gd(igd)%nxp+gd(igd)%nyp+gd(igd)%nzp) ! Dimension of intrp gz array

      ! Allocate storage:

      allocate(datintrp(gd(igd)%nxp,gd(igd)%nyp,ntgt,nd4), stat=a_err)
      if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'mvarfa_06')
      
      ! Calc "standard" interpolation levs (used below and later in PIPP_main):
      
      call prep_stdlevs(gd(igd)%nxp, gd(igd)%nyp, ntgt, (ztop-ONE_f4),  &
           a(ivt3dp), topoMGr, a(itoptn(igd)))

      ! Interpolate GCM data to above geopotential levels:

      call PRAMS_CartpreG_interp(gd(igd)%nxp, gd(igd)%nyp, nd3, nd4, ntgt,  &
           hItmp, datintrp, a(ivt3da), a(ivt3db), a(ivt3dc), a(ivt3dp),  &
           topoMGr, hItmp(:,:,:,4))

      ! Convert press/tempk to PI/theta:

      call convert_MGcmp(gd(igd)%nxp, gd(igd)%nyp, ntgt, datintrp(:,:,:,3),  &
           datintrp(:,:,:,6))

      ! Rotate horizontal velocities into polar stereographic coordinate
      ! system; then average the velocities to the correct points in the
      ! C-grid stagger:

      call varuv2(gd(igd)%nxp, gd(igd)%nyp, ntgt, igd, datintrp(:,:,:,1),  &
           datintrp(:,:,:,2))

      ! Write to GrADS format file:

      call write_GrADS_dfile(gd(igd)%nxp, gd(igd)%nyp, ntgt,  &
           datintrp(:,:,:,1), 'GCM     ')   ! u
      call write_GrADS_dfile(gd(igd)%nxp, gd(igd)%nyp, ntgt,  &
           datintrp(:,:,:,2), 'GCM     ')   ! v
      call write_GrADS_dfile(gd(igd)%nxp, gd(igd)%nyp, ntgt,  &
           datintrp(:,:,:,3), 'GCM     ')   ! theta
      call write_GrADS_dfile(gd(igd)%nxp, gd(igd)%nyp, ntgt,  &
           datintrp(:,:,:,6), 'GCM     ')   ! PI

      ! Deallocate storage:

      deallocate(datintrp, stat=a_err)
      if (a_err /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'mvarfa_06')

   end if

end if

!*** Fill sub-surface temperatures at all levels:

call SBS_temp(gd(igd)%nxp, gd(igd)%nyp, maxnxp, maxnyp, nzg, nd3_SBS,  &
     zlevSBS, gtMGhi, gtr, lmtime)

!*** Fill arrays with output values (Pi,theta,m.r.) and switch ordering 
!      [ a(k,i,j,v) = vItmp(i,j,k,v) ]:
!  NOTE: uc = dust_radius, vc = dust_massconc

if (wet_GCM .and. H2Ov_tog > 0) then
   idx_r = irvn(igd)
else
   idx_r = ivt3dc
end if

if (dust_tog > 0) then
   idx_d = idustncn(igd,1)
   d_nb = dust_nb
else
   idx_d = ivt3dc
   d_nb = 1
end if

call swtch2otp(gd(igd)%nxp, gd(igd)%nyp, gd(igd)%nzp, nd4, vItmp,  &
     a(iupn(igd)), a(ivpn(igd)), a(ippn(igd)), a(ithpn(igd)), a(idx_r),  &
     a(idx_d), wet_GCM, d_nb)

!*** Rotate horizontal velocities into polar stereographic coordinate system;
!    then average the velocities to the correct points in the C-grid stagger:

call varuv(gd(igd)%nzp, gd(igd)%nxp, gd(igd)%nyp, igd, a(iupn(igd)),  &
     a(ivpn(igd)))

deallocate(hItmp, dattmp, vItmp, topoMGr, lmtime, stat=a_err) ! Free work arrays
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'mvarfa_01')

end subroutine mvarfa


!```````````````````````````````````````````````````````````````````
subroutine varfile_nstfeed(igd, wet_GCM)

!+ Feed back variables between multiple grids

use vtables_mod, only: iupn, ivpn, ippn, ithpn, irvn, idn0n, ipi0n, ipcn,  &
     idustncn, irtgtn, idn0un, idn0vn
use main_memory_mod, only: a, iscr1, gd
use main_memory_bspec_mod, only: H2Ov_tog, dust_tog, dust_nb
use nest_mod, only: fdback

implicit none

!=== Argument declarations:

logical, intent(IN) :: wet_GCM
integer, intent(IN) :: igd

!== Local declarations:

integer :: ifm, icm, iu1, iv1, ip1, it1, ir1, iu2, iv2, ip2, it2, ir2, iq, ib

!=== Executable statements:

!*** Feed back the finer mesh to the coarser mesh:

ifm = igd
icm = gd(ifm)%nxtnest

iu1 = iupn(icm)
iv1 = ivpn(icm)
ip1 = ippn(icm)
it1 = ithpn(icm)
ir1 = irvn(icm)

iu2 = iupn(ifm)
iv2 = ivpn(ifm)
ip2 = ippn(ifm)
it2 = ithpn(ifm)
ir2 = irvn(ifm)

call fdback(a(iu1), a(iu2), a(idn0un(icm)), a(idn0un(ifm)), gd(icm)%nzp,  &
     gd(icm)%nxp, gd(icm)%nyp, gd(ifm)%nzp, gd(ifm)%nxp, gd(ifm)%nyp, ifm,  &
     'u', a(iscr1))

call fdback(a(iv1), a(iv2), a(idn0vn(icm)), a(idn0vn(ifm)), gd(icm)%nzp,  &
     gd(icm)%nxp, gd(icm)%nyp, gd(ifm)%nzp, gd(ifm)%nxp, gd(ifm)%nyp, ifm,  &
     'v', a(iscr1))

call fdback(a(ip1), a(ip2), a(idn0n(icm)), a(idn0n(ifm)), gd(icm)%nzp,  &
     gd(icm)%nxp, gd(icm)%nyp, gd(ifm)%nzp, gd(ifm)%nxp, gd(ifm)%nyp, ifm,  &
     'p', a(iscr1))

call fdback(a(it1), a(it2), a(idn0n(icm)), a(idn0n(ifm)), gd(icm)%nzp,  &
     gd(icm)%nxp, gd(icm)%nyp, gd(ifm)%nzp, gd(ifm)%nxp, gd(ifm)%nyp, ifm,  &
     't', a(iscr1))

if (H2Ov_tog > 0) then
   call fdback(a(ir1), a(ir2), a(idn0n(icm)), a(idn0n(ifm)), gd(icm)%nzp,  &
        gd(icm)%nxp, gd(icm)%nyp, gd(ifm)%nzp, gd(ifm)%nxp, gd(ifm)%nyp, ifm,  &
        't', a(iscr1))
end if

if (dust_tog > 0) then
   do ib=1,dust_nb
      call fdback(a(idustncn(icm,ib)), a(idustncn(ifm,ib)), a(idn0n(icm)),  &
           a(idn0n(ifm)), gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, gd(ifm)%nzp,  &
           gd(ifm)%nxp, gd(ifm)%nyp, ifm, 't', a(iscr1))
   end do
end if

if (gd(icm)%nstbot == 1) then
   call botset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
        gd(icm)%nyp, icm, a(iu1), 'U')
   call botset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
        gd(icm)%nyp, icm, a(iv1), 'V')
   call botset_atm_mass(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1,  &
        gd(icm)%nxp, 1, gd(icm)%nyp, icm, a(ip1), a(ipi0n(icm)), 'P')
   call botset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
        gd(icm)%nyp, icm, a(it1), 'T')
   if (H2Ov_tog > 0) then
      call botset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
           gd(icm)%nyp, icm, a(ir1), 'T')    ! RV
   end if
   if (dust_tog > 0) then
      do ib=1,dust_nb
         call botset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp,  &
              1, gd(icm)%nyp, icm, a(idustncn(icm,ib)), 'A')   ! Dust
      end do
   end if
end if

if (gd(icm)%nsttop == 1) then
   call topset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
        gd(icm)%nyp, icm, a(iu1), 'U')
   call topset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
        gd(icm)%nyp, icm, a(iv1), 'V')
   call topset_atm_mass(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1,  &
        gd(icm)%nxp, 1, gd(icm)%nyp, icm, a(ip1), a(ipi0n(icm)), a(it1),  &
        a(irtgtn(icm)), 'P')

   iq = gd(icm)%nzp*gd(icm)%nxp*gd(icm)%nyp-1      !
   a(ipcn(icm):ipcn(icm)+iq) = a(ip1:ip1+iq) ! Fill a(ipc) for theta topset use
   call topset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
        gd(icm)%nyp, icm, a(it1), 'T')     ! Theta

   if (H2Ov_tog >= 1) then
      call topset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp, 1,  &
           gd(icm)%nyp, icm, a(ir1), 'S')    ! RV
   end if
   if (dust_tog > 0) then
      do ib=1,dust_nb
         call topset(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, 1, gd(icm)%nxp,  &
              1, gd(icm)%nyp, icm, a(idustncn(icm,ib)), 'A')    ! Dust
      end do
   end if
end if

end subroutine varfile_nstfeed
