!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!``````````````````````````````````````````````````````````````````
module PIPP_main_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine gen_varfile_name(fname, prefix, post, fmtstr, fID, lsubs, UT)

!+ Create standard, timestamped varfile name

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: fID
real(f4), intent(IN) :: lsubs, UT
character(len=*), intent(IN) :: prefix, post, fmtstr

character(len=*), intent(INOUT) :: fname

!== Local declarations:

integer :: ic, ols, oUT
character(len=40) :: dstring

!=== Executable statements:

ols = nint(100._f4*lsubs)    ! Change to integer output representation   
oUT = nint(100._f4*UT)    !
write (dstring, '(a3,i3.3,a1,i5.5,a1,i4.4)') '-V-', fID, '-', ols, '-', oUT
ic = 19   ! Length of dstring

fname = prefix(1:len_trim(prefix))//dstring(1:ic)
if (len_trim(post) > 0) then
   fname = fname(1:len_trim(fname))//'-'//post(1:len_trim(post))
end if

fname = fname(1:len_trim(fname))//'.'//trim(fmtstr)    ! Final filename

end subroutine gen_varfile_name


end module PIPP_main_mod


!```````````````````````````````````````````````````````````````````
subroutine PIPP_driver(nlst_fn)

!+ Prepare initialization files for PRAMS from model output

use univ_kind_defs_mod, only: f4, f8
use univ_math_constants_mod, only: ZERO_f8
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_std_fileIO_mod, only: mkdir_p, find_file_IOunit, determine_IO_reclen
use univ_exception_mod, only: exception, FATAL
use univ_GrADS_format_mod, only: init_GrADS_format, exit_GrADS_format,  &
     close_GrADS_dfile, write_GrADS_hfile, write_GrADS_dfile,  &
     open_GrADS_dfile
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use PIPP_format_mod, only: nx0, ny0, nz0, nt, lsubs_ts, utc_ts, PIPPf_r_head,  &
     PIPPf_dealloc, dstring
use main_memory_mod, only: there_is_an_error, a, gd, ztop, guess1st,  &
     PIPP_nvf, vf_Ls, tgt_time, tgt_tlon, iapr, maxnxp, maxnyp, nigrids, nzg,  &
     varpfx, gen_vf_input_comparison, start_UT, nf_span, target_Ls,  &
     undef_value_f4, igdref
use main_memory_bspec_mod, only: H2Ov_tog, dust_nb, dust_tog
use vtables_mod, only: ithpn, ippn, ipi0n, ith0n, irvn, idn0n, idn0un,  &
     idn0vn, iupn, ivpn, itoptn, irtgtn, ivt3da, ivt3db, ivt3dc, ivt3dp,  &
     idustncn
use body_specific_constants_mod, only: mnsdy
use PRAMS_chkcfg_mod, only: chkcfg4
use PIPP_main_mod, only: gen_varfile_name
use nest_interpolation_mod, only: fmrefs1d
use PIPP_in_mod, only: ingest_ts
use PIPP_time_mod, only: PIPP_tfloc
use varfile_init_mod, only: varfile_refstate, refstate_loc
use PIPP_intrp_mod, only: PRAMS_CartpreM_interp
use state_IO_mod, only: cfgctl_rw
use PRAMS_init_mem_mod, only: refs3d

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: nlst_fn

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'PIPP_driver'

real(f4), allocatable, dimension(:,:,:,:) :: gcm, gtr
real(f4), allocatable, dimension(:,:,:) :: datintrpu, datintrpv, datintrpt,  &
     datintrpp, gir, gt_sbs, gtMGhi
real(f4), allocatable, dimension(:,:) :: topoMG, gt, latMG, lonMG
real, allocatable, dimension(:) :: zlev_sbs      ! {nk_sbs+1}
character(len=MAX_FILE_PATH_LEN), allocatable, dimension(:) :: dfile ! [nf_span]

integer :: nd1, nd2, nd3, nd4, ntime, i, data_type, begt_ts, begf_ts,  &
     in_unit, out_unit, e, igd, n1, n2, n3, ntgt, it, tloc, floc, ib,  &
     nd3_sbs, idx_r, IO_reclen
real(f4) :: lsubs, UT
character(len=3) :: csuff
character(len=MAX_FILE_PATH_LEN) :: locfn
character(len=MAX_FILE_PATH_LEN), dimension(2) :: gdfile, ghfile
character(len=8), dimension(2) :: file_handle 
character(len=20), dimension(4) :: vname
real(f4), allocatable, dimension(:,:) :: reclen_test
real(f4), allocatable, dimension(:) :: Ls_series, UT_series
integer, dimension(4) :: nlevs
logical, dimension(4) :: in_file
logical :: wet_GCM

!=== Executable statements:

!*** Process/check input PIPP namelist:

there_is_an_error = .FALSE.
call chkcfg4                  ! Check for configuration problems
if (there_is_an_error) call exception(procedure_name, FATAL, 'chkcfg4')

!*** Which input data source? (GCM, PRAMS, etc.)

select case (trim(guess1st))

case ('GCM')   ! Using PIPP-formatted GCM output

   data_type = 1
   write (*, '(/,a,/)') 'Using PIPP-formatted GCM output ("GCM") '//  &
        'as input ...'

   !%%% Initialization stuff:

   allocate(dfile(nf_span), stat=e)     ! Allocate some stuff 
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'dfile')
         
   ! Read PIPP header(s) of GCM datafile(s):
   
   do it=1,nf_span
      write (dfile(it), '(a,a4)')  &          ! Create header filename
           iapr(it)(1:len_trim(iapr(it))), '.hdr'
   end do
   
   call PIPPf_r_head(dfile, nf_span)  ! Read PIPP header; initialize
                                      !  PIPPformat arrays

   ! Open special header file; read subsurface depth info:

   in_unit = find_file_IOunit()
   open(in_unit, file=dfile(1)(1:len_trim(dfile(1)))//'.tzl',  &
        status='old', form='unformatted')
   read (in_unit) nd3_sbs
   allocate(zlev_sbs(nd3_sbs+1), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'zlev_sbs')
   read (in_unit) zlev_sbs
   close(in_unit)

   select case (dstring(1)(1:2))
   case ('02', '75', '76', '77', '79')
      wet_GCM = .TRUE.   ! Input includes a H2O vapor field
   case ('01', '78')
      wet_GCM = .FALSE.    ! Input does NOT include a H2O vapor field
   case default
      wet_GCM = .FALSE.  ! Assume that input does NOT include a H2O vapor field
   end select

   ! Determine default starting point in input file(s), based on namelist
   !  parameters:

   i = sum(nt)
   allocate(Ls_series(i), UT_series(i), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'series')

   i = 0
   do it=1,nf_span   ! Loop over input file(s)
      Ls_series(i+1:i+nt(it)) = lsubs_ts(1:nt(it),it)
      UT_series(i+1:i+nt(it)) = utc_ts(1:nt(it),it)
      i = i+nt(it)
   end do

   call PIPP_tfloc(Ls_series, UT_series, nt, vf_Ls, tgt_time, tgt_tlon,  &
        nf_span, begt_ts, begf_ts, it, target_Ls, start_UT, 1)

   tloc = begt_ts        ! Set dataset starting point
   floc = begf_ts        !

   deallocate(Ls_series, UT_series, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'series')

   !  Fill dimension variables (for starting file):
   
   nd1 = nx0(floc)       ! # of x-dir GCM points
   nd2 = ny0(floc)       ! # of y-dir GCM points
   nd3 = nz0(1,floc)     ! # of vertical GCM points
      ! # of vars in 'gcm' array (including mixing ratio):
   nd4 = 6; if (wet_GCM .and. dust_tog > 0) nd4 = nd4+dust_nb

   ! Allocate some work arrays:

   allocate(gcm(nd1+1,nd2,nd3,nd4), gt(nd1+1,nd2), topoMG(nd1+1,nd2),  &
        gt_sbs(nd1+1,nd2,nd3_sbs), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'gcm_gt_topoMG')
   allocate(gtr(maxnxp,maxnyp,nzg,nigrids+1), gir(maxnxp,maxnyp,nigrids+1),  &
        gtMGhi(maxnxp,maxnyp,nd3_sbs+1), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'gtr_gir_gtMGhi')
   allocate(latMG(nd1+1,nd2), lonMG(nd1+1,nd2), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'latlonMG')
      
   ! Open PIPP format GCM datafile:

   write (dfile(floc), '(a,a4)')  &          ! Create datafile name
        iapr(floc)(1:len_trim(iapr(floc))), '.dat'
         
   in_unit = find_file_IOunit()   ! Input file unit to use

   allocate(reclen_test(nd1,nd2), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'reclen1')
   IO_reclen = determine_IO_reclen(reclen_test)
   deallocate(reclen_test, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'reclen1')

   open(in_unit, file=dfile(floc)(1:len_trim(dfile(floc))), status='old',  &
        form='unformatted', access='direct', recl=IO_reclen)

   if (gen_vf_input_comparison) then    ! Write comparison files
         
      ! Initialize GrADS format module:
      
      file_handle(1) = 'GCM'
      file_handle(2) = 'PRAMS'
      call init_GrADS_format(2, file_handle)
      
      ! Open GrADS data file(s) for writing:
   
      gdfile(1) = varpfx(1:len_trim(varpfx))//'-varcomp_GCM.dat'
      gdfile(2) = varpfx(1:len_trim(varpfx))//'-varcomp_PRAMS.dat'
      ghfile(1) = varpfx(1:len_trim(varpfx))//'-varcomp_GCM.ctl'
      ghfile(2) = varpfx(1:len_trim(varpfx))//'-varcomp_PRAMS.ctl'

      i = index(varpfx, '/', back=.TRUE.)-1  ! Determine directory name length
      call mkdir_p(varpfx(1:i))  ! Create any needed directories

      call open_GrADS_dfile(gdfile(1), gd(1)%nxp, gd(1)%nyp, 'W',  &
           file_handle(1))
      call open_GrADS_dfile(gdfile(2), gd(1)%nxp, gd(1)%nyp, 'W',  &
           file_handle(2))

   end if

case ('PRAMS')   ! Using previous PRAMS output

   data_type = 2
   write (*, '(/,a,/)') 'Using previous PRAMS output as input ...'

end select

!*** Begin loop over number of times to be processed:

i = index(varpfx, '/', back=.TRUE.)-1  ! Determine directory name length
call mkdir_p(varpfx(1:i))  ! Create any needed directories

out_unit = find_file_IOunit()  ! After `open_GrADS_file` to avoid unit conflicts

varf_num: do ntime=1,PIPP_nvf

   data_source: select case (data_type)
   
   case (1)      ! Using PIPP-formatted GCM output

      if (ntime /= 1) then  ! Don't do for initial varfile (already done above)
         tloc = tloc+1
         if (tloc > nt(floc)) then  ! Need to switch datafile
            tloc = 1
            floc = floc+1  ! 'PIPP_tfloc' has screened for pathological cases

            ! Fill dimension variables (for current file):
            nd1 = nx0(floc)       ! # of x-dir GCM points
            nd2 = ny0(floc)       ! # of y-dir GCM points
            nd3 = nz0(1,floc)     ! # of vertical GCM points

            ! Deallocate some work arrays:
            deallocate(gcm, gt, gt_sbs, gtMGhi, topoMG, latMG, lonMG, stat=e)
            if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
                 procedure_name, 'gcmlatlonMG')

            ! Allocate some work arrays:
            allocate(gcm(nd1+1,nd2,nd3,nd4), gt(nd1+1,nd2),  &
                 topoMG(nd1+1,nd2), gt_sbs(nd1+1,nd2,nd3_sbs),  &
                 gtMGhi(maxnxp,maxnyp,nd3_sbs+1), stat=e)
            if (e /= 0) call illegal_alloc_stop(for_allocation,  &
                 procedure_name, 'gcmMG')
            allocate(latMG(nd1+1,nd2), lonMG(nd1+1,nd2), stat=e)
            if (e /= 0) call illegal_alloc_stop(for_allocation,  &
                 procedure_name, 'latlonMG')
      
            ! Open PIPP format GCM datafile:
            write (dfile(floc), '(a,a4)')  &          ! Create datafile name
                 iapr(floc)(1:len_trim(iapr(floc))), '.dat'

            close(in_unit)     ! Close previous data file

            allocate(reclen_test(nd1,nd2), stat=e)
            if (e /= 0) call illegal_alloc_stop(for_allocation,  &
                 procedure_name, 'reclen2')
            IO_reclen = determine_IO_reclen(reclen_test)
            deallocate(reclen_test, stat=e)
            if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
                 procedure_name, 'reclen2')

            ! Open current data file:
            open(in_unit, file=dfile(floc)(1:len_trim(dfile(floc))),  &
                 status='old', form='unformatted', access='direct',  &
                 recl=IO_reclen)
         end if
      end if

      ! Update current input snapshot stuff:
      lsubs = lsubs_ts(tloc,floc); UT = utc_ts(tloc,floc)

      write (*, '(a,i5,a,f6.2,a,f5.2,a)')   &
           'Beginning processing for varfile #', ntime,  &
           '   [ Ls = ', lsubs, ',  ', UT, ' UT ] ...'

      ! Read GCM fields (on lat/lon grid) into work arrays:

      call ingest_ts(in_unit, tloc, floc, nd1, nd2, nd3, nd4, nd3_sbs, gcm,  &
           gt, gt_sbs, topoMG, latMG, lonMG, wet_GCM)

      grid_num: do igd=1,nigrids    ! Begin loop over number of PRAMS grids

         ! *** Prepare varfile data fields:
 
         call mvarfa(igd, nd1, nd2, nd3, nd4, nd3_sbs, gcm, topoMG, gt,  &
              gt_sbs, gtr(:,:,:,igd), gir(:,:,igd), gtMGhi, nzg, UT, latMG,  &
              lonMG, zlev_sbs, wet_GCM)
          
      enddo grid_num

   case (2)      ! Using previous PRAMS output
      
      continue      ! Nothing here yet
      
   end select data_source
            
   !*** Potentially do nesting feedback:

   if (nigrids > 1) then    ! Only do if a multigrid run

      !*** Fill reference states for all grids:

      ! Determine master 1D reference state location/grid:
      do igd=1,nigrids
         call refstate_loc(gd(igd)%nxp, gd(igd)%nyp, igd, a(itoptn(igd)),  &
              a(irtgtn(igd)), 1, igd == 1)
      end do

      igd = igdref(1)
      if (H2Ov_tog > 0) then
         idx_r = irvn(igd)
      else
         idx_r = ivt3dc
      end if

      ! Compute master 1D reference state (put in grid #1 arrays):
      call varfile_refstate(gd(igd)%nzp, gd(igd)%nxp, gd(igd)%nyp, igd, 1,  &
           a(ithpn(igd)), a(ippn(igd)), a(idx_r), a(iupn(igd)), a(ivpn(igd)), 1)

      ! Compute 3D reference state from 1D reference state (grid #1):
      call refs3D(gd(1)%nzp, gd(1)%nxp, gd(1)%nyp, 1, a(ipi0n(1)),  &
           a(idn0n(1)), a(idn0un(1)), a(idn0vn(1)), a(ith0n(1)),  &
           a(itoptn(1)), a(irtgtn(1)), 1)

      do i=0,gd(1)%nxyzp-1
         a(ippn(1)+i) = a(ippn(1)+i)-a(ipi0n(1)+i)
      end do

      call fmrefs1D(2, nigrids)

      do igd=2,nigrids

         call fmrefs3d(igd)
         call fmdn0(igd)

         do i=0,gd(igd)%nxyzp-1
            a(ippn(igd)+i) = a(ippn(igd)+i)-a(ipi0n(igd)+i)
         end do

      end do

      !*** Feed back u,v,pi,t,and rt fields:

      do igd=nigrids,2,-1
         call varfile_nstfeed(igd)
      end do

      do igd=1,nigrids
         do i=0,gd(igd)%nxyzp-1
            a(ippn(igd)+i) = a(ippn(igd)+i)+a(ipi0n(igd)+i)
         end do
      end do

   end if

   if (gen_vf_input_comparison) then
    ! PRAMSinit / input_data comparison:

      ntgt = (gd(1)%nxp+gd(1)%nyp+gd(1)%nzp)  ! Dimension of intrp gz array

      ! Allocate storage:

      allocate(datintrpu(gd(1)%nxp,gd(1)%nyp,ntgt),  &
           datintrpv(gd(1)%nxp,gd(1)%nyp,ntgt),  &
           datintrpt(gd(1)%nxp,gd(1)%nyp,ntgt),  &
           datintrpp(gd(1)%nxp,gd(1)%nyp,ntgt), stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'datintrp')

      ! Interpolate PRAMS data to "standard" geopotential levels:

      call PRAMS_CartpreM_interp(gd(1)%nzp, gd(1)%nxp, gd(1)%nyp, ntgt,  &
           a(iupn(1)), datintrpu, a(ivt3da), a(ivt3db), a(ivt3dc), a(ivt3dp),  &
           a(itoptn(1)), ztop, gd(1)%zt)
      call PRAMS_CartpreM_interp(gd(1)%nzp, gd(1)%nxp, gd(1)%nyp, ntgt,  &
           a(ivpn(1)), datintrpv, a(ivt3da), a(ivt3db), a(ivt3dc), a(ivt3dp),  &
           a(itoptn(1)), ztop, gd(1)%zt)
      call PRAMS_CartpreM_interp(gd(1)%nzp, gd(1)%nxp, gd(1)%nyp, ntgt,  &
           a(ithpn(1)), datintrpt, a(ivt3da), a(ivt3db), a(ivt3dc),  &
           a(ivt3dp), a(itoptn(1)), ztop, gd(1)%zt)
      call PRAMS_CartpreM_interp(gd(1)%nzp, gd(1)%nxp, gd(1)%nyp, ntgt,  &
           a(ippn(1)), datintrpp, a(ivt3da), a(ivt3db), a(ivt3dc), a(ivt3dp),  &
           a(itoptn(1)), ztop, gd(1)%zt)

      ! Write to GrADS format file:
      
      call write_GrADS_dfile(gd(1)%nxp, gd(1)%nyp, ntgt, datintrpu,  &
           'PRAMS   ') ! u
      call write_GrADS_dfile(gd(1)%nxp, gd(1)%nyp, ntgt, datintrpv,  &
           'PRAMS   ') ! v
      call write_GrADS_dfile(gd(1)%nxp, gd(1)%nyp, ntgt, datintrpt,  &
           'PRAMS   ') ! theta
      call write_GrADS_dfile(gd(1)%nxp, gd(1)%nyp, ntgt, datintrpp,  &
           'PRAMS   ') ! PI
      
      ! Deallocate storage:

      deallocate(datintrpu, datintrpv, datintrpt, datintrpp, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'datintrp')

   end if

   do igd=1,nigrids

      write (csuff, '(a1,i2.2)') 'g', igd     ! Create grid suffix

      n1 = gd(igd)%nzp; n2 = gd(igd)%nxp; n3 = gd(igd)%nyp

      allocate(reclen_test(n2,n3), stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'reclen3')
      IO_reclen = determine_IO_reclen(reclen_test)
      deallocate(reclen_test, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
           'reclen3')

      ! Create varfile name:
      call gen_varfile_name(locfn, varpfx, csuff, 'tfm', ntime-1, lsubs, UT)    
      open(out_unit, file=locfn(1:len_trim(locfn)), form='unformatted',  &
           status='replace', access='direct', recl=IO_reclen)

     ! Subsurface variables:
 
      do i=1,nzg
         write (out_unit, rec=i) gtr(1:n2,1:n3,i,igd)
      end do

     ! Supersurface variables:

      write (out_unit, rec=nzg+1) gir(1:n2,1:n3,igd)

      close(out_unit)

      allocate(reclen_test(n1,n2*n3), stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'reclen4')
      IO_reclen = determine_IO_reclen(reclen_test)
      deallocate(reclen_test, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
           'reclen4')

      ! Create varfile name:
      call gen_varfile_name(locfn, varpfx, csuff, 'vfm', ntime-1, lsubs, UT)
      open(out_unit, file=locfn(1:len_trim(locfn)), form='unformatted',  &
           status='replace', access='direct', recl=IO_reclen)

     ! "Regular" variables:

      write (out_unit, rec=1) a(iupn(igd):iupn(igd)+n1*n2*n3-1)
      write (out_unit, rec=2) a(ivpn(igd):ivpn(igd)+n1*n2*n3-1)
      write (out_unit, rec=3) a(ippn(igd):ippn(igd)+n1*n2*n3-1)
      write (out_unit, rec=4) a(ithpn(igd):ithpn(igd)+n1*n2*n3-1)
      if (H2Ov_tog > 0) then                              !
         write (out_unit, rec=5) a(irvn(igd):irvn(igd)+n1*n2*n3-1)
      end if                                              !
      if (dust_tog > 0) then                              !
         do ib=1,dust_nb                                 !
            write (out_unit, rec=5+ib)  &
                 a(idustncn(igd,ib):idustncn(igd,ib)+n1*n2*n3-1)
         end do                                           !
      end if                                              !       

      close(out_unit)

   end do

   ! *** Write varfile group/individual headers:  

   locfn = varpfx(1:len_trim(varpfx))//'-Vtime.tag' ! Timefile filename
   open(out_unit, file=locfn(1:len_trim(locfn)) ,status='unknown',  &
        form='formatted')    ! Open timefile
   ! Write timing data:
   write (out_unit, '(a4,e14.7,2x,a4,e14.7)') 'Ls= ', target_Ls, 'UT= ',  &
        start_UT
   close(out_unit)

   call gen_varfile_name(locfn, varpfx, '', 'tag', ntime-1, lsubs, UT)
   open(out_unit, file=locfn(1:len_trim(locfn)), form='FORMATTED',  &
        status='REPLACE')
   write (out_unit, '(i2,3x,2(f14.5),3x,i5,3x,a)') nigrids, lsubs, UT,  &
        ntime-1, trim(guess1st)    ! Write human-readable header file
   close(out_unit)

   call gen_varfile_name(locfn, varpfx, '', 'hdr', ntime-1, lsubs, UT)
   open(out_unit, file=locfn(1:len_trim(locfn)), form='UNFORMATTED',  &
        status='REPLACE')
   call cfgctl_rw(out_unit, 'W', 'V  ')           ! Write binary header file
   close(out_unit)

end do varf_num

close(in_unit)   ! Close input datafile

if (gen_vf_input_comparison) then     ! Finish up comparison files

   vname(1) = 'u'
   vname(2) = 'v'
   vname(3) = 'theta'
   vname(4) = 'pi'
   ntgt = gd(1)%nxp+gd(1)%nyp+gd(1)%nzp
   do i=1,4
      nlevs(i) = ntgt
      in_file(i) = .TRUE.
   end do
   
   do i=1,2
      
      ! Close GrADS data files:
      
      call close_GrADS_dfile(file_handle(i))
      
      ! Write GrADS header file
      
      call write_GrADS_hfile(gdfile(i), ghfile(i), gd(1)%nxp, gd(1)%nyp,  &
           PIPP_nvf, undef_value_f4, 0, 'green', a(ivt3dp:ivt3dp+ntgt-1),  &
           ZERO_f8, mnsdy/16._f8, start_UT, vname, nlevs, vname, vname,  &
           in_file, mnsdy)
      
   end do

   call exit_GrADS_format    ! Exit GrADS format module:

end if

! Alert user to times used:
write (*, '(/,a,f5.2,a,f6.2)') 'Using start UT = ', start_UT, ' and Ls = ',  &
     target_ls

!*** Deallocate memory used:

call PIPPf_dealloc()
deallocate(gcm, topoMG, gt, gt_sbs, gtMGhi, gtr, gir, latMG, lonMG, dfile,  &
     zlev_sbs, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'gcmMG_n_pts')

end subroutine PIPP_driver
