!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
module PIPP_time_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine PIPP_tfloc(Ls, UT, n_tf, appxtgt_Ls, appxtgt_LMST,  &
     lon_appxtgt_LMST, n_cfg_vf, begt_snap, begf_snap, abs_begt_snap,  &
     actual_start_Ls, actual_start_UT, snap_stride)

!+ Find the snapshot and file indices (and associated UT and Ls values) that
!   best correspond to the given namelist configuration parameters.

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_exception_mod, only: exception, FATAL
use univ_numtype_constants_mod, only: epsilon_f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

integer, intent(IN) :: n_cfg_vf, snap_stride
real(f4), intent(IN) :: appxtgt_Ls, appxtgt_LMST, lon_appxtgt_LMST
real(f4), dimension(:), intent(IN) :: UT
integer, dimension(:), intent(IN) :: n_tf
real(f4), dimension(:), intent(INOUT) :: Ls
integer, intent(OUT) :: begt_snap, begf_snap, abs_begt_snap
real(f4), intent(OUT) :: actual_start_Ls, actual_start_UT

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'PIPP_tfloc'

integer, dimension(size(Ls, 1)) :: tmp_idx

integer :: cnt, e, ib, it, n_t, i
real(f4) :: ftmp, Lsf_b, Lsf_e, Lsf_tgt, test1, test2, test3, dLs, UT_tgt

!=== Executable statements:

n_t = size(Ls, 1)

!=== May need to extrapolate/interpolate some Ls values so that there are no
!     duplicate values in the 'Ls' array:

! Determine whether there are any duplicate values in the 'Ls' array:
cnt = 0
do it=2,n_t
   if (abs(Ls(it)-Ls(it-1)) < epsilon_f4*1.e2_f4) then
      cnt = cnt+1; exit
   end if
end do

if (cnt > 0) then    ! Extrapolate/interpolate Ls
   ! Assume that if the first 'Ls' value is at UT~0., it is a robust datapoint:
   ib = 1; cnt = 0; tmp_idx = -1
   if (UT(1)-(UT(2)-UT(1)) < -epsilon_f4) then    ! On UT=0. boundary
      ib = 2; cnt = cnt+1; tmp_idx(cnt) = 1
   end if

   ! Find any other robust 'Ls' datapoints, flag others with a sentinel value:
   ftmp = Ls(1)
   do it=ib,n_t
      if (abs(ftmp-Ls(it)) < epsilon_f4*1.e2_f4) then
         Ls(it) = -999._f4
      else
         ftmp = Ls(it); cnt = cnt+1; tmp_idx(cnt) = it
      end if
   end do
   if (cnt < 2) call exception(procedure_name, FATAL,  &
        'Not enough Ls data to interpolate/extrapolate with -- '//  &
        'add one or more files to IAPR (in namelist).')

   do e=1,cnt-1
      ftmp = (Ls(tmp_idx(e+1))-Ls(tmp_idx(e)))/  &
           real(tmp_idx(e+1)-tmp_idx(e), kind=f4)    ! delta-Ls per snapshot
      if (e == 1 .and. ib == 1) then
       ! Extrapolate Ls to the beginning of the time-series (assuming a
       !  constant delta-Ls):
         do it=tmp_idx(e)-1,1,-1
            Ls(it) = Ls(it+1)-ftmp
         end do
      end if
      ! Where possible, interpolate Ls (assuming a constant delta-Ls):
      do it=tmp_idx(e)+1,tmp_idx(e+1)-1
         Ls(it) = Ls(it-1)+ftmp
      end do
   end do
   ! If needed, extrapolate Ls to the end of the time-series (assuming a
   !  constant delta-Ls):
   if (tmp_idx(cnt) < n_t) then
      do it=tmp_idx(cnt)+1,n_t
         Ls(it) = Ls(it-1)+ftmp
      end do
   end if
   ! Enforce a range of 0.-359.99999 degrees for Ls:
   where (Ls < ZERO_f4)
      Ls = Ls+360._f4
   elsewhere (Ls >= 360._f4)
      Ls = Ls-360._f4
   end where
end if

! Check if Lsf_tgt is within the Ls range of the input time-series:
Lsf_b = Ls(1)    ! Ls of first input snapshot
Lsf_e = Ls(n_t)                             !
if (Lsf_b > Lsf_e) Lsf_e = Lsf_e+360._f4    ! Ls of last input snapshot
Lsf_tgt = appxtgt_Ls                                  !
if (Lsf_b > appxtgt_Ls) Lsf_tgt = Lsf_tgt+360._f4  ! Target Ls
if (Lsf_tgt < Lsf_b .or. Lsf_tgt > Lsf_e) then
   write (*, '(/a,f6.2,a,f6.2,a,f6.2,a/)') 'APPROX_LS_VF1 (',  &
        appxtgt_Ls, ') is outside the Ls range of IAPR input file(s) (',  &
        Ls(1), ' to ', Ls(n_t), ').'
   call exception(procedure_name, FATAL)
end if

! Find the snapshot with the value *nearest* 'Lsf_tgt':
dLs = 9999._f4
do it=1,n_t
   test1 = Ls(it)
   if (test1 < Lsf_b) test1 = test1+360._f4   ! Make Ls "function" continuous
   test2 = test1-Lsf_tgt
   if (abs(test2) < dLs) then
      dLs = abs(test2); begt_snap = it
   end if
end do

! Calculate UT of 'appxtgt_LMST':
UT_tgt = mod(appxtgt_Ls-lon_appxtgt_LMST/15._f4+24._f4, 24._f4)

! Starting from the Ls-based snapshot index, determine the snapshot index with
!  UT *nearest* 'UT_tgt':
test3 = UT(begt_snap)-UT_tgt
if (UT_tgt < UT(begt_snap)) then
   test1 = 24._f4-test3; test2 = test3
else
   test1 = -test3; test2 = 24._f4+test3
end if
e = -1; ftmp = ZERO_f4  ! Initialize sum 
if (test1 < test2) then
   do it=begt_snap,n_t-1
      test3 = UT(it+1)-UT(it)
      if (UT(it+1) < UT(it)) test3 = test3+24._f4
      ftmp = ftmp+test3
      if (ftmp >= test1) then
         e = it; if ((ftmp-test2)/test3 <= 0.5_f4) e = e+1
         exit
      end if
   end do
else
   do it=begt_snap,2,-1
      test3 = UT(it)-UT(it-1)
      if (UT(it) < UT(it-1)) test3 = test3+24._f4
      ftmp = ftmp+test3
      if (ftmp >= test2) then
         e = it-1; if ((ftmp-test2)/test3 > 0.5_f4) e = e+1
         exit
      end if
   end do
end if
if (e < 0) call exception(procedure_name, FATAL,  &
     'Targeted input file snapshot exceeds range in input file(s); '//  &
     'please adjust APPROX_LS_VF1, APPROX_LMST_VF1, LON_LMST_VF1, and/or IAPR.')
abs_begt_snap = e

actual_start_Ls = Ls(e); actual_start_UT = UT(e)
write (*, '(/,a,f5.2,a,f6.2)') 'Using start UT = ', actual_start_UT,  &
     ' and Ls = ', actual_start_Ls

! Determine which file/snapshot indices the targeted snapshot is associated
!  with:
cnt = 1
do i=1,size(n_tf, 1)
   ib = cnt+n_tf(i)
   if (e >= cnt .and. e < ib) then
      begt_snap = e-cnt+1; exit
   end if
   cnt = ib
end do
begf_snap = i

! Check if enough timesteps exist in file(s) to make `n_cfg_vf` varfiles:
if ((n_t-e)/snap_stride+1 < n_cfg_vf) then
   write (*, '(/,a,i4,a,i4,a)') 'Configured to make ', n_cfg_vf,  &
        ' varfiles, but only ', (n_t-e)/snap_stride+1,  &
        ' input snapshots are available.  Modify &PIPP_control and/or '//  &
        '&PIPP_control_bspec namelist parameters.'
   call exception(procedure_name, FATAL)
end if

end subroutine PIPP_tfloc


!```````````````````````````````````````````````````````````````````
subroutine PIPP_LSTcalc(m1,m2,glon,LST,curr_utc)

!+ Calculate LST for all points on PRAMS grid

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2
real(f4), dimension(m1,m2), intent(IN) :: glon
real(f4), intent(IN) :: curr_utc

real(f4), dimension(m1,m2), intent(INOUT) :: LST

!== Local declarations:

real(f4) :: ltmp
integer :: i,j

!=== Executable statements:

do j=1,m2
   do i=1,m1
      ltmp = curr_utc+(glon(i,j)/15._f4)
      if(ltmp >= 24._f4) then
         ltmp = ltmp-24._f4
      elseif(ltmp < ZERO_f4) then
         ltmp = ltmp+24._f4
      endif
      LST(i,j) = ltmp
   enddo
enddo

end subroutine PIPP_LSTcalc


end module PIPP_time_mod
