!```````````````````````````````````````````````````````````````````
module CFL_mod

!+ Module that contains CFL-violation-related routines

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine CFL_comp(n1, n2, n3, up, vp, wp, rtgt, f13t, f23t, dxt, dyt, mi0,  &
     mj0)

!+ Computes Courant numbers; warns user and/or stops model if criterion is
!   exceeded

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use main_memory_mod, only: vctr1, vctr2, vctr3, jdim, ztop, i_grid, c_gd,  &
     Cn_warning_threshold, enable_advection, prognose_uvwp

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, mi0, mj0
real(f4), dimension(n1,n2,n3), intent(IN) :: up, vp, wp
real(f4), dimension(n2,n3), intent(IN) :: rtgt, f13t, f23t, dxt, dyt

!== Local declarations:

integer :: i, j, k
real(f4) :: ht_rtgt, c1x, c1y, c1z, C_num_xyz

!=== Executable statements:

do j=1+jdim,n3-jdim
   do i=2,n2-1

      ! If this gridpoint is "under" a finer grid, ignore:       
      if (c_gd%Cn_mask(i,j)) cycle   ! On to the next gridpoint

      do k=2,n1-1
         vctr1(k) = 0.5_f4*(up(k,i,j)+up(k,i-1,j))*c_gd%dt_dyn*dxt(i,j)
         vctr2(k) = 0.5_f4*(vp(k,i,j)+vp(k,i,j-jdim))*c_gd%dt_dyn*dyt(i,j)
         ht_rtgt = rtgt(i,j)*c_gd%zt(k)/ztop-ONE_f4
         vctr3(k) = ((wp(k,i,j)+wp(k-1,i,j))+  &
              (up(k,i,j)+up(k,i-1,j))*f13t(i,j)*ht_rtgt+  &
              (vp(k,i,j)+vp(k,i,j-jdim))*f23t(i,j)*ht_rtgt)*  &
              0.5_f4*c_gd%dt_dyn*c_gd%dzt(k)
!print *,'cfl',i,j,k,wp(k,i,j),wp(k-1,i,j),up(k,i,j),up(k,i-1,j),f13t(i,j),  &
!     ht_rtgt,vp(k,i,j),vp(k,i,j-jdim),f23t(i,j),c_gd%dt_dyn,c_gd%dzt(k)!vctr1(k),vctr2(k),vctr3(k)
      end do

      do k=2,n1-1
         c1x = abs(vctr1(k)); c1y = abs(vctr2(k)); c1z = abs(vctr3(k))

         C_num_xyz = c1x+c1y+c1z
         c_gd%Courant_number_xyz = max(c_gd%Courant_number_xyz, C_num_xyz)
 
         if (enable_advection .or. prognose_uvwp) then  ! Only if needed
            if (C_num_xyz > Cn_warning_threshold) then
               write (*, '(a,f5.2,a2,f5.2,a2,f5.2,a,i2,3(a,i4),a1)')  &
                    'CFL warning: ', c1z, ', ', c1x, ', ', c1y,  &
                    ' on grid #', i_grid,  &
                    ' at gridpoint (', k, ',', i+mi0, ',', j+mj0, ')'
            end if
         end if
      end do

   end do
end do

end subroutine CFL_comp


end module CFL_mod


!```````````````````````````````````````````````````````````````````
subroutine cfl(n1, n2, n3, mi0, mj0, do_initialization)

!+ Driver for Courant number computation and CFL violation check

use main_memory_mod, only: n_grids, run_is_purely_2D, a, i_grid, jdim, gd, c_gd
use vtables_mod, only: iupn, ivpn, iwpn, irtgtn, if13tn, if23tn, idxtn, idytn
use CFL_mod, only: CFL_comp

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, mi0, mj0 
logical, intent(IN) :: do_initialization

!== Local declarations:

integer :: i, j, ifm

!=== Executable statements:

if (do_initialization) then
   c_gd%Cn_mask(:,:) = .FALSE.

   do j=1+jdim,n3-jdim
      do i=2,n2-1
         ! If this gridpoint is "under" a finer grid, ignore:       
         if (i_grid < n_grids) then
            do ifm=i_grid+1,n_grids
               if (gd(ifm)%nxtnest == i_grid .and.  &
                  i+mi0 >= gd(ifm)%ipm(1) .and.  &
                  i+mi0 <= gd(ifm)%ipm(gd(ifm)%nxp) .and.  &
                  j+mj0 >= gd(ifm)%jpm(1) .and.  &
                  j+mj0 <= gd(ifm)%jpm(gd(ifm)%nyp)) then
                  c_gd%Cn_mask(i,j) = .TRUE.
               end if
            end do
         end if
      end do
   end do

   do j=1,n3
      c_gd%Cn_mask(1,j) = .TRUE.
      c_gd%Cn_mask(n2,j) = .TRUE.
   end do

   if (.NOT.run_is_purely_2D) then
      do i=1,n2
         c_gd%Cn_mask(i,1) = .TRUE.
         c_gd%Cn_mask(i,n3) = .TRUE.
      end do
   end if
end if

call CFL_comp(n1, n2, n3, a(iupn(i_grid)), a(ivpn(i_grid)), a(iwpn(i_grid)),  &
     a(irtgtn(i_grid)), a(if13tn(i_grid)), a(if23tn(i_grid)),  &
     a(idxtn(i_grid)), a(idytn(i_grid)), mi0, mj0)

end subroutine cfl
