!```````````````````````````````````````````````````````````````````
module PRAMS_HH_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine fill_sounding

!+ Parse/fill sounding arrays with all needed info 

use univ_kind_defs_mod, only: f4
use univ_convert_units_mod, only: deg_to_rad_f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use main_memory_mod, only: vctr4
use main_memory_bspec_mod, only: hsnd, psnd, usnd, vsnd, tsnd, rtsnd, nsndp,  &
     iusflg, ipsflg, irtsflg, itsflg, thdsnd
use body_specific_constants_mod, only: g_Xoid, R_air, rocp, cp, cpor,  &
     p00i, p00

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ns
real(f4) :: zold2,zold1,dir,spd,tavg

!=== Executable statements:

! If namelist flag set, then read sounding from other file:

if(psnd(1) < ZERO_f4) then

   ! Zero out unwanted namelist values first:

   do ns=1,nsndp
      hsnd(ns) = ZERO_f4
      psnd(ns) = ZERO_f4
      tsnd(ns) = ZERO_f4
      rtsnd(ns) = ZERO_f4
      usnd(ns) = ZERO_f4
      vsnd(ns) = ZERO_f4
   enddo

   ! Read in new values:

   open(18,file='SND_IN',status='old',form='formatted')
   do ns=1,nsndp
      read (18,*) hsnd(ns),psnd(ns),tsnd(ns),rtsnd(ns),usnd(ns),vsnd(ns)
   enddo
   close(18)

endif

! Parse and fill each sounding level/field:

do ns=1,nsndp

   if(iusflg == 1) then
      dir = usnd(ns)
      spd = vsnd(ns)
      usnd(ns) = -spd*sin(deg_to_rad_f4*dir)
      vsnd(ns) = -spd*cos(deg_to_rad_f4*dir)
   endif

   if(ipsflg /= 1) then   ! PSND() = pressure [mb]

      psnd(ns) = psnd(ns)*1.e2_f4   ! Convert pressure from [mb] to [Pa]

   else         ! PSND(1) = sfc pressure [mb]

      !  If sounding moisture is expressed as a mixing ratio,
      !   take advantage of knowing virtual temperature effect when
      !   integrating hydrostatically to get sounding pressure:

      if(irtsflg == 0)then
         rtsnd(ns) = rtsnd(ns)*1.e-3_f4   ! Convert [g/kg] to [kg/kg]
      endif
      vctr4(ns) = (ONE_f4+0.61_f4*rtsnd(ns))   ! Virtual temperature factor

      if(ns == 1) then    ! Know pressure here already
         psnd(ns) = psnd(ns)*100._f4    ! Convert pressure from [mb] to [Pa]
         zold2 = ZERO_f4
      else        ! Need to compute hydrostatic pressure here
         zold1 = zold2
         zold2 = hsnd(ns)
         if(itsflg == 0) then   ! Using temperature [K]
            tavg = 0.5_f4*(tsnd(ns)*vctr4(ns)+tsnd(ns-1)*vctr4(ns-1))
            psnd(ns) = psnd(ns-1)*exp(-g_Xoid*(zold2-zold1)/(R_air*tavg))
         elseif(itsflg == 1) then      ! Using potential temperature [K]
            tavg = (tsnd(ns)*vctr4(ns)+tsnd(ns-1)*vctr4(ns-1)*(p00**rocp)  &
                 /psnd(ns-1)**rocp)*0.5_f4
            psnd(ns) = (psnd(ns-1)**rocp-g_Xoid*(zold2-zold1)*(p00**rocp)/  &
                 (cp*tavg))**cpor
         endif
      endif

   endif

   if(itsflg == 1) then
      ! Convert potential temperature [K] to temperature [K]
      tsnd(ns) = (psnd(ns)*p00i)**rocp*tsnd(ns)
   endif

enddo

if(ipsflg == 2) then   ! Compute geopotential height levels of input sounding:
   hsnd(1) = ZERO_f4
   do ns=2,nsndp
      hsnd(ns) = hsnd(ns-1)-R_air*0.5_f4*(tsnd(ns)*(ONE_f4+0.61_f4*rtsnd(ns))  &
           +tsnd(ns-1)*(ONE_f4+0.61_f4*rtsnd(ns-1)))*(log(psnd(ns))-log(psnd(ns-1)))/g_Xoid
   enddo
endif

do ns=1,nsndp
   thdsnd(ns) = tsnd(ns)*(p00/psnd(ns))**rocp
enddo

end subroutine fill_sounding


!```````````````````````````````````````````````````````````````````
subroutine sounding_refs1D(n2, n3, igd_ref, igd, topt, iq_ref)

!+ Computes 1D reference state from input sounding 

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_exception_mod, only: exception, FATAL
use univ_interpolation_mod, only: intrp_1D_arr_type01a
use main_memory_mod, only: gd, vctr1, vctr4, rtgref, vctr2
use main_memory_bspec_mod, only: thdsnd, hsnd, nsndp, usnd, vsnd, rtsnd,  &
     psnd, H2Ov_tog, gd_bd
use body_specific_constants_mod, only: g_Xoid, cp, p00i, rocp, cpor, p00,  &
     R_air

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd, n2, n3, iq_ref, igd_ref
real(f4), dimension(n2,n3), intent(IN) :: topt

!== Local declarations:

integer :: k
real(f4) :: d_z

!=== Executable statements:

do k=1,gd(igd)%nzp    ! Heights (AGL) of reference point column
   vctr2(k) = gd(igd_ref)%zt(k)*rtgref(iq_ref) 
end do

! Check if sounding goes high enough:
if (vctr2(gd(igd)%nzp) > hsnd(nsndp)) then
   write (*, '(a,/a,f12.2,a,f12.2)') 'Input sounding needs to go higher!',  &
        '    Sounding top [m] = ', hsnd(nsndp), ', Model top [m] = ',  &
        vctr2(gd(igd)%nzp)
   call exception('sounding_refs1D', FATAL)
end if

! Interpolate sounding values to reference point column heights:

!call htint(nsndp,thdsnd,hsnd,nnzp(grid),vctr1,ztn(1,grid))
vctr1(1:gd(igd)%nzp) = intrp_1D_arr_type01a(thdsnd(1:nsndp), hsnd(1:nsndp),  &
                                           vctr2(1:gd(igd)%nzp))
!#call htint(nsndp,thdsnd,hsnd,nnzp(grid),vctr1,vctr2)
!call htint(nsndp,usnd,hsnd,nnzp(grid),u01dn(1,grid),ztn(1,grid))
gd(igd)%u01d(1:gd(igd)%nzp) = intrp_1D_arr_type01a(usnd(1:nsndp), hsnd(1:nsndp),  &
                                                vctr2(1:gd(igd)%nzp))
!#call htint(nsndp,usnd,hsnd,nnzp(grid),u01dn(1,grid),vctr2)
!call htint(nsndp,vsnd,hsnd,nnzp(grid),v01dn(1,grid),ztn(1,grid))
gd(igd)%v01d(1:gd(igd)%nzp) = intrp_1D_arr_type01a(vsnd(1:nsndp), hsnd(1:nsndp),  &
                                                vctr2(1:gd(igd)%nzp))
!#call htint(nsndp,vsnd,hsnd,nnzp(grid),v01dn(1,grid),vctr2)

if (H2Ov_tog >= 1) then
!   call htint(nsndp,rtsnd,hsnd,nnzp(grid),rt01dn(1,grid),ztn(1,grid))
   gd_bd(igd)%rt01d(1:gd(igd)%nzp) = intrp_1D_arr_type01a(rtsnd(1:nsndp),  &
        hsnd(1:nsndp), vctr2(1:gd(igd)%nzp))
!#   call htint(nsndp,rtsnd,hsnd,nnzp(grid),rt01dn(1,grid),vctr2)
else
   gd_bd(igd)%rt01d(1:gd(igd)%nzp) = ZERO_f4
end if

! Convert theta to theta-v:

do k=1,gd(igd)%nzp
!%   th01dn(k,grid) = vctr1(k)*(1.+0.61*rt01dn(k,grid))
   gd(igd)%th01d(k) = vctr1(k)
end do

gd(igd)%u01d(1) = gd(igd)%u01d(2)    !
gd(igd)%v01d(1) = gd(igd)%v01d(2)    ! To be consistent with model BCs???
gd_bd(igd)%rt01d(1) = gd_bd(igd)%rt01d(2)  !
gd(igd)%th01d(1) = gd(igd)%th01d(2)  !

! Hydrostatic integration to obtain PI01 values:

!%pi01dn(1,grid) = cp*(psnd(1)*p00i)**rocp+g*(hsnd(1)-ztn(1,grid))  &
!%     /(0.5*(th01dn(1,grid)+thdsnd(1)*(1.+0.61*rtsnd(1))))
gd(igd)%pi01d(1) = cp*(psnd(1)*p00i)**rocp+g_Xoid*(hsnd(1)-vctr2(1))/  &
     (0.5_f4*(gd(igd)%th01d(1)+thdsnd(1)))
do k=2,gd(igd)%nzp
   d_z = rtgref(iq_ref)/gd(igd_ref)%dzm(k-1)
   gd(igd)%pi01d(k) = gd(igd)%pi01d(k-1)-g_Xoid*d_z/  &
        (0.5_f4*(gd(igd)%th01d(k)+gd(igd)%th01d(k-1)))
end do

! Calculate 1D refstate density from PI01 and TH01:

do k=1,gd(igd)%nzp
   vctr4(k) = (gd(igd)%pi01d(k)/cp)**cpor*p00
   gd(igd)%dn01d(k) = cp*vctr4(k)/(R_air*gd(igd)%th01d(k)*gd(igd)%pi01d(k))
end do

end subroutine sounding_refs1D


!```````````````````````````````````````````````````````````````````
subroutine init_fields_HH(n1,n2,n3,igd,uc,vc,pi0,theta,thp,pc,rv,topt  &
     ,topu,topv,rtgt,rtgu,rtgv,iq_ref)

!+ Initializes model fields using model reference state

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_interpolation_mod, only: intrp_1D_arr_type01a
use main_memory_mod, only: vctr5, vctr6, vctr3, vctr2, vctr11, vctr12,  &
     vctr13, gd, rtgref, topref, jdim, map_projection
use main_memory_bspec_mod, only: H2Ov_tog, gd_bd
use univ_OPS_proj_mod, only: xy_ll_OPS
use body_specific_constants_mod, only: radius
use polarst_mod, only: uevetouv

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1,n2,n3,igd,iq_ref
real(f4), dimension(n1,n2,n3), intent(IN) :: pi0
real(f4), dimension(n2,n3), intent(IN) :: topt,topu,topv,rtgt,rtgu,rtgv

real(f4), dimension(n1,n2,n3), intent(INOUT) :: uc,vc,pc,theta,thp,rv

!== Local declarations:

integer :: i,j,k
real(f4) :: qlatu,qlatv,qlonu,qlonv,dummy

!=== Executable statements:

do j=1,n3
   do i=1,n2

      do k=1,n1
         vctr2(k) = gd(igd)%zt(k)*rtgref(iq_ref)      ! z (AGL) of 1D refstate
!         vctr11(k) = ztn(k,ng)*rtgt(i,j)+topt(i,j)
         vctr11(k) = gd(igd)%zt(k)*rtgt(i,j)+(topt(i,j)-topref(iq_ref))
!         vctr12(k) = ztn(k,ng)*rtgu(i,j)+topu(i,j)
!@         vctr12(k) = ztn(k,ng)*rtgu(i,j)+(topu(i,j)-topref(iq_ref))
!         vctr13(k) = ztn(k,ng)*rtgv(i,j)+topv(i,j)
!@         vctr13(k) = ztn(k,ng)*rtgv(i,j)+(topv(i,j)-topref(iq_ref))
      enddo

!      call htint(n1,u01dn(1,ng),ztn(1,ng),n1,vctr5,vctr12)
!@      call htint(n1,u01dn(1,ng),vctr2,n1,vctr5,vctr12)
vctr5(1:n1) = intrp_1D_arr_type01a(gd(igd)%u01d(1:n1), vctr2(1:n1), vctr11(1:n1))
!#      call htint(n1,u01dn(1,ng),vctr2,n1,vctr5,vctr11)    ! At "T-point"
!      call htint(n1,v01dn(1,ng),ztn(1,ng),n1,vctr6,vctr13)
!@      call htint(n1,v01dn(1,ng),vctr2,n1,vctr6,vctr13)
vctr6(1:n1) = intrp_1D_arr_type01a(gd(igd)%v01d(1:n1), vctr2(1:n1), vctr11(1:n1))
!#      call htint(n1,v01dn(1,ng),vctr2,n1,vctr6,vctr11)    ! At "T-point"
!      call htint(n1,th01dn(1,ng),ztn(1,ng),n1,vctr3,vctr11)
vctr3(1:n1) = intrp_1D_arr_type01a(gd(igd)%th01d(1:n1), vctr2(1:n1), vctr11(1:n1))
!#      call htint(n1,th01dn(1,ng),vctr2,n1,vctr3,vctr11)
      if (H2Ov_tog >= 1) then
!         call htint(n1,rt01dn(1,ng),ztn(1,ng),n1,rv(1,i,j),vctr11)
         rv(1:n1,i,j) = intrp_1D_arr_type01a(gd_bd(igd)%rt01d(1:n1), vctr2(1:n1),  &
                                             vctr11(1:n1))
!#         call htint(n1,rt01dn(1,ng),vctr2,n1,rv(1,i,j),vctr11)
      end if

      ! If sounding winds are to be interpreted as eastward (U) and
      ! northward (V) components, rotate winds from geographic to
      ! polar stereographic orientation:
      !
      ! The below assumes that U and V are both valid at a "T-point" (good
      ! assumption for a sounding); will need to be averaged to C-grid
      ! stagger later:

      if (map_projection == 1) then
         
         call xy_ll_OPS(qlatu, qlonu, gd(igd)%plat, gd(igd)%plon,  &
              gd(igd)%xt(i), gd(igd)%yt(j), radius)
         call xy_ll_OPS(qlatv, qlonv, gd(igd)%plat, gd(igd)%plon,  &
              gd(igd)%xt(i), gd(igd)%yt(j), radius)

         do k=1,n1
            call uevetouv(uc(k,i,j), dummy, vctr5(k), vctr6(k), qlatu, qlonu,  &
                 gd(igd)%plat, gd(igd)%plon)
            call uevetouv(dummy, vc(k,i,j), vctr5(k), vctr6(k), qlatv, qlonv,  &
                 gd(igd)%plat, gd(igd)%plon)
         enddo

      else
         do k=1,n1
            uc(k,i,j) = vctr5(k)
            vc(k,i,j) = vctr6(k)
         enddo
      endif

!%      if(iaertog == 1) then
!%         do k=1,n1
!%            thp(k,i,j) = vctr3(k)/(1.+0.61*rv(k,i,j))
!%            rv(k,i,j) = rtp(k,i,j)
!%         enddo
!%      else
         do k=1,n1
            thp(k,i,j) = vctr3(k)
         enddo
!%      endif
      
      do k=1,n1
         theta(k,i,j) = thp(k,i,j)
         pc(k,i,j) = ZERO_f4
      enddo

   enddo
enddo

! Average U and V to C-grid stagger (from A-grid):

do j=1,n3
   do i=1,gd(igd)%nx
      do k=1,n1
         uc(k,i,j) = (uc(k,i,j)+uc(k,i+1,j))*0.5_f4
      enddo
   enddo
enddo

if(jdim == 1) then
   do j=1,gd(igd)%ny
      do i=1,n2
         do k=1,n1
            vc(k,i,j) = (vc(k,i,j)+vc(k,i,j+1))*0.5_f4
         enddo
      enddo
   enddo
else
   do j=1,gd(igd)%ny
      do i=1,n2
         do k=1,n1
            vc(k,i,j) = vc(k,i,j)
         enddo
      enddo
   enddo
endif

end subroutine init_fields_HH


end module PRAMS_HH_mod


!```````````````````````````````````````````````````````````````````
subroutine init_HH

!+ Initialize model grid, topography, etc. for horizontally homogeneous fields

use main_memory_mod, only: n_grids, gd, a, igdref
use main_memory_bspec_mod, only: H2Ov_tog
use vtables_mod, only: ipi0n,idn0n,idn0un,idn0vn,ith0n,itoptn,irtgtn,iucn,ivcn  &
     ,ithetan,ithpn,ipcn,irvn,itopun,itopvn,irtgun,irtgvn
use PRAMS_HH_mod, only: init_fields_HH, fill_sounding, sounding_refs1d
use PRAMS_init_mem_mod, only: refs3d
use varfile_init_mod, only: refstate_loc

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ifm, icm, iq_ref, itmp, igd

!=== Executable statements:

! Fill sounding arrays with all needed info:

call fill_sounding

! For grid 1, and the second hemispheric grid if applicable,
!  compute the 1-D reference state variables, the 3-D
!  reference state variables, the 3-D model fields, the surface
!  layer parameters, and the initial soil model fields:

iq_ref = 1

   ! Determine master 1D reference state location/grid:
do igd=1,n_grids
   call refstate_loc(gd(igd)%nxp, gd(igd)%nyp, igd, a(itoptn(igd)),  &
        a(irtgtn(igd)), iq_ref, igd == 1)
end do

   ! Compute master 1D reference state (put in grid #1 arrays):
igd = igdref(iq_ref)
call sounding_refs1D(gd(igd)%nxp, gd(igd)%nyp, igd, 1, a(itoptn(igd)), iq_ref)

do ifm=1,n_grids
   icm = gd(ifm)%nxtnest

   if (icm == 0) then
      call refs3D(gd(ifm)%nzp,gd(ifm)%nxp,gd(ifm)%nyp,ifm,a(ipi0n(ifm))  &
           ,a(idn0n(ifm)),a(idn0un(ifm)),a(idn0vn(ifm)),a(ith0n(ifm))  &
           ,a(itoptn(ifm)),a(irtgtn(ifm)),iq_ref)    ! 3D refstate

      if(H2Ov_tog > 0) then
         itmp = irvn(ifm)
      else
         itmp = 1
      endif
      call init_fields_HH(gd(ifm)%nzp,gd(ifm)%nxp,gd(ifm)%nyp,ifm,a(iucn(ifm))  &
           ,a(ivcn(ifm)),a(ipi0n(ifm)),a(ithetan(ifm)),a(ithpn(ifm))  &
           ,a(ipcn(ifm)),a(itmp),a(itoptn(ifm))  &
           ,a(itopun(ifm)),a(itopvn(ifm)),a(irtgtn(ifm)),a(irtgun(ifm))  &
           ,a(irtgvn(ifm)),iq_ref)    ! Initialize model fields

   end if
end do

end subroutine init_HH
