!```````````````````````````````````````````````````````````````````
module PRAMS_coriolis_mod

!~ 24.09.2008 (last modified; TIM)

!+

implicit none

private

public :: coriolis_u, coriolis_v, f_coriolis

contains


!```````````````````````````````````````````````````````````````````
subroutine f_coriolis(n2,n3,nx,ny,fcoru,fcorv,glat)

!+ Calculates the Coriolis parameter

!~ 21.01.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4
use univ_convert_units_mod, only: deg_to_rad_f4
use main_memory_mod, only: icorflg, gd, jdim
use body_specific_constants_mod, only: omega

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2,n3,nx,ny
real(f4), dimension(n2,n3), intent(IN) :: glat

real(f4), dimension(n2,n3), intent(INOUT) :: fcoru, fcorv

!== Local declarations:

integer :: i,j
real(f4) :: omega2

!=== Executable statements:

omega2 = 2._f4*omega

if(icorflg == 1) then       ! Coriolis parameter fully variable
   do j=1,ny
      do i=1,nx
         fcoru(i,j) = omega2*sin((glat(i,j)+glat(i+1,j))*0.5_f4*deg_to_rad_f4)
         fcorv(i,j) = omega2*sin((glat(i,j)+glat(i,j+jdim))*0.5_f4*deg_to_rad_f4)
      enddo
   enddo
elseif(icorflg == 2) then ! Coriolis parameter fixed at CENTLAT of coarse grid
   fcoru(1:nx,1:ny) = omega2*sin(gd(1)%centlat*deg_to_rad_f4)
   fcorv(1:nx,1:ny) = omega2*sin(gd(1)%centlat*deg_to_rad_f4)
endif

end subroutine f_coriolis


!```````````````````````````````````````````````````````````````````
subroutine coriolis_U(m1,m2,m3,i0,j0,ia,iz,ja,jz,UP,VP,UT,scr,TOP,RTG,FCOR)

!+ Compute Coriolis accelerations, update tendency (U-points)

!~ 21.01.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_interpolation_mod, only: intrp_1D_arr_type01a
use body_specific_constants_mod, only: radius
use main_memory_mod, only: vctr2, jdim, itopo, c_gd, vctr5,  &
     map_projection, initialization_type

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2,m3,i0,j0,ia,iz,ja,jz
real(f4), dimension(m1,m2,m3), intent(IN) :: UP,VP
real(f4), dimension(m2,m3), intent(IN) :: TOP,RTG,FCOR

real(f4), dimension(m1,m2,m3), intent(INOUT) :: UT,scr

!== Local declarations:

integer :: i,j,k
real(f4) :: c1

!=== Executable statements:

DO J=ja,jz
   DO I=ia,iz
      DO K=2,m1-1
         scr(K,I,J) = (VP(K,I,J)+VP(K,I,J-jdim)  &
              +VP(K,I+1,J)+VP(K,I+1,J-jdim))*0.25_f4
      ENDDO
   ENDDO
ENDDO

if (map_projection /= 0) then
   c1 = ONE_f4/(radius*radius*2._f4)
else
   c1 = ZERO_f4
end if

DO J=ja,jz
   DO I=ia,iz
      DO K=2,m1-1
         UT(K,I,J) = UT(K,I,J)-scr(K,I,J)*(-FCOR(I,J)  &
              +C1*(scr(K,I,J)*c_gd%XM(I+i0)-UP(K,I,J)*c_gd%YT(J+j0)))
      ENDDO
   ENDDO
ENDDO

if (initialization_type == 2) return

DO J=ja,jz
   DO I=ia,iz
      IF(itopo == 1) THEN
         DO K=1,m1
            VCTR2(K) = c_gd%ZT(K)*RTG(I,J)+TOP(I,J)
         ENDDO
         vctr5(1:c_gd%nz) = intrp_1D_arr_type01a(c_gd%v01d(1:c_gd%nzp),  &
              c_gd%zt(1:c_gd%nzp), vctr2(1:c_gd%nz))
!         CALL HTINT(NZP,V01DN(1,grid),ZT,NZ,VCTR5,VCTR2)
         DO K=2,m1-1
            UT(K,I,J) = UT(K,I,J)-FCOR(I,J)*VCTR5(K)
         ENDDO
      ELSE
         DO K=2,m1-1
            UT(K,I,J)=UT(K,I,J)-FCOR(I,J)*c_gd%V01D(K)
         ENDDO
      ENDIF
   ENDDO
ENDDO

end subroutine coriolis_U


!```````````````````````````````````````````````````````````````````
subroutine coriolis_V(m1,m2,m3,i0,j0,ia,iz,ja,jz,UP,VP,VT,scr,TOP,RTG,FCOR)

!+ Compute Coriolis accelerations, update tendency (V-points) 

!~ 21.01.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_interpolation_mod, only: intrp_1D_arr_type01a
use body_specific_constants_mod, only: radius
use main_memory_mod, only: vctr2, jdim, itopo, c_gd, vctr5,  &
     map_projection, initialization_type

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2,m3,i0,j0,ia,iz,ja,jz
real(f4), dimension(m1,m2,m3), intent(IN) :: UP,VP
real(f4), dimension(m2,m3), intent(IN) :: TOP,RTG,FCOR

real(f4), dimension(m1,m2,m3), intent(INOUT) :: vt,scr

!== Local declarations:

integer :: i,j,k
real(f4) :: c1

!=== Executable statements:

DO J=ja,jz
   DO I=ia,iz
      DO K=2,m1-1
         scr(K,I,J) = (UP(K,I,J)+UP(K,I-1,J)  &
              +UP(K,I,J+jdim)+UP(K,I-1,J+jdim))*0.25_f4
      ENDDO
   ENDDO
ENDDO

if (map_projection /= 0) then
   c1 = ONE_f4/(radius*radius*2._f4)
else
   c1 = ZERO_f4
end if

DO J=ja,jz
   DO I=ia,iz
      DO K=2,m1-1
         VT(K,I,J) = VT(K,I,J)-scr(K,I,J)*(FCOR(I,J)  &
              -C1*(VP(K,I,J)*c_gd%XT(I+i0)-scr(K,I,J)*c_gd%YM(J+j0)))
      ENDDO
   ENDDO
ENDDO

if (initialization_type == 2) return

DO J=ja,jz
   DO I=ia,iz
      IF(ITOPO == 1) THEN
         DO K=1,m1
            VCTR2(K) = c_gd%ZT(K)*RTG(I,J)+TOP(I,J)
         ENDDO
         vctr5(1:c_gd%nz) = intrp_1D_arr_type01a(c_gd%u01d(1:c_gd%nzp),  &
              c_gd%zt(1:c_gd%nzp), vctr2(1:c_gd%nz))
!         CALL HTINT(NZP,U01DN(1,grid),ZT,NZ,VCTR5,VCTR2)
         DO K=2,m1-1
            VT(K,I,J) = VT(K,I,J)+FCOR(I,J)*VCTR5(K)
         ENDDO
      ELSE
         DO K=2,m1-1
            VT(K,I,J) = VT(K,I,J)+FCOR(I,J)*c_gd%U01D(K)
         ENDDO
      ENDIF
   ENDDO
ENDDO

end subroutine coriolis_V


end module PRAMS_coriolis_mod


!```````````````````````````````````````````````````````````````````
subroutine coriolis(mzp,mxp,myp,i0,j0,ia,iz,ja,jz,izu,jzv)

!~ 21.01.2012 (last modified; TIM)

!+ Coriolis driver; compute Coriolis accelerations, update U,V tendencies

use main_memory_mod, only: a, iscr1, icorflg
use vtables_mod, only: iuc,ivc,iut,itopu,irtgu,ifcoru,itopv,irtgv,ifcorv,ivt
use PRAMS_coriolis_mod, only: coriolis_u, coriolis_v

implicit none

!=== Argument declarations:

integer, intent(IN) :: mzp,mxp,myp,i0,j0,ia,iz,ja,jz,izu,jzv

!== Local declarations:

! NONE

!=== Executable statements:

if(icorflg == 0) then    ! Coriolis computation not wanted
   return
endif

call coriolis_U(mzp,mxp,myp,i0,j0,ia,izu,ja,jz,a(IUC),a(IVC),a(IUT),a(ISCR1)  &
     ,a(ITOPU),a(IRTGU),a(IFCORU))
call coriolis_V(mzp,mxp,myp,i0,j0,ia,iz,ja,jzv,a(IUC),a(IVC),a(IVT),a(ISCR1)  &
     ,A(ITOPV),A(IRTGV),A(IFCORV))

end subroutine coriolis
