!```````````````````````````````````````````````````````````````````
program PRAMS

!+ Main program unit for the Planetary Regional Atmospheric Modeling System

use univ_std_fileIO_mod, only: find_file_IOunit
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use main_memory_mod, only: pbanner, pbnlen, PRAMS_version, run_is_parallel,  &
     terminate_run_now
use rootPE_driver_mod, only: rootPE_driver
use leafPE_driver_mod, only: leafPE_driver
use PRAMS_vtab_mod, only: vtable_fn
use namelist_mod, only: model_nml_fn

#if defined(USE_MPI)
 use MPI, only: MPI_COMM_WORLD, MPI_Init
#endif  /* end:USE_MPI */

implicit none

!== Local declarations:

integer :: this_n_PE, this_rank, cn, IO_unit, ierr, ic, icv
character(len=MAX_FILE_PATH_LEN) :: str_tmp
logical :: file_exists

logical :: run_intent_is_parallel
integer :: n_PE
character(len=MAX_FILE_PATH_LEN) :: namelist_fn, v_common_fn, v_body_fn
namelist /PRAMS_args/ run_intent_is_parallel, n_PE, namelist_fn, vtable_fn,  &
     v_common_fn, v_body_fn

!=== Executable statements:

! Set namelist defaults:
namelist_fn = 'PRAMS_IN'
run_intent_is_parallel = .FALSE.
n_PE = 1
v_common_fn = '0.0_r0'; v_body_fn = '0.0_r0'

! Announce the beginning of meaningful program execution:
write (*, '(/,a,/)') 'PRAMS starting ...'

! Find available IO unit:
IO_unit = find_file_IOunit()

! Read initial "command-line" namelist:

inquire(file=model_nml_fn, exist=file_exists)
if (.NOT.file_exists) call usage

open(IO_unit, file=model_nml_fn, status='old', form='formatted')

read (IO_unit, nml=PRAMS_args); close(IO_unit)

! Set and display version info:

cn = len_trim(v_common_fn)
inquire(file=v_common_fn(1:cn), exist=file_exists)
if (.NOT.file_exists) call exception('PRAMS_main', FATAL,  &
     'Model version file ('//v_common_fn(1:cn)//') not found.')
open(IO_unit, file=v_common_fn(1:cn), status='old', form='formatted')

PRAMS_version = '{c:'; icv = 4
do
   ! Read version info line:
   read (IO_unit, *, iostat=ierr) str_tmp; if (ierr /= 0) exit
   ic = index(str_tmp, '=')
   if (str_tmp(1:ic-1) == 'version') then
      cn = len_trim(str_tmp(ic+1:))
      PRAMS_version(icv:icv+cn-1) = str_tmp(ic+1:); icv = icv+cn
      exit
   end if
end do
close(IO_unit)

cn = len_trim(v_body_fn)
inquire(file=v_body_fn(1:cn), exist=file_exists)
if (.NOT.file_exists) call exception('PRAMS_main', FATAL,  &
     'Model version file ('//v_body_fn(1:cn)//') not found.')
open(IO_unit, file=v_body_fn(1:cn), status='old', form='formatted')

PRAMS_version(icv:icv+4-1) = ', b:'; icv = icv+4
do
   ! Read version info line:
   read (IO_unit, *, iostat=ierr) str_tmp; if (ierr /= 0) exit
   ic = index(str_tmp, '=')
   if (str_tmp(1:ic-1) == 'version') then
      cn = len_trim(str_tmp(ic+1:))
      PRAMS_version(icv:icv+cn-1) = str_tmp(ic+1:); icv = icv+cn
      exit
   end if
end do
close(IO_unit)
PRAMS_version(icv:icv+1-1) = '}'

write (*, '(/,3a,/)') 'PRAMS version ', trim(PRAMS_version), '.'

! Begin the initialization of the parallel infrastructure if desired:

!mach_size = 0; mach_rank = 0    ! Initialize values

if (run_intent_is_parallel) then   ! Should this truly be a parallel run?

#if defined(USE_MPI)
    call MPI_Init(ierr)    ! Initialize MPI library

    call MPI_Comm_rank(MPI_COMM_WORLD, this_rank, ierr)
    call MPI_Comm_size(MPI_COMM_WORLD, this_n_PE, ierr)

    if (this_n_PE > 1) then
       run_is_parallel = .TRUE.
    else
       call MPI_Finalize(ierr)   ! Exit MPI library if not truly a parallel run
       run_is_parallel = .FALSE.
    end if
#else  /* not:USE_MPI */
    call exception('PRAMS_main', FATAL,  &
         'Parallel run requested, but MPI library was not used at compile time.')
#endif  /* end:USE_MPI */

else    ! Intended to be a serial run

   this_rank = 0; this_n_PE = 1; run_is_parallel = .FALSE.

end if

! Display some useful run/setup information:

if (run_is_parallel) then
   write (*, '(/,a)') '*** This is a PE of a PARALLEL run ***'
else
   write (*, '(/,a)') '*** This is the only process of a SERIAL run ***'
end if

write (*, '(/,/,a)') '+-----------------------------------------------------'
write (*, '(a,5x,a)') '! PRAMS input namelist file:',  &
     namelist_fn(1:len_trim(namelist_fn))
write (*, '(a,5x,a)') '! PRAMS var table file:', trim(vtable_fn)

terminate_run_now = .FALSE.

if (this_rank /= 0) then   ! This is a leaf PE

   write (*, '(a,i6)') '! PRAMS leaf PE, with rank ', this_rank
   write (*, '(a,/,/)') '+-----------------------------------------------------'

   write (pbanner, '(a8,i6.6,a3)') 'LEAF_PE_', this_rank, ':  '
   pbnlen = len_trim(pbanner)+2

   call leafPE_driver(this_rank)

else                      ! This is the root PE

   write (*, '(a,i6)') '! PRAMS root PE, with rank ', this_rank
   write (*, '(a,i6)') '! Parallel run info:  total # of PEs = ', this_n_PE
   write (*, '(a,/,/)') '+-----------------------------------------------------'

   pbanner = 'ROOT_PE:  '; pbnlen = len_trim(pbanner)+2

   call rootPE_driver(this_n_PE, this_n_PE-1, this_rank, model_nml_fn,  &
        vtable_fn)

end if

#if defined(USE_MPI)
 if (run_is_parallel) then
    call MPI_Finalize(ierr)    ! Exit MPI library
    write (*, '(a,a)') pbanner(1:pbnlen),  &
         'Successfully exited the MPI library...'
 end if
#endif  /* end:USE_MPI */

if (terminate_run_now) then
   stop '*** Abnormal completion ***'
else
   stop '*** Normal completion ***'
end if


contains


!```````````````````````````````````````````````````````````````````
subroutine usage

!+ Prints PRAMS command-line usage information

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'usage'

!=== Executable statements:

write (*, '(/,a,/)')  &
     'USAGE:  Model run must be initiated using run_PRAMS, or another '//  &
     'similar helper script.'
call exception(procedure_name, FATAL)

end subroutine usage


end program PRAMS
