!```````````````````````````````````````````````````````````````````
subroutine no_atm_RT_interface(m1, m2, m3, m5, ia, iz, ja, jz)

!+ Radiative transfer code driver interface (RT with no atmospheric "meddling")

use univ_kind_defs_mod, only: f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
use univ_integer_time_mod, only: time_obj, fp_seconds_eqv_Time, operator(-),  &
     get_Time_component_values, operator(*), operator(/)
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_math_constants_mod, only: ZERO_f4, ONE_f4, ONE_HALF_f4
use univ_convert_units_mod, only: rad_to_deg_f4
use manage_NetCDF_classic_mod, only: mNc_populate_f_md_obj_w,  &
     mNc_MAX_NAME_LEN, mNc_FLOAT_type, mNc_file_md_Att_obj,  &
     mNc_define_f_md_Att_obj, mNc_sync_with_file, mNc_DOUBLE_type,  &
     mNc_recycle_f_md_obj, mNc_f_md_populated
use manage_NetCDF_classic_wv_mod, only: mNc_write_var
use main_memory_mod, only: islastog, current_sim_ET, a, c_gd, i_grid, gd,  &
     vctr1, vctr2, i0, j0, state_output_fpfx, undef_value_f4, Ls_of_RT_call,  &
     UT_of_RT_call, EOT_of_RT_call
use main_memory_bspec_mod, only: lonrad, ishadtog, RT_flux_trace_pt,  &
     RT_flux_trace_ti, RT_flux_trace_np, RT_flux_trace_outp_f_obj,  &
     n_RT_flux_snapshots, ip_mask, fluxout_dn, fluxout_up, dT_dt_out
use vtables_mod, only: ivt2dc, ialbedt, isfctempk, icosz,  &
     iglat, iglon, irlong, itazmang, itslopang, irtgt
use body_specific_constants_mod, only: solar_mean_broadband
use topo_RT_effects_dims_mod, only: nazm
use RT_utils_mod, only: stellar_angles
use F77_array_utils_mod, only: ptval_from_3D_array_f4, ptval_from_2D_array_f4
use no_atm_RT_driver_mod, only: n_spectral_bands, no_atm_RT_driver
 
implicit none

!=== Argument declarations:

integer, intent(IN) :: ia, iz, ja, jz, m1, m2, m3, m5

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'no_atm_RT_interface'

real(f4) :: xsdist, sdec_curr, tmp_f4, sflux_sw
logical :: init_rad

integer :: a_err, n_w, ip, ic, i, ix, iy
logical :: record_RT_flux_trace

integer, save :: ncall = 0

!==== For NetCDF-format file input/output:
integer :: n_dims, n_vars, n_g_Atts, max_var_ndims, max_var_nAtts, iat
character(len=mNc_MAX_NAME_LEN), allocatable, dimension(:) :: dim_name,  &
     var_name
integer, allocatable, dimension(:) :: dimv, var_type, var_ndims, var_nAtts
integer, allocatable, dimension(:,:) :: var_dimids
type(mNc_file_md_Att_obj), allocatable, dimension(:) :: g_Att
type(mNc_file_md_Att_obj), allocatable, dimension(:,:) :: var_Att
character(len=mNc_MAX_NAME_LEN) :: tmp_str

!=== Executable statements:

init_rad = .FALSE.   ! Assume init of rad transfer model has already been done
if (ncall == 0) then
   init_rad = .TRUE.  ! Radiative transfer model needs to be initialized 
   ncall = 1
end if

! Calculate solar zenith angle and azimuth (vt2dc):

if (lonrad == 0) then   ! Set centlat/lon to center of coarsest grid
   call stellar_angles(m1, m2, m3, ia, iz, ja, jz, a(iglat), a(iglon),  &
        a(icosz), xsdist, gd(1)%centlat, gd(1)%centlon, lonrad, a(ivt2dc),  &
        Ls_of_RT_call, .TRUE., UT_of_RT_call, EOT_of_RT_call, sdec_curr)
else
   call stellar_angles(m1, m2, m3, ia, iz, ja, jz, a(iglat), a(iglon),  &
        a(icosz), xsdist, c_gd%centlat, c_gd%centlon, lonrad, a(ivt2dc),  &
        Ls_of_RT_call, .TRUE., UT_of_RT_call, EOT_of_RT_call, sdec_curr)
end if

! These are always needed:
c_gd%idx_mask_xy = 0          ! Initialize
record_RT_flux_trace = .FALSE.  !

! Perform RT_flux_trace operations?
if (RT_flux_trace_np > 0) then
   if (current_sim_ET >= RT_flux_trace_ti(1) .and.  &  ! Within specified times?
        current_sim_ET <= RT_flux_trace_ti(2)) then

      if (.not.allocated(ip_mask)) then
         allocate(ip_mask(RT_flux_trace_np), stat=a_err)
         if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
              procedure_name, 'Alloc #1')
      end if
      ip_mask = .FALSE.  ! Initialize

      ! Check whether any 2-stream flux profile locations are in the current
      !  domain; if so, mark them for calculation by the RT code:
      do ip=1,RT_flux_trace_np
         if (i_grid == RT_flux_trace_pt(ip)%igd) then
            ix = RT_flux_trace_pt(ip)%ix; iy = RT_flux_trace_pt(ip)%iy
            if (ix >= ia+i0 .and. ix <= iz+i0 .and. iy >= ja+j0 .and.  &
                 iy <= jz+j0) then  ! Specified gridpoint found in this domain
               c_gd%idx_mask_xy(ix-i0,iy-j0) = ip
               ip_mask(ip) = .TRUE.
            end if
         end if
      end do

      record_RT_flux_trace = any(ip_mask)
   end if
end if

! Initialize RT_flux_trace operations:
if (record_RT_flux_trace) then
   n_w = n_spectral_bands   ! Longwave and shortwave
   if (.not.allocated(fluxout_up)) then
      allocate(fluxout_up(m1,n_w,RT_flux_trace_np),  &
           fluxout_dn(m1,n_w,RT_flux_trace_np),  &
           dT_dt_out(m1,RT_flux_trace_np), stat=a_err)
      if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'Alloc #2')
   end if

   if (.not.mNc_f_md_populated(RT_flux_trace_outp_f_obj(1))) then
      n_dims = 4; n_vars = 19; n_g_Atts = 1

      allocate(dim_name(n_dims), dimv(n_dims))

      allocate(var_name(n_vars), var_type(n_vars), var_ndims(n_vars),  &
           var_nAtts(n_vars))

      allocate(g_Att(n_g_Atts))

      max_var_ndims = 3; max_var_nAtts = 3
      allocate(var_dimids(max_var_ndims,n_vars), var_Att(max_var_nAtts,n_vars))

      i = 1; dim_name(i) = 'z_flux'; dimv(i) = m1
      i = 2; dim_name(i) = 'spectral_intv_midpt'; dimv(i) = n_w
      i = 3; dim_name(i) = 'time'; dimv(i) = -1
      i = 4; dim_name(i) = 'z'; dimv(i) = m1

      i = 1
      write (tmp_str, '(a)') 'Radiative transfer energy fluxes from a PRAMS run'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(g_Att(i), 'description', i, ic, tmp_str)

      i = 1; var_name(i) = 'z_flux'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 1 /)
      iat = 1; tmp_str = 'Altitude at layer boundaries'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'm'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 2; var_name(i) = 'z'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 4 /)
      iat = 1; tmp_str = 'Altitude at layer midpoints'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'm'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 3; var_name(i) = 'spectral_intv_midpt'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 2 /)
      iat = 1; tmp_str = 'wavelength of spectral interval midpoint'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'm'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 4; var_name(i) = 'spectral_intv_beg'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 2 /)
      iat = 1
      tmp_str = 'wavelength of the beginning (edge) of each spectral interval'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'm'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 5; var_name(i) = 'spectral_intv_end'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 2 /)
      iat = 1
      tmp_str = 'wavelength of the end (edge) of each spectral interval'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'm'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 6; var_name(i) = 'time'; var_type(i) = mNc_DOUBLE_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'elapsed time'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 's'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 7; var_name(i) = 'UT'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'mean UT (LMST at longitude=0)'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'body-hours'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 8; var_name(i) = 'LMST'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'local mean solar time'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'body-hours'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 9; var_name(i) = 'LTST'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'local true solar time'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'body-hours'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 10; var_name(i) = 'stellar_dist'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'distance between body and stellar energy source'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'AU'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 11; var_name(i) = 'Ls'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'body-centric solar longitude'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'degrees'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 12; var_name(i) = 'stellar_dec'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'declination (latitude) of nadir stellar beam'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'radians'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 13; var_name(i) = 'EOT'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1; tmp_str = 'equation of time'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'radians'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 14; var_name(i) = 'flux_down'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 3; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 1, 2, 3 /)
      iat = 1; tmp_str = 'RT flux, (stream) down'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'W/m^2'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 15; var_name(i) = 'flux_up'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 3; var_nAtts(i) = 2
      var_dimids(1:var_ndims(i),i) = (/ 1, 2, 3 /)
      iat = 1; tmp_str = 'RT flux, (stream) up'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'W/m^2'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)

      i = 16; var_name(i) = 'dT_dt'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 2; var_nAtts(i) = 3
      var_dimids(1:var_ndims(i),i) = (/ 4, 3 /)
      iat = 1; tmp_str = 'atmospheric RT heating rate (dT/dt)'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'K/s'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)
      iat = 3
      call mNc_define_f_md_Att_obj(var_Att(iat,i), '_FillValue', iat, 1,  &
           (/undef_value_f4/))

      i = 17; var_name(i) = 'other_sflux_down'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 3
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1
      tmp_str = 'Stellar flux (stream) down, outside of modeled shortwave '//  &
           'range, no atmospheric RT effects'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'W/m^2'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)
      iat = 3
      call mNc_define_f_md_Att_obj(var_Att(iat,i), '_FillValue', iat, 1,  &
           (/undef_value_f4/))

      i = 18; var_name(i) = 'sflux_sw'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 3
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1
      tmp_str = 'Stellar flux at 1 AU within the modeled shortwave range only'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'W/m^2'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)
      iat = 3
      call mNc_define_f_md_Att_obj(var_Att(iat,i), '_FillValue', iat, 1,  &
           (/undef_value_f4/))

      i = 19; var_name(i) = 'sflux_mean_broadband'; var_type(i) = mNc_FLOAT_type
      var_ndims(i) = 1; var_nAtts(i) = 3
      var_dimids(1:var_ndims(i),i) = (/ 3 /)
      iat = 1
      tmp_str = 'Stellar flux at 1 AU, broadband mean'
      ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'long_name', iat, ic,  &
           tmp_str)
      iat = 2; tmp_str = 'W/m^2'; ic = len_trim(tmp_str)
      call mNc_define_f_md_Att_obj(var_Att(iat,i), 'units', iat, ic, tmp_str)
      iat = 3
      call mNc_define_f_md_Att_obj(var_Att(iat,i), '_FillValue', iat, 1,  &
           (/undef_value_f4/))

      do ip=1,RT_flux_trace_np
         ! Populate an output file with its configuration metadata:
         write (tmp_str, '(2a,i2.2,2(a,i5.5),a)') trim(state_output_fpfx),  &
              '.g', RT_flux_trace_pt(ip)%igd, '_x', RT_flux_trace_pt(ip)%ix,  &
              '_y', RT_flux_trace_pt(ip)%iy, '.RT_flux.nc'
         call mNc_populate_f_md_obj_w(trim(tmp_str),  &
              RT_flux_trace_outp_f_obj(ip), n_dims, n_vars, n_g_Atts,  &
              dim_name, dimv, g_Att, var_name, var_type, var_ndims,  &
              var_nAtts, var_dimids, var_Att)
         n_RT_flux_snapshots(ip) = 0
      end do
   end if

   do ip=1,RT_flux_trace_np
      ! Write time-independent fields?
      if (ip_mask(ip) .and. n_RT_flux_snapshots(ip) == 0) then
         ix = RT_flux_trace_pt(ip)%ix; iy = RT_flux_trace_pt(ip)%iy

         vctr1 = c_gd%zm*ptval_from_2D_array_f4(m2, m3, a(irtgt), ix-i0, iy-j0)
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'z_flux', vctr1(1:m1))
         vctr1 = c_gd%zt*ptval_from_2D_array_f4(m2, m3, a(irtgt), ix-i0, iy-j0)
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'z', vctr1(1:m1))

         vctr1(1:n_w+1) =  &    ! Placeholders (only indicate SW vs. LW)
              (/ ONE_f4, 4.5e-6_f4, ZERO_f4 /)
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip),  &
              'spectral_intv_beg', vctr1(1:n_w))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip),  &
              'spectral_intv_end', vctr1(2:n_w+1))

         do i=1,n_w
            vctr2(i) = ONE_HALF_f4*(vctr1(i)+vctr1(i+1))
         end do
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip),  &
              'spectral_intv_midpt', vctr2(1:n_w))
      end if
   end do
end if

if (record_RT_flux_trace) then
   fluxout_up = ZERO_f4; fluxout_dn = ZERO_f4; dT_dt_out = undef_value_f4
   call no_atm_RT_driver(m1, m2, m3, m5, ia, iz, ja, jz, a(isfctempk),  &
        a(icosz), xsdist, a(ialbedt), c_gd%emisst, a(irlong),  &
        a(ivt2dc),  &
        a(itazmang), a(itslopang), islastog, ishadtog, nazm,  &
        c_gd%cosz_horizon(1:m2,1:m3,:), c_gd%TCx_obstruct(1:m2,1:m3,:),  &
        c_gd%TCz_obstruct(1:m2,1:m3,:), init_rad, solar_mean_broadband,  &
        c_gd%idx_mask_xy, c_gd%bottom_other_sflux_down,  &
        c_gd%bottom_SWflux_down, c_gd%topo_factor_sflux, RT_flux_trace_np,  &
        fluxout_up, fluxout_dn, dT_dt_out, sflux_sw)

   do ip=1,RT_flux_trace_np
      if (ip_mask(ip)) then   ! Write time-dependent vars
         ix = RT_flux_trace_pt(ip)%ix; iy = RT_flux_trace_pt(ip)%iy
         n_RT_flux_snapshots(ip) = n_RT_flux_snapshots(ip)+1
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'time',  &
              current_sim_ET, opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'UT',  &
              UT_of_RT_call, opt_start=(/n_RT_flux_snapshots(ip)/))
         tmp_f4 = UT_of_RT_call+  &
              ptval_from_2D_array_f4(m2, m3, a(iglon), ix-i0, iy-j0)/  &
              15._f4+24._f4
         tmp_f4 = mod(tmp_f4, 24._f4)
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'LMST',  &
              tmp_f4, opt_start=(/n_RT_flux_snapshots(ip)/))
         tmp_f4 = mod(  &
              tmp_f4+EOT_of_RT_call*rad_to_deg_f4*24._f4/360._f4+24._f4, 24._f4)
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'LTST',  &
              tmp_f4, opt_start=(/n_RT_flux_snapshots(ip)/))
         tmp_f4 = sqrt(xsdist)
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'stellar_dist',  &
              tmp_f4, opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'Ls',  &
              Ls_of_RT_call, opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'stellar_dec',  &
              sdec_curr, opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'EOT',  &
              EOT_of_RT_call, opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'flux_down',  &
              fluxout_dn(:,:,ip), opt_start=(/1,1,n_RT_flux_snapshots(ip)/),  &
              opt_count=(/size(fluxout_dn, 1),size(fluxout_dn, 2),1/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'flux_up',  &
              fluxout_up(:,:,ip), opt_start=(/1,1,n_RT_flux_snapshots(ip)/),  &
              opt_count=(/size(fluxout_up, 1),size(fluxout_up, 2),1/))
         dT_dt_out(1,ip) = undef_value_f4  ! Undefined at k=1
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'dT_dt',  &
              dT_dt_out(:,ip), opt_start=(/1,n_RT_flux_snapshots(ip)/),  &
              opt_count=(/size(dT_dt_out, 1),1/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'other_sflux_down',  &
              c_gd%bottom_other_sflux_down(ix,iy),  &
              opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip), 'sflux_sw',  &
              sflux_sw, opt_start=(/n_RT_flux_snapshots(ip)/))
         call mNc_write_var(RT_flux_trace_outp_f_obj(ip),  &
              'sflux_mean_broadband', solar_mean_broadband,  &
              opt_start=(/n_RT_flux_snapshots(ip)/))

         call mNc_sync_with_file(RT_flux_trace_outp_f_obj(ip))
      end if
   end do
else
   call no_atm_RT_driver(m1, m2, m3, m5, ia, iz, ja, jz, a(isfctempk),  &
        a(icosz), xsdist, a(ialbedt), c_gd%emisst, a(irlong),  &
        a(ivt2dc),  &
        a(itazmang), a(itslopang), islastog, ishadtog, nazm,  &
        c_gd%cosz_horizon(1:m2,1:m3,:), c_gd%TCx_obstruct(1:m2,1:m3,:),  &
        c_gd%TCz_obstruct(1:m2,1:m3,:), init_rad, solar_mean_broadband,  &
        c_gd%idx_mask_xy, c_gd%bottom_other_sflux_down,  &
        c_gd%bottom_SWflux_down, c_gd%topo_factor_sflux, 1)
end if

! Finalize RT_flux_trace operations (after specified time interval has passed):
if (RT_flux_trace_np > 0) then
   if (mNc_f_md_populated(RT_flux_trace_outp_f_obj(1)) .and.  &
        current_sim_ET > RT_flux_trace_ti(2)) then
      do ip=1,RT_flux_trace_np
         call mNc_recycle_f_md_obj(RT_flux_trace_outp_f_obj(ip))
      end do
   end if
end if

end subroutine no_atm_RT_interface
