!```````````````````````````````````````````````````````````````````
module RT_utils_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine dT2dtheta(m1, m2, m3, ia, iz, ja, jz, pp, pi0, fthrd)

!+ Converts temperature tendency to potential temperature tendency

use univ_kind_defs_mod, only: f4
use body_specific_constants_mod, only: cp

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: pp, pi0

real(f4), dimension(m1,m2,m3), intent(INOUT) :: fthrd

!== Local declarations:

integer :: i, j, k
real(f4) :: cp_PI

!=== Executable statements:

do j=ja,jz
   do i=ia,iz
      do k=1,m1
         cp_PI = cp/(pp(k,i,j)+pi0(k,i,j))
         fthrd(k,i,j) = fthrd(k,i,j)*cp_PI   ! dT/dt --> dtheta/dt
      end do
   end do
end do

end subroutine dT2dtheta


!```````````````````````````````````````````````````````````````````
subroutine ae3p3_TIM(n1, n2, n3, i1, i2, j1, j2, k1, k2, a, b, c)

!+ Transfer 2D data ("legacy" routine -- FIX ME!!!)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:
  
integer, intent(IN) :: n1, n2, n3, i1, i2, j1, j2, k1, k2
real(f4), dimension(n1,n2,n3), intent(IN) :: b, c

real(f4), dimension(n1,n2,n3), intent(OUT) :: a

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

do j=j1,j2
   do i=i1,i2
      do k=k1,k2
         a(k,i,j) = b(k,i,j)+c(k,i,j)
      end do
   end do
end do

end subroutine ae3p3_TIM


!```````````````````````````````````````````````````````````````````
subroutine stellar_angles(m1, m2, m3, ia, iz, ja, jz, glat, glon, cosz,  &
     xsdist, centlat, centlon, lonrad, stellar_azm, Ls_curr, display_info,  &
     UT_time, EOT_curr, sdec_curr)

!+ Computes stellar zenith angle and azimuth

use univ_kind_defs_mod, only: f4, f8
use univ_convert_units_mod, only: rad_to_deg_f4, deg_to_rad_f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4, ZERO_f8
use univ_numtype_constants_mod, only: epsilon_f4
use main_memory_mod, only: orbital_info_table, current_sim_ET,  &
     orbital_progression, diurnal_progression
use simple_astron_scheme_mod, only: get_current_sas_vals

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, lonrad, ia, iz, ja, jz
real(f4), dimension(m2,m3), intent(IN) :: glat, glon
real(f4), intent(IN) :: centlat, centlon
logical, intent(IN) :: display_info

real(f4), dimension(m2,m3), intent(OUT) :: cosz, stellar_azm
real(f4), intent(OUT):: xsdist, Ls_curr
real(f4), optional, intent(OUT) :: UT_time, EOT_curr, sdec_curr

!== Local declarations:

integer :: i, j
real(f4) :: sndec, csdec, radlat, cslcsd, snlsnd, gglon, dayhrr, hrangl,  &
     centlat_rad, sdcl, cdsl, xEOTadj, xsdeclin, xsradius, acosBC_lw,  &
     acosBC_up, mean_UT, tmp_cosz, tmp_stellar_azm, curr_mean_UT, ET0_mean_UT
real(f8) :: mean_UT_f8, Ls_curr_f8, xEOTadj_f8, xsdeclin_f8, xsradius_f8

!=== Executable statements:

acosBC_up = ZERO_f4; acosBC_lw = ZERO_f4;

! Calculate current orbital/time params:

if (orbital_progression .or. diurnal_progression) then
   call get_current_sas_vals(orbital_info_table, current_sim_ET, mean_UT_f8,  &
        Ls_curr_f8, xEOTadj_f8, xsdeclin_f8, xsradius_f8)  ! At current sim_ET
   curr_mean_UT = real(mean_UT_f8, kind=f4)
end if

if (orbital_progression) then
  ! Store current sim_ET values before they are (possibly) altered:
   Ls_curr = real(Ls_curr_f8, kind=f4)
   xEOTadj = real(xEOTadj_f8, kind=f4); xsdeclin = real(xsdeclin_f8, kind=f4)
   xsradius = real(xsradius_f8, kind=f4)
end if

if (.not.orbital_progression .or. .not.diurnal_progression) then
   call get_current_sas_vals(orbital_info_table, ZERO_f8, mean_UT_f8,  &
        Ls_curr_f8, xEOTadj_f8, xsdeclin_f8, xsradius_f8)  ! At sim_ET = 0.
   ET0_mean_UT = real(mean_UT_f8, kind=f4)
end if

if (.not. orbital_progression) then
  ! Use sim_ET = 0. values:
   Ls_curr = real(Ls_curr_f8, kind=f4)
   xEOTadj = real(xEOTadj_f8, kind=f4); xsdeclin = real(xsdeclin_f8, kind=f4)
   xsradius = real(xsradius_f8, kind=f4)
end if

if (diurnal_progression) then
   mean_UT = curr_mean_UT
else
   mean_UT = ET0_mean_UT
end if

if (present(UT_time)) UT_time = mean_UT
if (present(EOT_curr)) EOT_curr = xEOTadj
if (present(sdec_curr)) sdec_curr = xsdeclin

if (display_info) then
   write (*, '(/,a,1x,f9.4)') 'Body-centric longitude [Ls] =', Ls_curr
   write (*, '(a,1x,f8.4)') 'Body UT [body-hours] =', mean_UT
   write (*, '(a,1x,f9.4)') 'Substellar latitude [deg] =',  &
        xsdeclin*rad_to_deg_f4
   write (*, '(a,1x,f8.5)') 'body-star distance [AU] =', xsradius
end if

xsdist = xsradius*xsradius   ! Square of body's current stellar-centric radius
sndec = sin(xsdeclin)
csdec = cos(xsdeclin)

! Find the hour angle, then get cosine of the zenith angle, etc.:

if (lonrad == 0) centlat_rad = centlat*deg_to_rad_f4 ! Center latitude of domain

acosBC_up = ONE_f4-epsilon_f4
acosBC_lw = -ONE_f4+epsilon_f4

do j=ja,jz
   do i=ia,iz
      radlat = glat(i,j)*deg_to_rad_f4
      if (lonrad == 0) radlat = centlat_rad

         ! Reset lat if too close to sdec
      if (abs(xsdeclin-radlat) < 1.e-5_f4) radlat = radlat+1.e-5_f4

      cslcsd = cos(radlat)*csdec
      snlsnd = sin(radlat)*sndec
      
      gglon = glon(i,j)
      if (lonrad == 0) gglon = centlon ! Set longitude to central lon of domain

      dayhrr = mod(mean_UT+gglon/15._f4+24._f4, 24._f4) ! Local solar mean time
!      hrangl = (15._f4*(dayhrr-12._f4))*deg_to_rad_f4    ! Hour angle
      hrangl = (15._f4*(dayhrr-12._f4))*deg_to_rad_f4+xEOTadj ! Hour angle w/ EOT correction

      tmp_cosz = snlsnd+cslcsd*cos(hrangl)  ! cos(solar zenith angle)
      tmp_cosz = max(acosBC_lw, min(acosBC_up, tmp_cosz))  ! BC for cosine

      sdcl = sndec*cos(radlat)
      cdsl = csdec*sin(radlat)

      tmp_stellar_azm = acos(max(acosBC_lw,  &       ! Solar azimuth
           min(acosBC_up, (sdcl-cdsl*cos(hrangl))/sin(acos(tmp_cosz)))))
      if (dayhrr > 12._f4)  & ! If after noon, adjust azimuth (2*pi-stellar_azm)
           tmp_stellar_azm = (deg_to_rad_f4*360._f4)-tmp_stellar_azm

      stellar_azm(i,j) = tmp_stellar_azm
      cosz(i,j) = tmp_cosz

   end do
end do

if (display_info) then
   write (*, '(a,1x,f9.4)') 'Stellar zenith angle at grid center [deg] =',  &
        acos(cosz(max((ia+iz)/2, 1),max((ja+jz)/2, 1)))*rad_to_deg_f4
   write (*, '(a,1x,f9.4)') 'Stellar azimuth angle at grid center [deg] =',  &
        stellar_azm(max((ia+iz)/2, 1),max((ja+jz)/2, 1))*rad_to_deg_f4
end if

end subroutine stellar_angles


end module RT_utils_mod
