!```````````````````````````````````````````````````````````````````
module acoustic_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine coeff_z(m1, m2, m3, ia, iz, ja, jz, coeff_dm, coeff_em,  &
     coeff_fkdn0, coeff_gkdn0, dn0, pi0, th0, rtgt, betam1, amoe, amof, acoaa)

!+ Compute coefficients for acoustic model 

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use main_memory_mod, only: dts, c_gd, vctr11, vctr12, vctr9, acobb => vctr1,  &
     acocc => vctr2, impl, sspct
use body_specific_constants_mod, only: rocv

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: th0, pi0, dn0
real(f4), dimension(m2,m3), intent(IN) :: rtgt

real(f4), intent(INOUT) :: betam1
real(f4), dimension(m1,m2,m3), intent(INOUT) :: coeff_dm, coeff_em,  &
     coeff_fkdn0, coeff_gkdn0, acoaa, amoe, amof

!== Local declarations:

real(f4), parameter :: beta = 0.6_f4

real(f4) :: dtsbeta, c1, cc1
integer :: i, j, k

!=== Executable statements:

! Calculate coefficients for the vertical pressure gradient       
!  and divergence terms.  These will be combined later for the    
!  implicit computations.                                         

! Coefficients that are used to calculate elements of a tridiagonal
!  matrix below:

if (impl == 1) then
   dtsbeta = dts*beta   ! used to be dt2al2
   betam1 = (ONE_f4-beta)/beta   ! used to be a1da2
else    !???? are the below correct?
   dtsbeta = dts  ! used to be dt2al2 
   betam1 = ZERO_f4   ! used to be a1da2
end if

cc1 = rocv*dtsbeta*sspct**2

do j=ja,jz
   do i=ia,iz

      ! Coefficients used in vertical PI gradient term; valid at zm levs:

      c1 = 0.5_f4*dtsbeta/rtgt(i,j)
      do k=1,m1-1
         vctr9(k) = c1*(th0(k,i,j)+th0(k+1,i,j))*c_gd%dzm(k)    
         coeff_em(k,i,j) = -vctr9(k)  ! Factor for pi'(k+1)
         coeff_dm(k,i,j) = vctr9(k)  ! Factor for pi'(k)
      end do

      ! Miscellaneous values:

      c1 = cc1/rtgt(i,j)
      do k=1,m1
         vctr11(k) = c1*pi0(k,i,j)*c_gd%dzt(k)/dn0(k,i,j)  ! f(k)
      end do

      do k=1,m1-1
         vctr12(k) = 0.5*(dn0(k,i,j)+dn0(k+1,i,j))   ! dn0(m)
      end do
      
      ! Coefficients used in final PI prediction; valid at zt levs:

      do k=2,m1-1
         coeff_gkdn0(k,i,j) = -vctr11(k)*vctr12(k)    ! g(k)*dn0(m)
         coeff_fkdn0(k,i,j) = vctr11(k)*vctr12(k-1)   ! f(k)*dn0(m-1)
      end do
      
      ! Coefficients of tridiagonal matrix:

      do k=2,m1-1
         acoaa(k,i,j) = -coeff_dm(k,i,j)*coeff_fkdn0(k,i,j)
         acobb(k) = ONE_f4-coeff_em(k,i,j)*vctr11(k+1)*vctr12(k)-  &
              coeff_dm(k,i,j)*coeff_gkdn0(k,i,j)
         acocc(k) = -coeff_em(k,i,j)*coeff_gkdn0(k+1,i,j)
      end do
      acobb(1) = ONE_f4
      acocc(1) = ZERO_f4
      acoaa(m1,i,j) = ZERO_f4
      acobb(m1) = ONE_f4
      acocc(m1-1) = ZERO_f4   ! BC for acocc (since filled with junk in above loop)

      ! Coefficients needed to invert above tridiagonal matrix (via LU
      !  decomposition):

      amof(1,i,j) = acobb(1)
      do k=2,m1
         amoe(k,i,j) = acocc(k-1)/amof(k-1,i,j)
         amof(k,i,j) = acobb(k)-acoaa(k,i,j)*amoe(k,i,j)
      end do
      
   end do
end do

end subroutine coeff_z


!```````````````````````````````````````````````````````````````````
subroutine prdctu(m1, m2, m3, ia, iz, ja, jz, ibndc, up, ut, pp, vt3da, th0,  &
     dpdx, f13u, rtgu, rtgt, dxu, vt3dh, topu)

!+ Predict U (acoustic model)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_numtype_constants_mod, only: epsilon_f4
use main_memory_mod, only: c_gd, dts, itopo, distim, i_grid

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibndc
real(f4), dimension(m1,m2,m3), intent(IN) :: ut, th0, pp, vt3dh
real(f4), dimension(m2,m3), intent(IN) ::  f13u, rtgu, rtgt, dxu, topu

real(f4), dimension(m1,m2,m3), intent(INOUT) :: up, dpdx, vt3da

!== Local declarations:

real(f4) :: dxl
integer :: i, j, k

!=== Executable statements:

! U prediction:

dpdx(:,:,:) = ZERO_f4

! Calculate acoustic tendency (horizontal pressure gradient):

do j = ja,jz
   do i = ia,iz
      do k = 1,m1-1
         vt3da(k,i,j) = (pp(k,i,j) + pp(k+1,i,j)  &
              + pp(k,i+1,j) + pp(k+1,i+1,j)) * c_gd%hw4(k)
      end do
   end do
end do

do j = ja,jz
   do i = ia,iz
      dxl = dxu(i,j) / rtgu(i,j)
      do k = 2,m1-1
         dpdx(k,i,j) = -(th0(k,i,j) + th0(k,i+1,j)) * 0.5_f4  &
              * ((pp(k,i+1,j) * rtgt(i+1,j) - pp(k,i,j) * rtgt(i,j)) * dxl  &
              + (vt3da(k,i,j) - vt3da(k-1,i,j)) * c_gd%dzt(k) * f13u(i,j))
      end do
   end do
end do

if (distim > epsilon_f4) then
   call rayf(1, m1, m2, m3, ia, iz, ja, jz, ibndc, up, th0, vt3dh, rtgu, topu)
end if

do j = 1,m3
   do i = 1,m2
      do k = 1,m1
         up(k,i,j) = up(k,i,j) + dts * (dpdx(k,i,j) + ut(k,i,j))
      end do
   end do
end do

if (c_gd%nstbot == 1 .and. itopo == 1) then
   call botset(m1, m2, m3, ia, iz, ja, jz, i_grid, up, 'U')
end if

end subroutine prdctu


!```````````````````````````````````````````````````````````````````
subroutine prdctv(m1, m2, m3, ia, iz, ja, jz, ibndc, vp, vt, pp, vt3da, th0,  &
     dpdy, f23v, rtgv, rtgt, dyv, vt3dh, topv)

!+ Predict V (acoustic model)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_numtype_constants_mod, only: epsilon_f4
use main_memory_mod, only: c_gd, dts, itopo, jdim, i_grid, distim

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibndc
real(f4), dimension (m1,m2,m3), intent(IN) :: vt, th0, pp, vt3dh
real(f4), dimension(m2,m3), intent(IN) :: f23v, rtgv, rtgt, dyv, topv

real(f4), dimension (m1,m2,m3), intent(INOUT) :: vp, dpdy, vt3da

!== Local declarations:

real(f4) :: dyl
integer :: i, j, k

!=== Executable statements:

! V prediction:

dpdy(:,:,:) = ZERO_f4

if (jdim == 1) then

! Calculate acoustic tendency (horizontal pressure gradient):

   do j = ja,jz
      do i = ia,iz
         do k = 1,m1-1
            vt3da(k,i,j) = (pp(k,i,j) + pp(k+1,i,j)  &
                 + pp(k,i,j+1) + pp(k+1,i,j+1)) * c_gd%hw4(k)
         end do
      end do
   end do

   do j = ja,jz
      do i = ia,iz
         dyl = dyv(i,j) / rtgv(i,j)
         do k = 2,m1-1
            dpdy(k,i,j) = -(th0(k,i,j) + th0(k,i,j+1)) * 0.5_f4  &
                 * ((pp(k,i,j+1) * rtgt(i,j+1)-pp(k,i,j) * rtgt(i,j)) * dyl  &
                 + (vt3da(k,i,j) - vt3da(k-1,i,j)) * c_gd%dzt(k) * f23v(i,j))
         end do
      end do
   end do
end if

if (distim > epsilon_f4) then
   call rayf(2, m1, m2, m3, ia, iz, ja, jz, ibndc, vp, th0, vt3dh, rtgv, topv)
end if

do j = 1,m3
   do i = 1,m2
      do k = 1,m1
         vp(k,i,j) = vp(k,i,j) + dts * (dpdy(k,i,j) + vt(k,i,j))
      end do
   end do
end do

if (c_gd%nstbot == 1 .and. itopo == 1) then
   call botset(m1, m2, m3, ia, iz, ja, jz, i_grid, vp, 'VP')
end if

end subroutine prdctv


!```````````````````````````````````````````````````````````````````
subroutine prdctw1(m1, m2, m3, ia, iz, ja, jz, ibndc, wp, wt, pp, coeff_dm,  &
     coeff_em, betam1, vt3dh, rtgt, topt, thc, th0, realwp)

!+ Predict W - part 1 (acoustic model)

use univ_kind_defs_mod, only: f4
use univ_numtype_constants_mod, only: epsilon_f4
use main_memory_mod, only: dts, distim
use body_specific_constants_mod, only: g_Xoid

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibndc
real(f4), intent(IN) :: betam1
real(f4), dimension(m1,m2,m3), intent(IN) :: wt, pp, coeff_dm, coeff_em,  &
     vt3dh, thc, th0
real(f4), dimension(m2,m3), intent(IN) :: rtgt, topt

real(f4), dimension(m1,m2,m3), intent(INOUT) :: wp, realwp

!== Local declarations:

integer, parameter :: consistent_buoy = 0  ! Buoyancy formulation:
                                           ! 1 = consistent with eqs (th'/th0)
                                           ! 0 = "new" buoyancy      (th'/th)

integer :: i, j, k

!=== Executable statements:

! Store past w value for use in `prdctp1`:

do j=1,m3
   do i=1,m2
      do k=1,m1
         realwp(k,i,j) = wp(k,i,j)
      end do
   end do
end do

! Calculate Rayleigh friction effect on w:

if (distim > epsilon_f4) then
   call rayf(3, m1, m2, m3, ia, iz, ja, jz, ibndc, wp, wp, vt3dh, rtgt, topt)
end if

! Add advective and turbulent tendencies to w:

do j=1,m3
   do i=1,m2
      do k=1,m1-2
         wp(k,i,j) = wp(k,i,j)+dts*wt(k,i,j)
      end do
   end do
end do

! Add buoyancy tendency (theta part) to w:

if (consistent_buoy == 0) then
   do j=ja,jz
      do i=ia,iz
         do k=1,m1-2
            wp(k,i,j) = wp(k,i,j) +  &
                 dts*0.5_f4*g_Xoid*  &
                 (2._f4-(th0(k,i,j)/thc(k,i,j)+th0(k+1,i,j)/thc(k+1,i,j)))
         end do
      end do
   end do
else
   do j=ja,jz
      do i=ia,iz
         do k=1,m1-2
            wp(k,i,j) = wp(k,i,j) +  &
                 dts*0.5_f4*g_Xoid*(thc(k,i,j)/th0(k,i,j)+thc(k+1,i,j)/th0(k+1,i,j)-2._f4)
         end do
      end do
   end do
end if

! Add weighted dPI'/dz (at time t) term to w:

do j=ja,jz
   do i=ia,iz
      do k=1,m1-2
         wp(k,i,j) = wp(k,i,j) +  &
              betam1*(coeff_em(k,i,j)*pp(k+1,i,j)+coeff_dm(k,i,j)*pp(k,i,j))
      end do
   end do
end do

end subroutine prdctw1


!```````````````````````````````````````````````````````````````````
subroutine prdctw2(m1, m2, m3, ia, iz, ja, jz, wp, pp, coeff_dm, coeff_em,  &
     amof, amog, acoaa, w_lower_BC)

!+ Predict W - part 2 (acoustic model)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: c_gd, impl

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: pp, coeff_dm, coeff_em, amof, acoaa
real(f4), dimension(m2,m3), intent(IN) :: w_lower_BC

real(f4), dimension(m1,m2,m3), intent(INOUT) :: wp, amog

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

! Lower boundary condition for w (vertical component of u,v on sigma sfc):

if (c_gd%nstbot == 1) then
   do j=ja,jz
      do i=ia,iz
         wp(1,i,j) = w_lower_BC(i,j)
      end do
   end do
end if

! Upper boundary condition for w:

if (c_gd%nsttop == 1) then
   do j=ja,jz
      do i=ia,iz
         wp(c_gd%nz,i,j) = ZERO_f4
!         wp(nz,i,j) = wp(nz-1,i,j)+(wp(nz-1,i,j)-wp(nz-2,i,j))
      end do
   end do
end if

if (impl == 1) then

   ! "D" term (RHS of tridiagonal w matrix equation):

   do j=ja,jz
      do i=ia,iz
         do k=2,m1-2
            wp(k,i,j) = wp(k,i,j) +  &
                 (coeff_em(k,i,j)*pp(k+1,i,j)+coeff_dm(k,i,j)*pp(k,i,j))
         end do
      end do
   end do

   ! Forward substitution (matrix solver step):

   do j=ja,jz
      do i=ia,iz
         amog(1,i,j) = wp(1,i,j)/amof(1,i,j)
      end do
      do k=2,m1-2
         do i=ia,iz
            amog(k,i,j) = (wp(k,i,j)-acoaa(k,i,j)*amog(k-1,i,j))/amof(k,i,j)
         end do
      end do
   end do

end if

end subroutine prdctw2


!```````````````````````````````````````````````````````````````````
subroutine prdctw3(m1, m2, m3, ia, iz, ja, jz, wp, amog, amoe)

!+ Predict W - part 3 (acoustic model)

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: impl

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: amog, amoe

real(f4), dimension(m1,m2,m3), intent(INOUT) :: wp

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

! Backwards substitution (matrix solver final step):

if (impl == 1) then
 
   do j=ja,jz
      do i=ia,iz
         do k=m1-2,2,-1
            wp(k,i,j) = amog(k,i,j)-amoe(k+1,i,j)*wp(k+1,i,j)
         end do
      end do
   end do

end if

end subroutine prdctw3


!```````````````````````````````````````````````````````````````````
subroutine prdctp1(m1, m2, m3, ia, iz, ja, jz, pp, up, vp, pi0, dn0, pt,  &
     mass_fluxdiv, vert_uv_mass_flux, f13t, f23t, rtgt, rtgu, rtgv,  &
     w_lower_BC, fmapui, fmapvi, dxt, dyt, fmapt, thc, tht, realwp, betam1,  &
     coeff_fkdn0, coeff_gkdn0)

!+ Predict PI - part 1 (acoustic model)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use body_specific_constants_mod, only: rocv
use main_memory_mod, only: jdim, c_gd, dts, sspct

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: betam1
integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: up, vp, pi0, pt, dn0, thc, tht,  &
     realwp, coeff_fkdn0, coeff_gkdn0
real(f4), dimension(m2,m3), intent(IN) :: f13t, f23t, rtgt, rtgu, rtgv,  &
     fmapui, fmapvi, dxt, dyt, fmapt

real(f4), dimension(m1,m2,m3), intent(INOUT) :: pp

real(f4), dimension(m1,m2,m3), intent(OUT) :: vert_uv_mass_flux, mass_fluxdiv
real(f4), dimension(m2,m3), intent(OUT) :: w_lower_BC

!== Local declarations:

integer :: i, j, k
real(f4) :: rocv_ssred

!=== Executable statements:

mass_fluxdiv(:,:,:) = ZERO_f4

! Calculate vertical component of u,v mass flux (on zt-levels):

do j=ja,jz
   do i=ia,iz
      do k=1,m1
         vert_uv_mass_flux(k,i,j) = ( (up(k,i,j)+up(k,i-1,j))*f13t(i,j) +  &
              (vp(k,i,j)+vp(k,i,j-jdim))*f23t(i,j) )*dn0(k,i,j)
      end do
   end do
end do

! Average vertical component of u,v mass flux (to zm-levels); lower w BC:

do j=ja,jz
   do i=ia,iz
      do k=1,m1-1
         vert_uv_mass_flux(k,i,j) = (vert_uv_mass_flux(k,i,j)+  &
              vert_uv_mass_flux(k+1,i,j))*c_gd%hw4(k)
      end do
      ! NOTE: this must exactly zero out term in advection
      w_lower_BC(i,j) = -rtgt(i,j)*vert_uv_mass_flux(1,i,j)/  &
           (0.5_f4*(dn0(1,i,j)+dn0(2,i,j)))
   end do
end do

! Mass flux divergence; u,v vertical component portion:

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         mass_fluxdiv(k,i,j) = (vert_uv_mass_flux(k,i,j)-  &
              vert_uv_mass_flux(k-1,i,j))*c_gd%dzt(k)
      end do
   end do
end do

! Mass flux divergence; u,v horizontal component portion; finish:

rocv_ssred = rocv*sspct**2   ! Possibly reduce the speed of sound

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         
         mass_fluxdiv(k,i,j) = -rocv_ssred*pi0(k,i,j)/dn0(k,i,j) *  &
              (mass_fluxdiv(k,i,j) +  &
              fmapt(i,j)*(  &
              (up(k,i,j)*rtgu(i,j)*fmapui(i,j)*(dn0(k,i,j)+dn0(k,i+1,j)) -  &
              up(k,i-1,j)*rtgu(i-1,j)*fmapui(i-1,j)*  &
              (dn0(k,i,j)+dn0(k,i-1,j)))  &
              *dxt(i,j)*0.5_f4  +  &
              (vp(k,i,j)*rtgv(i,j)*fmapvi(i,j)*(dn0(k,i,j)+dn0(k,i,j+jdim)) - &
              vp(k,i,j-jdim)*rtgv(i,j-jdim)*fmapvi(i,j-jdim)*  &
              (dn0(k,i,j)+dn0(k,i,j-jdim)))  &
              *dyt(i,j)*0.5_f4)  &
              /rtgt(i,j))

      end do
   end do
end do

! Add in weighted past vertical velocity [dw/dz (at time t)] term:

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         pp(k,i,j) = pp(k,i,j)+  &
              betam1*(coeff_gkdn0(k,i,j)*realwp(k,i,j)+  &
              coeff_fkdn0(k,i,j)*realwp(k-1,i,j))
      end do
   end do
end do

! Add in diabatic heating term:

do j=ja,jz
   do i=ia,iz
      do k=1,m1-1     ! Should we override tht(m1) to reflect BC setting????? 
         pp(k,i,j) = pp(k,i,j)+  &
              (rocv_ssred*pi0(k,i,j)/thc(k,i,j)*tht(k,i,j))*dts
      end do
   end do
end do

! Add in condensation PI tendency and mass divergence terms:

do j=1,m3
   do i=1,m2
      do k=1,m1
         pp(k,i,j) = pp(k,i,j)+(pt(k,i,j)+mass_fluxdiv(k,i,j))*dts
      end do
   end do
end do

end subroutine prdctp1


!```````````````````````````````````````````````````````````````````
subroutine prdctp2(m1, m2, m3, ia, iz, ja, jz, pp, pi0, wp, coeff_fkdn0,  &
     coeff_gkdn0)

!+ Predict PI - part 2 (acoustic model)

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: c_gd, i_grid

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: pi0, wp, coeff_fkdn0, coeff_gkdn0

real(f4), dimension(m1,m2,m3), intent(INOUT) :: pp

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

! Finish PI prediction by adding vertical mass divergence term:

do j=ja,jz
   do i=ia,iz
      do k=2,m1-1
         pp(k,i,j) = pp(k,i,j) + &
              (coeff_gkdn0(k,i,j)*wp(k,i,j)+coeff_fkdn0(k,i,j)*wp(k-1,i,j))
      end do
   end do
end do

if (c_gd%nstbot == 1) then
!@   call botset(m1,m2,m3,ia,iz,ja,jz,grid,pp,'P')
   call botset_atm_mass(m1, m2, m3, ia, iz, ja, jz, i_grid, pp, pi0, 'P')
end if

end subroutine prdctp2


!```````````````````````````````````````````````````````````````````
subroutine theta_subtract_dt(m1, m2, m3, ia, iz, ja, jz, thc, thp, tht)

!+ Revert theta values to that of previous timestep

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: c_gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: thp, tht

real(f4), dimension(m1,m2,m3), intent(OUT) :: thc

!== Local declarations:

! NONE

!=== Executable statements:

thc = thp-tht*c_gd%dt_dyn

end subroutine theta_subtract_dt


!```````````````````````````````````````````````````````````````````
subroutine theta_update_acoust(m1, m2, m3, ia, iz, ja, jz, thc, tht)

!+ Advance theta values forward by one acoustic timestep

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: dts

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: tht

real(f4), dimension(m1,m2,m3), intent(INOUT) :: thc

!== Local declarations:

! NONE

!=== Executable statements:

thc = thc+tht*dts     ! Step in accordance with leapfrog timestep

end subroutine theta_update_acoust


end module acoustic_mod


!```````````````````````````````````````````````````````````````````
subroutine acoustic(iter, tx_bp, rx_bp, tx_ireq, rx_ireq)

!+ Acoustic term driver; advances model through acoustic (small) timesteps 

use univ_kind_defs_mod, only: f4
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: a, iscr2, c_gd, iscr1, prognose_uvwp,  &
     run_is_parallel, izu, ibndc, jzv, mzp, mxp, myp, ia, iz, ja, jz,  &
     n_PE_leaf, n_acoustic_lBC_optypes
use vtables_mod, only: ivt3dd, ivt3de, idn0, ipi0, ith0, irtgt,  &
     ivt3df, ivt3dg, iut, iup, ipp, if13u, irtgu, idxu, ivt3dh, itopu, ivp,  &
     ivt, if23v, irtgv, ivt3da, ivt3db, idyv, itopv, iwp, iwt, itopt, ipt,  &
     if13t, if23t, ivt2da, ifmapui, ifmapvi, idxt, idyt, ifmapt, ivt3dm,  &
     ithp, itht, ivt3dn, ivt3di, ivt3dj
use acoustic_mod, only: coeff_z, theta_subtract_dt, prdctu, prdctv,  &
     prdctw1, prdctw2, prdctw3, prdctp1, prdctp2, theta_update_acoust 
 
implicit none

!=== Argument declarations:

integer, intent(IN) :: iter

integer, dimension(n_PE_leaf,n_acoustic_lBC_optypes), intent(INOUT) ::  &
     rx_ireq, tx_ireq
#if defined(USE_MPI)
type(MPI_buffer_packed_obj), dimension(n_PE_leaf, n_acoustic_lBC_optypes),  &
     intent(INOUT) :: tx_bp, rx_bp
#else  /* Faux interface */
integer, dimension(n_PE_leaf, n_acoustic_lBC_optypes), intent(INOUT) ::  &
     tx_bp, rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

real(f4) :: betam1

!=== Executable statements:

if (.not.prognose_uvwp) return   ! Prognostic u, v, w, p not desired

! Get coefficients for computations:

if (iter == 1) then

   ! Legend for coefficient scratch arrays:
   !
   ! coeff_dm =     vt3dd
   ! coeff_em =     vt3de
   ! coeff_fkdn0 =  vt3df
   ! coeff_gkdn0 =  vt3dg
   ! amoe =         vt3di
   ! amof =         vt3dj
   ! acoaa =        scr2

   call coeff_z(mzp, mxp, myp, ia, iz, ja, jz, a(ivt3dd), a(ivt3de),  &
        a(ivt3df), a(ivt3dg), a(idn0), a(ipi0), a(ith0), a(irtgt), betam1,  &
        a(ivt3di), a(ivt3dj), a(iscr2))

   call theta_subtract_dt(mzp, mxp, myp, ia, iz, ja, jz, a(ivt3dm), a(ithp),  &
        a(itht))

end if

if (run_is_parallel) then
#if defined(USE_MPI)
   if (iter /= c_gd%n_dta_per_dtdyn) call leafPE_acoustic_lBC_receive(4,  &
        rx_bp, tx_ireq, rx_ireq, n_PE_leaf)
#endif  /* end:USE_MPI */
end if

call prdctu(mzp, mxp, myp, ia, izu, ja, jz, ibndc, a(iup), a(iut), a(ipp),  &
     a(ivt3da), a(ith0), a(ivt3db), a(if13u), a(irtgu), a(irtgt), a(idxu),  &
     a(ivt3dh), a(itopu))

if (run_is_parallel) then
#if defined(USE_MPI)
   if (iter /= c_gd%n_dta_per_dtdyn) call leafPE_acoustic_lBC_transmit(2,  &
        tx_bp, rx_bp, tx_ireq, rx_ireq, n_PE_leaf)
#endif  /* end:USE_MPI */
end if

if (c_gd%nxtnest == 0) call cyclic(c_gd%nzp, c_gd%nxp, c_gd%nyp, a(iup), 'U')

call prdctv(mzp, mxp, myp, ia, iz, ja, jzv, ibndc, a(ivp), a(ivt), a(ipp),  &
     a(ivt3da), a(ith0), a(ivt3db), a(if23v), a(irtgv), a(irtgt), a(idyv),  &
     a(ivt3dh), a(itopv))

if (run_is_parallel) then
#if defined(USE_MPI)
   if (iter /= c_gd%n_dta_per_dtdyn) then
      call leafPE_acoustic_lBC_transmit(3, tx_bp, rx_bp, tx_ireq, rx_ireq,  &
           n_PE_leaf)
   else
      call leafPE_acoustic_lBC_transmit(5, tx_bp, rx_bp, tx_ireq, rx_ireq,  &
           n_PE_leaf)
   end if
#endif  /* end:USE_MPI */
end if

if (c_gd%nxtnest == 0) call cyclic(c_gd%nzp, c_gd%nxp, c_gd%nyp, a(ivp), 'V')

call prdctw1(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iwp), a(iwt), a(ipp),  &
     a(ivt3dd), a(ivt3de), betam1, a(ivt3dh), a(irtgt), a(itopt), a(ivt3dm),  &
     a(ith0), a(ivt3dn))

if (run_is_parallel) then
#if defined(USE_MPI)
   if (iter /= c_gd%n_dta_per_dtdyn) then
      call leafPE_acoustic_lBC_receive(2, rx_bp, tx_ireq, rx_ireq, n_PE_leaf)
      call leafPE_acoustic_lBC_receive(3, rx_bp, tx_ireq, rx_ireq, n_PE_leaf)
   else
      call leafPE_acoustic_lBC_receive(5, rx_bp, tx_ireq, rx_ireq, n_PE_leaf)
   end if
#endif  /* end:USE_MPI */
end if

call prdctp1(mzp, mxp, myp, ia, iz, ja, jz, a(ipp), a(iup), a(ivp), a(ipi0),  &
     a(idn0), a(ipt), a(ivt3da), a(ivt3db), a(if13t), a(if23t), a(irtgt),  &
     a(irtgu), a(irtgv), a(ivt2da), a(ifmapui), a(ifmapvi), a(idxt), a(idyt),  &
     a(ifmapt), a(ivt3dm), a(itht), a(ivt3dn), betam1, a(ivt3df), a(ivt3dg))

call prdctw2(mzp, mxp, myp, ia, iz, ja, jz, a(iwp), a(ipp), a(ivt3dd),  &
     a(ivt3de), a(ivt3dj), a(iscr1), a(iscr2), a(ivt2da))

call prdctw3(mzp, mxp, myp, ia, iz, ja, jz, a(iwp), a(iscr1), a(ivt3di))

if (c_gd%nxtnest == 0) call cyclic(c_gd%nzp, c_gd%nxp, c_gd%nyp, a(iwp), 'W')

call prdctp2(mzp, mxp, myp, ia, iz, ja, jz, a(ipp), a(ipi0), a(iwp),  &
     a(ivt3df), a(ivt3dg))

if (c_gd%nxtnest == 0) call cyclic(c_gd%nzp, c_gd%nxp, c_gd%nyp, a(ipp), 'T')

if (run_is_parallel) then
#if defined(USE_MPI)
   if (iter /= c_gd%n_dta_per_dtdyn) then
      call leafPE_acoustic_lBC_transmit(4, tx_bp, rx_bp, tx_ireq, rx_ireq,  &
           n_PE_leaf)
   else
      call leafPE_acoustic_lBC_transmit(6, tx_bp, rx_bp, tx_ireq, rx_ireq,  &
           n_PE_leaf)
      call leafPE_acoustic_lBC_receive(6, rx_bp, tx_ireq, rx_ireq, n_PE_leaf)
   end if
#endif  /* end:USE_MPI */
end if

call theta_update_acoust(mzp, mxp, myp, ia, iz, ja, jz, a(ivt3dm), a(itht))

end subroutine acoustic
