!```````````````````````````````````````````````````````````````````
module advection_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine vel_advectc(m1, m2, m3, ia, iz, ja, jz, izu, jzv, uc, vc, wc, ut,  &
     vt, wt, dn0, dn0u, dn0v, flxu, flxv, flxw, dxt, dxu, dxv, dyt, dyu, dyv,  &
     rtgt, rtgu, rtgv, f13t, f23t, fmapt, fmapu, fmapv, fmapui, fmapvi)

!+ Compute advection of velocities

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: itopo, jdim, c_gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, izu, jzv
real(f4), dimension(m1,m2,m3), intent(IN) :: uc, vc, wc, dn0, dn0u, dn0v
real(f4), dimension(m2,m3), intent(IN) :: dxt, dxu, dxv, dyt, dyu, dyv, rtgt,  &
     rtgu, rtgv, f13t, f23t, fmapt, fmapu, fmapv, fmapui, fmapvi

real(f4), dimension(m1,m2,m3), intent(INOUT) :: flxu, flxv, flxw, ut, vt, wt

!== Local declarations:

integer :: j, i, k, jm, im
real(f4) :: c1z, c1x, c1y

!=== Executable statements:

! Compute momentum fluxes flxu, flxv, flxw:

do j = 1,m3
   do i = 1,m2
      do k = 1,m1
         flxu(k,i,j) = uc(k,i,j) * dn0u(k,i,j) * rtgu(i,j)  &
              * fmapui(i,j)
         flxv(k,i,j) = vc(k,i,j) * dn0v(k,i,j) * rtgv(i,j)  &
              * fmapvi(i,j)
      end do
   end do
end do

if (itopo == 0) then
   do j = 1,m3
      do i = 1,m2
         do k = 1,m1-1
            flxw(k,i,j) = wc(k,i,j) * 0.5_f4 * (dn0(k,i,j) + dn0(k+1,i,j))
         end do
      end do
   end do
else
   do j = 1,m3
      jm = max(j-1, 1)
      do i = 1,m2
         im = max(i-1, 1)
         do k = 1,m1-1
            flxw(k,i,j) = wc(k,i,j)  &
                 * 0.5_f4 * (dn0(k,i,j) + dn0(k+1,i,j))  &
                 + c_gd%hw4(k) * ((flxu(k,i,j) + flxu(k+1,i,j)  &
                 + flxu(k,im,j) + flxu(k+1,im,j)) * f13t(i,j)  &
                 + (flxv(k,i,j) + flxv(k+1,i,j)  &
                 + flxv(k,i,jm) + flxv(k+1,i,jm)) * f23t(i,j))
         end do
      end do
   end do
end if

! Compute advection contribution to U tendency:

do j = ja,jz
   do i = ia,izu
      c1z = 0.25_f4 / rtgu(i,j)
      c1x = c1z * fmapu(i,j) * dxu(i,j)
      c1y = c1z * fmapu(i,j) * dyu(i,j)

      do k = 2,m1-1
         ut(k,i,j) = ut(k,i,j) + c1x / dn0u(k,i,j) * (  &
              (flxu(k,i,j) + flxu(k,i-1,j))  &
              * (uc(k,i,j) + uc(k,i-1,j))  &
              - (flxu(k,i,j) + flxu(k,i+1,j))  &
              * (uc(k,i,j) + uc(k,i+1,j))  &
              + (flxu(k,i+1,j) - flxu(k,i-1,j)) * 2._f4* uc(k,i,j) )
      end do

      do k = 2,m1-1
         ut(k,i,j) = ut(k,i,j) + c1y / dn0u(k,i,j) * (  &
              (flxv(k,i,j-jdim) + flxv(k,i+1,j-jdim))  &
              * (uc(k,i,j) + uc(k,i,j-jdim))  &
              - (flxv(k,i,j) + flxv(k,i+1,j))  &
              * (uc(k,i,j) + uc(k,i,j+jdim))&
              + (flxv(k,i,j) + flxv(k,i+1,j) - flxv(k,i,j-jdim)  &
              - flxv(k,i+1,j-jdim)) * 2._f4* uc(k,i,j) )
      end do

      do k = 2,m1-1
         ut(k,i,j) = ut(k,i,j) + c1z * c_gd%dzt(k) / dn0u(k,i,j) * (  &
              (flxw(k-1,i,j) + flxw(k-1,i+1,j))  &
              * (uc(k,i,j) + uc(k-1,i,j))  &
              - (flxw(k,i,j) + flxw(k,i+1,j))  &
              * (uc(k,i,j) + uc(k+1,i,j))   &
              + (flxw(k,i,j) + flxw(k,i+1,j) - flxw(k-1,i,j)  &
              - flxw(k-1,i+1,j)) * 2._f4* uc(k,i,j) )
      end do
   end do
end do

! Compute advection contribution to V tendency

do j = ja,jzv
   do i = ia,iz
      c1z = 0.25_f4 / rtgv(i,j)
      c1x = c1z * fmapv(i,j) * dxv(i,j)
      c1y = c1z * fmapv(i,j) * dyv(i,j)
      
      do k = 2,m1-1
         vt(k,i,j) = vt(k,i,j) + c1x / dn0v(k,i,j) * (  &
              (flxu(k,i-1,j) + flxu(k,i-1,j+jdim))  &
              * (vc(k,i,j) + vc(k,i-1,j))  &
              - (flxu(k,i,j) + flxu(k,i,j+jdim))  &
              * (vc(k,i,j) + vc(k,i+1,j))  &
              + (flxu(k,i,j) + flxu(k,i,j+jdim) - flxu(k,i-1,j)  &
              - flxu(k,i-1,j+jdim)) * 2._f4* vc(k,i,j) )
      end do
      
      do k = 2,m1-1
         vt(k,i,j) = vt(k,i,j) + c1y / dn0v(k,i,j) * (  &
              (flxv(k,i,j) + flxv(k,i,j-jdim))  &
              * (vc(k,i,j) + vc(k,i,j-jdim))  &
              - (flxv(k,i,j) + flxv(k,i,j+jdim))  &
              * (vc(k,i,j) + vc(k,i,j+jdim))  &
              + (flxv(k,i,j+jdim) - flxv(k,i,j-jdim))  &
              * 2._f4* vc(k,i,j) )
      end do

      do k = 2,m1-1
         vt(k,i,j) = vt(k,i,j) + c1z * c_gd%dzt(k) / dn0v(k,i,j) * (  &
              (flxw(k-1,i,j) + flxw(k-1,i,j+jdim))  &
              * (vc(k,i,j) + vc(k-1,i,j))  &
              - (flxw(k,i,j) + flxw(k,i,j+jdim))  &
              * (vc(k,i,j) + vc(k+1,i,j))  &
              + (flxw(k,i,j) + flxw(k,i,j+jdim) - flxw(k-1,i,j)  &
              - flxw(k-1,i,j+jdim)) * 2._f4* vc(k,i,j) )
      end do
   end do
end do

! Compute advection contribution to W tendency

do j = ja,jz
   do i = ia,iz
      c1z = 0.5_f4 / rtgt(i,j)
      c1x = c1z * fmapt(i,j) * dxt(i,j)
      c1y = c1z * fmapt(i,j) * dyt(i,j)

      do k = 2,m1-2
         wt(k,i,j) = wt(k,i,j)  &
              + c1x / (dn0(k,i,j) + dn0(k+1,i,j)) * (  &
              (flxu(k,i-1,j) + flxu(k+1,i-1,j))  &
              * (wc(k,i,j) + wc(k,i-1,j))  &
              - (flxu(k,i,j) + flxu(k+1,i,j))  &
              * (wc(k,i,j) + wc(k,i+1,j))  &
              + (flxu(k,i,j) + flxu(k+1,i,j) - flxu(k,i-1,j)  &
              - flxu(k+1,i-1,j)) * 2._f4* wc(k,i,j) )
      end do
      
      do k = 2,m1-2
         wt(k,i,j) = wt(k,i,j)  &
              + c1y / (dn0(k,i,j) + dn0(k+1,i,j)) * (  &
              (flxv(k,i,j-jdim) + flxv(k+1,i,j-jdim))  &
              * (wc(k,i,j) + wc(k,i,j-jdim))  &
              - (flxv(k,i,j) + flxv(k+1,i,j))  &
              * (wc(k,i,j) + wc(k,i,j+jdim))  &
              + (flxv(k,i,j) + flxv(k+1,i,j) - flxv(k,i,j-jdim)  &
              - flxv(k+1,i,j-jdim)) * 2._f4* wc(k,i,j) )
      end do

      do k = 2,m1-2
         wt(k,i,j) = wt(k,i,j)  &
              + c1z * c_gd%dzm(k) / (dn0(k,i,j) + dn0(k+1,i,j)) * (  &
              (flxw(k,i,j) + flxw(k-1,i,j))  &
              * (wc(k,i,j) + wc(k-1,i,j))  &
              - (flxw(k,i,j) + flxw(k+1,i,j))  &
              * (wc(k,i,j) + wc(k+1,i,j))   &
              + (flxw(k+1,i,j) - flxw(k-1,i,j)) * 2._f4* wc(k,i,j) )
      end do
   end do
end do

end subroutine vel_advectc


!```````````````````````````````````````````````````````````````````
subroutine fa_preptc(m1, m2, m3, vt3da, vt3db, vt3dc, vt3dd, vt3de, vt3df,  &
     vt3dh, vt3di, vt3dj, vt3dk, dn0, dn0u, dn0v, rtgt, rtgu, rtgv, fmapt,  &
     fmapui, fmapvi, f13t, f23t, dxu, dyv, dxt, dyt, mynum)

!+ Compute quantities in preparation for scalar advection computations

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use main_memory_mod, only: c_gd, vctr1, vctr2, vctr25

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, mynum
real(f4), dimension(m1,m2,m3), intent(IN) :: dn0, dn0u, dn0v
real(f4), dimension(m2,m3), intent(IN) :: rtgt, rtgu, rtgv, fmapt, fmapui,  &
     fmapvi, f13t, f23t, dxu, dyv, dxt, dyt

real(f4), dimension(m1,m2,m3), intent(INOUT) :: vt3da, vt3db, vt3dc, vt3dd,  &
     vt3de, vt3df, vt3dh, vt3di, vt3dj, vt3dk

!== Local declarations:

integer :: j, i, k, im, ip, jm, jp
real(f4) :: c1, c2, c3, c4, rtgti

!=== Executable statements:

! VT3DA, VT3DB, and VT3DC are input as the velocity components (averaged
! between past and current time levels) times dtlt:

! Add contribution to VT3DC from horiz winds crossing sloping sigma surfaces,
!    and include 1/rtgt factor in VT3DC
! Compute half Courant numbers: VT3DD, VT3DE, and VT3DF
! Compute weight at scalar point: VT3DH
! Compute advective weights for the linear term: VT3DI, VCTR1, and VCTR2

do j = 1,m3
   jm = max(1, j-1)
   jp = min(m3, j+1)
   do i = 1,m2
      im = max(1, i-1)
      ip = min(m2, i+1)
      rtgti = ONE_f4 / rtgt(i,j)
      c1 = 0.5_f4 * dxu(i,j)
      c2 = 0.5_f4 * dyv(i,j)
      c3 = dxt(i,j) * fmapt(i,j) * rtgti
      c4 = dyt(i,j) * fmapt(i,j) * rtgti

      do k = 1,m1-1
         vt3dc(k,i,j) = ((vt3da(k,i,j) + vt3da(k+1,i,j)  &
              + vt3da(k,im,j) + vt3da(k+1,im,j)) * f13t(i,j)  &
              + (vt3db(k,i,j) + vt3db(k+1,i,j) + vt3db(k,i,jm)  &
              + vt3db(k+1,i,jm)) * f23t(i,j)) * c_gd%hw4(k)  &
              + vt3dc(k,i,j) * rtgti
         vt3dd(k,i,j) = c1 * vt3da(k,i,j)
         vt3de(k,i,j) = c2 * vt3db(k,i,j)
         vt3df(k,i,j) = 0.5_f4 * vt3dc(k,i,j) * c_gd%dzm(k)
         vctr25(k) = ONE_f4 / dn0(k,i,j)
         vt3dh(k,i,j) = c3 * vctr25(k)
         vt3dj(k,i,j) = c4 * vctr25(k)
         vt3dk(k,i,j) = c_gd%dzt(k) * vctr25(k)
      end do

!            vt3di(1,i,j) = dxu(i,j) / (dxu(i,j) + dxt(ip,j))
!            vt3di(2,i,j) = dxu(i,j) / (dxu(i,j) + dxt(i,j))
!            vt3di(3,i,j) = dyv(i,j) / (dyv(i,j) + dyt(i,jp))
!            vt3di(4,i,j) = dyv(i,j) / (dyv(i,j) + dyt(i,j))
! temporary override
      vt3di(1,i,j) = 0.5_f4
      vt3di(2,i,j) = 0.5_f4
      vt3di(3,i,j) = 0.5_f4
      vt3di(4,i,j) = 0.5_f4
   end do
end do

do k = 1,m1-1
   vctr1(k) = (c_gd%zt(k+1) - c_gd%zm(k)) * c_gd%dzm(k)
   vctr2(k) =  (c_gd%zm(k) - c_gd%zt(k)) * c_gd%dzm(k)
end do

! Convert velocity components * dtlt (VT3DA, VT3DB, VT3DC)
! into mass fluxes times dtlt:

do j = 1,m3
   do i = 1,m2
      c1 = fmapui(i,j) * rtgu(i,j)
      c2 = fmapvi(i,j) * rtgv(i,j)
      do k = 1,m1-1
         vt3da(k,i,j) = vt3da(k,i,j) * c1 * dn0u(k,i,j)
         vt3db(k,i,j) = vt3db(k,i,j) * c2 * dn0v(k,i,j)
         vt3dc(k,i,j) = vt3dc(k,i,j) * 0.5_f4 * (dn0(k,i,j) + dn0(k+1,i,j))
      end do
   end do
end do

end subroutine fa_preptc


!```````````````````````````````````````````````````````````````````
subroutine fa_xc(m1, m2, m3, ia, iz, ja, jz, scp, scr1, vt3da, vt3dd, vt3dg,  &
     vt3dh, vt3di, mynum)

!+ Compute advective scalar flux in x-direction

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, mynum
real(f4), dimension(m1,m2,m3), intent(IN) :: scp, vt3da, vt3dd, vt3dh, vt3di

real(f4), dimension(m1,m2,m3), intent(INOUT) :: scr1, vt3dg

!== Local declarations:

integer :: i, j, k
real(f4) :: dfact

!=== Executable statements:

dfact = 0.5_f4
do j = 1,m3
   do i = 1,iz

      ! Compute scalar flux times dtlt [VT3DG]

      do k = 2,m1-1
         vt3dg(k,i,j) = vt3da(k,i,j)  &
              * (vt3di(1,i,j) * scr1(k,i,j)  &
              +  vt3di(2,i,j) * scr1(k,i+1,j)  &
              +  vt3dd(k,i,j) * (scr1(k,i,j) - scr1(k,i+1,j)))
      end do

      ! Modify fluxes to retain positive-definiteness on scalar quantities.
      !    If a flux will remove 1/2 quantity during a timestep,
      !    reduce to first order flux. This will remain positive-definite
      !    under the assumption that ABS(CFL(i)) + ABS(CFL(i-1)) < 1.0 if
      !    both fluxes are evacuating the box.

      do k = 2,m1-1
         if (vt3da(k,i,j) > ZERO_f4) then
            if (vt3dg(k,i,j) * vt3dh(k,i,j) > dfact * scr1(k,i,j)) then
               vt3dg(k,i,j) = vt3da(k,i,j) * scr1(k,i,j)
            end if
         else if (vt3da(k,i,j) < ZERO_f4) then
            if (-vt3dg(k,i,j) * vt3dh(k,i+1,j) > dfact * scr1(k,i+1,j)) then
               vt3dg(k,i,j) = vt3da(k,i,j) * scr1(k,i+1,j)
            end if
         end if
      end do
   end do
end do

! Compute flux divergence

do j = 1,m3
   do i = ia,iz
      do k = 2,m1-1
         scr1(k,i,j) = scr1(k,i,j)  &
              + vt3dh(k,i,j) * (vt3dg(k,i-1,j) - vt3dg(k,i,j)  &
              + scp(k,i,j) * (vt3da(k,i,j) - vt3da(k,i-1,j)))
      end do
   end do
end do

end subroutine fa_xc


!```````````````````````````````````````````````````````````````````
subroutine fa_yc(m1, m2, m3, ia, iz, ja, jz, scp, scr1, vt3db, vt3de, vt3dg,  &
     vt3dj, vt3di, jdim, mynum)

!+ Compute advective scalar flux in y-direction

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, jdim, mynum
real(f4), dimension(m1,m2,m3), intent(IN) :: scp, vt3db ,vt3de, vt3dj, vt3di

real(f4), dimension(m1,m2,m3), intent(INOUT) :: scr1, vt3dg

!== Local declarations:

integer :: i, j, k
real(f4) :: dfact

!=== Executable statements:

dfact = 0.5_f4
do j = 1,jz
   do i = ia,iz

      ! Compute scalar flux VT3DG

      do k = 2,m1-1
         vt3dg(k,i,j) = vt3db(k,i,j)  &
              * (vt3di(3,i,j) * scr1(k,i,j)  &
              +  vt3di(4,i,j) * scr1(k,i,j+jdim)  &
              +  vt3de(k,i,j) * (scr1(k,i,j) - scr1(k,i,j+jdim)))
      end do

      !     Modify fluxes to retain positive-definiteness on scalar quantities.
      !         If a flux will remove 1/2 quantity during a timestep,
      !         reduce to first order flux. This will remain positive-definite
      !         under the assumption that ABS(CFL(i)) + ABS(CFL(i-1)) < 1.0 if
      !         both fluxes are evacuating the box.

      do k = 2,m1-1
         if (vt3db(k,i,j) > ZERO_f4) then
            if (vt3dg(k,i,j) * vt3dj(k,i,j) > dfact * scr1(k,i,j)) then
               vt3dg(k,i,j) = vt3db(k,i,j) * scr1(k,i,j)
            end if
         else if (vt3db(k,i,j) < ZERO_f4) then
            if (-vt3dg(k,i,j) * vt3dj(k,i,j+jdim) >  &
                 dfact * scr1(k,i,j+jdim)) then
               vt3dg(k,i,j) = vt3db(k,i,j) * scr1(k,i,j+jdim)
            end if
         end if
      end do
   end do
end do

! Compute flux divergence

do j = ja,jz
   do i = ia,iz
      do k = 2,m1-1
         scr1(k,i,j) = scr1(k,i,j)  &
              + vt3dj(k,i,j) * (vt3dg(k,i,j-jdim) - vt3dg(k,i,j)  &
              + scp(k,i,j) * (vt3db(k,i,j) - vt3db(k,i,j-jdim)))
      end do
   end do
end do

end subroutine fa_yc


!```````````````````````````````````````````````````````````````````
subroutine fa_zc_N(m1, m2, m3, ia, iz, ja, jz, scp, scr1, vt3dc, vt3df,  &
     vt3dg, vt3dk, vctr1, vctr2, mynum)

!+ Compute advective scalar flux in z-direction ("normal" scalar)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, mynum
real(f4), dimension(m1,m2,m3), intent(IN) :: scp, vt3dc, vt3df, vt3dk
real(f4), dimension(:), intent(IN) :: vctr1, vctr2

real(f4), dimension(m1,m2,m3), intent(INOUT) :: scr1, vt3dg

!== Local declarations:

integer :: i, j, k
real(f4) :: dfact

!=== Executable statements:

dfact = 0.5_f4
do j = ja,jz
   do i = ia,iz

      ! Compute scalar flux VT3DG

      do k = 1,m1-1
         vt3dg(k,i,j) = vt3dc(k,i,j)  &
              * (vctr1(k) * scr1(k,i,j)  &
              +  vctr2(k) * scr1(k+1,i,j)  &
              +  vt3df(k,i,j) * (scr1(k,i,j) - scr1(k+1,i,j)))
      end do

      ! Modify fluxes to retain positive-definiteness on scalar quantities.
      !    If a flux will remove 1/2 quantity during a timestep,
      !    reduce to first order flux. This will remain positive-definite
      !    under the assumption that ABS(CFL(i)) + ABS(CFL(i-1)) < 1.0 if
      !    both fluxes are evacuating the box.

      do k = 1,m1-1
         if (vt3dc(k,i,j) > ZERO_f4) then
            if (vt3dg(k,i,j) * vt3dk(k,i,j) > dfact * scr1(k,i,j)) then
               vt3dg(k,i,j) = vt3dc(k,i,j) * scr1(k,i,j)
            end if
         else if (vt3dc(k,i,j) < ZERO_f4) then
            if (-vt3dg(k,i,j) * vt3dk(k+1,i,j) > dfact * scr1(k+1,i,j)) then
               vt3dg(k,i,j) = vt3dc(k,i,j) * scr1(k+1,i,j)
            end if
         end if
      end do
   end do
end do

! Compute flux divergence

do j = ja,jz
   do i = ia,iz
      do k = 2,m1-1
         scr1(k,i,j) = scr1(k,i,j)  &
              + vt3dk(k,i,j) * (vt3dg(k-1,i,j) - vt3dg(k,i,j)  &
              + scp(k,i,j) * (vt3dc(k,i,j) - vt3dc(k-1,i,j)))
      end do
   end do
end do

end subroutine fa_zc_N


!```````````````````````````````````````````````````````````````````
subroutine fa_zc_A(m1, m2, m3, ia, iz, ja, jz, scp, scr1, vt3dc, vt3df,  &
     vt3dg, vt3dk, vctr1, vctr2, precip, mynum)

!+ Compute advective scalar flux in z-direction ("aerosol" scalar)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, mynum
real(f4), dimension(m1,m2,m3), intent(IN) :: scp, vt3dc, vt3df, vt3dk
real(f4), dimension(m2,m3), intent(IN) :: precip
real(f4), dimension(:), intent(IN) :: vctr1, vctr2

real(f4), dimension(m1,m2,m3), intent(INOUT) :: scr1, vt3dg

!== Local declarations:

integer :: i, j, k
real(f4) :: dfact

!=== Executable statements:

dfact = 0.5_f4
do j = ja,jz
   do i = ia,iz

      ! Compute scalar flux VT3DG

      do k = 1,m1-1
         vt3dg(k,i,j) = vt3dc(k,i,j)  &
              * (vctr1(k) * scr1(k,i,j)  &
              +  vctr2(k) * scr1(k+1,i,j)  &
              +  vt3df(k,i,j) * (scr1(k,i,j) - scr1(k+1,i,j)))
      end do

      ! Modify fluxes to retain positive-definiteness on scalar quantities.
      !    If a flux will remove 1/2 quantity during a timestep,
      !    reduce to first order flux. This will remain positive-definite
      !    under the assumption that ABS(CFL(i)) + ABS(CFL(i-1)) < 1.0 if
      !    both fluxes are evacuating the box.

      do k = 1,m1-1
         if (vt3dc(k,i,j) > ZERO_f4) then
            if (vt3dg(k,i,j) * vt3dk(k,i,j) > dfact * scr1(k,i,j)) then
               vt3dg(k,i,j) = vt3dc(k,i,j) * scr1(k,i,j)
            end if
         else if (vt3dc(k,i,j) < ZERO_f4) then
            if (-vt3dg(k,i,j) * vt3dk(k+1,i,j) > dfact * scr1(k+1,i,j)) then
               vt3dg(k,i,j) = vt3dc(k,i,j) * scr1(k+1,i,j)
            end if
         end if
      end do
   end do
end do

! Compute flux divergence

do j = ja,jz
   do i = ia,iz
      do k = 2,m1-1
         scr1(k,i,j) = scr1(k,i,j)  &
              + vt3dk(k,i,j) * (vt3dg(k-1,i,j) - vt3dg(k,i,j)  &
              + scp(k,i,j) * (vt3dc(k,i,j) - vt3dc(k-1,i,j)))
      end do
   end do
end do

! Compute surface precipitation amount:

!%do j=ja,jz
!%   do i=ia,iz
!%      precip(i,j) = precip(i,j)-vt3dg(1,i,j)   ! [kg(aerosol)/m^2 lost
!%                                               !   in dtlt seconds]
!%   enddo
!%enddo

end subroutine fa_zc_A


!```````````````````````````````````````````````````````````````````
subroutine advtndc(m1, m2, m3, ia, iz, ja, jz, scp, sca, sct, dtl, mynum)

!+ Compute and fill advective scalar tendencies

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, mynum
real(f4), dimension(m1,m2,m3), intent(IN) :: scp, sca
real(f4), intent(IN) :: dtl

real(f4), dimension(m1,m2,m3), intent(INOUT) :: sct

!== Local declarations:

integer :: i, j, k
real(f4) :: dtli

!=== Executable statements:

dtli = ONE_f4/dtl
do j = ja,jz
   do i = ia,iz
      do k = 2,m1-1
         sct(k,i,j) = sct(k,i,j) + (sca(k,i,j)-scp(k,i,j)) * dtli
      end do
   end do
end do

end subroutine advtndc


end module advection_mod


!```````````````````````````````````````````````````````````````````
subroutine advect_c(varn, mzp, mxp, myp, ia, iz, ja, jz, izu, jzv, mynum)

!+ Advection driver

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: a, c_gd, nscl, i_grid, iscr1, jdim, vctr1, vctr2,  &
     tstep, naerscl, enable_advection
use main_memory_bspec_mod, only: dust_nb, H2Oi_nb, CO2i_nb
use vtables_mod, only: iuc, ivc, iwc, iut, ivt, iwt, idn0, idn0u, idn0v,  &
     ivt3da, ivt3db, ivt3dc, idxt, idxu, idxv, idyt, idyu, idyv, irtgt,  &
     irtgu, irtgv, if13t, if23t, ifmapt, ifmapu, ifmapv, ifmapui, ifmapvi,  &
     iup, ivp, iwp, ivt3dd, ivt3de, ivt3df, ivt3dg, ivt3dh, ivt3di, ivt3dj,  &
     ivt3dk, iscp, isct, iaerscp, iaersct, fi_idx, ff_idx, aj_idx, ag_idx,  &
     rs_idx, idustfi, idustff, idustaj, idustag, idustrs, iH2Oifi, iH2Oiff,  &
     iH2Oiaj, iH2Oiag, iH2Oirs, iH2Odfi, iH2Odff, iH2Odaj, iH2Odag, iH2Odrs,  &
     iCO2ifi, iCO2iff, iCO2iaj, iCO2iag, iCO2irs, iCO2dfi, isclrp, iCO2dff,  &
     iCO2daj, iCO2dag, iCO2drs, iCO2wfi, iCO2wff, iCO2waj, iCO2wag, iCO2wrs,  &
     RV_vidx
use aermic_rad_mem_mod, only: aer_XOadvect, aer_tyet
use advection_mod, only: vel_advectc, fa_preptc, fa_xc, fa_yc, fa_zc_n,  &
     advtndc
use grid_mem_mod, only: switch_to_Nscalar, switch_to_Ascalar

implicit none

!=== Argument declarations:

integer, intent(IN) :: mzp, mxp, myp, ia, iz, ja, jz, izu, jzv, mynum
character(len=*), intent(IN) :: varn

!== Local declarations:

integer :: i, n, mxyzp1, ib
real(f4) :: dtlto2

!=== Executable statements:

if (.not.enable_advection) return    ! No advection calculation(s) desired

if (varn == 'V' .or. varn == 'ALL') then

   ! Advect  U, V, and W:

   call vel_advectc(mzp, mxp, myp, ia, iz, ja, jz, izu, jzv, a(iuc), a(ivc),  &
        a(iwc), a(iut), a(ivt), a(iwt), a(idn0), a(idn0u), a(idn0v),  &
        a(ivt3da), a(ivt3db), a(ivt3dc), a(idxt), a(idxu), a(idxv), a(idyt),  &
        a(idyu), a(idyv), a(irtgt), a(irtgu), a(irtgv), a(if13t), a(if23t),  &
        a(ifmapt), a(ifmapu), a(ifmapv), a(ifmapui), a(ifmapvi))

end if

if (varn == 'T' .or. varn == 'ALL') then    ! Advect scalars

   if (tstep == 1) then   ! Only do once (per grid) per model *run*

      ! Calculate surface source/sink array indices for use throughout the run:

      n = 0
      do ib=1,dust_nb   ! Dust aerosol group
         n = n+1
         fi_idx(n,i_grid) = idustfi(ib)
         ff_idx(n,i_grid) = idustff(ib)
         aj_idx(n,i_grid) = idustaj(ib)
         ag_idx(n,i_grid) = idustag(ib)
         rs_idx(n,i_grid) = idustrs(ib)
      end do
      do ib=1,H2Oi_nb     ! Water ice aerosol group
         n = n+1
         fi_idx(n,i_grid) = iH2Oifi(ib)
         ff_idx(n,i_grid) = iH2Oiff(ib)
         aj_idx(n,i_grid) = iH2Oiaj(ib)
         ag_idx(n,i_grid) = iH2Oiag(ib)
         rs_idx(n,i_grid) = iH2Oirs(ib)
      end do
      do ib=1,H2Oi_nb     ! Water ice aerosol group; dust core
         n = n+1
         fi_idx(n,i_grid) = iH2Odfi(ib)
         ff_idx(n,i_grid) = iH2Odff(ib)
         aj_idx(n,i_grid) = iH2Odaj(ib)
         ag_idx(n,i_grid) = iH2Odag(ib)
         rs_idx(n,i_grid) = iH2Odrs(ib)
      end do
      do ib=1,CO2i_nb   ! CO2 ice aerosol group
         n = n+1
         fi_idx(n,i_grid) = iCO2ifi(ib)
         ff_idx(n,i_grid) = iCO2iff(ib)
         aj_idx(n,i_grid) = iCO2iaj(ib)
         ag_idx(n,i_grid) = iCO2iag(ib)
         rs_idx(n,i_grid) = iCO2irs(ib)
      end do
      do ib=1,CO2i_nb   ! CO2 ice aerosol group; dust core
         n = n+1
         fi_idx(n,i_grid) = iCO2dfi(ib)
         ff_idx(n,i_grid) = iCO2dff(ib)
         aj_idx(n,i_grid) = iCO2daj(ib)
         ag_idx(n,i_grid) = iCO2dag(ib)
         rs_idx(n,i_grid) = iCO2drs(ib)
      end do
      do ib=1,CO2i_nb   ! CO2 ice aerosol group; H2O core
         n = n+1
         fi_idx(n,i_grid) = iCO2wfi(ib)
         ff_idx(n,i_grid) = iCO2wff(ib)
         aj_idx(n,i_grid) = iCO2waj(ib)
         ag_idx(n,i_grid) = iCO2wag(ib)
         rs_idx(n,i_grid) = iCO2wrs(ib)
      end do

   end if

   mxyzp1 = mxp*myp*mzp-1

   dtlto2 = 0.5_f4*c_gd%dt_dyn
   do i=0,mxyzp1
      a(ivt3da+i) = (a(iup+i) + a(iuc+i)) * dtlto2
      a(ivt3db+i) = (a(ivp+i) + a(ivc+i)) * dtlto2
      a(ivt3dc+i) = (a(iwp+i) + a(iwc+i)) * dtlto2
   end do

   call fa_preptc(mzp, mxp, myp, a(ivt3da), a(ivt3db), a(ivt3dc), a(ivt3dd),  &
        a(ivt3de), a(ivt3df), a(ivt3dh), a(ivt3di), a(ivt3dj), a(ivt3dk),  &
        a(idn0), a(idn0u), a(idn0v), a(irtgt), a(irtgu), a(irtgv), a(ifmapt),  &
        a(ifmapui), a(ifmapvi), a(if13t), a(if23t), a(idxu), a(idyv),  &
        a(idxt), a(idyt), mynum)

   ! Advect "normal scalars:

   do n=1,nscl
      call switch_to_Nscalar(i_grid, n)

      if (isclrp(n,i_grid) == RV_vidx) then ! Convert RV to "mixing ratio" [Mg/kg]
         if (.NOT. aer_tyet(0)) cycle   ! Do not advect RV yet (or at all)
         a(iscp:iscp+mxyzp1) = a(iscp:iscp+mxyzp1)/a(idn0:idn0+mxyzp1)
      end if

      a(iscr1:iscr1+mxyzp1) = a(iscp:iscp+mxyzp1)  ! Fill scratch array [Mg/kg]

      call fa_xc(mzp, mxp, myp, ia, iz, 1, myp, a(iscp), a(iscr1), a(ivt3da),  &
           a(ivt3dd), a(ivt3dg), a(ivt3dh), a(ivt3di), mynum)
      
      if (jdim == 1) then 
         call fa_yc(mzp, mxp, myp, ia, iz, ja, jz, a(iscp), a(iscr1),  &
              a(ivt3db), a(ivt3de), a(ivt3dg), a(ivt3dj), a(ivt3di), jdim,  &
              mynum)
      end if

      call fa_zc_N(mzp, mxp, myp, ia, iz, ja, jz, a(iscp), a(iscr1),  &
           a(ivt3dc), a(ivt3df), a(ivt3dg), a(ivt3dk), vctr1, vctr2, mynum)

      if (isclrp(n,i_grid) == RV_vidx) then ! Convert RV "m.r." back to mass conc
         a(iscr1:iscr1+mxyzp1) = a(iscr1:iscr1+mxyzp1)*a(idn0:idn0+mxyzp1)
         a(iscp:iscp+mxyzp1) = a(iscp:iscp+mxyzp1)*a(idn0:idn0+mxyzp1)
      end if

      call advtndc(mzp, mxp, myp, ia, iz, ja, jz, a(iscp), a(iscr1), a(isct),  &
           c_gd%dt_dyn, mynum)

   end do

   ! Advect "aerosol" scalars:

   do n=1,naerscl

!&&&&&&&&&&&
!      aer_XOadvect(n,grid) = .FALSE.
!&&&&&&&&&&&

      if (aer_XOadvect(n,i_grid)) then   ! Only do if requested/required
              
         call switch_to_Ascalar(i_grid, n)

         ! Convert mass conc to "mixing ratio" [Mg/kg]:
         a(iaerscp:iaerscp+mxyzp1) = a(iaerscp:iaerscp+mxyzp1)/  &
              a(idn0:idn0+mxyzp1)
         a(iscr1:iscr1+mxyzp1) = a(iaerscp:iaerscp+mxyzp1) ! Fill scratch array
 
         call fa_xc(mzp, mxp, myp, ia, iz, 1, myp, a(iaerscp), a(iscr1),  &
              a(ivt3da), a(ivt3dd), a(ivt3dg), a(ivt3dh), a(ivt3di), mynum)
      
         if (jdim == 1) then 
            call fa_yc(mzp, mxp, myp, ia, iz, ja, jz, a(iaerscp), a(iscr1),  &
                 a(ivt3db), a(ivt3de), a(ivt3dg), a(ivt3dj), a(ivt3di), jdim,  &
                 mynum)
         end if

         call fa_zc_N(mzp, mxp, myp, ia, iz, ja, jz, a(iaerscp), a(iscr1),  &
              a(ivt3dc), a(ivt3df), a(ivt3dg), a(ivt3dk), vctr1, vctr2, mynum)

         ! Convert back to mass concentration:
         a(iscr1:iscr1+mxyzp1) = a(iscr1:iscr1+mxyzp1)*a(idn0:idn0+mxyzp1)
         a(iaerscp:iaerscp+mxyzp1) = a(iaerscp:iaerscp+mxyzp1)*  &
              a(idn0:idn0+mxyzp1)

         call advtndc(mzp, mxp, myp, ia, iz, ja, jz, a(iaerscp), a(iscr1),  &
              a(iaersct), c_gd%dt_dyn, mynum)

      end if

   end do

end if

end subroutine advect_c
