!```````````````````````````````````````````````````````````````````
module alt_namelist_mod

!+ Module that contains routines related to processing an "alternative" (with
!   with respect to the standard, but minimal F90 form) type of human-readable
!   namelist

!~ 30.03.2011 (last modified; TIM)

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

private

logical, public, parameter :: nlst_debug = .FALSE. ! If .TRUE., prints namelist
                                                   !   debugging info

! Maximum length (in chars) of one namelist record:
integer, public, parameter :: altn_maxlen_line = 256

! Maximum length (in chars) of a string-valued namelist variable:
integer, public, parameter :: altn_maxlen_strvar = altn_maxlen_line-30

! Maximum length (in chars) of a namelist variable name:
integer, public, parameter :: altn_maxlen_vname = MAX_PROC_NAME_LEN

logical, public :: there_is_an_error

public :: token_list
type token_list
   ! .TRUE. = This variable has not yet been properly initialized by the
   !           provided interface 
   logical, public :: structure_is_raw = .TRUE. 

   integer, public :: n_tokens = 0
   character(len=altn_maxlen_line), public, pointer, dimension(:) ::  &
        tokens => NULL()
end type token_list

interface varca_c0
  module procedure varca_c0_0d, varca_c0_1d
end interface

interface varca_i0
  module procedure varca_i0_0d, varca_i0_1d
end interface

interface varca_f0
  module procedure varca_f0_0d, varca_f0_1d
end interface

public :: varca_c0, varca_i0, varca_f0, find_group, line_strip,  &
     tokenizer, tokenizer_flex, is_1st_a_letter, is_1st_a_number,  &
     is_1st_a_quote, vchk, token_list_destroy

contains


!```````````````````````````````````````````````````````````````````
subroutine token_list_init(tgt_token_list, nt)

!~ 17.11.2008 (last modified; TIM)

!+ Initializes a token_list derived type variable

implicit none

!=== Argument declarations:

integer, intent(IN) :: nt
type(token_list), intent(INOUT) :: tgt_token_list

!== Local declarations:

! NONE

!=== Executable statements:

if(tgt_token_list%structure_is_raw) then
   nullify(tgt_token_list%tokens)
   allocate(tgt_token_list%tokens(nt))
   tgt_token_list%n_tokens = nt
   tgt_token_list%structure_is_raw = .FALSE.
else
   if(tgt_token_list%n_tokens < nt) then    ! Need to reallocate 'tokens'
      deallocate(tgt_token_list%tokens)
      allocate(tgt_token_list%tokens(nt))
   endif
   tgt_token_list%n_tokens = nt
endif

end subroutine token_list_init


!```````````````````````````````````````````````````````````````````
subroutine token_list_destroy(tgt_token_list)

!~ 26.09.2008 (last modified; TIM)

!+ Destroys a token_list derived type variable; reverts it to its original state

implicit none

!=== Argument declarations:

type(token_list), intent(INOUT) :: tgt_token_list

!== Local declarations:

! NONE

!=== Executable statements:

deallocate(tgt_token_list%tokens)
nullify(tgt_token_list%tokens)
tgt_token_list%n_tokens = 0
tgt_token_list%structure_is_raw = .TRUE.

end subroutine token_list_destroy


!```````````````````````````````````````````````````````````````````
subroutine find_group(in_unit,group,group_end,nrec_group)

!~ 17.11.2008 (last modified; TIM)

!+ Looks for a group ID string in a file; returns the # of records the
!   group contains and the file IO unit positioned at the first data record
!   of the group. 

implicit none

!=== Argument declarations:

integer, intent(IN) :: in_unit
character(len=*), intent(IN) :: group,group_end

integer, intent(OUT) :: nrec_group

!== Local declarations:

character(len=altn_maxlen_line) :: line, line_format
integer :: nr,g_beg,g_end,idx,err

!=== Executable statements:

nr = 0
g_beg = 0
g_end = 0

write(line_format,'(a2,i3.3,a1)') '(a', altn_maxlen_line, ')'

! Find start of given group:

beg_loop: do while(.TRUE.)
   nr = nr+1
   read(in_unit,fmt=line_format,iostat=err) line   ! Read in a record

   if(err /= 0) then  ! If a read error, notify user, note it, and return
      write (*,'(/a,a,a/)') 'Read Error: Namelist group '  &
           ,group,' not found.'
      there_is_an_error = .TRUE.
      return
   endif

   idx = index(line,group)  ! Look for string
   if (idx /= 0) then   !
      g_beg = nr        ! If string is found, record rec #, and exit loop
      exit beg_loop     !
   endif                !
enddo beg_loop

! Find end of given group:

end_loop: do while(.TRUE.)
   nr = nr+1
   read(in_unit,fmt=line_format,iostat=err) line   ! Read in a record

   if(err /= 0) then  ! If a read error, notify user, note it, and return
      write (*,'(/a,a,a/)') 'Read Error: End of namelist group '  &
           ,group,'not found.'
      there_is_an_error = .TRUE.
      return
   endif

   idx = index(line,group_end)  ! Look for string
   if (idx /= 0) then   !
      g_end = nr        ! If string is found, record rec #, and exit loop
      exit end_loop     !
   endif                !
enddo end_loop

! Rewind file to first data record in group:

rew_loop: do nr=1,g_end-g_beg
   backspace in_unit
enddo rew_loop

nrec_group = g_end-g_beg-1   ! Number of records in group

end subroutine find_group


!```````````````````````````````````````````````````````````````````
subroutine line_strip(line1,line2,nc_line2)

!~ 17.11.2008 (last modified; TIM)

!+ Strips spaces/comments from line1; stores result in line2 (returns
!   length of line2 as well)

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: line1

character(len=*), intent(INOUT) :: line2
integer, intent(OUT) :: nc_line2

!== Local declarations:

integer :: nc_line1,nc,quote

!=== Executable statements:

nc_line1 = len(line1)
nc_line2 = 0
quote = 0

do nc=1,nc_line1
   if(quote == 0) then
      if(line1(nc:nc) /= ' ') then
         if(line1(nc:nc) == '!') then
            return
         endif
         nc_line2 = nc_line2+1
         line2(nc_line2:nc_line2) = line1(nc:nc)
      endif
   else
      nc_line2 = nc_line2+1
      line2(nc_line2:nc_line2) = line1(nc:nc)
   endif

   if(line1(nc:nc) == '''') then
      if(quote == 0) then
         quote = 1
      else
         quote = 0
      endif
   endif
enddo

end subroutine line_strip


!```````````````````````````````````````````````````````````````````
subroutine tokenizer(str,nc_str,tokens,ntokens)

!+ Parses string for tokens; stores tokens in separate strings

!~ 30.03.2011 (last modified; TIM)

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: str
integer, intent(IN) :: nc_str

integer, intent(OUT) :: ntokens
character(len=*), dimension(:), intent(INOUT) :: tokens

!== Local declarations:

integer, parameter :: nsep = 4

character(len=1), dimension(nsep) :: toksep = (/ '=', ',', '(', ')' /)
integer :: npt, nc, ns

!=== Executable statements:

ntokens = 0
npt = 1

do nc=1,nc_str
   sep_loop: do ns=1,nsep
      if(str(nc:nc) == toksep(ns)) then
         if(nc-npt >= 1) then
            ntokens = ntokens+1
            tokens(ntokens) = str(npt:nc-1)
         endif
         ntokens = ntokens+1
         tokens(ntokens) = str(nc:nc)
         npt = nc+1
         exit sep_loop
      endif
   enddo sep_loop
enddo

end subroutine tokenizer


!```````````````````````````````````````````````````````````````````
subroutine tokenizer_flex(str,tgt_token_list,toksep)

!~ 17.11.2008 (last modified; TIM)

!+ Parses string for tokens; stores tokens in separate strings
!
!  Sets the value of 'tgt_token_list%n_tokens' as follows:
!    -1  :  if a parsing error occurred
!     0  :  if no tokens were found
!    >1  :  if tokens were found and copied into the token_list object

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: str
character(len=1), dimension(:), intent(IN) :: toksep

type(token_list), intent(INOUT) :: tgt_token_list

!== Local declarations:

integer :: ntoks, nseps, nc_str, toksep_cnt, edge_cnt, nc, ns, prior_edge 
integer, dimension(altn_maxlen_line) :: toksep_locs, token_edges

!=== Executable statements:

ntoks = 0
nseps = size(toksep)

nc_str = len_trim(str)  ! String length without any trailing whitespace
if(nc_str == 0 .or. nc_str > altn_maxlen_line) then  ! Unable to process this string
   tgt_token_list%n_tokens = -1
   return
endif

toksep_cnt = 0;
do nc=1,nc_str
   do ns=1,nseps
      if(str(nc:nc) == toksep(ns)) then
         toksep_cnt = toksep_cnt+1
         toksep_locs(toksep_cnt) = nc
      endif
   enddo
enddo

if(toksep_cnt == nc_str) then   ! No tokens to be found
   tgt_token_list%n_tokens = 0
   return
endif   

edge_cnt = 1
token_edges(edge_cnt) = 0; prior_edge = 0
do ns=1,toksep_cnt
   if(toksep_locs(ns)-prior_edge > 1) then
      edge_cnt = edge_cnt+1
      prior_edge = toksep_locs(ns) 
   endif
   token_edges(edge_cnt) = toksep_locs(ns)
enddo

if(toksep_cnt > 0) then
   if(toksep_locs(toksep_cnt) < nc_str) then
      edge_cnt = edge_cnt+1
      token_edges(edge_cnt) = nc_str+1
   endif
endif

ntoks = edge_cnt-1

call token_list_init(tgt_token_list, ntoks)

do ns=1,ntoks
   tgt_token_list%tokens(ns) = str(token_edges(ns)+1:token_edges(ns+1)-1)
enddo

end subroutine tokenizer_flex


!```````````````````````````````````````````````````````````````````
function is_1st_a_letter(str) result(answer)

!~ 17.11.2008 (last modified; TIM)

!+ Logical-valued function that checks if first character of STR is a letter

implicit none

!=== Argument declarations:

logical :: answer

character(len=*), intent(IN) :: str

!== Local declarations:

! NONE

!=== Executable statements:

answer = .FALSE.
if((str(1:1) >= 'A' .and. str(1:1) <= 'Z') .or.  &
     (str(1:1) >= 'a' .and. str(1:1) <= 'z')) then
   answer = .TRUE.
endif

end function is_1st_a_letter


!```````````````````````````````````````````````````````````````````
function is_1st_a_number(str) result(answer)

!~ 17.11.2008 (last modified; TIM)

!+ Logical-valued function that checks if first character of STR is numeric

implicit none

!=== Argument declarations:

logical :: answer

character(len=*), intent(IN) :: str

!== Local declarations:

! NONE

!=== Executable statements:

answer = .FALSE.

! DON'T allow for `.003` notation (force `0.003` instead):
   !if((str(1:1) >= '0' .and. str(1:1) <= '9') .or.  &
   !     (str(1:1) == '.' .or. str(1:1) == '-')) then
   !   answer = 1
   !endif
if((str(1:1) >= '0' .and. str(1:1) <= '9') .or. str(1:1) == '-') then
   answer = .TRUE.
endif

end function is_1st_a_number


!```````````````````````````````````````````````````````````````````
function is_1st_a_quote(str) result(answer)

!~ 17.11.2008 (last modified; TIM)

!+ Checks if first character of STR is numeric; returns 0 = no, 1 = yes

implicit none

!=== Argument declarations:

logical :: answer

character(len=*), intent(IN) :: str

!== Local declarations:

! NONE

!=== Executable statements:

answer = .FALSE.
if(str(1:1) == "'") then
   answer = .TRUE.
endif

end function is_1st_a_quote


!```````````````````````````````````````````````````````````````````
subroutine varca_c0_1d(varname,var,sub,nvals,maxsub,value_raw,lmin,lmax)

!~ 17.11.2008 (last modified; TIM)

!+ Assigns string value to string variable (after a bounds check)

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname
integer, intent(IN) :: sub,maxsub,lmin,lmax,nvals

character(len=*), dimension(maxsub), intent(INOUT) :: value_raw, var

!== Local declarations:

integer :: nv

!=== Executable statements:

if(sub == 0) then    ! Then there is a full array of values

   ! Check for over- and under-determination of variable:

   if(nvals < maxsub) then
      write (*,'(/3a,/,a,i4,a,i4,a)') 'ERROR: Namelist variable '  &
           ,varname(1:len_trim(varname)),' -- is under-determined.'  &
           ,'    (currently',nvals,' values, but needs',maxsub,' values)'
      there_is_an_error = .TRUE.
   elseif(nvals > maxsub) then
      if(nlst_debug) then
         write (*,'(/3a,/,a,i4,a,i4,a)') 'WARNING: Namelist variable '  &
              ,varname(1:len_trim(varname))  &
              ,' -- is over-determined; extra values ignored.'  &
              ,'  (currently',nvals,' values, but only needs',maxsub,' values)'
         there_is_an_error = .TRUE.
      endif
   endif

   do nv=1,nvals   ! Loop over all array values
      if(nv <= maxsub) then  ! Check to make sure array subscript within bounds
         call varca_c(varname,var(nv),value_raw(nv),lmin,lmax)
      endif
   enddo

else    ! Only one array element given

   if(sub > maxsub) then   ! Make sure within upper array bound
      write (*,'(/3a,/,a,i4,a,i4,a)') 'WARNING: Namelist variable '  &
           ,varname(1:len_trim(varname)),' -- is over-determined.'  &
           ,'  (current idx is ',sub,', but array size only',maxsub,')'
      there_is_an_error = .TRUE.
      return
   endif

   call varca_c(varname,var(sub),value_raw(1),lmin,lmax)

endif

end subroutine varca_c0_1d


!```````````````````````````````````````````````````````````````````
subroutine varca_c0_0d(varname,var,sub,nvals,maxsub,value_raw,lmin,lmax)

!~ 17.11.2008 (last modified; TIM)

!+ Assigns string value to string variable (after a bounds check)

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname
integer, intent(IN) :: sub,maxsub,lmin,lmax,nvals

character(len=*), dimension(maxsub), intent(INOUT) :: value_raw
character(len=*), intent(INOUT) :: var

!== Local declarations:

! NONE

!=== Executable statements:

call varca_c(varname,var,value_raw(1),lmin,lmax)

end subroutine varca_c0_0d


!```````````````````````````````````````````````````````````````````
subroutine varca_c(varname,var,value_raw,lmin,lmax)

!~ 09.12.2008 (last modified; TIM)

!+ Assigns string value to string variable (after a bounds check)
!     *** Can work in concert with `varca_c0`

implicit none

!=== Argument declarations:

integer, intent(IN) :: lmin,lmax
character(len=*), intent(IN) :: varname

character(len=*), intent(INOUT) :: var, value_raw

!== Local declarations:

integer :: n, nw, nl, nd

!=== Executable statements:

nl = len_trim(value_raw)
nd = index(value_raw(1:nl),"'")
nd = index(value_raw(nd+1:nl),"'")

if(nd == 0) then
   write (*,'(/2a,/,a)') 'ERROR: Namelist variable '  &
        ,varname(1:len_trim(varname))  &
        ,' -- must have CHARACTER/STRING value(s) [e.g., ''d'', ''green-24'']'
   there_is_an_error = .TRUE.
   return
endif

! Determine length `nw` of character value string (and strip off quotes):

n = 2
nw = 0
do while(value_raw(n:n) == ' ')  !
   n = n+1                       ! Check for leading whitespace
enddo                            !

do while(value_raw(n:n) /= "'")
   nw = nw+1
   value_raw(nw:nw) = value_raw(n:n)
   n = n+1
enddo

do while(value_raw(nw:nw) == ' ')  !
   nw = nw-1                       ! Check for trailing whitespace
enddo                              !

var = value_raw(1:nw)

if(nw < lmin .or. nw > lmax) then  ! If too small or big, fatal error
   write (*,'(/3a,i4,a,/,a,i4,a,i4,a)') 'ERROR: Namelist variable '  &
        ,varname(1:len_trim(varname))  &
        ,' -- has an improper length of',nw,' characters.'  &
        ,'       Allowable length: ',lmin,' to',lmax,' characters'
   there_is_an_error = .TRUE.
endif

end subroutine varca_c


!```````````````````````````````````````````````````````````````````
subroutine varca_i0_1d(varname,var,sub,nvals,maxsub,value_raw,vmin,vmax)

!+ Assigns string value to integer variable (after a bounds check)

!~ 22.07.2010 (last modified; TIM)

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname
integer, intent(IN) :: sub, maxsub, vmin, vmax, nvals
character(len=*), dimension(maxsub), intent(IN) :: value_raw

integer, dimension(maxsub), intent(OUT)  :: var

!== Local declarations:

integer :: nv

!=== Executable statements:

if(sub == 0) then    ! Then there is a full array of values
   
   ! Check for over- and under-determination of variable:
   
   if(nvals < maxsub) then
      write (*,'(/3a,/,a,i4,a,i4,a)') 'ERROR: Namelist variable '  &
           ,varname(1:len_trim(varname)),' -- is under-determined.'  &
           ,'    (currently',nvals,' values, but needs',maxsub,' values)'
      there_is_an_error = .TRUE.
   elseif(nvals > maxsub) then
      if(nlst_debug) then
         write (*,'(/3a,/,a,i4,a,i4,a)') 'WARNING: Namelist variable '  &
              ,varname(1:len_trim(varname))  &
              ,' -- is over-determined; extra values ignored.'  &
              ,'  (currently',nvals,' values, but only needs',maxsub,' values)'
         there_is_an_error = .TRUE.
      endif
   endif

   do nv=1,nvals   ! Loop over all array values
      if(nv <= maxsub) then  ! Check to make sure array subscript within bounds
         call varca_i(varname,var(nv),value_raw(nv),vmin,vmax)
      endif
   enddo

else    ! Only one array element given

   if(sub > maxsub) then   ! Make sure within upper array bound
      write (*,'(/3a,/,a,i4,a,i4,a)') 'WARNING: Namelist variable '  &
           ,varname(1:len_trim(varname)),' -- is over-determined.'  &
           ,'  (current idx is ',sub,', but array size only',maxsub,')'
      there_is_an_error = .TRUE.
      return
   endif

   call varca_i(varname,var(sub),value_raw(1),vmin,vmax)
   
endif

end subroutine varca_i0_1d


!```````````````````````````````````````````````````````````````````
subroutine varca_i0_0d(varname,var,sub,nvals,maxsub,value_raw,vmin,vmax)

!+ Assigns string value to integer variable (after a bounds check)

!~ 22.07.2010 (last modified; TIM)

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname
integer, intent(IN) :: sub, maxsub, vmin, vmax, nvals
character(len=*), dimension(maxsub), intent(IN) :: value_raw

integer, intent(OUT)  :: var

!== Local declarations:

! NONE

!=== Executable statements:

call varca_i(varname,var,value_raw(1),vmin,vmax)

end subroutine varca_i0_0d


!```````````````````````````````````````````````````````````````````
subroutine varca_i(varname,var,value_raw,vmin,vmax)

!~ 17.11.2008 (last modified; TIM)

!+ Assigns string value to integer variable (after conversion, bounds check)
!     *** Can work in concert with `varca_i0`

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname,value_raw
integer, intent(IN) :: vmin,vmax

integer, intent(OUT) :: var

!== Local declarations:

integer :: tmpvar, nl, nd
character(len=5) :: ic_format

!=== Executable statements:

nl = len_trim(value_raw)
nd = index(value_raw(1:nl),'.')

if(nd /= 0) then
   write (*,'(/2a,/,a)') 'ERROR: Namelist variable '  &
        ,varname(1:len_trim(varname))  &
        ,' -- must have INTEGER value(s) [e.g., 0, -43, 222]'
   there_is_an_error = .TRUE.
   return
endif

write (ic_format,'(a2,i2,a1)') '(i',nl,')'
read (value_raw,ic_format) tmpvar

var = tmpvar

if(var < vmin .or. var > vmax) then  ! If too small or big, fatal error
   write (*,'(/3a,i10,/,a,i10,a,i10)') 'ERROR: Namelist variable '  &
        ,varname(1:len_trim(varname))  &
        ,' -- has an improper value of',var  &
        ,'       Allowable range: ',vmin,' to',vmax
   there_is_an_error = .TRUE.
endif

end subroutine varca_i


!```````````````````````````````````````````````````````````````````
subroutine varca_f0_1d(varname,var,sub,nvals,maxsub,value_raw,vmin,vmax)

!+ Assigns string value to real variable (after a bounds check)

!~ 22.07.2010 (last modified; TIM)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname
integer, intent(IN) :: sub, maxsub, nvals
character(len=*), dimension(maxsub), intent(IN) :: value_raw
real(f4), intent(IN) :: vmin, vmax

real(f4), dimension(maxsub), intent(OUT) :: var

!== Local declarations:

integer :: nv

!=== Executable statements:

if(sub == 0) then    ! Then there is a full array of values
   
   ! Check for over- and under-determination of variable:
   
   if(nvals < maxsub) then
      write (*,'(/3a,/,a,i4,a,i4,a)') 'ERROR: Namelist variable '  &
           ,varname(1:len_trim(varname)),' -- is under-determined.'  &
           ,'    (currently',nvals,' values, but needs',maxsub,' values)'
      there_is_an_error = .TRUE.
   elseif(nvals > maxsub) then
      if(nlst_debug) then
         write (*,'(/3a,/,a,i4,a,i4,a)') 'WARNING: Namelist variable '  &
              ,varname(1:len_trim(varname))  &
              ,' -- is over-determined; extra values ignored.'  &
              ,'  (currently',nvals,' values, but only needs',maxsub,' values)'
         there_is_an_error = .TRUE.
      endif
   endif

   do nv=1,nvals   ! Loop over all array values
      if(nv <= maxsub) then  ! Check to make sure array subscript within bounds
         call varca_f(varname,var(nv),value_raw(nv),vmin,vmax)
      endif
   enddo

else    ! Only one array element given

   if(sub > maxsub) then   ! Make sure within upper array bound
      write (*,'(/3a,/,a,i4,a,i4,a)') 'WARNING: Namelist variable '  &
           ,varname(1:len_trim(varname)),' -- is over-determined.'  &
           ,'  (current idx is ',sub,', but array size only',maxsub,')'
      there_is_an_error = .TRUE.
      return
   endif

   call varca_f(varname,var(sub),value_raw(1),vmin,vmax)
   
endif

end subroutine varca_f0_1d


!```````````````````````````````````````````````````````````````````
subroutine varca_f0_0d(varname,var,sub,nvals,maxsub,value_raw,vmin,vmax)

!+ Assigns string value to real variable (after a bounds check)

!~ 22.07.2010 (last modified; TIM)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname
integer, intent(IN) :: sub, maxsub, nvals
character(len=*), dimension(maxsub), intent(IN) :: value_raw
real(f4), intent(IN) :: vmin, vmax

real(f4), intent(OUT) :: var

!== Local declarations:

! NONE

!=== Executable statements:

call varca_f(varname,var,value_raw(1),vmin,vmax)
   
end subroutine varca_f0_0d


!```````````````````````````````````````````````````````````````````
subroutine varca_f(varname,var,value_raw,vmin,vmax)

!~ 17.11.2008 (last modified; TIM)

!+ Assigns string value to real variable (after conversion, bounds check)
!     *** Can work in concert with `varca_f0`

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: varname,value_raw
real(f4), intent(IN) :: vmin,vmax

real(f4), intent(OUT) :: var

!== Local declarations:

integer :: nd,nl
character(len=8) :: fc_format
real(f4) :: tmpvar

!=== Executable statements:

nl = len_trim(value_raw)
nd = index(value_raw(1:nl),'.')

if(nd == 0) then
   write (*,'(/2a,/,a)') 'ERROR: Namelist variable '  &
        ,varname(1:len_trim(varname))  &
        ,' -- must have FLOATING POINT value(s) [e.g., 0., 3.14159, 0.326]'
   there_is_an_error = .TRUE.
   return
endif

nd = nl-nd  ! Number of decimal places

write (fc_format,'(a2,i2,a1,i2,a1)') '(f',nl,'.',nd,')'
read (value_raw,fc_format) tmpvar

var = tmpvar

if(var < vmin .or. var > vmax) then  ! If too small or big, fatal error
   write (*,'(/3a,f15.5,/,a,f15.5,a,f15.5)') 'ERROR: Namelist variable '  &
        ,varname(1:len_trim(varname))  &
        ,' -- has an improper value of',var  &
        ,'       Allowable range: ',vmin,' to',vmax
   there_is_an_error = .TRUE.
endif

end subroutine varca_f


!```````````````````````````````````````````````````````````````````
subroutine vchk(vn,group,vlist,vcnt,nvar,var_is_not_known)

!~ 17.11.2008 (last modified; TIM)

!+ Checks if each namelist variable is known (if yes, notes that it is there);
!    checks if all group vars are present (vn = 'last_check!')

implicit none

!=== Argument declarations:

integer, intent(IN) :: nvar
character(len=*), intent(IN) :: vn,group
character(len=*), dimension(nvar), intent(IN) :: vlist

integer, dimension(nvar), intent(INOUT) :: vcnt

logical, intent(OUT) :: var_is_not_known

!== Local declarations:

integer :: n

!=== Executable statements:

if(vn /= 'last_check!') then   ! Recognize variable?

   do n=1,nvar
      if(vn(1:len_trim(vn)) == vlist(n)(1:len_trim(vlist(n)))) then
         vcnt(n) = vcnt(n)+1
         var_is_not_known = .FALSE.
         return    ! Return if recognized
      endif
   enddo

   var_is_not_known = .TRUE.
   write (*,'(/,4a)') 'ERROR: In namelist group ',group(1:len_trim(group))  &
        ,', unknown variable ',vn(1:len_trim(vn))

else   ! Last check for any missing namelist variables in group

   do n=1,nvar
      if(vcnt(n) == 0) then    ! If variable hasn't been seen
         write (*,'(/,4a)') 'ERROR: In namelist group '  &
              ,group(1:len_trim(group)),', missing variable '  &
              ,vlist(n)(1:len_trim(vlist(n)))
         there_is_an_error = .TRUE.  ! Set this to stop the run
      endif
   enddo

endif

end subroutine vchk


end module alt_namelist_mod
