!```````````````````````````````````````````````````````````````````
subroutine botset(m1, m2, m3, ia, iz, ja, jz, igd, aa, vnam)

!+ Sets lower boundary condition for "non-atm_mass" variables

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, igd
character(len=*), intent(IN) :: vnam

real(f4), dimension(m1,m2,m3), intent(INOUT) :: aa

!== Local declarations:

integer :: i, j

!=== Executable statements:

do i=1,m2
   do j=1,m3
      aa(1,i,j) = aa(2,i,j)
   end do
end do

end subroutine botset


!```````````````````````````````````````````````````````````````````
subroutine botset_atm_mass(m1, m2, m3, ia, iz, ja, jz, igd, aa, aa0, vnam)

!+ Sets lower boundary condition for "atm_mass" variables

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, igd
real(f4), dimension(m1,m2,m3), intent(IN) :: aa0
character(len=*), intent(IN) :: vnam

real(f4), dimension(m1,m2,m3), intent(INOUT) :: aa

!== Local declarations:

integer :: i, j
real(f4) :: dzmr, tmp1, tmp2, tmp3

!=== Executable statements:

if (vnam == 'P') then        ! Variable is PI
   dzmr = gd(igd)%dzm(2)/gd(igd)%dzm(1)
   do i=1,m2
      do j=1,m3
         tmp2 = aa(2,i,j)+aa0(2,i,j)  ! Compute full PI at overlying levels
         tmp3 = aa(3,i,j)+aa0(3,i,j)  !
         
         tmp1 = tmp2+(tmp2-tmp3)*dzmr   ! Linearly extrapolate full PI (in z)
         aa(1,i,j) = tmp1-aa0(1,i,j)      ! Obtain PI' from full PI
      end do
   end do
end if

end subroutine botset_atm_mass


!```````````````````````````````````````````````````````````````````
subroutine topset(m1, m2, m3, ia, iz, ja, jz, igd, aa, vnam)

!+ Sets upper boundary condition for "non-atm_mass" variables

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, igd
character(len=*), intent(IN) :: vnam

real(f4), dimension(m1,m2,m3), intent(INOUT) :: aa

!== Local declarations:

integer :: i, j
real(f4) :: dzmr

!=== Executable statements:

! Computation of all "non-mass" prognostic variables (other than W) at
!  level M1 by extrapolation from below:

if (vnam == 'U' .or. vnam == 'V') then    ! Velocity components (but not W)
   dzmr = gd(igd)%dzm(m1-2)/gd(igd)%dzm(m1-1)
   do j=1,m3
      do i=1,m2
         aa(m1,i,j) = aa(m1-1,i,j)+dzmr*(aa(m1-1,i,j)-aa(m1-2,i,j))
      end do
   end do

else if (vnam == 'S') then      ! Non-aerosol, non-theta scalars
   dzmr = gd(igd)%dzm(m1-2)/gd(igd)%dzm(m1-1)
   do j=1,m3
      do i=1,m2
         aa(m1,i,j) = max(ZERO_f4,  &
              aa(m1-1,i,j)+dzmr*(aa(m1-1,i,j)-aa(m1-2,i,j)))
      end do
   end do

else if (vnam == 'T') then       ! Theta (scalar)

   dzmr = gd(igd)%dzm(m1-2)/gd(igd)%dzm(m1-1)
   do j=1,m3
      do i=1,m2
         aa(m1,i,j) = aa(m1-1,i,j)+dzmr*(aa(m1-1,i,j)-aa(m1-2,i,j))
      end do
   end do

else if (vnam == 'A') then     ! Aerosol scalars
!%   do j=1,m3
!%      do i=1,m2
!@         aa(m1,i,j) = aa(m1-1,i,j)    ! Zero gradient
!@         aa(m1,i,j) = 0.       ! constant (in time) value (zero)
!@         aa(m1,i,j) = max(0.,  &      ! Simple linear extrapolation; bounded
!@              aa(m1-1,i,j)+dzmr*(aa(m1-1,i,j)-aa(m1-2,i,j)))
!%      enddo
!%   enddo
end if

end subroutine topset


!```````````````````````````````````````````````````````````````````
subroutine topset_atm_mass(m1, m2, m3, ia, iz, ja, jz, igd, aa, aa0, thp,  &
     rtgt, vnam)

!+ Sets upper boundary condition for "atm_mass" variables

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: gd
use body_specific_constants_mod, only: cpi, cp

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, igd
real(f4), dimension(m1,m2,m3), intent(IN) :: aa0, thp
real(f4), dimension(m2,m3), intent(IN) :: rtgt
character(len=*), intent(IN) :: vnam

real(f4), dimension(m1,m2,m3), intent(INOUT) :: aa

!== Local declarations:

integer :: i, j
real(f4) :: dzmr, tmp1, tmp2, tmp3

!=== Executable statements:

if (vnam == 'P') then         ! Variable is PI

   dzmr = gd(igd)%dzm(m1-2)/gd(igd)%dzm(m1-1)
   do j=1,m3
      do i=1,m2
         tmp1 = (aa(m1-1,i,j)+aa0(m1-1,i,j))*cpi*thp(m1-1,i,j) ! T(m1-1)
         tmp2 = (aa(m1-2,i,j)+aa0(m1-2,i,j))*cpi*thp(m1-2,i,j) ! T(m1-2)
         tmp3 = (tmp1-tmp2)*rtgt(i,j)*gd(igd)%dzm(m1-2)  ! Calc dT/dz(m1-2)

         tmp1 = thp(m1-1,i,j)+dzmr*  &  ! Linearly extrap. to get theta(m1)
              (thp(m1-1,i,j)-thp(m1-2,i,j))

         ! Assume that dT/dz(m1-2)=dT/dz(m1-1) and that extrapolated theta(m1)
         !  is correct value... then diagnose full_PI(m1), from which PI'(m1)
         !  is later obtained:  
         tmp2 = (tmp3*cp*rtgt(i,j)/gd(igd)%dzm(m1-1)+  &  ! PI(m1)
              (aa(m1-1,i,j)+aa0(m1-1,i,j))*thp(m1-1,i,j))/tmp1
         aa(m1,i,j) = tmp2-aa0(m1,i,j)             ! Get PI'
      end do
   end do

end if

end subroutine topset_atm_mass


!```````````````````````````````````````````````````````````````````
subroutine rayft

!+ Rayleigh friction (for theta; upper BC)

use main_memory_mod, only: a, nfpt, distim, mxp, myp, mzp, ia, iz, ja, jz, ibndc
use vtables_mod, only: ivt3da, itheta, irv, ith0, itht, irtgt, itopt

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: mxyzp1

!=== Executable statements:

if (nfpt <= 0 .or. distim < tiny(distim)) return    ! Return if not desired

mxyzp1 = mxp*myp*mzp-1

! First load past virtual theta into temporary array:

!%if(H2Ov_tog >= 1) then
!%   do ii=0,mxyzp1
!%      a(ivt3da+ii) = a(itheta+ii)*(1.+0.61*a(irv+ii))
!%   enddo
!%else
   a(ivt3da:ivt3da+mxyzp1) = a(itheta:itheta+mxyzp1)
!%endif

! Now get Rayleigh THETA friction tendency:

call rayf(4, mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ivt3da), a(ith0),  &
     a(itht), a(irtgt), a(itopt))

end subroutine rayft


!```````````````````````````````````````````````````````````````````
subroutine rayf(type, m1, m2, m3, ia, iz, ja, jz, ibcon, var, th0, tht,  &
     rtgx, topx)

!+ Computes Rayleigh friction terms (upper BC) for velocity, theta

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use univ_interpolation_mod, only: intrp_1D_arr_type01a
use main_memory_mod, only: ztop, dts, vctr2, c_gd, vctr5, jdim, nfpt,  &
     distim, icorflg, gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: type, m1, m2, m3, ia, iz, ja, jz, ibcon
real(f4), dimension(m1,m2,m3), intent(IN) :: th0
real(f4), dimension(m2,m3), intent(IN) :: rtgx, topx

real(f4), dimension(m1,m2,m3), intent(INOUT) :: var, tht

!== Local declarations:

integer :: kf, i, j, k
real(f4) :: zmkf, c1, c2

!=== Executable statements:

! This routine calculates Rayleigh friction terms for velocity and theta_il:

if (nfpt == 0 .or. distim < tiny(distim)) return

kf = gd(1)%nz-abs(nfpt)
zmkf = gd(1)%zm(kf)
c1 = ONE_f4/(distim*(ztop-zmkf))
c2 = dts*c1

select case(type)

case(1)

   ! u friction:

   if (nfpt < 0) return
   
   do j = ja,jz
      do i = ia,iz
         do k = 1,c_gd%nzp
            vctr2(k) = c_gd%zt(k) * rtgx(i,j) + topx(i,j)
         end do
         vctr5(1:c_gd%nzp) = intrp_1D_arr_type01a(c_gd%u01d(1:c_gd%nzp), c_gd%zt(1:c_gd%nzp),  &
                                             vctr2(1:c_gd%nzp))
!         call htint(nzp,u01dn(1,grid),zt,nzp,vctr5,vctr2)

         do k = c_gd%nz,2,-1
            if (vctr2(k) <= zmkf) exit
            var(k,i,j) = var(k,i,j) + c2 * (vctr2(k) - zmkf)  &
                 * (vctr5(k) - var(k,i,j))
         end do
      end do
   end do

case(2)

   ! V friction:

   if (nfpt < 0 .or. (jdim == 0 .and. icorflg == 0)) return
   do j = ja,jz
      do i = ia,iz
         do k = 1,c_gd%nzp
            vctr2(k) = c_gd%zt(k) * rtgx(i,j) + topx(i,j)
         end do
         vctr5(1:c_gd%nzp) = intrp_1D_arr_type01a(c_gd%v01d(1:c_gd%nzp),  &
              c_gd%zt(1:c_gd%nzp), vctr2(1:c_gd%nzp))
!         call htint(nzp,v01dn(1,grid),zt,nzp,vctr5,vctr2)
         do k = c_gd%nz,2,-1
            if (vctr2(k) <= zmkf) exit
            var(k,i,j) = var(k,i,j) + c2 * (vctr2(k) - zmkf)  &
                 * (vctr5(k) - var(k,i,j))
         end do
      end do
   end do

case(3)

   ! W friction:

   do j = ja,jz
      do i = ia,iz
         do k = c_gd%nz,2,-1
            vctr2(k) = c_gd%zt(k) * rtgx(i,j) + topx(i,j)
            if (vctr2(k) <= zmkf) exit
            var(k,i,j) = var(k,i,j) - c2 * (vctr2(k) - zmkf) * var(k,i,j)
         end do
      end do
   end do

case(4)

   ! THETA FRICTION:

   if (nfpt < 0) return

   do j = ja,jz
      do i = ia,iz
         do k = c_gd%nz,2,-1
            vctr2(k) = c_gd%zt(k) * rtgx(i,j) + topx(i,j)
            if (vctr2(k) <= zmkf) exit
            tht(k,i,j) = tht(k,i,j) + c1 * (vctr2(k) - zmkf)  &
                 * (th0(k,i,j) - var(k,i,j))
         end do
      end do
   end do

end select

end subroutine rayf


!```````````````````````````````````````````````````````````````````
subroutine bound_scalars

!+ Imposes boundary conditions on scalars

use main_memory_mod, only: a, nscl, naerscl, c_gd, i_grid, run_is_parallel,  &
     mzp, mxp, myp, ia, iz, ja, jz, ibndc
use vtables_mod, only: iscp, iup, ivp, idxu, idxm, idyv, idym, itkep,  &
     iaerscp, isclrp, THP_vidx
use grid_mem_mod, only: switch_to_Ascalar, switch_to_Nscalar
use turbulent_K_coeffs_mod, only: tkeinit

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: n

!=== Executable statements:

! Apply lateral, top, and bottom boundary conditions:

do n=1,nscl    ! "Normal" scalars

   call switch_to_Nscalar(i_grid, n)
   if (c_gd%nxtnest == 0) then
      if (i_grid == 1 .and. .NOT.run_is_parallel) then
         call cyclic(c_gd%nzp, c_gd%nxp, c_gd%nyp, a(iscp), 'T')
      endif
      call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iscp), a(iup),  &
           a(ivp), 'TR', a(idxu), a(idxm), a(idyv), a(idym))
   end if
   if (c_gd%nsttop == 1) then
      if (isclrp(n,i_grid) == THP_vidx) then   ! for THP
         call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iscp), 'T')
      else
         call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iscp), 'S')
      end if
   end if
   if (c_gd%nstbot == 1) then 
      call botset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iscp), 'T')
   end if

end do

do n=1,naerscl    ! "Aerosol" scalars

   call switch_to_Ascalar(i_grid, n)
   if (c_gd%nxtnest == 0) then
      if (i_grid == 1 .and. .NOT.run_is_parallel) then
         call cyclic(c_gd%nzp, c_gd%nxp, c_gd%nyp, a(iaerscp), 'T')
      end if
      call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iaerscp), a(iup),  &
           a(ivp), 'TR', a(idxu), a(idxm), a(idyv), a(idym))
   end if
   if (c_gd%nsttop == 1) then
      call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iaerscp), 'A')
   end if
   if (c_gd%nstbot == 1) then 
      call botset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iaerscp), 'A')
   end if

end do

! Make sure all positive definite quantities remain such:

if (c_gd%idiffk == 1 .or. c_gd%idiffk == 4 .or. c_gd%idiffk == 5 .or.  &
     c_gd%idiffk == 7 .or. c_gd%idiffk == 9) then
   call tkeinit(mzp, mxp, myp, a(itkep))
end if

call negadj(mzp, mxp, myp, i_grid, 'ALL')

end subroutine bound_scalars


!```````````````````````````````````````````````````````````````````
subroutine cyclic(n1, n2, n3, var, vpnt)

!+ Cyclic boundary conditions

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: jdim, ibnd, jbnd

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3
character(len=*), intent(IN) :: vpnt

real(f4), dimension(n1,n2,n3), intent(INOUT) :: var

!== Local declarations:

integer :: ncycle, nshift, i, j, k

!=== Executable statements:

! This routine inputs a 3d variable VAR dimensioned by N1,N2,N3
!  and sets the boundaries to cyclic symmetry.  This version is
!  set up for second order advection only.

if (ibnd == 4) then
   ncycle = n2-3
   nshift = 0
   if (vpnt == 'U') nshift = 1
   do j = 1,n3
      do k = 1,n1
         var(k,1,j) = var(k,ncycle+1,j)
         var(k,n2-nshift,j) = var(k,n2-nshift-ncycle,j)
      end do
   end do

   if (vpnt /= 'U') then
      do j = 1,n3
         do k = 1,n1
            var(k,2,j) = 0.5_f4 * (var(k,2,j) + var(k,n2-1,j))
            var(k,n2-1,j) = var(k,2,j)
         end do
      end do
   end if
end if

if (jbnd == 4 .and. jdim == 1) then
   ncycle = n3 - 3
   nshift = 0
   if (vpnt == 'V') nshift = 1
   do i = 1,n2
      do k = 1,n1
         var(k,i,1) = var(k,i,ncycle+1)
         var(k,i,n3-nshift) = var(k,i,n3-nshift-ncycle)
      end do
   end do

   if (vpnt /= 'V') then
      do i = 1,n2
         do k = 1,n1
            var(k,i,2) = 0.5_f4 * (var(k,i,2) + var(k,i,n3-1))
            var(k,i,n3-1) = var(k,i,2)
         end do
      end do
   end if

end if

end subroutine cyclic


!```````````````````````````````````````````````````````````````````
subroutine latset(m1, m2, m3, ia, iz, ja, jz, ibcon, ap, uc, vc, vnam, dxu,  &
     dxm, dyv, dym)

!+ Set lateral boundary conditions

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: dtlv, vctr17, vctr18, jdim, ibnd, jbnd,  &
     lsflg, c_gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibcon
real(f4), dimension(m1,m2,m3), intent(IN) :: uc, vc
real(f4), dimension(m2,m3), intent(IN) :: dxu, dxm, dyv, dym
character(len=*), intent(IN) :: vnam

real(f4), dimension(m1,m2,m3), intent(INOUT) :: ap

!== Local declarations:

real(f4) :: thresh, dtlx, c0, c1, dxr, dyr
integer :: i, j, k, lbw, lbe, lbs, lbn

!=== Executable statements:

if (iand(ibcon, 1) > 0) lbw = ia - 1
if (iand(ibcon, 2) > 0) lbe = iz + 1
if (iand(ibcon, 4) > 0) lbs = ja - 1
if (iand(ibcon, 8) > 0) lbn = jz + 1

thresh = 0.
if (vnam == 'U' .or. vnam == 'V' .or. vnam == 'W' .or. vnam == 'P') then
   dtlx = dtlv
else
   dtlx = c_gd%dt_dyn
end if

if (ibnd /= 4 .and. vnam /= 'U' .and. lsflg /= 3) then

   ! Western and Eastern boundaries for zero gradient option

   if (lsflg == 0) then
      if (iand(ibcon, 1) > 0) then
         do j=1,m3
            do k=1,m1
               ap(k,lbw,j) = ap(k,ia,j)
            end do
         end do
      end if
      if (iand(ibcon, 2) > 0) then
         do j=1,m3
            do k=1,m1
               ap(k,lbe,j) = ap(k,iz,j)
            end do
         end do
      end if
   else

      ! Western boundary for lsflg = 1 or 2

      if (iand(ibcon, 1) > 0) then
         do j=1,m3
            if (vnam == 'V') then
               dxr = dxm(ia,j)/dxm(lbw,j)
               c1 = dtlx*dxm(lbw,j)
               if (j < m3) then
                  c1 = c1*0.5_f4
                  do k=1,m1
                     vctr17(k) = -c1*(uc(k,lbw,j)+uc(k,lbw,j+jdim))
                  end do
               else
                  do k=1,m1
                     vctr17(k) = -c1*uc(k,lbw,j)
                  end do
               end if
            else if (vnam == 'W') then
               dxr = dxu(ia,j)/dxu(lbw,j)
               c0 = dtlx*dxu(lbw,j)
               c1 = c0*0.5_f4
               do k=1,m1-1
                  vctr17(k) = -c1*(uc(k,lbw,j)+uc(k+1,lbw,j))
               end do
               k = m1
               vctr17(k) = -c0*uc(k,lbw,j)
            else
               dxr = dxu(ia,j)/dxu(lbw,j)
               c1 = dtlx*dxu(lbw,j)
               do k=1,m1
                  vctr17(k) = -c1*uc(k,lbw,j)
               end do
            end if
            do k=1,m1
               vctr18(k) = ap(k,ia,j)+dxr*(ap(k,ia,j)-ap(k,ia+1,j))
            end do
            do k=1,m1
               if (vctr17(k) >= thresh) then
                  ap(k,lbw,j) = vctr18(k)
               else if (lsflg == 1) then
                  ap(k,lbw,j) = ap(k,ia,j)
               end if
            end do
         end do
      end if

      ! Eastern Boundary for LSFLG = 1 or 2

      if (iand(ibcon, 2) > 0) then
         do j=1,m3
            if (vnam == 'V') then
               dxr = dxm(iz-1,j)/dxm(iz,j)
               c1 = dtlx*dxm(iz,j)
               if (j < m3) then
                  c1 = c1*0.5_f4
                  do k=1,m1
                     vctr17(k) = c1*(uc(k,iz,j)+uc(k,iz,j+jdim))
                  end do
               else
                  do k=1,m1
                     vctr17(k) = c1*uc(k,iz,j)
                  end do
               end if
            else if (vnam == 'W') then
               dxr = dxu(iz-1,j)/dxu(iz,j)
               c0 = dtlx*dxu(iz,j)
               c1 = c0*0.5_f4
               do k=1,m1-1
                  vctr17(k) = c1*(uc(k,iz,j)+uc(k+1,iz,j))
               end do
               k = m1
               vctr17(k) = c0*uc(k,iz,j)
            else
               dxr = dxu(iz-1,j)/dxu(iz,j)
               c1 = dtlx*dxu(iz,j)
               do k=1,m1
                  vctr17(k) = c1*uc(k,iz,j)
               end do
            end if
            do k=1,m1
               vctr18(k) = ap(k,iz,j)+dxr*(ap(k,iz,j)-ap(k,iz-1,j))
            end do
            do k=1,m1
               if (vctr17(k) >= thresh) then
                  ap(k,lbe,j) = vctr18(k)
               else if (lsflg == 1) then
                  ap(k,lbe,j) = ap(k,iz,j)
               end if
            end do
         end do
      end if
   end if
end if

if (jdim == 1 .and. jbnd /= 4 .and. vnam /= 'V' .and. lsflg /= 3) then

   ! Southern and Northern boundaries for zero gradient option

   if (lsflg == 0) then
      if (iand(ibcon, 4) > 0) then
         do i=1,m2
            do k=1,m1
               ap(k,i,lbs) = ap(k,i,ja)
            end do
         end do
      end if
      if (iand(ibcon, 8) > 0) then
         do i=1,m2
            do k=1,m1
               ap(k,i,lbn) = ap(k,i,jz)
            end do
         end do
      end if
   else
      
      ! Southern boundary for LSFLG = 1 or 2

      if (iand(ibcon, 4) > 0) then
         do i=1,m2
            if (vnam == 'U') then
               dyr = dym(i,ja)/dym(i,lbs)
               c1 = dtlx*dym(i,lbs)
               if (i < m2) then
                  c1 = c1*0.5_f4 
                  do k=1,m1
                     vctr17(k) = -c1*(vc(k,i,lbs)+vc(k,i+1,lbs))
                  end do
               else
                  do k=1,m1
                     vctr17(k) = -c1*vc(k,i,lbs)
                  end do
               end if
            else if (vnam == 'W') then
               dyr = dyv(i,ja)/dyv(i,lbs)
               c0 = dtlx*dyv(i,lbs)
               c1 = c0*0.5_f4
               do k=1,m1-1
                  vctr17(k) = -c1*(vc(k,i,lbs)+vc(k+1,i,lbs))
               end do
               k = m1
               vctr17(k) = -c0*vc(k,i,lbs)
            else
               dyr = dyv(i,ja)/dyv(i,lbs)
               c1 = dtlx*dyv(i,lbs)
               do k=1,c_gd%nz
                  vctr17(k) = -c1*vc(k,i,lbs)
               end do
            end if
            do k=1,m1
               vctr18(k) = ap(k,i,ja)+dyr*(ap(k,i,ja)-ap(k,i,ja+1))
            end do
            do k=1,m1
               if (vctr17(k) >= thresh) then
                  ap(k,i,lbs) = vctr18(k)
               else if (lsflg == 1) then
                  ap(k,i,lbs) = ap(k,i,ja)
               end if
            end do
         end do
      end if

      ! Northern Boundary for LSFLG = 1 or 2

      if (iand(ibcon, 8) > 0) then
         do i=1,m2
            if (vnam == 'U') then
               dyr = dym(i,jz-1)/dym(i,jz)
               c1 = dtlx*dym(i,jz)
               if (i < m2) then 
                  c1 = c1*0.5_f4
                  do k=1,m1
                     vctr17(k) = c1*(vc(k,i,jz)+vc(k,i+1,jz))
                  end do
               else
                  do k=1,m1
                     vctr17(k) = c1*vc(k,i,jz)
                  end do
               end if
            elseif(vnam == 'W') then
               dyr = dyv(i,jz-1)/dyv(i,jz)
               c0 = dtlx*dyv(i,jz)
               c1 = c0*0.5_f4
               do k=1,m1-1
                  vctr17(k) = c1*(vc(k,i,jz)+vc(k+1,i,jz))
               end do
               k = m1
               vctr17(k) = c0*vc(k,i,jz)
            else
               dyr = dyv(i,jz-1)/dyv(i,jz)
               c1 = dtlx*dyv(i,jz)
               do k=1,m1
                  vctr17(k) = c1*vc(k,i,jz)
               end do
            end if
            do k=1,m1
               vctr18(k) = ap(k,i,jz)+dyr*(ap(k,i,jz)-ap(k,i,jz-1))
            end do
            do k=1,m1
               if (vctr17(k) >= thresh) then
                  ap(k,i,lbn) = vctr18(k)
               else if (lsflg == 1) then
                  ap(k,i,lbn) = ap(k,i,jz)
               end if
            end do
         end do
      end if
   end if
end if

end subroutine latset


!```````````````````````````````````````````````````````````````````
subroutine latbnd

!+ Driver for radiative lateral boundary conditions (for normal velocity)

use main_memory_mod, only: a, ibnd, jbnd, c_gd, mzp, mxp, myp, ia, iz, ja,  &
     jz, ibndc
use vtables_mod, only: iup, iuc, iut, ivp, ivc, ivt, idxt, idyt

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

! This routine drives the computation of the radiative lateral
!  boundary condition for normal velocity on the coarsest grid
!  and the recomputation of all boundary tendencies on nested grids
!  after the first nested grid timestep.

if (c_gd%nxtnest == 0) then

   ! Radiative and/or mesoscale compensation region lateral boundary conditions

   if (ibnd <= 3 .or. jbnd <= 3) then
      
      call latnormv(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iup), a(iuc),  &
           a(iut), a(ivp), a(ivc), a(ivt), a(idxt), a(idyt))
      
   end if

end if

end subroutine latbnd


!```````````````````````````````````````````````````````````````````
subroutine latnormv(m1, m2, m3, ia, iz, ja, jz, ibcon, up, uc, ut, vp, vc,  &
     vt, dxt, dyt)

!+ Radiative lateral boundary conditions (for normal velocity)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use main_memory_mod, only: dtlv, jdim, cphas

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibcon
real(f4), dimension(m1,m2,m3), intent(IN) :: up, uc, vp, vc
real(f4), dimension(m2,m3), intent(IN) :: dxt, dyt

real(f4), dimension(m1,m2,m3), intent(INOUT) :: ut, vt

!== Local declarations:

integer :: i, j, k
real(f4) :: dxl, dxr, cphx1, cphx2, cphy1, cphy2

!=== Executable statements:

!     This routine ultimately updates tendencies at lateral boundaries
!     after first diagnosing appropriate phase speeds.
!
!     IBND and JBND are flags for the radiative type in the X and Y
!     direction. Their meaning is:
!
!        IBND=1......Klemp-Wilhelmson (1978) type; phase speed given
!                    by CPHAS
!        IBND=2......Klemp-Lilly (1980) type; doppler shifted phase
!                    speed constant with height and diagnosed from
!                    average of Orlanski speeds, i.e. function of
!                    only (X,Y)
!        IBND=3......Orlanski(1974) type; Phase speeds diagnosed
!                    from local conditions and function of (x,y,z)

!     Calculate the diagnostic phase
!     speed. The Orlanski(1976) leapfrog method is to use three time
!     levels of information, namely the T-2, T-1, and T level to
!     evaluate the phase speed given by - du/dt / du/dx = u + C.
!     If this is to be an Orlanski or Klemp-Lilly type boundary in the x
!     direction then this following diagnostic procedure is necessary.

!     If this is the first call to a routine, initialize the phase
!       speed arrays if necessary.

if (ibcon == 0) return

! First compute "X" boundaries:

if (iand(ibcon, 1) /= 0) then
   do j = 1,m3
      dxl = 1. / (dtlv * dxt(2,j))
      do k = 1,m1

         cphx1 = min(ZERO_f4, max(-dxl, (up(k,1,j)-cphas)))
         ut(k,1,j) = ut(k,1,j) - cphx1 * dxt(2,j)  &
              * (up(k,2,j) + ut(k,2,j) * dtlv - up(k,1,j))

      end do
   end do
end if

if (iand(ibcon, 2) /= 0) then
   do j = 1,m3
      dxr = ONE_f4 / (dtlv * dxt(m2-1,j))
      do k = 1,m1
         
         cphx2 = max(ZERO_f4, min(dxr, (up(k,m2-1,j)+cphas)))
         ut(k,m2-1,j) = ut(k,m2-1,j) - cphx2 * dxt(m2-1,j)  &
              * (up(k,m2-1,j) - (up(k,m2-2,j) + ut(k,m2-2,j) * dtlv))
         
      end do
   end do
end if

! South and north boundaries:

if (jdim == 1) then

   if (iand(ibcon, 4) /= 0) then
      do i = 1,m2
         dxl = ONE_f4 / (dtlv * dyt(i,2))
         do k = 1,m1
            cphy1 = min(ZERO_f4, max(-dxl, (vp(k,i,1)-cphas)))
            vt(k,i,1) = vt(k,i,1) - cphy1 * dyt(i,2)  &
                 * (vp(k,i,2) + vt(k,i,2) * dtlv - vp(k,i,1))
         end do
      end do
   end if

   if (iand(ibcon, 8) /= 0) then
      do i = 1,m2
         dxr = ONE_f4 / (dtlv * dyt(i,m3-1))
         do k = 1,m1
            cphy2 = max(ZERO_f4, min(dxr, (vp(k,i,m3-1)+cphas)))
            vt(k,i,m3-1) = vt(k,i,m3-1) - cphy2 * dyt(i,m3-1)  &
                 * (vp(k,i,m3-1) - (vp(k,i,m3-2) + vt(k,i,m3-2) * dtlv))
         end do
      end do
   end if

end if

end subroutine latnormv


!```````````````````````````````````````````````````````````````````
subroutine bound_velP

!+ Velocity/Exner function boundary conditions

use main_memory_mod, only: a, c_gd, i_grid, ia, iz, ja, jz, ibndc, mzp, mxp, myp
use vtables_mod, only: iup, ivp, idxu, idxm, idyv, idym, iwp, ipp, iuc, ivc,  &
     iwc, ipc, ipi0, ithp, irtgt

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

if (c_gd%nxtnest == 0) then
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iup), a(iup), a(ivp),  &
        'U', a(idxu), a(idxm), a(idyv), a(idym))
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ivp), a(iup), a(ivp),  &
        'V', a(idxu), a(idxm), a(idyv), a(idym))
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iwp), a(iup), a(ivp),  &
        'W', a(idxu), a(idxm), a(idyv), a(idym))
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ipp), a(iup), a(ivp),  &
        'P', a(idxu), a(idxm), a(idyv), a(idym))
end if

if (c_gd%nsttop == 1) then
   call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iup), 'U')
   call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ivp), 'V')
end if
if (c_gd%nstbot == 1) then
   call botset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iup), 'U')
   call botset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ivp), 'V')
end if
!@call topset(mzp,mxp,myp,ia,iz,ja,jz,i_grid,a(ipp),'P')
call topset_atm_mass(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ipp), a(ipi0),  &
     a(ithp), a(irtgt), 'P')
!@call botset(mzp,mxp,myp,ia,iz,ja,jz,i_grid,a(ipp),'P')
call botset_atm_mass(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ipp), a(ipi0), 'P')
call dumset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iwp), 'W')

call dumset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iup), 'U')
call dumset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ivp), 'V')

if (c_gd%nxtnest == 0) then
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iuc), a(iup), a(ivp),  &
        'U', a(idxu), a(idxm), a(idyv), a(idym))
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ivc), a(iup), a(ivp),  &
        'V', a(idxu), a(idxm), a(idyv), a(idym))
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iwc), a(iup), a(ivp),  &
        'W', a(idxu), a(idxm), a(idyv), a(idym))
   call latset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ipc), a(iup), a(ivp),  &
        'P', a(idxu), a(idxm), a(idyv), a(idym))
end if

if (c_gd%nsttop == 1) then
   call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iuc), 'U')
   call topset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ivc), 'V')
end if
if (c_gd%nstbot == 1) then
   call botset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(iuc), 'U')
   call botset(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ivc), 'V')
end if
!@call topset(mzp,mxp,myp,ia,iz,ja,jz,i_grid,a(ipc),'P')
call topset_atm_mass(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ipc), a(ipi0),  &
     a(ithp), a(irtgt), 'P')
!@call botset(mzp,mxp,myp,ia,iz,ja,jz,i_grid,a(ipc),'P')
call botset_atm_mass(mzp, mxp, myp, ia, iz, ja, jz, i_grid, a(ipc), a(ipi0), 'P')
call dumset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iwc), 'W')
call dumset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(iuc), 'U')
call dumset(mzp, mxp, myp, ia, iz, ja, jz, ibndc, a(ivc), 'V')

end subroutine bound_velP


!```````````````````````````````````````````````````````````````````
subroutine dumset(m1, m2, m3, ia, iz, ja, jz, ibcon, aa, vnam)

!+ Set boundary point values to adjacent point values

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: jdim

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibcon
character(len=*), intent(IN) :: vnam

real(f4), dimension(m1,m2,m3), intent(INOUT) :: aa

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

if (vnam == 'U' .and. iand(ibcon, 2) > 0) then
   do j = 1,m3
      do k = 1,m1
         aa(k,m2,j) = aa(k,m2-1,j)
      end do
   end do
else if (vnam == 'V' .and. iand(ibcon, 8) > 0) then
   do i = 1,m2
      do k = 1,m1
         aa(k,i,m3) = aa(k,i,m3-jdim)
      end do
   end do
else if (vnam == 'W') then
   do j = 1,m3
      do i = 1,m2
         aa(m1,i,j) = aa(m1-1,i,j)
      end do
   end do
end if

end subroutine dumset


!```````````````````````````````````````````````````````````````````
subroutine sfc3Dbc(m4, m2, m3, m5, ia, iz, ja, jz, jdim, SBPS_var)

!+ Sets 3D SBPS variable boundary conditions

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m4, m2, m3, m5, ia, iz, ja, jz, jdim

real(f4), dimension(m4,m2,m3,m5), intent(INOUT) :: SBPS_var

!== Local declarations:

integer :: i, j, k, ip

!=== Executable statements:

do ip=1,m5

   do j=1,m3
      do k=1,m4
         SBPS_var(k,1,j,ip) = SBPS_var(k,2,j,ip)
         SBPS_var(k,m2,j,ip) = SBPS_var(k,m2-1,j,ip)
      end do
   end do

   if (jdim == 1) then
      do i=1,m2
         do k=1,m4
            SBPS_var(k,i,1,ip) = SBPS_var(k,i,2,ip)
            SBPS_var(k,i,m3,ip) = SBPS_var(k,i,m3-1,ip)
         end do
      end do
   end if

end do

end subroutine sfc3Dbc


!```````````````````````````````````````````````````````````````````
subroutine sfc2Dbc(m2, m3, m5, ia, iz, ja, jz, jdim, SBPS_var)

!+ Sets 2D SBPS variable boundary conditions

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m2, m3, m5, ia, iz, ja, jz, jdim

real(f4), dimension(m2,m3,m5), intent(INOUT) :: SBPS_var

!== Local declarations:

integer :: i, j, ip

!=== Executable statements:

do ip=1,m5

   do j=1,m3
      SBPS_var(1,j,ip) = SBPS_var(2,j,ip)
      SBPS_var(m2,j,ip) = SBPS_var(m2-1,j,ip)
   end do

   if (jdim == 1) then
      do i=1,m2
         SBPS_var(i,1,ip) = SBPS_var(i,2,ip)
         SBPS_var(i,m3,ip) = SBPS_var(i,m3-1,ip)
      end do
   end if

end do

end subroutine sfc2Dbc
