#!/usr/bin/env python
"""
Reformats a PRAMS-style namelist as a sparse Fortran 95 namelist.

A heavily commented PRAMS-style namelist is relatively easy for a human to read
and edit, but is difficult for a Fortran program to ingest/digest.  To mitigate
this problem, this program tokenizes a PRAMS-style namelist and outputs an
equivalent, but sparse, Fortran 95 namelist file.

This program requires either Python version 2.6+ or 3.2+, and is importable as a
python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import shlex
import re
import os
import getopt
import io

#--------------------------------------------------------------------------
def rfmt_Ps_nml(input_filename, debug):
    """Reformats a PRAMS-style namelist as a sparse Fortran 95 namelist."""

    # Open input file:
    input_file = io.open(input_filename, "rt", encoding="ascii")

    # Run lexer (with custom configuration) on input file contents: 
    lexer = shlex.shlex(input_file)
    lexer.commenters = '!'       # Set character(s) which begin comments
    lexer.wordchars += ".-+&%@" # Allows &red, +/-1, -3.1, "g.dat", "gnew.dat",
                                #        %green, @purple
    lexer.whitespace += ",="      # Treat ',' and '=' as token delimiters, too

    # Store all tokens in a list object:
    token_list = [token for token in lexer]

    # Close input file:
    input_file.close()

    # Parse the list of tokens, reformat into a Fortran namelist, and store the
    #  results (for now) in 'output_str_list':
    output_str_list = []
    after_first_namelist_block = False
    resolve_full_path = False
    array_valued_item = ''
    array_vitem_idx = 0
    for token in token_list:
        if debug:
            print("::::", token)
        token_match_attempt = re.match("^[\'\"]([\s\S]*?)[\'\"]$", token)
        if token_match_attempt:
          # String data:
            if re.match("^\s*$", token_match_attempt.group(1)):
                if debug:
                    print(">>>> this is an empty or whitespace (quoted) string")
                tmp_str = "'"+"__NULL__"+"'"
                if array_valued_item:
                    array_vitem_idx += 1
                    arr_tmp = re.sub("\(.*?\)", '('+str(array_vitem_idx)+')',
                                     array_valued_item)+" = "
                    tmp_str = arr_tmp+tmp_str
                output_str_list.append(' '+tmp_str)
            else:
              # Not empty and not entirely whitespace:
                if debug:
                    print(">>>> this quoted string is not empty")
                tmp_str = re.sub('\n', ' ', token)
                if resolve_full_path:
                    tmp_str = os.path.abspath(tmp_str.strip("'"))
                    tmp_str = "'"+tmp_str+"'"
                if array_valued_item:
                    array_vitem_idx += 1
                    arr_tmp = re.sub("\(.*?\)", '('+str(array_vitem_idx)+')',
                                     array_valued_item)+" = "
                    tmp_str = arr_tmp+tmp_str
                output_str_list.append(' '+tmp_str)
        else:
          # Not string data:
            if re.match(".*?[\W].*?", token) and \
                    not re.match("^[\%\@]\w+$", token):
              # FP, s_int, boolean, nml, junk:
                if re.match("^[\&]\w+$", token) and \
                        re.match("^[\&][\D]", token):
                  # Namelist block name:
                    if debug:
                        print(">>>> is a namelist block name")
                    if after_first_namelist_block:
                        output_str_list.append("/\n")
                    else:
                        after_first_namelist_block = True
                    output_str_list.append(token)
                elif (token.lower() == ".true.") or \
                        (token.lower() == ".false."):
                  # Boolean:
                    if debug:
                        print(">>>> is a boolean")
                    arr_tmp = ''
                    if array_valued_item:
                        array_vitem_idx += 1
                        arr_tmp = re.sub("\(.*?\)",
                                         '('+str(array_vitem_idx)+')',
                                         array_valued_item)+" = "
                    output_str_list.append(' '+arr_tmp+token.lower())
                elif re.match("([-+]\d+)(?![\.\d])", token):
                  # Signed integer:
                    if debug:
                        print(">>>> is a signed integer")
                    arr_tmp = ''
                    if array_valued_item:
                        array_vitem_idx += 1
                        arr_tmp = re.sub("\(.*?\)",
                                         '('+str(array_vitem_idx)+')',
                                         array_valued_item)+" = "
                    output_str_list.append(' '+arr_tmp+token)
                elif re.match("^[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?$",
                              token):
                  # Floating point value:
                    if debug:
                        print(">>>> is a floating point number")
                    arr_tmp = ''
                    if array_valued_item:
                        array_vitem_idx += 1
                        arr_tmp = re.sub("\(.*?\)",
                                         '('+str(array_vitem_idx)+')',
                                         array_valued_item)+" = "
                    output_str_list.append(' '+arr_tmp+token)
                else:
                  # Junk:
                    msg = ''.join(("Unclassifiable input string type #1: ",
                                   token))
                    raise ValueError(msg)
            else:
              # Variable name, unsigned integer, or junk:
                if re.match("^\d+$", token):
                  # Unsigned integer:
                    if debug:
                        print(">>>> is an unsigned integer")
                    arr_tmp = ''
                    if array_valued_item:
                        array_vitem_idx += 1
                        arr_tmp = re.sub("\(.*?\)",
                                         '('+str(array_vitem_idx)+')',
                                         array_valued_item)+" = "
                    output_str_list.append(' '+arr_tmp+token)
                elif re.match("^[\D]", token):
                  # Variable name:
                    if debug:
                        print(">>>> is a variable name")
                    both_match = re.match("^[%]", token)
                    token_tmp = token+" ="
                    if both_match or re.match("^[_]", token):
                      # Need to resolve full path:
                        if not both_match:
                            token_tmp = token[1:]+" ="
                        resolve_full_path = True 
                    elif resolve_full_path:
                      # Turn off full path res flag: 
                        resolve_full_path = False
                    if both_match or re.match("^[@]", token):
                      # An array-valued item:
                        array_valued_item = token[1:]+"( )"
                        array_vitem_idx = 0
                        token_tmp = ''  # Output a "blank" line
                    elif array_valued_item:
                      # Turn off array-valued item flag: 
                        array_valued_item = ''
                    output_str_list.append(' '+token_tmp)
                else:
                  # Junk:
                    msg = ''.join(("Unclassifiable input string type #2: ",
                                   token))
                    raise ValueError(msg)
    output_str_list.append('/')    # Finish last namelist block

    # Concatenate all items in the output string list, separated by newlines: 
    output_str = "\n".join(output_str_list)

    return output_str
#--------------------------------------------------------------------------


#--------------------------------------------------------------------------
def main(input_file, debug, output_file):
    """Driver routine."""

    # Reformat input namelist, return results as a complex string:
    output_str = rfmt_Ps_nml(input_file, debug)

    # Open output file; write the Fortran namelist contents; then implicitly
    #  close the output file:
    with io.open(output_file, "wt", encoding="ascii") as output_f:
        output_f.write(output_str)
#--------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    usage_msg = """
usage: reformat_PRAMS_namelist.py [-h] [-d]  input_file  output_file

Reformats a PRAMS namelist as a sparse Fortran 95 namelist. A heavily commented
 PRAMS-style namelist is relatively easy for a human to read and edit, but is
 difficult for a Fortran program to ingest/digest.  To mitigate this problem,
 this program tokenizes a PRAMS-style namelist and outputs an equivalent, but
 sparse, Fortran 95 namelist file.  This program requires python version 2.7 or
 later, and is importable as a python module.

mandatory argument:
  input_file   File (input) containing a PRAMS-style namelist.
  output_file   File (output) containing a sparse Fortran 95 namelist.

optional arguments:
  -h, --help   Show this help message and exit.
  -d, --debug   Activate debugging statements (default: False)
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd", ["help", "debug"])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    use_debugger = False
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        elif o in ("-d", "--debug"):
            use_debugger = True
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 2 or len(args) < 2):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)
    input_file = args[0]
    output_file = args[1]

    # Run driver:
    main(input_file, debug, output_file)
