#!/usr/bin/env python
"""
Sets up and executes a PRAMS run.

This program requires either Python version 2.6+ or 3.2+, and is importable as a
python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import sys
import os
import re
import errno
import subprocess
import getopt
import string
import random
import multiprocessing
import time
import datetime
import io
import reformat_PRAMS_namelist

#------------------------------------------------------------------------------
def mkdir_p(path):
    """Emulates 'mkdir -p' functionality."""
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
if "check_output" not in dir(subprocess):
 # subprocess.check_output does not exist (Python 2.6), so use a backported
 #  version from Python 2.7 instead, and "add" it to subprocess:
    def check_output(*popenargs, **kwargs):
        """Run command with arguments and return its output."""

        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            error = subprocess.CalledProcessError(retcode, cmd)
            error.output = output
            raise error
        return output

    subprocess.check_output = check_output
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def run_PRAMS_instance(run_root_dir_full, run_PRAMS_with_leafPEs, n_PEs,
                       exe_file_full, generated_nml_fn, nml_file_full,
                       run_etc_dir_full, i_vf_range, generated_nml_str,
                       init_delay):
    """Set up and execute a PRAMS instance (can run as a separate thread)."""

    # Create configured run directory (and its parent dirs), if needed:
    mkdir_p(run_root_dir_full)

    # Change to configured run directory:
    os.chdir(run_root_dir_full)

    # Open new output namelist file; write the Fortran namelist contents; then
    #  implicitly close the output file:
    try:
        text_to_write = unicode(generated_nml_str, "ascii")  # v2.6+
    except:
        text_to_write = generated_nml_str  # v3.2+
    with io.open(generated_nml_fn, "wt", encoding="ascii") as output_f:
        output_f.write(text_to_write)

    # Generate '&PRAMS_args' Fortran namelist block, and append it to the
    #  namelist file produced above; implicitly close output file afterwards:
    if run_PRAMS_with_leafPEs:
        riip = ".TRUE."
    else:
        riip = ".FALSE."
    i_beg, i_end = i_vf_range
    text_to_write = \
            "\n&PRAMS_args\n" \
            " n_PE = "+str(n_PEs)+"\n" \
            " namelist_fn = '"+nml_file_full+"'\n" \
            " vtable_fn = '"+os.path.join(run_etc_dir_full, "VTABLE")+"'\n" \
            " v_common_fn = '"+os.path.join(run_etc_dir_full, "version_info.common")+"'\n" \
            " v_body_fn = '"+os.path.join(run_etc_dir_full, "version_info.body")+"'\n" \
            " run_intent_is_parallel = "+riip+"\n" \
            "/\n\n"
    with io.open(generated_nml_fn, "at", encoding="ascii") as output_f:
        output_f.write(text_to_write)

    # Try to detect specific mpiexec implementation, in order to add helpful
    #  command-line options:
    cmd_line_seq = ["mpiexec", "-V"]
    added_opts = []
    try:
        output_info = subprocess.check_output(cmd_line_seq,
                                              stderr=subprocess.STDOUT,
                                              universal_newlines=True)
    except Exception:
        pass  # Unknown MPI implementation
    else:
        if "OpenRTE" in output_info:
            added_opts = ["--tag-output"]        # OpenMPI
        elif any([x in output_info for x in ["mpich", "MPICH"]]):
            added_opts = ["-prepend-rank"]     # MPICH

    # Construct command-line sequence:
    cmd_line_seq = []
    if run_PRAMS_with_leafPEs:
        cmd_line_seq.append("mpiexec")
        cmd_line_seq.extend(["-n", str(n_PEs)])
        cmd_line_seq.extend(added_opts)
    else:
        if use_debugger:
            cmd_line_seq.append("gdb")
    cmd_line_seq.append(exe_file_full) 

    # Delay for a specified amount of time (possibly to allow time_staggering
    #  of multiple IO-intensive processes):
    time.sleep(init_delay)

    # Try to execute the completed command-line sequence; return the command,
    #  its exit status, and any exception raised:
    try:
        p = subprocess.Popen(cmd_line_seq, universal_newlines=True)
        return ' '.join(cmd_line_seq), p.wait(), None
    except Exception as e:
        return ' '.join(cmd_line_seq), None, str(e)
#--------------------------------------------------------------------------


#--------------------------------------------------------------------------
def run_PRAMS_instance_star(single_compound_arg):
    """Convert given `f([1,2])` or `f((1,2))` to a `f(1,2)` call."""
    return run_PRAMS_instance(*single_compound_arg)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def main(nml_file, run_etc_dir, run_root_dir, exe_file_pfx, n_PEs):
    """Driver routine."""

    # Define some useful parameters:

    nml_file_full = os.path.abspath(nml_file)
    run_etc_dir_full = os.path.abspath(run_etc_dir)
    run_root_dir_full = os.path.abspath(run_root_dir)
    exe_file_pfx_full = os.path.abspath(exe_file_pfx)

    generated_nml_fn_pfx = "auto_gen-PRAMS.nml"

    generated_nml_fn = generated_nml_fn_pfx
    exe_file_full = exe_file_pfx_full

    # Reformat a more human-readable namelist into a more computer-friendly
    #  one; store it in a (complex) string:
    generated_nml_str0 = reformat_PRAMS_namelist.rfmt_Ps_nml(nml_file_full,
                                                             False)

    # Is this a "MAKE_VAR_FILES" run?  This expression searches for
    #  'MAKE_VAR_FILES' (in between 'RUN_TYPE' and 'N_GRIDS'):
    recd1 = re.compile(".*?(RUN_TYPE).*?(MAKE_VAR_FILES).*?(N_GRIDS)",
                       re.DOTALL)
    # What is the value of N_VF?  This expression searches for its value (in
    #  between 'N_VF' and 'NIGRIDS'):
    recd2 = re.compile(".*?(N_VF).*?(\\d+).*?(NIGRIDS)", re.DOTALL)

    # Search for relevant information in the generated namelist string contents:
    i_vf_range = []
    if recd1.search(generated_nml_str0):
      # This *is* a "MAKE_VAR_FILES" run:
        n_exe_PEs = 1                   #
        run_PRAMS_with_leafPEs = False  # In each thread, run exe in serial only
        match2 = recd2.search(generated_nml_str0)
        if match2:
          # Found the value of N_VF:
            n_vf = int(match2.group(2))
            n_py_threads = min(n_PEs, n_vf)  # No superfluous thread use
            n_vf_remainder = n_vf%n_py_threads
            n_vf_ptoffset = min(n_vf_remainder, 1)
            n_vf_per_thread = (n_vf//n_py_threads)+n_vf_ptoffset

            i_beg = 1
            # Possibly assign some items to thread(s):
            for thd in range(n_py_threads-1):
                i_end = i_beg+n_vf_per_thread-1
                i_vf_range.append((i_beg, i_end))
                i_beg = i_end+1
            # Assign any remaining items to the final thread:
            i_end = n_vf
            i_vf_range.append((i_beg, i_end))
#%        else:
#%            raise RuntimeError("varfile run, but the value of N_VF is unknown")
    if not i_vf_range:
      # This is *not* a "MAKE_VAR_FILES" run:
        n_exe_PEs = n_PEs
        run_PRAMS_with_leafPEs = n_exe_PEs > 1
        n_py_threads = 1
        i_vf_range.append((1,1))   # Dummy value

    today = datetime.date.today()

    if n_py_threads > 1:
      # For a distributed "MAKE_VAR_FILES" run, force a "CHECK_SFC_FILES" run
      #  to be performed first (to ensure that sfcfiles are OK and available):
        # Create "unique" run_root directory path for this process:
        run_root_dir_full_wID = os.path.join(run_root_dir_full,
                                             "autogen_temporary-PRAMS",
                                             str(today)+'-'+ \
               ''.join(random.choice(string.ascii_uppercase + string.digits)
                  for _ in range(8)))
        generated_nml_strMSF = generated_nml_str0.replace("MAKE_VAR_FILES",
                                                          "CHECK_SFC_FILES", 1)
        run_PRAMS_instance(run_root_dir_full_wID, run_PRAMS_with_leafPEs,
                           n_exe_PEs, exe_file_full, generated_nml_fn,
                           nml_file_full, run_etc_dir_full, i_vf_range[0],
                           generated_nml_strMSF, 0.)

    # Create list of tuples containing the arguments for `run_PRAMS_instance`:
    arg_tuple_l = []
    for ip in range(n_py_threads):
        # Create "unique" run_root directory path for this process:
        run_root_dir_full_wID = os.path.join(run_root_dir_full,
                                             "autogen_temporary-PRAMS",
                                             str(today)+'-'+ \
               ''.join(random.choice(string.ascii_uppercase + string.digits)
                  for _ in range(8)))
        # Specify a delay [s] between process instantiations (possibly to allow
        #  time_staggering of multiple IO-intensive processes):
        if n_py_threads == 1:
            init_delay = 0.
        else:
            init_delay = float(ip)*5.

        arg_tuple_l.append((run_root_dir_full_wID, run_PRAMS_with_leafPEs,
                             n_exe_PEs, exe_file_full, generated_nml_fn,
                             nml_file_full, run_etc_dir_full, i_vf_range[ip],
                             generated_nml_str0, init_delay))

    # Use a pool of processes (may only have one worker) to do the configured
    #  work, with special treatment to allow a "clean" Ctrl-C termination:
    p_pool = multiprocessing.Pool(processes=n_py_threads)
    arg_tuple_t = tuple(arg_tuple_l)  # List of tuples -> tuple of tuples
    try:
        results = p_pool.map_async(run_PRAMS_instance_star,
                                   arg_tuple_t).get(99999999)
    except KeyboardInterrupt:
        print("main(): Caught KeyboardInterrupt, terminating any workers.")
        p_pool.terminate()
        p_pool.join()
    else:
        for cmd, status, raised_exception in results:
            if raised_exception is None:
                fmt = "({cmd}) done, status {status}"
            else:
                fmt = "failed to run ({cmd}), reason: {error}"
            print(fmt.format(**vars()))
        p_pool.close()
        p_pool.join()
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    usage_msg = """
usage: run_PRAMS.py [-h] [-f NML_FILE] [-n NPES] [-d] [--exe-file-prefix PFX]
                    [--run-etc-dir DIR] [--run-root_dir DIR]

Sets up and executes a PRAMS run.

optional arguments:
  -h, --help   Show this help message and exit.
  -f NML_FILE, --nml-file NML_FILE   The namelist filename/path to use
                                      (default: 'PRAMS_IN')
  -n NPES   The number of parallel elements (PEs) to use (default: 1); for a
             'MAKE_VAR_FILES' run, this many concurrent serial processes will
             share the work of producing the varfiles
  -d, --debug   Run PRAMS inside a debugger utility (gdb) (default: False)
  --exe-file-prefix PFX   PRAMS executable location/name prefix
                           (default: './exe/prams')
  --run-etc-dir DIR   Directory where necessary PRAMS configuration files (e.g.,
                       VTABLE) are located (default: './etc')
  --run-root-dir DIR   Directory where the PRAMS executable will/should be run
                        from (it and its parents will be created if needed
                        (default: '.')
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:n:d", ["help", "nml-file=",
                  "debug", "exe-file-prefix=", "run-etc-dir=", "run-root-dir="])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    nml_file = "PRAMS_IN"
    n_PEs = 1
    use_debugger = False
    exe_file_pfx = os.path.join("exe", "prams")
    run_etc_dir = "./etc"
    run_root_dir = '.'
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        elif o in ("-f", "--nml-file"):
            nml_file = a
        elif o in ("-n"):
            n_PEs = int(a)
        elif o in ("-d", "--debug"):
            use_debugger = True
        elif o in ("--exe-file-prefix"):
            exe_file_pfx = a
        elif o in ("--run-etc-dir"):
            run_etc_dir = a
        elif o in ("--run-root-dir"):
            run_root_dir = a
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 0 or len(args) < 0):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)

    # Run driver:
    main(nml_file, run_etc_dir, run_root_dir, exe_file_pfx, n_PEs)
