!```````````````````````````````````````````````````````````````````
module grid_mem_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine map_facts(n2, n3, glat, glon, fmapu, fmapv, fmapt, fmapm, fmapui,  &
     fmapvi, fmapti, fmapmi, xm, xt, ym, yt, plat, plon)

!+ Calculate map factors and per-gridpoint lat/lon values

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use main_memory_mod, only: map_projection
use body_specific_constants_mod, only: radius
use univ_OPS_proj_mod, only: xy_ll_OPS

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3
real(f4), dimension(n2), intent(IN) :: xm, xt
real(f4), dimension(n3), intent(IN) :: ym, yt
real(f4), intent(IN) :: plat, plon

real(f4), dimension(n2,n3), intent(INOUT) :: glat, glon
real(f4), dimension(n2,n3), intent(INOUT) :: fmapu, fmapv, fmapt, fmapm,  &
     fmapui, fmapvi, fmapti, fmapmi

!== Local declarations:

integer :: i, j
real(f4) :: c1, xm2, xt2, ym2, yt2

!=== Executable statements:

! Calculates map factors and inverse map factors at u,v,t,m-points and
! geographical lat/lon at t-points for a given polar stereographic grid:

if (map_projection == 1) then    ! OPS projection

   c1 = (2._f4*radius)**2

   do j=1,n3
      do i=1,n2
         xm2 = xm(i)*xm(i)
         xt2 = xt(i)*xt(i)
         ym2 = ym(j)*ym(j)
         yt2 = yt(j)*yt(j)
         
         fmapt(i,j) = ONE_f4+(xt2+yt2)/c1
         fmapu(i,j) = ONE_f4+(xm2+yt2)/c1
         fmapv(i,j) = ONE_f4+(xt2+ym2)/c1
         fmapm(i,j) = ONE_f4+(xm2+ym2)/c1
         
         fmapui(i,j) = ONE_f4/fmapu(i,j)
         fmapvi(i,j) = ONE_f4/fmapv(i,j)
         fmapti(i,j) = ONE_f4/fmapt(i,j)
         fmapmi(i,j) = ONE_f4/fmapm(i,j)
      end do
   end do

else if (map_projection == 0) then ! Cartesian projection (all map factors = 1.)

   fmapt(1:n2,1:n3) = ONE_f4
   fmapu(1:n2,1:n3) = ONE_f4
   fmapv(1:n2,1:n3) = ONE_f4
   fmapm(1:n2,1:n3) = ONE_f4
   fmapui(1:n2,1:n3) = ONE_f4
   fmapvi(1:n2,1:n3) = ONE_f4
   fmapti(1:n2,1:n3) = ONE_f4
   fmapmi(1:n2,1:n3) = ONE_f4

end if

! Calculate lat/lon pair of each gridpoint:

do j=1,n3
   do i=1,n2
      call xy_ll_OPS(glat(i,j), glon(i,j), plat, plon, xt(i), yt(j), radius)
   end do
end do

end subroutine map_facts


!```````````````````````````````````````````````````````````````````
subroutine grid_spacing(n2, n3, nx, ny, dxu, dxv, dxt, dxm, dyu, dyv, dyt,  &
     dym, fmapu, fmapv, fmapt, fmapm, xt, xm, yt, ym, deltay)

!+ Calculate gridspacings on staggered grids, etc.

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use main_memory_mod, only: jdim

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, nx, ny
real(f4), dimension(n2,n3), intent(IN) :: fmapu, fmapv, fmapt, fmapm
real(f4), dimension(n2), intent(IN) :: xt, xm
real(f4), dimension(n3), intent(IN) :: ym, yt
real(f4), intent(IN) :: deltay

real(f4), dimension(n2,n3), intent(INOUT) :: dxu, dxv, dxt, dxm, dyu, dyv,  &
     dyt, dym

!== Local declarations:

integer :: i, j

!=== Executable statements:

do j=1,n3
   do i=1,nx
      dxu(i,j) = fmapu(i,j)/(xt(i+1)-xt(i))
      dxm(i,j) = fmapm(i,j)/(xt(i+1)-xt(i))
   end do
   dxu(n2,j) = dxu(nx,j)*fmapu(n2,j)/fmapu(nx,j)
   dxm(n2,j) = dxm(nx,j)*fmapm(n2,j)/fmapm(nx,j)
   do i=2,n2
      dxv(i,j) = fmapv(i,j)/(xm(i)-xm(i-1))
      dxt(i,j) = fmapt(i,j)/(xm(i)-xm(i-1))
   end do
   dxv(1,j) = dxv(2,j)*fmapv(1,j)/fmapv(2,j)
   dxt(1,j) = dxt(2,j)*fmapt(1,j)/fmapt(2,j)
end do

if (jdim == 1) then
   do i=1,n2
      do j=1,ny
         dyv(i,j) = fmapv(i,j)/(yt(j+1)-yt(j))
         dym(i,j) = fmapm(i,j)/(yt(j+1)-yt(j))
      end do
      dyv(i,n3) = dyv(i,ny)*fmapv(i,n3)/fmapv(i,ny)
      dym(i,n3) = dym(i,ny)*fmapm(i,n3)/fmapm(i,ny)
      do j=2,n3
         dyu(i,j) = fmapu(i,j)/(ym(j)-ym(j-1))
         dyt(i,j) = fmapt(i,j)/(ym(j)-ym(j-1))
      end do
      dyu(i,1) = dyu(i,2)*fmapu(i,1)/fmapu(i,2)
      dyt(i,1) = dyt(i,2)*fmapt(i,1)/fmapt(i,2)
   end do
else
   dyu(1:n2,1:n3) = ONE_f4/deltay
   dyv(1:n2,1:n3) = ONE_f4/deltay
   dyt(1:n2,1:n3) = ONE_f4/deltay
   dym(1:n2,1:n3) = ONE_f4/deltay
end if

end subroutine grid_spacing


!```````````````````````````````````````````````````````````````````
subroutine coord_transforms(n1, n2, n3, nx, ny, igd, topt, topu, topv, topm,  &
     rtgt, rtgu, rtgv, rtgm, f13u, f13v, f13t, f13m, f23u, f23v, f23t, f23m,  &
     dxu, dxv, dxt, dxm, dyu, dyv, dyt, dym, xm, xt, ym, yt, zm, zt)

!+ Computes the coordinate transformation constants based values of TOPT

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_exception_mod, only: exception, FATAL
use main_memory_mod, only: jdim, ztop, itopo, gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, nx, ny, igd
real(f4), dimension(n2,n3), intent(IN) :: topt, dxu, dxv, dxt, dxm, dyu, dyv,  &
     dyt, dym
real(f4), dimension(n2), intent(IN) :: xm, xt
real(f4), dimension(n3), intent(IN) :: ym, yt
real(f4), dimension(n1), intent(IN) :: zm, zt

real(f4), dimension(n2,n3), intent(INOUT) :: topu, topv, topm, rtgt, rtgu,  &
     rtgv, rtgm, f13u, f13v, f13t, f13m, f23u, f23v, f23t, f23m

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'coord_transforms'

integer :: iztflag, i, j, k
real(f4) :: terdev

data iztflag/0/

!=== Executable statements:

terdev = ZERO_f4
DO J=1,n3
   DO I=1,nx
      TOPU(I,J) = TOPT(I,J)+(TOPT(I+1,J)-TOPT(I,J))  &
           *(XM(I)-XT(I))/(XT(I+1)-XT(I))
      TERDEV = MAX(TERDEV, ABS(TOPT(I,J)))
   END DO
   TOPU(n2,J) = TOPT(n2,J)+(TOPT(n2,J)-TOPT(NX,J))  &
        *(XM(n2)-XT(n2))/(XT(n2)-XT(NX))
END DO
IF (TERDEV < 1.e-6_f4) THEN
   ITOPO = 0
ELSE
   ITOPO = 1
END IF

IF (JDIM == 1) THEN
   DO I=1,n2
      DO J=1,NY
         TOPV(I,J) = TOPT(I,J)+(TOPT(I,J+1)-TOPT(I,J))  &
              *(YM(J)-YT(J))/(YT(J+1)-YT(J))
         TOPM(I,J) = TOPU(I,J)+(TOPU(I,J+1)-TOPU(I,J))  &
              *(YM(J)-YT(J))/(YT(J+1)-YT(J))
      END DO
      TOPV(I,n3) = TOPT(I,n3)+(TOPT(I,n3)-TOPT(I,NY))  &
           *(YM(n3)-YT(n3))/(YT(n3)-YT(NY))
      TOPM(I,n3) = TOPU(I,n3)+(TOPU(I,n3)-TOPU(I,NY))  &
              *(YM(n3)-YT(n3))/(YT(n3)-YT(NY))
   END DO
ELSE
   DO I=1,n2
      DO J=1,n3
         TOPV(I,J) = TOPT(I,J)
         TOPM(I,J) = TOPU(I,J)
      END DO
   END DO
END IF

ZTOP = gd(1)%zm(gd(1)%nzp-1)

DO K=1,n1
   gd(igd)%HT(K) = ZT(K)/ZTOP-ONE_f4
   gd(igd)%HW(K) = ZM(K)/ZTOP-ONE_f4
END DO
DO K=1,n1
   gd(igd)%HT2(K) = 0.5_f4*gd(igd)%HT(K)
   gd(igd)%HT4(K) = 0.25_f4*gd(igd)%HT(K)
   gd(igd)%HW2(K) = 0.5_f4*gd(igd)%HW(K)
   gd(igd)%HW4(K) = 0.25_f4*gd(igd)%HW(K)
END DO

DO J=1,n3
   DO I=1,n2
      RTGT(I,J) = ONE_f4-TOPT(I,J)/ZTOP
      RTGU(I,J) = ONE_f4-TOPU(I,J)/ZTOP
      RTGV(I,J) = ONE_f4-TOPV(I,J)/ZTOP
      RTGM(I,J) = ONE_f4-TOPM(I,J)/ZTOP
      if (topt(i,j) > 0.5_f4*ztop) then
         write (*, '(/,a)')   &
              'ERROR: Terrain height is over half the model domain height at'
         write (*, '(a,i2,3x,i5,2x,i5,2x,2f11.2)')  &
              'grid, i, j, topt, ztop = ', igd, i, j, topt(i,j), ztop
         iztflag = 1
      end if
   end do
end do

if (iztflag == 1) call exception(procedure_name, FATAL, 'Domain is too shallow')

DO J=1,n3
   DO I=2,n2
      F13T(I,J) = (TOPU(I,J)-TOPU(I-1,J))*DXT(I,J)/RTGT(I,J)
      F13V(I,J) = (TOPM(I,J)-TOPM(I-1,J))*DXV(I,J)/RTGV(I,J)
   END DO
   DO I=1,NX
      F13U(I,J) = (TOPT(I+1,J)-TOPT(I,J))*DXU(I,J)/RTGU(I,J)
      F13M(I,J) = (TOPV(I+1,J)-TOPV(I,J))*DXM(I,J)/RTGM(I,J)
   END DO
   F13T(1,J) = F13U(1,J)
   F13V(1,J) = F13M(1,J)
   F13U(n2,J) = F13T(n2,J)
   F13M(n2,J) = F13V(n2,J)
END DO

DO I=1,n2
   DO J=2,n3
      F23T(I,J) = (TOPV(I,J)-TOPV(I,J-JDIM))*DYT(I,J)/RTGT(I,J)
      F23U(I,J) = (TOPM(I,J)-TOPM(I,J-JDIM))*DYU(I,J)/RTGU(I,J)
   END DO
   DO J=1,NY
      F23V(I,J) = (TOPT(I,J+JDIM)-TOPT(I,J))*DYV(I,J)/RTGV(I,J)
      F23M(I,J) = (TOPU(I,J+JDIM)-TOPU(I,J))*DYM(I,J)/RTGM(I,J)
   END DO
   IF (JDIM == 1) THEN
      F23T(I,1) = F23V(I,1)
      F23U(I,1) = F23M(I,1)
      F23V(I,n3) = F23T(I,n3)
      F23M(I,n3) = F23U(I,n3)
   END IF
END DO

end subroutine coord_transforms


!```````````````````````````````````````````````````````````````````
subroutine switch_to_grid(igd)

!+ Given a grid #, fill the grid-specific 1D vars from the grid-dep arrays
!   (i.e., "switch grids")

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: i_grid, c_gd, gd, nscl, dts, dtlv, naerscl,  &
     run_is_parallel, my_rank, mxp, myp, mzp, ia, iz, ja, jz, i0, j0, ibndc
use vtables_mod, only: vtable_memidx, ind2d, ind3d, ind3ds, ind2ds, ivtab2d,  &
     ivtab3d, ivtab2ds, ivtab3ds, ind2dn, ind3dn, ind3dsn, ind2dsn, nv2d,  &
     nv3d, nv2ds, nv3ds, nsclr, naersclr
use main_memory_bspec_mod, only: c_gd_bd, gd_bd 

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd

!== Local declarations:

integer :: nv

!=== Executable statements:

i_grid = igd    ! Current grid

c_gd => gd(igd)   ! Set pointer to current grid object
c_gd_bd => gd_bd(igd)

! Fill the "A"-array indices:

do nv=1,nv2d(igd)
   ind2D(ivtab2D(nv,igd)) = ind2Dn(igd,ivtab2D(nv,igd))
end do
do nv=1,nv2Ds(igd)
   ind2Ds(ivtab2Ds(nv,igd)) = IND2DSN(igd,IVTAB2DS(NV,igd))
end do
do nv=1,NV3DS(igd)
   IND3DS(IVTAB3DS(NV,igd)) = IND3DSN(igd,IVTAB3DS(NV,igd))
end do
do nv=1,NV3D(igd)
   IND3D(IVTAB3D(NV,igd)) = IND3DN(igd,IVTAB3D(NV,igd))
end do

call vtable_memidx(igd, .TRUE.)    ! Assign named indices

! "Normal" scalar count:

NSCL = NSCLR(igd)

! "Aerosol" scalar count:

naerscl = naersclr(igd)

! Grid spacings:

!@DELTAZ = ZMN(2,igd)-ZMN(1,igd)

! Option flags:

! Timestep info:

DTS = 2._f4*c_gd%dt_dyn/real(c_gd%n_dta_per_dtdyn, kind=f4)
DTLV = 2._f4*c_gd%dt_dyn

! Node gridpoint info:

if (run_is_parallel .and. my_rank /= 0) then
   mxp = c_gd%mmxp
   myp = c_gd%mmyp
   mzp = c_gd%mmzp
   ia = c_gd%mia
   iz = c_gd%miz
   ja = c_gd%mja
   jz = c_gd%mjz
   i0 = c_gd%mi0
   j0 = c_gd%mj0
   ibndc = c_gd%mibndc
end if

end subroutine switch_to_grid


!```````````````````````````````````````````````````````````````````
subroutine switch_to_Nscalar(igd, iscnum)

!+ Given a "normal" scalar #, fill the scalar-specific 1D vars from the
!   multidimensional arrays (i.e., "switch scalars")

use vtables_mod, only: ind3dn, ind3dm, isclrp, isclrt, iscp, isct

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd, iscnum

!== Local declarations:

! NONE

!=== Executable statements:

! Set up several indices for a scalar variable:

ISCP = IND3DN(igd,ISCLRP(ISCNUM,igd))
ISCT = IND3DM(ISCLRT(ISCNUM,igd))

end subroutine switch_to_Nscalar


!```````````````````````````````````````````````````````````````````
subroutine switch_to_Ascalar(igd, iscnum)

!+ Given an "aerosol" scalar #, fill the scalar-specific 1D vars from the
!   multidimensional arrays (i.e., "switch scalars")

use vtables_mod, only: ind3dn, ind3dm, iaersclp, iaersclt, iaerscp, iaersct,  &
     iaerfi, iaerff, iaeraj, iaerag, iaerrs, iaersfi, iaersff, iaersaj,  &
     iaersag, iaersrs, ind2dn

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd, iscnum

!== Local declarations:

! NONE

!=== Executable statements:

! Set up several indices for a scalar variable:

iaerscp = ind3dn(igd,iaersclp(iscnum,igd))
iaersct = ind3dm(iaersclt(iscnum,igd))
iaerfi = ind2dn(igd,iaersfi(iscnum,igd))
iaerff = ind2dn(igd,iaersff(iscnum,igd))
iaeraj = ind2dn(igd,iaersaj(iscnum,igd))
iaerag = ind2dn(igd,iaersag(iscnum,igd))
iaerrs = ind2dn(igd,iaersrs(iscnum,igd))

end subroutine switch_to_Ascalar


!```````````````````````````````````````````````````````````````````
subroutine a2e3(n1, n2, n3, a_in, a_out, k_tgt)

!+ Copy 2D slice of 3D array

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, k_tgt
real(f4), dimension(n1,n2,n3), intent(IN) :: a_in

real(f4), dimension(n2,n3), intent(OUT) :: a_out

!== Local declarations:

integer :: i, j

!=== Executable statements:

do j=1,n3
   do i=1,n2
      a_out(i,j) = a_in(k_tgt,i,j)
   end do
end do

end subroutine a2e3


end module grid_mem_mod
