!```````````````````````````````````````````````````````````````````
module leafPE_driver_mod

!+ Module that contains PRAMS leaf PE routines

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine leafPE_driver(this_rank)

!+ PRAMS driver for a leaf PE

use univ_kind_defs_mod, only: f8, f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_math_constants_mod, only: ZERO_f4
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_wait_for_all, MPI_buffer_packed_obj,  &
     mpiq_check_buffer_packed
#endif  /* end:USE_MPI */
use univ_integer_time_mod, only: fp_seconds_eqv_time, set_time
use main_memory_mod, only: pbanner, pbnlen, a, b, memsizeA, memsizeB,  &
     manage_misc_memory, orbital_info_table, target_Ls,  &
     start_UT, RT_paradigm, n_grids, gd, manage_grid_object, mnsdy_Time,  &
     morbpd_Time, manage_nml_var_memory,  &
     zero_Time, module_tps, rootPE_rank, my_rank, manage_parallel_memory,  &
     c_gd, i_grid, initialization_type, manage_misc_memory
use main_memory_bspec_mod, only: dust_ivr, RTc_tog, Ls_of_periapse,  &
     orbital_eccentricity, axial_obliquity, AU_to_m_knob, c_gd_bd,  &
     gd_bd, manage_grid_object_bspec, manage_nml_var_memory_bspec
use vtables_mod, only: vtable_mem, naersclr
use aermic_rad_mem_mod, only: aermicrad_AD
use parallel_init_vars_mod, only: leafPE_get_IDs, leafPE_get_nlst_vars,  &
     leafPE_get_vtables, leafPE_get_root_grid, leafPE_get_leaf_grid,  &
     leafPE_get_gen_grid, leafPE_get_nest_coeffs, leafPE_get_misc
use setup_model_storage_mod, only: leafPE_memsetA, mem_setupB
use simple_astron_scheme_mod, only: sas_t_obj_recycle, populate_sas_table,  &
     set_sloc_sas_table
use body_specific_constants_mod, only: morbpd, smaxis, mass_bodysys,  &
     massratio_Sun2bodysys, mnsdy
use aemic_drvint_mod, only: aer_setup
use aerad_interface_mod, only: aerad_setup
use ARCnRT_interface_mod, only: ARCnRT_setup
use parallel_samegrid_lBC_mod, only: leafPE_longdt_lBC_init,  &
     leafPE_acoustic_lBC_init
use parallel_nesting_mod, only: leafPE_dnscale_nBC_init, leafPE_upscale_nBC_init

implicit none

!=== Argument declarations:

integer, intent(IN) :: this_rank

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'leafPE_driver'

integer :: a_err, igd

#if defined(USE_MPI)
type(MPI_buffer_packed_obj) :: rx_buffer_packed
#endif  /* end:USE_MPI */

real(f8) :: tmp1_f8, tmp2_f8

!=== Executable statements:

my_rank = this_rank     ! Set module parameter

! Receive model initialization info from the root PE:

#if defined(USE_MPI)
call mpiq_wait_for_all(my_rank, 555)

call leafPE_get_IDs(.TRUE., rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_IDs; DONE'; flush 6

call leafPE_get_nlst_vars(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_nlst_vars; DONE'; flush 6
#endif  /* end:USE_MPI */

  ! Compute/set some important time parameters:
zero_Time = set_time(0, 0, module_tps)
mnsdy_Time = fp_seconds_eqv_time(mnsdy, module_tps)
morbpd_Time = fp_seconds_eqv_time(morbpd, module_tps)

#if defined(USE_MPI)
call leafPE_get_vtables(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_vtables; DONE'; flush 6
#endif  /* end:USE_MPI */

call manage_misc_memory(dim_maxnnp=.TRUE., dim_SBPS=.TRUE.,  &
     dim_isched=.TRUE.,  dim_vf_refs=.TRUE.)

#if defined(USE_MPI)
call leafPE_get_root_grid(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_root_grid; DONE'; flush 6
#endif  /* end:USE_MPI */

call manage_parallel_memory(dim_misc=.TRUE.)   ! Allocate arrays

! Grid-dependent memory, INDEPENDENT of the parallel decomposition (base object
!  already allocated, with component pointers nullified):
do igd=1,n_grids
   call manage_grid_object(gd(igd), allocate_pd_ind_pointers=.TRUE.)
   call manage_grid_object_bspec(gd_bd(igd), gd(igd),  &
        allocate_pd_ind_pointers=.TRUE.)
end do

#if defined(USE_MPI)
call leafPE_get_leaf_grid(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_leaf_grid; DONE'; flush 6
#endif  /* end:USE_MPI */

! Grid-dependent memory, DEPENDENT on the parallel decomposition (base object
!  already allocated, with component pointers nullified):
do igd=1,n_grids
   call manage_grid_object(gd(igd), allocate_pd_dep_pointers=.TRUE.)
   call manage_grid_object_bspec(gd_bd(igd), gd(igd),  &
        allocate_pd_dep_pointers=.TRUE.)
end do

#if defined(USE_MPI)
call leafPE_get_gen_grid(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_gen_grid; DONE'; flush 6

call leafPE_get_nest_coeffs(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_nest_coeffs; DONE'; flush 6

call leafPE_get_misc(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_misc; DONE'; flush 6
#endif  /* end:USE_MPI */

! Initialize static parameters that depend on the communication schedule(s):

call leafPE_longdt_lBC_init
call leafPE_acoustic_lBC_init
call leafPE_dnscale_nBC_init
call leafPE_upscale_nBC_init

! Setup aerosol array dimensions, etc.:

call aer_setup
write (*, '(2a)') pbanner(1:pbnlen), 'aer_setup; DONE'; flush 6

! Setup "A"-array indices, etcetera (only use the leaf PE subdomain size)

call leafPE_memsetA(.TRUE.)

! Setup "B"-array indices, etc. (needed for nest boundary operations):

call mem_setupB(.TRUE.)    ! Setup "B"-array indices, etc. 

allocate(a(memsizeA), b(memsizeB), stat=a_err)   ! Allocate main memory
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'A')

write (*, '(/,a,a)') pbanner(1:pbnlen), 'Allocated "A" array...'
write (*, '(a,a)') pbanner(1:pbnlen), 'Allocated "B" array...'
flush 6

a(1:memsizeA) = ZERO_f4  ! Zero out array values
b(1:memsizeB) = ZERO_f4  !
write (*, '(/,2a)') pbanner(1:pbnlen),  &
     'Initialized main model memory (to zero).'; flush 6

! Initialize orbital info table:
call populate_sas_table(orbital_info_table, morbpd, 1000._f8, smaxis,  &
     real(AU_to_m_knob, kind=f8), mass_bodysys, massratio_Sun2bodysys,  &
     real(orbital_eccentricity, kind=f8), real(Ls_of_periapse, kind=f8),  &
     real(axial_obliquity, kind=f8))
! Find and record starting location in orbital/time table:
tmp1_f8 = real(target_Ls, kind=f8); tmp2_f8 = real(start_UT, kind=f8)
call set_sloc_sas_table(orbital_info_table, tmp2_f8, mnsdy, tmp1_f8)
target_Ls = real(tmp1_f8, kind=f4)

write (*, '(/,2a)') pbanner(1:pbnlen),  &
     'Initialized annual orbital/time parameter table...'; flush 6

! Possibly initialize RT code:

if ((RT_paradigm == 2 .and. (RTc_tog == 2 .or. RTc_tog == 3)) .or.  &
     dust_ivr >= 2) call aerad_setup(2, gd(1)%mmzp, naersclr(1))

if ((RT_paradigm == 2 .and. RTc_tog == 4) .or.  &
     dust_ivr >= 2) call ARCnRT_setup(2, gd(1)%mmzp, naersclr(1))

#if defined(USE_MPI)
! Get fields from the root PE and finish initialization:
call leafPE_get_fields(rx_buffer_packed)
write (*, '(2a)') pbanner(1:pbnlen), 'get_fields; DONE'; flush 6

! Deallocate initialization receive buffer (no longer needed):
call mpiq_check_buffer_packed(rx_buffer_packed, 0)

! Notify the root PE of initialization completion; wait for all other PEs:
call mpiq_wait_for_all(rootPE_rank, 555)
#endif  /* end:USE_MPI */

! Initialize CFL arrays:
do igd=1,n_grids
   i_grid = igd
   c_gd => gd(igd); c_gd_bd => gd_bd(igd)  ! Switch grid (incomplete switch)
   
   c_gd%Courant_number_xyz = ZERO_f4   ! Initialize

   call cfl(c_gd%mmzp, c_gd%mmxp, c_gd%mmyp, c_gd%mi0, c_gd%mj0, .TRUE.)
end do

! Call the model time integration driver:
call leafPE_model

! Done with run, so deallocate stuff:

call sas_t_obj_recycle(orbital_info_table)    ! Deallocate orbital info table

if (initialization_type == 2) call manage_misc_memory(dim_n_varftot=.FALSE.)

! Deallocate main memory:

deallocate(a, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'B')
write (*, '(/,2a)') pbanner(1:pbnlen),  &
     'Successfully DEallocated "A" array...'; flush 6

deallocate(b, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'C')
write (*, '(2a)') pbanner(1:pbnlen),  &
     'Successfully DEallocated "B" array...'; flush 6

call manage_parallel_memory(dim_misc=.FALSE.)   ! Deallocate arrays

call manage_misc_memory(dim_n_varftot=.FALSE., dim_maxnnp=.FALSE.,  &
     dim_SBPS=.FALSE., dim_isched=.FALSE.,  dim_vf_refs=.FALSE.)

call aermicrad_AD(for_deallocation, 0)        ! Deallocate relevant arrays
call vtable_mem(for_deallocation)    ! Deallocate memory for variable tables

do igd=1,n_grids
   call manage_grid_object(gd(igd), recycle_pointers=.TRUE.)
   call manage_grid_object_bspec(gd_bd(igd), gd(igd), recycle_pointers=.TRUE.)
end do
call manage_nml_var_memory(dim_n_grids=.FALSE.)
call manage_nml_var_memory_bspec(dim_n_grids=.FALSE.)

call manage_parallel_memory(dim_n_PE_leaf=.FALSE.)    ! Deallocate arrays

end subroutine leafPE_driver


!```````````````````````````````````````````````````````````````````
subroutine leafPE_model

!+ Model time integration driver for a leaf PE

use univ_kind_defs_mod, only: f4, f8
use univ_math_constants_mod, only: ONE_f4
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_wait_for_all, MPI_buffer_packed_obj,  &
     mpiq_check_buffer_packed
#endif  /* end:USE_MPI */
use parallel_timing_mod, only: leafPE_put_timings, leafPE_get_dtsched
use grid_mem_mod, only: switch_to_grid
use main_memory_mod, only: tstep, i_grid, isstp, nsubs, isched, pbanner,  &
     pbnlen, n_grids, gd, current_sim_ET, maximum_sim_ET, c_gd,  &
     my_rank, n_PE_leaf, begin_dtdyn_sim_ET
use aemic_drvint_mod, only: aerproc_check

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ifm, icm, nfeed, npass

integer, dimension(n_PE_leaf) :: rx_ireq, tx_ireq

#if defined(USE_MPI)
type(MPI_buffer_packed_obj) :: rx_buffer_p_sync, tx_buffer_p_sync
type(MPI_buffer_packed_obj), dimension(n_PE_leaf) :: rx_buffer_p_async,  &
        tx_buffer_p_async
#endif  /* end:USE_MPI */

logical :: run_is_finished, all_results_to_rootPE_now,  &
     all_fields_from_rootPE_now

!=== Executable statements:

! Initialize various items for use in the main (long) timestep loop below:

gd(1:n_grids)%RT_call = -1
tstep = 0

all_results_to_rootPE_now = .FALSE.; all_fields_from_rootPE_now = .FALSE.

run_is_finished = ( current_sim_ET >= maximum_sim_ET )

! Main (long) timestep loop (loop until max simulation time reached):

main_loop: do

   if (run_is_finished) exit main_loop    ! Finished

   tstep = tstep+1   ! Increment timestep
   begin_dtdyn_sim_ET = current_sim_ET    ! Store ET value at start of timestep

#if defined(USE_MPI)
      ! Receive message(s) from root PE containing control flags, new dt, etc:
   call leafPE_get_dtsched(all_results_to_rootPE_now,  &
        all_fields_from_rootPE_now, rx_buffer_p_sync)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
    write (*, '(/,2a)') pbanner(1:pbnlen),  &
         'leafPE_get_dtsched; DONE'; flush 6
#endif  /* end:MPI_DEBUG */

      ! Manage model process overrides for aerosol variables:
   if (tstep > 1) then
      call aerproc_check(.FALSE.) 
   else
      call aerproc_check(.TRUE.)    ! Initialize
   end if

   gd(1:n_grids)%Courant_number_xyz = -ONE_f4

   ! Start the timestep schedule to loop through all grids and advance them
   ! in time an increment equal to gd(1)%dt_dyn:

   npass_loop: do npass=1,nsubs

      isstp = isched(npass,3)   ! Substep #

         ! Switch indices to new grid (current grid to step fwd in time):
      call switch_to_grid(isched(npass,1))

         ! Current (sub)time:
      current_sim_ET = begin_dtdyn_sim_ET+  &
           real((isched(npass,5)-1)*c_gd%dt_dyn, kind=f8)

         ! Set some parallel processing indices:
      call set_leafPE_indices

#if defined(MPI_DEBUG)
      write (*, '(/,a,2(a,i2))') pbanner(1:pbnlen),  &
            'timestep driver; BEGIN -- grid ', i_grid, ', npass ', npass
      flush 6
#endif  /* end:MPI_DEBUG */
         ! Execute one timestep's operations
      call timestep
#if defined(MPI_DEBUG)
      write (*, '(/,a,2(a,i2))') pbanner(1:pbnlen),  &
            'timestep driver; DONE -- grid ', i_grid, ', npass ', npass
      flush 6
#endif  /* end:MPI_DEBUG */

         ! Set a legacy (?) parameter:
      c_gd%ngbegun = 1

         ! Is it time to transmit relevant parent grid points to other
         !  leaf PEs for use as finer grid BCs?
      if (isched(npass,2) /= 0) then
         ifm = isched(npass,2); isstp = isched(npass,3)
         icm = gd(ifm)%nxtnest

         call switch_to_grid(ifm)
#if defined(USE_MPI)
         call leafPE_dnscale_nBC(ifm, icm, tx_buffer_p_async,  &
              rx_buffer_p_async, tx_ireq, rx_ireq, n_PE_leaf)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
         write (*, '(/,2a,i2)') pbanner(1:pbnlen),  &
              'leafPE_dnscale_nBC; DONE -- grid ', i_grid
         flush 6
#endif  /* end:MPI_DEBUG */
      end if

         ! Is it time to feedback fields from fine grids to coarser grids?
      if (isched(npass,4) /= 0) then
         i_grid = isched(npass,1)
         do nfeed=1,isched(npass,4)
            call switch_to_grid(i_grid)
#if defined(USE_MPI)
            call leafPE_upscale_nBC(c_gd%nxtnest, i_grid,  &
                 tx_buffer_p_async(1), rx_buffer_p_async(1), tx_ireq,  &
                 rx_ireq, n_PE_leaf)
#endif  /* end:USE_MPI */
            i_grid = c_gd%nxtnest
         end do
#if defined(MPI_DEBUG)
         write (*, '(/,2a,i2)') pbanner(1:pbnlen),  &
              'leafPE_upscale_nBC; DONE -- grid ', i_grid
         flush 6
#endif  /* end:MPI_DEBUG */
      end if

   end do npass_loop

#if defined(USE_MPI)
   ! Transmit timing info/CFL numbers to root PE:
   call leafPE_put_timings(2._f4, 2._f4, tx_buffer_p_sync)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
   write (*, '(/,2a)') pbanner(1:pbnlen), 'leafPE_put_timings; DONE'
   flush 6
#endif  /* end:MPI_DEBUG */

   ! Send entire leaf PE subdomain back to the root PE every now and then:

   if (all_results_to_rootPE_now) then
      call leafPE_put_all_results
#if defined(USE_MPI)
      call mpiq_wait_for_all(my_rank, 9999)  ! Wait for completion
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
      write (*, '(/,2a)') pbanner(1:pbnlen), 'leafPE_put_all_results; DONE'
      flush 6
#endif  /* end:MPI_DEBUG */
   end if

   ! Update main time variable by a long timestep:
   
   current_sim_ET = begin_dtdyn_sim_ET+real(gd(1)%dt_dyn, kind=f8)
   run_is_finished = ( current_sim_ET >= maximum_sim_ET )

   ! This part is for the dynamic balancing and sending new varfile info:
   if (all_fields_from_rootPE_now .and. .not.run_is_finished) then
!            call initpara (0,A,ioff,iaddr,b,iboff,iaddrb,ntsdum)
#if defined(USE_MPI)
      call leafPE_get_fields(rx_buffer_p_sync)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
      write (*, '(/,2a)') pbanner(1:pbnlen), 'leafPE_get_fields; DONE'
      flush 6
#endif  /* end:MPI_DEBUG */
   end if

end do main_loop

! Deallocate some communication buffers:
#if defined(USE_MPI)
call mpiq_check_buffer_packed(rx_buffer_p_sync, 0)
call mpiq_check_buffer_packed(tx_buffer_p_sync, 0)
#endif  /* end:USE_MPI */

end subroutine leafPE_model


!```````````````````````````````````````````````````````````````````
subroutine set_leafPE_indices

!+ Set some useful leaf_PE-specific values:

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: ia_1, ia_2, ia_3, ia, ia1, ia2, ia3, iz_1,  &
     iz_2, iz_3, iz, iz1, iz2, iz, ja_1, ja_2, ja_3, ja, ja1, ja2, ja3,  &
     jz_1, jz_2, jz_3, jz, jz1, jz2, jz, izu, jzv, ibndc, myp, mxp, iz3, jz3

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'set_leafPE_indices'

!=== Executable statements:

ia_1 = max(ia-1, 1); ia_2 = max(ia-2, 1); ia_3 = max(ia-3, 1)
ia1 = ia+1; ia2 = ia+2; ia3 = ia+3

iz_1 = iz-1; iz_2 = iz-2; iz_3 = iz-3
iz1 = min(iz+1, mxp); iz2 = min(iz+2, mxp); iz3 = min(iz+3, mxp)

izu = iz
if (iand(ibndc,2) /= 0) izu = iz-1

if (myp > 1) then

   ja_1 = max(ja-1, 1); ja_2 = max(ja-2, 1); ja_3 = max(ja-3, 1)
   ja1 = ja+1; ja2 = ja+2; ja3 = ja+3

   jz_1 = jz-1; jz_2 = jz-2; jz_3 = jz-3
   jz1 = min(jz+1, myp); jz2 = min(jz+2, myp); jz3 = min(jz+3, myp)

   jzv = jz
   if (iand(ibndc,8) /= 0) jzv = jz-1

else

   call exception(procedure_name, FATAL,  &
        'Trying to do 2D run???  No 2D runs permitted in parallel.')

!%   ja_1 = 1; ja_2 = 1; ja_3 = 1
!%   ja1 = 1; ja2 = 1; ja3 = 1

!%   jz_1 = 1; jz_2 = 1; jz_3 = 1
!%   jz1 = 1; jz2 = 1; jz3 = 1

!%   jzv = 1

end if

end subroutine set_leafPE_indices


end module leafPE_driver_mod
