!```````````````````````````````````````````````````````````````````
module namelist_read_mod

!+ Module that contains routines and variables related to reading the PRAMS
!   namelist (dependent on both common and body-specific modules)

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine read_namelist(namelist_fn)

!+ Main driver for reading PRAMS namelist 

use univ_std_fileIO_mod, only: find_file_IOunit
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN
use namelist_mod, only: proc_nml_static_params, proc_nml_basic_control,  &
     proc_nml_time_config, proc_nml_spatial_config, proc_nml_inout_config,  &
     proc_nml_model_options, proc_nml_PIPP_control, nml_info_check
use namelist_bspec_mod, only: nml_info_check_bspec,  &
     proc_nml_model_options_bspec, proc_nml_initial_sndg,  &
     proc_nml_inout_config_bspec, proc_nml_basic_control_bspec

implicit none

!=== Argument declarations:
 
character(len=*), intent(IN) :: namelist_fn

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'read_namelist'

integer :: namelist_fh, cn
character(len=MAX_MESSAGE_LEN) :: msg
logical :: file_exists

!=== Executable statements:

! Obtain an available file unit:
namelist_fh = find_file_IOunit()

! Check whether the namelist file exists; if not, abort program execution:
cn = len_trim(namelist_fn)
inquire(file=namelist_fn(1:cn), exist=file_exists)
if (.NOT.file_exists) then
   write(msg, '(3a)') 'Input PRAMS namelist file (', namelist_fn(1:cn),  &
        ') not found.'
   call exception(procedure_name, FATAL, trim(msg))
end if

! Open namelist file:
open(namelist_fh, file=namelist_fn(1:cn), status='old', form='formatted')

call nml_info_check(namelist_fh)

call nml_info_check_bspec(namelist_fh)   ! Perform body-specific check(s)

!== Read/process additional namelist blocks:

call proc_nml_static_params(namelist_fh)

call proc_nml_basic_control(namelist_fh)

call proc_nml_basic_control_bspec(namelist_fh)

call proc_nml_time_config(namelist_fh)

call proc_nml_spatial_config(namelist_fh)

call proc_nml_inout_config(namelist_fh)

call proc_nml_inout_config_bspec(namelist_fh)

call proc_nml_model_options(namelist_fh)

call proc_nml_model_options_bspec(namelist_fh)

call proc_nml_initial_sndg(namelist_fh)

call proc_nml_PIPP_control(namelist_fh)

close(namelist_fh)  ! Close file

end subroutine read_namelist


end module namelist_read_mod
