!```````````````````````````````````````````````````````````````````
module parallel_core_mod

!+ Core routines for the parallel computing infrastructure

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine p_decomp_est_work(work)

!+ Obtain estimates of the fraction of computational time (work) required
!   for each column in the root domain

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4

implicit none

!=== Argument declarations:

real(f4), dimension(:,:), intent(OUT) :: work

!== Local declarations:

real(f4), parameter :: bnd_work_factor = 0.2_f4

integer :: n2, n3, i, j

!=== Executable statements:

!***** IMPROVE ME!!! I'm not doing much right now!!!!!!

n2 = size(work, 1); n3 = size(work, 2)

! Over most of the grid, leaf PEs have work to do:

work(2:n2-1,2:n3-1) = ONE_f4     ! Constant amount of work

! Leaf PEs perform less work on actual (not decomposed) grid boundaries:

do j=1,n3
   work(1,j) = bnd_work_factor*work(2,j)
   work(n2,j) = bnd_work_factor*work(n2-1,j)
end do
do i=1,n2
   work(i,1) = bnd_work_factor*work(i,2)
   work(i,n3) = bnd_work_factor*work(i,n3-1)
end do

end subroutine p_decomp_est_work


!```````````````````````````````````````````````````````````````````
subroutine p_decomp(n_PEs, work, igrid)

!+ Decomposes the root domain, taking into account the number of leaf PEs
!   and their estimated "work" capabilities
!
!  This routine decomposes root grid domains of size (root_n2, root_n3) into a
!   number of rectangular subdomains (specified by the number of PEs
!   available/desired).  The convention is followed that any internal
!   boundaries (between subdomains) that are parallel to the x-axis run
!   continuously across the full domain, while boundaries parallel to the
!   y-axis may or may not run the full distance across the domain.  For
!   convenience, regions of the domain bounded by adjacent x-dir internal
!   boundaries are termed "subdomains", while smaller divisions within each
!   "subdomain" are termed "blocks".  Each "block" is required to have a
!   minimum size of 2 by 2 grid cells.  If this cannot be satisfied with
!   with the given input parameters, the subroutine stops.

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_MESSAGE_LEN
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_exception_mod, only: exception, FATAL
use main_memory_mod, only: gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: igrid, n_PEs
real(f4), dimension(:,:), intent(IN) :: work

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'p_decomp'

integer :: i, j, n, n_subdomains, min_n_blocks, n_tough_subdomains, isub,  &
     iblock, i_PE, j_PE, k_PE, iPE, jPE, root_n2, root_n3, a_err
real(f4) :: e_subdomains, total_speed, work_per_root_domain, work_so_far,  &
     work_accum, subdomain_speed, work_per_subdomain
character(len=MAX_MESSAGE_LEN) :: str_tmp

real(f4), dimension(size(work,2)) :: work_per_row
real(f4), dimension(size(work,1)) :: work_per_col
real(f4), dimension(n_PEs) :: relative_leafPE_speed

integer, allocatable, dimension(:) :: j_rows, n_blocks, bottom_j
real(f4), allocatable, dimension(:) ::  work_per_block, workload

!=== Executable statements:

relative_leafPE_speed(:) = ONE_f4  ! Default = 1. for leaf PEs of uniform speed
total_speed = sum(relative_leafPE_speed)

root_n2 = size(work,1); root_n3 = size(work,2)

! Estimate the number of subdomains to be used ('e_subdomains'), and compute a
!  final nearest integer value (n_subdomains) which is limited to allowable
!  values:

e_subdomains = sqrt(real(n_PEs, kind=f4)*real(root_n3, kind=f4)/  &
     real(root_n2, kind=f4))
n_subdomains = min(n_PEs, max(1, nint(e_subdomains)))

allocate(j_rows(n_subdomains), n_blocks(n_subdomains),  &
     bottom_j(n_subdomains), workload(n_subdomains), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'A')

! Compute total work load over each row and over entire root domain:

do j=1,root_n3
   work_per_row(j) = sum(work(:,j))
end do
work_per_root_domain = sum(work_per_row(:))

work_per_row(2) = work_per_row(2)+work_per_row(1)
work_per_row(root_n3-1) = work_per_row(root_n3-1)+work_per_row(root_n3)

! Determine number of blocks and the average workload for each subdomain:

min_n_blocks = n_PEs/n_subdomains
n_tough_subdomains = n_PEs-min_n_blocks*n_subdomains; n = 0
do isub=1,n_subdomains
   workload(isub) = ZERO_f4; n_blocks(isub) = min_n_blocks
   if (isub <= n_tough_subdomains) n_blocks(isub) = min_n_blocks+1
   do iblock=1,n_blocks(isub)
      n = n+1
      workload(isub) = workload(isub)+  &
           work_per_root_domain*relative_leafPE_speed(n)/total_speed
   end do
end do

allocate(work_per_block(maxval(n_blocks)), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'B')

! Assign all j-rows to their respective subdomains in a way that balances the
!  workload among subdomains according to their respective numbers of assigned
!  PEs (blocks). The array 'j_rows' counts the number of rows in each
!  subdomain, and the array 'bottom_j' is the index of the (root domain) row
!  nearest the "bottom" edge of each subdomain:

j_rows(:) = 0; isub = 0; work_accum = ZERO_f4; work_so_far = ZERO_f4
do j=2,root_n3-1
   work_accum = work_accum+work_per_row(j)
   if (work_accum-0.5_f4*work_per_row(j) > work_so_far .and.  &
        isub < n_subdomains) then
      isub = isub+1; bottom_j(isub) = j
      work_so_far = work_so_far+workload(isub)
   end if
   j_rows(isub) = j_rows(isub)+1
end do

i_PE = 0; j_PE = 0; k_PE = 0; gd(igrid)%ixe(:) = 0; iPE = 0
do isub = 1,n_subdomains

   ! Compute the total work load for each subdomain and for each i-column in
   !  the subdomain:

   work_per_subdomain = ZERO_f4; work_per_col(:) = ZERO_f4
   do i=1,root_n2
      do j=bottom_j(isub),bottom_j(isub)+j_rows(isub)-1
         work_per_col(i) = work_per_col(i)+work(i,j)
      end do
      work_per_subdomain = work_per_subdomain+work_per_col(i)
   end do
   work_per_col(2) = work_per_col(2)+work_per_col(1)
   work_per_col(root_n2-1) = work_per_col(root_n2-1)+work_per_col(root_n2)

   ! Determine average workload for each block:

   subdomain_speed = ZERO_f4
   do iblock=1,n_blocks(isub)
      j_PE = j_PE+1
      subdomain_speed = subdomain_speed+relative_leafPE_speed(j_PE)
   end do

   do iblock=1,n_blocks(isub)
      k_PE = k_PE+1
      work_per_block(iblock) = work_per_subdomain*  &
           relative_leafPE_speed(k_PE)/subdomain_speed
   end do

   ! Assign the i-columns of each subdomain to their respective blocks in a
   !  way that balances the work load among the blocks:

   work_accum = ZERO_f4; work_so_far = ZERO_f4

   iblock = 0
   do i=2,root_n2-1
      work_accum = work_accum+work_per_col(i)

      if (work_accum-0.5_f4*work_per_col(i) > work_so_far .and.  &
           iblock < n_blocks(isub)) then
         iblock = iblock+1

         ! Some "final" leaf PE control parameters:
         iPE = iPE+1
         gd(igrid)%ixb(iPE) = i
         gd(igrid)%iyb(iPE) = bottom_j(isub)
         gd(igrid)%iye(iPE) = gd(igrid)%iyb(iPE)+j_rows(isub)-1

         work_so_far = work_so_far+work_per_block(iblock)
      end if
      gd(igrid)%ixe(iPE) = gd(igrid)%ixe(iPE)+1
   end do
end do

! Some "final" leaf PE control parameters:
do jPE=1,n_PEs
   gd(igrid)%ixe(jPE) = gd(igrid)%ixb(jPE)+gd(igrid)%ixe(jPE)-1
end do

! Check to make sure that each subdomain (for each leaf PE) has at least 4
!  interior horizontal points (2 points in each horizontal direction):

do jPE=1,n_PEs
   if (gd(igrid)%iye(jPE)-gd(igrid)%iyb(jPE) < 1 .or.  &
        gd(igrid)%ixe(jPE)-gd(igrid)%ixb(jPE) < 1) then
      write (str_tmp, '(a,i3,a)') 'grid #', igrid,  &
           ' has too small of a leaf PE domain.'
      call exception(procedure_name, FATAL, trim(str_tmp))
   end if
end do

! Deallocate storage:

deallocate(j_rows, n_blocks, bottom_j, workload, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'A')

deallocate(work_per_block, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'B')

end subroutine p_decomp


!```````````````````````````````````````````````````````````````````
subroutine p_decomp_bounds(root_n2g, root_n3g, nbnd2, nbnd3)

!+ Compute various subdomain boundary parameters for leaf PEs

use main_memory_mod, only: n_PE_leaf, gd

implicit none

!=== Argument declarations:

integer, dimension(:), intent(IN) :: root_n2g, root_n3g
integer, intent(IN) :: nbnd2, nbnd3

!== Local declarations:

integer :: igrid, n_grids, n, root_n2, root_n3

!=== Executable statements:

! leafPE_xb, leafPE_xe, leafPE_yb, leafPE_ye - bounds of root domain region
!                                              that each leaf PE receives
!                                              (includes overlap region and
!                                              noncompute points)
!
! leafPE_ia, leafPE_iz, leafPE_ja, leafPE_jz - bounds of subdomain (leaf PE
!                                              domain) points on which each
!                                              leaf PE *computes* values 
!
! leafPE_i0, leafPE_j0 - leaf PE subdomain offsets relative to the root domain
!
! leafPE_ibndc - bitflag denoting if root domain grid boundary is on leaf PE
!                 subdomain (bit 1=west, bit 2=east, bit 3=south, bit 4=north)

n_grids = size(root_n2g, 1)

do igrid=1,n_grids
   root_n2 = root_n2g(igrid); root_n3 = root_n3g(igrid)

   do n=1,n_PE_leaf

      gd(igrid)%leafPE_ibndc(n) = 0

      if (gd(igrid)%ixb(n) == 2) then
         gd(igrid)%leafPE_xb(n) = 1; gd(igrid)%leafPE_i0(n) = 0
         gd(igrid)%leafPE_ia(n) = 2
         gd(igrid)%leafPE_ibndc(n) = gd(igrid)%leafPE_ibndc(n)+1
      else
         gd(igrid)%leafPE_xb(n) = gd(igrid)%ixb(n)-nbnd2
         gd(igrid)%leafPE_i0(n) = gd(igrid)%leafPE_xb(n)-1
         gd(igrid)%leafPE_ia(n) = nbnd2+1
      end if

      if (gd(igrid)%ixe(n) == root_n2-1) then
         gd(igrid)%leafPE_xe(n) = root_n2
         gd(igrid)%leafPE_ibndc(n) = gd(igrid)%leafPE_ibndc(n)+2
         gd(igrid)%leafPE_iz(n) = (1+gd(igrid)%ixe(n)-gd(igrid)%ixb(n))+  &
              gd(igrid)%leafPE_ia(n)-1
      else
         gd(igrid)%leafPE_xe(n) = gd(igrid)%ixe(n)+nbnd2
         gd(igrid)%leafPE_iz(n) = (gd(igrid)%ixe(n)-gd(igrid)%ixb(n))+  &
              gd(igrid)%leafPE_ia(n)
      end if

      if (gd(igrid)%iyb(n) == 2) then
         gd(igrid)%leafPE_yb(n) = 1; gd(igrid)%leafPE_j0(n) = 0
         gd(igrid)%leafPE_ja(n) = 2
         gd(igrid)%leafPE_ibndc(n) = gd(igrid)%leafPE_ibndc(n)+4
      else
         gd(igrid)%leafPE_yb(n) = gd(igrid)%iyb(n)-nbnd3
         gd(igrid)%leafPE_j0(n) = gd(igrid)%leafPE_yb(n)-1
         gd(igrid)%leafPE_ja(n) = nbnd3+1
      end if

      if (gd(igrid)%iye(n) == root_n3-1) then
         gd(igrid)%leafPE_ye(n) = root_n3
         gd(igrid)%leafPE_ibndc(n) = gd(igrid)%leafPE_ibndc(n)+8
         gd(igrid)%leafPE_jz(n) = (1+gd(igrid)%iye(n)-gd(igrid)%iyb(n))+  &
              gd(igrid)%leafPE_ja(n)-1
      else
         gd(igrid)%leafPE_ye(n) = gd(igrid)%iye(n)+nbnd3
         gd(igrid)%leafPE_jz(n) = (gd(igrid)%iye(n)-gd(igrid)%iyb(n))+  &
              gd(igrid)%leafPE_ja(n)
      end if

   end do

   do n=1,n_PE_leaf
      gd(igrid)%leafPE_xy(n) =  &
           (gd(igrid)%leafPE_xe(n)-gd(igrid)%leafPE_xb(n)+1)*  &
           (gd(igrid)%leafPE_ye(n)-gd(igrid)%leafPE_yb(n)+1)
   end do

end do

end subroutine p_decomp_bounds


!```````````````````````````````````````````````````````````````````
subroutine determine_comm_paths

!+ Compute communication schedule for all PEs

use univ_kind_defs_mod, only: f4_B
use main_memory_mod, only: n_grids, ibnd, jbnd, gd, i_long_dt_lBC_comm,  &
     i_ac_dt_u_lBC_comm, i_ac_dt_v_lBC_comm, i_ac_dt_pp_lBC_comm,  &
     i_c_to_f_grid_comm, i_f_to_c_grid_comm, i_f_to_c_grid_info, n_PE_leaf,  &
     PE_rank, cyclic_paths_t, cyclic_paths_u, cyclic_paths_v,  &
     acoustic_lBC_sd_max_pB, dnscale_nBC_sd_max_pB, upscale_nBC_sd_max_pB,  &
     n_acoustic_lBC_optypes

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: igrid, nnn, idn, isn, i, j, idx_t, idx_u, idx_v, n_ij_t, n_ij_u,  &
     n_ij_v, self_t, self_u, self_v, ij, is, js, id, jd, root_n2, root_n3,  &
     n_subdomain_W, j1, j2, i1, i2, it, k1, k2, ng_f, ng_c
integer, dimension(3) :: is0_t, js0_t, is0_u, js0_u, is0_v, js0_v

!=== Executable statements:

do igrid=1,n_grids
   gd(igrid)%rx_paths_all(:,:,:) = 0; gd(igrid)%tx_paths_all(:,:,:,:) = 0
end do

do igrid=1,n_grids
   nnn = gd(igrid)%nxtnest
   do idn=1,n_PE_leaf     ! Destination PE
      do isn=1,n_PE_leaf    ! Source PE
 
         ! Long timestep overlap regions:
         
         if (isn /= idn .and.  &
              .NOT.(gd(igrid)%leafPE_xb(idn) > gd(igrid)%ixe(isn) .or.  &
              gd(igrid)%leafPE_yb(idn) > gd(igrid)%iye(isn) .or.   &
              gd(igrid)%leafPE_xe(idn) < gd(igrid)%ixb(isn) .or.  &
              gd(igrid)%leafPE_ye(idn) < gd(igrid)%iyb(isn))) then

            gd(igrid)%rx_paths_all(i_long_dt_lBC_comm,isn,idn) = PE_rank(isn)

            gd(igrid)%tx_paths_all(1,i_long_dt_lBC_comm,idn,isn) =  &
                 max(gd(igrid)%ixb(isn), gd(igrid)%leafPE_xb(idn))
            gd(igrid)%tx_paths_all(2,i_long_dt_lBC_comm,idn,isn) =  &
                 min(gd(igrid)%ixe(isn), gd(igrid)%leafPE_xe(idn))
            gd(igrid)%tx_paths_all(3,i_long_dt_lBC_comm,idn,isn) =  &
                 max(gd(igrid)%iyb(isn), gd(igrid)%leafPE_yb(idn))
            gd(igrid)%tx_paths_all(4,i_long_dt_lBC_comm,idn,isn) =  &
                 min(gd(igrid)%iye(isn), gd(igrid)%leafPE_ye(idn))
            gd(igrid)%tx_paths_all(5,i_long_dt_lBC_comm,idn,isn) = PE_rank(idn)
            
            ! Expand 'tx_paths_all' to include (coarse) grid external boundary
            !  points:
            
!cb               if (nnn .eq. 0) then
            if (gd(igrid)%tx_paths_all(1,i_long_dt_lBC_comm,idn,isn) == 2)  &
                 gd(igrid)%tx_paths_all(1,i_long_dt_lBC_comm,idn,isn) = 1
            if (gd(igrid)%tx_paths_all(2,i_long_dt_lBC_comm,idn,isn) == gd(igrid)%nxp-1)  &
                 gd(igrid)%tx_paths_all(2,i_long_dt_lBC_comm,idn,isn) = gd(igrid)%nxp
            if (gd(igrid)%tx_paths_all(3,i_long_dt_lBC_comm,idn,isn) == 2)  &
                 gd(igrid)%tx_paths_all(3,i_long_dt_lBC_comm,idn,isn) = 1
            if (gd(igrid)%tx_paths_all(4,i_long_dt_lBC_comm,idn,isn) == gd(igrid)%nyp-1)  &
                 gd(igrid)%tx_paths_all(4,i_long_dt_lBC_comm,idn,isn) = gd(igrid)%nyp
!cb               endif

            ! Acoustic timestep overlap regions for U:
            
            if ((gd(igrid)%ixb(idn)-1 == gd(igrid)%ixe(isn)) .and.  &
                 (gd(igrid)%iye(idn) >= gd(igrid)%iyb(isn)) .and.  &
                 (gd(igrid)%iyb(idn) <= gd(igrid)%iye(isn))) then
               
               gd(igrid)%rx_paths_all(i_ac_dt_u_lBC_comm,isn,idn) = PE_rank(isn)
               
               gd(igrid)%tx_paths_all(1,i_ac_dt_u_lBC_comm,idn,isn) =  &
                    gd(igrid)%ixe(isn)
               gd(igrid)%tx_paths_all(2,i_ac_dt_u_lBC_comm,idn,isn) =  &
                    gd(igrid)%ixe(isn)
               gd(igrid)%tx_paths_all(3,i_ac_dt_u_lBC_comm,idn,isn) =  &
                    max(gd(igrid)%iyb(isn), gd(igrid)%iyb(idn))
               gd(igrid)%tx_paths_all(4,i_ac_dt_u_lBC_comm,idn,isn) =  &
                    min(gd(igrid)%iye(isn), gd(igrid)%iye(idn))
               gd(igrid)%tx_paths_all(5,i_ac_dt_u_lBC_comm,idn,isn) =  &
                    PE_rank(idn)
               
            end if

            ! Acoustic timestep overlap regions for V:

            if(gd(igrid)%iyb(idn)-1 == gd(igrid)%iye(isn) .and.  &
                 gd(igrid)%ixe(idn) >= gd(igrid)%ixb(isn) .and.  &
                 gd(igrid)%ixb(idn) <= gd(igrid)%ixe(isn)) then
            
               gd(igrid)%rx_paths_all(i_ac_dt_v_lBC_comm,isn,idn) = PE_rank(isn)
               
               gd(igrid)%tx_paths_all(1,i_ac_dt_v_lBC_comm,idn,isn) =  &
                    max(gd(igrid)%ixb(isn), gd(igrid)%ixb(idn))
               gd(igrid)%tx_paths_all(2,i_ac_dt_v_lBC_comm,idn,isn) =  &
                    min(gd(igrid)%ixe(isn), gd(igrid)%ixe(idn))
               gd(igrid)%tx_paths_all(3,i_ac_dt_v_lBC_comm,idn,isn) =  &
                    gd(igrid)%iye(isn)
               gd(igrid)%tx_paths_all(4,i_ac_dt_v_lBC_comm,idn,isn) =  &
                    gd(igrid)%iye(isn)
               gd(igrid)%tx_paths_all(5,i_ac_dt_v_lBC_comm,idn,isn) =  &
                    PE_rank(idn)
            
            end if

            ! Acoustic timestep overlap regions for PI':

            if ((gd(igrid)%ixe(idn)+1 == gd(igrid)%ixb(isn)) .and.  &
                 (gd(igrid)%iye(idn) >= gd(igrid)%iyb(isn)) .and.  &
                 (gd(igrid)%iyb(idn) <= gd(igrid)%iye(isn))) then
            
               gd(igrid)%rx_paths_all(i_ac_dt_pp_lBC_comm,isn,idn) =  &
                    PE_rank(isn)
               
               gd(igrid)%tx_paths_all(1,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    gd(igrid)%ixb(isn)
               gd(igrid)%tx_paths_all(2,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    gd(igrid)%ixb(isn)
               gd(igrid)%tx_paths_all(3,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    max(gd(igrid)%iyb(isn), gd(igrid)%iyb(idn))
               gd(igrid)%tx_paths_all(4,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    min(gd(igrid)%iye(isn), gd(igrid)%iye(idn))
               gd(igrid)%tx_paths_all(5,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    PE_rank(idn)
            
            else if ((gd(igrid)%iye(idn)+1 == gd(igrid)%iyb(isn)) .and.  &
                 (gd(igrid)%ixe(idn) >= gd(igrid)%ixb(isn)) .and.  &
                 (gd(igrid)%ixb(idn) <= gd(igrid)%ixe(isn))) then
               
               gd(igrid)%rx_paths_all(i_ac_dt_pp_lBC_comm,isn,idn) =  &
                    PE_rank(isn)
               
               gd(igrid)%tx_paths_all(1,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    max(gd(igrid)%ixb(isn), gd(igrid)%ixb(idn))
               gd(igrid)%tx_paths_all(2,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    min(gd(igrid)%ixe(isn), gd(igrid)%ixe(idn))
               gd(igrid)%tx_paths_all(3,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    gd(igrid)%iyb(isn)
               gd(igrid)%tx_paths_all(4,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    gd(igrid)%iyb(isn)
               gd(igrid)%tx_paths_all(5,i_ac_dt_pp_lBC_comm,idn,isn) =  &
                    PE_rank(idn)

            end if

         end if

         ! Coarse grid to fine grid interpolation communication:

         if (nnn /= 0) then

            if (.NOT.(gd(igrid)%ipm(gd(igrid)%ixb(idn)-1)-2 > gd(nnn)%ixe(isn) .or.  &
                 gd(igrid)%jpm(gd(igrid)%iyb(idn)-1)-2 > gd(nnn)%iye(isn) .or.  &
                 gd(igrid)%ipm(gd(igrid)%ixe(idn)+1)+1 < gd(nnn)%ixb(isn) .or.  &
                 gd(igrid)%jpm(gd(igrid)%iye(idn)+1)+1 < gd(nnn)%iyb(isn))) then
         
               gd(igrid)%rx_paths_all(i_c_to_f_grid_comm,isn,idn) = PE_rank(isn)
               
               gd(igrid)%tx_paths_all(1,i_c_to_f_grid_comm,idn,isn) =  &
                    max(gd(nnn)%ixb(isn), gd(igrid)%ipm(gd(igrid)%ixb(idn)-1)-2)
               gd(igrid)%tx_paths_all(2,i_c_to_f_grid_comm,idn,isn) =  &
                    min(gd(nnn)%ixe(isn), gd(igrid)%ipm(gd(igrid)%ixe(idn)+1)+1)
               gd(igrid)%tx_paths_all(3,i_c_to_f_grid_comm,idn,isn) =  &
                    max(gd(nnn)%iyb(isn), gd(igrid)%jpm(gd(igrid)%iyb(idn)-1)-2)
               gd(igrid)%tx_paths_all(4,i_c_to_f_grid_comm,idn,isn) =  &
                    min(gd(nnn)%iye(isn), gd(igrid)%jpm(gd(igrid)%iye(idn)+1)+1)
               gd(igrid)%tx_paths_all(5,i_c_to_f_grid_comm,idn,isn) =  &
                    PE_rank(idn)

            end if

            ! Fine grid to coarse grid averaging communication:

!c rtimh - micphys: in the following lines: change ixb, ixe, iyb, and iye
!c to nxbeg, nxend, nybeg, and nyend on coarse grid destination leaf PEs
!c in order to keep internal CG boundaries up to date, since they have
!c already been updated by their CG neighboring leaf PEs??

            if (.NOT.(gd(nnn)%leafPE_xb(idn) > gd(igrid)%ipm(gd(igrid)%ixe(isn)) .or.  &
                 gd(nnn)%leafPE_yb(idn) > gd(igrid)%jpm(gd(igrid)%iye(isn)) .or.  &
                 gd(nnn)%leafPE_xe(idn) < gd(igrid)%ipm(gd(igrid)%ixb(isn)) .or.  &
                 gd(nnn)%leafPE_ye(idn) < gd(igrid)%jpm(gd(igrid)%iyb(isn)))) then
         
               gd(igrid)%rx_paths_all(i_f_to_c_grid_comm,isn,idn) = PE_rank(isn)
            
               gd(igrid)%tx_paths_all(1,i_f_to_c_grid_comm,idn,isn) =  &
                    max(gd(igrid)%ipm(gd(igrid)%ixb(isn)),  &
                    gd(nnn)%leafPE_xb(idn))
               gd(igrid)%tx_paths_all(2,i_f_to_c_grid_comm,idn,isn) =  &
                    min(gd(igrid)%ipm(gd(igrid)%ixe(isn)),  &
                    gd(nnn)%leafPE_xe(idn))
               gd(igrid)%tx_paths_all(3,i_f_to_c_grid_comm,idn,isn) =  &
                    max(gd(igrid)%jpm(gd(igrid)%iyb(isn)),  &
                    gd(nnn)%leafPE_yb(idn))
               gd(igrid)%tx_paths_all(4,i_f_to_c_grid_comm,idn,isn) =  &
                    min(gd(igrid)%jpm(gd(igrid)%iye(isn)),  &
                    gd(nnn)%leafPE_ye(idn))
               gd(igrid)%tx_paths_all(5,i_f_to_c_grid_comm,idn,isn) =  &
                    PE_rank(idn)
         
! A second index value of 7 of the tx_paths_all array is used to determine
! the loop limits in fdbackp for averaging the fm over the overlap
! between the cm leaf PE and fm leaf PE, rather than always over the full
! fm leaf PE.  It is not used for actually sending stuff.  The
! tx_paths_all(*,6,*,*,*) part of the array is still used for sending the
! block of averaged cm points from the fm leaf PE to the cm leaf PE.

               do i=gd(igrid)%ixb(isn),gd(igrid)%ixe(isn)
                  if (gd(igrid)%ipm(i) == gd(igrid)%tx_paths_all(1,6,idn,isn)) then
                     gd(igrid)%tx_paths_all(1,i_f_to_c_grid_info,idn,isn) = i
                     exit   ! Exit this loop
                  end if
               end do
               
               do i=gd(igrid)%ixe(isn),gd(igrid)%ixb(isn),-1
                  if (gd(igrid)%ipm(i) == gd(igrid)%tx_paths_all(2,6,idn,isn)) then
                     gd(igrid)%tx_paths_all(2,i_f_to_c_grid_info,idn,isn) = i
                     exit   ! Exit this loop
                  end if
               end do
               
               do j=gd(igrid)%iyb(isn),gd(igrid)%iye(isn)
                  if (gd(igrid)%jpm(j) == gd(igrid)%tx_paths_all(3,6,idn,isn)) then
                     gd(igrid)%tx_paths_all(3,i_f_to_c_grid_info,idn,isn) = j
                     exit   ! Exit this loop
                  end if
               end do
               
               do j=gd(igrid)%iye(isn),gd(igrid)%iyb(isn),-1
                  if (gd(igrid)%jpm(j) == gd(igrid)%tx_paths_all(4,6,idn,isn)) then
                     gd(igrid)%tx_paths_all(4,i_f_to_c_grid_info,idn,isn) = j
                     exit   ! Exit this loop
                  end if
               end do
         
            end if

         end if

      end do
   end do
end do

!=== Determine maximum subdomain byte size for the acoustic routine's tx/rx
!     buffers:

acoustic_lBC_sd_max_pB = -999
do isn=1,n_PE_leaf
   do idn=1,n_PE_leaf
      do igrid=1,n_grids
         do it=1,n_acoustic_lBC_optypes
            if (gd(igrid)%tx_paths_all(1,it,idn,isn) /= 0) then
               i1 = gd(igrid)%tx_paths_all(1,it,idn,isn)
               i2 = gd(igrid)%tx_paths_all(2,it,idn,isn)
               j1 = gd(igrid)%tx_paths_all(3,it,idn,isn)
               j2 = gd(igrid)%tx_paths_all(4,it,idn,isn)
            
               n_subdomain_W = (i2-i1+1)*(j2-j1+1)*gd(igrid)%nzp
            
               acoustic_lBC_sd_max_pB =  &
                    max(acoustic_lBC_sd_max_pB, n_subdomain_W*f4_B)
            end if
         end do
      end do
   end do
end do

!=== Determine maximum subdomain byte sizes for the nesting BC routines' tx/rx
!     buffers:

dnscale_nBC_sd_max_pB = -999
do isn=1,n_PE_leaf
   do idn=1,n_PE_leaf
      do ng_c=1,n_grids
         do ng_f=1,n_grids
            if (gd(ng_f)%tx_paths_all(1,i_c_to_f_grid_comm,idn,isn) /= 0) then
               i1 = gd(ng_f)%tx_paths_all(1,i_c_to_f_grid_comm,idn,isn)
               i2 = gd(ng_f)%tx_paths_all(2,i_c_to_f_grid_comm,idn,isn)
               j1 = gd(ng_f)%tx_paths_all(3,i_c_to_f_grid_comm,idn,isn)
               j2 = gd(ng_f)%tx_paths_all(4,i_c_to_f_grid_comm,idn,isn)

               k1 = 1
               k2 = gd(1)%nzp ! mmzp(icm)

               n_subdomain_W = (i2-i1+1)*(j2-j1+1)*(k2-k1+1)

               dnscale_nBC_sd_max_pB =  &
                    max(dnscale_nBC_sd_max_pB, n_subdomain_W*f4_B)
            end if
         end do
      end do
   end do
end do

upscale_nBC_sd_max_pB = -999
do isn=1,n_PE_leaf
   do idn=1,n_PE_leaf
      do ng_c=1,n_grids
         do ng_f=1,n_grids
            if (gd(ng_f)%tx_paths_all(1,i_f_to_c_grid_comm,idn,isn) /= 0) then
               i1 = gd(ng_f)%tx_paths_all(1,i_f_to_c_grid_comm,idn,isn)
               i2 = gd(ng_f)%tx_paths_all(2,i_f_to_c_grid_comm,idn,isn)
               j1 = gd(ng_f)%tx_paths_all(3,i_f_to_c_grid_comm,idn,isn)
               j2 = gd(ng_f)%tx_paths_all(4,i_f_to_c_grid_comm,idn,isn)
               
               k1 = 2  ! kpm(2,ifm)
               k2 = gd(1)%nzp-1  ! kpm(mmzp(ifm)-1,ifm)

               n_subdomain_W = (i2-i1+1)*(j2-j1+1)*(k2-k1+1)

               upscale_nBC_sd_max_pB =  &
                    max(upscale_nBC_sd_max_pB, n_subdomain_W*f4_B)
            end if
         end do
      end do
   end do
end do

!=== Cyclic boundary conditions (grid 1 only):

idx_t = 0; idx_u = 0; idx_v = 0
root_n2 = gd(1)%nxp; root_n3 = gd(1)%nyp


do jd=1,root_n3      ! Loop over all destination ij columns:
   do id=1,root_n2   !

      if (id <= 2 .or. id >= root_n2-1 .or. jd <= 2 .or. jd >= root_n3-1) then

         ! Determine 1, 2, or 3 source ij columns for this destination ij
         !  column:

         n_ij_t = 0; n_ij_u = 0; n_ij_v = 0

         if (ibnd == 4) then

            ! "T-pts":
            if (id <= 2 .and. jd >= 2 .and. jd <= root_n3-1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id+root_n2-3; js0_t(n_ij_t) = jd
            else if (id >= root_n2-1 .and. jd >= 2 .and. jd <= root_n3-1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id-root_n2+3; js0_t(n_ij_t) = jd
            end if

            ! U-pts:
            if (id == 1 .and. jd >= 2 .and. jd <= root_n3-1) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id+root_n2-3; js0_u(n_ij_u) = jd
            else if (id == root_n2-1 .and. jd >= 2 .and. jd <= root_n3-1) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id-root_n2+3; js0_u(n_ij_u) = jd
            end if

            ! V-pts:
            if (id <= 2 .and. jd >= 2 .and. jd < root_n3-1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id+root_n2-3; js0_v(n_ij_v) = jd
            else if (id >= root_n2-1 .and. jd >= 2 .and. jd < root_n3-1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id-root_n2+3; js0_v(n_ij_v) = jd
            end if

         end if
          
         if (jbnd == 4) then

            ! "T-pts":
            if (jd <= 2 .and. id >= 2 .and. id <= root_n2-1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id; js0_t(n_ij_t) = jd+root_n3-3
            else if (jd >= root_n3-1 .and. id >= 2 .and. id <= root_n2-1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id; js0_t(n_ij_t) = jd-root_n3+3
            end if

            ! U-pts:
            if (jd <= 2 .and. id >= 2 .and. id < root_n2-1) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id; js0_u(n_ij_u) = jd+root_n3-3
            else if (jd >= root_n3-1 .and. id >= 2 .and. id < root_n2-1) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id; js0_u(n_ij_u) = jd-root_n3+3
            end if

            ! V-pts:
            if (jd == 1 .and. id >= 2 .and. id <= root_n2-1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id; js0_v(n_ij_v) = jd+root_n3-3
            else if (jd == root_n3-1 .and. id >= 2 .and. id <= root_n2-1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id; js0_v(n_ij_v) = jd-root_n3+3
            end if

         end if
          
         if (ibnd == 4 .and. jbnd == 4) then   ! group of 2x2 pts each corner

            ! "T-pts":
            if (id <= 2 .and. jd <= 2) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id+root_n2-3; js0_t(n_ij_t) = jd+root_n3-3
            else if (id >= root_n2-1 .and. jd <= 2) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id-root_n2+3; js0_t(n_ij_t) = jd+root_n3-3
            else if (id <= 2 .and. jd >= root_n3-1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id+root_n2-3; js0_t(n_ij_t) = jd-root_n3+3
            else if (id >= root_n2-1 .and. jd >= root_n3-1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id-root_n2+3; js0_t(n_ij_t) = jd-root_n3+3
            end if

            ! U-pts:
            if (id == 1 .and. jd <= 2) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id+root_n2-3; js0_u(n_ij_u) = jd+root_n3-3
            else if (id == root_n2-1 .and. jd <= 2) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id-root_n2+3; js0_u(n_ij_u) = jd+root_n3-3
            else if (id == 1 .and. jd >= root_n3-1) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id+root_n2-3; js0_u(n_ij_u) = jd-root_n3+3
            else if (id == root_n2-1 .and. jd >= root_n3-1) then
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id-root_n2+3; js0_u(n_ij_u) = jd-root_n3+3
            end if

            ! V-pts:
            if (id <= 2 .and. jd == 1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id+root_n2-3; js0_v(n_ij_v) = jd+root_n3-3
            else if (id >= root_n2-1 .and. jd == 1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id-root_n2+3; js0_v(n_ij_v) = jd+root_n3-3
            else if (id <= 2 .and. jd == root_n3-1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id+root_n2-3; js0_v(n_ij_v) = jd-root_n3+3
            else if (id >= root_n2-1 .and. jd == root_n3-1) then
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id-root_n2+3; js0_v(n_ij_v) = jd-root_n3+3
            end if

         else if (ibnd == 4) then                ! single pt each corner

            ! "T-pts", U-pts, V-pts:
            if (id == 1 .and. (jd == 1 .or. jd == root_n3)) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id+root_n2-3; js0_t(n_ij_t) = jd
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id+root_n2-3; js0_u(n_ij_u) = jd
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id+root_n2-3; js0_v(n_ij_v) = jd
            else if (id == root_n2 .and. (jd == 1 .or. jd == root_n3)) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id-root_n2+3; js0_t(n_ij_t) = jd
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id-root_n2+3; js0_u(n_ij_u) = jd
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id-root_n2+3; js0_v(n_ij_v) = jd
            end if

         else if (jbnd == 4) then                ! single pt each corner

            ! "T-pts", U-pts, V-pts:
            if ((id == 1 .or. id == root_n2) .and. jd == 1) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id; js0_t(n_ij_t) = jd+root_n3-3
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id; js0_u(n_ij_u) = jd+root_n3-3
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id; js0_v(n_ij_v) = jd+root_n3-3
            else if ((id == 1 .or. id == root_n2) .and. jd == root_n3) then
               n_ij_t = n_ij_t+1
               is0_t(n_ij_t) = id; js0_t(n_ij_t) = jd-root_n3+3
               n_ij_u = n_ij_u+1
               is0_u(n_ij_u) = id; js0_u(n_ij_u) = jd-root_n3+3
               n_ij_v = n_ij_v+1
               is0_v(n_ij_v) = id; js0_v(n_ij_v) = jd-root_n3+3
            end if

         end if

         if (id == 1 .or. id == root_n2 .or. jd == 1 .or. jd == root_n3) then
            self_t = 0; self_u = 0; self_v = 0
         else if (id == root_n2 - 1) then
            self_t = 1; self_u = 0; self_v = 1
            if (jd == root_n3 - 1) self_v = 0
         else if (jd == root_n3 - 1) then
            self_t = 1; self_u = 1; self_v = 0
         else
            self_t = 1; self_u = 1; self_v = 1
         end if
                  
         ! Loop over each destination leaf PE and check if the current ij
         !  destination column is in its subdomain:

         do idn=1,n_PE_leaf     ! Destination PE
            if (id >= gd(1)%leafPE_xb(idn) .and. id <= gd(1)%leafPE_xe(idn) .and. &
                 jd >= gd(1)%leafPE_yb(idn) .and. jd <= gd(1)%leafPE_ye(idn)) then
                
               ! Loop over up to 3 source ij columns:

               do ij=1,n_ij_t
                  is = is0_t(ij); js = js0_t(ij)

                  ! Loop over each source leaf PE and check if current ij
                  !  source column is in its subdomain:

                  do isn=1,n_PE_leaf    ! Source PE
                     if (is >= gd(1)%ixb(isn) .and. is <= gd(1)%ixe(isn) .and.  &
                          js >= gd(1)%iyb(isn) .and. js <= gd(1)%iye(isn)) then
                
                        idx_t = idx_t+1
                        cyclic_paths_t(1,idx_t) = isn    ! source leaf PE
                        cyclic_paths_t(2,idx_t) = is     ! source leaf PE i
                        cyclic_paths_t(3,idx_t) = js     ! source leaf PE j
                        cyclic_paths_t(4,idx_t) = idn    ! destination leaf PE
                        cyclic_paths_t(5,idx_t) = id     ! destination leaf PE i
                        cyclic_paths_t(6,idx_t) = jd     ! destination leaf PE j
                        cyclic_paths_t(7,idx_t) = self_t ! self count
                        cyclic_paths_t(8,idx_t) = n_ij_t ! other point count

                     end if
                  end do

               end do

               do ij=1,n_ij_u
                  is = is0_u(ij); js = js0_u(ij)

                  ! Loop over each source leaf PE and check if current ij
                  !  source column is in its subdomain:

                  do isn = 1,n_PE_leaf    ! Source PE
                     if (is >= gd(1)%ixb(isn) .and. is <= gd(1)%ixe(isn) .and.  &
                          js >= gd(1)%iyb(isn) .and. js <= gd(1)%iye(isn)) then

                        idx_u = idx_u+1
                        cyclic_paths_u(1,idx_u) = isn    ! source leaf PE
                        cyclic_paths_u(2,idx_u) = is     ! source leaf PE i
                        cyclic_paths_u(3,idx_u) = js     ! source leaf PE j
                        cyclic_paths_u(4,idx_u) = idn    ! destination leaf PE
                        cyclic_paths_u(5,idx_u) = id     ! destination leaf PE i
                        cyclic_paths_u(6,idx_u) = jd     ! destination leaf PE j
                        cyclic_paths_u(7,idx_u) = self_u ! self count
                        cyclic_paths_u(8,idx_u) = n_ij_u ! other point count

                     end if
                  end do

               end do

               do ij=1,n_ij_v
                  is = is0_v(ij); js = js0_v(ij)

                  ! Loop over each source leaf PE and check if current ij
                  !  source column is in its subdomain:

                  do isn = 1,n_PE_leaf    ! Source PE
                     if (is >= gd(1)%ixb(isn) .and. is <= gd(1)%ixe(isn) .and.  &
                          js >= gd(1)%iyb(isn) .and. js <= gd(1)%iye(isn)) then

                        idx_v = idx_v+1
                        cyclic_paths_v(1,idx_v) = isn    ! source leaf PE
                        cyclic_paths_v(2,idx_v) = is     ! source leaf PE i
                        cyclic_paths_v(3,idx_v) = js     ! source leaf PE j
                        cyclic_paths_v(4,idx_v) = idn    ! destination leaf PE
                        cyclic_paths_v(5,idx_v) = id     ! destination leaf PE i
                        cyclic_paths_v(6,idx_v) = jd     ! destination leaf PE j
                        cyclic_paths_v(7,idx_v) = self_v ! self count
                        cyclic_paths_v(8,idx_v) = n_ij_v ! other point count

                     end if
                  end do

               end do

            end if
         end do

      end if

   end do
end do

end subroutine determine_comm_paths


!```````````````````````````````````````````````````````````````````
subroutine leafPE_domain_decomp

!+ Initializes parallel processing indices/control variables

use univ_kind_defs_mod, only: f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: n_grids, gd, n_PE_leaf

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'leafPE_domain_decomp'

real(f4), allocatable, dimension(:,:) :: work

integer :: n2, n3, a_err, igrid, n

!=== Executable statements:

! Decompose all grids into subdomains:

do igrid=1,n_grids
   n2 = gd(igrid)%nxp; n3 = gd(igrid)%nyp

   allocate(work(n2,n3), stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'A')

   ! Obtain estimates of the fraction of computational time (work) required
   !  for each column in the root domain:
   
   call p_decomp_est_work(work)

   ! Decompose this grid, taking into account the above fractional work values:

   call p_decomp(n_PE_leaf, work, igrid)

   write (*, '(/,a)') '     Root domain decomposition:'
   write (*, '(a)') '   igrid   PE#   ixb   ixe   iyb   iye   work_measure'
   do n=1,n_PE_leaf
      write (*, '(7i6)') igrid, n, gd(igrid)%ixb(n), gd(igrid)%ixe(n),  &
           gd(igrid)%iyb(n), gd(igrid)%iye(n),  &
           (1+gd(igrid)%ixe(n)-gd(igrid)%ixb(n))*(1+gd(igrid)%iye(n)-gd(igrid)%iyb(n))
   end do

   deallocate(work, stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'A')

end do

! Compute various boundary parameters for each of the leaf PE subdomains:

call p_decomp_bounds(gd(1:n_grids)%nxp, gd(1:n_grids)%nyp, 1, 1)

! Determine communication schedule(s) for all PEs:

call determine_comm_paths

end subroutine leafPE_domain_decomp


!```````````````````````````````````````````````````````````````````
subroutine check_leafPE_and_rootPE_comm(all_results_from_leafPE_now,  &
     all_fields_from_rootPE_now)

!+ Determines whether (and what info) leaf PEs should transmit to the root PE
!   at the end of a long timestep

use univ_kind_defs_mod, only: f8, f4
use main_memory_mod, only: vtime, n_varf, wfreq_state_file, epsilon_sim_ET,  &
     ioutput, initialization_type, current_sim_ET, maximum_sim_ET, gd

implicit none

!=== Argument declarations:

logical, intent(OUT) :: all_results_from_leafPE_now,  &
     all_fields_from_rootPE_now

!== Local declarations:

real(f8) :: timemf

!=== Executable statements:

timemf = current_sim_ET+real(gd(1)%dt_dyn, kind=f8)

! Check if state file output is due:

if (ioutput /= 0) then
   if (mod(real(timemf, kind=f4), wfreq_state_file) < gd(1)%dt_dyn)  &
        all_results_from_leafPE_now = .TRUE.
end if

! Check if run is finished:

if (timemf >= maximum_sim_ET-epsilon_sim_ET)  &
     all_results_from_leafPE_now = .TRUE.

! Check if varfile fields need to be updated:

if (initialization_type == 2) then
   if (real(timemf, kind=f4) >= vtime(n_varf+1) .and.  &
        timemf < maximum_sim_ET) then
      all_fields_from_rootPE_now = .TRUE.
      all_results_from_leafPE_now = .TRUE.
   end if
end if

end subroutine check_leafPE_and_rootPE_comm


end module parallel_core_mod
