!```````````````````````````````````````````````````````````````````
module parallel_init_vars_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_IDs(task_IDs, tx_bp)

!+ Transmits basic parallel element (PE) identification info to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_deallocation
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_check_buffer_packed,  &
     mpiq_pack_buffer, mpiq_transmit_packed, mpiq_transmit_plain,  &
     MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL
use main_memory_mod, only:  rootPE_rank, n_PE_leaf, PE_rank

implicit none

!=== Argument declarations:

integer, dimension(:), intent(IN) :: task_IDs

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'rootPE_put_IDs'

integer :: n, n_packed_B

!=== Executable statements:

#if defined(USE_MPI)

!-- Consistency check; fill module array 'PE_rank':

if (size(task_IDs) /= n_PE_leaf) call exception(procedure_name, FATAL,  &
     'size of task_IDS array inconsistent.')

PE_rank(:) = task_IDs(:)

!-- First, transmit a short message to all leaf PEs, providing them with
!    the rank of their root PE:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(rootPE_rank, PE_rank(n))  ! Transmit message to PE
end do

!-- Now perform other messaging tasks:

! Bytecount of primary message:

n_packed_B = i4_B*(3 + 1*n_PE_leaf) +  &
             i8_B*0 +  &
             f4_B*0 +  &
             f8_B*0 +  &
             cn_B*0 +  &
             bn_B*0

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

! Primary message:

! INTEGER (4-byte) data:

do n=1,n_PE_leaf
   call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

   call mpiq_pack_buffer(PE_rank(n), tx_bp)       
   call mpiq_pack_buffer(n, tx_bp)             ! Fill transmit buffer with data
   call mpiq_pack_buffer(n_PE_leaf, tx_bp)             
   call mpiq_pack_buffer(PE_rank, n_PE_leaf, tx_bp)

   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

!-- Output some useful (and human-readable) information:

do n=1,n_PE_leaf
   write (*, '(a,i3,a,i3,a,i3)') 'leaf PE #', n, ': rank = ', PE_rank(n),  &
        ', rank of root PE = ', rootPE_rank 
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_IDs


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_nlst_vars(tx_bp)

!+ Transmits relevant namelist info to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: initialization_type, start_UT, n_grids,  &
     map_projection, n_dta_per_dtdyn, dt_mode, nestz1, nudlat,  &
     ioutput, naddsc, icorflg, ibnd, jbnd, lsflg, nfpt, islastog, npatch,  &
     ihorgrad, nzg, nzs, nstratz1, maxnzp, target_Ls, deltax_g1, deltay_g1,  &
     deltaz_g1, dzrat, dzmax, polelat, polelon, tnudlat, tnudcent, tnudtop,  &
     znudtop, wfreq_state_file, cphas, emissivity, diurnal_progression,  &
     distim, radfrq, zrough, albedo, zz, maxnnp, current_sim_ET, TI_val,  &
     gd, start_sim_ET, dt_max_frac_increase_g1, maximum_sim_ET, min_ssfrac,  &
     n_substeps_max, enable_advection, enable_turb_diffusion,  &
     enable_aerosol_microphysics, RT_paradigm, prognose_SBPS, prognose_uvwp,  &
     max_dt_dynamics, default_input_data_dir, n_isched_steps, n_PE_leaf,  &
     PE_rank, impl, iadvl, iadvf, izflat, orbital_progression
use main_memory_bspec_mod, only: dust_nb, H2Oi_nb, CO2i_nb, dust_tog,  &
     H2Ov_tog, H2Oi_tog, CO2i_tog, dust_rmn, dust_mrt, H2Oi_rmn, H2Oi_mrt,  &
     CO2i_rmn, lfmcut, nrdlevt, nslcon, lonrad, ishadtog, CO2i_mrt, dust_ivr,  &
     dust_rta, H2Oi_rta, CO2i_rta, rtc_tog, bdst_igd, bdst_imr, bdst_tog,  &
     bdst_icn, bdst_irt, bdst_iht, bdst_itt, CO2_sfci, dust_icn, dust_igd,  &
     dust_iht, dust_imr, dust_inn, dust_irt, dust_itt, dust_jef, dust_jth,  &
     H2Ov_ivr, sfci_ils, conrnu, tautot, pctlcon, Ls_of_periapse, slmstr,  &
     orbital_eccentricity, axial_obliquity, AU_to_m_knob, slz, stgoff,  &
     RT_flux_trace_np, use_gas_opacities, dust_op_case, dust_ctau_refp
use aermic_rad_mem_mod, only: iaertog, aer_rad_gpbeg, aer_gpbeg, aer_ngrp,  &
     aer_csr_gpbeg, aer_inj_gpbeg, aer_rvbeg
use namelist_bspec_mod, only: default_input_data_dir_bspec

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, n, igd

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

n_packed_B = i4_B*(47 + 12*n_grids + maxnzp) +  &
             i8_B*0 +  &
             f4_B*(54 + 1*maxnzp + 9*n_grids + 3*nzg) +  &
             f8_B*(4 + 4*aer_ngrp) +  &
             cn_B*(0 + 1*len(default_input_data_dir) +  &
                   1*len(default_input_data_dir_bspec)) +  &
             bn_B*(11)

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

! INTEGER (4-byte) data:

call mpiq_pack_buffer(n_grids, tx_bp)          ! Fill transmit buffer with data
call mpiq_pack_buffer(map_projection, tx_bp)
call mpiq_pack_buffer(initialization_type, tx_bp)
call mpiq_pack_buffer(dt_mode, tx_bp)
call mpiq_pack_buffer(n_dta_per_dtdyn, tx_bp)
call mpiq_pack_buffer(n_substeps_max, tx_bp)  
call mpiq_pack_buffer(nzg, tx_bp)
call mpiq_pack_buffer(nzs, tx_bp)
call mpiq_pack_buffer(nestz1, tx_bp)
call mpiq_pack_buffer(ioutput, tx_bp)
call mpiq_pack_buffer(nudlat, tx_bp)
!... pnLES_np: ???
call mpiq_pack_buffer(naddsc, tx_bp)
call mpiq_pack_buffer(icorflg, tx_bp)
call mpiq_pack_buffer(RT_paradigm, tx_bp)
call mpiq_pack_buffer(ibnd, tx_bp)
call mpiq_pack_buffer(jbnd, tx_bp)
call mpiq_pack_buffer(lsflg, tx_bp)
call mpiq_pack_buffer(nfpt, tx_bp)
call mpiq_pack_buffer(islastog, tx_bp)
call mpiq_pack_buffer(ihorgrad, tx_bp)
call mpiq_pack_buffer(npatch, tx_bp)
call mpiq_pack_buffer(CO2_sfci, tx_bp)
call mpiq_pack_buffer(nslcon, tx_bp)
call mpiq_pack_buffer(rtc_tog, tx_bp)
call mpiq_pack_buffer(lonrad, tx_bp)
call mpiq_pack_buffer(ishadtog, tx_bp)
call mpiq_pack_buffer(RT_flux_trace_np, tx_bp)
call mpiq_pack_buffer(lfmcut, tx_bp)
call mpiq_pack_buffer(nrdlevt, tx_bp)
call mpiq_pack_buffer(dust_op_case, tx_bp)
call mpiq_pack_buffer(bdst_tog, tx_bp)
call mpiq_pack_buffer(H2Ov_tog, tx_bp)
call mpiq_pack_buffer(H2Ov_ivr, tx_bp)
call mpiq_pack_buffer(dust_tog, tx_bp)
call mpiq_pack_buffer(dust_nb, tx_bp)
call mpiq_pack_buffer(dust_ivr, tx_bp)
call mpiq_pack_buffer(H2Oi_tog, tx_bp)
call mpiq_pack_buffer(H2Oi_nb, tx_bp)
call mpiq_pack_buffer(CO2i_tog, tx_bp)
call mpiq_pack_buffer(CO2i_nb, tx_bp)
!... nsndp: not needed by leafPEs
!... ipsflg: not needed by leafPEs
!... itsflg: not needed by leafPEs
!... irtsflg: not needed by leafPEs
!... iusflg: not needed by leafPEs

call mpiq_pack_buffer(impl, tx_bp)   ! Relevant "static" namelist parameters
call mpiq_pack_buffer(izflat, tx_bp) !
call mpiq_pack_buffer(iadvl, tx_bp)  !
call mpiq_pack_buffer(iadvf, tx_bp)  !

call mpiq_pack_buffer(iaertog, tx_bp) ! Not in namelist
call mpiq_pack_buffer(maxnnp, tx_bp) ! Not in namelist
call mpiq_pack_buffer(n_isched_steps, tx_bp) ! Not in namelist

do igd=1,n_grids
   !... gd(:)%n_dt_per_pgdt: effectively given to leafPEs in rootPE_put_dtsched
   call mpiq_pack_buffer(gd(igd)%nxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nxtnest, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nstratx, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nstraty, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nknest, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nsttop, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nstbot, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ninest, tx_bp)
   call mpiq_pack_buffer(gd(igd)%njnest, tx_bp)
   !... gd(:)%itoptflg: not needed by leafPEs
   !... gd(:)%itopsflg: not needed by leafPEs
   !... gd(:)%itinflg: not needed by leafPEs
   !... gd(:)%ialbflg: not needed by leafPEs
   !... gd(:)%iz0flg: not needed by leafPEs
   call mpiq_pack_buffer(gd(igd)%idiffk, tx_bp)
   !... gd(:)%nofilflg: not needed by leafPEs
end do

call mpiq_pack_buffer(nstratz1, maxnzp, tx_bp)

! REAL (4-byte) data:

call mpiq_pack_buffer(target_Ls, tx_bp)
call mpiq_pack_buffer(start_UT, tx_bp)
!... dt_dynamics: effectively given to leafPEs in rootPE_put_dtsched
call mpiq_pack_buffer(min_ssfrac, tx_bp)
call mpiq_pack_buffer(max_dt_dynamics, tx_bp)
call mpiq_pack_buffer(dt_max_frac_increase_g1, tx_bp)  
call mpiq_pack_buffer(deltax_g1, tx_bp)            
call mpiq_pack_buffer(deltay_g1, tx_bp)       ! Fill transmit buffer with data
call mpiq_pack_buffer(deltaz_g1, tx_bp)
call mpiq_pack_buffer(dzrat, tx_bp)
call mpiq_pack_buffer(dzmax, tx_bp)
call mpiq_pack_buffer(polelat, tx_bp)
call mpiq_pack_buffer(polelon, tx_bp)
call mpiq_pack_buffer(wfreq_state_file, tx_bp)
call mpiq_pack_buffer(tnudlat, tx_bp)
call mpiq_pack_buffer(tnudcent, tx_bp)
call mpiq_pack_buffer(tnudtop, tx_bp)
call mpiq_pack_buffer(znudtop, tx_bp)
!... z0fact: not needed by leafPEs
call mpiq_pack_buffer(cphas, tx_bp)
call mpiq_pack_buffer(distim, tx_bp)
call mpiq_pack_buffer(zrough, tx_bp)
call mpiq_pack_buffer(albedo, tx_bp)
call mpiq_pack_buffer(TI_val, tx_bp)
call mpiq_pack_buffer(emissivity, tx_bp)
call mpiq_pack_buffer(pctlcon, tx_bp)
call mpiq_pack_buffer(sfci_iLs, tx_bp)
call mpiq_pack_buffer(radfrq, tx_bp)
call mpiq_pack_buffer(Ls_of_periapse, tx_bp)
call mpiq_pack_buffer(orbital_eccentricity, tx_bp)
call mpiq_pack_buffer(axial_obliquity, tx_bp)
call mpiq_pack_buffer(AU_to_m_knob, tx_bp)
call mpiq_pack_buffer(tautot, tx_bp)
call mpiq_pack_buffer(conrnu, tx_bp)
call mpiq_pack_buffer(dust_ctau_refp, tx_bp)
call mpiq_pack_buffer(bdst_imr, tx_bp)
call mpiq_pack_buffer(bdst_igd, tx_bp)
call mpiq_pack_buffer(bdst_icn, tx_bp)
call mpiq_pack_buffer(bdst_irt, tx_bp)
call mpiq_pack_buffer(bdst_iht, tx_bp)
call mpiq_pack_buffer(bdst_itt, tx_bp)
call mpiq_pack_buffer(dust_rmn, tx_bp)
call mpiq_pack_buffer(dust_mrt, tx_bp)
call mpiq_pack_buffer(dust_jef, tx_bp)
call mpiq_pack_buffer(dust_jth, tx_bp)
call mpiq_pack_buffer(dust_igd, tx_bp)
call mpiq_pack_buffer(dust_imr, tx_bp)
call mpiq_pack_buffer(dust_inn, tx_bp)
call mpiq_pack_buffer(dust_icn, tx_bp)
call mpiq_pack_buffer(dust_irt, tx_bp)
call mpiq_pack_buffer(dust_iht, tx_bp)
call mpiq_pack_buffer(dust_itt, tx_bp)
call mpiq_pack_buffer(H2Oi_rmn, tx_bp)
call mpiq_pack_buffer(H2Oi_mrt, tx_bp)
call mpiq_pack_buffer(CO2i_rmn, tx_bp)
call mpiq_pack_buffer(CO2i_mrt, tx_bp)

call mpiq_pack_buffer(zz, maxnzp, tx_bp)

do igd=1,n_grids
   call mpiq_pack_buffer(gd(igd)%centlat, tx_bp)
   call mpiq_pack_buffer(gd(igd)%centlon, tx_bp)
   !... gd(:)%toptenh: not needed by leafPEs
   !... gd(:)%toptwvlx: not needed by leafPEs
   !... gd(:)%toptwvly: not needed by leafPEs
   !... gd(:)%tinwvlx: not needed by leafPEs
   !... gd(:)%tinwvly: not needed by leafPEs
   !... gd(:)%albwvlx: not needed by leafPEs
   !... gd(:)%albwvly: not needed by leafPEs
   !... gd(:)%z0max: not needed by leafPEs
   !... gd(:)%z0_of_topolevel: not needed by leafPEs
   !... gd(:)%topolevel_for_z0: not needed by leafPEs
   call mpiq_pack_buffer(gd(igd)%gridu, tx_bp)
   call mpiq_pack_buffer(gd(igd)%gridv, tx_bp)
   call mpiq_pack_buffer(gd(igd)%csx, tx_bp)
   call mpiq_pack_buffer(gd(igd)%csz, tx_bp)
   call mpiq_pack_buffer(gd(igd)%xkhkm, tx_bp)
   call mpiq_pack_buffer(gd(igd)%zkhkm, tx_bp)
   call mpiq_pack_buffer(gd(igd)%akmin, tx_bp)
end do

!... hsnd(nsndp): not needed by leafPEs
!... psnd(nsndp): not needed by leafPEs
!... tsnd(nsndp): not needed by leafPEs
!... rtsnd(nsndp): not needed by leafPEs
!... usnd(nsndp): not needed by leafPEs
!... vsnd(nsndp): not needed by leafPEs

call mpiq_pack_buffer(slz, nzg, tx_bp)
call mpiq_pack_buffer(stgoff, nzg, tx_bp)
call mpiq_pack_buffer(slmstr, nzg, tx_bp)

! REAL (8-byte) data:

call mpiq_pack_buffer(maximum_sim_ET, tx_bp)
!... dust_rtt: supplied by 'aer_rad_gpbeg'
!... H2Oi_rtt: supplied by 'aer_rad_gpbeg'
!... CO2i_rtt: supplied by 'aer_rad_gpbeg'
call mpiq_pack_buffer(aer_rvbeg, tx_bp)   ! H2Ov_beg (in namelist)
!... dust_beg: supplied by 'aer_gpbeg'
!... dust_jsr: supplied by 'aer_csr_gpbeg'
!... dust_jbg: supplied by 'aer_inj_gpbeg'
!... H2Oi_beg: supplied by 'aer_gpbeg'
!... CO2i_beg: supplied by 'aer_gpbeg'

call mpiq_pack_buffer(start_sim_ET, tx_bp)       ! 0. or time_rst (in namelist) 
call mpiq_pack_buffer(current_sim_ET, tx_bp)     ! not in namelist

call mpiq_pack_buffer(aer_rad_gpbeg, aer_ngrp, tx_bp)  ! *_rtt; dust, H2Oi, CO2i
call mpiq_pack_buffer(aer_gpbeg, aer_ngrp, tx_bp) ! dust_beg, H2Oi_beg, CO2i_beg
call mpiq_pack_buffer(aer_csr_gpbeg, aer_ngrp, tx_bp)  ! dust_jsr
call mpiq_pack_buffer(aer_inj_gpbeg, aer_ngrp, tx_bp)  ! dust_jbg

! CHARACTER (1-byte) data:

!... simulation_description: not needed by leafPEs
!... run_type: not needed by leafPEs
!... output_dir_root: not needed by leafPEs
!... simulation_ID: not needed by leafPEs
!... run_suffix: not needed by leafPEs
!... input_varfile_pfx: not needed by leafPEs
!... input_restart_file_pfx: not needed by leafPEs
call mpiq_pack_buffer(default_input_data_dir, len(default_input_data_dir),  &
     tx_bp)
!... input_sfcfile_pfx: not needed by leafPEs
!... pnLES_fn: ???
!... input_sounding_fn: not needed by leafPEs

!... pnLES_pt: ???

!... gd(:)%itoptfn: not needed by leafPEs
!... gd(:)%itinfn: not needed by leafPEs
!... gd(:)%ialbfn: not needed by leafPEs

call mpiq_pack_buffer(default_input_data_dir_bspec,  &  ! Not in namelist
     len(default_input_data_dir_bspec), tx_bp)

! LOGICAL data:

call mpiq_pack_buffer(orbital_progression, tx_bp)
call mpiq_pack_buffer(diurnal_progression, tx_bp)
!... stop_if_file_exists: not needed by leafPEs
!... grid_in_state_outp: not needed by leafPEs  ! Derived from state_file_grids
!... preserve_dt_on_restart: not needed by leafPEs
!... run_sfcinit_user: not needed by leafPEs
!... run_toptinit_user: not needed by leafPEs
call mpiq_pack_buffer(enable_advection, tx_bp)
call mpiq_pack_buffer(enable_turb_diffusion, tx_bp)
call mpiq_pack_buffer(enable_aerosol_microphysics, tx_bp)
call mpiq_pack_buffer(prognose_SBPS, tx_bp)
call mpiq_pack_buffer(prognose_uvwp, tx_bp)
!... perturb_nsfc_theta: ???
call mpiq_pack_buffer(use_gas_opacities, tx_bp)
call mpiq_pack_buffer(dust_rta, tx_bp)
call mpiq_pack_buffer(H2Oi_rta, tx_bp)
call mpiq_pack_buffer(CO2i_rta, tx_bp)
!... use_ext_sounding_file: not needed by leafPEs

do n=1,n_PE_leaf
   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_nlst_vars


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_vtables(tx_bp)

!+ Transmit variable tables to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, n_vars, pbanner, pbnlen, n_PE_leaf, PE_rank
use vtables_mod, only: nv3dmt, ivtab3m, nv2d, ivtab2d, nv2ds, ivtab2ds,  &
     nv3ds, ivtab3ds, nv3d, ivtab3d, nsclr, nmp31, nmp21, nmps31, nmps21,  &
     nmp32, nmp22, nmps32, nmps22, nmp33, nmp23, nmps33, nmps23, nmp3i,  &
     nmp2i, nmps3i, nmps2i, ivopt3m, ivopt3d, ivopt2d, ivopt3ds, ivopt2ds,  &
     mp3tab1, mp3tab2, mp3tab3, mp3tabi, mp2tab1, mp2tab2, mp2tab3, mp2tabi,  &
     mp3stab1, mp3stab2, mp3stab3, mp3stabi, mp2stab1, mp2stab2, mp2stab3,  &
     mp2stabi, isclrp, isclrt, ivopt_clen, naersclr, iaersclp, iaersclt,  &
     ivnstr_clen, iaerschrp, unvar_bndidx, unvar_nbnd, unvar_ntype,  &
     iaersfi, iaersff, iaersaj, iaersag, iaersrs, ischrp

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'rootPE_put_vtables'

character, allocatable, dimension(:) :: c_buffer

integer :: n_packed_B, n, nn, a_err, iq, iqg, iqv, iqc

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

n_packed_B = i4_B*(2 + 22*n_grids + 1*n_vars + 29*n_grids*n_vars + 1*unvar_nbnd*unvar_ntype) +  &
             i8_B*0 +  &
             f4_B*0 +  &
             f8_B*0 +  &
             cn_B*(1*n_vars*ivopt_clen + 4*n_vars*n_grids*ivopt_clen +  &
                  2*n_vars*n_grids*ivnstr_clen) +  &
             bn_B*0

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

! INTEGER (4-byte) data:

call mpiq_pack_buffer(n_vars, tx_bp)
call mpiq_pack_buffer(nv3dmt, tx_bp)          ! Fill transmit buffer with data

call mpiq_pack_buffer(nv3d, n_grids, tx_bp)
call mpiq_pack_buffer(nv2d, n_grids, tx_bp)
call mpiq_pack_buffer(nv3ds, n_grids, tx_bp)
call mpiq_pack_buffer(nv2ds, n_grids, tx_bp)
call mpiq_pack_buffer(nmp31, n_grids, tx_bp)
call mpiq_pack_buffer(nmp32, n_grids, tx_bp)
call mpiq_pack_buffer(nmp33, n_grids, tx_bp)
call mpiq_pack_buffer(nmp3i, n_grids, tx_bp)
call mpiq_pack_buffer(nmp21, n_grids, tx_bp)
call mpiq_pack_buffer(nmp22, n_grids, tx_bp)
call mpiq_pack_buffer(nmp23, n_grids, tx_bp)
call mpiq_pack_buffer(nmp2i, n_grids, tx_bp)
call mpiq_pack_buffer(nmps31, n_grids, tx_bp)
call mpiq_pack_buffer(nmps21, n_grids, tx_bp)
call mpiq_pack_buffer(nmps32, n_grids, tx_bp)
call mpiq_pack_buffer(nmps22, n_grids, tx_bp)
call mpiq_pack_buffer(nmps33, n_grids, tx_bp)
call mpiq_pack_buffer(nmps23, n_grids, tx_bp)
call mpiq_pack_buffer(nmps3i, n_grids, tx_bp)
call mpiq_pack_buffer(nmps2i, n_grids, tx_bp)
call mpiq_pack_buffer(nsclr, n_grids, tx_bp)
call mpiq_pack_buffer(naersclr, n_grids, tx_bp)

call mpiq_pack_buffer(ivtab3m, n_vars, tx_bp)

call mpiq_pack_buffer(ivtab3d, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(ivtab2d, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(ivtab3ds, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(ivtab2ds, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3tab1, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3tab2, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3tab3, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3tabi, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2tab1, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2tab2, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2tab3, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2tabi, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3stab1, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3stab2, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3stab3, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2stab1, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2stab2, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2stab3, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp3stabi, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(mp2stabi, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(isclrp, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(isclrt, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersclp, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersclt, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersfi, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersff, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersaj, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersag, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(iaersrs, n_vars, n_grids, tx_bp)

call mpiq_pack_buffer(unvar_bndidx, unvar_nbnd, unvar_ntype, tx_bp)

! CHARACTER (1-byte) data:

call mpiq_pack_buffer(ivopt3m, ivopt_clen, n_vars, tx_bp)

! Cannot rely on higher array dimensions working; work around:

nn = n_vars*n_grids*ivopt_clen

allocate(c_buffer(nn), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'A')

!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         c_buffer(iq) = ivopt3d(iqv,iqg)(iqc:iqc); iq = iq+1
!      end do
!   end do
!end do
call mpiq_pack_buffer(ivopt3d, ivopt_clen, n_vars, n_grids, tx_bp)  ! Fill transmit buffer
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         c_buffer(iq) = ivopt2d(iqv,iqg)(iqc:iqc); iq = iq+1
!      end do
!   end do
!end do
call mpiq_pack_buffer(ivopt2d, ivopt_clen, n_vars, n_grids, tx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         c_buffer(iq) = ivopt3ds(iqv,iqg)(iqc:iqc); iq = iq+1
!      end do
!   end do
!end do
call mpiq_pack_buffer(ivopt3ds, ivopt_clen, n_vars, n_grids, tx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         c_buffer(iq) = ivopt2ds(iqv,iqg)(iqc:iqc); iq = iq+1
!      end do
!   end do
!end do
call mpiq_pack_buffer(ivopt2ds, ivopt_clen, n_vars, n_grids, tx_bp)

deallocate(c_buffer, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'B')

nn = n_vars*n_grids*ivnstr_clen

allocate(c_buffer(nn), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'B')

!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivnstr_clen
!         c_buffer(iq) = iaerschrp(iqv,iqg)(iqc:iqc); iq = iq+1
!      end do
!   end do
!end do
call mpiq_pack_buffer(iaerschrp, ivnstr_clen, n_vars, n_grids, tx_bp)
call mpiq_pack_buffer(ischrp, ivnstr_clen, n_vars, n_grids, tx_bp)

deallocate(c_buffer, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'B')

do n=1,n_PE_leaf
   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_vtables


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_root_grid(tx_bp)

!+ Transmits some dimensions/flags of root domain to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, jdim, gd, n_PE_leaf, PE_rank

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, n, igd

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

n_packed_B = i4_B*(1 + 12*n_grids) +  &
             i8_B*0 +  &
             f4_B*0 +  &
             f8_B*0 +  &
             cn_B*0 +  &
             bn_B*0

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

! INTEGER (4-byte) data:

call mpiq_pack_buffer(jdim, tx_bp)

do igd=1,n_grids
   call mpiq_pack_buffer(gd(igd)%nx, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nx1, tx_bp)  ! Fill tx buffer
   call mpiq_pack_buffer(gd(igd)%nx2, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ny, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ny1, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ny2, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nz, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nz1, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nxyzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nxys3p, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nxys2p, tx_bp)
   call mpiq_pack_buffer(gd(igd)%nxyp, tx_bp)
end do

do n=1,n_PE_leaf
   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_root_grid


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_leaf_grid(tx_bp)

!+ Transmits some dimensions/flags of leaf domains to the leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, gd, n_PE_leaf, PE_rank, n_pts_cyclic,  &
     cyclic_paths_t, cyclic_paths_u, cyclic_paths_v, n_comm_path_entries_max,  &
     n_comm_path_optypes_max, n_cyclic_entries

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B,  n, igd

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

n_packed_B = i4_B*(52*n_grids*n_PE_leaf + 3*8*n_pts_cyclic) +  &
             i8_B*0 +  &
             f4_B*0 +  &
             f8_B*0 +  &
             cn_B*0 +  &
             bn_B*0

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

! INTEGER (4-byte) data:

do igd=1,n_grids
   do n=1,n_PE_leaf
      gd(igd)%leafPE_mxp(n) = gd(igd)%leafPE_xe(n)-gd(igd)%leafPE_xb(n)+1
      gd(igd)%leafPE_myp(n) = gd(igd)%leafPE_ye(n)-gd(igd)%leafPE_yb(n)+1
      gd(igd)%leafPE_mzp(n) = gd(igd)%nzp
   end do
end do

do n=1,n_PE_leaf
   call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

   ! Fill transmit buffer with data:

   do igd=1,n_grids
      call mpiq_pack_buffer(gd(igd)%leafPE_mxp, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_myp, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_mzp, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_ia, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_iz, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_ja, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_jz, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_i0, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_j0, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%leafPE_ibndc, n_PE_leaf, tx_bp)
   end do

   do igd=1,n_grids
      call mpiq_pack_buffer(gd(igd)%tx_paths_all(:,:,:,n),  &
           n_comm_path_entries_max, n_comm_path_optypes_max, n_PE_leaf, tx_bp)
      call mpiq_pack_buffer(gd(igd)%rx_paths_all(:,:,n),  &
           n_comm_path_optypes_max, n_PE_leaf, tx_bp)
   end do

   call mpiq_pack_buffer(cyclic_paths_t, n_cyclic_entries, n_pts_cyclic, tx_bp)
   call mpiq_pack_buffer(cyclic_paths_u, n_cyclic_entries, n_pts_cyclic, tx_bp)
   call mpiq_pack_buffer(cyclic_paths_v, n_cyclic_entries, n_pts_cyclic, tx_bp)

   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_leaf_grid


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_gen_grid(tx_bp)

!+ Transmits grid setup info to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, maxnzp, maxnxp, maxnyp, nzg, nzs, ztop,  &
     SBPS_Tlev, gd, n_PE_leaf, PE_rank  

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, n, igd

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

n_packed_B = i4_B*(2*maxnzp*n_grids + 1*maxnxp*n_grids + 1*maxnyp*n_grids) +  &
             i8_B*0 +  &
             f4_B*(1 + (5+2*maxnxp+2*maxnyp+6*maxnzp)*n_grids + (nzg+nzs)) +  &
             f8_B*0 +  &
             cn_B*0 +  &
             bn_B*0

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

! INTEGER (4-byte) data:

call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

do igd=1,n_grids
   call mpiq_pack_buffer(gd(igd)%nrz, maxnzp, tx_bp) ! Fill tx buffer
   call mpiq_pack_buffer(gd(igd)%ipm, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%jpm, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%kpm, maxnzp, tx_bp)
end do

! REAL (4-byte) data:

call mpiq_pack_buffer(ztop, tx_bp)

do igd=1,n_grids
   call mpiq_pack_buffer(gd(igd)%deltax, tx_bp)
   call mpiq_pack_buffer(gd(igd)%deltay, tx_bp)
   call mpiq_pack_buffer(gd(igd)%deltaz, tx_bp)
   call mpiq_pack_buffer(gd(igd)%plat, tx_bp)
   call mpiq_pack_buffer(gd(igd)%plon, tx_bp)

   call mpiq_pack_buffer(gd(igd)%xm, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%xt, maxnxp, tx_bp)

   call mpiq_pack_buffer(gd(igd)%ym, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%yt, maxnyp, tx_bp)

   call mpiq_pack_buffer(gd(igd)%zm, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%zt, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%dzm, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%dzm2, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%dzt, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%dzt2, maxnzp, tx_bp)
end do

call mpiq_pack_buffer(SBPS_Tlev, nzg+nzs, tx_bp)

do n=1,n_PE_leaf
   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_gen_grid


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_nest_coeffs(tx_bp)

!+ Transmits nesting interpolation coefficients to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, maxnzp, maxnxp, maxnyp, gd, n_PE_leaf,  &
     PE_rank

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, n, igd

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

n_packed_B = i4_B*0 +  &
             i8_B*0 +  &
             f4_B*((7*maxnxp + 7*maxnyp + 7*maxnzp + 1*maxnzp*4)*n_grids) +  &
             f8_B*0 +  &
             cn_B*0 +  &
             bn_B*0

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

! REAL (4-byte) data:

call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

do igd=1,n_grids
   call mpiq_pack_buffer(gd(igd)%ei1, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ei2, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ei3, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ei4, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ei5, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ei6, maxnxp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ei7, maxnxp, tx_bp)

   call mpiq_pack_buffer(gd(igd)%ej1, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ej2, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ej3, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ej4, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ej5, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ej6, maxnyp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ej7, maxnyp, tx_bp)

   call mpiq_pack_buffer(gd(igd)%ek1, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ek2, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ek3, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ek4, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ek5, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ek6, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ek7, maxnzp, tx_bp)

   call mpiq_pack_buffer(gd(igd)%fbcf, maxnzp, 4, tx_bp)
end do

do n=1,n_PE_leaf
   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_nest_coeffs


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_misc(tx_bp)

!+ Transmits any other miscellaneous info to all leaf PEs

use univ_kind_defs_mod, only: i4_B, i8_B, f4_B, f8_B, cn_B, bn_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, mpiq_transmit_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, maxnzp, itopo, n_varftot, n_varf, vtime,  &
     nsubs, gd, n_PE_leaf, PE_rank, acoustic_lBC_sd_max_pB,  &
     dnscale_nBC_sd_max_pB, upscale_nBC_sd_max_pB, initialization_type
use main_memory_bspec_mod, only: gd_bd
use radn_netflux_mem_mod, only: MMAXU0, MTAMTO, MMAXALB, fluxnet

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, n, igd

!=== Executable statements:

#if defined(USE_MPI)

! Bytecount of primary message:

if (initialization_type == 2) then
   n_packed_B = i4_B*(7) +  &
        i8_B*0 +  &
        f4_B*(12*maxnzp*n_grids + 1*n_varftot) +  &
        f8_B*(1*MTAMTO*MMAXU0*MMAXALB) +  &
        cn_B*0 +  &
        bn_B*0
else
   n_packed_B = i4_B*(7) +  &
        i8_B*0 +  &
        f4_B*(12*maxnzp*n_grids) +  &
        f8_B*(1*MTAMTO*MMAXU0*MMAXALB) +  &
        cn_B*0 +  &
        bn_B*0
end if

! Header message:

do n=1,n_PE_leaf
   call mpiq_transmit_plain(n_packed_B, PE_rank(n))   ! Transmit message to PE
end do

!--- Primary message:

call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare transmit buffer

! INTEGER (4-byte) data:

call mpiq_pack_buffer(nsubs, tx_bp)
call mpiq_pack_buffer(itopo, tx_bp)         ! Fill transmit buffer with data
call mpiq_pack_buffer(n_varftot, tx_bp)
call mpiq_pack_buffer(n_varf, tx_bp)
call mpiq_pack_buffer(acoustic_lBC_sd_max_pB, tx_bp)
call mpiq_pack_buffer(dnscale_nBC_sd_max_pB, tx_bp)
call mpiq_pack_buffer(upscale_nBC_sd_max_pB, tx_bp)

! REAL (4-byte) data:

if (initialization_type == 2)  &
     call mpiq_pack_buffer(vtime, n_varftot, tx_bp)  ! Fill transmit buffer with data

do igd=1,n_grids
   call mpiq_pack_buffer(gd(igd)%u01d, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%v01d, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%pi01d, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%th01d, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%dn01d, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd_bd(igd)%rt01d, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ht, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%hw, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ht2, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%ht4, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%hw2, maxnzp, tx_bp)
   call mpiq_pack_buffer(gd(igd)%hw4, maxnzp, tx_bp)
end do

! REAL (8-byte) data:

call mpiq_pack_buffer(fluxnet, MTAMTO, MMAXU0, MMAXALB, tx_bp)

do n=1,n_PE_leaf
   call mpiq_transmit_packed(tx_bp, PE_rank(n))       ! Transmit message to PE
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_misc


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_IDs(want_to_init, rx_bp)

!+ Receives basic parallel element (PE) identification info from root PE

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_receive_plain, mpiq_unpack_buffer,  &
     mpiq_check_buffer_packed, mpiq_receive_packed, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: rootPE_rank, n_PE_leaf, PE_rank, my_rank, my_ID,  &
     manage_parallel_memory

implicit none

!=== Argument declarations:

logical, intent(IN) :: want_to_init

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B

!=== Executable statements:

#if defined(USE_MPI)

if (want_to_init) then

   !-- First, get the rank of the root PE, so that the correct message source
   !    rank can be targeted from now on:

   call mpiq_receive_plain(rootPE_rank, -1) ! Use source rank of MPI_ANY_SOURCE

   !-- Now perform other messaging tasks:

   ! Get header message:

   call mpiq_receive_plain(n_packed_B, rootPE_rank)

   ! Get message body:

   call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
   call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

   call mpiq_unpack_buffer(my_rank, rx_bp)
   call mpiq_unpack_buffer(my_ID, rx_bp)   ! Extract data from receive buffer
   call mpiq_unpack_buffer(n_PE_leaf, rx_bp)

   call manage_parallel_memory(dim_n_PE_leaf=.TRUE.)   ! Allocate arrays

   call mpiq_unpack_buffer(PE_rank, n_PE_leaf, rx_bp)

end if

#endif  /* end:USE_MPI */

end subroutine leafPE_get_IDs


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_nlst_vars(rx_bp)

!+ Receives relevant namelist info from root PE

use univ_IO_mem_status_mod, only: for_allocation
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: initialization_type, start_UT, n_grids,  &
     map_projection, n_dta_per_dtdyn, dt_mode, nestz1, nudlat, ioutput,  &
     naddsc, icorflg, ibnd, jbnd, lsflg, nfpt, islastog, npatch, ihorgrad,  &
     nzg, nzs, nstratz1, maxnzp, target_Ls, deltax_g1, deltay_g1, deltaz_g1,  &
     dzrat, dzmax, polelat, polelon, tnudlat, tnudcent, tnudtop, znudtop,  &
     wfreq_state_file, cphas, distim, emissivity, diurnal_progression,  &
     radfrq, zrough, albedo, zz, maxnnp, TI_val, gd, dt_max_frac_increase_g1,  &
     maximum_sim_ET, maxnxp, maxnyp, min_ssfrac, n_substeps_max,  &
     manage_nml_var_memory, start_sim_ET, current_sim_ET,  &
     enable_advection, enable_turb_diffusion, enable_aerosol_microphysics,  &
     RT_paradigm, prognose_SBPS, prognose_uvwp, max_dt_dynamics,   &
     default_input_data_dir, n_isched_steps, rootPE_rank, impl, iadvl,  &
     iadvf, izflat, orbital_progression
use main_memory_bspec_mod, only: dust_nb, H2Oi_nb, CO2i_nb, dust_tog,  &
     H2Ov_tog, H2Oi_tog, CO2i_tog, dust_rmn, dust_mrt, H2Oi_rmn, H2Oi_mrt,  &
     CO2i_rmn, CO2i_mrt, dust_ivr, dust_rta, H2Oi_rta, CO2i_rta, rtc_tog,  &
     bdst_igd, bdst_imr, bdst_tog, bdst_icn, bdst_irt, bdst_iht, bdst_itt,  &
     CO2_sfci, dust_icn, dust_igd, dust_iht, dust_imr, dust_inn, dust_irt,  &
     dust_itt, dust_jef, dust_jth, H2Ov_ivr, sfci_ils, conrnu,  &
     Ls_of_periapse, orbital_eccentricity, axial_obliquity, AU_to_m_knob,  &
     lfmcut, nrdlevt, lonrad, ishadtog,nslcon,tautot,pctlcon,slz, stgoff,  &
     slmstr, manage_nml_var_memory_bspec, RT_flux_trace_np, dust_op_case,  &
     use_gas_opacities, dust_ctau_refp
use aermic_rad_mem_mod, only: iaertog, aer_rad_gpbeg, aer_gpbeg, aer_ngrp,  &
     aer_csr_gpbeg, aer_inj_gpbeg, aer_rvbeg
use namelist_bspec_mod, only: default_input_data_dir_bspec

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, igd

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

!--- Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! INTEGER (4-byte) data:

call mpiq_unpack_buffer(n_grids, rx_bp)      ! Extract data from receive buffer
call mpiq_unpack_buffer(map_projection, rx_bp)
call mpiq_unpack_buffer(initialization_type, rx_bp)
call mpiq_unpack_buffer(dt_mode, rx_bp)
call mpiq_unpack_buffer(n_dta_per_dtdyn, rx_bp)
call mpiq_unpack_buffer(n_substeps_max, rx_bp)  
call mpiq_unpack_buffer(nzg, rx_bp)
call mpiq_unpack_buffer(nzs, rx_bp)
call mpiq_unpack_buffer(nestz1, rx_bp)
call mpiq_unpack_buffer(ioutput, rx_bp)
call mpiq_unpack_buffer(nudlat, rx_bp)
!... pnLES_np: ???
call mpiq_unpack_buffer(naddsc, rx_bp)
call mpiq_unpack_buffer(icorflg, rx_bp)
call mpiq_unpack_buffer(RT_paradigm, rx_bp)
call mpiq_unpack_buffer(ibnd, rx_bp)
call mpiq_unpack_buffer(jbnd, rx_bp)
call mpiq_unpack_buffer(lsflg, rx_bp)
call mpiq_unpack_buffer(nfpt, rx_bp)
call mpiq_unpack_buffer(islastog, rx_bp)
call mpiq_unpack_buffer(ihorgrad, rx_bp)
call mpiq_unpack_buffer(npatch, rx_bp)
call mpiq_unpack_buffer(CO2_sfci, rx_bp)
call mpiq_unpack_buffer(nslcon, rx_bp)
call mpiq_unpack_buffer(rtc_tog, rx_bp)
call mpiq_unpack_buffer(lonrad, rx_bp)
call mpiq_unpack_buffer(ishadtog, rx_bp)
call mpiq_unpack_buffer(RT_flux_trace_np, rx_bp)
call mpiq_unpack_buffer(lfmcut, rx_bp)
call mpiq_unpack_buffer(nrdlevt, rx_bp)
call mpiq_unpack_buffer(dust_op_case, rx_bp)
call mpiq_unpack_buffer(bdst_tog, rx_bp)
call mpiq_unpack_buffer(H2Ov_tog, rx_bp)
call mpiq_unpack_buffer(H2Ov_ivr, rx_bp)
call mpiq_unpack_buffer(dust_tog, rx_bp)
call mpiq_unpack_buffer(dust_nb, rx_bp)
call mpiq_unpack_buffer(dust_ivr, rx_bp)
call mpiq_unpack_buffer(H2Oi_tog, rx_bp)
call mpiq_unpack_buffer(H2Oi_nb, rx_bp)
call mpiq_unpack_buffer(CO2i_tog, rx_bp)
call mpiq_unpack_buffer(CO2i_nb, rx_bp)
!... nsndp: not needed by leafPEs
!... ipsflg: not needed by leafPEs
!... itsflg: not needed by leafPEs
!... irtsflg: not needed by leafPEs
!... iusflg: not needed by leafPEs

call mpiq_unpack_buffer(impl, rx_bp)   ! Relevant "static" namelist parameters
call mpiq_unpack_buffer(izflat, rx_bp) !
call mpiq_unpack_buffer(iadvl, rx_bp)  !
call mpiq_unpack_buffer(iadvf, rx_bp)  !

call mpiq_unpack_buffer(iaertog, rx_bp) ! Not in namelist
call mpiq_unpack_buffer(maxnnp, rx_bp) ! Not in namelist
call mpiq_unpack_buffer(n_isched_steps, rx_bp) ! Not in namelist

call manage_nml_var_memory(dim_n_grids=.TRUE.) ! Allocate grid object(s)
call manage_nml_var_memory_bspec(dim_n_grids=.TRUE.) ! Allocate grid object(s)

do igd=1,n_grids
   !... gd(:)%n_dt_per_pgdt: effectively given by rootPE in leafPE_get_dtsched
   call mpiq_unpack_buffer(gd(igd)%nxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nxtnest, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nstratx, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nstraty, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nknest, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nsttop, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nstbot, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ninest, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%njnest, rx_bp)
   !... gd(:)%itoptflg: not needed by leafPEs
   !... gd(:)%itopsflg: not needed by leafPEs
   !... gd(:)%itinflg: not needed by leafPEs
   !... gd(:)%ialbflg: not needed by leafPEs
   !... gd(:)%iz0flg: not needed by leafPEs
   call mpiq_unpack_buffer(gd(igd)%idiffk, rx_bp)
   !... gd(:)%nofilflg: not needed by leafPEs
end do

maxnxp = maxval(gd(1:n_grids)%nxp)
maxnyp = maxval(gd(1:n_grids)%nyp)
maxnzp = maxval(gd(1:n_grids)%nzp)
  ! Allocate some more items:
call manage_nml_var_memory(dim_maxnzp=.TRUE., dim_pnLES_np=.TRUE.)
call manage_nml_var_memory_bspec(dim_nzg=.TRUE.)

call mpiq_unpack_buffer(nstratz1, maxnzp, rx_bp)

! REAL (4-byte) data:

call mpiq_unpack_buffer(target_Ls, rx_bp)
call mpiq_unpack_buffer(start_UT, rx_bp)
!... dt_dynamics: effectively given by rootPE in leafPE_get_dtsched
call mpiq_unpack_buffer(min_ssfrac, rx_bp)
call mpiq_unpack_buffer(max_dt_dynamics, rx_bp)
call mpiq_unpack_buffer(dt_max_frac_increase_g1, rx_bp)
call mpiq_unpack_buffer(deltax_g1, rx_bp)            
call mpiq_unpack_buffer(deltay_g1, rx_bp)     ! Extract data from receive buffer
call mpiq_unpack_buffer(deltaz_g1, rx_bp)
call mpiq_unpack_buffer(dzrat, rx_bp)
call mpiq_unpack_buffer(dzmax, rx_bp)
call mpiq_unpack_buffer(polelat, rx_bp)
call mpiq_unpack_buffer(polelon, rx_bp)
call mpiq_unpack_buffer(wfreq_state_file, rx_bp)
call mpiq_unpack_buffer(tnudlat, rx_bp)
call mpiq_unpack_buffer(tnudcent, rx_bp)
call mpiq_unpack_buffer(tnudtop, rx_bp)
call mpiq_unpack_buffer(znudtop, rx_bp)
!... z0fact: not needed by leafPEs
call mpiq_unpack_buffer(cphas, rx_bp)
call mpiq_unpack_buffer(distim, rx_bp)
call mpiq_unpack_buffer(zrough, rx_bp)
call mpiq_unpack_buffer(albedo, rx_bp)
call mpiq_unpack_buffer(TI_val, rx_bp)
call mpiq_unpack_buffer(emissivity, rx_bp)
call mpiq_unpack_buffer(pctlcon, rx_bp)
call mpiq_unpack_buffer(sfci_iLs, rx_bp)
call mpiq_unpack_buffer(radfrq, rx_bp)
call mpiq_unpack_buffer(Ls_of_periapse, rx_bp)
call mpiq_unpack_buffer(orbital_eccentricity, rx_bp)
call mpiq_unpack_buffer(axial_obliquity, rx_bp)
call mpiq_unpack_buffer(AU_to_m_knob, rx_bp)
call mpiq_unpack_buffer(tautot, rx_bp)
call mpiq_unpack_buffer(conrnu, rx_bp)
call mpiq_unpack_buffer(dust_ctau_refp, rx_bp)
call mpiq_unpack_buffer(bdst_imr, rx_bp)
call mpiq_unpack_buffer(bdst_igd, rx_bp)
call mpiq_unpack_buffer(bdst_icn, rx_bp)
call mpiq_unpack_buffer(bdst_irt, rx_bp)
call mpiq_unpack_buffer(bdst_iht, rx_bp)
call mpiq_unpack_buffer(bdst_itt, rx_bp)
call mpiq_unpack_buffer(dust_rmn, rx_bp)
call mpiq_unpack_buffer(dust_mrt, rx_bp)
call mpiq_unpack_buffer(dust_jef, rx_bp)
call mpiq_unpack_buffer(dust_jth, rx_bp)
call mpiq_unpack_buffer(dust_igd, rx_bp)
call mpiq_unpack_buffer(dust_imr, rx_bp)
call mpiq_unpack_buffer(dust_inn, rx_bp)
call mpiq_unpack_buffer(dust_icn, rx_bp)
call mpiq_unpack_buffer(dust_irt, rx_bp)
call mpiq_unpack_buffer(dust_iht, rx_bp)
call mpiq_unpack_buffer(dust_itt, rx_bp)
call mpiq_unpack_buffer(H2Oi_rmn, rx_bp)
call mpiq_unpack_buffer(H2Oi_mrt, rx_bp)
call mpiq_unpack_buffer(CO2i_rmn, rx_bp)
call mpiq_unpack_buffer(CO2i_mrt, rx_bp)

call mpiq_unpack_buffer(zz, maxnzp, rx_bp)

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%centlat, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%centlon, rx_bp)
   !... gd(:)%toptenh: not needed by leafPEs
   !... gd(:)%toptwvlx: not needed by leafPEs
   !... gd(:)%toptwvly: not needed by leafPEs
   !... gd(:)%tinwvlx: not needed by leafPEs
   !... gd(:)%tinwvly: not needed by leafPEs
   !... gd(:)%albwvlx: not needed by leafPEs
   !... gd(:)%albwvly: not needed by leafPEs
   !... gd(:)%z0max: not needed by leafPEs
   !... gd(:)%z0_of_topolevel: not needed by leafPEs
   !... gd(:)%topolevel_for_z0: not needed by leafPEs
   call mpiq_unpack_buffer(gd(igd)%gridu, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%gridv, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%csx, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%csz, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%xkhkm, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%zkhkm, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%akmin, rx_bp)
end do

!... hsnd(nsndp): not needed by leafPEs
!... psnd(nsndp): not needed by leafPEs
!... tsnd(nsndp): not needed by leafPEs
!... rtsnd(nsndp): not needed by leafPEs
!... usnd(nsndp): not needed by leafPEs
!... vsnd(nsndp): not needed by leafPEs

call mpiq_unpack_buffer(slz, nzg, rx_bp)
call mpiq_unpack_buffer(stgoff, nzg, rx_bp)
call mpiq_unpack_buffer(slmstr, nzg, rx_bp)

! REAL (8-byte) data:

call mpiq_unpack_buffer(maximum_sim_ET, rx_bp)
!... dust_rtt: supplied by 'aer_rad_gpbeg'
!... H2Oi_rtt: supplied by 'aer_rad_gpbeg'
!... CO2i_rtt: supplied by 'aer_rad_gpbeg'
call mpiq_unpack_buffer(aer_rvbeg, rx_bp)   ! H2Ov_beg (in namelist)
!... dust_beg: supplied by 'aer_gpbeg'
!... dust_jsr: supplied by 'aer_csr_gpbeg'
!... dust_jbg: supplied by 'aer_inj_gpbeg'
!... H2Oi_beg: supplied by 'aer_gpbeg'
!... CO2i_beg: supplied by 'aer_gpbeg'

call mpiq_unpack_buffer(start_sim_ET, rx_bp)  ! 0. or restart time (not in nml) 
call mpiq_unpack_buffer(current_sim_ET, rx_bp)     ! not in namelist

call mpiq_unpack_buffer(aer_rad_gpbeg, aer_ngrp, rx_bp)  ! *_rtt; dust,H2Oi,CO2i
call mpiq_unpack_buffer(aer_gpbeg, aer_ngrp, rx_bp)  ! *_beg; dust, H2Oi, CO2i
call mpiq_unpack_buffer(aer_csr_gpbeg, aer_ngrp, rx_bp)  ! dust_jsr
call mpiq_unpack_buffer(aer_inj_gpbeg, aer_ngrp, rx_bp)  ! dust_jbg

! CHARACTER (1-byte) data:

!... simulation_description: not needed by leafPEs
!... run_type: not needed by leafPEs
!... output_dir_root: not needed by leafPEs
!... simulation_ID: not needed by leafPEs
!... run_suffix: not needed by leafPEs
!... input_varfile_pfx: not needed by leafPEs
!... input_restart_file_pfx: not needed by leafPEs
call mpiq_unpack_buffer(default_input_data_dir, len(default_input_data_dir),  &
     rx_bp)
!... input_sfcfile_pfx: not needed by leafPEs
!... pnLES_fn: ???
!... input_sounding_fn: not needed by leafPEs

!... pnLES_pt: ???

!... gd(:)%itoptfn: not needed by leafPEs
!... gd(:)%itinfn: not needed by leafPEs
!... gd(:)%ialbfn: not needed by leafPEs

call mpiq_unpack_buffer(default_input_data_dir_bspec,  &  ! Not in namelist
     len(default_input_data_dir_bspec), rx_bp)

! LOGICAL data:

call mpiq_unpack_buffer(orbital_progression, rx_bp)
call mpiq_unpack_buffer(diurnal_progression, rx_bp)
!... stop_if_file_exists: not needed by leafPEs
!... grid_in_state_outp: not needed by leafPEs  ! Derived from state_file_grids
!... preserve_dt_on_restart: not needed by leafPEs
!... run_sfcinit_user: not needed by leafPEs
!... run_toptinit_user: not needed by leafPEs
call mpiq_unpack_buffer(enable_advection, rx_bp)
call mpiq_unpack_buffer(enable_turb_diffusion, rx_bp)
call mpiq_unpack_buffer(enable_aerosol_microphysics, rx_bp)
call mpiq_unpack_buffer(prognose_SBPS, rx_bp)
call mpiq_unpack_buffer(prognose_uvwp, rx_bp)
!... perturb_nsfc_theta: ???
call mpiq_unpack_buffer(use_gas_opacities, rx_bp)
call mpiq_unpack_buffer(dust_rta, rx_bp)
call mpiq_unpack_buffer(H2Oi_rta, rx_bp)
call mpiq_unpack_buffer(CO2i_rta, rx_bp)
!... use_ext_sounding_file: not needed by leafPEs

#endif  /* end:USE_MPI */

end subroutine leafPE_get_nlst_vars


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_vtables(rx_bp)

!+ Receives variable tables from root PE

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_vars, n_grids, rootPE_rank
use vtables_mod, only: nv3dmt, ivtab3m, nv2d, ivtab2d, nv2ds, ivtab2ds,  &
     nv3ds, ivtab3ds, nv3d, ivtab3d, nsclr, nmp31, nmp21, nmps31, nmps21,  &
     nmp32, nmp22, nmps32, nmps22, nmp33, nmp23, nmps33, nmps23, nmp3i,  &
     nmp2i, nmps3i, nmps2i, ivopt3m, ivopt3d, ivopt2d, ivopt3ds, ivopt2ds,  &
     mp3tab1, mp3tab2, mp3tab3, mp3tabi, mp2tab1, mp2tab2, mp2tab3, mp2tabi,  &
     mp3stab1, mp3stab2, mp3stab3, mp3stabi, mp2stab1, mp2stab2, mp2stab3,  &
     mp2stabi, isclrp, isclrt, vtable_mem, ivopt_clen, naersclr, iaersclp,  &
     iaersclt, ivnstr_clen, iaerschrp, unvar_bndidx, unvar_nbnd, unvar_ntype,  &
     iaersfi, iaersff, iaersaj, iaersag, iaersrs, ischrp

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'leafPE_get_vtables'

character, allocatable, dimension(:) :: c_buffer

integer :: n_packed_B, a_err, iq, iqg, iqv, iqc, nn

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

!--- Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! INTEGER (4-byte) data:

call mpiq_unpack_buffer(n_vars, rx_bp)      ! Extract data from receive buffer

call vtable_mem(for_allocation)    ! Allocate memory for variable tables

call mpiq_unpack_buffer(nv3dmt, rx_bp)

call mpiq_unpack_buffer(nv3d, n_grids, rx_bp)
call mpiq_unpack_buffer(nv2d, n_grids, rx_bp)
call mpiq_unpack_buffer(nv3ds, n_grids, rx_bp)
call mpiq_unpack_buffer(nv2ds, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp31, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp32, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp33, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp3i, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp21, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp22, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp23, n_grids, rx_bp)
call mpiq_unpack_buffer(nmp2i, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps31, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps21, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps32, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps22, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps33, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps23, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps3i, n_grids, rx_bp)
call mpiq_unpack_buffer(nmps2i, n_grids, rx_bp)
call mpiq_unpack_buffer(nsclr, n_grids, rx_bp)
call mpiq_unpack_buffer(naersclr, n_grids, rx_bp)

call mpiq_unpack_buffer(ivtab3m, n_vars, rx_bp)

call mpiq_unpack_buffer(ivtab3d, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(ivtab2d, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(ivtab3ds, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(ivtab2ds, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3tab1, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3tab2, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3tab3, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3tabi, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2tab1, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2tab2, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2tab3, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2tabi, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3stab1, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3stab2, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3stab3, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2stab1, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2stab2, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2stab3, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp3stabi, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(mp2stabi, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(isclrp, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(isclrt, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersclp, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersclt, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersfi, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersff, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersaj, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersag, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(iaersrs, n_vars, n_grids, rx_bp) 

call mpiq_unpack_buffer(unvar_bndidx, unvar_nbnd, unvar_ntype, rx_bp)

! CHARACTER (1-byte) data:

call mpiq_unpack_buffer(ivopt3m, ivopt_clen, n_vars, rx_bp) ! Extract data from rx buffer

! Cannot rely on higher array dimensions working; work around:

nn = n_vars*n_grids*ivopt_clen

allocate(c_buffer(nn), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'A')

call mpiq_unpack_buffer(ivopt3d, ivopt_clen, n_vars, n_grids, rx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         ivopt3d(iqv,iqg)(iqc:iqc) = c_buffer(iq); iq = iq+1
!      end do
!   end do
!end do
call mpiq_unpack_buffer(ivopt2d, ivopt_clen, n_vars, n_grids, rx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         ivopt2d(iqv,iqg)(iqc:iqc) = c_buffer(iq); iq = iq+1
!      end do
!   end do
!end do
call mpiq_unpack_buffer(ivopt3ds, ivopt_clen, n_vars, n_grids, rx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         ivopt3ds(iqv,iqg)(iqc:iqc) = c_buffer(iq); iq = iq+1
!      end do
!   end do
!end do
call mpiq_unpack_buffer(ivopt2ds, ivopt_clen, n_vars, n_grids, rx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivopt_clen
!         ivopt2ds(iqv,iqg)(iqc:iqc) = c_buffer(iq); iq = iq+1
!      end do
!   end do
!end do

deallocate(c_buffer, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'A')

nn = n_vars*n_grids*ivnstr_clen

allocate(c_buffer(nn), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'B')

call mpiq_unpack_buffer(iaerschrp, ivnstr_clen, n_vars, n_grids, rx_bp)
call mpiq_unpack_buffer(ischrp, ivnstr_clen, n_vars, n_grids, rx_bp)
!iq = 1; do iqg=1,n_grids
!   do iqv=1,n_vars
!      do iqc=1,ivnstr_clen
!         iaerschrp(iqv,iqg)(iqc:iqc) = c_buffer(iq); iq = iq+1
!      end do
!   end do
!end do

deallocate(c_buffer, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'B')

#endif  /* end:USE_MPI */

end subroutine leafPE_get_vtables


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_root_grid(rx_bp)

!+ Receives some dimensions/flags of root domain from the root PE

#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, jdim, gd, rootPE_rank

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, igd

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

!--- Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! INTEGER (4-byte) data:

call mpiq_unpack_buffer(jdim, rx_bp)

do igd=1,n_grids   ! Extract data from receive buffer
   call mpiq_unpack_buffer(gd(igd)%nx, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nx1, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nx2, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ny, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ny1, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ny2, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nz, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nz1, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nxyzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nxys3p, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nxys2p, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%nxyp, rx_bp)
end do

#endif  /* end:USE_MPI */

end subroutine leafPE_get_root_grid


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_leaf_grid(rx_bp)

!+ Receives some dimensions/flags of leaf domains from the root PE

#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, gd, n_PE_leaf, PE_rank, rootPE_rank,  &
     n_pts_cyclic, cyclic_paths_t, cyclic_paths_u, cyclic_paths_v, my_rank,  &
     n_comm_path_entries_max, n_comm_path_optypes_max, n_cyclic_entries

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, igd

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

! Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! Extract data from receive buffer

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%leafPE_mxp, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_myp, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_mzp, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_ia, n_PE_leaf, rx_bp)    
   call mpiq_unpack_buffer(gd(igd)%leafPE_iz, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_ja, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_jz, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_i0, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_j0, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%leafPE_ibndc, n_PE_leaf, rx_bp)
end do

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%tx_paths_leaf, n_comm_path_entries_max,  &
        n_comm_path_optypes_max, n_PE_leaf, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%rx_paths_leaf, n_comm_path_optypes_max,  &
        n_PE_leaf, rx_bp)
end do

call mpiq_unpack_buffer(cyclic_paths_t, n_cyclic_entries, n_pts_cyclic, rx_bp)
call mpiq_unpack_buffer(cyclic_paths_u, n_cyclic_entries, n_pts_cyclic, rx_bp)
call mpiq_unpack_buffer(cyclic_paths_v, n_cyclic_entries, n_pts_cyclic, rx_bp)

do igd=1,n_grids
   gd(igd)%mmxp = gd(igd)%leafPE_mxp(my_rank)
   gd(igd)%mmyp = gd(igd)%leafPE_myp(my_rank)
   gd(igd)%mmzp = gd(igd)%leafPE_mzp(my_rank)
   gd(igd)%mia = gd(igd)%leafPE_ia(my_rank)
   gd(igd)%miz = gd(igd)%leafPE_iz(my_rank)
   gd(igd)%mja = gd(igd)%leafPE_ja(my_rank)
   gd(igd)%mjz = gd(igd)%leafPE_jz(my_rank)
   gd(igd)%mi0 = gd(igd)%leafPE_i0(my_rank)
   gd(igd)%mj0 = gd(igd)%leafPE_j0(my_rank)
   gd(igd)%mibndc = gd(igd)%leafPE_ibndc(my_rank)
end do

#endif  /* end:USE_MPI */

end subroutine leafPE_get_leaf_grid


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_gen_grid(rx_bp)

!+ Receives grid setup info from root PE

#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, maxnzp, maxnxp, maxnyp, nzg, nzs, ztop,  &
     SBPS_Tlev, gd, rootPE_rank

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, igd

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

!--- Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! INTEGER (4-byte) data:

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%nrz, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ipm, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%jpm, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%kpm, maxnzp, rx_bp)
end do

! REAL (4-byte) data:

call mpiq_unpack_buffer(ztop, rx_bp)

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%deltax, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%deltay, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%deltaz, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%plat, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%plon, rx_bp)

   call mpiq_unpack_buffer(gd(igd)%xm, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%xt, maxnxp, rx_bp)

   call mpiq_unpack_buffer(gd(igd)%ym, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%yt, maxnyp, rx_bp)

   call mpiq_unpack_buffer(gd(igd)%zm, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%zt, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%dzm, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%dzm2, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%dzt, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%dzt2, maxnzp, rx_bp)
end do

call mpiq_unpack_buffer(SBPS_Tlev, nzg+nzs, rx_bp)

#endif  /* end:USE_MPI */

end subroutine leafPE_get_gen_grid


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_nest_coeffs(rx_bp)

!+ Receives nesting interpolation coefficients from the root PE

#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, maxnzp, maxnxp, maxnyp, gd, rootPE_rank

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, igd

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

!--- Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! REAL (4-byte) data:

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%ei1, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ei2, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ei3, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ei4, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ei5, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ei6, maxnxp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ei7, maxnxp, rx_bp)

   call mpiq_unpack_buffer(gd(igd)%ej1, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ej2, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ej3, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ej4, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ej5, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ej6, maxnyp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ej7, maxnyp, rx_bp)

   call mpiq_unpack_buffer(gd(igd)%ek1, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ek2, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ek3, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ek4, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ek5, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ek6, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ek7, maxnzp, rx_bp)

   call mpiq_unpack_buffer(gd(igd)%fbcf, maxnzp, 4, rx_bp)
end do

#endif  /* end:USE_MPI */

end subroutine leafPE_get_nest_coeffs


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_misc(rx_bp)

!+ Receives any other miscellaneous info from the root PE

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, mpiq_receive_plain, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: n_grids, maxnzp, nsubs, itopo, vtime, n_varftot,  &
     n_varf, gd, rootPE_rank, acoustic_lBC_sd_max_pB, dnscale_nBC_sd_max_pB,  &
     upscale_nBC_sd_max_pB, manage_misc_memory, initialization_type
use main_memory_bspec_mod, only: gd_bd
use radn_netflux_mem_mod, only: fluxnet, MMAXU0, MTAMTO, MMAXALB

implicit none

!=== Argument declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'leafPE_get_misc'

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n_packed_B, igd

!=== Executable statements:

#if defined(USE_MPI)

!--- Get header message:

call mpiq_receive_plain(n_packed_B, rootPE_rank)

!--- Get message body:

call mpiq_check_buffer_packed(rx_bp, n_packed_B)   ! Prepare receive buffer
call mpiq_receive_packed(rx_bp, rootPE_rank)      ! Receive message

! INTEGER (4-byte) data:

call mpiq_unpack_buffer(nsubs, rx_bp)
call mpiq_unpack_buffer(itopo, rx_bp)           
call mpiq_unpack_buffer(n_varftot, rx_bp)   ! Extract data from receive buffer 
call mpiq_unpack_buffer(n_varf, rx_bp)
call mpiq_unpack_buffer(acoustic_lBC_sd_max_pB, rx_bp)
call mpiq_unpack_buffer(dnscale_nBC_sd_max_pB, rx_bp)
call mpiq_unpack_buffer(upscale_nBC_sd_max_pB, rx_bp)

! REAL (4-byte) data:

if (initialization_type == 2) then
   call manage_misc_memory(dim_n_varftot=.TRUE.)
   call mpiq_unpack_buffer(vtime, n_varftot, rx_bp)
end if

do igd=1,n_grids
   call mpiq_unpack_buffer(gd(igd)%u01d, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%v01d, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%pi01d, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%th01d, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%dn01d, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd_bd(igd)%rt01d, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ht, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%hw, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ht2, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%ht4, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%hw2, maxnzp, rx_bp)
   call mpiq_unpack_buffer(gd(igd)%hw4, maxnzp, rx_bp)
end do

! REAL (8-byte) data:

call mpiq_unpack_buffer(fluxnet, MTAMTO, MMAXU0, MMAXALB, rx_bp)

#endif  /* end:USE_MPI */

end subroutine leafPE_get_misc


!```````````````````````````````````````````````````````````````````
subroutine fsdbuff_3D(a, b, n1, n2, n3, m2, m3, ic1, ic2, jc1, jc2)

!+ Copies data in leaf PE subdomain to buffer (3D "normal" variables)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, m2, m3, ic1, ic2, jc1, jc2
real(f4), dimension(n1,n2,n3), intent(IN) :: a

real(f4), dimension(n1,m2,m3), intent(INOUT) :: b

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

do j=jc1,jc2
   do i=ic1,ic2
      do k=1,n1
         b(k,i-ic1+1,j-jc1+1) = a(k,i,j)
      end do
   end do
end do

end subroutine fsdbuff_3D


!```````````````````````````````````````````````````````````````````
subroutine fsdbuff_2D(a, b, n2, n3, m2, m3, ic1, ic2, jc1, jc2)

!+ Copies data in leaf PE subdomain to buffer (2D "normal" variables)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, m2, m3, ic1, ic2, jc1, jc2
real(f4), dimension(n2,n3), intent(IN) :: a

real(f4), dimension(m2,m3), intent(INOUT) :: b

!== Local declarations:

integer :: i, j

!=== Executable statements:

do j=jc1,jc2
   do i=ic1,ic2
      b(i-ic1+1,j-jc1+1) = a(i,j)
   end do
end do

end subroutine fsdbuff_2D


!```````````````````````````````````````````````````````````````````
subroutine fsdbuff_3Ds(a, b, n4, n2, n3, n5, m2, m3, ic1, ic2, jc1, jc2)

!+ Copies data in leaf PE subdomain to buffer (3D SBPS variables)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, n4, n5, m2, m3, ic1, ic2, jc1, jc2
real(f4), dimension(n4,n2,n3,n5), intent(IN) :: a

real(f4), dimension(n4,m2,m3,n5), intent(INOUT) :: b

!== Local declarations:

integer :: i, j, k, ip

!=== Executable statements:

do ip=1,n5
   do j=jc1,jc2
      do i=ic1,ic2
         do k=1,n4
            b(k,i-ic1+1,j-jc1+1,ip) = a(k,i,j,ip)
         end do
      end do
   end do
end do

end subroutine fsdbuff_3Ds


!```````````````````````````````````````````````````````````````````
subroutine fsdbuff_2Ds(a, b, n2, n3, n5, m2, m3, ic1, ic2, jc1, jc2)

!+ Copies data in leaf PE subdomain to buffer (2D SBPS variables)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, n5, m2, m3, ic1, ic2, jc1, jc2
real(f4), dimension(n2,n3,n5), intent(IN) :: a

real(f4), dimension(m2,m3,n5), intent(INOUT) :: b

!== Local declarations:

integer :: i, j, ip

!=== Executable statements:

do ip=1,n5
   do j=jc1,jc2
      do i=ic1,ic2
         b(i-ic1+1,j-jc1+1,ip) = a(i,j,ip)
      end do
   end do
end do

end subroutine fsdbuff_2Ds


end module parallel_init_vars_mod


!```````````````````````````````````````````````````````````````````
subroutine rootPE_put_fields(tx_bp)

!+ Transmits all relevant initial fields to leaf PEs

use univ_kind_defs_mod, only: f4_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_transmit_packed, mpiq_pack_buffer,  &
     mpiq_check_buffer_packed, MPI_buffer_packed_obj, mpiq_transmit_plain
#endif  /* end:USE_MPI */
use main_memory_mod, only: n_grids, npatch, nzg, nzs, a, iscr2, gd,  &
     n_PE_leaf, PE_rank, n_varf
use vtables_mod, only: nmp3i, nmp2i, ind3dn, mp3tabi, ind2dn, mp2tabi,  &
     nmps3i, ind3dsn, mp3stabi, nmps2i, ind2dsn, mp2stabi, ithpn
use parallel_init_vars_mod, only: fsdbuff_2d, fsdbuff_3d, fsdbuff_2ds,  &
     fsdbuff_3ds
use topo_RT_effects_dims_mod, only: nazm

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: tx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: n, igd, nmp, idx, qxp, qyp, qxyp, qxyzp, qxys2p, qxys3p,  &
     n_packed_B, ixb, ixe, iyb, iye

!=== Executable statements:

#if defined(USE_MPI)

do n=1,n_PE_leaf
    ! Just in case the current varfile index (n_varf) has been modified (only
    !  true for a subset of calls to this routine): 
   call mpiq_transmit_plain(n_varf, PE_rank(n))  ! Transmit message to PE

   do igd=1,n_grids

      qxp = gd(igd)%leafPE_mxp(n); qyp = gd(igd)%leafPE_myp(n)
      qxyp = qxp*qyp; qxyzp = qxyp*gd(igd)%leafPE_mzp(n)

      qxys2p = qxyp*npatch; qxys3p = qxyp*(nzg+nzs)*npatch

      ! Transmit variables that the leaf PEs will only have a subdomain of:

      n_packed_B = f4_B*(qxyzp)
      do nmp=1,nmp3i(igd)
         call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
         idx = ind3dn(igd, mp3tabi(nmp,igd))
         call fsdbuff_3D(a(idx:idx+gd(igd)%nxyzp-1), a(iscr2:iscr2+qxyzp-1),  &
              gd(igd)%nzp, gd(igd)%nxp, gd(igd)%nyp, qxp, qyp,  &
              gd(igd)%leafPE_xb(n), gd(igd)%leafPE_xe(n),  &
              gd(igd)%leafPE_yb(n), gd(igd)%leafPE_ye(n))
         call mpiq_pack_buffer(a(iscr2:iscr2+qxyzp-1), qxyzp, tx_bp)
         call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE
      end do

      n_packed_B = f4_B*(qxyp)
      do nmp=1,nmp2i(igd)
         call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
         idx = ind2dn(igd,mp2tabi(nmp,igd))
         call fsdbuff_2D(a(idx:idx+gd(igd)%nxyp-1), a(iscr2:iscr2+qxyp-1),  &
              gd(igd)%nxp, gd(igd)%nyp, qxp, qyp, gd(igd)%leafPE_xb(n),  &
              gd(igd)%leafPE_xe(n), gd(igd)%leafPE_yb(n), gd(igd)%leafPE_ye(n))
         call mpiq_pack_buffer(a(iscr2:iscr2+qxyp-1), qxyp, tx_bp)
         call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE
      end do

      n_packed_B = f4_B*(qxys3p)
      do nmp=1,nmps3i(igd)
         call mpiq_check_buffer_packed(tx_bp, n_packed_B)   ! Prepare tx buffer
         idx = ind3dsn(igd,mp3stabi(nmp,igd))
         call fsdbuff_3Ds(a(idx:idx+gd(igd)%nxys3p-1),  &
              a(iscr2:iscr2+qxys3p-1), nzg+nzs, gd(igd)%nxp, gd(igd)%nyp,  &
              npatch, qxp, qyp, gd(igd)%leafPE_xb(n), gd(igd)%leafPE_xe(n),  &
              gd(igd)%leafPE_yb(n), gd(igd)%leafPE_ye(n))      
         call mpiq_pack_buffer(a(iscr2:iscr2+qxys3p-1), qxys3p, tx_bp)
         call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE
      end do

      n_packed_B = f4_B*(qxys2p)
      do nmp=1,nmps2i(igd)
         call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
         idx = ind2dsn(igd,mp2stabi(nmp,igd))
         call fsdbuff_2Ds(a(idx:idx+gd(igd)%nxys2p-1),  &
              a(iscr2:iscr2+qxys2p-1), gd(igd)%nxp, gd(igd)%nyp, npatch, qxp,  &
              qyp, gd(igd)%leafPE_xb(n), gd(igd)%leafPE_xe(n),  &
              gd(igd)%leafPE_yb(n), gd(igd)%leafPE_ye(n))
         call mpiq_pack_buffer(a(iscr2:iscr2+qxys2p-1), qxys2p, tx_bp)
         call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE
      end do

      qxp = gd(igd)%leafPE_mxp(n); qyp = gd(igd)%leafPE_myp(n)
      ixb = gd(igd)%leafPE_xb(n); ixe = gd(igd)%leafPE_xe(n)
      iyb = gd(igd)%leafPE_yb(n); iye = gd(igd)%leafPE_ye(n)

      n_packed_B = f4_B*qxp*qyp
      call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
      call mpiq_pack_buffer(gd(igd)%emisst(ixb:ixe,iyb:iye),  &
           qxp, qyp, tx_bp)
      call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE

      n_packed_B = f4_B*qxp*qyp*npatch
      call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
      call mpiq_pack_buffer(gd(igd)%emissivity_static(ixb:ixe,iyb:iye,:),  &
           qxp, qyp, npatch, tx_bp)
      call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE

      n_packed_B = f4_B*qxp*qyp*nazm

      call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
      call mpiq_pack_buffer(gd(igd)%cosz_horizon(ixb:ixe,iyb:iye,:),  &
           qxp, qyp, nazm, tx_bp)
      call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE

      call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
      call mpiq_pack_buffer(gd(igd)%TCx_obstruct(ixb:ixe,iyb:iye,:),  &
           qxp, qyp, nazm, tx_bp)
      call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE

      call mpiq_check_buffer_packed(tx_bp, n_packed_B) ! Prepare tx buffer
      call mpiq_pack_buffer(gd(igd)%TCz_obstruct(ixb:ixe,iyb:iye,:),  &
           qxp, qyp, nazm, tx_bp)
      call mpiq_transmit_packed(tx_bp, PE_rank(n))   ! Transmit message to PE
      
   end do
end do

#endif  /* end:USE_MPI */

end subroutine rootPE_put_fields


!```````````````````````````````````````````````````````````````````
subroutine leafPE_get_fields(rx_bp)

!+ Receives all relevant initial fields from the root PE

use univ_kind_defs_mod, only: f4_B
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_unpack_buffer, mpiq_receive_packed,  &
     mpiq_check_buffer_packed, MPI_buffer_packed_obj, mpiq_receive_plain
#endif  /* end:USE_MPI */
use main_memory_mod, only: a, n_grids, npatch, nzg, nzs, rootPE_rank, gd,  &
     my_rank, n_varf
use vtables_mod, only: nmp3i, nmp2i, ind3dn, mp3tabi, ind2dn, mp2tabi,  &
     nmps3i, ind3dsn, mp3stabi, nmps2i, ind2dsn, mp2stabi, ialbedtn
use topo_RT_effects_dims_mod, only: nazm

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: rx_bp
#else  /* Faux interface */
integer, intent(INOUT) :: rx_bp
#endif  /* end:USE_MPI */

!== Local declarations:

integer :: igd, nmp, idx, qxp, qyp, qxyp, qxyzp, qxys2p, qxys3p, n_packed_B

!=== Executable statements:

#if defined(USE_MPI)

! Just in case the current varfile index (n_varf) has been modified (only
!  true for a subset of calls to this routine): 
call mpiq_receive_plain(n_varf, rootPE_rank)

do igd=1,n_grids

   qxp = gd(igd)%leafPE_mxp(my_rank); qyp = gd(igd)%leafPE_myp(my_rank)
   qxyp = qxp*qyp; qxyzp = qxyp*gd(igd)%leafPE_mzp(my_rank)

   qxys2p = qxyp*npatch; qxys3p = qxyp*(nzg+nzs)*npatch

   ! Receive variables that leaf PEs will only have a subdomain of:

   n_packed_B = f4_B*(qxyzp)   
   do nmp=1,nmp3i(igd)
      call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
      idx = ind3dn(igd,mp3tabi(nmp,igd))
      call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
      call mpiq_unpack_buffer(a(idx:idx+qxyzp-1), qxyzp, rx_bp) ! Extract data
   end do
 
   n_packed_B = f4_B*(qxyp)
   do nmp=1,nmp2i(igd)
      call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
      idx = ind2dn(igd,mp2tabi(nmp,igd))
      call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
      call mpiq_unpack_buffer(a(idx:idx+qxyp-1), qxyp, rx_bp) ! Extract data
   end do
 
   n_packed_B = f4_B*(qxys3p)
   do nmp=1,nmps3i(igd)
      call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
      idx = ind3dsn(igd,mp3stabi(nmp,igd))
      call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
      call mpiq_unpack_buffer(a(idx:idx+qxys3p-1), qxys3p, rx_bp) ! Extract data
   end do
 
   n_packed_B = f4_B*(qxys2p)
   do nmp=1,nmps2i(igd)
      call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
      idx = ind2dsn(igd,mp2stabi(nmp,igd))
      call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
      call mpiq_unpack_buffer(a(idx:idx+qxys2p-1), qxys2p, rx_bp) ! Extract data
   end do

   qxp = gd(igd)%mmxp; qyp = gd(igd)%mmyp

   n_packed_B = f4_B*qxp*qyp
   call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
   call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
   call mpiq_unpack_buffer(gd(igd)%emisst, qxp, qyp, rx_bp)  ! Extract data

   n_packed_B = f4_B*qxp*qyp*npatch
   call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
   call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
   call mpiq_unpack_buffer(gd(igd)%emissivity_static, qxp, qyp, npatch,  &
        rx_bp)  ! Extract data

   n_packed_B = f4_B*qxp*qyp*nazm

   call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
   call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
   call mpiq_unpack_buffer(gd(igd)%cosz_horizon, qxp, qyp,  &  ! Extract data
        nazm, rx_bp)

   call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
   call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
   call mpiq_unpack_buffer(gd(igd)%TCx_obstruct, qxp, qyp,  &  ! Extract data
        nazm, rx_bp)

   call mpiq_check_buffer_packed(rx_bp, n_packed_B) ! Prepare receive buffer
   call mpiq_receive_packed(rx_bp, rootPE_rank)   ! Receive message
   call mpiq_unpack_buffer(gd(igd)%TCz_obstruct, qxp, qyp,  &  ! Extract data
        nazm, rx_bp)

end do

#endif  /* end:USE_MPI */

end subroutine leafPE_get_fields
