!```````````````````````````````````````````````````````````````````
module rootPE_driver_mod

!+ Module that contains PRAMS root PE routines

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine rootPE_driver(n_PE, n_leafPE, this_rank, nlst_fn, vtab_fn)

!+ PRAMS driver for the root PE

use univ_kind_defs_mod, only: f4
use univ_exception_mod, only: exception, FATAL
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_wait_for_all, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use univ_run_timing_mod, only: run_timing_obj, record_wall_time, START, END,  &
     record_CPU_time, calc_elapsed_time, display_elapsed_time
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use state_header_for_postp_mod, only: state_table_for_postp
use main_memory_mod, only: simulation_description, maxnnp, maxnxp, maxnyp,  &
     maxnzp, n_grids, gd, initialization_type, i_grid,  &
     run_type, manage_nml_var_memory, dt_dynamics, there_is_an_error,  &
     pbanner, pbnlen, a, b, memsizeA, memsizeB, PRAMS_version, vtime,  &
     varfile_name, c_gd, Cn_violation_threshold,  &
     Cn_warning_threshold, manage_grid_object, manage_misc_memory,  &
     orbital_info_table, RT_paradigm, run_is_parallel, n_PE_leaf, n_PE_total,  &
     rootPE_rank, manage_parallel_memory, my_rank, nzg, nzs, dt_mode,  &
     preserve_dt_on_restart, tstep
use main_memory_bspec_mod, only: nsndp, RTc_tog, manage_nml_var_memory_bspec,  &
     manage_grid_object_bspec, gd_bd, RT_flux_trace_np, c_gd_bd
use parallel_init_vars_mod, only: rootPE_put_misc, rootPE_put_nest_coeffs,  &
     rootPE_put_gen_grid, rootPE_put_leaf_grid, rootPE_put_root_grid,  &
     rootPE_put_vtables, rootPE_put_nlst_vars, rootPE_put_IDs
use vtables_mod, only: vtable_mem, isbpstmcd
use aermic_rad_mem_mod, only: aermicrad_AD
use aertxrad_mem_mod, only: aertxrad_AD
use ARCnRT_memory_mod, only: ARCnRT_mem_AD
use radn_mem_mod, only: radn_mem_AD
use schedule_mod, only: model_schedule
use setup_model_storage_mod, only: rootPE_memsetA, mem_setupB, mem_setupA
use aemic_drvint_mod, only: aer_setup
use PRAMS_init_mem_mod, only: dims_calc
use PRAMS_vtab_mod, only: vtable_setup
use PRAMS_chkcfg_mod, only: chkcfg1
use namelist_read_mod, only: read_namelist
use parallel_core_mod, only: leafPE_domain_decomp
use simple_astron_scheme_mod, only: sas_t_obj_recycle

implicit none

!=== Argument declarations:

integer, intent(IN) :: this_rank, n_PE, n_leafPE
character(len=*), intent(IN) :: nlst_fn, vtab_fn

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'rootPE_driver'

integer, allocatable, dimension(:) :: task_IDs

integer :: n, igd, nndtflg, e

type(run_timing_obj) :: wall_timing_run, wall_timing_init, CPU_timing_init

logical, parameter :: perform_init = .TRUE., perform_run = .TRUE.

#if defined(USE_MPI)
type(MPI_buffer_packed_obj) :: tx_buffer_packed, rx_buffer_packed
#endif  /* end:USE_MPI */

!=== Executable statements:

if (perform_init) then   ! Perform model initialization

   wall_timing_run%description = '_'
   call record_wall_time(wall_timing_run, START)  ! Get real-world start time

   wall_timing_init%description = '_'
   call record_wall_time(wall_timing_init, START)
   CPU_timing_init%description = '_'
   call record_CPU_time(CPU_timing_init, START)

   my_rank = this_rank   ! Set this here due to "early" dependencies

     ! Read model configuration namelist:
   call read_namelist(nlst_fn)

     ! Set array dimension parameter:
   maxnnp = max(maxnxp, maxnyp, maxnzp, nsndp, nzg+nzs)

     ! Print initial banner:
   write (*, '(/,78a1)') ('*', n=1,78)
   write (*, '(a1,4x,3a)') '*',  &
        'PRAMS - Version ', PRAMS_version, ' (based on RAMS v4.4)'
   write (*, '(a1)') '*'
   write (*, '(a1,4x,a)') '*', trim(simulation_description)
   write (*, '(78a1,/)') ('*', n=1,78)
   flush 6

     ! If this is a MAKE_SFC_FILES, CHECK_SFC_FILES, MAKE_VAR_FILES, or
     !  GRID_INFO_FILE run, *DO NOT* run in parallel:
   if ((run_type == 'MAKE_SFC_FILES' .or. run_type == 'CHECK_SFC_FILES' .or.  &
        run_type == 'MAKE_VAR_FILES' .or. run_type == 'GRID_INFO_FILE') .and.  &
        run_is_parallel) then
      call exception(procedure_name, FATAL,  &
           'PRAMS must be run in serial mode for a '//  &
           'GRID_INFO_FILE/MAKE_SFC_FILES/CHECK_SFC_FILES/MAKE_VAR_FILES run.')
   end if

     ! Do a further sanity check on namelist variable values:
   there_is_an_error = .FALSE.
   call chkcfg1
   if (there_is_an_error) call exception(procedure_name, FATAL)

     ! Initialize variable tables:
   call vtable_setup(vtab_fn)

     ! Allocate miscellanous, bookkeeping, etc. arrays:
   call manage_misc_memory(dim_maxnnp=.TRUE., dim_SBPS=.TRUE.,  &
        dim_isched=.TRUE., dim_vf_refs=.TRUE.)

     ! Calculate some values that are a function of NNXP, NNYP, or NNZP, etc.:
   call dims_calc

     ! Setup aerosol array dimensions, etc.:
   call aer_setup

   if (.not.run_is_parallel) then    ! Allocate non-parallel memory

      pbanner = 'SERIAL_ROOT:  '    ! Running as the serial root process
      pbnlen = len_trim(pbanner)+2

      call mem_setupA(.TRUE.)   ! Setup "A"-array indices, etc. 

   else     ! Allocate root PE memory

      call rootPE_memsetA(.TRUE.)    ! Setup "A"-array indices, etc.

   end if

   if (run_is_parallel) then
      rootPE_rank = this_rank    ! Set some module values
      n_PE_total = n_PE          !
      n_PE_leaf = n_leafPE       !
   end if

     ! Allocate grid-dependent memory (base already allocated, with component
     !  pointers nullified, in namelist processing routines):
   do igd=1,n_grids
      call manage_grid_object(gd(igd), allocate_pd_ind_pointers=.TRUE.,  &
           allocate_pd_dep_pointers=.TRUE.)
      call manage_grid_object_bspec(gd_bd(igd), gd(igd),  &
           allocate_pd_ind_pointers=.TRUE., allocate_pd_dep_pointers=.TRUE.)
   end do

   call mem_setupB(.TRUE.)    ! Setup "B"-array indices, etc. 

   allocate(a(memsizeA), b(memsizeB), stat=e)   ! Allocate main memory
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'a_b')

   write (*, '(/,a,a)') pbanner(1:pbnlen), 'Allocated "A" array...'
   write (*, '(a,a)') pbanner(1:pbnlen), 'Allocated "B" array...'
   flush 6

   a(1:memsizeA) = ZERO_f4  ! Zero out array values
   b(1:memsizeB) = ZERO_f4  !
   write (*, '(/,2a)') pbanner(1:pbnlen),  &
        'Initialized main model memory (to zero).'
   flush 6
   
     ! Call the main initialization driver:
   call initialize(nlst_fn)

   if (run_type == 'INITIAL_START' .or. run_type == 'RESTART') then

      ! Compute Courant numbers, needed for `dtset` operation below:

      if (.not.(run_type == 'RESTART' .and. dt_mode < 0 .and.  &
           preserve_dt_on_restart)) then
           ! First guess at dt_dyn for all grids, needed for `cfl` call below:
         gd(1)%dt_dyn = dt_dynamics
         do igd=2,n_grids
            gd(igd)%dt_dyn = gd(gd(igd)%nxtnest)%dt_dyn/  &
                 real(gd(igd)%n_dt_per_pgdt, kind=f4)
         end do
      end if

      do igd=1,n_grids
         i_grid = igd
         c_gd => gd(igd); c_gd_bd => gd_bd(igd)  ! Switch grid (incomplete here)

         c_gd%Courant_number_xyz = ZERO_f4   ! Initialize

         call cfl(c_gd%nzp, c_gd%nxp, c_gd%nyp, 0, 0, .TRUE.)
      end do

      write (*, '(a,i9,a)') 'Courant number metric at tstep #', 0,  &
           ' (grid; max Cn; max dt):'
      do igd=1,n_grids
         write (*, '(3x,i2,2(a,f11.5),a)')  &
              igd, ' ; ', gd(igd)%Courant_number_xyz, ' ; ',  &
              Cn_warning_threshold/max(epsilon(ONE_f4),  &
              gd(igd)%Courant_number_xyz)*gd(igd)%dt_dyn, ' [s]'
      end do
      flush 6

        ! If configured with a constant timestep, check for a CFL violation:
      if (dt_mode >= 0 .and. maxval(gd(1:n_grids)%Courant_number_xyz) >  &
           Cn_violation_threshold) then
         call exception(procedure_name, FATAL,  &
              'Initial timestep configuration violates the CFL criterion.')
      end if
   
        ! Initialize dt_dyn, n_dt_per_pgdt, and n_dta_per_dtdyn, and compute the
        !  timestepping schedule for all grid operations:
      if (run_type == 'INITIAL_START') then
         do igd=1,n_grids
            gd(igd)%timestep_OK(:) = .TRUE.
         end do
         call dtset(nndtflg, 0, 0)
      else
         call dtset(nndtflg, 1, 0)
      end if
      call model_schedule(n_grids)

        ! Initialize SBPS timestep estimate:
      do igd=1,n_grids
         ! Based on a 1 mm thick layer, with k=0.6 (~800 tiu), rho~1500.,
         !  cs=711. [SI units] -- dt < rho*cs*dz*dz/(k*2.).  A more efficient
         !  (clever?) way to do this would require a different timestep for
         !  each column/point.
         gd(igd)%dt_SBPS_estimate = min(0.75_f4,   &
              0.75_f4*0.6_f4/maxval(a(isbpstmcd:isbpstmcd+gd(igd)%nxys3p-1),  &
              mask=a(isbpstmcd:isbpstmcd+gd(igd)%nxys3p-1) < 1.e5_f4))
      end do

      call record_CPU_time(CPU_timing_init, END)
      call record_wall_time(wall_timing_init, END)

      call calc_elapsed_time(CPU_timing_init, accumulate_elapsed_time=.FALSE.)
      call calc_elapsed_time(wall_timing_init, accumulate_elapsed_time=.FALSE.)
      call display_elapsed_time(CPU_timing_init,  &
           '+++++ Basic initialization stage elapsed CPU time:')
      call display_elapsed_time(wall_timing_init,  &
           '+++++ Basic initialization stage elapsed wall time:')

      if (run_is_parallel) then

         call record_wall_time(wall_timing_init, START)
         call record_CPU_time(CPU_timing_init, START)

         ! Provide leaf PEs with relevant initialization information:

         write (*, '(/,3a)') '++++ ', pbanner(1:pbnlen),  &
              ' starting leaf PE initialization ...'
         flush 6

         allocate(task_IDs(n_PE_leaf), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'task_IDs')

         do n=1,n_leafPE    ! Loop over number of leaf PEs
            task_IDs(n) = n
         end do

         call manage_parallel_memory(dim_n_PE_leaf=.TRUE.)  ! Allocate arrays

#if defined(USE_MPI)
           ! Wait for leaf PEs to start:
         call mpiq_wait_for_all(rootPE_rank, 555)
     
           ! Transmit PE ID info to leaf PEs:
         call rootPE_put_IDs(task_IDs, tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_IDs; DONE'
         flush 6

           ! Transmit relevant namelist values to leaf PEs:
         call rootPE_put_nlst_vars(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_nlst_vars; DONE'
         flush 6

           ! Transmit variable tables to leaf PEs:
         call rootPE_put_vtables(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_vtables; DONE'
         flush 6

           ! Transmit some dimensions/flags of the root domain to leaf PEs:
         call rootPE_put_root_grid(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_root_grid; DONE'
         flush 6
#endif  /* end:USE_MPI */
      
         call manage_parallel_memory(dim_misc=.TRUE.)     ! Allocate arrays

           ! Calculate the per-leaf_PE decomposition of the root domain:
         call leafPE_domain_decomp
         write (*, '(2a)') pbanner(1:pbnlen), 'domain decomposition; DONE'
         flush 6

#if defined(USE_MPI)
           ! Transmit some dimensions/flags of leaf domains to leaf PEs:
         call rootPE_put_leaf_grid(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_leaf_grid; DONE'
         flush 6

           ! Transmit grid setup info to leaf PEs:
         call rootPE_put_gen_grid(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_gen_grid; DONE'
         flush 6

           ! Transmit nesting interpolation coefficients to leaf PEs:
         call rootPE_put_nest_coeffs(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_nest_coeffs; DONE'
         flush 6

           ! Transmit any other miscellaneous info to leaf PEs:
         call rootPE_put_misc(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_misc; DONE'
         flush 6

           ! Transmit initial fields to leaf PEs:
         call rootPE_put_fields(tx_buffer_packed)
         write (*, '(2a)') pbanner(1:pbnlen), 'put_fields; DONE'
         flush 6

           ! Wait for leaf PEs to finish initializing:
         call mpiq_wait_for_all(rootPE_rank, 555)
#endif  /* end:USE_MPI */

         call record_CPU_time(CPU_timing_init, END)
         call record_wall_time(wall_timing_init, END)

         call calc_elapsed_time(CPU_timing_init,  &
              accumulate_elapsed_time=.FALSE.)
         call calc_elapsed_time(wall_timing_init,  &
              accumulate_elapsed_time=.FALSE.)
         call display_elapsed_time(CPU_timing_init,  &
              '+++++ Leaf PE initialization stage elapsed CPU time:')
         call display_elapsed_time(wall_timing_init,  &
              '+++++ Leaf PE initialization stage elapsed wall time:')

      end if

   end if

   tstep = 0
end if

if (perform_run) then    ! Perform run
   if (run_type == 'INITIAL_START' .or. run_type == 'RESTART') then

        ! Call the model time integration driver:
      if (run_is_parallel) then
#if defined(USE_MPI)
         call rootPE_model(tx_buffer_packed, rx_buffer_packed)
#endif  /* end:USE_MPI */ 

           ! Deallocate arrays:
         call manage_parallel_memory(dim_n_PE_leaf=.FALSE., dim_misc=.FALSE.)
      else
         call serial_root_model
      end if

   end if
end if

if (.not.perform_init .and. .not.perform_run) then   ! Finalize model
   if (run_type == 'INITIAL_START' .or. run_type == 'RESTART') then

        ! Deallocate state file header postp-support table:
      if (associated(state_table_for_postp)) then
         deallocate(state_table_for_postp, stat=e)
         if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
              procedure_name, '01')
         nullify(state_table_for_postp)
      end if

        ! Deallocate radiation code memory:
      if (RT_paradigm == 2) then
         select case (RTc_tog)
         case (1)
            call radn_mem_AD(for_deallocation)
         case (2, 3)
            call aertxrad_AD(for_deallocation)
         case (4)
            call ARCnRT_mem_AD(for_deallocation)
         end select
      end if

        ! PRAMS finished, leave message passing package and deallocate
        !  memory before exiting:
      if (initialization_type == 2) then
         deallocate(varfile_name, vtime, stat=e)  ! Deallocate varfile arrays 
         if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
              procedure_name, '02')
      end if

   end if

     ! Deallocate main memory:
   deallocate(a, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '03')

   if (.NOT.run_is_parallel) then
      deallocate(b, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
           '04')
   end if

     ! Deallocate orbital info table:
   call sas_t_obj_recycle(orbital_info_table)   

     ! Deallocate namelist variables, etc.:
   call manage_misc_memory(dim_n_varftot=.FALSE., dim_maxnnp=.FALSE.,  &
        dim_SBPS=.FALSE., dim_isched=.FALSE.,  dim_vf_refs=.FALSE.)
   call aermicrad_AD(for_deallocation, 0)
   call vtable_mem(for_deallocation)

   do igd=1,n_grids
      call manage_grid_object(gd(igd), recycle_pointers=.TRUE.)
      call manage_grid_object_bspec(gd_bd(igd), gd(igd),  &
           recycle_pointers=.TRUE.)
   end do
   call manage_nml_var_memory(dim_n_grids=.FALSE.)
   call manage_nml_var_memory_bspec(dim_n_grids=.FALSE.)

   call manage_nml_var_memory(dim_maxnzp=.FALSE., dim_pnLES_np=.FALSE.,  &
        dim_nf_span=.FALSE.)
   call manage_nml_var_memory_bspec(dim_nzg=.FALSE., dim_nsndp=.FALSE.)
   if (RT_flux_trace_np > 0) call manage_nml_var_memory_bspec(  &
        dim_RT_flux_trace_np=.FALSE.)

   call record_wall_time(wall_timing_run, END)
   call calc_elapsed_time(wall_timing_run, accumulate_elapsed_time=.FALSE.)
   call display_elapsed_time(wall_timing_run,  &
        '+++++ Total wall time to complete run:')

end if

end subroutine rootPE_driver


!```````````````````````````````````````````````````````````````````
subroutine serial_root_model

!+ Model time integration driver for non-parallel run (serial root)

use univ_kind_defs_mod, only: f4, f8
use univ_math_constants_mod, only: ONE_f8, ZERO_f4, ONE_f4
use univ_run_timing_mod, only: run_timing_obj, record_wall_time, START, END,  &
     record_CPU_time, calc_elapsed_time, display_elapsed_time,  &
     elapsed_time_s_eqv
use univ_integer_time_mod, only: time_obj, fp_seconds_eqv_Time
use univ_exception_mod, only: exception, WARN, FATAL
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN
use univ_namelist_utils_mod, only: Time_to_gen_time_strrep
use main_memory_mod, only: n_grids, i_grid, gd, dt_mode, sspct, min_ssfrac,  &
     tstep, nsubs, isched, isstp, Cn_warning_threshold,  &
     Cn_violation_threshold, n_Cn_records, c_gd, current_sim_ET, my_rank,  &
     maximum_sim_ET, begin_dtdyn_sim_ET, start_sim_ET, module_tps,  &
     n_dtdyn_per_state_copy, enable_advection, prognose_uvwp, mnsdy_Time,  &
     morbpd_Time, terminate_run_now
use schedule_mod, only: model_schedule
use grid_mem_mod, only: switch_to_grid
use aemic_drvint_mod, only: aerproc_check

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'serial_root_model'

integer, parameter :: n_tsteps_per_wall_timing = 10,  & !100,  &
                      n_tsteps_per_CPU_timing = 10,  &
                      max_n_dt_redos = 25

type(time_obj) :: tmp_Time

real(f8) :: wall_time_ratio, current_sim_ET_backup
integer :: npass, nndtflg, icm, ifm, nfeed, igd, tstep_backup,  &
     tstep_with_issue, recovery_try_count
character(len=40) :: gen_time_strrep
character(len=MAX_MESSAGE_LEN) :: msg
logical :: wall_timing_this_tstep, CPU_timing_this_tstep,  &
     state_copy_this_tstep, recovering_from_issue, run_is_finished

logical, dimension(n_grids) :: dt_had_no_issues
integer, dimension(n_grids) :: RT_call_backup, n_dt_per_pgdt_backup
real(f4), dimension(n_grids) :: dt_dyn_backup, Courant_number_xyz_backup,  &
     dt_SBPS_estimate_backup
real(f4), dimension(n_Cn_records,n_grids) :: Cn_value_record_backup,  &
     Cn_change_record_backup
real(f4) :: dt_factor

type(run_timing_obj) :: wall_timing_ref, wall_timing_loop, CPU_timing_loop

!=== Executable statements:

if (tstep == 0) then
   write (*, '(/,a,/)') '******* Starting model time integration...'

   wall_timing_ref%description = '_'
   wall_timing_loop%description = '_'
   CPU_timing_loop%description = '_'

   call record_wall_time(wall_timing_ref, START) ! Determine reference wall time

   ! Initialize various items for use in the (long) timestep loop below:

   ! Make a copy of the current model state?
   if (dt_mode < 0) then
      call copy_model_state(.TRUE.)
      do igd=1,n_grids
         RT_call_backup(igd) = gd(igd)%RT_call
         dt_dyn_backup(igd) = gd(igd)%dt_dyn
         n_dt_per_pgdt_backup(igd) = gd(igd)%n_dt_per_pgdt
         Cn_value_record_backup(:,igd) = gd(igd)%Cn_value_record(:)
         Cn_change_record_backup(:,igd) = gd(igd)%Cn_change_record(:)
         Courant_number_xyz_backup(igd) = gd(igd)%Courant_number_xyz
         dt_SBPS_estimate_backup(igd) = gd(igd)%dt_SBPS_estimate
      end do
      tstep_backup = tstep+1  ! Special initial case
      current_sim_ET_backup = current_sim_ET
      write (*, '(a,i9)') 'State copy created at timestep #1'
   end if

   dt_had_no_issues(:) = .TRUE.
   tstep_with_issue = -1; recovery_try_count = 0
end if

run_is_finished = ( current_sim_ET >= maximum_sim_ET )

! Main (long) timestep loop (loop until max simulation time reached):

main_loop: do

   if (run_is_finished) exit main_loop     ! Finished

   tstep = tstep+1   ! Increment timestep
   begin_dtdyn_sim_ET = current_sim_ET  ! Store ET value at start of timestep

      ! Store status of prior timestep:
   if (tstep /= tstep_with_issue .and. recovery_try_count == 0) then
      do igd=1,n_grids
         gd(igd)%timestep_OK(n_Cn_records-1) = dt_had_no_issues(igd)
      end do
   end if

      ! Is the model currently recovering from a timestep issue?
   recovering_from_issue = ( tstep_with_issue > 0 )

      ! Is it time for certain operations to be performed?
   wall_timing_this_tstep =  &
        ( tstep == 1 .or. mod(tstep, n_tsteps_per_wall_timing) == 0 )
   CPU_timing_this_tstep =  &
        ( tstep == 1 .or. mod(tstep, n_tsteps_per_CPU_timing) == 0 )
   state_copy_this_tstep =  &
        ( dt_mode < 0 .and. mod(tstep, n_dtdyn_per_state_copy) == 0 .and.  &
        tstep_backup /= tstep )
   state_copy_this_tstep =  &
        ( state_copy_this_tstep .or.  &
        (tstep == tstep_with_issue .and. tstep_backup /= tstep) )

      ! Begin timings:
   if (wall_timing_this_tstep) call record_wall_time(wall_timing_loop, START)
   if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing_loop, START)

      ! Copy the current model state?
   if (state_copy_this_tstep) then
      call copy_model_state(.TRUE.)
      do igd=1,n_grids
         RT_call_backup(igd) = gd(igd)%RT_call
         dt_dyn_backup(igd) = gd(igd)%dt_dyn
         n_dt_per_pgdt_backup(igd) = gd(igd)%n_dt_per_pgdt
         Cn_value_record_backup(:,igd) = gd(igd)%Cn_value_record(:)
         Cn_change_record_backup(:,igd) = gd(igd)%Cn_change_record(:)
         Courant_number_xyz_backup(igd) = gd(igd)%Courant_number_xyz
         dt_SBPS_estimate_backup(igd) = gd(igd)%dt_SBPS_estimate
      end do
      tstep_backup = tstep
      current_sim_ET_backup = current_sim_ET
      write (*, '(a,i9)') 'State copy created at timestep #', tstep
   end if
   
      ! Set dynamics (long) timestep, and/or redetermine the scheduling
      !  table(s):  
   call dtset(nndtflg, 2, recovery_try_count)
   if (nndtflg > 0) call model_schedule(n_grids)

      ! Check if sound speed fraction is < the minimum value; if so, trigger
      !  graceful model stop:
   if (sspct < min_ssfrac-epsilon(ZERO_f4)) then
      write (msg, '(a,f8.3,a,f8.3,a)') 'Sound speed fraction (', sspct,  &
           ') is < the minimum value(', min_ssfrac, ').' 
      call exception(procedure_name, FATAL, trim(msg))
!@      terminate_run_now = .TRUE.
   end if

   ! Determine SBPS timestep parameters:
   do igd=1,n_grids
         ! # of iterations per long (grid 1) timestep:
      gd(igd)%n_iter_SBPS = max(1,  &
           nint(gd(igd)%dt_dyn/gd(igd)%dt_SBPS_estimate))
      dt_factor = ONE_f4/real(gd(igd)%n_iter_SBPS, kind=f4)
      gd(igd)%dt_SBPS = gd(igd)%dt_dyn*dt_factor      ! Actual SBPS timestep
   end do

      ! Initial status for this timestep:
   gd(1:n_grids)%timestep_OK(n_Cn_records) = .TRUE.
   if (tstep /= tstep_with_issue .and. recovery_try_count <= 1)  &
        dt_had_no_issues(:) = .TRUE.

      ! Manage model process overrides for aerosol variables:
   if (tstep > 1) then
      call aerproc_check(.FALSE.) 
   else
      call aerproc_check(.TRUE.)    ! Initialize
   end if

      ! Initialize maximum xyz Courant number storage:
   gd(1:n_grids)%Courant_number_xyz = -ONE_f4

   ! Start the timestep schedule to loop through all grids and advance them
   ! in time an increment equal to gd(1)%dt_dyn:

   npass_loop: do npass=1,nsubs

      isstp = isched(npass,3)   ! Substep #

         ! Switch indices to new grid (current grid to step fwd in time):
      call switch_to_grid(isched(npass,1))

         ! Current (sub)time:
      current_sim_ET = begin_dtdyn_sim_ET+  & 
           real((isched(npass,5)-1)*c_gd%dt_dyn, kind=f8)

         ! Set parallel processing indices to serial values:
      call set_serial_root_indices
      
         ! Execute one timestep's operations:
      call timestep

         ! CFL violation check:
      if (enable_advection .or. prognose_uvwp) then  ! Only if needed
         if (c_gd%Courant_number_xyz > Cn_violation_threshold) then
            c_gd%timestep_OK(n_Cn_records) = .FALSE.
            dt_had_no_issues(i_grid) = .FALSE.
            exit npass_loop
         end if
      end if

         ! Set a legacy (?) parameter:
      c_gd%ngbegun = 1

         ! Copy relevant parent grid points for use as finer grid BCs:
      if (isched(npass,2) /= 0) then
         call switch_to_grid(isched(npass,2))
         isstp = isched(npass,3)
         icm = c_gd%nxtnest
         call prgintrp(gd(icm)%nzp, gd(icm)%nxp, gd(icm)%nyp, gd(icm)%nzp,  &
              gd(icm)%nxp, gd(icm)%nyp, 0, 0, i_grid, 0, my_rank)
      end if

         ! Feedback fields from fine grids to coarser grids:
      if (isched(npass,4) /= 0) then
         ifm = isched(npass,1)
         icm = gd(ifm)%nxtnest
         do nfeed=1,isched(npass,4)
            call switch_to_grid(ifm)
            call nest_feedback(ifm, icm)
!%            call movenest(a, ifm, icm)
            ifm = icm
            icm = gd(ifm)%nxtnest
         end do
      end if

   end do npass_loop

   write (*, '(a,i9,a)') 'Courant number metric at tstep #', tstep,  &
        ' (grid; max Cn; max dt):'
   do ifm=1,n_grids
      write (*, '(3x,i2,2(a,f11.5),a)')  &
           ifm, ' ; ', gd(ifm)%Courant_number_xyz, ' ; ',  &
           Cn_warning_threshold/max(epsilon(ONE_f4),  &
           gd(ifm)%Courant_number_xyz)*gd(ifm)%dt_dyn, ' [s]'
   end do

   ! Check if this timestep completed satisfactorily, and act accordingly:

   if (all(gd(1:n_grids)%timestep_OK(n_Cn_records))) then  ! Continue normally
      if (.NOT.recovering_from_issue) then
         recovery_try_count = 0
      else if (tstep == tstep_with_issue) then  ! Timestep issue is now resolved
         tstep_with_issue = -1
         recovering_from_issue = .FALSE.
         recovery_try_count = 0
      end if
   else           ! This timestep had an issue -- stop or redo?
      tstep_with_issue = tstep
      recovery_try_count = recovery_try_count+1
      if (dt_mode >= 0) then              ! No redo possible
         terminate_run_now = .TRUE.     ! Stop model run
      else                                ! Redo possible
         write (*, '(a,i9,a,i3,a,i9)') 'Will redo timestep #',  &
              tstep_with_issue, ' (try #', recovery_try_count,  &
              '); reverting to the start of timestep #', tstep_backup; flush 6
         if (recovery_try_count == max_n_dt_redos) then
            call exception(procedure_name, WARN,  &
                 'Too many timestep redos -- not converging.')
            terminate_run_now = .TRUE.   ! Stop model run
         else             ! Revert to a prior model state
            call copy_model_state(.FALSE.) ! Revert model state to backup copy
            tstep = tstep_backup-1
            gd(1:n_grids)%RT_call = RT_call_backup(:)
            if (tstep_backup /= tstep_with_issue) then ! For longer tstep span
               gd(1:n_grids)%dt_dyn = dt_dyn_backup(:)
               gd(1:n_grids)%dt_SBPS_estimate = dt_SBPS_estimate_backup
               do igd=1,n_grids
                  n_dt_per_pgdt_backup(igd) = gd(igd)%n_dt_per_pgdt
                  Cn_value_record_backup(:,igd) = gd(igd)%Cn_value_record(:)
                  Cn_change_record_backup(:,igd) = gd(igd)%Cn_change_record(:)
                  Courant_number_xyz_backup(igd) = gd(igd)%Courant_number_xyz
               end do
            end if
            current_sim_ET = current_sim_ET_backup ! Reset elapsed time
            cycle main_loop
         end if
      end if
   end if
   
   current_sim_ET = begin_dtdyn_sim_ET+real(gd(1)%dt_dyn, kind=f8)
   run_is_finished = ( current_sim_ET >= maximum_sim_ET )

   ! Output state file(s), and/or switch varfiles:
   call state_output(terminate_run_now, .FALSE.)

   ! Begin a "graceful" model run shutdown in the event of a severe anomaly:
   if (terminate_run_now) exit main_loop

   ! Finish and display timings (perhaps):

   if (CPU_timing_this_tstep) then
      call record_CPU_time(CPU_timing_loop, END)
      call calc_elapsed_time(CPU_timing_loop, accumulate_elapsed_time=.FALSE.)
      tmp_Time = fp_seconds_eqv_Time(current_sim_ET, module_tps)
      call Time_to_gen_time_strrep(tmp_Time, gen_time_strrep,  &
           len(gen_time_strrep), mnsdy_Time, morbpd_Time,  &
           short_unit_style=.TRUE.)
      msg = '+++++ ['//trim(gen_time_strrep)//'] CPU timing:'
      call display_elapsed_time(CPU_timing_loop, trim(msg))
   else
      write (*, '(a,i9)') '+++++ (run) TIMESTEP #', tstep
   end if

   if (wall_timing_this_tstep) then
      call record_wall_time(wall_timing_loop, END)
      call calc_elapsed_time(wall_timing_loop, accumulate_elapsed_time=.FALSE.)
      call display_elapsed_time(wall_timing_loop,  &
        '+++++ Elapsed wall time for current timestep:')

      call record_wall_time(wall_timing_ref, END)
      call calc_elapsed_time(wall_timing_ref, accumulate_elapsed_time=.FALSE.)
      wall_time_ratio = max(epsilon(ONE_f8), (current_sim_ET-start_sim_ET)/  &
           max(epsilon(ONE_f8), elapsed_time_s_eqv(wall_timing_ref)))
      if (wall_time_ratio < ONE_f8) then
         write (msg, '(f15.2)') ONE_f8/wall_time_ratio
         write (*, '(a,i1,a3,a)')  &
              '+++++ Average simulated:wall time ratio since run start -- ',  &
              1, ' : ', trim(adjustl(msg))
      else
         write (msg, '(f15.2)') wall_time_ratio
         write (*, '(2a,a3,i1)')  &
              '+++++ Average simulated:wall time ratio since run start -- ',  &
              trim(adjustl(msg)), ' : ', 1
      end if
   end if

end do main_loop    ! End of time integration loop

end subroutine serial_root_model


!```````````````````````````````````````````````````````````````````
subroutine rootPE_model(tx_buffer_packed, rx_buffer_packed)

!+ Model time integration driver for a root PE

use univ_kind_defs_mod, only: f4, f8
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_run_timing_mod, only: run_timing_obj, record_wall_time, START, END,  &
     record_CPU_time, calc_elapsed_time, display_elapsed_time,  &
     elapsed_time_s_eqv
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_MESSAGE_LEN
use univ_integer_time_mod, only: time_obj, fp_seconds_eqv_Time
use univ_math_constants_mod, only: ONE_f4, ONE_f8, ZERO_f4
#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: mpiq_wait_for_all, MPI_buffer_packed_obj
#endif  /* end:USE_MPI */
use univ_exception_mod, only: exception, FATAL
use univ_namelist_utils_mod, only: Time_to_gen_time_strrep
use main_memory_mod, only: n_grids, gd, tstep, pbanner, pbnlen,  &
     current_sim_ET, maximum_sim_ET, n_PE_leaf, rootPE_rank,  &
     n_Cn_records, Cn_warning_threshold, min_ssfrac, module_tps, sspct,  &
     start_sim_ET, Cn_violation_threshold, terminate_run_now, mnsdy_Time,  &
     morbpd_Time
use schedule_mod, only: model_schedule
use parallel_timing_mod, only: rootPE_put_dtsched, rootPE_get_timings
use parallel_core_mod, only: check_leafPE_and_rootPE_comm

implicit none

!=== Argument declarations:

#if defined(USE_MPI)
type(MPI_buffer_packed_obj), intent(INOUT) :: tx_buffer_packed, rx_buffer_packed
#else  /* Faux interface */
integer, intent(INOUT) :: tx_buffer_packed, rx_buffer_packed
#endif  /* end:USE_MPI */

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'rootPE_model'

integer, parameter :: n_tsteps_per_wall_timing = 10,  & !100,  &
                      n_tsteps_per_CPU_timing = 10

real(f4), allocatable, dimension(:,:) :: leafPE_times

type(run_timing_obj) :: wall_timing_ref, wall_timing_loop, CPU_timing_loop

integer :: a_err, nndtflg, igd

logical :: wall_timing_this_tstep, CPU_timing_this_tstep, run_is_finished,  &
     all_results_from_leafPE_now, all_fields_to_leafPE_now

real(f8) :: wall_time_ratio
type(time_obj) :: tmp_Time
character(len=40) :: gen_time_strrep
character(len=MAX_MESSAGE_LEN) :: msg
real(f4) :: dt_factor

!=== Executable statements:

! Allocate leaf PE timings array:
allocate(leafPE_times(n_PE_leaf,2), stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
     'leafPE_times')

write (*, '(/,2a,/)') pbanner(1:pbnlen), 'starting model time integration...'
flush 6

wall_timing_ref%description = '_'
wall_timing_loop%description = '_'
CPU_timing_loop%description = '_'

call record_wall_time(wall_timing_ref, START)  ! Determine reference wall time

! Initialize various items for use in the main (long) timestep loop below:

tstep = 0

run_is_finished = ( current_sim_ET >= maximum_sim_ET )

all_results_from_leafPE_now = .FALSE.; all_fields_to_leafPE_now = .FALSE.

! Main (long) timestep loop (loop until max simulation time reached):

main_loop: do

   if (run_is_finished) exit main_loop     ! Finished

   tstep = tstep+1   ! Increment timestep

   wall_timing_this_tstep = ( mod(tstep, n_tsteps_per_wall_timing) == 0 )
   CPU_timing_this_tstep = ( mod(tstep, n_tsteps_per_CPU_timing) == 0 )

      ! Begin timings:
   if (wall_timing_this_tstep) call record_wall_time(wall_timing_loop, START)
   if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing_loop, START)

      ! Determine if leaf PEs transmit certain data back at the END of *this*
      !  timestep:
   all_results_from_leafPE_now = .FALSE.
   all_fields_to_leafPE_now = .FALSE.
   call check_leafPE_and_rootPE_comm(all_results_from_leafPE_now,  &
        all_fields_to_leafPE_now)

      ! Examine Courant numbers in case model needs to be stopped
      !  or (if dt_mode < 0), to update dt_dyn, n_dt_per_parent_grid_dt,
      !  nnacoust, sspct and isched:
   call dtset(nndtflg, 2, 0) ! last arg kludged!!!
   if (nndtflg > 0) call model_schedule(n_grids)

      ! Check if sound speed fraction is < the minimum value; if so, trigger
      !  graceful model stop:
   if (sspct < min_ssfrac-epsilon(ZERO_f4)) then
      write (msg, '(a,f8.3,a,f8.3,a)') 'Sound speed fraction (', sspct,  &
           ') is < the minimum value(', min_ssfrac, ').' 
      call exception(procedure_name, FATAL, trim(msg))
!@      terminate_run_now = .TRUE.
   end if

   ! Determine SBPS timestep parameters:
   do igd=1,n_grids
         ! # of iterations per long (grid 1) timestep:
      gd(igd)%n_iter_SBPS = max(1,  &
           nint(gd(igd)%dt_dyn/gd(igd)%dt_SBPS_estimate))
      dt_factor = ONE_f4/real(gd(igd)%n_iter_SBPS, kind=f4)
      gd(igd)%dt_SBPS = gd(igd)%dt_dyn*dt_factor      ! Actual SBPS timestep
   end do

   if (terminate_run_now) then ! If run will be aborted, get stuff from leaf PEs
      all_results_from_leafPE_now = .TRUE.
   end if

#if defined(USE_MPI)
   call rootPE_put_dtsched(all_results_from_leafPE_now,  &
        all_fields_to_leafPE_now, tx_buffer_packed)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
   write (*, '(/,2a)') pbanner(1:pbnlen), 'rootPE_put_dtsched; DONE'
   flush 6
#endif  /* end:MPI_DEBUG */

      ! Initial status for this timestep:
   gd(1:n_grids)%timestep_OK(n_Cn_records) = .TRUE. ! Init for this timestep

      ! Get CPUtime, wallclock time, and CFL numbers from leaf PEs:
#if defined(USE_MPI)
   call rootPE_get_timings(leafPE_times, rx_buffer_packed)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
   write (*, '(/,2a)') pbanner(1:pbnlen), 'rootPE_get_timings; DONE'
   flush 6
#endif  /* end:MPI_DEBUG */

   write (*, '(a,i9,a)') 'Courant number metric at tstep #', tstep,  &
        ' (grid; max Cn; max dt):'
   do igd=1,n_grids
      if (gd(igd)%Courant_number_xyz > Cn_violation_threshold) then
         gd(igd)%timestep_OK(n_Cn_records) = .FALSE.
      end if

      write (*, '(3x,i2,2(a,f11.5),a)')  &
           igd, ' ; ', gd(igd)%Courant_number_xyz, ' ; ',  &
           Cn_warning_threshold/max(epsilon(ONE_f4),  &
           gd(igd)%Courant_number_xyz)*gd(igd)%dt_dyn, ' [s]'
   end do

      ! Check if this timestep completed satisfactorily; if so, exit this
      !  inner loop and proceed to the next timestep:
   if (all(gd(1:n_grids)%timestep_OK(n_Cn_records))) then
      terminate_run_now = .FALSE.
   else      ! This timestep is broken
      terminate_run_now = .TRUE.
   end if

   if (all_results_from_leafPE_now) then
      ! Wait to get whole subdomains from leaf PEs:
      call rootPE_get_all_results

#if defined(USE_MPI)
      call mpiq_wait_for_all(rootPE_rank, 99999)  ! Wait for completion
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
      write (*, '(/,2a)') pbanner(1:pbnlen), 'rootPE_get_all_results; DONE'
      flush 6
#endif  /* end:MPI_DEBUG */
   end if

   current_sim_ET = current_sim_ET+real(gd(1)%dt_dyn, kind=f8)
   run_is_finished = ( current_sim_ET >= maximum_sim_ET )

   ! Output state file(s), and/or switch varfiles:

   call state_output(terminate_run_now, .FALSE.)
#if defined(MPI_DEBUG)
   write (*, '(/,2a)') pbanner(1:pbnlen), 'state_output; DONE'
   flush 6
#endif  /* end:MPI_DEBUG */

   ! Begin a "graceful" model run shutdown in the event of a severe anomaly:
   if (terminate_run_now) exit main_loop

   ! Reinitialize subdomains (this part is for the dynamic balancing, sending
   !  new varfile info, and model state copying):

   if (all_fields_to_leafPE_now .and. .not.run_is_finished) then
!      call "leafPE_decomp()"
#if defined(USE_MPI)
      call rootPE_put_fields(tx_buffer_packed)
#endif  /* end:USE_MPI */
#if defined(MPI_DEBUG)
      write (*, '(/,2a)') pbanner(1:pbnlen), 'rootPE_put_fields; DONE'
      flush 6
#endif  /* end:MPI_DEBUG */
   end if

   ! Finish and display timings (perhaps):

   if (CPU_timing_this_tstep) then
      call record_CPU_time(CPU_timing_loop, END)
      call calc_elapsed_time(CPU_timing_loop, accumulate_elapsed_time=.FALSE.)
      tmp_Time = fp_seconds_eqv_Time(current_sim_ET, module_tps)
      call Time_to_gen_time_strrep(tmp_Time, gen_time_strrep,  &
           len(gen_time_strrep), mnsdy_Time, morbpd_Time,  &
           short_unit_style=.TRUE.)
      msg = '+++++ ['//trim(gen_time_strrep)//'] CPU timing:'
      call display_elapsed_time(CPU_timing_loop, trim(msg))
   else
      write (*, '(a,i9)') '+++++ (run) TIMESTEP #', tstep
   end if

   if (wall_timing_this_tstep) then
      call record_wall_time(wall_timing_loop, END)
      call calc_elapsed_time(wall_timing_loop, accumulate_elapsed_time=.FALSE.)
      call display_elapsed_time(wall_timing_loop,  &
        '+++++ Elapsed wall time for current timestep:')

      call record_wall_time(wall_timing_ref, END)
      call calc_elapsed_time(wall_timing_ref, accumulate_elapsed_time=.FALSE.)
      wall_time_ratio = max(epsilon(ONE_f8), (current_sim_ET-start_sim_ET)/  &
           max(epsilon(ONE_f8), elapsed_time_s_eqv(wall_timing_ref)))
      if (wall_time_ratio < ONE_f8) then
         write (msg, '(f15.2)') ONE_f8/wall_time_ratio
         write (*, '(a,i1,a3,a)')  &
              '+++++ Average simulated:wall time ratio since run start -- ',  &
              1, ' : ', trim(adjustl(msg))
      else
         write (msg, '(f15.2)') wall_time_ratio
         write (*, '(2a,a3,i1)')  &
              '+++++ Average simulated:wall time ratio since run start -- ',  &
              trim(adjustl(msg)), ' : ', 1
      end if
   end if

end do main_loop

! Deallocate leaf PE timings array:
deallocate(leafPE_times, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'leafPE_times')

end subroutine rootPE_model


!```````````````````````````````````````````````````````````````````
subroutine set_serial_root_indices

!+ Initializes parallel processing indices/control variables for a serial run

use main_memory_mod, only: c_gd, mxp, myp, mzp, ia, iz, ja, jz, i0, j0,  &
     ibndc, ia_1, ia_2, ia_3, ia1, ia2, ia3, ja_1, ja_2, ja_3, ja1, ja2, ja3,  &
     iz_1, iz_2, iz_3, iz1, iz2, iz3, jz_1, jz_2, jz_3, jz1, jz2, jz3, izu, jzv

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

mxp = c_gd%nxp; myp = c_gd%nyp; mzp = c_gd%nzp

! x-direction indices:

ia = 2; iz = mxp-1
ia_1 = max(ia-1,1); ia_2 = max(ia-2,1); ia_3 = max(ia-3,1)
ia1 = ia+1; ia2 = ia+2; ia3 = ia+3

iz_1 = iz-1; iz_2 = iz-2; iz_3 = iz-3
iz1 = min(iz+1,mxp); iz2 = min(iz+2,mxp); iz3 = min(iz+3,mxp)
izu = iz-1

! y-direction indices:

if (c_gd%nyp > 1) then
   ja = 2; jz = myp-1
   ja_1 = max(ja-1,1); ja_2 = max(ja-2,1); ja_3 = max(ja-3,1)
   ja1 = ja+1; ja2 = ja+2; ja3 = ja+3

   jz_1 = jz-1; jz_2 = jz-2; jz_3 = jz-3
   jz1 = min(jz+1,myp); jz2 = min(jz+2,myp); jz3 = min(jz+3,myp)
   jzv = jz-1
else
   ja = 1; jz = 1
   ja_1 = 1; ja_2 = 1; ja_3 = 1
   ja1 = 1; ja2 = 1; ja3 = 1

   jz_1 = 1; jz_2 = 1; jz_3 = 1
   jz1 = 1; jz2 = 1; jz3 = 1
   jzv = 1
end if

i0 = 0; j0 = 0
ibndc = 1+2+4+8

end subroutine set_serial_root_indices


end module rootPE_driver_mod
