!```````````````````````````````````````````````````````````````````
module schedule_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine model_schedule(n_grids)

!+ Determine model timestep schedule

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: nsubs, isched, n_isched_steps, gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: n_grids

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'model_schedule'

integer, dimension(n_grids) :: ncnt, nnfm
integer :: igd, igrid, nent, nstep, is, isstp, n_isched

!=== Executable statements:

do igd=1,n_grids
   nnfm(igd) = 1
   ncnt(igd) = 0
end do

n_isched = size(isched, dim=1)
do nent=1,5
   do nstep=1,n_isched
      isched(nstep,nent) = 0
   end do
end do

is = 0   ! Initialize step counter
do igd=1,n_grids

   if (gd(igd)%nxtnest == 0) then

      igrid = igd
      
30    is = is+1  ! Increment step counter
      if (is > n_isched) call exception(procedure_name, FATAL,  &
           'Bounds of scheduling array, isched, have been exceeded.')
      isstp = mod(ncnt(igrid), gd(igrid)%n_dt_per_pgdt)+1  ! Increment substep counter

      ! Table entry #1 is the grid number to be stepped forward:
      isched(is,1) = igrid

      ! Table entry #3 is the substep counter:
      isched(is,3) = isstp
         
      ncnt(igrid) = ncnt(igrid)+1   ! Increment total substep counter
      ! Table entry #5 is the total substep counter:
      isched(is,5) = ncnt(igrid)

      ! Look for next nested grid of current grid; don't exceed N_GRIDS:

40    nnfm(igrid) = nnfm(igrid)+1
      if (nnfm(igrid) > n_grids) then
         goto 50
      else if (gd(nnfm(igrid))%nxtnest /= igrid) then
         goto 40
      end if

      igrid = nnfm(igrid)   ! Make nested grid the current grid
         
      ! Table entry #2 is the grid # to interpolate; 0 = no interpolation:
      isched(is,2) = igrid
         
      goto 30
   
50    nnfm(igrid) = 1
      if (gd(igrid)%nxtnest == 0) goto 80
      if (ncnt(igrid) < ncnt(gd(igrid)%nxtnest)*gd(igrid)%n_dt_per_pgdt) goto 30

      ! Table entry #4 is the number of grids to feed back:
      isched(is,4) = isched(is,4)+1

      igrid=gd(igrid)%nxtnest

      goto 40

80    continue

   end if
end do

nsubs = is

if (nsubs > n_isched_steps) call exception(procedure_name, FATAL,  &
     'Must increase value of parameter n_isched_steps (in the source code)')

! Display model schedule (albeit cryptic):

write (*, '(/,3x,a)') '------- Timestepping Schedule -------'
write (*, '(/a)')   &
     'g#_to_stepfwd  g#_to_intrp  substep  #g_to_fdback  tot_substeps'
do nstep=1,is
   write (*, '(4x,i5,8x,i5,7x,i5,8x,i5,8x,i5)') (isched(nstep,nent), nent=1,5)
end do

end subroutine model_schedule


!```````````````````````````````````````````````````````````````````
function value_gp(n1, n2, n3, k, i, j, a)  result(data_value)

!+ "Extract" data value, given array and gridpoint indices

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, k, i, j
real(f4), dimension(n1,n2,n3), intent(IN) :: a

real(f4) :: data_value

!== Local declarations:

! NONE

!=== Executable statements:

data_value = a(k,i,j)

end function value_gp


end module schedule_mod


!```````````````````````````````````````````````````````````````````
subroutine dtset(nndtflg, mode, n_redo_iterations)

!+ Set model timestep(s) and related values

use univ_kind_defs_mod, only: f4
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use main_memory_mod, only: a, tstep, n_substeps_max_curr, n_grids, sspct,  &
     substep_austerity_level, n_Cn_records, Cn_warning_threshold, dt_mode,  &
     n_dta_per_dtdyn, gd, n_substeps_max, preserve_dt_on_restart,  &
     dt_max_frac_increase_g1, min_ssfrac, max_dt_dynamics
use body_specific_constants_mod, only: cpi, R_air, cp_o_cv
use vtables_mod, only: idxtn, ithpn, ippn, ipi0n

implicit none

!=== Argument declarations:

integer, intent(IN) :: mode, n_redo_iterations

integer, intent(INOUT) :: nndtflg

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'dtset'

real(f4), parameter :: dt_min = 1.e-5_f4   ! [s]
real(f4), parameter :: Cn_target_default = Cn_warning_threshold,  &
                       Cn_tgt_stepdec_slow = 0.005_f4,  & ! -0.08 / 16 substeps
                       Cn_tgt_stepdec_fast = 0.0025_f4    ! -0.08 / 32 substeps

integer, dimension(n_grids) :: n_dt_per_pgdt_copy
real(f4), dimension(n_grids) :: ss_Courant_number, dt_dyn_copy, Cn_target,  &
     dtlongn_basis, nsspct
integer, dimension(2,n_grids) :: n_grid_substeps
real(f4) :: ss_max, ftmp, Cn_max_curr_g, Cn_target_default_curr,  &
     dt_max_frac_incr_curr_g, max_factor, substep_factor_curr_g
integer :: ifm, icm, iq, ic, iteration, max_n_substeps_for_any_grid
logical :: redo_g1_dt_adj

!=== Executable statements:

! Mode: 0 = model initialization, runtype 'INITIAL'; placeholder dt_dyn
!           already set in `rootPE_driver`, but dt_dyn, n_dta_per_dtdyn, sspct, 
!           substep_austerity_level (if IDELTAT < 0), and n_dt_per_pgdt (if
!           DT_MODE /= 0) need to be set in this routine.
!       1 = model initialization, runtype 'RESTART'; dt_dyn, n_dta_per_dtdyn,
!           n_dt_per_pgdt, and sspct have been already filled
!           from the restart file, but may be modified below.
!       2 = during run's time integration loop (mode 0 or 1 should have been
!           performed already); if DT_MODE < 0, dt_dyn, n_dta_per_dtdyn,
!           n_dt_per_pgdt, substep_austerity_level, and sspct may need to be
!           modified -- otherwise, update just sspct.

if (mode < 2) then    ! For all initialization modes
   ! Maximum 1/dx (and by enforced symmetry, 1/dy as well) on each grid:
   do ifm=1,n_grids
      gd(ifm)%dxyt_max = maxval(a(idxtn(ifm):idxtn(ifm)+gd(ifm)%nxyp-1))
   end do

   if (dt_mode < 0) substep_austerity_level = max(ZERO_f4,  &
        real(abs(dt_mode)-1, kind=f4))
end if

! Compute current max sound speed metric:
do ifm=1,n_grids
   ss_max = ZERO_f4
   do iq=0,gd(ifm)%nxyzp-1
      ftmp = a(ithpn(ifm)+iq)*(a(ippn(ifm)+iq)+a(ipi0n(ifm)+iq))  ! T*cp
      ss_max = max(ss_max, ftmp)    ! max T*cp for now
   end do
   ss_max = ss_max*cpi   ! max T now
   ss_max = sqrt(cp_o_cv*R_air*ss_max)   ! Current max sound speed on this grid

   gd(ifm)%ss_over_dxy = ss_max*gd(ifm)%dxyt_max
end do

! Initialize various parameters:
if (mode <= 1) then
   ! NOTE: Placeholder dt_dyn values were already set in `rootPE_driver`

   if (dt_mode == 0) then     ! Constant timesteps (specified)

      gd(1)%n_dta_per_dtdyn = n_dta_per_dtdyn
      do ifm=2,n_grids
         icm = gd(ifm)%nxtnest
         gd(ifm)%n_dta_per_dtdyn = n_dta_per_dtdyn
      end do
      
   else if (dt_mode > 0) then  ! Constant timesteps (computed) *UNFINISHED*

      call exception(procedure_name, FATAL, 'DT_MODE > 0 is not yet supported.')

   else                  ! Variable timesteps (computed)

      if (mode == 0 .or. (mode == 1 .and. .not.preserve_dt_on_restart)) then
         do ifm=1,n_grids
            gd(ifm)%Cn_value_record(:) = Cn_target_default
            gd(ifm)%Cn_change_record(:) = ZERO_f4
            gd(ifm)%timestep_OK(:) = .TRUE.
            gd(ifm)%n_dta_per_dtdyn = n_dta_per_dtdyn  ! Constant throughout run
         end do
      end if

   end if

end if

! Adjust timestep, if desired:

nndtflg = 0

if (dt_mode < 0) then     ! Variable (computed) timesteps
   if (mode == 2 .and. tstep == 1) return  ! Already made one pass in mode 0/1

   ! Save a copy of previous values:
   n_dt_per_pgdt_copy(1:n_grids) = gd(1:n_grids)%n_dt_per_pgdt
   dt_dyn_copy(1:n_grids) = gd(1:n_grids)%dt_dyn
   do ic=1,n_Cn_records-1
      do ifm=1,n_grids
         gd(ifm)%Cn_value_record(ic) = gd(ifm)%Cn_value_record(ic+1)
         gd(ifm)%Cn_change_record(ic) = gd(ifm)%Cn_change_record(ic+1)
      end do
   end do

   ! Current maximum Courant number on grid #1: 
   Cn_max_curr_g = gd(1)%Courant_number_xyz
   Cn_max_curr_g = max(epsilon(ONE_f4), Cn_max_curr_g)   ! Arbitrary minimum
   gd(1)%Cn_value_record(n_Cn_records) = Cn_max_curr_g
   gd(1)%Cn_change_record(n_Cn_records) =  &
        gd(1)%Cn_value_record(n_Cn_records)-  &
        gd(1)%Cn_value_record(n_Cn_records-1)

   ! Set some current "default" values:
   Cn_target_default_curr = Cn_target_default
   n_substeps_max_curr = n_substeps_max
   substep_factor_curr_g = real(gd(1)%n_dta_per_dtdyn, kind=f4)*  &
        Cn_tgt_stepdec_fast

   ! Determine maximum fractional amount to increment timestep by (decreases
   !  sharply as 'Cn_target_default_curr' is neared/surpassed):
   if (Cn_max_curr_g <= 0.75_f4*Cn_target_default_curr) then
      dt_max_frac_incr_curr_g = dt_max_frac_increase_g1   ! Full throttle
!^print *,'Initial triage (full throttle)',dt_max_frac_incr_curr_g
   else                                             ! Throttle back
      dt_max_frac_incr_curr_g = dt_max_frac_increase_g1*  &
           (Cn_target_default_curr-  &
           min(Cn_max_curr_g, 0.99_f4*Cn_target_default_curr)-0.01_f4)/  &
           Cn_target_default_curr
!^print *,'Initial triage (throttle back)',dt_max_frac_incr_curr_g
   end if


   ! First guess at long timestep on grid #1: 

   Cn_target(1) = max(0.1_f4*Cn_target_default_curr,  &
        Cn_target_default_curr-  &
        substep_austerity_level*substep_factor_curr_g)
!^print *,'First guess at Cn_target(1)',Cn_target(1)

!^print *,'Perform grid 1 dt_dyn block?',  &
!^     .NOT.(n_redo_iterations > 0 .and. gd(1)%timestep_OK(n_Cn_records)),  &
!^     n_redo_iterations > 0, gd(1)%timestep_OK(n_Cn_records)

   if (.NOT.(n_redo_iterations > 0 .and. gd(1)%timestep_OK(n_Cn_records))) then 

      if (Cn_max_curr_g < Cn_target(1)) then
         ftmp = dt_max_frac_incr_curr_g
         ! Throttle if Courant number is increasing too quickly (could be a
         !  rapidly amplifying simulated phenomenon):
         if (gd(1)%Cn_change_record(n_Cn_records) >  &
              dt_max_frac_increase_g1*2._f4 .and.  &
              gd(1)%Cn_value_record(n_Cn_records) > 0.5_f4*Cn_target(1)) then
            ftmp = substep_factor_curr_g
            ftmp = (-0.01_f4-0.2_f4*ftmp)*0.5_f4*  &
                 gd(1)%Cn_change_record(n_Cn_records)
         end if
         gd(1)%dt_dyn = min(ONE_f4+ftmp, Cn_target(1)/Cn_max_curr_g)*  &
              dt_dyn_copy(1)
!^print *,'grid 1 dt_dyn increase',ftmp,gd(1)%dt_dyn,dt_dyn_copy(1)
      else                                     ! Decrease timestep (nonlinearly)
         ! Decrease by at least 1% over what is necessary, with an additional
         !  amount of decrease proportional to the number of substeps and redo
         !  iterations:
         ftmp = substep_factor_curr_g+  &
              real(max(0, n_redo_iterations-1)*10, kind=f4)*Cn_tgt_stepdec_slow
         ftmp = Cn_target(1)/Cn_max_curr_g-0.01_f4-0.2_f4*ftmp
         gd(1)%dt_dyn = ftmp*dt_dyn_copy(1)
!^print *,'grid 1 dt_dyn decrease',ftmp,gd(1)%dt_dyn,dt_dyn_copy(1)
      end if
      gd(1)%dt_dyn = min(max(gd(1)%dt_dyn, dt_min), max_dt_dynamics) ! Clip
   end if

   iteration = 0
   g1_dt_adj: do
      iteration = iteration+1

      ! Do not allow sound speed fraction of grid #1 to fall below ~1;
      ftmp = 0.95*real(gd(1)%n_dta_per_dtdyn, kind=f4)/(4._f4*gd(1)%ss_over_dxy)
      gd(1)%dt_dyn = min(ftmp, gd(1)%dt_dyn)
!^print *,'grid 1 dt_dyn ssfrac adjust',iteration,ftmp,gd(1)%dt_dyn

      n_grid_substeps(1,1) = 1
      n_grid_substeps(2,1) = n_grid_substeps(1,1)*gd(1)%n_dta_per_dtdyn

      ! Every finer grid must have a timestep <= that of its parent grid:

      do ifm=2,n_grids   ! Grid #1 was dealt with above

         icm = gd(ifm)%nxtnest

         if (iteration == 1) then         
            ! Current maximum Courant number on this grid:
            if (gd(ifm)%Courant_number_xyz < -epsilon(ONE_f4)) then
               Cn_max_curr_g = max(gd(ifm)%Cn_value_record(n_Cn_records-1),  &
                    epsilon(ONE_f4))
            else        
               Cn_max_curr_g = max(gd(ifm)%Courant_number_xyz, epsilon(ONE_f4))
            end if
!^print *,'grid X Cn_max_curr_g',ifm,gd(ifm)%Courant_number_xyz < -epsilon(ONE_f4),Cn_max_curr_g
 
            gd(ifm)%Cn_value_record(n_Cn_records) = Cn_max_curr_g
            gd(ifm)%Cn_change_record(n_Cn_records) =  &
                 gd(ifm)%Cn_value_record(n_Cn_records)-  &
                 gd(ifm)%Cn_value_record(n_Cn_records-1)

            ! Determine current # of substeps for this grid (per grid #1
            !  timestep):
            n_grid_substeps(1,ifm) = n_grid_substeps(1,icm)*  &
                 gd(ifm)%n_dt_per_pgdt
            n_grid_substeps(2,ifm) = n_grid_substeps(1,ifm)*  &
                 gd(ifm)%n_dta_per_dtdyn

            ic = icm; iq = n_grid_substeps(2,ifm)
            do
               iq = iq+n_grid_substeps(2,ic)
               ic = gd(ic)%nxtnest
               if (ic == 0) exit
            end do
 
            substep_factor_curr_g =  &
                 real(n_grid_substeps(1,ifm)-1, kind=f4)*Cn_tgt_stepdec_slow+  &
                 real(iq, kind=f4)*Cn_tgt_stepdec_fast
           
            Cn_target(ifm) = max(0.1_f4*Cn_target_default_curr,  &
                 Cn_target_default_curr-substep_austerity_level*  &
                 substep_factor_curr_g)
!^print *,'grid X Cn_target',ifm,substep_factor_curr_g,Cn_target(ifm)

!^print *,'grid X alter dt_dyn block?',ifm,  &
!^     .NOT.(n_redo_iterations > 0 .and. gd(ifm)%timestep_OK(n_Cn_records)),  &
!^     n_redo_iterations > 0, gd(ifm)%timestep_OK(n_Cn_records)

            ! When possible, try to increase timestep:
            if (.NOT.(n_redo_iterations > 0 .and.  &
                 gd(ifm)%timestep_OK(n_Cn_records))) then 
               if (Cn_max_curr_g < Cn_target(ifm)) then
                     ! Adapt to new parent grid dt:
                  ic = max(1, gd(ifm)%n_dt_per_pgdt-1)
                  ! Conservative estimate of the factor by which this grid's
                  !  timestep must increase to achieve a dt ratio of 'ic':
                  ftmp = gd(gd(ifm)%nxtnest)%dt_dyn/(real(ic, kind=f4)*  &
                       dt_dyn_copy(ifm))

                  max_factor = (ONE_f4-0.01_f4-0.25_f4*substep_factor_curr_g)* &
                       Cn_target(ifm)/Cn_max_curr_g
                  if (ftmp <= max_factor .and.  &      ! "Quickly" increase dt
                       gd(ifm)%timestep_OK(n_Cn_records-1)) then
                     gd(ifm)%dt_dyn = ftmp*dt_dyn_copy(ifm)

                     do ic=gd(ifm)%n_dt_per_pgdt-2,1,-1  ! Loop may be ignored
                        ftmp = gd(gd(ifm)%nxtnest)%dt_dyn/  &
                             (real(ic, kind=f4)*dt_dyn_copy(ifm))   
                        if (ftmp > max_factor) exit
                        gd(ifm)%dt_dyn = ftmp*dt_dyn_copy(ifm)
                     end do
                     ! Set to a value that will not trigger a dt throttling
                     !  for the next timestep:
                     gd(ifm)%Cn_value_record(n_Cn_records) = Cn_target_default
                  else   ! Try to increase the timestep relatively slowly
                     if (Cn_max_curr_g <= 0.75_f4*Cn_target(ifm)) then
                                                       ! Full throttle
                        dt_max_frac_incr_curr_g = dt_max_frac_increase_g1
                     else                              ! Throttle back
                        dt_max_frac_incr_curr_g = dt_max_frac_increase_g1*  &
                             (Cn_target(ifm)-  &
                         min(Cn_max_curr_g, 0.99_f4*Cn_target(ifm))-0.01_f4)/  &
                             Cn_target(ifm)
                     end if
                     ftmp = dt_max_frac_incr_curr_g

                     ! Throttle if Courant number is increasing too quickly
                     !  (could be rapidly amplifying simulated phenomenon):
                     if (gd(ifm)%Cn_change_record(n_Cn_records) >  &
                          dt_max_frac_increase_g1*2._f4 .and.  &
                          gd(ifm)%Cn_value_record(n_Cn_records) >  &
                          0.5_f4*Cn_target(ifm)) then
                        ftmp = substep_factor_curr_g
                        ftmp = (-0.01_f4-0.2_f4*ftmp)*0.5_f4*  &
                             gd(ifm)%Cn_change_record(n_Cn_records)
                     end if
                     gd(ifm)%dt_dyn = (ONE_f4+ftmp)*dt_dyn_copy(ifm)
                  end if
               else   ! Decrease timestep
!^print *,'fefefef',n_redo_iterations,ifm,Cn_target(ifm),substep_factor_curr_g,Cn_max_curr_g,dt_dyn_copy(ifm)
                  ! Decrease by at least 1% over what is necessary, with an
                  !  additional amount of decrease proportional to the number of
                  !  substeps and redo iterations:
                  ftmp = real(max(0, n_redo_iterations-1)*10, kind=f4)*  &
                       Cn_tgt_stepdec_slow+substep_factor_curr_g
                  ftmp = Cn_target(ifm)/Cn_max_curr_g-0.01_f4-0.2*ftmp 
                  gd(ifm)%dt_dyn = ftmp*dt_dyn_copy(ifm)
               end if
            end if
 
            dtlongn_basis(ifm) = gd(ifm)%dt_dyn
         end if
         gd(ifm)%dt_dyn = min(gd(icm)%dt_dyn, dtlongn_basis(ifm))

         ! Do not allow sound speed fraction of this grid to fall below ~1;
         ftmp = 0.95*real(gd(ifm)%n_dta_per_dtdyn, kind=f4)/  &
              (4._f4*gd(ifm)%ss_over_dxy)
         gd(ifm)%dt_dyn = min(ftmp, gd(ifm)%dt_dyn)

         ! Find a parent:child timestep ratio that fits the constraints:
         ftmp = gd(icm)%dt_dyn/gd(ifm)%dt_dyn
         iq = max(1, int(ftmp)-1)   ! Smallest nndtrat value to consider
         do
            ftmp = gd(icm)%dt_dyn/real(iq, kind=f4)
            if (ftmp <= gd(ifm)%dt_dyn) exit
            iq = iq+1   ! Try a more severe nndtrat value
         end do
         gd(ifm)%dt_dyn = ftmp; gd(ifm)%n_dt_per_pgdt = iq
      end do

      ! Check for issues:
      redo_g1_dt_adj = .FALSE.
      n_grid_substeps(1,1) = 1
      n_grid_substeps(2,1) = n_grid_substeps(1,1)*gd(1)%n_dta_per_dtdyn
      max_n_substeps_for_any_grid = n_grid_substeps(2,1)
      do ifm=2,n_grids    ! Grid #1 was dealt with above
         icm = gd(ifm)%nxtnest
         n_grid_substeps(1,ifm) = n_grid_substeps(1,icm)*  &
              gd(ifm)%n_dt_per_pgdt
         n_grid_substeps(2,ifm) = n_grid_substeps(1,ifm)*gd(ifm)%n_dta_per_dtdyn

         ic = icm; iq = n_grid_substeps(2,ifm)
         do
            iq = iq+n_grid_substeps(2,ic)
            ic = gd(ic)%nxtnest
            if (ic == 0) exit
         end do
         max_n_substeps_for_any_grid = max(max_n_substeps_for_any_grid, iq)

         ! If there are too many substeps, gradually reduce the grid #1
         !  timestep and retry the parent/child timestep determination process
         !  above:
         if (iq > n_substeps_max_curr) then
            gd(1)%dt_dyn = gd(1)%dt_dyn*real(n_substeps_max_curr, kind=f4)/  &
                 real(iq, kind=f4) 
            gd(1)%dt_dyn = min(max(gd(1)%dt_dyn, dt_min), max_dt_dynamics)
            redo_g1_dt_adj = .TRUE.; exit
         end if
      end do
      if (.NOT. redo_g1_dt_adj) then
         exit g1_dt_adj
      end if

      if (iteration == 100) call exception(procedure_name, FATAL,  &
           'Too many g1_dt_adj iterations.')

   end do g1_dt_adj

   do ifm=1,n_grids
      ! If there are any updates to n_dt_per_pgdt, set nndtflg = 1 to indicate
      !  that `modsched` must be run again to update the model schedule:
      if (gd(ifm)%n_dt_per_pgdt /= n_dt_per_pgdt_copy(ifm)) nndtflg = 1
   end do

end if

! Compute current sound speed criteria for all grids ('sspct' used by acoustic
!  routines):
sspct = ONE_f4  ! Nominal/maximum value
do ifm=1,n_grids
   ss_Courant_number(ifm) = gd(ifm)%ss_over_dxy*4._f4*gd(ifm)%dt_dyn/  &
        real(gd(ifm)%n_dta_per_dtdyn, kind=f4)
   nsspct(ifm) = 0.95_f4/ss_Courant_number(ifm)
   sspct = min(sspct, nsspct(ifm))
end do

if (dt_mode < 0 .or. mode < 2 .or.  &
     maxval(nsspct) < min_ssfrac-epsilon(ZERO_f4)) then
   ! Display initial timestep-related values:
   write (*, '(/,a,/,a)') '      ------- Timestep info -------',  &
    '   grid#  n_dt_per_pgdt  n_dta_per_dtdyn   dt_dyn  ss_Cnum  ss_frac  tstep'
   do ifm=1,n_grids
      write (*, '(a4,i3,i8,i9,f13.5,2f11.5,i9)') '##+ ', ifm,  &
           gd(ifm)%n_dt_per_pgdt, gd(ifm)%n_dta_per_dtdyn,  &
           gd(ifm)%dt_dyn, ss_Courant_number(ifm), nsspct(ifm), tstep
   end do
   if (dt_mode < 0)  write (*, '(a,i9)')  &
        'Maximum number of substeps for any grid: ', max_n_substeps_for_any_grid
end if
   
end subroutine dtset
