!```````````````````````````````````````````````````````````````````
subroutine speval(xa,ya,y2a,n,x,y,m)

!~ 19.11.2008 (last modified; TIM)

!+ Cubic spline interpolation routine for 8-byte floating-point data

use univ_kind_defs_mod, only: f8

implicit none

!=== Argument declarations:

integer, intent(IN) :: n,m
real(f8), dimension(n), intent(IN) :: xa,ya,y2a
real(f8), dimension(m), intent(IN) :: x

real(f8), dimension(m), intent(OUT) :: y

!== Local declarations:

!     PURPOSE:
!        To evaluate the Cubic Spline functions calcualted by SPLINE.
!     Given the arrays XA and YA of length N, which tabulate a function
!     (with the XA's in order - These are the X, Y, Y2 arrays used in
!     SPLINE to get the natural spline).  Given the array Y2A, which
!     is the output from SPLINE, and given an array X, this routine
!     returns a cubic-spline interpolated value in array Y.  The X and Y
!     arrays are formally dimensioned to 1, while M is the number of
!     elements in the X and Y arrays.  The basic algorithm was also
!     stolen from "Numerical Recipies".
!
!     AUTHOR
!        Taken from NUMERICAL RECIPIES by Press et. al.
!
!     UPDATES FOR
!        Jim Pollack
!
!     ENVIRONMENT
!        NAS Cray-YMP      UNICOS       FORTRAN
!
!     REVISION HISTORY
!        See notes from J. Pollack dated 2/20/90 and 3/12/90.
!
!     INPUT PARAMETERS
!     m       - Number of elements in arrays x and y.
!     n       - Number of values in the data arrays xa, ya, and y2a.
!     x(i)    - Abscissa values at which the Cubic Spline is to be
!               Evaluated.  An array of length m.
!     xa(i)   - Abscissa values of the data with which the Cublic Spline
!               coefficients were calculated in SPLINE.  An array of 
!               length n.  It is assumed that 
!                         xa(1) < xa(2) < . . . < xa(n).
!     ya(i)   - Functional values, ya(i) = f(xa(i)), with which the
!               Cubic Spline coefficients were calculated in SPLINE.  An
!               array of length n.
!     y2a(i)  - Array of length n which contains the second derivatives
!               of the interpolating function at the tabulated points
!               xa(i).  These values were computed in SPLINE, using n,
!               xa, and ya as input.
!
!     OUTPUT PARAMETERS
!     y(i)    - An array of length m which holds the computed value of
!               the Spline function at x(i).
!
!     CALLED BY
!        SETRAD
!
!     SUBROUTINES CALLED
!        None

integer :: i,k,khi,klo
real(f8) :: h,a,b

!=== Executable statements:

! Loop over the M items of the X-array

do i=1,m
     
   klo = 1
   khi = n

! Use bi-section to find the correct place in the table

10 if(khi-klo.gt.1) then
      k = (khi+klo)/2
      if(xa(k).gt.x(i)) then
         khi = k
      else
         klo = k
      endif
      goto 10
   endif

! Cubic spline is now evulated and answer placed in Y(i).

   h = xa(khi)-xa(klo)
   a = (xa(khi)-x(i))/h
   b = (x(i)-xa(klo))/h
   y(i) = a*ya(klo)+b*ya(khi)+  &
        ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6._f8

enddo

end subroutine speval


!```````````````````````````````````````````````````````````````````
subroutine speval4(xa,ya,y2a,n,x,y,m)

!~ 19.11.2008 (last modified; TIM)

!+ Cubic spline interpolation routine for 4-byte floating-point data

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n,m
real(f4), dimension(n), intent(IN) :: xa,ya,y2a
real(f4), dimension(m), intent(IN) :: x

real(f4), dimension(m), intent(OUT) :: y

!== Local declarations:

!     PURPOSE:
!        To evaluate the Cubic Spline functions calcualted by SPLINE.
!     Given the arrays XA and YA of length N, which tabulate a function
!     (with the XA's in order - These are the X, Y, Y2 arrays used in
!     SPLINE to get the natural spline).  Given the array Y2A, which
!     is the output from SPLINE, and given an array X, this routine
!     returns a cubic-spline interpolated value in array Y.  The X and Y
!     arrays are formally dimensioned to 1, while M is the number of
!     elements in the X and Y arrays.  The basic algorithm was also
!     stolen from "Numerical Recipies".
!
!     AUTHOR
!        Taken from NUMERICAL RECIPIES by Press et. al.
!
!     UPDATES FOR
!        Jim Pollack
!
!     ENVIRONMENT
!        NAS Cray-YMP      UNICOS       FORTRAN
!
!     REVISION HISTORY
!        See notes from J. Pollack dated 2/20/90 and 3/12/90.
!
!     INPUT PARAMETERS
!     m       - Number of elements in arrays x and y.
!     n       - Number of values in the data arrays xa, ya, and y2a.
!     x(i)    - Abscissa values at which the Cubic Spline is to be
!               Evaluated.  An array of length m.
!     xa(i)   - Abscissa values of the data with which the Cublic Spline
!               coefficients were calculated in SPLINE.  An array of 
!               length n.  It is assumed that 
!                         xa(1) < xa(2) < . . . < xa(n).
!     ya(i)   - Functional values, ya(i) = f(xa(i)), with which the
!               Cubic Spline coefficients were calculated in SPLINE.  An
!               array of length n.
!     y2a(i)  - Array of length n which contains the second derivatives
!               of the interpolating function at the tabulated points
!               xa(i).  These values were computed in SPLINE, using n,
!               xa, and ya as input.
!
!     OUTPUT PARAMETERS
!     y(i)    - An array of length m which holds the computed value of
!               the Spline function at x(i).
!
!     CALLED BY
!        SETRAD
!
!     SUBROUTINES CALLED
!        None

integer :: i,k,khi,klo
real(f4) :: h,a,b

!=== Executable statements:

! Loop over the M items of the X-array

do i=1,m
     
   klo = 1
   khi = n

! Use bi-section to find the correct place in the table

   do while(khi-klo > 1)
      k = (khi+klo)/2
      if(xa(k) > x(i)) then
         khi = k
      else
         klo = k
      endif
   enddo

! Cubic spline is now evualated and answer placed in Y(i).

   h = xa(khi)-xa(klo)
   a = (xa(khi)-x(i))/h
   b = (x(i)-xa(klo))/h
   y(i) = a*ya(klo)+b*ya(khi)+  &
        ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h*h)/6._f4

enddo

end subroutine speval4
