!```````````````````````````````````````````````````````````````````
module spline_mod

!~ 18.11.2008 (last modified; TIM)

!+ Contains routines directly related to spline interpolation and extrapolation
!   calculations

implicit none

private

! Determine (in a portable fashion) some useful datatype KINDs:
!%integer, parameter :: i4 = selected_int_kind(9),  &    ! can hold +/- 10^9
!%     f4 = selected_real_kind(6,30),  & ! >= 6 dec_dig prec, dec_exp of +/- 30
!%     f8 = selected_real_kind(12,200)   ! >= 12 dec_dig prec, dec_exp of +/- 200

interface calc_spline
   module procedure calc_spline_f4, calc_spline_f8
end interface

interface spline_interpolate
   module procedure spline_intp_f4, spline_intp_f8
end interface

public :: calc_spline, spline_interpolate

contains


!```````````````````````````````````````````````````````````````````
subroutine calc_spline_f4(x,f_x,n,df2_dx2)

!~ 18.11.2008 (last modified; TIM)

!+ For a set of function values f_x[i] = f(x[i]), with the values of x[i] in
!   ascending order (x[1] < x[2] < ... x[n]), returns the second derivatives
!   of the interpolating function ("natural" splines) at x[i].
!
!  Requires/returns 4-byte floating point data; Merely a front-end to
!   `natural_spline_d2`.

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

integer, intent(IN) :: n
real(f4), dimension(n), intent(IN) :: x, f_x

real(f4), dimension(n), intent(OUT) :: df2_dx2

!== Local declarations:

real(f8), dimension(n) :: inp_1, inp_2

!=== Executable statements:

inp_1 = real(x, kind=f8)
inp_2 = real(f_x, kind=f8)

df2_dx2 = real(natural_spline_d2(inp_1,inp_2), kind=f4)

end subroutine calc_spline_f4


!```````````````````````````````````````````````````````````````````
subroutine calc_spline_f8(x,f_x,n,df2_dx2)

!~ 18.11.2008 (last modified; TIM)

!+ For a set of function values f_x[i] = f(x[i]), with the values of x[i] in
!   ascending order (x[1] < x[2] < ... x[n]), returns the second derivatives
!   of the interpolating function ("natural" splines) at x[i].
!
!  Requires/returns 8-byte floating point data; Merely a front-end to
!   `natural_spline_d2`.

use univ_kind_defs_mod, only: f8

implicit none

!=== Argument declarations:

integer, intent(IN) :: n
real(f8), dimension(n), intent(IN) :: x, f_x

real(f8), dimension(n), intent(OUT) :: df2_dx2

!== Local declarations:

! NONE

!=== Executable statements:

df2_dx2 = natural_spline_d2(x,f_x)

end subroutine calc_spline_f8


!```````````````````````````````````````````````````````````````````
function natural_spline_d2(x,f_x)

!~ 18.11.2008 (last modified; TIM)

!+ For a set of function values f_x[i] = f(x[i]), with the values of x[i] in
!   ascending order (x[1] < x[2] < ... x[n]), returns the second derivatives
!   of the interpolating function ("natural" splines) at x[i].
!
!  Based on "Numerical Recipes" technique; Requires/returns 8-byte floating
!   point data.

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ZERO_f8

implicit none

!=== Argument declarations:

real(f8), dimension(:), intent(IN) :: x, f_x

!== Local declarations:

real(f8), dimension(size(x)) :: natural_spline_d2  ! Function type declaration

integer :: i, n
real(f8) :: p, s

real(f8), dimension(size(x)) :: z, u

!=== Executable statements:

n = size(x)

! Use a tridiagonal matrix solution technique to solve for unknowns:

z(1) = ZERO_f8; z(n) = ZERO_f8; u(1) = ZERO_f8  ! "Natural" spline BC case

do i=2,n-1
   s = (x(i)-x(i-1))/(x(i+1)-x(i-1))
   p = s*z(i-1)+2._f8
   z(i) = (s-1._f8)/p
   u(i) = (6._f8*((f_x(i+1)-f_x(i))/(x(i+1)-x(i))-(f_x(i)-f_x(i-1))/  &
        (x(i)-x(i-1)))/(x(i+1)-x(i-1))-s*u(i-1))/p
enddo

do i=n-1,1,-1 
   z(i) = z(i)*z(i+1)+u(i)                    
enddo

natural_spline_d2 = z

end function natural_spline_d2


!```````````````````````````````````````````````````````````````````
subroutine spline_intp_f4(xp,f_xp,df2_dxp2,np,x,f_x,n)

!~ 18.11.2008 (last modified; TIM)

!+ For a set of function values f_xp[i] = f(xp[i]), with the values of xp[i] in
!   ascending order (xp[1] < xp[2] < ... xp[n]), and the corresponding second
!   derivatives of the interpolating function ("natural" splines) at xp[i]
!   (e.g., output from `calc_spline`, above):  Returns 
!   natural-spline-interpolated values valid at x[i].
!
!  Requires/returns 4-byte floating point data; Merely a front-end to
!   `spline_interpolated_f_x`.

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

integer, intent(IN) :: np, n
real(f4), dimension(np), intent(IN) :: xp, f_xp, df2_dxp2
real(f4), dimension(n), intent(IN) :: x

real(f4), dimension(n), intent(OUT) :: f_x

!== Local declarations:

real(f8), dimension(np) :: inp_1, inp_2, inp_3
real(f8), dimension(n) :: inp_4

!=== Executable statements:

inp_1 = real(xp, kind=f8)
inp_2 = real(f_xp, kind=f8)
inp_3 = real(df2_dxp2, kind=f8)
inp_4 = real(x, kind=f8)

f_x = real(spline_interpolated_f_x(inp_1,inp_2,inp_3,inp_4), kind=f4)

end subroutine spline_intp_f4


!```````````````````````````````````````````````````````````````````
subroutine spline_intp_f8(xp,f_xp,df2_dxp2,np,x,f_x,n)

!~ 18.11.2008 (last modified; TIM)

!+ For a set of function values f_xp[i] = f(xp[i]), with the values of xp[i] in
!   ascending order (xp[1] < xp[2] < ... xp[n]), and the corresponding second
!   derivatives of the interpolating function ("natural" splines) at xp[i]
!   (e.g., output from `calc_spline`, above):  Returns 
!   natural-spline-interpolated values valid at x[i].
!
!  Requires/returns 8-byte floating point data; Merely a front-end to
!   `spline_interpolated_f_x`.

use univ_kind_defs_mod, only: f8

implicit none

!=== Argument declarations:

integer, intent(IN) :: np, n
real(f8), dimension(np), intent(IN) :: xp, f_xp, df2_dxp2
real(f8), dimension(n), intent(IN) :: x

real(f8), dimension(n), intent(OUT) :: f_x

!== Local declarations:

! NONE

!=== Executable statements:

f_x = spline_interpolated_f_x(xp,f_xp,df2_dxp2,x)

end subroutine spline_intp_f8


!```````````````````````````````````````````````````````````````````
function spline_interpolated_f_x(xp,f_xp,df2_dxp2,x)

!~ 18.11.2008 (last modified; TIM)

!+ For a set of function values f_xp[i] = f(xp[i]), with the values of xp[i] in
!   ascending order (xp[1] < xp[2] < ... xp[n]), and the corresponding second
!   derivatives of the interpolating function ("natural" splines) at xp[i]
!   (e.g., output from `calc_spline`, above):  Returns 
!   natural-spline-interpolated values valid at x[i].
!
!  Based on "Numerical Recipes" technique; Requires/returns 8-byte floating
!   point data.

use univ_kind_defs_mod, only: f8

implicit none

!=== Argument declarations:

real(f8), dimension(:), intent(IN) :: xp, f_xp, df2_dxp2, x

!== Local declarations:

real(f8), dimension(size(x)) :: spline_interpolated_f_x  ! Function type decl.

integer :: np, n, up_idx, lw_idx, i, iq
real(f8) :: up_fitp, lw_fitp, dxp

!=== Executable statements:

np = size(xp); n = size(x)   ! Dimensions

lw_idx = 1; up_idx = 2    ! First guess for location

do iq=1,n

   ! If necessary, perform a bisection search for the appropriate location in
   !  the spline function tabulation:

   if(.NOT. (xp(lw_idx) <= x(iq) .and. xp(up_idx) > x(iq))) then
      up_idx = np; lw_idx = 1
      do while(up_idx-lw_idx > 1)
         i = (up_idx+lw_idx)/2
         if(xp(i) > x(iq)) then
            up_idx = i
         else
            lw_idx = i
         endif
      enddo
   endif

   ! Location found; calculate interpolated function value:

   dxp = xp(up_idx)-xp(lw_idx)
   up_fitp = (xp(up_idx)-x(iq))/dxp
   lw_fitp = (x(iq)-xp(lw_idx))/dxp

   spline_interpolated_f_x(iq) = up_fitp*f_xp(lw_idx)+lw_fitp*f_xp(up_idx)+  &
        ((up_fitp**3-up_fitp)*df2_dxp2(lw_idx)+  &
        (lw_fitp**3-lw_fitp)*df2_dxp2(up_idx))*(dxp*dxp)/6._f8

enddo

end function spline_interpolated_f_x


end module spline_mod
