!```````````````````````````````````````````````````````````````````
module surface_data_prep_mod

!+

use univ_kind_defs_mod, only: f4

implicit none

interface sfcopqr
   module procedure sfcopqr_i1, sfcopqr_i2, sfcopqr_i4, sfcopqr_f4, sfcopqr_f8
end interface sfcopqr


contains


include "surface_data_prep.f90.inc01"


!```````````````````````````````````````````````````````````````````
subroutine topoq(niq, njq, n2, n3, datq, sdq, shaq, datre, itopsflg,  &
     toptwvlx, toptwvly, toptenh)

!+ Calculates orographic enhancement schemes for topography

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_numtype_constants_mod, only: tiny_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: niq, njq, n2, n3, itopsflg
real(f4), intent(IN) :: toptwvlx, toptwvly, toptenh
real(f4), dimension(niq,njq), intent(IN) :: sdq, shaq

real(f4), dimension(n2,n3), intent(INOUT) :: datre
real(f4), dimension(niq,njq), intent(INOUT) :: datq

!== Local declarations:

real(f4) :: average, radx, rady, remax, remin, total
integer :: iq, jq, count, imax, imin, ire, jmax, jmin, jre

!=== Executable statements:

if (itopsflg < 0) then              ! No topography desired
   do jq=1,njq
      do iq=1,niq
         datq(iq,jq) = ZERO_f4
      end do
   end do
   write (*, '(/,a)') 'INFO: Topography set to zero.'
   return
end if

select case(itopsflg)

case(0)                    ! Average

   write (*, '(/,a)') 'INFO: No topography enhancement applied.'

case(1)                   ! Silhouette

   do jq=1,njq
      do iq=1,niq
         datq(iq,jq) = SHAQ(IQ,JQ)*toptenh+DATQ(IQ,JQ)*(ONE_f4-toptenh)
      end do
   end do
   write (*, '(/,a)') 'INFO: Silhouette topography enhancement applied.'

case(2)                   ! Envelope

   do jq=1,njq
      do iq=1,niq
         datq(iq,jq) = datq(iq,jq)+toptenh*sdq(iq,jq)
      end do
   end do
   write (*, '(/,a)') 'INFO: Envelope topography enhancement applied.'
   write (*, '(a,f5.2,a)') '        enhancement = ', toptenh, ' x STD DEV'

case(3)                  ! Reflected Envelope

   ! the radius we want to search for the current pts relative
   ! height should correspond well to half the filtering wavelength
   ! used on the topo (toptwvlx/y)
 
   radx = toptwvlx*0.5_f4; rady = toptwvly*0.5_f4
   do jq=1,njq
      do iq=1,niq
         datre(iq,jq) = datq(iq,jq)
      end do
   end do

   do jq=1,njq
      do iq=1,niq
         remax = datre(iq,jq); remin = datre(iq,jq)
         jmin = jq-nint(rady); imin = iq-nint(radx)
         jmax = jq+nint(rady); imax = iq+nint(radx)
         count = 0; total = ZERO_f4
         do jre=max(1, jmin),min(njq, jmax)
            do ire=max(1, imin),min(niq, imax)
               if ((real(iq-ire, kind=f4)**2+real(jq-jre, kind=f4)**2) <=  &
                    radx*rady) then
                  count = count+1
                  total = total+datre(ire,jre)
                  remax = max(remax, datre(ire,jre))
                  remin = min(remin, datre(ire,jre))
               end if
            end do
         end do
         average = total/real(count, kind=f4)
         total = remax-remin  ! Temporary quantity
         if (abs(total) > tiny_f4) datq(iq,jq) = datre(iq,jq)+  &
              (datre(iq,jq)-average)/(total*0.5_f4)*toptenh*sdq(iq,jq)
      end do
   end do

   write (*, '(/,a)') 'INFO: Reflected envelope topography enhancement applied.'
   write (*, '(a,f5.2,a)') '        enhancement = ', toptenh, ' x STD DEV,'
   write (*, '(a,2(f7.2))') '        search radii (x,y; gridpts) = ', radx,  &
        rady 

end select

end subroutine topoq


!```````````````````````````````````````````````````````````````````
subroutine z0q(niq, njq, datq, sdq, itopsflg, iz0flg, z0fact, z0max, zrough)

!+ Calculates orographically enhanced z0 (roughness length)

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: niq, njq, itopsflg, iz0flg
real(f4), dimension(niq,njq), intent(IN) :: sdq
real(f4), intent(IN) :: z0fact, z0max, zrough

real(f4), dimension(niq,njq), intent(INOUT) :: datq

!== Local declarations:

integer :: iq, jq

!=== Executable statements:

if (itopsflg < 0) then    ! No topography

   do jq=1,njq
      do iq=1,niq
         datq(iq,jq) = zrough
      end do
   end do
   write (*, '(/,a,f7.3,a)') 'INFO: Surface roughness set to ', zrough, ' m'

else if (iz0flg == 1) then

   do jq=1,njq
      do iq=1,niq  
         datq(iq,jq) = min(z0fact*sdq(iq,jq), z0max)
      end do
   end do
   write (*, '(/,a)') 'INFO: Subgrid terrain roughness applied with'
   write (*, '(a,f7.3,a,f7.3,a,f7.3)') '      factor = ', z0fact,  &
        ' x STD DEV = ', minval(sdq), ' to ', maxval(sdq)
   write (*, '(a,f7.3)') '      maximum = ', z0max

else

   do jq=1,njq
      do iq=1,niq
         datq(iq,jq) = zrough
      end do
   end do
   write (*, '(/,a,f7.3,a)') 'INFO: Surface roughness set to ', zrough, ' m'

end if

end subroutine z0q


!```````````````````````````````````````````````````````````````````
subroutine xy_to_f_loc_via_ll(x, y, plat, plon, offset_lat, offset_lon,  &
     d_lat, d_lon, start_lat, start_lon, xi, yj)

!+ X/Y OPS coords to lat/lon, then to file coords 

use univ_kind_defs_mod, only: f4
use univ_OPS_proj_mod, only: xy_ll_OPS
use univ_math_constants_mod, only: ONE_f4
use univ_numtype_constants_mod, only: epsilon_f4
use body_specific_constants_mod, only: radius

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: plat, plon, x, y, offset_lat, offset_lon, d_lat,  &
     d_lon, start_lat, start_lon

real(f4), intent(OUT) :: xi, yj

!== Local declarations:

real(f4) :: lat, lon

!=== Executable statements:

call xy_ll_OPS(lat, lon, plat, plon, x, y, radius)    ! OPS coords to lat/lon

! Adjust lat/lon using given offsets:

lat = max(-90._f4+epsilon_f4,  &
     min(90._f4-epsilon_f4, lat+offset_lat))  ! Latitude of point

lon = lon+offset_lon          !
if (lon > 180._f4) then       !  
   lon = lon-360._f4          !  Longitude of point
else if (lon < -180._f4) then !
   lon = lon+360._f4          !
end if                        !

! Lat/lon coords to file coords {file coord (1,1) is (start_lon,start_lat)}: 
xi = max(ONE_f4, ONE_f4+(lon-start_lon)/d_lon)
yj = max(ONE_f4, ONE_f4+(lat-start_lat)/d_lat)

end subroutine xy_to_f_loc_via_ll


!```````````````````````````````````````````````````````````````````
subroutine surface_data_prep(n2, n3, data_r, input_fn_primary, igd, vnam)

!+ Orographic z0 / surface data input / interpolation driver

use univ_kind_defs_mod, only: f4, i2
use univ_exception_mod, only: exception, FATAL
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_numtype_constants_mod, only: huge_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN,  &
     MAX_MESSAGE_LEN
use body_specific_constants_mod, only: dist_deglat
use main_memory_mod, only: gd
use manage_NetCDF_classic_mod, only: mNc_populate_f_md_obj_r,  &
     mNc_file_metadata_obj, mNc_extract_g_Att_val, mNc_extract_var_Att_val,  &
     mNc_var_dims, mNc_var_idx, mNc_SHORT_type, mNc_FLOAT_type,  &
     mNc_recycle_f_md_obj, mNc_f_md_var_type

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: input_fn_primary
character(len=3), intent(IN) :: vnam
integer, intent(IN) :: n2, n3, igd

real(f4), dimension(n2,n3), intent(INOUT) :: data_r

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'surface_data_prep'

character(len=MAX_MESSAGE_LEN) :: msg
character(len=MAX_FILE_PATH_LEN), dimension(2) :: input_fn
integer :: niq, njq, np_x, np_y, e, bin_scheme, i, n_inpfn, i_inpfn
logical :: last_inpfn, has_global_extent
real(f4) :: delta_lat, delta_lon, deltaxq, deltayq, deltaxp, deltayp,  &
     enh_val, filt_wvlx, filt_wvly, offset_lon, ffill
real(f4), allocatable, dimension(:,:) :: datp, datq, tmpq_1, tmpq_2
type(mNc_file_metadata_obj), save :: input_file_obj

real(f4), dimension(1) :: f4_tmp_1d_1
integer(i2), dimension(1) :: i2_tmp_1d_1

!=== Executable statements:

! Configure grid specs for raw data to PRAMS grid (R) transfer:
select case (vnam)
case('TOP')
   write (*, '(/,a,i4,/)') 'Starting topography on grid:', igd
   bin_scheme = gd(igd)%itopsflg
   enh_val = gd(igd)%toptenh
   filt_wvlx = gd(igd)%toptwvlx; filt_wvly = gd(igd)%toptwvly
   ffill = -99999._f4
case('Z0T')
   write (*, '(/,a,i4,/)') 'Starting surface roughness on grid:', igd
   bin_scheme = gd(igd)%itopsflg
   enh_val = gd(igd)%toptenh
   filt_wvlx = gd(igd)%toptwvlx; filt_wvly = gd(igd)%toptwvly
   ffill = -99999._f4
case('TIN')
   write (*, '(/,a,i4,/)') 'Starting apparent thermal inertia on grid:', igd
   bin_scheme = 0   ! Bin via pure averaging
   enh_val = ZERO_f4   ! Not applicable
   filt_wvlx = gd(igd)%tinwvlx; filt_wvly = gd(igd)%tinwvly
   ffill = -ONE_f4
case('ALB')
   write (*, '(/,a,i4,/)') 'Starting surface albedo on grid:', igd
   bin_scheme = 0   ! Bin via pure averaging
   enh_val = ZERO_f4   ! Not applicable
   filt_wvlx = gd(igd)%albwvlx; filt_wvly = gd(igd)%albwvly
   ffill = -ONE_f4
end select

if ((vnam == 'TOP' .or. vnam  == 'Z0T') .and.  &
     (bin_scheme == 1 .and. enh_val > ONE_f4)) then
   msg = ';'
   write (msg, '(a,f9.4,a,i2,a)') 'Silhouette weight (', enh_val,  &
        ') is too high for grid ', igd, '; valid range is from 0. to 1.'
   call exception(procedure_name, FATAL, msg)
end if

! Dimensions of temp grid (Q) - wavelength filtering not applied to z0:
if (vnam /= 'Z0T') then
   deltaxq = 0.5_f4*filt_wvlx*gd(igd)%deltax
   deltayq = 0.5_f4*filt_wvly*gd(igd)%deltay
else
   deltaxq = gd(igd)%deltax
   deltayq = gd(igd)%deltay
end if
niq = int(real(gd(igd)%nxp-1, kind=f4)*gd(igd)%deltax/deltaxq)+4
njq = int(real(gd(igd)%nyp-1, kind=f4)*gd(igd)%deltay/deltayq)+4

! Allocate temp grid (Q) and related arrays:
allocate(datq(niq,njq), tmpq_1(niq,njq), tmpq_2(niq,njq), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1')

! Initialize temp grid (Q) with sentinel values:
datq = huge_f4

input_fn(1) = trim(input_fn_primary)
input_fn(2) = 'NONE'
n_inpfn = 2  ! Default (assume use of both primary and auxiliary input files)
i = len_trim(input_fn(2))
if (i >= 4) then
   if (input_fn(2)(i-3:i) == 'NONE') n_inpfn = n_inpfn-1  ! No auxiliary file
end if

do i_inpfn=1,n_inpfn
   last_inpfn = ( i_inpfn == n_inpfn )   ! Should the field be finalized?

   has_global_extent = ( index(trim(input_fn(i_inpfn)), 'global') > 0 )

   ! Configure raw data grid (O) parameters:

   call mNc_populate_f_md_obj_r(trim(input_fn(i_inpfn)), input_file_obj)

   select case (vnam)
   case('TOP')
      call mNc_extract_g_Att_val(input_file_obj, 'lat_increment', f4_tmp_1d_1)
      delta_lat = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_increment', f4_tmp_1d_1)
      delta_lon = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_native_to_IAU2000',  &
           f4_tmp_1d_1)
      offset_lon = f4_tmp_1d_1(1)
   case('Z0T')
      call mNc_extract_g_Att_val(input_file_obj, 'lat_increment', f4_tmp_1d_1)
      delta_lat = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_increment', f4_tmp_1d_1)
      delta_lon = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_native_to_IAU2000',  &
           f4_tmp_1d_1)
      offset_lon = f4_tmp_1d_1(1)
   case('TIN')
      call mNc_extract_g_Att_val(input_file_obj, 'lat_increment', f4_tmp_1d_1)
      delta_lat = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_increment', f4_tmp_1d_1)
      delta_lon = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_native_to_IAU2000',  &
           f4_tmp_1d_1)
      offset_lon = f4_tmp_1d_1(1)
   case('ALB')
      call mNc_extract_g_Att_val(input_file_obj, 'lat_increment', f4_tmp_1d_1)
      delta_lat = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_increment', f4_tmp_1d_1)
      delta_lon = f4_tmp_1d_1(1)
      call mNc_extract_g_Att_val(input_file_obj, 'lon_native_to_IAU2000',  &
           f4_tmp_1d_1)
      offset_lon = f4_tmp_1d_1(1)
   end select

   ! Dimensions of interpolated raw data grid (P):
   np_x = max(1, int(deltaxq/(delta_lon*dist_deglat)))
   np_y = max(1, int(deltayq/(delta_lat*dist_deglat)))
   deltaxp = deltaxq/real(np_x, kind=f4)
   deltayp = deltayq/real(np_y, kind=f4)

   ! Allocate interpolated raw data grid (P):
   allocate(datp(np_x,np_y), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2')

   select case (vnam)
   case('TOP')
      call mNc_extract_var_Att_val(input_file_obj, 'topo', 'missing_value',  &
           i2_tmp_1d_1)
      call sfcopqr(niq, njq, n2, n3, gd(igd)%xt, gd(igd)%yt, deltaxp,  &
           deltayp, deltaxq, deltayq, datp, datq, data_r, vnam, igd,  &
           bin_scheme, enh_val, filt_wvlx, filt_wvly, gd(igd)%iz0flg,  &
           np_x, np_y, input_file_obj, 'topo', i2_tmp_1d_1(1), delta_lon,  &
           delta_lat, offset_lon, gd(igd)%plat, gd(igd)%plon, tmpq_1, tmpq_2,  &
           last_inpfn, has_global_extent, ffill)
   case('Z0T')
      call mNc_extract_var_Att_val(input_file_obj, 'topo', 'missing_value',  &
           i2_tmp_1d_1)
      call sfcopqr(niq, njq, n2, n3, gd(igd)%xt, gd(igd)%yt, deltaxp,  &
           deltayp, deltaxq, deltayq, datp, datq, data_r, vnam, igd,  &
           bin_scheme, enh_val, filt_wvlx, filt_wvly, gd(igd)%iz0flg,  &
           np_x, np_y, input_file_obj, 'topo', i2_tmp_1d_1(1), delta_lon,  &
           delta_lat, offset_lon, gd(igd)%plat, gd(igd)%plon, tmpq_1, tmpq_2,  &
           last_inpfn, has_global_extent, ffill)
   case('TIN')
      call mNc_extract_var_Att_val(input_file_obj, 'TI', 'missing_value',  &
           i2_tmp_1d_1)
      call sfcopqr(niq, njq, n2, n3, gd(igd)%xt, gd(igd)%yt, deltaxp,  &
           deltayp, deltaxq, deltayq, datp, datq, data_r, vnam, igd,  &
           bin_scheme, enh_val, filt_wvlx, filt_wvly, gd(igd)%iz0flg,  &
           np_x, np_y, input_file_obj, 'TI', i2_tmp_1d_1(1), delta_lon,  &
           delta_lat, offset_lon, gd(igd)%plat, gd(igd)%plon, tmpq_1, tmpq_2,  &
           last_inpfn, has_global_extent, ffill)
   case('ALB')
      i = mNc_var_idx(input_file_obj, 'albedo')
      select case (mNc_f_md_var_type(input_file_obj, i))
      case (mNc_SHORT_type)
         call mNc_extract_var_Att_val(input_file_obj, 'albedo',  &
              'missing_value', i2_tmp_1d_1)
         call sfcopqr(niq, njq, n2, n3, gd(igd)%xt, gd(igd)%yt, deltaxp,  &
              deltayp, deltaxq, deltayq, datp, datq, data_r, vnam, igd,  &
              bin_scheme, enh_val, filt_wvlx, filt_wvly, gd(igd)%iz0flg,  &
              np_x, np_y, input_file_obj, 'albedo', i2_tmp_1d_1(1),  &
              delta_lon, delta_lat, offset_lon, gd(igd)%plat, gd(igd)%plon,  &
              tmpq_1, tmpq_2, last_inpfn, has_global_extent, ffill)
      case (mNc_FLOAT_type)
         call mNc_extract_var_Att_val(input_file_obj, 'albedo',  &
              'missing_value', f4_tmp_1d_1)
         call sfcopqr(niq, njq, n2, n3, gd(igd)%xt, gd(igd)%yt, deltaxp,  &
              deltayp, deltaxq, deltayq, datp, datq, data_r, vnam, igd,  &
              bin_scheme, enh_val, filt_wvlx, filt_wvly, gd(igd)%iz0flg,  &
              np_x, np_y, input_file_obj, 'albedo', f4_tmp_1d_1(1),  &
              delta_lon, delta_lat, offset_lon, gd(igd)%plat, gd(igd)%plon,  &
              tmpq_1, tmpq_2, last_inpfn, has_global_extent, ffill)
      end select
   end select
   call mNc_recycle_f_md_obj(input_file_obj)   ! Finished

   deallocate(datp, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '2')

end do  !END: i_inpfn

deallocate(datq, tmpq_1, tmpq_2, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

end subroutine surface_data_prep


end module surface_data_prep_mod
