!```````````````````````````````````````````````````````````````````
module surface_file_mod

!+

implicit none

integer, parameter :: n_config_parameters_f4 = 17, n_config_parameters_i4 = 6


contains


!```````````````````````````````````````````````````````````````````
subroutine sfc_files(this_is_a_restart)

!+ Driver to read surface data from/to files

use state_IO_mod, only: n_grids_st
use main_memory_mod, only: run_type, n_grids, input_sfcfile_pfx, nzg, nzs,  &
     npatch

implicit none

!=== Argument declarations:

logical, intent(IN) :: this_is_a_restart

!== Local declarations:

integer :: igd, beg_grid, end_grid
logical :: will_make_sfcfiles, there_is_work_to_do, file_is_not_OK

!=== Executable statements:

there_is_work_to_do = .TRUE.                ! In most cases
if (this_is_a_restart) then
   if (n_grids <= n_grids_st) then  ! Not adding any new grids
      there_is_work_to_do = .FALSE.         ! Nothing to do here in this case
   else                        ! Adding new grid(s)
      beg_grid = n_grids_st+1; end_grid = n_grids
   end if
else
   beg_grid = 1; end_grid = n_grids
end if

if (there_is_work_to_do) then

   if (run_type == 'MAKE_SFC_FILES') then
    ! Always remake surface files in this case:
      will_make_sfcfiles = .TRUE.
   else
    ! Satisfactory input data file(s) may already exist...
      will_make_sfcfiles = .FALSE.; file_is_not_OK = .FALSE. ! Assume file(s) OK
      if (input_sfcfile_pfx == 'NONE') then
       ! No input file(s); prepare from scratch, but do not write to disk:
         call surface_nest(beg_grid, end_grid, igd, nzg+nzs, npatch, 'FILEDATA')
         call topo_RT_effects_init(beg_grid, end_grid) ! Topo shadow/slope info
      else
         do igd=beg_grid,end_grid   ! Loop over all relevant grids
            call sfcfile_read(igd, .TRUE., file_is_not_OK) ! Attempt file read
            if (file_is_not_OK) then ! Exit loop if input file mangled/missing
               will_make_sfcfiles = .TRUE.
               exit
            end if
         end do
      end if
   end if

   ! If any surface files do not exist or are incorrect, remake them all (except
   !   in the case of a state restart):
   if (will_make_sfcfiles) then
      call surface_nest(beg_grid, end_grid, nzg+nzs, npatch, 'FILEDATA')
      call topo_RT_effects_init(beg_grid, end_grid) ! Topo shadowing/slope info

      write (*, '(/,a,/)') 'Writing surface files ...'
      call sfcfile_out(beg_grid, end_grid)
   end if

end if

end subroutine sfc_files


end module surface_file_mod


!```````````````````````````````````````````````````````````````````
subroutine sfcfile_out(beg_grid, end_grid)

!+ Outputs surface data files for later ingestion by the model

use univ_kind_defs_mod, only: f4
use univ_numtype_constants_mod, only: huge_f4
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_PROC_NAME_LEN
use univ_std_fileIO_mod, only: mkdir_p
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use manage_NetCDF_classic_mod, only: mNc_MAX_NAME_LEN, mNc_file_md_Att_obj,  &
     mNc_file_metadata_obj, mNc_define_f_md_Att_obj, mNc_populate_f_md_obj_w,  &
     mNc_FLOAT_type, mNc_recycle_f_md_Att_obj, mNc_recycle_f_md_obj,  &
     mNc_sync_with_file, mNc_f_md_Att_populated
use manage_NetCDF_classic_wv_mod, only: mNc_write_var
use main_memory_mod, only: a, gd, nzg, nzs, z0fact, input_sfcfile_pfx,  &
     npatch, PRAMS_version, check_undef_val_f4
use vtables_mod, only: iglatn, iglonn, itoptn, itopz0n, ipatchfcvn, itistatn,  &
     isbpsdensn, isbpstmcpn, isbpstmcdn, ialbdstatn, itslopangn, itazmangn
use topo_RT_effects_dims_mod, only: nazm
use surface_file_mod, only: n_config_parameters_f4, n_config_parameters_i4

implicit none

!=== Argument declarations:

integer, intent(IN) :: beg_grid, end_grid

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'sfcfile_out'

character(len=MAX_FILE_PATH_LEN) :: flnmd
character(len=3) :: cgrid
integer :: lc, igd, a_err, dim_1, dim_2, dim_3, dim_4, dim_5, i, i_d1, i_d2,  &
     ic, ia, ab, ae, nxyp
logical :: initialize

real(f4), dimension(n_config_parameters_f4) :: tmp_1d_f4
real(f4), allocatable, dimension(:,:) :: a_tmp_2d
real(f4), allocatable, dimension(:,:,:) :: a_tmp_3d
real(f4), allocatable, dimension(:,:,:,:) :: a_tmp_4d

type(mNc_file_metadata_obj) :: output_file_obj

! Items for storage/processing of NetCDF metadata:
integer :: n_dims, n_vars, n_g_Atts, max_var_ndims, max_var_nAtts
character(len=mNc_MAX_NAME_LEN), allocatable, dimension(:) :: dim_name,  &
     var_name
real(f4), allocatable, dimension(:) :: var_f4_missing_value
integer, allocatable, dimension(:) :: dimv, var_type, var_ndims, var_nAtts
integer, allocatable, dimension(:,:) :: var_dimids
type(mNc_file_md_Att_obj), allocatable, dimension(:) :: g_Att
type(mNc_file_md_Att_obj), allocatable, dimension(:,:) :: var_Att
character(len=mNc_MAX_NAME_LEN) :: tmp_mNc_str

!=== Executable statements:

! Write surface characteristics to files (one file per grid):

lc = index(input_sfcfile_pfx, '/', back=.TRUE.)-1 ! Determine directory name len
call mkdir_p(input_sfcfile_pfx(1:lc))  ! Create any needed directories

initialize = .TRUE.
do igd=beg_grid,end_grid   ! Loop over all desired grids

   ! Total number of dimensions, variables, and global attributes:
   n_dims = 5; n_vars = 14; n_g_Atts = 7

   ! Allocate and define "dimensions":
   if (initialize) then
      allocate(dim_name(n_dims), dimv(n_dims), stat=a_err)
      if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
           procedure_name, 'dim_n_dims')
   end if
   i = 1; dim_name(i) = 'x'; dimv(i) = gd(igd)%nxp
   i = 2; dim_name(i) = 'y'; dimv(i) = gd(igd)%nyp
   i = 3; dim_name(i) = 'z_SBPS'; dimv(i) = nzg+nzs
   i = 4; dim_name(i) = 'patches'; dimv(i) = npatch
   i = 5; dim_name(i) = 'TC_azm'; dimv(i) = nazm

   ! Allocate some variable- and global_attribute-related items:
   if (initialize) then
      allocate(var_name(n_vars), var_type(n_vars), var_ndims(n_vars),  &
           var_nAtts(n_vars), var_f4_missing_value(n_vars), stat=a_err) 
      if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
           procedure_name, 'dim_n_vars')
      allocate(g_Att(n_g_Atts), stat=a_err) 
      if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
           procedure_name, 'dim_n_g_Atts')    
   end if

   ! Define "global attributes":
   i = 1; tmp_mNc_str = trim(PRAMS_version); ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(g_Att(i), 'PRAMS_version', i, ic, tmp_mNc_str)
   i = 2; call mNc_define_f_md_Att_obj(g_Att(i), 'grid', i, 1, (/ igd /))
   i = 3; tmp_1d_f4(:) = (/ gd(igd)%deltax, gd(igd)%deltay, gd(igd)%plat,  &
        gd(igd)%plon, a(iglatn(igd)), a(iglonn(igd)), gd(igd)%toptenh,  &
        gd(igd)%toptwvlx, gd(igd)%toptwvly, gd(igd)%tinwvlx, gd(igd)%tinwvly,  &
        gd(igd)%albwvlx, gd(igd)%albwvly, gd(igd)%z0max, z0fact,  &
        gd(igd)%topolevel_for_z0, gd(igd)%z0_of_topolevel /)
   call mNc_define_f_md_Att_obj(g_Att(i), 'config_parameters_f4', i,  &
        n_config_parameters_f4, tmp_1d_f4)
   i = 4; call mNc_define_f_md_Att_obj(g_Att(i), 'config_parameters_i4', i,  &
        n_config_parameters_i4,  &
        (/ gd(igd)%itoptflg, gd(igd)%itinflg, gd(igd)%ialbflg,   &
        gd(igd)%nofilflg, gd(igd)%itopsflg, gd(igd)%iz0flg /))
   i = 5; tmp_mNc_str = trim(gd(igd)%itoptfn); ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(g_Att(i), 'itoptfn', i, ic, tmp_mNc_str)
   i = 6; tmp_mNc_str = trim(gd(igd)%itinfn); ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(g_Att(i), 'itinfn', i, ic, tmp_mNc_str)
   i = 7; tmp_mNc_str = trim(gd(igd)%ialbfn); ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(g_Att(i), 'ialbfn', i, ic, tmp_mNc_str)

   ! Allocate variable-related items:
   max_var_ndims = 4; max_var_nAtts = 4
   if (initialize) then
      allocate(var_dimids(max_var_ndims, n_vars), stat=a_err) 
      if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
           procedure_name, 'dim_max_var_ndims')
      allocate(var_Att(max_var_nAtts, n_vars), stat=a_err) 
      if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
           procedure_name, 'dim_max_var_nAtts')
   end if

   ! Define "variables":

   i = 1; var_name(i) = 'topo'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 2; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2 /)
   ia = 1; tmp_mNc_str = 'PRAMS topography'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'm'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -20000._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 2; var_name(i) = 'z0'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 2; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2 /)
   ia = 1; tmp_mNc_str = 'PRAMS aerodynamic surface roughness'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'm'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 3; var_name(i) = 'patch_fcv'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS fractional patch area'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'none'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 4; var_name(i) = 'TI_static'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS thermal inertia (static)'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'SI_units'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 5; var_name(i) = 'mass_density'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 4; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 3, 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS SBPS mass density'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'kg/m^3'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 6; var_name(i) = 'thermal_capacity'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 4; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 3, 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS SBPS thermal capacity'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'J/(kg.K)'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 7; var_name(i) = 'thermal_conductivity'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 4; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 3, 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS SBPS thermal conductivity'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'W/(m.K)'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 8; var_name(i) = 'albedo_static'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS surface albedo (static)'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'none'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 9; var_name(i) = 'cosZ_horizon'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 5 /)
   ia = 1; tmp_mNc_str = 'PRAMS horizon angle/elevation'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'none'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -100._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 10; var_name(i) = 'TCx_obstruct'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 5 /)
   ia = 1; tmp_mNc_str = 'TC horizontal planar distance to horizon obstruction'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'm'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 11; var_name(i) = 'TCz_obstruct'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 5 /)
   ia = 1; tmp_mNc_str = 'TC height of horizon obstruction'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'm'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -huge_f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 12; var_name(i) = 'sfc_slope_angle'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 2; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2 /)
   ia = 1; tmp_mNc_str = 'PRAMS surface slope angle'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'radians'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -huge_f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 13; var_name(i) = 'sfc_azimuth_angle'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 2; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2 /)
   ia = 1; tmp_mNc_str = 'PRAMS surface azimuth angle (orientation)'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'radians'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -huge_f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   i = 14; var_name(i) = 'emissivity_static'; var_type(i) = mNc_FLOAT_type
   var_ndims(i) = 3; var_nAtts(i) = 4
   var_dimids(1:var_ndims(i),i) = (/ 1, 2, 4 /)
   ia = 1; tmp_mNc_str = 'PRAMS surface emissivity (static)'
   ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'long_name', ia, ic, tmp_mNc_str)
   ia = 2; tmp_mNc_str = 'none'; ic = len_trim(tmp_mNc_str)
   call mNc_define_f_md_Att_obj(var_Att(ia,i), 'units', ia, ic, tmp_mNc_str)
   var_f4_missing_value(i) = -1._f4
   ia = 3; call mNc_define_f_md_Att_obj(var_Att(ia,i), 'missing_value', ia,  &
        1, (/ var_f4_missing_value(i) /))
   ia = 4; call mNc_define_f_md_Att_obj(var_Att(ia,i), '_FillValue', ia, 1,  &
        (/ var_f4_missing_value(i) /))

   ! Create output data file name:
   write (cgrid, '(a1,i2.2)') 'g', igd
   flnmd = trim(input_sfcfile_pfx)//'-F-'//cgrid//'.nc'

   ! Begin writing the output file:
   call mNc_populate_f_md_obj_w(flnmd, output_file_obj, n_dims, n_vars,  &
        n_g_Atts, dim_name, dimv, g_Att, var_name, var_type, var_ndims,  &
        var_nAtts, var_dimids, var_Att)

   ! Write variables to the output file:

   dim_1 = gd(igd)%nxp; dim_2 = gd(igd)%nyp; nxyp = gd(igd)%nxyp
   dim_3 = nzg+nzs; dim_4 = npatch; dim_5 = nazm

   allocate(a_tmp_2d(dim_1,dim_2), stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
        procedure_name, 'dim_1_2')

   ab = itoptn(igd); ae = ab+nxyp-1
   a_tmp_2d = reshape(a(ab:ae), (/ dim_1, dim_2 /))
   call mNc_write_var(output_file_obj, 'topo', a_tmp_2d)
   
   ab = itopz0n(igd); ae = ab+nxyp-1
   a_tmp_2d = reshape(a(ab:ae), (/ dim_1, dim_2 /))
   call mNc_write_var(output_file_obj, 'z0', a_tmp_2d)

   allocate(a_tmp_3d(dim_1,dim_2,dim_4), a_tmp_4d(dim_3,dim_1,dim_2,dim_4),  &
        stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
        procedure_name, 'dim_1_2_4+dim_1_2_3_4')

   ! Fractional patch area:
   ab = ipatchfcvn(igd); ae = ab+nxyp*dim_4-1
   a_tmp_3d = reshape(a(ab:ae), (/ dim_1, dim_2, dim_4 /))
   call mNc_write_var(output_file_obj, 'patch_fcv', a_tmp_3d)

   ! Thermal inertia (static):
   ab = itistatn(igd); ae = ab+nxyp*dim_4-1
   a_tmp_3d = reshape(a(ab:ae), (/ dim_1, dim_2, dim_4 /))
   call mNc_write_var(output_file_obj, 'TI_static', a_tmp_3d)

   ! SBPS mass density:
   ab = isbpsdensn(igd); ae = ab+nxyp*dim_3*dim_4-1
   do i=ab,ae
      if (a(i) >= check_undef_val_f4) a(i) = var_f4_missing_value(5)
   end do
   a_tmp_4d = reshape(a(ab:ae), (/ dim_3, dim_1, dim_2, dim_4 /))
   call mNc_write_var(output_file_obj, 'mass_density', a_tmp_4d)  
      
   ! SBPS thermal capacity:
   ab = isbpstmcpn(igd); ae = ab+nxyp*dim_3*dim_4-1
   do i=ab,ae
      if (a(i) >= check_undef_val_f4) a(i) = var_f4_missing_value(5)
   end do
   a_tmp_4d = reshape(a(ab:ae), (/ dim_3, dim_1, dim_2, dim_4 /))
   call mNc_write_var(output_file_obj, 'thermal_capacity', a_tmp_4d)      
     
   ! SBPS thermal conductivity:
   ab = isbpstmcdn(igd); ae = ab+nxyp*dim_3*dim_4-1
   do i=ab,ae
      if (a(i) >= check_undef_val_f4) a(i) = var_f4_missing_value(5)
   end do
   a_tmp_4d = reshape(a(ab:ae), (/ dim_3, dim_1, dim_2, dim_4 /))
   call mNc_write_var(output_file_obj, 'thermal_conductivity', a_tmp_4d)      
      
   ! Static surface albedo:
   ab = ialbdstatn(igd); ae = ab+nxyp*dim_4-1
   a_tmp_3d = reshape(a(ab:ae), (/ dim_1, dim_2, dim_4 /))
   call mNc_write_var(output_file_obj, 'albedo_static', a_tmp_3d)

   ! Reallocate 'a_tmp_3d' using different dimensions:
   deallocate(a_tmp_3d, stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_deallocation,  &
        procedure_name, 'dim_1_2_4')
   allocate(a_tmp_3d(dim_1,dim_2,dim_5), stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_allocation,  &
        procedure_name, 'dim_1_2_5')

   ! "Horizon angle":
   a_tmp_3d = gd(igd)%cosz_horizon(1:dim_1,1:dim_2,1:dim_5)
   call mNc_write_var(output_file_obj, 'cosZ_horizon', a_tmp_3d)

   ! TC horizontal planar distance to horizon obstruction:
   a_tmp_3d = gd(igd)%TCx_obstruct(1:dim_1,1:dim_2,1:dim_5)
   call mNc_write_var(output_file_obj, 'TCx_obstruct', a_tmp_3d)

   ! TC height of horizon obstruction:
   a_tmp_3d = gd(igd)%TCz_obstruct(1:dim_1,1:dim_2,1:dim_5)
   call mNc_write_var(output_file_obj, 'TCz_obstruct', a_tmp_3d)

   ! Surface slope:
   ab = itslopangn(igd); ae = ab+nxyp-1
   a_tmp_2d = reshape(a(ab:ae), (/ dim_1, dim_2 /))
   call mNc_write_var(output_file_obj, 'sfc_slope_angle', a_tmp_2d)
   
   ! Surface azimuth angle (orientation):
   ab = itazmangn(igd); ae = ab+nxyp-1
   a_tmp_2d = reshape(a(ab:ae), (/ dim_1, dim_2 /))
   call mNc_write_var(output_file_obj, 'sfc_azimuth_angle', a_tmp_2d)

   ! Static surface emissivity:
   call mNc_write_var(output_file_obj, 'emissivity_static',  &
        gd(igd)%emissivity_static)

   call mNc_sync_with_file(output_file_obj)   ! Optional

   ! Finalize file, free object memory:
   call mNc_recycle_f_md_obj(output_file_obj)

   ! Must "recycle" the attribute objects before reusing:
   do i_d2=1,size(var_Att,2)
      do i_d1=1,size(var_Att,1)
         if (mNc_f_md_Att_populated(var_Att(i_d1,i_d2)))  &
              call mNc_recycle_f_md_Att_obj(var_Att(i_d1,i_d2))
      end do
   end do
   do i_d1=1,size(g_Att,1)
      if (mNc_f_md_Att_populated(g_Att(i_d1)))  &
           call mNc_recycle_f_md_Att_obj(g_Att(i_d1))
   end do

   deallocate(a_tmp_2d, a_tmp_3d, a_tmp_4d, stat=a_err)
   if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'a_tmp_Xd')

   initialize = .FALSE.   
end do

deallocate(dim_name, dimv, stat=a_err)
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'dim_n_dims')
deallocate(var_name, var_type, var_ndims, var_nAtts, var_f4_missing_value,  &
     stat=a_err) 
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'dim_n_vars')
deallocate(g_Att, stat=a_err) 
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'dim_n_g_Atts')    
deallocate(var_dimids, stat=a_err) 
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'dim_max_var_ndims')
deallocate(var_Att, stat=a_err) 
if (a_err /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'dim_max_var_nAtts')

end subroutine sfcfile_out


!```````````````````````````````````````````````````````````````````
subroutine sfcfile_read(ifm, want_to_read_data, file_is_not_OK)

!+ Ingests premade surface data files into the model

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN !%, MAX_PROC_NAME_LEN
use univ_numtype_constants_mod, only: epsilon_f4
use manage_NetCDF_classic_mod, only: mNc_file_metadata_obj,  &
     mNc_populate_f_md_obj_r, mNc_extract_g_Att_val, mNc_recycle_f_md_obj,  &
     mNc_dim_val
use manage_NetCDF_classic_rv_mod, only: mNc_read_var
use main_memory_mod, only: a, gd, nzg, nzs, input_sfcfile_pfx, z0fact,  &
     npatch
use vtables_mod, only: iglatn, iglonn, itoptn, itopz0n, ipatchfcvn, itistatn,  &
     isbpsdensn, isbpstmcpn, isbpstmcdn, ialbdstatn, itslopangn, itazmangn
use topo_RT_effects_dims_mod, only: nazm
use surface_file_mod, only: n_config_parameters_f4, n_config_parameters_i4

implicit none

!=== Argument declarations:

integer, intent(IN) :: ifm
logical, intent(IN) :: want_to_read_data

logical, intent(OUT) :: file_is_not_OK

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'sfcfile_read'

character(len=MAX_FILE_PATH_LEN) :: flnmd
character(len=MAX_FILE_PATH_LEN), dimension(3) :: tmp_fn
character(len=3) :: cgrid
logical :: df_is_there
integer :: i, ab, ae, nxyp, n4, n5, n2, n3
type(mNc_file_metadata_obj) :: file_in
real(f4), dimension(n_config_parameters_f4) :: tmp_grid_cfg_md_f4,  &
     current_grid_cfg_md_f4
integer, dimension(n_config_parameters_i4) :: tmp_grid_cfg_md_i4,  &
     current_grid_cfg_md_i4

real(f4), pointer, dimension(:,:) :: p_tmp_2d => NULL()
real(f4), pointer, dimension(:,:,:) :: p_tmp_3d => NULL()
real(f4), pointer, dimension(:,:,:,:) :: p_tmp_4d => NULL()

!=== Executable statements:

!=== Read surface characteristics, one file for each grid:

write (cgrid, '(a1,i2.2)') 'g', ifm
flnmd = trim(input_sfcfile_pfx)//'-F-'//cgrid//'.nc'

write (*, '(/,a,i3,1x,a,/)') 'Checking for grid', ifm, 'surface data file...'
write (*, '(2x,2a,/,12x,a)') '-- File: ', flnmd(1:len_trim(flnmd))

! Check if surface file exists for this grid (if not, exit):
inquire(file=flnmd(1:len_trim(flnmd)), exist=df_is_there)
file_is_not_OK = .not.df_is_there
if (file_is_not_OK) then
   write (*, '(2x,a,i3,1x,a)') '!! Surface file for grid', ifm, 'not found.'
   write (*, '(2x,a)') '!! (RE)CREATING all surface files ...'
   return
end if

! Read header file info; check for consistency with current model run
!  configuration:

write (*, '(2x,a)') '-- Reading surface file metadata ...'

call mNc_populate_f_md_obj_r(flnmd, file_in)

if (mNc_dim_val(file_in, 1) /= gd(ifm)%nxp .or.  &
     mNc_dim_val(file_in, 2) /= gd(ifm)%nyp .or.  &
     mNc_dim_val(file_in, 3) /= nzg+nzs .or.  &
     mNc_dim_val(file_in, 4) /= npatch) file_is_not_OK = .TRUE.

call mNc_extract_g_Att_val(file_in, 'config_parameters_f4', tmp_grid_cfg_md_f4)
current_grid_cfg_md_f4 = (/ gd(ifm)%deltax, gd(ifm)%deltay, gd(ifm)%plat,  &
        gd(ifm)%plon, a(iglatn(ifm)), a(iglonn(ifm)), gd(ifm)%toptenh,  &
        gd(ifm)%toptwvlx, gd(ifm)%toptwvly, gd(ifm)%tinwvlx, gd(ifm)%tinwvly,  &
        gd(ifm)%albwvlx, gd(ifm)%albwvly, gd(ifm)%z0max, z0fact,  &
        gd(ifm)%topolevel_for_z0, gd(ifm)%z0_of_topolevel /)
do i=1,size(tmp_grid_cfg_md_f4)
   if (abs(tmp_grid_cfg_md_f4(i)-current_grid_cfg_md_f4(i)) >  &
        abs(10._f4*current_grid_cfg_md_f4(i)*epsilon_f4))  &
        file_is_not_OK = .TRUE.
end do
 
call mNc_extract_g_Att_val(file_in, 'config_parameters_i4', tmp_grid_cfg_md_i4)
current_grid_cfg_md_i4 = (/ gd(ifm)%itoptflg, gd(ifm)%itinflg,  &
     gd(ifm)%ialbflg, gd(ifm)%nofilflg, gd(ifm)%itopsflg, gd(ifm)%iz0flg /)
do i=1,size(tmp_grid_cfg_md_i4)
   if (tmp_grid_cfg_md_i4(i) /= current_grid_cfg_md_i4(i))  &
        file_is_not_OK = .TRUE.
end do

tmp_fn(:) = ';'   ! Intialize before populating

call mNc_extract_g_Att_val(file_in, 'itoptfn', tmp_fn(1))
i = index(tmp_fn(1), '/', back=.TRUE.)+1         ! Start of "basename"
ab = index(gd(ifm)%itoptfn, '/', back=.TRUE.)+1  !
if (tmp_fn(1)(i:len_trim(tmp_fn(1))) /=  &
     gd(ifm)%itoptfn(ab:len_trim(gd(ifm)%itoptfn))) file_is_not_OK = .TRUE.

call mNc_extract_g_Att_val(file_in, 'itinfn', tmp_fn(2))
i = index(tmp_fn(2), '/', back=.TRUE.)+1         ! Start of "basename"
ab = index(gd(ifm)%itinfn, '/', back=.TRUE.)+1  !
if (tmp_fn(2)(i:len_trim(tmp_fn(2))) /=  &
     gd(ifm)%itinfn(ab:len_trim(gd(ifm)%itinfn))) file_is_not_OK = .TRUE.

call mNc_extract_g_Att_val(file_in, 'ialbfn', tmp_fn(3))
i = index(tmp_fn(3), '/', back=.TRUE.)+1         ! Start of "basename"
ab = index(gd(ifm)%ialbfn, '/', back=.TRUE.)+1  !
if (tmp_fn(3)(i:len_trim(tmp_fn(3))) /=  &
     gd(ifm)%ialbfn(ab:len_trim(gd(ifm)%ialbfn))) file_is_not_OK = .TRUE.

if (file_is_not_OK) then
   write (*, *) '!! SFCfile mismatch on grid:', ifm
   write (*, *) '!! Values: model, file'
   write (*, *) '!! -------------------'
   write (*, *) '!! nxp:', gd(ifm)%nxp, mNc_dim_val(file_in, 1)
   write (*, *) '!! nyp:', gd(ifm)%nyp, mNc_dim_val(file_in, 2)
   write (*, *) '!! nzg+nzs:', nzg+nzs, mNc_dim_val(file_in, 3)
   write (*, *) '!! npatch:', npatch, mNc_dim_val(file_in, 4)
   write (*, *) '!! deltax:', gd(ifm)%deltax, tmp_grid_cfg_md_f4(1)
   write (*, *) '!! deltay:', gd(ifm)%deltay, tmp_grid_cfg_md_f4(2)
   write (*, *) '!! platn:', gd(ifm)%plat, tmp_grid_cfg_md_f4(3)
   write (*, *) '!! plonn:', gd(ifm)%plon, tmp_grid_cfg_md_f4(4)
   write (*, *) '!! SW lat:', a(iglatn(ifm)), tmp_grid_cfg_md_f4(5)
   write (*, *) '!! SW lon:', a(iglonn(ifm)), tmp_grid_cfg_md_f4(6)
   write (*, *) '!! itoptflg:', gd(ifm)%itoptflg, tmp_grid_cfg_md_i4(1)
   write (*, *) '!! itinflg:', gd(ifm)%itinflg, tmp_grid_cfg_md_i4(2)
   write (*, *) '!! ialbflg:', gd(ifm)%ialbflg, tmp_grid_cfg_md_i4(3)
   write (*, *) '!! nofilflg:', gd(ifm)%nofilflg, tmp_grid_cfg_md_i4(4)
   write (*, *) '!! itopsflg:', gd(ifm)%itopsflg, tmp_grid_cfg_md_i4(5)
   write (*, *) '!! toptenh:', gd(ifm)%toptenh, tmp_grid_cfg_md_f4(7)
   write (*, *) '!! toptwvlx:', gd(ifm)%toptwvlx, tmp_grid_cfg_md_f4(8)
   write (*, *) '!! toptwvly:', gd(ifm)%toptwvly, tmp_grid_cfg_md_f4(9)
   write (*, *) '!! tinwvlx:', gd(ifm)%tinwvlx, tmp_grid_cfg_md_f4(10)
   write (*, *) '!! tinwvly:', gd(ifm)%tinwvly, tmp_grid_cfg_md_f4(11)
   write (*, *) '!! albwvlx:', gd(ifm)%albwvlx, tmp_grid_cfg_md_f4(12)
   write (*, *) '!! albwvly:', gd(ifm)%albwvly, tmp_grid_cfg_md_f4(13)
   write (*, *) '!! iz0flg:', gd(ifm)%iz0flg, tmp_grid_cfg_md_i4(6)
   write (*, *) '!! z0max:', gd(ifm)%z0max, tmp_grid_cfg_md_f4(14)
   write (*, *) '!! z0fact:', z0fact, tmp_grid_cfg_md_f4(15)
   write (*, *) '!! topolevel_for_z0:', gd(ifm)%topolevel_for_z0,  &
        tmp_grid_cfg_md_f4(16)
   write (*, *) '!! z0_of_topolevel:', gd(ifm)%z0_of_topolevel,  &
        tmp_grid_cfg_md_f4(17)
   write (*, *) '!! itoptfn:', trim(gd(ifm)%itoptfn), ' ', trim(tmp_fn(1))
   write (*, *) '!! itinfn:', trim(gd(ifm)%itinfn), ' ', trim(tmp_fn(2))
   write (*, *) '!! ialbfn:', trim(gd(ifm)%ialbfn), ' ', trim(tmp_fn(3))
   write (*, *) '!! -------------------'

   write (*, '(2x,a)') '** Will remake file...'
   return
end if

! We now assume that the file is OK:
write (*, '(2x,a,1x,i3,1x,a)') '-- Surface file passed sanity check.'

! Check if user wants to actually read in the data:

if (.not.want_to_read_data) return

n4 = nzg+nzs; n5 = npatch
nxyp = gd(ifm)%nxyp; n2 = gd(ifm)%nxp; n3 = gd(ifm)%nyp

! Read surface data file:

write (*, '(2x,a)') '-- Reading surface file data...'
   
call mNc_read_var(file_in, 'topo', p_tmp_2d)
ab = itoptn(ifm); ae = ab+nxyp-1
a(ab:ae) = reshape(p_tmp_2d, (/ nxyp /))

call mNc_read_var(file_in, 'z0', p_tmp_2d)
ab = itopz0n(ifm); ae = ab+nxyp-1
a(ab:ae) = reshape(p_tmp_2d, (/ nxyp /))

call mNc_read_var(file_in, 'patch_fcv', p_tmp_3d)
ab = ipatchfcvn(ifm); ae = ab+nxyp*n5-1
a(ab:ae) = reshape(p_tmp_3d, (/ nxyp*n5 /))

call mNc_read_var(file_in, 'TI_static', p_tmp_3d)
ab = itistatn(ifm); ae = ab+nxyp*n5-1
a(ab:ae) = reshape(p_tmp_3d, (/ nxyp*n5 /))

call mNc_read_var(file_in, 'mass_density', p_tmp_4d)
ab = isbpsdensn(ifm); ae = ab+nxyp*n4*n5-1
a(ab:ae) = reshape(p_tmp_4d, (/ nxyp*n4*n5 /))

call mNc_read_var(file_in, 'thermal_capacity', p_tmp_4d)
ab = isbpstmcpn(ifm); ae = ab+nxyp*n4*n5-1
a(ab:ae) = reshape(p_tmp_4d, (/ nxyp*n4*n5 /))

call mNc_read_var(file_in, 'thermal_conductivity', p_tmp_4d)
ab = isbpstmcdn(ifm); ae = ab+nxyp*n4*n5-1
a(ab:ae) = reshape(p_tmp_4d, (/ nxyp*n4*n5 /))

call mNc_read_var(file_in, 'albedo_static', p_tmp_3d)
ab = ialbdstatn(ifm); ae = ab+nxyp*n5-1
a(ab:ae) = reshape(p_tmp_3d, (/ nxyp*n5 /))

call mNc_read_var(file_in, 'cosZ_horizon', p_tmp_3d)
gd(ifm)%cosz_horizon(1:n2,1:n3,1:nazm) = p_tmp_3d(:,:,:)

call mNc_read_var(file_in, 'TCx_obstruct', p_tmp_3d)
gd(ifm)%TCx_obstruct(1:n2,1:n3,1:nazm) = p_tmp_3d(:,:,:)

call mNc_read_var(file_in, 'TCz_obstruct', p_tmp_3d)
gd(ifm)%TCz_obstruct(1:n2,1:n3,1:nazm) = p_tmp_3d(:,:,:)

call mNc_read_var(file_in, 'sfc_slope_angle', p_tmp_2d)
ab = itslopangn(ifm); ae = ab+nxyp-1
a(ab:ae) = reshape(p_tmp_2d, (/ nxyp /))

call mNc_read_var(file_in, 'sfc_azimuth_angle', p_tmp_2d)
ab = itazmangn(ifm); ae = ab+nxyp-1
a(ab:ae) = reshape(p_tmp_2d, (/ nxyp /))

call mNc_read_var(file_in, 'emissivity_static', p_tmp_3d)
gd(ifm)%emissivity_static = p_tmp_3d

call mNc_recycle_f_md_obj(file_in)   ! Close file, free object memory

end subroutine sfcfile_read
