!```````````````````````````````````````````````````````````````````
module surface_init_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine toptinit(n2, n3, topt)

!+ Initialize topography with default values (all zero)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3

real(f4), dimension(n2,n3), intent(INOUT) :: topt

!== Local declarations:

! NONE

!=== Executable statements:

! This default can be overridden by changing toptinit_user (in PRAMS_user.f90):

topt(:,:) = ZERO_f4

end subroutine toptinit


!```````````````````````````````````````````````````````````````````
subroutine TI_init(ng, n2, n3, n4, n5, n6, TI_static, sbps_density,  &
     sbps_thrmcap, sbps_thrmcnd)

!+ Initialize sub/super surface thermal inertia quantities (per grid basis)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: undef_value_f4
use main_memory_bspec_mod, only: nslcon

implicit none

!=== Argument declarations:

integer, intent(IN) :: ng, n2, n3, n4, n5, n6
real(f4), dimension(n4,n2,n3,n5), intent(IN) :: sbps_density, sbps_thrmcap
real(f4), dimension(n2,n3,n5), intent(IN) :: TI_static 

real(f4), dimension(n4,n2,n3,n5), intent(INOUT) :: sbps_thrmcnd

!== Local declarations:

integer :: i, j, ip, k 

!=== Executable statements:

do ip=1,n5      ! Loop over all patches

   ! Fill levels of SUBsurface:
 
   do j=1,n3
      do i=1,n2

         ! Thermal conductivity:

         if (nslcon == 0) then    ! No subsurface variation of apparent TI

            sbps_thrmcnd(1:n6,i,j,ip) = TI_static(i,j,ip)*TI_static(i,j,ip)/  & 
                 (sbps_density(1:n6,i,j,ip)*sbps_thrmcap(1:n6,i,j,ip))

         else if (nslcon == 1) then  ! subsurface apparent TI varies
            
            ! Only set thermal conductivity that has not already been set:
            do k=1,n6
               if (sbps_thrmcnd(k,i,j,ip) < ZERO_f4) then
                  sbps_thrmcnd(k,i,j,ip) = TI_static(i,j,ip)**2/  & 
                       (sbps_density(k,i,j,ip)*sbps_thrmcap(k,i,j,ip))
               end if
            end do

         end if

      end do
   end do

   ! Fill supersurface:
   sbps_thrmcnd(n6+1:n4,:,:,ip) = undef_value_f4

end do

end subroutine TI_init


!```````````````````````````````````````````````````````````````````
subroutine albedo_init(n2, n3, n5, ng, albedo, albedo_stat, glat)

!+ Check surface albedo for max/min violations (per grid)

use univ_kind_defs_mod, only: f4
  
implicit none

!=== Argument declarations:

integer, intent(IN) :: ng, n2, n3, n5
real(f4), dimension(n2,n3), intent(IN) :: glat
real(f4), intent(IN) :: albedo

real(f4), dimension(n2,n3,n5), intent(INOUT) :: albedo_stat

!== Local declarations:

real(f4), parameter ::  &
      albmin_perm = 0.02_f4,  &          ! Min permitted albedo
      albmax_perm = 0.7_f4            ! Max permitted albedo (H2O ice)

integer :: i, j, ip

!=== Executable statements:

do ip=1,n5      ! Loop over all patches

   do j=1,n3
      do i=1,n2

         if (albedo_stat(i,j,ip) < albmin_perm) then
            albedo_stat(i,j,ip) = albmin_perm
         else if (albedo_stat(i,j,ip) > albmax_perm) then
            albedo_stat(i,j,ip) = albmax_perm
         end if

      end do
   end do

end do

end subroutine albedo_init


!```````````````````````````````````````````````````````````````````
subroutine kludge_SBPSprop(n4, n2, n3, n5, n6, igd, TI_static, sbps_density,  &
     sbps_thrmcap, sbps_thrmcnd, glat, glon, albedo_static)

!+ Kludge subsurface material density, thermal conductivity, and/or
!   heat capacity for certain cases

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use body_specific_constants_mod, only: rho_CO2ice, s_rho, s_hc
use main_memory_bspec_mod, only: slz, nslcon

use PRAMS_SBPS_mod, only: cp_CO2ice

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd, n2, n3, n4, n5, n6
real(f4), dimension(n2,n3,n5), intent(IN) :: TI_static, albedo_static 
real(f4), dimension(n2,n3), intent(IN) :: glat, glon

real(f4), dimension(n4,n2,n3,n5), intent(INOUT) :: sbps_density,  &
     sbps_thrmcap, sbps_thrmcnd

!== Local declarations:

! A gridcell with TI <= this value will not be assumed to have significant
!  water ice from the surface down.  Somewhat arbitrary, but rooted in
!  TES-based nonpolar TI values.
real(f4), parameter :: max_nonice_TI = 850._f4

! Maximum gridcell TI for which surface-down water ice is considered dusty
!  (thus having properties with values < pure water ice).  Even more arbitrary,
!  but chosen to avoid horizontal "cliffs" in the subsurface characteristic
!  fields (as much as practical). 
real(f4), parameter :: max_nonpure_H2Oice_TI = 2000._f4

real(f4), parameter :: approx_H2Oi_cp = 2100._f4,  &  ! [J/(kg.K)]
                       approx_H2Oi_rho = 930._f4   ! [kg/m^3]


! Hand-digitized subsurface hydrogen (proxy for H2O) concentration map from
!  http://grs.lpl.arizona.edu/resultsimages/h2o_map.png, created using data
!  from the 2001 Mars Odyssey Gamma Ray Spectrometer:

!#integer, parameter :: n_GRS_Hmap_ref = 7
!#real(f4), dimension(n_GRS_Hmap_ref), parameter :: GRS_Hmap_ref_lat =  &
!#     (/ -60._f4, -45._f4, -30._f4, 15._f4, 30._f4, 45._f4, 60._f4 /)
!#real(f4), dimension(n_GRS_Hmap_ref), parameter :: GRS_Hmap_ref_px =  &
!#     (/ 489._f4, 457._f4, 423._f4, 322._f4, 288._f4, 255._f4, 222._f4 /)

integer, parameter :: n_GRS_Hmap_bnd = 48

real(f4), parameter :: GRS_Hmap_dlon = 7.5_f4,  &   ! [deg_E]
                       GRS_Hmap_beglon = -180._f4  ! [deg_E]

! Southern hemisphere: lon from -180 to 172.5, increment ~7.5 deg (+2 wrap-
!  around values):

!#real(f4), dimension(n_GRS_Hmap_bnd), parameter :: GRS_Hmap_S_bnd_px =  &
!#     (/ 466._f4, 466._f4, 465._f4, 465._f4, 466._f4, 467._f4, 469._f4,  &
!#     471._f4, 472._f4, 473._f4, 473._f4, 473._f4, 473._f4, 473._f4, 472._f4, &
!#     471._f4, 471._f4, 471._f4, 471._f4, 470._f4, 469._f4, 469._f4, 470._f4, &
!#     472._f4, 474._f4, 474._f4, 473._f4, 472._f4, 470._f4, 466._f4, 462._f4, &
!#     458._f4, 455._f4, 452._f4, 451._f4, 452._f4, 452._f4, 451._f4, 451._f4, &
!#     449._f4, 450._f4, 451._f4, 453._f4, 456._f4, 459._f4, 462._f4, 464._f4, &
!#     465._f4 /)

real(f4), dimension(0:n_GRS_Hmap_bnd+1), parameter :: GRS_Hmap_Sb_lat =  &
     (/ -48.75000_f4,  &    ! [deg_N]
     -49.21875_f4, -49.21875_f4, -48.75000_f4, -48.75000_f4, -49.21875_f4,  &
     -49.68750_f4, -50.62500_f4, -51.56250_f4, -52.03125_f4, -52.50000_f4,  &
     -52.50000_f4, -52.50000_f4, -52.50000_f4, -52.50000_f4, -52.03125_f4,  &
     -51.56250_f4, -51.56250_f4, -51.56250_f4, -51.56250_f4, -51.09375_f4,  &
     -50.62500_f4, -50.62500_f4, -51.09375_f4, -52.03125_f4, -52.96875_f4,  &
     -52.96875_f4, -52.50000_f4, -52.03125_f4, -51.09375_f4, -49.21875_f4,  &
     -47.34375_f4, -45.46875_f4, -44.11765_f4, -42.79412_f4, -42.35294_f4,  &
     -42.79412_f4, -42.79412_f4, -42.35294_f4, -42.35294_f4, -41.47059_f4,  &
     -41.91177_f4, -42.35294_f4, -43.23529_f4, -44.55882_f4, -45.93750_f4,  &
     -47.34375_f4, -48.28125_f4, -48.75000_f4,  &
     -49.21875_f4 /)

! Northern hemisphere: lon from -180 to 180., increment ~7.5 deg (+2 wrap-
!  around values):

!#real(f4), dimension(n_GRS_Hmap_bnd), parameter :: GRS_Hmap_N_bnd_px =  &
!#     (/ 258._f4, 264._f4, 269._f4, 272._f4, 276._f4, 277._f4, 273._f4,  &
!#     271._f4, 268._f4, 264._f4, 263._f4, 263._f4, 262._f4, 260._f4, 251._f4, &
!#     244._f4, 238._f4, 235._f4, 233._f4, 235._f4, 237._f4, 242._f4, 244._f4, &
!#     246._f4, 246._f4, 246._f4, 250._f4, 252._f4, 251._f4, 248._f4, 243._f4, &
!#     242._f4, 241._f4, 242._f4, 240._f4, 240._f4, 241._f4, 243._f4, 247._f4, &
!#     251._f4, 254._f4, 258._f4, 265._f4, 268._f4, 271._f4, 268._f4, 264._f4, &
!#     258._f4 /)

real(f4), dimension(0:n_GRS_Hmap_bnd+1), parameter :: GRS_Hmap_Nb_lat =  &
     (/ 43.63636_f4,  &   ! [deg_N]
     43.63636_f4,  40.90909_f4,  38.63636_f4,  37.27273_f4,  35.45454_f4,  &
     35.00000_f4,  36.81818_f4,  37.72727_f4,  39.09091_f4,  40.90909_f4,  &
     41.36364_f4,  41.36364_f4,  41.81818_f4,  42.72728_f4,  46.81818_f4,  &
     50.00000_f4,  52.72727_f4,  54.09091_f4,  55.00000_f4,  54.09091_f4,  &
     53.18182_f4,  50.90909_f4,  50.00000_f4,  49.09091_f4,  49.09091_f4,  &
     49.09091_f4,  47.27273_f4,  46.36364_f4,  46.81818_f4,  48.18182_f4,  &
     50.45454_f4,  50.90909_f4,  51.36364_f4,  50.90909_f4,  51.81818_f4,  &
     51.81818_f4,  51.36364_f4,  50.45454_f4,  48.63636_f4,  46.81818_f4,  &
     45.45454_f4,  43.63636_f4,  40.45454_f4,  39.09091_f4,  37.72727_f4,  &
     39.09091_f4,  40.90909_f4,  43.63636_f4,  &
     43.63636_f4 /)

! From NASA ARC Mars GCM 2.1 v23:
!  Depth below the surface where substantial subsurface H2O-ice begins in each
!  hemisphere [m]:
real(f4), parameter :: depth_SBS_H2Oi_N = 0.0545_f4,  &
                       depth_SBS_H2Oi_S = 0.0805_f4

integer :: i, j, ip, i_lon, k, k_b, k_e, kmax_SBS_H2Oi_N, kmax_SBS_H2Oi_S
real(f4) :: ice_purity_factor, ice_purity_TI_f, ice_purity_albedo_f,  &
     min_Nb_lat, max_Sb_lat, lat_test, intrp_f, f_lon

!=== Executable statements:

min_Nb_lat = minval(GRS_Hmap_Nb_lat)
do k=1,n6
   if (slz(k) > -depth_SBS_H2Oi_N) exit
end do
kmax_SBS_H2Oi_N = min(k, n6)      ! Top of SBS H2O ice table in NH

max_Sb_lat = maxval(GRS_Hmap_Sb_lat)
do k=1,n6
   if (slz(k) > -depth_SBS_H2Oi_S) exit
end do
kmax_SBS_H2Oi_S = min(k, n6)      ! Top of SBS H2O ice table in SH

do ip=1,n5      ! Loop over all patches

   do j=1,n3
      do i=1,n2

         if (nslcon == 1) then
            sbps_thrmcnd(1:n6,i,j,ip) = -ONE_f4  ! Sentinel value for `TI_init`
         end if
    
         if (glat(i,j) > 70._f4 .and.  &    ! North polar region
              (TI_static(i,j,ip) > max_nonice_TI .or.  &
              albedo_static(i,j,ip) > 0.25_f4)) then
            ! Assume that there is only perennial H2O ice in this region.  
            ice_purity_TI_f = (TI_static(i,j,ip)-max_nonice_TI)/  &
                 (max_nonpure_H2Oice_TI-max_nonice_TI)
            ice_purity_albedo_f = (albedo_static(i,j,ip)-0.25_f4)/  &
                 (0.28_f4-0.25_f4)
            ice_purity_factor = max(ice_purity_TI_f, ice_purity_albedo_f)
            ice_purity_factor = min(ice_purity_factor, ONE_f4)
            ice_purity_factor = sqrt(ice_purity_factor)   ! Arbitrary mod 
            sbps_thrmcap(1:n6,i,j,ip) = s_hc+  &
                 ice_purity_factor*(approx_H2Oi_cp-s_hc)
            sbps_density(1:n6,i,j,ip) = s_rho+  &
                 ice_purity_factor*(approx_H2Oi_rho-s_rho)

         else if (nslcon == 1 .and. glat(i,j) >= min_Nb_lat) then ! NH sbs ice
    
            f_lon = (glon(i,j)-GRS_Hmap_beglon)/GRS_Hmap_dlon
            i_lon = int(f_lon); intrp_f = mod(f_lon, ONE_f4)
            lat_test = (ONE_f4-intrp_f)*GRS_Hmap_Nb_lat(i_lon)+  &
                 intrp_f*GRS_Hmap_Nb_lat(i_lon+1)
            if (glat(i,j) >= lat_test) then
               ice_purity_factor = 0.85_f4  ! Somewhat arbitrary
               k_b = 1; k_e = kmax_SBS_H2Oi_N
               sbps_thrmcap(k_b:k_e,i,j,ip) =  &
                    (ONE_f4-ice_purity_factor)*s_hc+  &
                    ice_purity_factor*approx_H2Oi_cp
               sbps_density(k_b:k_e,i,j,ip) =  &
                    (ONE_f4-ice_purity_factor)*s_rho+  &
                    ice_purity_factor*approx_H2Oi_rho
               sbps_thrmcnd(k_b:k_e,i,j,ip) = max_nonpure_H2Oice_TI**2/  &
                    (sbps_density(k_b:k_e,i,j,ip)*sbps_thrmcap(k_b:k_e,i,j,ip))

               ice_purity_factor = ice_purity_factor*0.5_f4 ! "Blend" with soil
               k = kmax_SBS_H2Oi_N+1
               if (k <= n6) then    ! Only if ice table top is subsurface 
                  sbps_thrmcap(k,i,j,ip) = (ONE_f4-ice_purity_factor)*s_hc+  &
                       ice_purity_factor*approx_H2Oi_cp
                  sbps_density(k,i,j,ip) = (ONE_f4-ice_purity_factor)*s_rho+  &
                       ice_purity_factor*approx_H2Oi_rho
                  sbps_thrmcnd(k,i,j,ip) =  &   ! Uses average soil and ice TI
                       (0.5_f4*(TI_static(i,j,ip)+max_nonpure_H2Oice_TI))**2/  &
                       (sbps_density(k,i,j,ip)*sbps_thrmcap(k,i,j,ip))
               end if
            end if

         else if (glat(i,j) < -70._f4 .and.  &    ! South polar region
            TI_static(i,j,ip) > max_nonice_TI) then

            if (albedo_static(i,j,ip) > 0.26_f4) then ! Assume perennial CO2 ice
               sbps_thrmcap(1:n6,i,j,ip) = cp_CO2ice(140._f4)  ! Representative
               sbps_density(1:n6,i,j,ip) = rho_CO2ice
            else    ! Assume perennial H2O ice
               ice_purity_factor = (TI_static(i,j,ip)-max_nonice_TI)/  &
                    (max_nonpure_H2Oice_TI-max_nonice_TI)             
               ice_purity_factor = min(ice_purity_factor, ONE_f4)
               ice_purity_factor = sqrt(ice_purity_factor)   ! Arbitrary mod 
               sbps_thrmcap(1:n6,i,j,ip) = s_hc+  &
                    ice_purity_factor*(approx_H2Oi_cp-s_hc)
               sbps_density(1:n6,i,j,ip) = s_rho+  &
                    ice_purity_factor*(approx_H2Oi_rho-s_rho)
            end if

         else if (nslcon == 1 .and. glat(i,j) <= max_Sb_lat) then ! SH sbs ice
    
            f_lon = (glon(i,j)-GRS_Hmap_beglon)/GRS_Hmap_dlon
            i_lon = int(f_lon); intrp_f = mod(f_lon, ONE_f4)
            lat_test = (ONE_f4-intrp_f)*GRS_Hmap_Sb_lat(i_lon)+  &
                 intrp_f*GRS_Hmap_Sb_lat(i_lon+1)
            if (glat(i,j) <= lat_test) then
               ice_purity_factor = 0.85_f4  ! Somewhat arbitrary
               k_b = 1; k_e = kmax_SBS_H2Oi_S
               sbps_thrmcap(k_b:k_e,i,j,ip) =  &
                    (ONE_f4-ice_purity_factor)*s_hc+  &
                    ice_purity_factor*approx_H2Oi_cp
               sbps_density(k_b:k_e,i,j,ip) =  &
                    (ONE_f4-ice_purity_factor)*s_rho+  &
                    ice_purity_factor*approx_H2Oi_rho
               sbps_thrmcnd(k_b:k_e,i,j,ip) = max_nonpure_H2Oice_TI**2/  &
                    (sbps_density(k_b:k_e,i,j,ip)*sbps_thrmcap(k_b:k_e,i,j,ip))

               ice_purity_factor = ice_purity_factor*0.5_f4 ! "Blend" with soil
               k = kmax_SBS_H2Oi_S+1
               if (k <= n6) then    ! Only if ice table top is subsurface 
                  sbps_thrmcap(k,i,j,ip) = (ONE_f4-ice_purity_factor)*s_hc+  &
                       ice_purity_factor*approx_H2Oi_cp
                  sbps_density(k,i,j,ip) = (ONE_f4-ice_purity_factor)*s_rho+  &
                       ice_purity_factor*approx_H2Oi_rho
                  sbps_thrmcnd(k,i,j,ip) =  &   ! Uses average soil and ice TI
                       (0.5_f4*(TI_static(i,j,ip)+max_nonpure_H2Oice_TI))**2/  &
                       (sbps_density(k,i,j,ip)*sbps_thrmcap(k,i,j,ip))
               end if
            end if

         end if

      end do
   end do

end do

end subroutine kludge_SBPSprop


!```````````````````````````````````````````````````````````````````
subroutine calc_SBPS_Tlev(n6)

!+ Compute subsurface tempK LEVEL depths

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: SBPS_Tlev
use main_memory_bspec_mod, only: slz

implicit none

!=== Argument declarations:

integer, intent(IN) :: n6

!== Local declarations:

integer :: iq

!=== Executable statements:

! Compute subsurface tempK LEVEL depths (Kq):

do iq=2,n6
   SBPS_Tlev(iq) = 0.5_f4*(slz(iq-1)+slz(iq))
end do
SBPS_Tlev(1) = slz(1)-(SBPS_Tlev(2)-slz(1)) ! Depth of lowest layer midpoint

end subroutine calc_SBPS_Tlev


!```````````````````````````````````````````````````````````````````
subroutine get_TES_cap_lat(nlon, tgt_lon, Ls, cap_lat, cap_type, cap_loc,  &
     cap_year)

!+ Compute latitude of specified volatile cap boundaries

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: nlon, cap_year
real(f4), intent(IN) :: Ls
character(len=1), intent(IN) :: cap_loc     ! 'N' or 'S'
character(len=3), intent(IN) :: cap_type       ! 'IR' or 'vis'

real(f4), dimension(nlon), intent(INOUT) :: tgt_lon   ! range: -180. (W) to 180. (E)
real(f4), dimension(nlon), intent(INOUT) :: cap_lat

!== Local declarations:

integer, parameter :: IR_ncf = 51, vis_ncf = 11, c_ncf = 2

integer :: i

real(f4), dimension(IR_ncf) :: IR_coeff
real(f4), dimension(vis_ncf) :: vis_coeff
real(f4), dimension(c_ncf) :: c

real(f4), dimension(nlon) :: Ls_tmp

!>>>>>>>>
! Fourier coefficients for TES "IR" cap boundary (presumed CO2 ice extent) and
! the TES "vis" cap boundary (presumed water ice frost extent), for TES Mapping
! Years 1, 2, and 3.
!
! North polar data provided circa June 2005 by Tim N. Titus (USGS, Flagstaff).
!>>>>>>>>

real(f4), dimension(IR_ncf) :: IR_NPcoeff_TESY1 = (/ 72.3281_f4, -12.1983_f4,  &
     20.4711_f4, -1.32937_f4, -0.822584_f4, 0.712160_f4, -6.24307_f4,  &
     -0.823993_f4, 0.707180_f4, 1.64216_f4, 1.03764_f4, -0.232388_f4,  &
     0.222496_f4, -0.912184_f4, 0.564221_f4, 0.104080_f4, 0.111464_f4,  &
     0.169148_f4, -0.391652_f4, -0.0828682_f4, -0.122583_f4, 0.199745_f4,  &
     -0.209504_f4, -0.124834_f4, -0.110465_f4, -0.114683_f4, -0.113324_f4,  &
     -0.107670_f4, 0.272397_f4, 0.399377_f4, 0.0210035_f4, 0.0461162_f4,  &
     0.00840926_f4, -0.147839_f4, -0.129811_f4, -0.175722_f4, 0.304604_f4,  &
     -0.0951550_f4, 0.0653323_f4, 0.194438_f4, -0.0316031_f4, -0.109061_f4,  &
     -0.276948_f4, -0.00198612_f4, -0.0130749_f4, -0.0701235_f4,  &
     0.0366394_f4, 0.135202_f4, 0.0980943_f4, 0.0659373_f4, -0.0162412_f4 /)

real(f4), dimension(vis_ncf) :: vis_NPcoeff_TESY1 = (/ 5.05356_f4,  &
     -4.29222_f4, -3.30846_f4, -1.80604_f4, -1.16437_f4, 0.246219_f4,  &
     1.06446_f4, -0.774537_f4, 0.150922_f4, -0.135457_f4, -0.164423_f4 /)

real(f4), dimension(c_ncf) :: c_NP_TESY1 = (/ 58.3897_f4, 0.243563_f4 /)

!>>>>>>>>

real(f4), dimension(IR_ncf) :: IR_NPcoeff_TESY2 = (/ 70.811824_f4,  &
     -11.322888_f4, 19.073221_f4, -1.1750241_f4, -0.52092031_f4,  &
     1.3712435_f4, -4.1939773_f4, -0.75605192_f4, 0.60528853_f4,  &
     0.41391396_f4, 0.00027637038_f4, -0.14771808_f4, 0.32617855_f4,  &
     0.052349274_f4, 0.51238023_f4, 0.084538623_f4, 0.048988179_f4,  &
     -0.21770713_f4, -0.36854501_f4, -0.020694434_f4, -0.10936085_f4,  &
     -0.028692843_f4, -0.33863015_f4, -0.012854056_f4, -0.044757179_f4,  &
     -0.12805081_f4, -0.17579913_f4, -0.12815096_f4, 0.060011661_f4,  &
     0.27696709_f4, 0.031415002_f4, 0.15946522_f4, -0.010809866_f4,  &
     -0.046817164_f4, 0.10329803_f4, -0.12109210_f4, 0.15248830_f4,  &
     -0.044969476_f4, 0.072034690_f4, 0.083727224_f4, -0.085026144_f4,  &
     -0.013130822_f4, -0.081219906_f4, -0.070679569_f4, -0.053422027_f4,  &
     -0.017686033_f4, 0.027528034_f4, 0.097165793_f4, -0.019993762_f4,  &
     0.070942103_f4, 0.013876701_f4 /)

real(f4), dimension(vis_ncf) :: vis_NPcoeff_TESY2 = (/ 6.2956434_f4,  &
     -5.2049832_f4, -4.2829944_f4, -1.5004717_f4, -0.57648525_f4,  &
     0.19135480_f4, 0.44920057_f4, -0.66633841_f4, -0.027930542_f4,  &
     -0.17585616_f4, 0.0053278554_f4 /)

real(f4), dimension(c_ncf) :: c_NP_TESY2 = (/ 58.016699_f4, 0.24736803_f4 /)

!>>>>>>>>

real(f4), dimension(IR_ncf) :: IR_NPcoeff_TESY3 = (/ 70.9108_f4, -11.3940_f4,  &
     18.5213_f4, -1.06301_f4, -0.713416_f4, 1.36164_f4, -3.83795_f4,  &
     -0.781803_f4, 0.689798_f4, -0.0656154_f4, 0.498944_f4, -0.274294_f4,  &
     0.240560_f4, 0.314759_f4, -0.366134_f4, 0.149635_f4, 0.109752_f4,  &
     -0.239675_f4, 0.167794_f4, -0.0552464_f4, -0.0721578_f4, 0.136903_f4,  &
     -0.119583_f4, -0.184120_f4, -0.0225941_f4, -0.0831832_f4, -0.159195_f4,  &
     -0.303261_f4, 0.139970_f4, 0.333742_f4, 0.0557606_f4, 0.129828_f4,  &
     0.0590731_f4, -0.0174061_f4, -0.127668_f4, -0.0878801_f4, 0.260152_f4,  &
     -0.134625_f4, -0.0189369_f4, 0.150733_f4, 0.0793099_f4, -0.0232880_f4,  &
     -0.209568_f4, 0.0496815_f4, -0.0582192_f4, -0.0557934_f4, 0.0833890_f4,  &
     0.101041_f4, 0.0601398_f4, 0.0492828_f4, 0.0317827_f4 /)

real(f4), dimension(vis_ncf) :: vis_NPcoeff_TESY3 = (/ 7.02581_f4,  &
     -5.80642_f4, -4.74344_f4, -0.801428_f4, 0.155751_f4, 0.224038_f4,  &
     -0.250260_f4, -0.655322_f4, 0.134421_f4, -0.0995696_f4, -0.167452_f4 /)

real(f4), dimension(c_ncf) :: c_NP_TESY3 = (/ 58.1478_f4, 0.243209_f4 /)

!>>>>>>>>

!=== Executable statements:

Ls_tmp(:) = Ls

! Convert longitudes to East longitude:
do i=1,nlon
   if (tgt_lon(i) < ZERO_f4) tgt_lon(i) = tgt_lon(i)+360._f4
end do

if (cap_loc(1:1) == 'N') then

   if (cap_year == 1) then
      IR_coeff = IR_NPcoeff_TESY1
      vis_coeff = vis_NPcoeff_TESY1
      c = c_NP_TESY1
   else if (cap_year == 2) then
      IR_coeff = IR_NPcoeff_TESY2
      vis_coeff = vis_NPcoeff_TESY2
      c = c_NP_TESY2
   else if (cap_year == 3) then
      IR_coeff = IR_NPcoeff_TESY3
      vis_coeff = vis_NPcoeff_TESY3
      c = c_NP_TESY3
   end if

   if (cap_type(1:2) == 'IR') then  ! "CO2 ice" cap

      call fft2dcoe2z(nlon, Ls_tmp, tgt_lon, IR_ncf, IR_coeff, 5, 3, cap_lat)

   else if (cap_type(1:3) == 'vis') then   ! "H2O ice" cap

      call fft2dcoe2z(nlon, Ls_tmp, tgt_lon, vis_ncf, vis_coeff, 1, 3, cap_lat)
      cap_lat = cap_lat+(c(1)+c(2)*Ls_tmp(:))

      !@   wvob=where(a gt 80 or a lt 60)
      !@   if wvob[0] ne -1 then a[wvob]=!values.f_nan
      !@   endif

   end if

   cap_lat = min(cap_lat, 90._f4)

else if (cap_loc(1:1) == 'S') then

   continue

end if

end subroutine get_TES_cap_lat


!```````````````````````````````````````````````````````````````````
subroutine fft2dcoe2z(nx_ny, x, y, ncoeff, coeff, n0, n1, zz)

!+ Return the value of f(x,y) given Fourier coefficients

use univ_kind_defs_mod, only: f4
use univ_convert_units_mod, only: deg_to_rad_f4 

implicit none

!=== Argument declarations:

integer, intent(IN) :: nx_ny, ncoeff, n0, n1
real(f4), dimension(ncoeff), intent(IN) :: coeff
real(f4), dimension(nx_ny), intent(IN) :: x, y

real(f4), dimension(nx_ny), intent(INOUT) :: zz

!== Local declarations:

integer :: i, j, c_cnt
real(f4), dimension(nx_ny,n0*4+n0*n1*2) :: array
real(f4), dimension(nx_ny) :: xr, yr

!=== Executable statements:

! Purpose: To return the value of f(x,y) given coef from fft2dx.
!
! Inputs:
!   x: the abscissa for the series to be fit (in degrees).

xr = x*deg_to_rad_f4
yr = y*deg_to_rad_f4

c_cnt = 0

do i=1,n0
   array(:,c_cnt+1) = cos(real(i, kind=f4)*xr(:))
   array(:,c_cnt+2) = sin(real(i, kind=f4)*xr(:))
   array(:,c_cnt+3) = cos(real(i, kind=f4)*yr(:))
   array(:,c_cnt+4) = sin(real(i, kind=f4)*yr(:))
   
   c_cnt = c_cnt+4
end do

do i=1,n0
   do j=1,n1
      array(:,c_cnt+1) = cos(real(i, kind=f4)*xr(:)+real(j, kind=f4)*yr(:))
      array(:,c_cnt+2) = sin(real(i, kind=f4)*xr(:)+real(j, kind=f4)*yr(:))
      
      c_cnt = c_cnt+2
   end do
end do

zz = coeff(1)+matmul(array,coeff(2:ncoeff))

end subroutine fft2dcoe2z


!```````````````````````````````````````````````````````````````````
subroutine sfcinit_kludge(n1, n2, n3, n4, n5, n6, igd, vnam, glat, glon,  &
     TI_static, albd_static, topt, TI_flag, albd_flag, topt_flag)

!+ Kludge(s) for sfc characteristics

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL
use univ_math_constants_mod, only: twopi_f4, ZERO_f4
use main_memory_mod, only: initialization_type, gd
use body_specific_constants_mod, only: dist_deglat

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, n4, n5, n6, igd, TI_flag, albd_flag,  &
     topt_flag
character(len=*), intent(IN) :: vnam
real(f4), dimension(n2,n3), intent(IN) :: glat, glon, topt

real(f4), dimension(n2,n3,n5), intent(INOUT) :: TI_static, albd_static

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'sfcinit_kludge'

integer :: i, j, k, ip, TI_src_type, cnt_min, albd_src_type
real(f4) :: max_glat, min_glat, fgrad, fmax
real(f4), dimension(3) :: ftmp, dlat, fmax_a
integer, dimension(3) :: cnt, cnt_min_a
logical :: must_kludge_TI, must_kludge_albd

real(f4) :: albd_NP_klat, TI_NP_klat, albd_SP_klat, TI_SP_klat

!=== Executable statements:

if (vnam == 'FILEDATA') then

   if (initialization_type == 2) then

      TI_src_type = 0; TI_NP_klat = 91._f4; TI_SP_klat = -91._f4

      i = len_trim(gd(igd)%itinfn)
      if (i > 19) then
         if (gd(igd)%itinfn(i-19:i) == 'TI-global-daytime.nc') then
            TI_src_type = 1     ! Not tested
            TI_NP_klat = 86.5_f4   ! [deg N] latitude
            TI_SP_klat = -86.5_f4   ! [deg N] latitude 
         end if
      end if
      if (i > 21) then
         if (gd(igd)%itinfn(i-21:i) == 'TI-global-nighttime.nc') then
            TI_src_type = 2
            TI_NP_klat = 86.5_f4   ! [deg N] latitude
            TI_SP_klat = -86.5_f4   ! [deg N] latitude 
         end if
      end if

      albd_src_type = 0; albd_NP_klat = 91._f4; albd_SP_klat = -91._f4

      i = len_trim(gd(igd)%ialbfn)
      if (i > 23) then
         if (gd(igd)%ialbfn(i-23:i) == 'albedo-global-1_8_deg.nc') then
            albd_src_type = 1
            albd_NP_klat = 86.5_f4  ! [deg N] latitude 
            albd_SP_klat = -86.5_f4  !
         end if
      end if
      if (i > 20) then
         if (gd(igd)%ialbfn(i-20:i-5) == 'albedo-global-MY') then
            albd_src_type = 2    ! Not tested
            albd_NP_klat = 86.5_f4  ! [deg N] latitude 
            albd_SP_klat = -86.5_f4  !
         end if
      end if

      do ip=1,n5  ! Loop over all patches

   !^^^^^^^^^^^^^^^^ North polar kludges ^^^^^^^^^^^^^^^^:

         must_kludge_TI = .FALSE.    ! Assume that kludging is not necessary
         must_kludge_albd = .FALSE.  !

         max_glat = maxval(glat)   ! Maximum latitude on grid
         if (max_glat > TI_NP_klat ) must_kludge_TI = ( TI_flag /= 0 )
         if (max_glat > albd_NP_klat) must_kludge_albd = ( albd_flag /= 0 )

         !------ Albedo must be kludged first ------:

         if (must_kludge_albd) then

            if (albd_src_type == 1 .or. albd_src_type == 2) then

               cnt_min = int(dist_deglat/gd(igd)%deltax*twopi_f4*  &
                    (90._f4-albd_NP_klat))

               dlat(1) = 0.1_f4
               do
                  cnt(1) = 0
                  fmax = ZERO_f4
                  do j=1,n3
                     do i=1,n2
                        if (glat(i,j) <= albd_NP_klat .and.  &  ! lat_range ring
                             (albd_NP_klat-glat(i,j)) <= dlat(1)) then 
                           fmax = max(fmax, albd_static(i,j,ip))
                           cnt(1) = cnt(1)+1
                        end if
                     end do
                  end do
               
                  if (cnt(1) >= cnt_min) then
                     exit
                  else if (dlat(1) > 360._f4) then
                     call exception(procedure_name, FATAL,  &
                          'NP albedo kludge failed -- try interpolating albedo from parent grid?')
                  end if
                  dlat(1) = dlat(1)+0.1_f4
               end do
            
               ftmp(1) = 0.98_f4*fmax
               do j=1,n3
                  do i=1,n2
                     if (glat(i,j) > albd_NP_klat) albd_static(i,j,ip) = ftmp(1)
                  end do
               end do

            end if

         end if

         !------ Now kludge thermal inertia [f(albd_static)] ------:

         if (must_kludge_TI) then

            if (TI_src_type == 1 .or. TI_src_type == 2) then
               cnt_min = int(dist_deglat/gd(igd)%deltax*twopi_f4*  &
                    (90._f4-TI_NP_klat))

               dlat(1) = 0.1_f4
               do
                  cnt(1) = 0
                  fmax = ZERO_f4
                  do j=1,n3
                     do i=1,n2
                        if (glat(i,j) <= TI_NP_klat .and.  & ! In lat_range ring
                             (TI_NP_klat-glat(i,j)) <= dlat(1)) then 
                           fmax = max(fmax, TI_static(i,j,ip))
                           cnt(1) = cnt(1)+1
                        end if
                     end do
                  end do
               
                  if (cnt(1) >= cnt_min) then
                     exit
                  else if (dlat(1) > 360._f4) then
                     call exception(procedure_name, FATAL,  &
                          'NP TI kludge failed -- try interpolating TI from parent grid?')
                  end if
                  dlat(1) = dlat(1)+0.1_f4
               end do
            
               ftmp(1) = 0.98_f4*fmax
               do j=1,n3
                  do i=1,n2
                     if (glat(i,j) > TI_NP_klat) TI_static(i,j,ip) = ftmp(1)
                  end do
               end do
!#               do j=1,n3
!#                  do i=1,n2
!#                     if (glat(i,j) > TI_NP_klat) then
!#                        TI_static(i,j,ip) = 3000._f4*albd_static(i,j,ip) 
!#                     end if
!#                  end do
!#               end do
            end if

         end if

   !^^^^^^^^^^^^^^^^ South polar kludges ^^^^^^^^^^^^^^^^:

         must_kludge_TI = .FALSE.    ! Assume that kludging is not necessary
         must_kludge_albd = .FALSE.  !

         min_glat = minval(glat)   ! Minimum latitude on grid
         if (min_glat < TI_SP_klat ) then
            if (TI_flag /= 0) must_kludge_TI = .TRUE.
         end if
         if (min_glat < albd_SP_klat) then
            if (albd_flag /= 0) must_kludge_albd = .TRUE.
         end if

         !------ Albedo must be kludged first ------:

         if (must_kludge_albd) then

            if (albd_src_type == 1 .or. albd_src_type == 2) then

               cnt_min = int(dist_deglat/gd(igd)%deltax*twopi_f4*  &
                    (90._f4+albd_SP_klat))
               cnt_min_a(1) = max(1,  &
                    int(real(cnt_min, kind=f4)*(147._f4/360._f4)))
               cnt_min_a(2) = max(1,  &
                    int(real(cnt_min, kind=f4)*(68._f4/360._f4)))
               cnt_min_a(3) = max(1,  &
                    int(real(cnt_min, kind=f4)*(125._f4/360._f4)))

               dlat = 0.1_f4
               do
                  cnt = 0
                  ftmp = ZERO_f4
                  do j=1,n3
                     do i=1,n2
                        if (glat(i,j) >= albd_SP_klat) then 
                           if (glon(i,j) >= -107._f4 .and.  &
                                glon(i,j) <= 40._f4 .and.  &
                                abs(glat(i,j)-albd_SP_klat) <= dlat(1)) then 
                              ftmp(1) = max(ftmp(1),albd_static(i,j,ip))
                              cnt(1) = cnt(1)+1    ! Residual cap
                           else if (glon(i,j) >= -175._f4 .and.  &
                                glon(i,j) <= -117._f4 .and.  &
                                abs(glat(i,j)-albd_SP_klat) <= dlat(2)) then 
                              ftmp(2) = ftmp(2)+albd_static(i,j,ip)
                              cnt(2) = cnt(2)+1    ! Darker plain
                           else if (glon(i,j) >= 50._f4 .and.  &
                                glon(i,j) <= 175._f4 .and.  &
                                abs(glat(i,j)-albd_SP_klat) <= dlat(3)) then 
                              ftmp(3) = ftmp(3)+albd_static(i,j,ip)
                              cnt(3) = cnt(3)+1    ! Brighter plain
                           end if
                        end if
                     end do
                  end do
                  
                  i = 3
                  do k=1,3
                     if (cnt(k) < cnt_min_a(k)) then
                        dlat(k) = dlat(k)+0.1_f4
                        i = i-1
                     end if
                  end do

                  if (i == 3) then
                     exit
                  else if (maxval(dlat) > 360._f4) then
                     call exception(procedure_name, FATAL,  &
                          'SP albedo kludge failed -- try interpolating albedo from parent grid?')
                  end if
               end do

               ftmp(2) = ftmp(2)/real(cnt(2), kind=f4)
               ftmp(3) = ftmp(3)/real(cnt(3), kind=f4)             

               do j=1,n3
                  do i=1,n2
                     if (glat(i,j) < albd_SP_klat) then
                        if (glon(i,j) > -107._f4 .and. glon(i,j) < 40._f4) then 
                           albd_static(i,j,ip) = ftmp(1)
                        else if (glon(i,j) >= -175._f4 .and.  &
                             glon(i,j) < -117._f4) then 
                           albd_static(i,j,ip) = ftmp(2)
                        else if (glon(i,j) > 50._f4 .and.  &
                             glon(i,j) <= 175._f4) then 
                           albd_static(i,j,ip) = ftmp(3)
                        else if (glon(i,j) >= -117._f4 .and.  &
                             glon(i,j) <= -107._f4) then
                           fgrad = (ftmp(1)-ftmp(2))/10._f4
                           albd_static(i,j,ip) = ftmp(2)+  &
                                fgrad*(glon(i,j)+117._f4)
                        else if (glon(i,j) >= 40._f4 .and.  &
                             glon(i,j) <= 50._f4) then
                           fgrad = (ftmp(3)-ftmp(1))/10._f4
                           albd_static(i,j,ip) = ftmp(3)+  &
                                fgrad*(glon(i,j)-50._f4)
                        else if (glon(i,j) <= -175._f4) then
                           fgrad = (ftmp(2)-ftmp(3))/10._f4
                           albd_static(i,j,ip) = ftmp(2)+  &
                                fgrad*(glon(i,j)+175._f4)
                        else if (glon(i,j) >= 175._f4) then
                           fgrad = (ftmp(2)-ftmp(3))/10._f4
                           albd_static(i,j,ip) = 0.5_f4*(ftmp(3)+ftmp(2))+  &
                                fgrad*(glon(i,j)-180._f4)
                        end if
                     end if
                  end do
               end do

            end if

         end if

         !------ Now kludge thermal inertia [f(albd_static)] ------:

         if (must_kludge_TI) then

            if (TI_src_type == 1 .or. TI_src_type == 2) then
               cnt_min = int(dist_deglat/gd(igd)%deltax*twopi_f4*  &
                    (90._f4+TI_SP_klat))
               cnt_min_a(1) = max(1,  &
                    int(real(cnt_min, kind=f4)*(147._f4/360._f4)))
               cnt_min_a(2) = max(1,  &
                    int(real(cnt_min, kind=f4)*(68._f4/360._f4)))
               cnt_min_a(3) = max(1,  &
                    int(real(cnt_min, kind=f4)*(125._f4/360._f4)))

               dlat = 0.1_f4
               do
                  cnt = 0
                  ftmp = ZERO_f4
                  fmax_a = ZERO_f4
                  do j=1,n3
                     do i=1,n2
                        if (glat(i,j) >= TI_SP_klat) then
                           if (glon(i,j) >= -107._f4 .and.  &
                                glon(i,j) <= 40._f4 .and.  &
                                abs(glat(i,j)-TI_SP_klat) <= dlat(1)) then 
                              ftmp(1) = ftmp(1)+TI_static(i,j,ip)
                              fmax_a(1) = max(fmax_a(1), TI_static(i,j,ip))
                              cnt(1) = cnt(1)+1    ! Residual cap
                           else if (glon(i,j) >= -175._f4 .and.  &
                                glon(i,j) <= -117._f4 .and.  &
                                abs(glat(i,j)-TI_SP_klat) <= dlat(2)) then 
                              ftmp(2) = ftmp(2)+TI_static(i,j,ip)
                              fmax_a(2) = max(fmax_a(2),TI_static(i,j,ip))
                              cnt(2) = cnt(2)+1    ! Darker plain
                           else if (glon(i,j) >= 50._f4 .and.  &
                                glon(i,j) <= 175._f4 .and.  &
                                abs(glat(i,j)-TI_SP_klat) <= dlat(3)) then 
                              ftmp(3) = ftmp(3)+TI_static(i,j,ip)
                              fmax_a(3) = max(fmax_a(3),TI_static(i,j,ip))
                              cnt(3) = cnt(3)+1    ! Brighter plain
                           end if
                        end if
                     end do
                  end do
                 
                  i = 3
                  do k=1,3
                     if (cnt(k) < cnt_min_a(k)) then
                        dlat(k) = dlat(k)+0.1_f4
                        i = i-1
                     end if
                  end do

                  if (i == 3) then
                     exit
                  else if (maxval(dlat) > 360._f4) then
                     call exception(procedure_name, FATAL,  &
                          'SP TI kludge failed -- try interpolating TI from parent grid?')
                  end if
               end do

               ftmp(:) = ftmp(:)/real(cnt(:), kind=f4)
               ftmp(:) = ftmp(:)*1.2_f4
!               ftmp(1) = ftmp(1)+0.2*(fmax_a(1)-ftmp(1))
!               ftmp(2) = ftmp(2)+0.5*(fmax_a(2)-ftmp(2))
!               ftmp(3) = ftmp(3)+0.5*(fmax_a(3)-ftmp(3))

               do j=1,n3
                  do i=1,n2
                     if (glat(i,j) < TI_SP_klat) then
                        if (glon(i,j) > -107._f4 .and. glon(i,j) < 40._f4) then 
                           TI_static(i,j,ip) = ftmp(1)
                        else if (glon(i,j) >= -175._f4 .and.  &
                             glon(i,j) < -117._f4) then 
                           TI_static(i,j,ip) = ftmp(2)
                        else if (glon(i,j) > 50._f4 .and.  &
                             glon(i,j) <= 175._f4) then 
                           TI_static(i,j,ip) = ftmp(3)
                        else if (glon(i,j) >= -117._f4 .and.  &
                             glon(i,j) <= -107._f4) then
                           fgrad = (ftmp(1)-ftmp(2))/10._f4
                           TI_static(i,j,ip) = ftmp(2)+fgrad*(glon(i,j)+117._f4)
                        else if (glon(i,j) >= 40._f4 .and.  &
                             glon(i,j) <= 50._f4) then
                           fgrad = (ftmp(3)-ftmp(1))/10._f4
                           TI_static(i,j,ip) = ftmp(3)+fgrad*(glon(i,j)-50._f4)
                        else if (glon(i,j) <= -175._f4) then
                           fgrad = (ftmp(2)-ftmp(3))/10._f4
                           TI_static(i,j,ip) = ftmp(2)+fgrad*(glon(i,j)+175._f4)
                        else if (glon(i,j) >= 175._f4) then
                           fgrad = (ftmp(2)-ftmp(3))/10._f4
                           TI_static(i,j,ip) = 0.5_f4*(ftmp(3)+ftmp(2))+  &
                                fgrad*(glon(i,j)-180._f4)
                        end if
                     end if
                  end do
               end do
!#               do j=1,n3
!#                  do i=1,n2
!#                     if (glat(i,j) < TI_SP_klat) then 
!#                        if (albd_static(i,j,ip) >= 0.28_f4) then
!#                           TI_static(i,j,ip) =  &
!#                                albd_static(i,j,ip)*albd_static(i,j,ip)*  &
!#                                (albd_static(i,j,ip)-0.28_f4)*35000._f4+400._f4
!#                        else if (albd_static(i,j,ip) >= 0.215_f4) then
!#                           TI_static(i,j,ip) =  &
!#                                albd_static(i,j,ip)*600._f4
!#                        else
!#                           TI_static(i,j,ip) =  &
!#                                (0.19_f4/albd_static(i,j,ip))**2*140._f4
!#                        end if
!#                     end if
!#                  end do
!#               end do
            end if

         end if

   !^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

      end do

   end if
 
else if (vnam == 'NOFILE') then

   if (igd == 1) then

      !***** WHATEVER ******

   end if

end if

end subroutine sfcinit_kludge


!```````````````````````````````````````````````````````````````````
subroutine sfcinit_aerrs(n1, n2, n3, n4, n5, n6, ng, naerscl, glat, glon,  &
     TI_static, albd_static, albedt, aer_rsv, sbps_lyrthkn)

!+ Initialize surface reservoir

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use aermic_rad_mem_mod, only: iv_is_dust

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, n4, n5, n6, ng, naerscl
real(f4), dimension(n2,n3), intent(IN) :: glat, glon, albedt
real(f4), dimension(n2,n3,n5), intent(IN) :: TI_static, albd_static
real(f4), dimension(n4,n2,n3,n5), intent(IN) :: sbps_lyrthkn

real(f4), dimension(n2,n3,naerscl), intent(INOUT) :: aer_rsv

!== Local declarations:

integer :: i, j, iv, ip

real(f4), parameter :: noliftonice_NP_lat = 70._f4,  &  ! 
                       noliftonice_SP_lat = -70._f4    ! [deg]

!=== Executable statements:

if (naerscl == 0) return

ip = 1

do iv=1,naerscl
   if (iv_is_dust(iv)) then   ! This is a dust aerosol scalar
      do j=1,n3
         do i=1,n2
            if (sbps_lyrthkn(n6+1,i,j,ip) > ZERO_f4 .or.  &
                 ( glat(i,j) > noliftonice_NP_lat .and.  &
                 albedt(i,j) >= 0.25_f4 )) then
               aer_rsv(i,j,iv) = ZERO_f4
            else
               aer_rsv(i,j,iv) = 1.e6_f4
            end if
         end do
      end do
   end if
end do

end subroutine sfcinit_aerrs


end module surface_init_mod


!```````````````````````````````````````````````````````````````````
subroutine sfcinit(n1, n2, n3, n4, n5, n6, igd, vnam, patch_fcov,  &
     sbps_thrmcap, sbps_density, ti_static, albedo_static, sbps_thrmcnd,  &
     sbps_lyrthkn, sbps_q, albedt, sfc_tempK, z0net, z0stat, topz0, theta,  &
     pi0, pp, glat, glon, topt)

!+ Initialize some surface quantities (for horizontally homogeneous runs,
!   sfc variables without data files, etc.)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use univ_numtype_constants_mod, only: tiny_f4
use univ_std_fileIO_mod, only: find_file_IOunit, determine_IO_reclen
use main_memory_mod, only: npatch, initialization_type, albedo, zrough,  &
     TI_val, SBPS_Tlev, varfile_name, n_varf, a, gd, undef_value_f4, emissivity
use main_memory_bspec_mod, only: pctlcon, slz, stgoff, CO2_sfci, sfci_iLs,  &
     dust_tog, H2Oi_tog, CO2i_tog
use body_specific_constants_mod, only: cp, cpi, p00, cpor, s_rho, s_hc,  &
     rho_CO2ice, emiss_CO2ice
use vtables_mod, only: iaerrs, naersclr
use surface_init_mod, only: calc_sbps_tlev, get_tes_cap_lat, sfcinit_aerrs
use PRAMS_SBPS_mod, only: tsat_co2ice, cp_co2ice, albedo_co2ice
use grid_mem_mod, only: switch_to_Ascalar

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: vnam
integer, intent(IN) :: n1, n2, n3, n4, n5, n6, igd
real(f4), dimension(n1,n2,n3), intent(IN) :: theta, pi0, pp
real(f4), dimension(n2,n3), intent(IN) :: topz0, glat, glon, topt

real(f4), dimension(n2,n3), intent(INOUT) :: albedt
real(f4), dimension(n4,n2,n3,n5), intent(INOUT) :: sbps_thrmcap,  &
     sbps_density, sbps_thrmcnd, sbps_lyrthkn, sbps_q
real(f4), dimension(n2,n3,n5), intent(INOUT) :: patch_fcov, z0net, z0stat,  &
     sfc_tempK
real(f4), dimension(n2,n3,n5), intent(INOUT) :: ti_static, albedo_static

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'sfcinit'

integer :: ip, e, i, j, iq, nc, IOunit, IO_reclen
character(len=40) :: cgrid
character(len=MAX_FILE_PATH_LEN) :: flnm
real(f4), allocatable, dimension(:,:,:) :: gt
real(f4), allocatable, dimension(:,:) :: gi
real(f4) :: c1, airtempK, pi_sfc, pr_sfc, Tsat, ftmp, tgt_lat

real(f4) :: CO2ice_depth, z0_here
integer, parameter :: ncaplb = 360
real(f4), dimension(ncaplb) :: cap_lon
real(f4), dimension(0:ncaplb+1) :: cap_NP_H2Oi_lat, cap_NP_CO2i_lat,  &
     cap_SP_H2Oi_lat, cap_SP_CO2i_lat
real(f4), allocatable, dimension(:,:) :: reclen_test
logical :: CO2ice_on_ground, H2Oice_on_ground

!=== Executable statements:

! Some of these values may be overridden by:
!
!  * Interpolation from coarser grids
!  * Reading data from standard PRAMS data files
!
! All of these values may be overridden by:
!
!  * Specifying new values in `sfcinit_user` (in PRAMS_user.f90) 

if (vnam == 'FILEDATA') then

   patch_fcov(:,:,1) = pctlcon   ! Percentage of patch #1 coverage
   
   do ip=2,npatch 
      patch_fcov(:,:,ip) = pctlcon   ! Percent coverage of other patches
   end do

   ! Initialize SBPS quantities (FOR ALL CASES):

   do ip=1,npatch

      ! Thermal inertia:
      ti_static(:,:,ip) = TI_val   ! [SI units] Initial thermal
                                   !  inertia value

      ! Static surface albedo:
      albedo_static(:,:,ip) = albedo

      ! Static surface emissivity:
      gd(igd)%emissivity_static(:,:,ip) = emissivity

      ! Fill subsurface:
      sbps_thrmcap(1:n6,:,:,ip) = s_hc    ! SBPS thermal (heat) capacity
      sbps_density(1:n6,:,:,ip) = s_rho   ! SBPS density
           ! Thermal conductivity:
      do j=1,n3
         do i=1,n2
            do iq=1,n6
               sbps_thrmcnd(iq,i,j,ip) =  &
                    ti_static(i,j,ip)*ti_static(i,j,ip)/  &
                    (sbps_density(iq,i,j,ip)*sbps_thrmcap(iq,i,j,ip))
            end do
         end do
      end do

      ! Fill supersurface:
      sbps_thrmcap(n6+1:n4,:,:,ip) = undef_value_f4 ! SBPS thermal/heat capacity
      sbps_density(n6+1:n4,:,:,ip) = undef_value_f4   ! SBPS density
      sbps_thrmcnd(n6+1:n4,:,:,ip) = undef_value_f4   ! Thermal conductivity
         
   end do
 
else if (vnam == 'NOFILE') then

   ! Surface roughness (z0):

   select case(gd(igd)%iz0flg)
   case(0)
      do j=1,n3
         do i=1,n2
            z0stat(i,j,1) = zrough
            z0net(i,j,1) = z0stat(i,j,1)
         end do
      end do
   case(1)
      do j=1,n3
         do i=1,n2
            z0stat(i,j,1) = topz0(i,j)
            z0net(i,j,1) = z0stat(i,j,1)
         end do
      end do
   case(2)     ! Set z0 at/below/above a topo level
      z0stat(:,:,1) = zrough   ! Default value
      z0_here = abs(gd(igd)%z0_of_topolevel)    ! Prevent a negative z0 value
      if (gd(igd)%z0_of_topolevel < -tiny_f4) then  ! At/below given topo level
         where (topt <= gd(igd)%topolevel_for_z0) z0stat(:,:,1) = z0_here
      else                         ! At/above given topo level
         where (topt >= gd(igd)%topolevel_for_z0) z0stat(:,:,1) = z0_here
      end if
      z0net(:,:,1) = z0stat(:,:,1)
   end select

   !*** Initialize subsurface/supersurface temperature profiles, etc.:

   ! Compute subsurface tempK LEVEL depths (Kq):

   call calc_SBPS_Tlev(n6)

   ! Compute subsurface layer thickness:
   
   do j=1,n3
      do i=1,n2
         do iq=2,n6   
            SBPS_lyrthkn(iq,i,j,1) = slz(iq)-slz(iq-1)
         end do
         SBPS_lyrthkn(1,i,j,1) = 2._f4*(slz(1)-SBPS_Tlev(1))  ! Lowest layer dz
      end do
   end do

   if (initialization_type == 1) then    ! ONLY if INITIAL=1

      c1 = 0.5_f4/cp
      do j=1,n3
         do i=1,n2
            albedt(i,j) = albedo_static(i,j,1)  ! Init time-variable albedo
            gd(igd)%emisst(i,j) = gd(igd)%emissivity_static(i,j,1)

            ! Subsurface values:

            airtempK = theta(2,i,j)*c1*(pi0(1,i,j)+pi0(2,i,j)  &
                 +pp(1,i,j)+pp(2,i,j))
            
            do iq=1,n6
               sbps_Q(iq,i,j,1) = airtempK+stgoff(iq)    ! [K] (temporarily)
            end do

            ! Supersurface values:

            SBPS_lyrthkn(n6+1,i,j,1) = ZERO_f4       ! No CO2 ice
            SBPS_Q(n6+1,i,j,1) = ZERO_f4             !

            ! Init sfc temp for routines:

            sfc_tempK(i,j,1) = sbps_Q(n6,i,j,1)

         end do
      end do

   else if (initialization_type == 2) then ! Only initialize this way for variable init

      allocate(gt(gd(igd)%nxp,gd(igd)%nyp,n6), gi(gd(igd)%nxp,gd(igd)%nyp),  &
           stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'sfcinit')

      write (cgrid, '(a2,i2.2,a4)') '-g', igd, '.tfm' ! Create grid-specific suffix

      nc = len_trim(varfile_name(n_varf))-4       ! Make tempK varfile name from
      flnm = varfile_name(n_varf)(1:nc)//cgrid  !  the name of the first varfile

      IOunit = find_file_IOunit()

      allocate(reclen_test(gd(igd)%nxp,gd(igd)%nyp), stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'reclen')
      IO_reclen = determine_IO_reclen(reclen_test)
      deallocate(reclen_test, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
           'reclen')

      open(IOunit, file=flnm(1:len_trim(flnm)), form='unformatted',  &
           status='old', access='direct', recl=IO_reclen) 
      do i=1,n6
         read (IOunit, rec=i) gt(:,:,i)   ! Sfc tempK
!         call binDA_rw2Df(2,i,nnxp(ng),nnyp(ng),gt(:,:,i),'R')   ! Sfc tempK
      end do
      read (IOunit, rec=n6+1) gi(:,:)
!      call binDA_rw2Df(2,n6+1,nnxp(ng),nnyp(ng),gi(:,:),'R')  ! Ice mass
      close(IOunit)   ! Close varfile

      !*** Status report to user

      write (*, '(/,a,/)') 'Read initial subsurface temperature profile dataset.'

      if (CO2_sfci == 3) then

         ! Create longitude interpolation table: 
         do i=1,ncaplb 
            cap_lon(i) = -179.5_f4+real(i-1, kind=f4)
         end do
         
         if (maxval(glat) > ZERO_f4) then ! Grid includes a northern hemisphere area

            ! Compute boundary latitudes for both type of cap:
            call get_TES_cap_lat(ncaplb, cap_lon, sfci_iLs,  &
                 cap_NP_CO2i_lat(1:ncaplb), 'IR ', 'N', 1)
            cap_NP_CO2i_lat(0) = cap_NP_CO2i_lat(ncaplb)    ! Wrap edges
            cap_NP_CO2i_lat(ncaplb+1) = cap_NP_CO2i_lat(1)  !
            call get_TES_cap_lat(ncaplb, cap_lon, sfci_iLs,  &
                 cap_NP_H2Oi_lat(1:ncaplb), 'vis', 'N', 1)
            cap_NP_H2Oi_lat(0) = cap_NP_H2Oi_lat(ncaplb)    ! Wrap edges
            cap_NP_H2Oi_lat(ncaplb+1) = cap_NP_H2Oi_lat(1)  !
         
         end if

         if (minval(glat) < ZERO_f4) then ! Grid includes a southern hemisphere area

            ! Compute boundary latitudes for both type of cap:
!%            call get_TES_cap_lat(ncaplb,cap_lon,sfci_iLs  &
!%                 ,cap_SP_CO2i_lat(1:ncaplb),'IR','S',1)
!%            cap_SP_CO2i_lat(0) = cap_SP_CO2i_lat(ncaplb)    ! Wrap edges
!%            cap_SP_CO2i_lat(ncaplb+1) = cap_SP_CO2i_lat(1)  !
!%            call get_TES_cap_lat(ncaplb,cap_lon,sfci_iLs  &
!%                 ,cap_SP_H2Oi_lat(1:ncaplb),'vis','S',1)
!%            cap_SP_H2Oi_lat(0) = cap_SP_H2Oi_lat(ncaplb)    ! Wrap edges
!%            cap_SP_H2Oi_lat(ncaplb+1) = cap_SP_H2Oi_lat(1)  !
            cap_SP_CO2i_lat = -90._f4
            cap_SP_H2Oi_lat = -90._f4
         
         end if

      end if

      do j=1,n3
         do i=1,n2

            ! Initialize SPS and topmost SBS layers:

            if (CO2_sfci == 3) then

               ftmp = glon(i,j)+180.5_f4
               iq = int(ftmp)
               ftmp = ftmp-real(iq, kind=f4)

               if (glat(i,j) > ZERO_f4) then   ! Northern hemisphere
                  tgt_lat = cap_NP_CO2i_lat(iq)*(1.-ftmp)+  &
                       cap_NP_CO2i_lat(iq+1)*ftmp 
                  CO2ice_on_ground = ( glat(i,j) >= tgt_lat )
                  CO2ice_depth = 0.2_f4  ! [m]
                  tgt_lat = cap_NP_H2Oi_lat(iq)*(1.-ftmp)+  &
                       cap_NP_H2Oi_lat(iq+1)*ftmp 
                  H2Oice_on_ground = ( glat(i,j) >= tgt_lat )
               else   ! Southern hemisphere
                  tgt_lat = cap_SP_CO2i_lat(iq)*(1.-ftmp)+  &
                       cap_SP_CO2i_lat(iq+1)*ftmp 
                  CO2ice_on_ground = ( glat(i,j) <= tgt_lat )
                  CO2ice_depth = 0.2_f4  ! [m]
                  tgt_lat = cap_SP_H2Oi_lat(iq)*(1.-ftmp)+  &
                       cap_SP_H2Oi_lat(iq+1)*ftmp 
                  H2Oice_on_ground = ( glat(i,j) <= tgt_lat )
               end if

            else         

               H2Oice_on_ground = .FALSE.
               CO2ice_on_ground = ( gi(i,j) > ZERO_f4 )
               CO2ice_depth = abs(gi(i,j))/rho_CO2ice

            end if

            if (CO2ice_on_ground) then

               ! (pi/cp)_sfc:
               pi_sfc = (pp(1,i,j)+pp(2,i,j)+pi0(1,i,j)+pi0(2,i,j))*0.5_f4*cpi
               pr_sfc = p00*pi_sfc**cpor   ! "Surface" pressure [Pa]

               Tsat = Tsat_CO2ice(pr_sfc)  ! CO2 saturation temperature [K]

               sbps_Q(n6,i,j,1) = gt(i,j,n6) ! topmost SBS tempK (temporarily)

               ! Depth of ice layer (Implicitly / by 1 m^2):
               sbps_lyrthkn(n6+1,i,j,1) = CO2ice_depth

               albedt(i,j) = albedo_CO2ice(SBPS_lyrthkn(n6+1,i,j,1)  &
                    ,albedo_static(i,j,1))  ! Init time-variable albedo
               gd(igd)%emisst(i,j) = emiss_CO2ice
               sfc_tempK(i,j,1) = Tsat      ! Init sfc temp
  
               sbps_thrmcap(n6+1,i,j,1) = cp_CO2ice(Tsat)   ! SPS cp
               sbps_density(n6+1,i,j,1) = rho_CO2ice   ! SPS density
               sbps_Q(n6+1,i,j,1) = Tsat    ! SPS tempK (temporarily)

            else if (H2Oice_on_ground) then

               sbps_Q(n6,i,j,1) = gt(i,j,n6)
               albedt(i,j) = 0.495_f4    ! Init time-variable albedo
               gd(igd)%emisst(i,j) = gd(igd)%emissivity_static(i,j,1)
               sfc_tempK(i,j,1) = sbps_Q(n6,i,j,1) ! Init sfc temp for routines

               sbps_lyrthkn(n6+1,i,j,1) = ZERO_f4       ! No CO2 ice
               sbps_Q(n6+1,i,j,1) = ZERO_f4             !
                 
            else     ! No ice on ground 
               
               sbps_Q(n6,i,j,1) = gt(i,j,n6)
               albedt(i,j) = albedo_static(i,j,1)  ! Init time-variable albedo
               gd(igd)%emisst(i,j) = gd(igd)%emissivity_static(i,j,1)
               sfc_tempK(i,j,1) = sbps_Q(n6,i,j,1) ! Init sfc temp for routines

               sbps_lyrthkn(n6+1,i,j,1) = ZERO_f4       ! No CO2 ice
               sbps_Q(n6+1,i,j,1) = ZERO_f4             !

            end if

            ! Initialize remaining SBS layers:

            do iq=1,n6-1
               sbps_Q(iq,i,j,1) = gt(i,j,iq)   ! [K] (temporarily)
            end do

         end do
      end do

      deallocate(gt, gi, stat=e)      ! Deallocate work array
      if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
           'sfcinit')

   end if

   ! Compute heat storage quantity (what SHOULD be in SBPS_Q array):
   
   do j=1,n3
      do i=1,n2
      
         do iq=2,n6
            SBPS_Q(iq,i,j,1) = SBPS_Q(iq,i,j,1)*  &
                 SBPS_density(iq,i,j,1)*SBPS_thrmcap(iq,i,j,1)
!#                 ((SBPS_density(iq-1,i,j,1)*SBPS_thrmcap(iq-1,i,j,1)+  & 
!#                 SBPS_density(iq,i,j,1)*SBPS_thrmcap(iq,i,j,1))*0.5_f4)
         end do
         SBPS_Q(1,i,j,1) = SBPS_Q(1,i,j,1)*  &
              SBPS_density(1,i,j,1)*SBPS_thrmcap(1,i,j,1)  
         
         if (sbps_Q(n6+1,i,j,1) > 1.e-3_f4) then ! CO2 ice on ground;
                                             !  arbitrary test value (just want
                                             !  to check if Q is zero or not) 
            iq = n6+1   ! Initialize SPS layer(s) since present
            SBPS_Q(iq,i,j,1) = SBPS_Q(iq,i,j,1)*  &
                 SBPS_density(iq,i,j,1)*SBPS_thrmcap(iq,i,j,1)
         end if
    
      end do
   end do

   ! Calculate/set surface reservoir content:

   if (dust_tog > 0 .or. H2Oi_tog > 0 .or. CO2i_tog > 0) then
      call switch_to_Ascalar(igd, 1)  ! Fill 'iaerscp/t', etc. indices
      call sfcinit_aerrs(n1, n2, n3, n4, n5, n6, igd, naersclr(igd), glat,  &
           glon, TI_static, albedo_static, albedt, a(iaerrs), sbps_lyrthkn)
   end if

end if

end subroutine sfcinit
