!```````````````````````````````````````````````````````````````````
module topo_RT_effects_mod

!+ Module for topographic shading/slope/aspect routines

use univ_kind_defs_mod, only: f4

implicit none

real(f4), parameter :: dazm_cutoff0 = 400000._f4  ! Max distance [m] to
                                                  !  traverse along each
                                                  !  azimuth

real(f4) :: shd_plon, shd_plat, dazm_cutoff
real(f4), allocatable, dimension(:) :: shd_deltax,  &  ! {n_grids}
     shd_deltay
integer, allocatable, dimension(:) :: shd_nxp, shd_nyp   ! {n_grids}

!*** Shadow OPS grid arrays:

real(f4), allocatable, dimension(:) :: shd_xt, shd_yt
real(f4), allocatable, dimension(:,:) :: shd_fmapt, shd_fmapti,  &
     shd_glat, shd_glon, shd_dxt, shd_dyt, shd_topt, shd_xmn, shd_ymn
integer, allocatable, dimension(:) :: extn_W, extn_E, extn_S, extn_N


contains


!```````````````````````````````````````````````````````````````````
subroutine shd_extn(n2, n3, rdxt, rdyt, extn_W, extn_E, extn_S, extn_N)

!+ Computes magnitude of the shadow extensions to a given PRAMS grid

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3
real(f4), dimension(n2,n3), intent(IN) :: rdxt, rdyt

integer, intent(OUT) :: extn_W, extn_E, extn_S, extn_N

!== Local declarations:

real(f4) :: mdxt_W, mdxt_E, mdyt_S, mdyt_N

!=== Executable statements:

mdxt_W = maxval(rdxt(1,:))    ! Minimum grid spacing on edges of current PRAMS
mdxt_E = maxval(rdxt(n2,:))   !   OPS grid
mdyt_S = maxval(rdyt(:,1))    !
mdyt_N = maxval(rdyt(:,n3))   !

extn_W = max(1, int(dazm_cutoff*mdxt_W))+1  ! Number of points in shadow
extn_E = max(1, int(dazm_cutoff*mdxt_E))+1  ! extension (on each grid edge)
extn_S = max(1, int(dazm_cutoff*mdyt_S))+1  !
extn_N = max(1, int(dazm_cutoff*mdyt_N))+1  !

end subroutine shd_extn


!```````````````````````````````````````````````````````````````````
subroutine shd_gridset(igd, sn2, sn3)

!+ Compute oblique polar stereographic grid and lat/lon of grid points

use univ_kind_defs_mod, only: f4
use univ_OPS_proj_mod, only: xy_ll_OPS
use main_memory_mod, only: gd, polelat, polelon
use body_specific_constants_mod, only: radius

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd, sn2, sn3

!== Local declarations:

integer :: i, j, igdp

!=== Executable statements:

shd_plat = polelat    !  Set pole point location
shd_plon = polelon    !

if (igd == 1) then      ! If coarse grid

   !*** Set lower SW corner of shadow grid based on location of PRAMS grid:
   shd_xmn(1,igd) = gd(igd)%xm(1)-extn_W(igd)*shd_deltax(igd)
   shd_ymn(1,igd) = gd(igd)%ym(1)-extn_S(igd)*shd_deltay(igd)

else        ! If a nested grid

   igdp = gd(igd)%nxtnest      ! Number of parent grid

   !*** Set lower SW corner based on nesting location within parent grid:

   shd_xmn(1,igd) = gd(igdp)%xm(gd(igd)%ninest)-extn_W(igd)*shd_deltax(igd)
   shd_ymn(1,igd) = gd(igdp)%ym(gd(igd)%njnest)-extn_S(igd)*shd_deltay(igd)

end if

!*** Fill the remaining shadow OPS grid location values:

do i=2,sn2        ! In x-direction
   shd_xmn(i,igd) = shd_xmn(i-1,igd)+shd_deltax(igd)
   shd_xt(i) = 0.5_f4*(shd_xmn(i,igd)+shd_xmn(i-1,igd))
end do
shd_xt(1) = (1.5_f4*shd_xmn(1,igd))-(0.5_f4*shd_xmn(2,igd))

do j=2,sn3        ! In y-direction
   shd_ymn(j,igd) = shd_ymn(j-1,igd)+shd_deltay(igd)
   shd_yt(j) = 0.5_f4*(shd_ymn(j,igd)+shd_ymn(j-1,igd))
end do
shd_yt(1) = (1.5_f4*shd_ymn(1,igd))-(0.5_f4*shd_ymn(2,igd))

!*** Calculate lat/lon at T-points for current OPS grid:

do j=1,sn3
   do i=1,sn2
      call xy_ll_OPS(shd_glat(i,j), shd_glon(i,j), shd_plat, shd_plon,  &
           shd_xt(i), shd_yt(j), radius)     ! OPS to lat/lon coords
   end do
end do

end subroutine shd_gridset


!```````````````````````````````````````````````````````````````````
subroutine shd_grdspc(igd, n2, n3)

!+ Calculate OPS map factors and grid spacings on the actual surface of the body

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use body_specific_constants_mod, only: radius

!=== Argument declarations:

integer, intent(IN) :: n2, n3, igd

!== Local declarations:

integer :: i, j
real(f4) :: xt2, yt2, c1

!=== Executable statements:

!*** Calculates map factors and inverse map factors at T-points:

c1 = (2._f4*radius)**2
do j=1,n3
   do i=1,n2

      xt2 = shd_xt(i)*shd_xt(i)   ! OPS x-coordinate squared
      yt2 = shd_yt(j)*shd_yt(j)   ! OPS y-coordinate squared

      shd_fmapt(i,j) = ONE_f4+(xt2+yt2)/c1        ! map factor 
      shd_fmapti(i,j) = ONE_f4/shd_fmapt(i,j)   ! inverse map factor

   end do
end do

do j=1,n3               ! X-direction grid spacings
   do i=2,n2
      shd_dxt(i,j) = shd_fmapt(i,j)/(shd_xmn(i,igd)-shd_xmn(i-1,igd))
   end do
   shd_dxt(1,j) = shd_dxt(2,j)*shd_fmapt(1,j)/shd_fmapt(2,j)
end do

do i=1,n2               ! Y-direction grid spacings
   do j=2,n3
      shd_dyt(i,j) = shd_fmapt(i,j)/(shd_ymn(j,igd)-shd_ymn(j-1,igd))
   end do
   shd_dyt(i,1) = shd_dyt(i,2)*shd_fmapt(i,1)/shd_fmapt(i,2)
end do

!*** Invert the grid spacings so they are in usual units (m):

do j=1,n3
   do i=1,n2
      shd_dxt(i,j) = ONE_f4/shd_dxt(i,j)
      shd_dyt(i,j) = ONE_f4/shd_dyt(i,j)
   end do
end do

end subroutine shd_grdspc


!```````````````````````````````````````````````````````````````````
subroutine shd_sfc_data_prep(igd, input_fn_primary, toptenh, toptwvlx,  &
     toptwvly, itopsflg, ffill)

!+ Find topography on shadow OPS grid; DRIVER

use univ_kind_defs_mod, only: f4, i2
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use univ_numtype_constants_mod, only: huge_f4
use body_specific_constants_mod, only: dist_deglat
use surface_data_prep_mod, only: sfcopqr
use manage_NetCDF_classic_mod, only: mNc_file_metadata_obj,  &
     mNc_populate_f_md_obj_r, mNc_extract_g_Att_val, mNc_extract_var_Att_val,  &
     mNc_recycle_f_md_obj

implicit none

!=== Argument declarations:

integer, intent(IN) :: itopsflg, igd
real(f4), intent(IN) :: toptwvlx, toptwvly, toptenh, ffill
character(len=*), intent(IN) :: input_fn_primary

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'shd_sfc_data_prep'

real(f4), allocatable, dimension(:,:) :: datp, datq, tmpq_1, tmpq_2

character(len=MAX_FILE_PATH_LEN), dimension(2) :: input_fn
integer :: niq, njq, np_x, np_y, e, n_inpfn, i_inpfn, i
logical :: last_inpfn, has_global_extent
real(f4) :: deltaxq, deltayq, deltaxp, deltayp, delta_lat, delta_lon, offset_lon

type(mNc_file_metadata_obj), save :: input_file_obj

real(f4), dimension(1) :: f4_tmp_1d_1
integer(i2), dimension(1) :: i2_tmp_1d_1

!=== Executable statements:

! Dimensions of temp grid (Q) - wavelength filtering only applied to topo:
deltaxq = 0.5_f4*toptwvlx*shd_deltax(igd)
deltayq = 0.5_f4*toptwvly*shd_deltay(igd)
niq = int(real(shd_nxp(igd)-1, kind=f4)*shd_deltax(igd)/deltaxq)+4
njq = int(real(shd_nyp(igd)-1, kind=f4)*shd_deltay(igd)/deltayq)+4

! Allocate temp grid (Q) and related arrays:
allocate(datq(niq,njq), tmpq_1(niq,njq), tmpq_2(niq,njq), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1')

! Initialize temp grid (Q) with sentinel values:
datq = huge_f4

input_fn(1) = trim(input_fn_primary)
input_fn(2) = 'NONE'
n_inpfn = 2  ! Default (assume use of both primary and auxiliary input files)
i = len_trim(input_fn(2))
if (i >= 4) then
   if (input_fn(2)(i-3:i) == 'NONE') n_inpfn = n_inpfn-1  ! No auxiliary file
end if
do i_inpfn=1,n_inpfn
   last_inpfn = ( i_inpfn == n_inpfn )   ! Should the field be finalized?

   has_global_extent = ( index(trim(input_fn(i_inpfn)), 'global') > 0 )

   ! Configure raw data grid (O) parameters:

   call mNc_populate_f_md_obj_r(trim(input_fn(i_inpfn)), input_file_obj)

   call mNc_extract_g_Att_val(input_file_obj, 'lat_increment', f4_tmp_1d_1)
   delta_lat = f4_tmp_1d_1(1)
   call mNc_extract_g_Att_val(input_file_obj, 'lon_increment', f4_tmp_1d_1)
   delta_lon = f4_tmp_1d_1(1)
   call mNc_extract_g_Att_val(input_file_obj, 'lon_native_to_IAU2000',  &
        f4_tmp_1d_1)
   offset_lon = f4_tmp_1d_1(1)

   ! Dimensions of interpolated raw data grid (P):
   np_x = max(1, int(deltaxq/(delta_lon*dist_deglat)))
   np_y = max(1, int(deltayq/(delta_lat*dist_deglat)))
   deltaxp = deltaxq/real(np_x, kind=f4)
   deltayp = deltayq/real(np_y, kind=f4)

   ! Allocate interpolated raw data grid (P):
   allocate(datp(np_x,np_y), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2')

   ! Interpolate/filter topo data onto OPS grid:
   call mNc_extract_var_Att_val(input_file_obj, 'topo', 'missing_value',  &
        i2_tmp_1d_1)
   call sfcopqr(niq, njq, shd_nxp(igd), shd_nyp(igd), shd_xt, shd_yt,  &
        deltaxp, deltayp, deltaxq, deltayq, datp, datq, shd_topt, 'TOP', igd,  &
        itopsflg, toptenh, toptwvlx, toptwvly, 0, np_x, np_y, input_file_obj,  &
        'topo', i2_tmp_1d_1(1), delta_lon, delta_lat, offset_lon, shd_plat,  &
        shd_plon, tmpq_1, tmpq_2, last_inpfn, has_global_extent, ffill)

   call mNc_recycle_f_md_obj(input_file_obj)   ! Finished

   deallocate(datp, stat=e)    ! Deallocate work arrays
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '2')

end do  !END: i_inpfn

deallocate(datq, tmpq_1, tmpq_2, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

end subroutine shd_sfc_data_prep


!```````````````````````````````````````````````````````````````````
subroutine topo_RT_effects_calc(igd, itgt, jtgt, n2, n3, glon, glat, topt,  &
     tslope_angle, tazm_angle)

!+ Calculates azimuthally-varying topographic shadowing/slope/aspect

use univ_kind_defs_mod, only: f4, f8
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_math_constants_mod, only: ZERO_f4, ONE_f4, pi_f4, twopi_f4
use univ_convert_units_mod, only: deg_to_rad_f4
use univ_interpolation_mod, only: intrp_2D_pnt_type02g
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use body_specific_constants_mod, only: radius
use main_memory_mod, only: islastog
use main_memory_bspec_mod, only: ishadtog
use Mod_Euler_Rotation_mod, only: alpha, beta, gamma, euler_rotation_matrix
use topo_RT_effects_dims_mod, only: nazm

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, itgt, jtgt, igd
real(f4), dimension(n2,n3), intent(IN) :: glon, glat, topt

real(f4), dimension(n2,n3), intent(INOUT) :: tslope_angle, tazm_angle

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'topo_RT_effects_calc'

integer, allocatable, dimension(:) :: ndazm_cnt
real(f4), allocatable, dimension(:,:) :: azm_TCx, azm_TCy, azm_TCz
real(f4), allocatable, dimension(:) :: TCx_tgt, TCy_tgt, TCz_tgt, azm_rho,  &
     azm_tha, azm_phi, azm_OPSx, azm_OPSy, junk

integer :: iaz, ndazm, id, ndazm0, e
real(f4) :: azm, dazm_incr, azm_incr_r, xi, yj, atmp, dzsdx, dzsdy, tol,  &
     Xoid_radius, pi2, xb1, xb2, yb1, yb2, lon0, lat0, rho0, tha0, phi0,  &
     ACx0, ACy0, ACz0
real(f4), dimension(4) :: stopo

!=== Executable statements:

!*** Miscellaneous constants:

Xoid_radius = radius  ! Xoid radius (a constant spherical radius -- for now) 
pi2 = pi_f4*0.5_f4            ! Half of pi

!*** OPS grid boundaries:

xb1 = shd_xt(1)+shd_deltax(igd)             ! Westernmost edge of OPS grid
xb2 = shd_xt(shd_nxp(igd))-shd_deltax(igd)  ! Easternmost edge of OPS grid
yb1 = shd_yt(1)+shd_deltay(igd)             ! Southernmost edge of OPS grid
yb2 = shd_yt(shd_nyp(igd))-shd_deltay(igd)  ! Northernmost edge of OPS grid

lon0 = glon(itgt,jtgt)*deg_to_rad_f4    ! Longitude of current point, in radians
lat0 = glat(itgt,jtgt)*deg_to_rad_f4    ! Latitude of current point, in radians

!^
!print *,'lat0/lon0',lat0/pi_o_180_f4,lon0/pi_o_180_f4
!^

if (ishadtog == 1 .or. islastog == 1) then

!*** TOPOGRAPHIC SHADOWING:

   azm_incr_r = twopi_f4/real(nazm, kind=f4) ! Azimuth angle increment (radians)

   !*** Azimuth traverse quantities:

                                  ! Arbitrary distance increment along azimuth
   dazm_incr = shd_deltax(igd)/4._f4 ! Will cause problems if too small, as 
                                  ! interpolation errors can produce large topo
                                  ! gradients at small scales, but if too
                                  ! large, will underestimate topo shadowing

   ndazm0 = int(dazm_cutoff/dazm_incr)  ! MAXIMUM number of points along
                                        !   each azimuth

   allocate(TCx_tgt(ndazm0), junk(ndazm0),  &         ! Allocate azimuth arrays
        azm_TCx(ndazm0,nazm), azm_TCy(ndazm0,nazm), azm_TCz(ndazm0,nazm),  &
        azm_rho(ndazm0), azm_tha(ndazm0), azm_phi(ndazm0), azm_OPSx(ndazm0),  &
        azm_OPSy(ndazm0), ndazm_cnt(nazm), TCy_tgt(ndazm0), TCz_tgt(ndazm0),  &
        stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'topo_RT_effects_calc')

   do id=1,ndazm0
      TCx_tgt(id) = real(id, kind=f4)*dazm_incr ! Target TC x-coords per azimuth
      TCy_tgt(id) = ZERO_f4                     !
      TCz_tgt(id) = ZERO_f4                     !
   end do

   !*** Convert lat/lon coords of origin point to spherical coords:

   rho0 = Xoid_radius+topt(itgt,jtgt)  ! Mars radius at current point

   if (lon0 < ZERO_f4) then   !
      tha0 = -lon0            !
   else                       !  Theta of current point
      tha0 = twopi_f4-lon0    !
   end if                     !
   
   phi0 = pi2-lat0         !  Phi of current point

!^
!   print *,'rho0/tha0/phi0',rho0,tha0/pi_o_180_f4,phi0/pi_o_180_f4
!^

   ACx0 = rho0*sin(phi0)*cos(tha0)    ! Bodycentric Cartesian coords for
   ACy0 = rho0*sin(phi0)*sin(tha0)    !   origin point 
   ACz0 = rho0*cos(phi0)              !

   !*** Azimuth-independent coordinate rotation angles:

   alpha = real(tha0, kind=f8)    !  First coord rotation angle about z-axis
   beta = real(phi0, kind=f8)     !  Second coord rotation angle about y'-axis

   !*** Calculate hemispherical viewshed around current point:

   do iaz=1,nazm     ! Loop over number of azimuth divisions

      ! Note: azimuth goes from `azm_incr_r` to (2.*pi)
      azm = real(iaz, kind=f4)*azm_incr_r    ! Azimuth (radians clockwise from north)

      ! x''-axis is oriented towards the north (0deg), so need to rotate it
      ! so the x'''-axis is oriented along the current azimuth:

      gamma = real(azm-pi_f4, kind=f8)   ! Third coord rotation angle about z''-axis

      call Euler_Rotation_Matrix    ! Set up coord transformation matrix for
                                    !   current azimuth

      ndazm = ndazm0      ! Reset this to maximum
      call get_rotated_coords('TC2PS', ndazm0, ACx0, ACy0, ACz0, pi2,  &
           deg_to_rad_f4, shd_plat, shd_plon, xb1, xb2, yb1, yb2, azm_rho,  &
           azm_tha, azm_phi, azm_OPSx, azm_OPSy, junk, TCx_tgt, TCy_tgt,  &
           TCz_tgt, ndazm)

      ndazm_cnt(iaz) = ndazm  ! Record # of points in this azimuth traverse

      !*** Take note that NDAZM may have been adjusted in routine above:

      do id=1,ndazm   ! Loop over number of distance increments along azimuth

         ! Fractional location in OPS grid of current azimuth point:

         xi = (azm_OPSx(id)-xb1)/shd_deltax(igd)+ONE_f4
         yj = (azm_OPSy(id)-yb1)/shd_deltay(igd)+ONE_f4

         !*** Find topo height at current azimuth point via interpolation:

         atmp = intrp_2D_pnt_type02g(shd_topt, xi, yj)
!         call gdtost(shd_topt,shd_nxp(igd),shd_nyp(igd),xi,yj,atmp)

         azm_rho(id) = Xoid_radius+atmp     ! Mars radius at current point

         ! Fill cardinal direction topo values for slope/aspect calcs

         if ((mod(iaz, nazm/4) == 0) .and. (id == 1))  &
              stopo(iaz/(nazm/4)) = atmp

      end do              ! End distance increment loop

      ! Transform the traverse point coords to topocentric coord system,
      !   with z in the direction of the local zenith, and x outward along
      !   azimuth traverse:

      call get_rotated_coords('BS2TC', ndazm0, ACx0, ACy0, ACz0, pi2,  &
           deg_to_rad_f4, shd_plat, shd_plon, xb1, xb2, yb1, yb2, azm_rho,  &
           azm_tha, azm_phi, azm_TCx(:,iaz), azm_TCy(:,iaz), azm_TCz(:,iaz),  &
           TCx_tgt, TCy_tgt, TCz_tgt, ndazm)

   end do     ! End azimuth increment loop

   !*** Calculate horizon angle:

   if (ishadtog == 1)  &
        call find_horizon(igd, itgt, jtgt, ndazm0, ndazm_cnt, azm_TCx,  &
        azm_TCy, azm_TCz)

   !*** TOPOGRAPHIC SLOPE/ASPECT:

   if (islastog == 1) then

      tol = 1.e-20_f4      ! Tolerance for smallness of slope [m]

      dzsdx = (stopo(1)-stopo(3))/(2._f4*dazm_incr) ! Topo slope from W to E
      dzsdy = (stopo(4)-stopo(2))/(2._f4*dazm_incr) ! Topo slope from S to N

      ! If slope is very small, set to a small value:
      if (abs(dzsdx) < tol) dzsdx = tol
      if (abs(dzsdy) < tol) dzsdy = tol

      tslope_angle(itgt,jtgt) = atan(sqrt(dzsdx*dzsdx+dzsdy*dzsdy))

      !*** Topographic aspect angle (clockwise from north):

      ! = (pi/2.)-atan2(dzsdy,dzsdx) What RAMS uses -- I think it is **WRONG**
      tazm_angle(itgt,jtgt) = pi_f4+atan2(dzsdx, dzsdy)
   
   end if

end if

! DONE.  Clean up:

deallocate(TCx_tgt, junk,  &               ! Deallocate azimuth arrays
     azm_TCx, azm_TCy, azm_TCz, azm_rho, azm_tha, azm_phi, azm_OPSx,  &
     azm_OPSy, ndazm_cnt, TCy_tgt, TCz_tgt, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
     'topo_RT_effects_calc')

end subroutine topo_RT_effects_calc


!```````````````````````````````````````````````````````````````````
subroutine find_horizon(igd, itgt, jtgt, ndazm0, ndazm_cnt, xx, yy, zz)

!+ Calculates the cosine of the horizon zenith angle at a given OPS grid point

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use topo_RT_effects_dims_mod, only: nazm
use main_memory_mod, only: gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: ndazm0, igd, itgt, jtgt
real(f4), dimension(ndazm0,nazm), intent(IN) :: xx, yy, zz
integer, dimension(nazm), intent(IN) :: ndazm_cnt

!== Local declarations:

integer :: iaz, ndazm, tloc1
real(f4) :: azm_max, elev_angle
integer, dimension(1) :: tloc

!=== Executable statements:

do iaz=1,nazm

   ndazm = ndazm_cnt(iaz)

   azm_max = maxval(zz(1:ndazm,iaz)) ! Find largest surface z-dir value
                                     !   along azimuth traverse

   if (azm_max > epsilon(azm_max) .or. azm_max < -epsilon(azm_max)) then     !*** Horizon is obstructed or extended
      
      tloc = maxloc(zz(1:ndazm,iaz))    ! Location of obstruction in array
      tloc1 = tloc(1)

      gd(igd)%TCx_obstruct(itgt,jtgt,iaz) = xx(tloc1,iaz) ! Obstruction distance
      gd(igd)%TCz_obstruct(itgt,jtgt,iaz) = zz(tloc1,iaz)  ! Obstruction height
      elev_angle = atan(zz(tloc1,iaz)/xx(tloc1,iaz)) ! Horizon elevation angle

!   elseif(azm_max < -epsilon(azm_max)) then          !*** Horizon is extended

!      !***NOTE: For the time being, ignore horizon extension:

!      elev_angle = 0.                     ! Elevation angle of horizon

   else                      !*** Horizon is neither obstructed nor extended

      elev_angle = ZERO_f4                     ! Elevation angle of horizon

   end if

   gd(igd)%cosz_horizon(itgt,jtgt,iaz) = sin(elev_angle) ! Cosine of the horizon
                                                         !  zenith angle

end do

end subroutine find_horizon


!```````````````````````````````````````````````````````````````````
subroutine get_rotated_coords(iotype, ndazm0, BCx0, BCy0, BCz0, pi2,  &
     pi_o_180_f4, plat, plon, W_ops_bnd, E_ops_bnd, S_ops_bnd, N_ops_bnd,  &
     rho, tha, phi, x, y, z, xi, yi, zi, ndazm)

!+ Performs coordinate transformations on a given set of points

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_OPS_proj_mod, only: ll_xy_OPS
use univ_math_constants_mod, only: ZERO_f4
use univ_exception_mod, only: exception, FATAL
use mod_Euler_rotation_mod, only: NC_to_OC, OC_to_NC
use body_specific_constants_mod, only: radius

implicit none

!=== Argument declarations:

integer, intent(IN) :: ndazm0
real(f4), intent(IN) :: BCx0, BCy0, BCz0, pi2, pi_o_180_f4, plat, plon,  &
     W_ops_bnd, E_ops_bnd, S_ops_bnd, N_ops_bnd
real(f4), dimension(ndazm0), intent(IN) :: rho, xi, yi, zi
character(len=5), intent(IN) :: iotype

integer, intent(INOUT) :: ndazm
real(f4), dimension(ndazm0), intent(INOUT) :: tha, phi, x, y, z

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'get_rotated_coords'

integer :: id
real(f4) :: x1, y1, z1, ct1, xx, yy, zz, p1, t1, q1, q2, q3

!=== Executable statements:

! *** Input type (OTYPE) possibilities ***
! BC = Bodycentric Cartesian
! TC = Topocentric Cartesian
! BS = Bodycentric Spherical
! 
! *** Output type (OTYPE) possibilities ***
! BC = Bodycentric Cartesian
! TC = Topocentric Cartesian
! BS = Bodycentric Spherical
! PS = Oblique Polar Stereographic
!
! *** e.g., iotype = 'TC2BS' ***

if (iotype == 'TC2PS') then

   azm_trav: do id=1,ndazm

      ! TC coords --> BC coords:

      q1 = NC_to_OC(1,1)*xi(id)+NC_to_OC(1,2)*yi(id)+  &
           NC_to_OC(1,3)*zi(id)+BCx0   ! BC x
      q2 = NC_to_OC(2,1)*xi(id)+NC_to_OC(2,2)*yi(id)+  &
           NC_to_OC(2,3)*zi(id)+BCy0   ! BC y
      q3 = NC_to_OC(3,1)*xi(id)+NC_to_OC(3,2)*yi(id)+  &
           NC_to_OC(3,3)*zi(id)+BCz0   ! BC z

      ! BC coords --> BS coords:

      tha(id) = -atan2(q2,q1)                   
      phi(id) = pi2-acos(q3/sqrt(q1*q1+q2*q2+q3*q3))

      call ll_xy_OPS(phi(id)/pi_o_180_f4, tha(id)/pi_o_180_f4, plat, plon,  &
           x(id), y(id), radius)

      !*** Test if x(:), y(:) are within bounds; exit loop, set param if not:
      
      if (x(id) < W_ops_bnd .or. x(id) > E_ops_bnd .or.  &
           y(id) < S_ops_bnd .or. y(id) > N_ops_bnd) then
         write (*, '(a,10x,i5,2x,a,2x,i5)')  &
              'ADAPTING edge...', ndazm, 'changed to', id-1
         ndazm = id-1
         exit azm_trav
      end if

   end do azm_trav

   if (ndazm == 0) call exception(procedure_name, FATAL,  &
        'No point successfully found on azimuth -- I refuse to continue.')

else if (iotype == 'BS2TC') then

   do id=1,ndazm

      p1 = pi2-phi(id)
      
      t1 = tha(id)
      if (t1 < ZERO_f4) then
         t1 = -t1
      else if (t1 >= ZERO_f4) then
         t1 = 4._f4*pi2-t1
      end if
      
      ct1 = rho(id)*sin(p1)
      x1 = ct1*cos(t1)         !
      y1 = ct1*sin(t1)         ! BC coordinates of given spherical point
      z1 = rho(id)*cos(p1)     !
      
      xx = x1-BCx0             !
      yy = y1-BCy0             ! Translated (to reference BC point) BC coords
      zz = z1-BCz0             !

      ! TC coordinates for a given point:

      x(id) = OC_to_NC(1,1)*xx+OC_to_NC(1,2)*yy+OC_to_NC(1,3)*zz
      y(id) = OC_to_NC(2,1)*xx+OC_to_NC(2,2)*yy+OC_to_NC(2,3)*zz
      z(id) = OC_to_NC(3,1)*xx+OC_to_NC(3,2)*yy+OC_to_NC(3,3)*zz
      
   end do

end if

end subroutine get_rotated_coords


!```````````````````````````````````````````````````````````````````
subroutine insert_model_topo(igd, n2, n3, topt)

!+ Overwrite the core region (i.e., not the topo shadowing extension regions)
!   of the shadow OPS grid topography (which does *not* include nesting
!   feedback) with the already prepared (by `surface_nest`) model topography,
!   which does include nesting feedback.

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, igd
real(f4), dimension(n2,n3), intent(IN) :: topt

!== Local declarations:

integer :: ii, jj, i, j

!=== Executable statements:

do j=1,n3
   jj = j+extn_S(igd)
   do i=1,n2
      ii = i+extn_W(igd)
      shd_topt(ii,jj) = topt(i,j)
   end do
end do

end subroutine insert_model_topo


end module topo_RT_effects_mod


!```````````````````````````````````````````````````````````````````
subroutine topo_RT_effects_init(beg_grid, end_grid)

!+ Driver for the calculation of topographic effects on insolation

use univ_kind_defs_mod, only: f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_numtype_constants_mod, only: epsilon_f4
use main_memory_mod, only: islastog, n_grids, gd, initialization_type,  &
     map_projection, a
use main_memory_bspec_mod, only: ishadtog
use vtables_mod, only: idxtn, idytn, iglonn, iglatn, itoptn, itslopangn,  &
     itazmangn
use topo_RT_effects_mod, only: shd_extn, shd_gridset, shd_sfc_data_prep,  &
     topo_RT_effects_calc, shd_nxp, shd_nyp, extn_N, extn_E, extn_S, extn_W,  &
     shd_xt, shd_topt, dazm_cutoff, shd_deltax, shd_deltay, shd_xmn, shd_ymn,  &
     shd_glat, shd_glon, shd_yt, dazm_cutoff0, insert_model_topo, shd_fmapt,  &
     shd_fmapti, shd_dxt, shd_dyt, shd_grdspc
use surface_init_mod, only: toptinit

implicit none

!=== Argument declarations:

integer, intent(IN) :: beg_grid, end_grid

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'topo_RT_effects_init'

integer :: i, j, igd, shd_nxpmax, shd_nypmax, e

!=== Executable statements:

!*** Check if user wants to enable topographic insolation effects at all:

if (ishadtog == 0 .and. islastog == 0) then     ! NO topo effects wanted
   return         ! Continue with normal model initialization
else if (ishadtog == 1 .and. map_projection == 0) then
   call exception(procedure_name, FATAL,  &
        'Cannot use shadowing on Cartesian grid (MAP_PROJECTION=0)')
end if

allocate(extn_W(n_grids),extn_E(n_grids),  &   ! Allocate arrays to hold # of
     extn_S(n_grids),extn_N(n_grids),  &      ! extra pts on shadow OPS grid
     shd_deltax(n_grids), shd_deltay(n_grids), shd_nxp(n_grids),  &
     shd_nyp(n_grids), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '01')

!*** Calculate shadow OPS grid dimensions/stuff for all grids:

if (initialization_type == 1) then  ! "Idealized" OPS grid
   if (islastog <= 1) then   ! Need to do further processing
      ! Assume "localized" shadowing <~ the largest horiz dimension of grid #1:
      dazm_cutoff = min(max(gd(1)%xt(gd(1)%nxp)-gd(1)%xt(1),  &
           gd(1)%yt(gd(1)%nyp)-gd(1)%yt(1))*1.1_f4, dazm_cutoff0)
   end if
else   ! "Normal" cutoff
   dazm_cutoff = dazm_cutoff0
end if

shd_nxpmax = 0   ! Initialize these to zero
shd_nypmax = 0   !
do igd=1,n_grids            ! Loop over all grids
   shd_deltax(igd) = gd(igd)%deltax
   shd_deltay(igd) = gd(igd)%deltay

   if (abs(shd_deltax(igd)-shd_deltay(igd)) > epsilon_f4 .or.  & ! Some quick parameter checks
        shd_deltax(igd) > dazm_cutoff) then
      call exception(procedure_name, FATAL,  &
           'Delta-X and delta-Y too large or unequal -- I refuse to proceed.')
   end if

   !*** Extend PRAMS OPS grid by how many points in each direction?

   call shd_extn(gd(igd)%nxp, gd(igd)%nyp, a(idxtn(igd)), a(idytn(igd)),  &
        extn_W(igd), extn_E(igd), extn_S(igd), extn_N(igd))

   !*** NOTE: Shadow grid larger than original model grid:
   shd_nxp(igd) = extn_W(igd)+gd(igd)%nxp+extn_E(igd)
   shd_nyp(igd) = extn_S(igd)+gd(igd)%nyp+extn_N(igd)

   shd_nxpmax = max(shd_nxpmax, shd_nxp(igd))     ! Max number of points
   shd_nypmax = max(shd_nypmax, shd_nyp(igd))     ! on any grid
end do

!*** Allocate grid coordinate array:

allocate(shd_xmn(shd_nxpmax,n_grids), shd_ymn(shd_nypmax,n_grids), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '02')

!*** Loop through all grids, calculating topographic shadowing:

do igd=beg_grid,end_grid

   !*** Allocate work arrays for the current shadow grid:

   allocate(shd_xt(shd_nxp(igd)), shd_yt(shd_nyp(igd)),  &  ! OPS x/y gridpoints
        stat=e) 
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '03')
   allocate(shd_topt(shd_nxp(igd),shd_nyp(igd)),  & ! Topography on shadow grid
        stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '04')
   allocate(shd_glat(shd_nxp(igd),shd_nyp(igd)),  & ! Lat/lon of OPS gridpoints
        shd_glon(shd_nxp(igd),shd_nyp(igd)), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '05')
   allocate(shd_fmapt(shd_nxp(igd),shd_nyp(igd)),  & ! Map factors (and inverse)
        shd_fmapti(shd_nxp(igd),shd_nyp(igd)), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '06')
   allocate(shd_dxt(shd_nxp(igd),shd_nyp(igd)),  &  ! Real-world grid spacings
        shd_dyt(shd_nxp(igd),shd_nyp(igd)), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '07')

   write (*, '(/,a,2x,i2)') 'Setting up shadow OPS grid for PRAMS grid:',  &
        igd          ! Report status to user

   !*** Calculate OPS grid (and lat/lon pairs) for shadow calculations:
   call shd_gridset(igd, shd_nxp(igd), shd_nyp(igd))

   !*** Calculate actual grid spacings / map factors for shadow grid:
   call shd_grdspc(igd, shd_nxp(igd), shd_nyp(igd))

   !*** Find topography on shadow OPS grid:

   write (*, '(/,a,/)') 'Starting topography on shadow grid ...'

   call toptinit(shd_nxp(igd), shd_nyp(igd), shd_topt)

   if (gd(igd)%itoptflg <= 1) then    ! Full variable topography
      call shd_sfc_data_prep(igd, gd(igd)%itoptfn,  &
           gd(igd)%toptenh, gd(igd)%toptwvlx, gd(igd)%toptwvly,  &
           gd(igd)%itopsflg, -99999._f4)
   end if
   
   ! Allow for user-defined topography alterations:
   call toptinit_user(shd_nxp(igd), shd_nyp(igd), igd, shd_topt, shd_fmapt,  &
        shd_xt, shd_yt)

   ! Overwrite the core region (i.e., not the topo shadowing extension regions)
   !  of the shadow OPS grid topography (which does *not* include nesting
   !  feedback) with the already prepared (by `surface_nest`) model topography,
   !  which does include nesting feedback:
   call insert_model_topo(igd, gd(igd)%nxp, gd(igd)%nyp, a(itoptn(igd)))

   !*** Calculate azimuthally-varying horizon angles AND 
   !      topographic slope/aspect at each gridpoint:

   write (*, '(/,a,2x,i2)')  &
        'Calculating topographic shadowing and orientation on grid:',  &
        igd          ! Report status to user

   do j=1,gd(igd)%nyp
      write (*, '(a,i5,a,i5)') 'Row ', j, ' of ', gd(igd)%nyp
      do i=1,gd(igd)%nxp
         call topo_RT_effects_calc(igd, i, j, gd(igd)%nxp, gd(igd)%nyp,  &
              a(iglonn(igd)), a(iglatn(igd)), a(itoptn(igd)),  &
              a(itslopangn(igd)), a(itazmangn(igd)))
      end do
   end do

   !*** Deallocate work arrays for the current OPS shadow grid:
   
   deallocate(shd_xt, shd_yt, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '01')
   deallocate(shd_glat, shd_glon, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '02')
   deallocate(shd_topt, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '03')
   deallocate(shd_fmapt, shd_fmapti, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '04')
   deallocate(shd_dxt, shd_dyt, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '05')

end do

! Deallocate work arrays:

deallocate(shd_xmn, shd_ymn, stat=e)              
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '04')
deallocate(extn_W, extn_E, extn_S, extn_N, shd_deltax, shd_deltay, shd_nxp,  &
     shd_nyp, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '05')

end subroutine topo_RT_effects_init
