!```````````````````````````````````````````````````````````````````
module tstep_driver_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine advance_aersclrbnd(m1, m2, m3, asclp, asclt)

!+ Step aerosol scalar boundaries forward in time

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: run_is_purely_2D, c_gd

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3
real(f4), dimension(m1,m2,m3), intent(IN) :: asclt 

real(f4), dimension(m1,m2,m3), intent(INOUT) :: asclp 

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

do j=1,m3
   do k=1,m1
      asclp(k,1,j) = asclp(k,1,j)+asclt(k,1,j)*c_gd%dt_dyn
      asclp(k,m2,j) = asclp(k,m2,j)+asclt(k,m2,j)*c_gd%dt_dyn
   end do
end do

if (.NOT.run_is_purely_2D) then
   do i=1,m2
      do k=1,m1
         asclp(k,i,1) = asclp(k,i,1)+asclt(k,i,1)*c_gd%dt_dyn
         asclp(k,i,m3) = asclp(k,i,m3)+asclt(k,i,m3)*c_gd%dt_dyn
      end do
   end do
end if

end subroutine advance_aersclrbnd


!```````````````````````````````````````````````````````````````````
subroutine disp_var(m1, m2, m3, tgt_k, tgt_i, tgt_j, varp, vart, locn)

!+ Display desired values of variable(s) (debugging tool)

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: tstep, i_grid

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, tgt_k, tgt_i, tgt_j, locn
real(f4), dimension(m1,m2,m3), intent(IN) :: varp, vart

!== Local declarations:

! NONE

!=== Executable statements:

if (tgt_k > 0 .and. tgt_k < m1 .and. tgt_i > 0 .and. tgt_i < m2 .and.  &
     tgt_j > 0 .and. tgt_j < m3) then
   write (*, '(a,i4,i9,i4,e16.6,e16.6)')  &
        'grid,tstep,loc#,thp,tht', i_grid, tstep, locn,  &
        varp(tgt_k,tgt_i,tgt_j), vart(tgt_k,tgt_i,tgt_j)
else
   write (*, '(a,i4,i9,i4,e16.6,e16.6)')  &
        'grid,tstep: tgt point not applicable to this grid', i_grid, tstep
end if

write (*, '(2x,a,2x,4e16.6)') 'mvals', maxval(varp), minval(varp),  &
     maxval(vart), minval(vart)

end subroutine disp_var


!```````````````````````````````````````````````````````````````````
subroutine faux_CO2lth_add(m1, m2, m3, ia, iz, ja, jz, pp, pi0, thp, tht)

!+ Adjust theta in an attempt to partially account for CO2 latent heat
!  released by cloud formation (non-microphysical; doesn't alter PI)

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: c_gd
use body_specific_constants_mod, only: cpi, cpor, p00
use PRAMS_SBPS_mod, only: Tsat_CO2ice

implicit none

!=== Argument declarations:

integer, intent(IN) :: ia, iz, ja, jz, m1, m2, m3
real(f4), dimension(m1,m2,m3), intent(IN) :: pp, pi0

real(f4), dimension(m1,m2,m3), intent(INOUT) :: thp, tht

!== Local declarations:

integer :: k, i, j
real(f4) :: PIf_over_cp, press, tempk, Tsat_adj, new_thp

!=== Executable statements:

do j=ja,jz     ! Loop over all computational (horizontal) points
   do i=ia,iz  !

      do k=m1-1,2,-1   ! Loop from the top down...
         PIf_over_cp = (pp(k,i,j)+pi0(k,i,j))*cpi
         press = p00*PIf_over_cp**cpor
         if (press < 5._f4) then   ! Only perform where pressure is < 5 Pa
            Tsat_adj = Tsat_CO2ice(press)-10._f4 ! CO2 Tsat-10K for good measure
            tempk = thp(k,i,j)*PIf_over_cp
            
            ! If air possibly cold enough for CO2 clouds to form, assume
            !  the real_tempk = Tsat_adj (due to "unresolved" CO2 latent
            !  heating):
            if (tempk < Tsat_adj) then
               new_thp = Tsat_adj/PIf_over_cp  ! New potential temperature
               ! Adjust tht (for use by acoustic model):
               tht(k,i,j) = tht(k,i,j)+(new_thp-thp(k,i,j))/c_gd%dt_dyn
               ! Set thp:
               thp(k,i,j) = new_thp
            end if
         else
            exit   ! Exit k-loop (pressures below us should all be greater)
         end if
      end do

   end do
end do

end subroutine faux_CO2lth_add


end module tstep_driver_mod


!```````````````````````````````````````````````````````````````````
subroutine timestep

!+ Main timestep driver

#if defined(USE_MPI)
use univ_MPI_F90_intf_mod, only: MPI_buffer_packed_obj, mpiq_check_buffer_packed
#endif  /* end:USE_MPI */
use univ_run_timing_mod, only: run_timing_obj, record_wall_time, START, END,  &
     record_CPU_time, calc_elapsed_time, display_elapsed_time,  &
     initialize_elapsed_time
use main_memory_mod, only: a, i_grid, tstep, iscr1, c_gd,  &
     Cn_violation_threshold, initialization_type,  &
     run_is_parallel, mzp, mxp, myp, ia, iz, ja, jz, i0, j0, izu, jzv,  &
     my_rank, ibndc, n_PE_leaf, n_acoustic_lBC_optypes
use main_memory_bspec_mod, only: H2Ov_tog, CO2i_tog
use vtables_mod, only: ithp, itheta, irv, itht, ipp, ipi0
use aermic_rad_mem_mod, only: iaertog
use tstep_driver_mod, only: faux_co2lth_add, disp_var
use PRAMS_thermo_mod, only: dry_thermo

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

logical, parameter :: debug_this = .FALSE.
!logical, parameter :: debug_this = .TRUE. 

integer :: iq, disp_vi1, disp_vi2, tk, ti, tj, n

integer, dimension(n_PE_leaf,n_acoustic_lBC_optypes) :: rx_ireq, tx_ireq
#if defined(USE_MPI)
type(MPI_buffer_packed_obj), target,  &
     dimension(n_PE_leaf, n_acoustic_lBC_optypes) ::  &
     tx_buffer_packed, rx_buffer_packed
#else  /* Faux interface */
integer, target,  &
     dimension(n_PE_leaf, n_acoustic_lBC_optypes) :: tx_buffer_packed, rx_buffer_packed
#endif  /* end:USE_MPI */

logical, parameter :: do_CPU_timing = .FALSE.
integer, parameter :: n_tsteps_per_CPU_timing = 10
logical :: CPU_timing_this_tstep

type(run_timing_obj), dimension(19) :: CPU_timing

!=== Executable statements:

CPU_timing_this_tstep = ( do_CPU_timing .and.  &
     (mod(tstep, n_tsteps_per_CPU_timing) == 0 .and. i_grid == 1) )

!disp_vi1 = ithp
!disp_vi2 = itht
!tk = mzp-1
!ti = 29
!tj = 1

! Timestep driver for the hybrid non-hydrostatic time-split model:

if (CPU_timing_this_tstep) call initialize_elapsed_time(CPU_timing)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(1), START)
!#### timing
call tend0             ! Zero out all tendency arrays
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(1), END)
   CPU_timing(1)%description = 'tend0'
   call calc_elapsed_time(CPU_timing(1), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 1)
            
if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(2), START)
!#### timing
call RT_driver(mzp, mxp, myp, ia, iz, ja, jz)      ! Compute radiative transfer
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(2), END)
   CPU_timing(2)%description = 'RT_driver'
   call calc_elapsed_time(CPU_timing(2), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 2)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(3), START)
!#### timing
call sfclyr(mzp, mxp, myp, ia, iz, ja, jz, ibndc) ! Sub/super surface layer model
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(3), END)
   CPU_timing(3)%description = 'sfclyr'
   call calc_elapsed_time(CPU_timing(3), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 3)

if (run_is_parallel) then
#if defined(USE_MPI)
   call leafPE_longdt_lBC_transmit(tx_buffer_packed(:,1),  &
        rx_buffer_packed(:,1),  &   ! Send boundaries to adjoining leaf PEs
        tx_ireq(:,1), rx_ireq(:,1), n_PE_leaf)
#endif  /* end:USE_MPI */
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 4)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(4), START)
!#### timing
call coriolis(mzp,mxp,myp,i0,j0,ia,iz,ja,jz,izu,jzv)   ! Coriolis terms
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(4), END)
   CPU_timing(4)%description = 'coriolis'
   call calc_elapsed_time(CPU_timing(4), accumulate_elapsed_time=.TRUE.)
end if
if(debug_this) then
   call disp_var(mzp,mxp,myp,tk,ti,tj,a(disp_vi1),a(disp_vi2),5)
endif

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(5), START)
!#### timing
call advect_c('V', mzp, mxp, myp, ia, iz, ja, jz, izu, jzv, my_rank)  ! Advect velocities
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(5), END)
   CPU_timing(5)%description = 'advectV'
   call calc_elapsed_time(CPU_timing(5), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 6)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(6), START)
!#### timing
if (initialization_type == 2) then
   call data_assim       ! Nudging boundary condition
end if
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(6), END)
   CPU_timing(6)%description = 'data_assim'
   call calc_elapsed_time(CPU_timing(6), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 7)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(7), START)
!#### timing
if (c_gd%nxtnest >= 1) then
   call nstbdriv         ! Nested grid boundaries
end if
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(7), END)
   CPU_timing(7)%description = 'nest_bound'
   call calc_elapsed_time(CPU_timing(7), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 8)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(8), START)
!#### timing
call rayft              ! Rayleigh friction for theta
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(8), END)
   CPU_timing(8)%description = 'ray_friction'
   call calc_elapsed_time(CPU_timing(8), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 9)

if (run_is_parallel) then
#if defined(USE_MPI)
   call leafPE_longdt_lBC_receive(rx_buffer_packed(:,1), &
        tx_ireq(:,1),  &      ! Get the overlap region between leaf PEs
        rx_ireq(:,1), n_PE_leaf)
#endif  /* end:USE_MPI */
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 10)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(9), START)
!#### timing
call diffuse          ! Turbulent diffusion
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(9), END)
   CPU_timing(9)%description = 'diffuse'
   call calc_elapsed_time(CPU_timing(9), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 11)

!TIM  ??????
! 6/10/99 call cyclic here for grid 1 for parallel run 
!TIM  ??????

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(10), START)
!#### timing
call advect_c('T', mzp, mxp, myp, ia, iz, ja, jz, izu, jzv, my_rank)    ! Advect scalars
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(10), END)
   CPU_timing(10)%description = 'advectS'
   call calc_elapsed_time(CPU_timing(10), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 12)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(11), START)
!#### timing
call advance_scalars      ! Update scalars
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(11), END)
   CPU_timing(11)%description = 'advance_scalars'
   call calc_elapsed_time(CPU_timing(11), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 13)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(12), START)
!#### timing
call negadj(mzp, mxp, myp, i_grid, 'RV')
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(12), END)
   CPU_timing(12)%description = 'negadj'
   call calc_elapsed_time(CPU_timing(12), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 14)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(13), START)
!#### timing
call aer_micro(mzp, mxp, myp, ia, iz, ja, jz, my_rank)     ! Aerosol microphysics
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(13), END)
   CPU_timing(13)%description = 'aer_micro'
   call calc_elapsed_time(CPU_timing(13), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 15)

! Adjust theta in an attempt to partially account for CO2 latent heat released
!  by CO2 ice cloud formation (non-microphysical; doesn't alter PI):
if (CO2i_tog == 0) then
   call faux_CO2lth_add(mzp, mxp, myp, ia, iz, ja, jz, a(ipp), a(ipi0),  &
        a(ithp), a(itht))
end if

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(14), START)
!#### timing
if (H2Ov_tog > 0) then
   call dry_thermo(mzp, mxp, myp, ia, iz, ja, jz, a(ithp), a(itheta), a(irv),  &
        iaertog)                  ! Thermodynamic diagnosis
else
   call dry_thermo(mzp, mxp, myp, ia, iz, ja, jz, a(ithp), a(itheta),  &
        a(iscr1), iaertog)                  ! Thermodynamic diagnosis
end if
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(14), END)
   CPU_timing(14)%description = 'thermo'
   call calc_elapsed_time(CPU_timing(14), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 16)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(15), START)
!#### timing
call bound_scalars    ! Apply scalar boundary conditions
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(15), END)
   CPU_timing(15)%description = 'bnd_scalars'
   call calc_elapsed_time(CPU_timing(15), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 17)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(16), START)
!#### timing
call latbnd            ! Lateral boundaries (velocity)
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(16), END)
   CPU_timing(16)%description = 'latbnd'
   call calc_elapsed_time(CPU_timing(16), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 18)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(17), START)
!#### timing
call advance_velP(1)         ! Update u,v,w,p; First stage of Asselin filter
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(17), END)
   CPU_timing(17)%description = 'advance_velP1'
   call calc_elapsed_time(CPU_timing(17), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 19)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(18), START)
!#### timing
do iq=1,c_gd%n_dta_per_dtdyn
!   write (*,'(a,i2,a,i2)') 'acoustic pass #',iq,', grid #',grid
!   pause
   call acoustic(iq, tx_buffer_packed, rx_buffer_packed, tx_ireq,  &
        rx_ireq)          ! Acoustic model

   ! Calculate Courant number; CFL violation check:
   if (run_is_parallel) then
      call cfl(mzp, mxp, myp, c_gd%mi0, c_gd%mj0, .FALSE.)
   else
      call cfl(mzp, mxp, myp, 0, 0, .FALSE.)
   end if
!@   if (c_gd%Courant_number_xyz > Cn_violation_threshold) return
end do
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(18), END)
   CPU_timing(18)%description = 'acoustic'
   call calc_elapsed_time(CPU_timing(18), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 20)

if (CPU_timing_this_tstep) call record_CPU_time(CPU_timing(19), START)
!#### timing
call advance_velP(2)         ! Update u,v,w,p; last stage of Asselin filter
call bound_velP            ! Velocity/pressure boundary conditions
!#### timing
if (CPU_timing_this_tstep) then
   call record_CPU_time(CPU_timing(19), END)
   CPU_timing(19)%description = 'advance_velP2'
   call calc_elapsed_time(CPU_timing(19), accumulate_elapsed_time=.TRUE.)
end if
if (debug_this) call disp_var(mzp, mxp, myp, tk, ti, tj, a(disp_vi1),  &
     a(disp_vi2), 21)

! Display timings:
if (CPU_timing_this_tstep) call display_elapsed_time(CPU_timing,  &
     'Main timestep driver CPU timings:')

! Deallocate communications buffers:
if (run_is_parallel) then
#if defined(USE_MPI)
   do iq=1,n_acoustic_lBC_optypes
      do n=1,n_PE_leaf
         call mpiq_check_buffer_packed(rx_buffer_packed(n,iq), 0)
         call mpiq_check_buffer_packed(tx_buffer_packed(n,iq), 0)
      end do
   end do
#endif  /* end:USE_MPI */
end if

end subroutine timestep


!```````````````````````````````````````````````````````````````````
subroutine tend0

!+ Zero out all tendency arrays

use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: a, i_grid, nscl, naerscl, mxp, myp, mzp
use vtables_mod, only: iut, ivt, iwt, ipt, isct, iaersct
use grid_mem_mod, only: switch_to_Nscalar, switch_to_Ascalar

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: n, mxyzp1

!=== Executable statements:

mxyzp1 = mxp*myp*mzp-1

! U,V,P tendencies:

a(iut:iut+mxyzp1) = ZERO_f4
a(iut:ivt+mxyzp1) = ZERO_f4
a(iut:iwt+mxyzp1) = ZERO_f4
a(iut:ipt+mxyzp1) = ZERO_f4

! "Normal" scalar tendencies:

do n=1,nscl
   call switch_to_Nscalar(i_grid, n)
   a(isct:isct+mxyzp1) = ZERO_f4
end do

! "Aerosol" scalar tendencies:

do n=1,naerscl
   call switch_to_Ascalar(i_grid, n)
   a(iaersct:iaersct+mxyzp1) = ZERO_f4
end do

end subroutine tend0


!```````````````````````````````````````````````````````````````````
subroutine advance_scalars

!+ Step scalar variables from t to t+1

use main_memory_mod, only: a, i_grid, c_gd, nscl, naerscl, mxp, myp, mzp
use vtables_mod, only: iscp, isct, iaerscp, iaersct
use grid_mem_mod, only: switch_to_Nscalar, switch_to_Ascalar
use tstep_driver_mod, only: advance_aersclrbnd

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: mxyzp1, n

!=== Executable statements:

mxyzp1 = mxp*myp*mzp-1

! "Normal" scalars:

do n=1,nscl
   call switch_to_Nscalar(i_grid, n)
   a(iscp:iscp+mxyzp1) = a(iscp:iscp+mxyzp1)+a(isct:isct+mxyzp1)*c_gd%dt_dyn
end do

! "Aerosol" scalars:

do n=1,naerscl
   call switch_to_Ascalar(i_grid, n)
!###  *** Tendency (for computational pts) now added within aerosol model ***
!###   a(iaerscp:iaerscp+mxyzp1) = a(iaerscp:iaerscp+mxyzp1)+  &
!###        a(iaersct:iaersct+mxyzp1)*dtlt
   ! Add boundary tendencies (yes, most are non-zero):
   call advance_aersclrbnd(mzp, mxp, myp, a(iaerscp), a(iaersct))
end do

end subroutine advance_scalars


!```````````````````````````````````````````````````````````````````
subroutine advance_velP(action)

!+ Advance velocities and pressure from t to t+1; apply Asselin filter

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: a, dtlv, jdim, icorflg, prognose_uvwp, mxp, myp, mzp
use vtables_mod, only: iuc, iup, iut, ivt3da, ivc, ivp, ivt, iwc, iwp, iwt,  &
     ipc, ipp, ipt

implicit none

!=== Argument declarations:

integer, intent(IN) :: action

!== Local declarations:

integer :: mxyzp
real(f4) :: eps

!=== Executable statements:

if (.not.prognose_uvwp) return   ! Prognostic u, v, w, p not desired

! It is here that the Asselin filter is applied.  For the velocities
!  and pressure, this must be done in two stages, the first when
!  action=1 and the second when action=2.

mxyzp = mxp*myp*mzp
eps = 0.2_f4

! For both action=1 and action=2, call PREDICT for U, V, W, and P.

call predict(mxyzp, a(iuc), a(iup), a(iut), a(ivt3da), action, dtlv, eps)

if (icorflg >= 1 .or. jdim == 1) call predict(mxyzp, a(ivc), a(ivp), a(ivt),  &
     a(ivt3da), action, dtlv, eps)

call predict(mxyzp, a(iwc), a(iwp), a(iwt), a(ivt3da), action, dtlv, eps)
call predict(mxyzp, a(ipc), a(ipp), a(ipt), a(ivt3da), action, dtlv, eps)

end subroutine advance_velP


!```````````````````````````````````````````````````````````````````
subroutine predict(npts, ac, ap, fa, af, action, dtlp, eps)

!+ Advance variable from t to t+1; apply Asselin filter

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: c_gd, i_grid, run_is_parallel

implicit none

!=== Argument declarations:

integer, intent(IN) :: npts, action
real(f4), intent(IN) :: dtlp, eps
real(f4), dimension(npts), intent(IN) :: fa

real(f4), dimension(npts), intent(INOUT) :: ac, ap, af

!== Local declarations:

real(f4) :: epsu
integer :: m

!=== Executable statements:

! For IAC=3, this routine moves the arrays AC and AP forward by
!  1 time level by adding in the prescribed tendency. It also
!  applies the Asselin filter given by:

!              {AC} = AC + EPS * (AP - 2 * AC + AF)

!     where AP,AC,AF are the past, current and future time levels of A.
!     All IAC=1 does is to perform the {AC} calculation without the AF
!     term present.  IAC=2 completes the calculation of {AC} by adding
!     the AF term only, and advances AC by filling it with input AP
!     values which were already updated in ACOUSTC.

epsu = eps
if (c_gd%ngbegun == 0) epsu = 0.5_f4

if (action == 1) then
   do m=1,npts
      ac(m) = ac(m) + epsu * (ap(m) - 2._f4 * ac(m))
   end do
   return
else if (action == 2) then
   do m = 1,npts
      af(m) = ap(m)
      ap(m) = ac(m) + epsu * af(m)
   end do
else if (action == 3) then
   do m = 1,npts
      af(m) = ap(m) + dtlp * fa(m)
   end do
   if (i_grid == 1 .and. .NOT.run_is_parallel) call cyclic(c_gd%nzp,  &
        c_gd%nxp, c_gd%nyp, af, 'T')
   do m = 1,npts
      ap(m) = ac(m) + epsu * (ap(m) - 2._f4 * ac(m) + af(m))
   end do
end if

do m = 1,npts
   ac(m) = af(m)
end do

end subroutine predict
