!```````````````````````````````````````````````````````````````````
module varfile_init_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine varfile_find

!+ Searches for varfiles to use during run

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use univ_filelist_mod, only : filelist, filelist_count
use univ_std_fileIO_mod, only: find_file_IOunit
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use main_memory_mod, only: input_varfile_pfx, n_varf, varfile_name, vtime,  &
     n_varftot, current_sim_ET, manage_misc_memory
use body_specific_constants_mod, only: mnsdy

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'varfile_find'

character(len=MAX_FILE_PATH_LEN) :: fpref
integer :: nc, nf, vfnum, nvfiles, junki, IO_unit
real(f4) :: vfLs, vfUT, vfLs0, vfUT0, vftinc

!=== Executable statements:

! Construct varfile header prefix search string:

nc = len_trim(input_varfile_pfx)
fpref = input_varfile_pfx(1:nc)//'-V-'
nc = len_trim(fpref)+1
fpref(nc:) = '*.tag'

! Find, count, and store all filenames that match the prefix search string:

nvfiles = 0
call filelist_count(fpref, nvfiles)
if (nvfiles == 0) then
   write (*, '(/,a,/,6x,a)') 'INFO: No varfile header found with form:',  &
        fpref(1:len_trim(fpref))
   n_varf = 0
   return    ! Exit, since nothing else to do (without any varfiles)
else
   n_varftot = nvfiles
   call manage_misc_memory(dim_n_varftot=.TRUE.)
   call filelist(varfile_name, fpref, nvfiles)
end if

IO_unit = find_file_IOunit()

! Open each varfile header, and read its time variables:

open(IO_unit, file=varfile_name(1), form='FORMATTED', status='OLD')
read (IO_unit, *) junki, vfLs0, vfUT0
close(IO_unit)

write (*, '(/,a,/)') 'Varfile#   ID#      Ins_time     time_inc    Ls         UT'

do nf=1,nvfiles
   
   open(IO_unit, file=varfile_name(nf), form='FORMATTED', status='OLD')
   read (IO_unit, *) junki, vfLs, vfUT, vfnum
   close(IO_unit)

   ! Calculate varfile time increment:

   if (vfUT0 > vfUT) then
      vftinc = vfUT-(vfUT0-24._f4)
   else
      vftinc = vfUT-vfUT0
   end if
   
   vfUT0 = vfUT   ! Save current time for next iteration

   ! Compute insertion time for each varfile (in seconds):

   vtime(nf) = real(vfnum)*vftinc*real(mnsdy, kind=f4)/24._f4

   ! Remove the last seven characters from varfile (i.e., remove "-g1.vfm"):
!   fpref = varfile_name(nf)
!   nc = len_trim(fpref)
!   varfile_name(nf) = fpref(1:nc-7)

   write (*, '(2x,i5,2x,i5,2x,f15.2,2x,f6.2,2x,f9.3,2x,f8.3)') nf, vfnum,  &
        vtime(nf), vftinc, vfLs, vfUT
   
end do
write (*, '(/)')

! Find starting varfile (primarily for a state restart):

find_start: do nf=1,nvfiles
   if (real(current_sim_ET, kind=f4)+ONE_f4 > vtime(nf) .and.  &
        real(current_sim_ET, kind=f4) < vtime(nf+1)) exit find_start
end do find_start
n_varf = nf

end subroutine varfile_find


!```````````````````````````````````````````````````````````````````
subroutine refstate_loc(n2, n3, igd, topt, rtgt, iq_ref, init_topref)

!+ Determine location/grid of a 1D reference state

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: topref, iref, jref, igdref, rtgref

implicit none

!=== Argument declarations:

integer, intent(IN) :: n2, n3, igd, iq_ref
logical, intent(IN) :: init_topref
real(f4), dimension(n2,n3), intent(IN) :: topt, rtgt

!== Local declarations:

integer :: i, j

!=== Executable statements:

! Determine location/grid of 1D reference state (will be taken to be the point
!  with the lowest topography):

if (init_topref) topref(iq_ref) = 1.e10_f4
do j=1,n3
   do i=1,n2                  ! Find point on this grid with lowest topo
      if (topt(i,j) < topref(iq_ref)) then
         iref(iq_ref) = i; jref(iq_ref) = j; igdref(iq_ref) = igd
         rtgref(iq_ref) = rtgt(i,j)
         topref(iq_ref) = topt(i,j)
      end if
   end do
end do

end subroutine refstate_loc


!```````````````````````````````````````````````````````````````````
subroutine varfile_refstate(n1, n2, n3, igd_ref, igd, thp, pp, rv, up, vp,  &
     iq_ref)

!+ Calculates a 1D atmospheric reference state for PIPP to use

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: rtgref, iref, jref, gd
use main_memory_bspec_mod, only: H2Ov_tog, gd_bd
use body_specific_constants_mod, only: cp, p00, R_air, cvor

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, igd, igd_ref, iq_ref
real(f4), dimension(n1,n2,n3), intent(IN) :: thp, pp, rv, up, vp

!== Local declarations:

integer :: k
real(f4) :: c1

!=== Executable statements:

! Compute 1D reference state (`refstate_loc` must be called one or more times
!  before this routine to determine the reference state location):

! Heights (AGL) of reference point "sounding":
!%do k=1,n1
!%   vctr2(k) = gd(igd_ref)%zt(k)*rtgref(iq_ref)
!%end do

! Fill u,v velocity components' 1D refstates (directly from refpt "sounding"):
!
!  -- used *ONLY* in Rayleigh friction and/or Coriolis calcs (i.e., when in HH
!     mode), where they are assumed to be valid on pure sigma-z levels
!     [zt(*), no stretching (rtgt) applied]
!  -- effectively strengthens vertical wind gradients above high
!     topography, but will maintain low windspeeds near the ground, high
!     windspeeds aloft

do k=1,n1
   gd(igd)%u01d(k) = up(k,iref(iq_ref),jref(iq_ref))
   gd(igd)%v01d(k) = vp(k,iref(iq_ref),jref(iq_ref))
end do
!call htint2(n1,up(1,iref,jref),vctr2,n1,u01dn(1,ng),ztn(1:n1,ng))
!call htint2(n1,vp(1,iref,jref),vctr2,n1,v01dn(1,ng),ztn(1:n1,ng))

! Fill moisture mixing ratio 1D refstate (directly from refpt "sounding"):

if (H2Ov_tog >= 1) then
!@   call htint2(n1,rv(1,iref,jref),vctr2,n1,rt01dn(1,ng),ztn(1:n1,ng))
   do k=1,n1
      gd_bd(igd)%rt01d(k) = rv(k,iref(iq_ref),jref(iq_ref))
   end do
else
   do k=1,n1
      gd_bd(igd)%rt01d(k) = ZERO_f4
   end do
end if

! Fill potential temperature and Exner function 1D refstates (directly from
!  refpt "sounding"):

!@call htint2(n1,thp(1,iref,jref),vctr2,n1,vctr1,ztn(1:n1,ng))
do k=1,n1
   gd(igd)%th01d(k) = thp(k,iref(iq_ref),jref(iq_ref))   !# *(1.+0.61*rt01dn(k,ng))
   gd(igd)%pi01d(k) = pp(k,iref(iq_ref),jref(iq_ref))
end do

gd(igd)%u01d(1) = gd(igd)%u01d(2)    !
gd(igd)%v01d(1) = gd(igd)%v01d(2)    ! To be consistent with model BCs???
gd_bd(igd)%rt01d(1) = gd_bd(igd)%rt01d(2)  !
gd(igd)%th01d(1) = gd(igd)%th01d(2)  !

! Fill density 1D refstate (from th01dn and pi01dn):

c1 = p00/(R_air*(cp**cvor))
do k=1,n1
   gd(igd)%dn01d(k) = c1*(gd(igd)%pi01d(k)**cvor)/gd(igd)%th01d(k)
end do

end subroutine varfile_refstate


!```````````````````````````````````````````````````````````````````
subroutine adjust_varstate(n1, n2, n3, igd, thp, pp, pi0, th0, dn0, rtgt)

!+ Adjusts atmospheric reference state to be more hydrostatically balanced
!   (in the linearized, numerical sense)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use main_memory_mod, only: vctr1, vctr2, vctr3, vctr4, vctr5, vctr6, gd,  &
     vctr7, vctr8, vctr9, vctr10
use body_specific_constants_mod, only: cvor, g_Xoid

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, igd
real(f4), dimension(n1,n2,n3), intent(IN) :: pp, pi0, th0, dn0
real(f4), dimension(n2,n3), intent(IN) :: rtgt

real(f4), dimension(n1,n2,n3), intent(INOUT) :: thp

!== Local declarations:

integer :: i, j, k

!=== Executable statements:

do j=1,n3
   do i=1,n2

      do k=1,n1
         vctr1(k) = pp(k,i,j)
!         vctr1(k) = pp(k,i,j)-pi0(k,i,j)      ! pi'(k)
!         print *,k,vctr1(k),pp(k,i,j),pi0(k,i,j)
      end do

      do k=1,n1-1
         vctr2(k) = (th0(k,i,j)+th0(k+1,i,j))*0.5_f4   ! th0(m)
         vctr3(k) = (thp(k,i,j)+thp(k+1,i,j))*0.5_f4   ! thp(m)
         vctr4(k) = (vctr1(k)+vctr1(k+1))*0.5_f4       ! pi'(m)
         vctr5(k) = (pi0(k,i,j)+pi0(k+1,i,j))*0.5_f4   ! pi0(m)
         vctr6(k) = (vctr1(k+1)-vctr1(k))*gd(igd)%dzm(k)/rtgt(i,j)  ! [dpi'/dz](m)
         !         print *,k,vctr2(k),vctr3(k),vctr4(k),vctr5(k),vctr6(k)
      end do
     
      do k=1,n1-1
         vctr7(k) = vctr2(k)*vctr6(k)/g_Xoid
         vctr8(k) = cvor*vctr4(k)/vctr5(k)
!         print *,'dfds',k,vctr2(k),vctr7(k),vctr8(k),(vctr7(k)+vctr8(k)+1.)  &
!              ,vctr4(k),vctr5(k),vctr6(k),vctr3(k)-vctr2(k),vctr2(k)
         vctr10(k) = vctr2(k)*(vctr7(k)+vctr8(k)+ONE_f4)   ! New thp(m)
      end do

!      vctr10(n1) = thp(n1,i,j)
!      print *,'thp',n1,i,j,thp(n1,i,j),vctr2(n1)*(vctr7(n1)+vctr8(n1)+1.)  &
!           ,vctr10(n1)!,vctr10(n1)-thp(n1,i,j)
!      do k=n1-1,1,-1
!         vctr10(k) = 2.*vctr2(k)*(vctr7(k)+vctr8(k)+1.)-vctr10(k+1)
!         print *,'thp',k,i,j,thp(k,i,j),vctr2(k)*(vctr7(k)+vctr8(k)+1.)  &
!              ,vctr10(k)!,vctr10(k)-thp(k,i,j)
!      enddo

!@      vctr9(1) = thp(1,i,j)
!@      vctr9(n1) = thp(n1,i,j)
      do k=2,n1-1
!@         vctr9(k) = thp(k,i,j)
         thp(k,i,j) = (vctr10(k-1)+vctr10(k))*0.5_f4   ! New thp(k)
      end do
      thp(1,i,j) = thp(2,i,j)
      thp(n1,i,j) = thp(n1-1,i,j)+(thp(n1-1,i,j)-thp(n1-2,i,j)) 


!@      do k=1,n1
!@         print *,'thp',k,i,j,vctr9(k),thp(k,i,j),vctr10(k)  &
!@      ,thp(k,i,j)-vctr9(k)
!@      enddo
!@      pause

   end do
end do

end subroutine adjust_varstate


!```````````````````````````````````````````````````````````````````
subroutine varweights(n1, n2, n3, igd, varwts, topt, rtgt)

!+ Compute weights for varfile nudging

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL
use main_memory_mod, only: gd, ztop, tnudlat, tnudtop, tnudcent, znudtop, nudlat

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3, igd
real(f4), dimension(n2,n3), intent(IN) :: topt, rtgt

real(f4), dimension(n1,n2,n3), intent(INOUT) :: varwts

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'varweights'

integer :: i, j, k
real(f4) :: tnudcenti, tnudtopi, tnudlati, delzi, rown, rows, rowe, roww,  &
     wtlat, zloc, wttop

!=== Executable statements:

if (nudlat <= 0) return   ! No nudging desired, so no weighting

tnudcenti = ZERO_f4
if (tnudcent > 0.01_f4) tnudcenti = ONE_f4/tnudcent

tnudtopi = ZERO_f4
if (tnudtop > 0.01_f4) tnudtopi = ONE_f4/tnudtop-tnudcenti

tnudlati = ZERO_f4
if (tnudlat > 0.01_f4) tnudlati = ONE_f4/tnudlat-tnudcenti

if (ztop > znudtop) then
   delzi = ONE_f4/(ztop-znudtop)
else if (tnudtop > 0.01_f4) then
   write (*, *) 'Incorrect specification of ZNUDTOP (must be < ztop):'
   write (*, *) ' znudtop = ', znudtop, '; ztop = ', ztop
   call exception(procedure_name, FATAL)
end if

do j=1,n3
   do i=1,n2

      ! Quadratic weighting function for lateral boundaries:

      rown = max(ZERO_f4, real(j+nudlat-n3, kind=f4))
      rows = max(ZERO_f4, real(nudlat+1-j, kind=f4))
      rowe = max(ZERO_f4, real(i+nudlat-n2, kind=f4))
      roww = max(ZERO_f4, real(nudlat+1-i, kind=f4))
      wtlat = max(rown*rown, rows*rows, rowe*rowe, roww*roww)/  &
           real(nudlat*nudlat, kind=f4)

      ! Linear weighting function for top boundary:

      do k=1,n1
         zloc = gd(igd)%zt(k)*rtgt(i,j)+topt(i,j)
         wttop = max(ZERO_f4, (zloc-znudtop)*delzi)

         ! Full 3D weighting function values:

         varwts(k,i,j) = tnudcenti+max(tnudlati*wtlat, tnudtopi*wttop)

      end do

   end do
end do

end subroutine varweights


!```````````````````````````````````````````````````````````````````
subroutine vartendp(n1, n2, n3, pp1, varp)

!+ Fills k=1 pressure array ???

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1, n2, n3
real(f4), dimension(n1,n2,n3), intent(IN) ::  pp1

real(f4), dimension(n2,n3), intent(OUT) :: varp

!== Local declarations:

integer :: i, j

!=== Executable statements:

do j=1,n3
   do i=1,n2
      varp(i,j) = pp1(1,i,j)
   end do
end do

end subroutine vartendp


!```````````````````````````````````````````````````````````````````
subroutine nudge(m1, m2, m3, ia, iz, ja, jz, ibndc, varwts, varup, varvp,  &
     vartp, varrp, varpp, varuf, varvf, vartf, varrf, varpf, up, vp, theta,  &
     rv, pp, ut, vt, tht, rvt, pt, varp, varp2, pi0, rtgt)

!+ Nudge fields as configuration dictates

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: vctr1, vctr2, vctr3, vctr4, vctr5, vctr10, vctr11,  &
     jdim, vtime, n_varf, current_sim_ET
use main_memory_bspec_mod, only: H2Ov_tog, H2Ov_ivr

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, ibndc
real(f4), dimension(m1,m2,m3), intent(IN) :: varup, varvp, vartp, varrp,  &
     varuf, varvf, vartf, varrf, varwts, pi0, up, vp, theta, rv, pp
real(f4), dimension(m2,m3), intent(IN) :: varp, varp2, rtgt

real(f4), dimension(m1,m2,m3), intent(INOUT) :: varpp, varpf, ut, vt, tht,  &
     rvt, pt

!== Local declarations:

integer :: il, ir, jl, jr, i, j, k
real(f4) :: tfact

!=== Executable statements:

! Compute hydrostatic "observed" exner function at early and late
!  data times:

call varhyd(m1, m2, m3, 1, m2, 1, m3, vartp, varrp, varp, varpp, pi0, rtgt)
call varhyd(m1, m2, m3, 1, m2, 1, m3, vartf, varrf, varp2, varpf, pi0, rtgt)

! Set bounds for nudging this sub-domain:

il = ia
ir = iz
jl = ja
jr = jz

! West and east boundaries:

if (iand(ibndc, 1) /= 0) il = 1
if (iand(ibndc, 2) /= 0) ir = m2

! South and north boundaries:

if (jdim == 1) then
   if (iand(ibndc, 4) /= 0) jl = 1
   if (iand(ibndc, 8) /= 0) jr = m3
end if

! Linearly interpolate values in time, then nudge:

tfact = (real(current_sim_ET, kind=f4)-vtime(n_varf))/  &
     (vtime(n_varf+1)-vtime(n_varf))
do j=jl,jr
   do i=il,ir

      do k=1,m1
         vctr1(k) = varup(k,i,j)+(varuf(k,i,j)-varup(k,i,j))*tfact
         vctr2(k) = varvp(k,i,j)+(varvf(k,i,j)-varvp(k,i,j))*tfact
         vctr3(k) = vartp(k,i,j)+(vartf(k,i,j)-vartp(k,i,j))*tfact
         vctr4(k) = varrp(k,i,j)+(varrf(k,i,j)-varrp(k,i,j))*tfact
         vctr5(k) = varpp(k,i,j)+(varpf(k,i,j)-varpp(k,i,j))*tfact
         
         vctr10(k) = (varwts(k,i,j)+varwts(k,min(m2,i+1),j))*0.5_f4
         vctr11(k) = (varwts(k,i,j)+varwts(k,i,min(m3,j+jdim)))*0.5_f4
      end do

      do k=1,m1
         ut(k,i,j) = ut(k,i,j)+vctr10(k)*(vctr1(k)-up(k,i,j))
         vt(k,i,j) = vt(k,i,j)+vctr11(k)*(vctr2(k)-vp(k,i,j))
         tht(k,i,j) = tht(k,i,j)+varwts(k,i,j)*(vctr3(k)-theta(k,i,j))
         pt(k,i,j) = pt(k,i,j)+varwts(k,i,j)*(vctr5(k)-pp(k,i,j))
      end do

      if (H2Ov_tog >= 1) then
         if (H2Ov_ivr == 0) then   ! Nudge RV field
            do k=1,m1
               rvt(k,i,j) = rvt(k,i,j)+varwts(k,i,j)*(vctr4(k)-rv(k,i,j))
            end do
         else     ! No RV nudging (no "wet" MGCM fields)
            do k=1,m1
               rvt(k,i,j) = rvt(k,i,j) ! Temporary for now, since no RV in MGCM
            end do
         end if
      end if

   end do
end do

end subroutine nudge


!```````````````````````````````````````````````````````````````````
subroutine varhyd(m1, m2, m3, ia, iz, ja, jz, VTH, VRT, VP1, VPIEX, PI0K, RTG)

!+ Compute hydrostatic "observed" Exner function

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use main_memory_mod, only: vctr1, vctr2, c_gd
use body_specific_constants_mod, only: g_Xoid

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: VTH, VRT, PI0K
real(f4), dimension(m2,m3), intent(IN) :: rtg, vp1

real(f4), dimension(m1,m2,m3), intent(INOUT) :: VPIEX

!== Local declarations:

integer :: i, j, k
real(f4) :: c1

!=== Executable statements:

DO j=1,m3
   do i=1,m2
      C1 = g_Xoid*RTG(i,j)*2._f4
      DO K=1,m1
         VCTR2(K) = VTH(K,i,j)*(ONE_f4+0.61_f4*VRT(K,i,j))
      END DO
      DO K=1,m1-1
         VCTR1(K) = C1/(c_gd%DZM(K)*(VCTR2(K)+VCTR2(K+1)))
      END DO

      VPIEX(1,i,j) = VP1(i,j)+PI0K(1,i,j)
      DO K=2,m1
         VPIEX(K,i,j) = VPIEX(K-1,i,j)-VCTR1(K-1)
      END DO
      DO K=1,m1
         VPIEX(K,i,j) = VPIEX(K,i,j)-PI0K(K,i,j)
      END DO
   END DO
END DO

end subroutine varhyd


end module varfile_init_mod


!```````````````````````````````````````````````````````````````````
subroutine initv(igd, want_to_init_fields, init_master_refstate)

!+ "Variable init" initialization routine

use univ_kind_defs_mod, only: f4
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_PROC_NAME_LEN
use univ_std_fileIO_mod, only: find_file_IOunit, determine_IO_reclen
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use main_memory_mod, only: varfile_name, n_varf, gd, iscr1, iscr2, a, vtime,  &
     n_varftot, tnudcent, tnudtop
use main_memory_bspec_mod, only: H2Ov_tog, dust_nb, dust_tog, dust_ivr,  &
     H2Ov_ivr
use vtables_mod, only: iucn, ivcn, ipcn, ithpn, ivarupn, ivarvpn, ivartpn,  &
     ivarrpn, ivarwtsn, itoptn, irtgtn, ipi0n, ith0n, idn0n, idn0un, idn0vn,  &
     ivarpn, ivarp2n, ivarufn, ivarvfn, ivartfn, ivarrfn, irvn, idustncn,  &
     ivt3db
use state_IO_mod, only: cfgctl_rw
use varfile_init_mod, only: varfile_refstate, varweights, vartendp
use PRAMS_init_mem_mod, only: refs3d

implicit none

!=== Argument declarations:

integer, intent(IN) :: igd
logical, intent(IN) :: want_to_init_fields, init_master_refstate

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'initv'

character(len=MAX_FILE_PATH_LEN) :: flnm
character(len=3) :: cgrid
logical :: will_nudge
integer :: nc, icm, ivvu, ivvv, ivvp, ivvt, ivvr, n1, n2, n3, i, ib, IO_unit,  &
     IO_reclen, e
real(f4), allocatable, dimension(:,:) :: reclen_test

!=== Executable statements:

if (n_varf == 0) call exception(procedure_name, FATAL,  &
     'Unable to proceed further without any varfiles.')

! If initial time, read in varfile fields; if any other time, see if nudging
!  is desired on this grid (only nested grids have a choice):

icm = gd(igd)%nxtnest

if (icm > 0 .and. tnudcent+tnudtop < 0.001_f4) then
   will_nudge = .FALSE.
else
   will_nudge = .TRUE.
end if
   
if (.not.(want_to_init_fields .or. will_nudge)) return  ! Nothing else to do

! Where do we put first varfile fields?
   
if (want_to_init_fields) then
   ivvu = iucn(igd)
   ivvv = ivcn(igd)
   ivvp = ipcn(igd)
   ivvt = ithpn(igd)
   if (H2Ov_tog > 0) then
      ivvr = irvn(igd)
   else
      ivvr = ivt3db  ! Used only as a valid a-array index for the call to
                     !  `varfile_refstate` (no I/O in this use case)
   end if
else
   ivvu = ivarupn(igd)
   ivvv = ivarvpn(igd)
   ivvp = iscr1
   ivvt = ivartpn(igd)
   ivvr = ivarrpn(igd)
end if
   
! First varfile: check compatibility; read in fields:

nc = len_trim(varfile_name(n_varf))         ! Create current varfile header name
flnm = varfile_name(n_varf)(1:nc-4)//'.hdr' !
nc = len_trim(flnm)                         !

IO_unit = find_file_IOunit()

open(IO_unit, file=flnm(1:nc), form='unformatted', status='old')
call cfgctl_rw(IO_unit, 'R', 'Q1V') ! Check for compatibility with current config
call cfgctl_rw(IO_unit, 'R', 'Q2V') !
write (*, '(a)') '-- Varfile is compatible with current config...'
close(IO_unit)

write (cgrid, '(a1,i2.2)') 'g', igd         !
nc = len_trim(varfile_name(n_varf))         ! Create current varfile name
flnm = varfile_name(n_varf)(1:nc-4)//'-'//cgrid//'.vfm'  !
nc = len_trim(flnm)                         !

write (*, '(3a,/,a,f14.3,a)') 'Opening varfile [ ', flnm(1:nc), ' ]',  &
     '   configured insertion time = ', vtime(n_varf), ' seconds'

allocate(reclen_test(gd(igd)%nxyzp,1), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'reclen1')
IO_reclen = determine_IO_reclen(reclen_test)
deallocate(reclen_test, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'reclen1')

open(IO_unit, file=flnm(1:nc), form='unformatted', status='old',  &
     access='direct', recl=IO_reclen)

n1 = gd(igd)%nzp; n2 = gd(igd)%nxp; n3 = gd(igd)%nyp

read (IO_unit, rec=1) a(ivvu:ivvu+n1*n2*n3-1)
!call binDA_rw3Df(2,1,n1,n2,n3,a(ivvu),'R')                  !
read (IO_unit, rec=2) a(ivvv:ivvv+n1*n2*n3-1)
!call binDA_rw3Df(2,2,n1,n2,n3,a(ivvv),'R')                  !
read (IO_unit, rec=3) a(ivvp:ivvp+n1*n2*n3-1)
!call binDA_rw3Df(2,3,n1,n2,n3,a(ivvp),'R')                  !
read (IO_unit, rec=4) a(ivvt:ivvt+n1*n2*n3-1)
!call binDA_rw3Df(2,4,n1,n2,n3,a(ivvt),'R')                  ! Read data
if (H2Ov_tog > 0 .and. H2Ov_ivr == 0) then                   !
   read (IO_unit, rec=5) a(ivvr:ivvr+n1*n2*n3-1)
!   call binDA_rw3Df(2,5,n1,n2,n3,a(ivvr),'R')               !
end if                                                       !
if (want_to_init_fields .and. dust_tog > 0 .and. dust_ivr == 0) then                   !
   do ib=1,dust_nb                                          !
      read (IO_unit, rec=5+ib) a(idustncn(igd,ib):idustncn(igd,ib)+n1*n2*n3-1)
!      call binDA_rw3Df(2,5+ib,n1,n2,n3,a(idustncn(ng,ib)),'R')  !
   end do                                                    !
end if                                                       !

close(IO_unit)
   
! Compute varfile weighting factors for nudging:
   
if (icm == 0) then   ! Only if not a nested grid
   call varweights(n1, n2, n3, igd, a(ivarwtsn(igd)), a(itoptn(igd)),  &
        a(irtgtn(igd)))
end if

! Compute the model reference state:
   
if (want_to_init_fields .and. init_master_refstate) then  ! Init ref state
   call varfile_refstate(n1, n2, n3, igd, 1, a(ivvt), a(ivvp), a(ivvr),  &
        a(ivvu), a(ivvv), 1)
   ! Compute 3D reference state from 1D reference state (grid #1):
   call refs3D(gd(1)%nzp, gd(1)%nxp, gd(1)%nyp, 1, a(ipi0n(1)),  &
        a(idn0n(1)), a(idn0un(1)), a(idn0vn(1)), a(ith0n(1)),  &
        a(itoptn(1)), a(irtgtn(1)), 1)
   return   ! Done
end if

do i=0,gd(igd)%nxyzp-1
   a(ivvp+i) = a(ivvp+i)-a(ipi0n(igd)+i)
end do

! Ensure that no vertical accelerations will be induced by fields:
!call adjust_varstate(n1,n2,n3,ng,a(ivvt),a(ivvp),a(ipi0n(ng)),a(ith0n(ng))  &
!     ,a(idn0n(ng)),a(irtgtn(ng)))  

if (will_nudge) then
   call vartendp(n1, n2, n3, a(ivvp), a(ivarpn(igd)))
end if

! Read and process next varfile:
!  (also, only need it for nested grids if nudging will be done)

if (will_nudge) then

   if (n_varf+1 > n_varftot) call exception(procedure_name, FATAL,  &
        'Ran out of varfiles (too few to complete run).')

   nc = len_trim(varfile_name(n_varf+1))         ! Current varfile header name
   flnm = varfile_name(n_varf+1)(1:nc-4)//'.hdr' !
   nc = len_trim(flnm)                           !

   open(IO_unit, file=flnm(1:nc), form='unformatted', status='old')
   call cfgctl_rw(IO_unit, 'R', 'Q1V') ! Check for compatibility with current
   call cfgctl_rw(IO_unit, 'R', 'Q2V') !  configuration
   write (*, '(a)') '-- Varfile is compatible with current config...'
   close(IO_unit)

   write (cgrid, '(a1,i2.2)') 'g', igd           !
   nc = len_trim(varfile_name(n_varf+1))         ! Create current varfile name
   flnm = varfile_name(n_varf+1)(1:nc-4)//'-'//cgrid//'.vfm'  !
   nc = len_trim(flnm)                           !

   write (*, '(3a,/,a,f14.3,a)') 'Opening varfile [ ', flnm(1:nc), ' ]',  &
        '   configured insertion time = ', vtime(n_varf+1), ' seconds'

   allocate(reclen_test(gd(igd)%nxyzp,1), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'reclen2')
   IO_reclen = determine_IO_reclen(reclen_test)
   deallocate(reclen_test, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'reclen2')

   open(IO_unit, file=flnm(1:nc), form='unformatted', status='old',  &
        access='direct', recl=IO_reclen)

   n1 = gd(igd)%nzp; n2 = gd(igd)%nxp; n3 = gd(igd)%nyp

   read (IO_unit, rec=1) a(ivarufn(igd):ivarufn(igd)+n1*n2*n3-1)
!   call binDA_rw3Df(2,1,n1,n2,n3,a(ivarufn(ng)),'R')            !
   read (IO_unit, rec=2) a(ivarvfn(igd):ivarvfn(igd)+n1*n2*n3-1)
!   call binDA_rw3Df(2,2,n1,n2,n3,a(ivarvfn(ng)),'R')            !
   read (IO_unit, rec=3) a(iscr2:iscr2+n1*n2*n3-1)
!   call binDA_rw3Df(2,3,n1,n2,n3,a(iscr2),'R')                  !
   read (IO_unit, rec=4) a(ivartfn(igd):ivartfn(igd)+n1*n2*n3-1)
!   call binDA_rw3Df(2,4,n1,n2,n3,a(ivartfn(ng)),'R')            ! Read data
   if (H2Ov_tog >= 1 .and. H2Ov_ivr == 0) then                   !
      read (IO_unit, rec=5) a(ivarrfn(igd):ivarrfn(igd)+n1*n2*n3-1)
!      call binDA_rw3Df(2,5,n1,n2,n3,a(ivarrfn(ng)),'R')         !
   end if                                                        !
   if (dust_tog > 0 .and. dust_ivr == 0) then                    !
      do ib=1,dust_nb                                           !
!%         call binDA_rw3Df(2,5+ib,n1,n2,n3,a(idustncn(ng,ib)),'R') !
      end do                                                     !
   end if                                                        !
   
   close(IO_unit)

   do i=0,gd(igd)%nxyzp-1
      a(iscr2+i) = a(iscr2+i)-a(ipi0n(igd)+i)
   end do

   ! Ensure that no vertical accelerations will be induced by fields:
!   call adjust_varstate(n1,n2,n3,ng,a(ivartfn(ng)),a(iscr2),a(ipi0n(ng))  &
!        ,a(ith0n(ng)),a(idn0n(ng)),a(irtgtn(ng)))
   
   call vartendp(n1, n2, n3, a(iscr2), a(ivarp2n(igd)))
   
end if

! Finally, if this is an initialization and we are nudging, put
!   first varfile's data into nudging arrays:

if (want_to_init_fields .and. will_nudge) then
   a(ivarupn(igd):ivarupn(igd)+gd(igd)%nxyzp-1) = a(ivvu:ivvu+gd(igd)%nxyzp-1)
   a(ivarvpn(igd):ivarvpn(igd)+gd(igd)%nxyzp-1) = a(ivvv:ivvv+gd(igd)%nxyzp-1)
   a(ivartpn(igd):ivartpn(igd)+gd(igd)%nxyzp-1) = a(ivvt:ivvt+gd(igd)%nxyzp-1)
   a(ivarrpn(igd):ivarrpn(igd)+gd(igd)%nxyzp-1) = a(ivvr:ivvr+gd(igd)%nxyzp-1)
end if

end subroutine initv


!```````````````````````````````````````````````````````````````````
subroutine data_assim

!+ Runtime data assimilation ("nudging") driver

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: nudlat, tnudcent, tnudtop, c_gd, tnudlat, a,  &
     iscr1, iscr2, mzp, mxp, myp, ia, iz, ja, jz, ibndc
use main_memory_bspec_mod, only: H2Ov_tog 
use vtables_mod, only: ivarwts, ivarup, ivarvp, ivartp, ivarrp, ivaruf,  &
     ivarvf, ivartf, ivarrf, iup, ivp, itheta, irv, ipp, iut, ivt, itht,  &
     irvt, ipt, ivarp, ivarp2, ipi0, irtgt
use varfile_init_mod, only: nudge

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

! Return if nudging not wanted:

if (nudlat <= 0) return

if ((tnudcent+tnudtop < 0.01_f4 .and. c_gd%nxtnest >= 1) .or.  &
     tnudcent+tnudtop+tnudlat < 0.01_f4) return

! Call nudging routine:

if (H2Ov_tog > 0) then
   call nudge(mzp, mxp, myp, ia, iz, ja, jz, ibndc, A(IVARWTS), A(IVARUP),  &
        A(IVARVP), A(IVARTP), A(IVARRP), A(ISCR1), A(IVARUF), A(IVARVF),  &
        A(IVARTF), A(IVARRF), A(ISCR2), A(IUP), A(IVP), A(ITHETA), a(irv),  &
        A(IPP), A(IUT), A(IVT), A(ITHT), a(irvt), A(IPT), A(IVARP),  &
        A(IVARP2), A(IPI0), A(IRTGT))
else
   call nudge(mzp, mxp, myp, ia, iz, ja, jz, ibndc, A(IVARWTS), A(IVARUP),  &
        A(IVARVP), A(IVARTP), A(IVARRP), A(ISCR1), A(IVARUF), A(IVARVF),  &
        A(IVARTF), A(IVARRF), A(ISCR2), A(IUP), A(IVP), A(ITHETA), a(iscr1),  &
        A(IPP), A(IUT), A(IVT), A(ITHT), a(iscr1), A(IPT), A(IVARP),  &
        A(IVARP2), A(IPI0), A(IRTGT))
end if

end subroutine data_assim
