!```````````````````````````````````````````````````````````````````
module aermic_rad_mem_mod

!+ Module that contains memory structures common to both aerosol microphysics
!   and radiative transfer codes

use univ_kind_defs_mod, only: f4, f8
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN

implicit none

integer :: iaertog
integer :: maxnaerscl
integer, parameter :: aer_ngrp = 3   ! # of species groups
integer, parameter :: max_ndist_ps = 9  ! Max # of mass
                                        ! distributions (per aerosol scalar)

logical :: aer_active,faux_dust_rtxinfo_needed

! Minimum mixing ratio permitted (any smaller, and certain single precision
!  floating-point ops have BIG issues): 

real(f4), parameter :: aer_MRmin = 1.e-18_f4  

! Aerosol scalar quantity descriptor arrays:
!
!   aer_group ==>  PRAMS group ID (i.e., dust, H2O ice, or CO2 ice)
!   aer_subgrp ==>  PRAMS subgroup ID (type of aerosol concentration)
!   aer_ie ==>  mapping array (PRAMS aerscl var# -> microphysics code elem#) 
!   aer_ib ==>  mapping array (PRAMS aerscl var# -> microphysics code bin#) 
!   aer_ig ==>  mapping array (PRAMS aerscl var# -> microphysics code group#) 
!   aer_grpbins ==>  number of bins in each element of PRAMS aerosol group
!   aer_grpelems ==>  number of elements in each PRAMS aerosol group

integer, allocatable, dimension(:) :: aer_group  &
     ,aer_subgrp,aer_ie,aer_ib,aer_ig             ! [maxnaerscl]
integer, dimension(aer_ngrp) :: aer_grpbins,aer_grpelems,aer_grpgas
integer :: aer_ngasses,aer_nelements,aer_ngroups,aer_nsolutes  &
     ,aer_maxnbins
integer, allocatable, dimension(:) :: aer_gasgrp   ! [aer_ngasses]

! On/off toggle arrays for various model processes:

logical, allocatable, dimension(:,:) :: aer_XOdiffuse,aer_XOadvect  &
     ,aer_XOprecip,aer_XOradiate,aer_XOsfcsrc,aer_XOnucleate  &
     ,aer_diffuse_mask,aer_advect_mask,aer_precip_mask,aer_radiate_mask  &
     ,aer_sfcsrc_mask,aer_nucleate_mask             ! [maxnaerscl,ngrids]

real(f8), dimension(aer_ngrp) :: aer_rad_gpbeg   ! RT start times
real(f8), dimension(aer_ngrp) :: aer_gpbeg   ! Process start times
real(f8), dimension(aer_ngrp) :: aer_inj_gpbeg   ! Atm inject start times
real(f8), dimension(aer_ngrp) :: aer_csr_gpbeg ! Sfc_res char start times
real(f8) :: aer_rvbeg                          

logical, dimension(0:aer_ngrp) :: aer_tyet,aer_rad_tyet
logical, dimension(aer_ngrp) :: aer_inj_tyet,aer_csr_tyet

character(len=30), dimension(aer_ngrp) :: aer_grpname
character(len=60), dimension(aer_ngrp,0:aer_ngrp-1) :: aer_elemname

character(len=MAX_FILE_PATH_LEN) :: dataset_path, dataset_path_bspec

real(f4), allocatable, dimension(:,:) :: aer_mdist_bfrac  &
     ,aer_mdist_bm2n                    ! [max_ndist_ps,aer_maxnbins]
real(f4), allocatable, dimension(:,:) ::  &
     iv_distfrac                 ! [max_ndist_ps,maxnaerscl]
integer, allocatable, dimension(:,:) ::  &
     iv_distID                 ! [max_ndist_ps,maxnaerscl]
integer, allocatable, dimension(:) :: iv_ndist     ! [maxnaerscl]
logical, allocatable, dimension(:) :: iv_is_dust    ! [maxnaerscl]

integer :: ust_tab_nent_T,ust_tab_nent_rhof
real(f4), allocatable, dimension(:,:) ::  &
     ustar_t_min           ! {ust_tab_nent_rhof,ust_tab_nent_T}

integer, parameter :: n_weibull = 3
integer :: lmflx_tab_nent_u,lmflx_tab_nent_ut
real(f4), allocatable, dimension(:,:,:) ::  &
     intmflux           ! {lmflx_tab_nent_ut,lmflx_tab_nent_u,n_weibull}
logical :: laminar_only


contains


!```````````````````````````````````````````````````````````````````
subroutine aermicrad_AD(on_or_off, phase)

!+ Allocates or deallocates key module arrays

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: n_grids
use vtables_mod, only: naersclr

implicit none

!=== Argument declarations:

logical, intent(IN) :: on_or_off
integer, intent(IN) :: phase

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'aermicrad_AD'

integer :: e, ng, ma, mb, mds

!=== Executable statements:
 
! Allocate memory:

ng = n_grids 
maxnaerscl = maxval(naersclr(1:ng))   ! Set max dimension
ma = maxnaerscl
mb = aer_maxnbins
mds = max_ndist_ps

if (on_or_off) then

   if (phase == 1) then

      if (aer_ngasses > 0) then
         allocate(aer_gasgrp(aer_ngasses), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'aermcrdAD00')
      end if

      if (ma > 0) then
         allocate(aer_group(ma), aer_subgrp(ma), aer_ie(ma), aer_ib(ma),  &
              aer_ig(ma), iv_distfrac(mds,ma), iv_distID(mds,ma), &
              iv_ndist(ma), iv_is_dust(ma), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation,  &
              procedure_name, 'aermcrdAD01')

         allocate(aer_XOdiffuse(ma,ng), aer_XOadvect(ma,ng),  &
           aer_XOprecip(ma,ng), aer_XOradiate(ma,ng), aer_XOsfcsrc(ma,ng),  &
           aer_XOnucleate(ma,ng), aer_diffuse_mask(ma,ng),  &
           aer_advect_mask(ma,ng), aer_precip_mask(ma,ng),  &
           aer_radiate_mask(ma,ng), aer_sfcsrc_mask(ma,ng),  &
           aer_nucleate_mask(ma,ng), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation,  &
              procedure_name, 'aermcrdAD02')
      end if
        
   else if (phase == 2) then

      if (mb > 0) then
         allocate(aer_mdist_bfrac(mds,mb), aer_mdist_bm2n(mds,mb), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation,  &
              procedure_name, 'aermcrdAD03')
      end if

   end if

else

   if (aer_ngasses > 0) then
      deallocate(aer_gasgrp, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aermcrdAD00')
   end if

   if (ma > 0) then
      deallocate(aer_group, aer_subgrp, aer_ie, aer_ib, aer_ig, iv_distfrac,  &
           iv_distID, iv_ndist, iv_is_dust, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aermcrdAD01')

      deallocate(aer_XOdiffuse, aer_XOadvect, aer_XOprecip, aer_XOradiate,  &
           aer_XOsfcsrc, aer_XOnucleate, aer_diffuse_mask, aer_advect_mask,  &
           aer_precip_mask, aer_radiate_mask, aer_sfcsrc_mask,  &
           aer_nucleate_mask, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aermcrdAD02')
   end if

   if (mb > 0) then
      deallocate(aer_mdist_bfrac, aer_mdist_bm2n, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aermcrdAD03')
   end if

end if

end subroutine aermicrad_AD


!```````````````````````````````````````````````````````````````````
subroutine mvar_dust_params(m1,m2,m3,ia,iz,ja,jz,Ls_curr,glat,tautot_ij  &
     ,conrath_nu,constant_lat)

!+ Calculate "more variable" (in time and latitude, but idealized) dust optical
!   depth and Conrath nu parameter
!
!  If 'constant_lat' present, set tgt_lat to given value
!  NOTE: Algorithm(s) created mostly by Tony Colaprete

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: pi_f4, ONE_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2,m3,ia,iz,ja,jz
real(f4), dimension(m2,m3), intent(IN) :: glat
real(f4), intent(IN) :: Ls_curr
real(f4), optional, intent(IN) :: constant_lat

real(f4), dimension(m2,m3), intent(OUT) :: tautot_ij,conrath_nu

!== Local declarations:

integer :: i,j
real(f4) :: dust_tau_pre,rLs_comp_A,rLs_comp_B,rlat_comp_A,rlat_comp_B  &
     ,MGCM_dlat,MGCM_fJ

real(f4), dimension(m2,m3) :: tgt_lat

integer, parameter :: MGCM_L_J = 24   ! *HARDWIRED*: 'L_J' in Ames
                                                !  MGCM (C-grid)

real(f4), parameter :: Lsbnd_A = 270._f4  &    ! Ls boundary? [deg] 
                      ,Lsbnd_B = 90._f4   &    ! Ls boundary? [deg]

                      ,dustod_min = 0.05_f4  &   ! Min dust opt depth [none]
                      ,dustod_max = 0.5_f4  &    ! Max dust opt depth [none]
                           
                      ,p_A = 0.7_f4  &
                      ,p_B = 0.65_f4  &
                      ,p_C = 0.2_f4  &
                      ,p_D = 0.2_f4  &
                      ,pm_A = 0.9_f4  &
                      ,pm_B = 2.2_f4  &
                      ,pd_A = 1.4_f4

!=== Executable statements:

if(present(constant_lat)) then
   tgt_lat(:,:) = constant_lat
else
   tgt_lat(:,:) = glat(:,:)
endif

do j=ja,jz
   do i=ia,iz

      rLs_comp_A = sin(pi_f4*(Ls_curr-Lsbnd_A)/360._f4)**2
      rLs_comp_B = sin(pi_f4*(Ls_curr-Lsbnd_B)/360._f4)**2

      MGCM_dlat = 180._f4/real(MGCM_L_J)
      MGCM_fJ = (tgt_lat(i,j)+90._f4)/MGCM_dlat
      
      rlat_comp_A = rLs_comp_A*atan((MGCM_fJ/real(MGCM_L_J))**p_A)  &
           *atan((ONE_f4-(MGCM_fJ-2._f4)/real(MGCM_L_J))**p_C)
      rlat_comp_B = rLs_comp_B*atan((ONE_f4-MGCM_fJ/real(MGCM_L_J))**p_B)  &
           *atan(((MGCM_fJ+2._f4)/real(MGCM_L_J))**p_D)
      
      dust_tau_pre = (pm_A*rlat_comp_A+pm_B*rlat_comp_B)/pd_A
          
      dust_tau_pre = max(dustod_min,dust_tau_pre)   ! Bound optical depth
      dust_tau_pre = min(dustod_max,dust_tau_pre)   !
      
      tautot_ij(i,j) = dust_tau_pre
      
      conrath_nu(i,j) = 0.03_f4/exp(1.8_f4*(dust_tau_pre/0.3_f4-ONE_f4))

   enddo
enddo

end subroutine mvar_dust_params


end module aermic_rad_mem_mod
