!```````````````````````````````````````````````````````````````````
module aemic_coag_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine coag_loss

!+ Calculates aerosol coagulation loss rates

use aermicro_mem_mod, only: ngroups,nbins,ienconc,icoag,nz_mic,FEW_PC,pconmax  &
     ,coaglg,ckernel,pcl,volx

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

!  Calculates coagulation loss rates <coaglgg>.
!  See [Jacobson, et al., Atmos. Env., 28, 1327, 1994] for details
!  on the coagulation algorithm.
!
!  The loss rates for all particle elements in a particle group are equal.

integer :: ig,ie,ig_c,ie_c,ig_r,k,ib,ib_c

!=== Executable statements:

! Loop over particle groups for which coagulation loss is being calculated:

do ig=1,ngroups

   ie = ienconc(ig)     ! Particle number concentration element

   ! Loop over particle groups that particle in group ig might collide with:

   do ig_c=1,ngroups

      ie_c = ienconc(ig_c)

      ! Particle resulting from coagulation between groups <ig> and <ig_c> goes
      !  to group <ig_r>:

      ig_r = icoag(ig,ig_c)

      ! Resulting particle is in same group as particle under consideration --
      !  partial loss (muliplies <volx>):

      if(ig_r == ig) then

         do k=1,nz_mic

            ! Bypass calculation if few source particles present:

            if(pconmax(k,ig_c)*pconmax(k,ig) > FEW_PC**2) then

               do ib=1,nbins(ig)-1
                  do ib_c=1,nbins(ig_c)
                     coaglg(k,ib,ig) = coaglg(k,ib,ig)+  &
                          ckernel(k,ib,ib_c,ig,ig_c)*pcl(k,ib_c,ie_c)*  &
                          volx(ig_r,ig,ig_c,ib,ib_c) 
                  enddo
               enddo
            endif
         enddo

         ! Resulting particle is in a different group -- complete loss
         !  (no <volx>):

      elseif(ig_r /= ig .and. ig_r /= 0) then

         ! Bypass calculation if few particles present:

         do k=1,nz_mic
            if(pconmax(k,ig_c)*pconmax(k,ig) > FEW_PC**2) then
               do ib=1,nbins(ig)
                  do ib_c=1,nbins(ig_c)
                     coaglg(k,ib,ig) = coaglg(k,ib,ig)+  &
                          ckernel(k,ib,ib_c,ig,ig_c)*pcl(k,ib_c,ie_c)
                  enddo
               enddo
            endif
         enddo
      endif

   enddo
enddo

! Boundary condition: Particles from bin <NBIN> are only lost by
!  coagulating into other elements. (This is taken care of by <NBIN>-1
!  limit above)

end subroutine coag_loss


!```````````````````````````````````````````````````````````````````
subroutine coag_production(ibin,ielem)

!+ Calculates aerosol coagulation production rates

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use aermicro_mem_mod, only: igelem,ienconc,FEW_PC,pconmax,npairu,icoagelem  &
     ,I_COREMASS,itype,I_VOLATILE,I_VOLCORE,ncore,rmass,pc,I_INVOLATILE  &
     ,coagpe,pcl,pkernel,ilow,jlow,iup,jup,I_CORE2MOM,iglow,jglow,igup,jgup  &
     ,npairl,icorelem,nz_mic  &
     ,rmi => fscr1     ! [nz_mic]

implicit none

!=== Argument declarations:

integer, intent(IN) :: ibin,ielem

!== Local declarations:

! Calculates coagulation production terms <coagpe>.
!  See [Jacobson, et al., Atmos. Env., 28, 1327, 1994] for details
!  on the coagulation algorithm.

integer :: igrp,iquad,ig,iz,iefrom,je,ic,i,jg,j,i_pkern,iecore
real(f4) :: elemass,rmasscore,fracmass,totmass

!=== Executable statements:

! Definition of i,j,k,n used in comments: collision between i and j bins
!  yields particle between bins k and k+1.  Production in bin n is calculated.

igrp = igelem(ielem)   ! group that particles are produced in

! Particle number production:

! Coagulation between particle in group <ig> bin <i> with particle in
!  group <jg> bin <j> results in particle with mass between bins k and k+1.
!  First, loop over group-bin quads <ig,i,jg,j> resulting in production in
!  bin <ibin> = k+1.  The set of quads <igup,jgup,iup,jup> is
!  defined in setupcoag:

do iquad=1,npairu(igrp,ibin)

   ig = igup(igrp,ibin,iquad)           ! source group
   jg = jgup(igrp,ibin,iquad)           ! source group
   i  = iup(igrp,ibin,iquad)            ! source bin
   j  = jup(igrp,ibin,iquad)            ! source bin

   iefrom = icoagelem(ielem,ig)                ! source element for <i> particle

   ! If <iefrom> = 0 then there is no contribution to production:

   if(iefrom /= 0) then

      je = ienconc(jg)                    ! source element for <j> particle

      ! If ielem is core mass type and <ig> is a CN type and <ig> is different
      !  from <igrp>, then we must multiply production by mass per particle
      !  (<rmi>) of element <icoagelem>.  (this is <rmass> for all source
      !  elements except particle number concentration in a multicomponent CN
      !  group):

      do iz=1,nz_mic

         ! Bypass calculation if few source particles present:

         if(pconmax(iz,ig)*pconmax(iz,jg) > FEW_PC**2) then

            rmi(iz) = ONE_f4
            i_pkern = 1
 
            if(itype(ielem) == I_COREMASS .or.  &
                 itype(ielem) == I_VOLCORE) then    ! core mass

               i_pkern = 3     ! Use different kernel for core mass production

               if((itype(ienconc(ig)) == I_INVOLATILE .or.  &
                    itype(ienconc(ig)) == I_VOLATILE) .and.  &
                    ig /= igrp) then     ! CN source and ig different from igrp

                  if(ncore(ig) == 0) then         ! No cores in source group
                     rmi(iz) = rmass(i,ig)
                  elseif(itype(iefrom) == I_INVOLATILE .or.  &
                       itype(iefrom) == I_VOLATILE) then

                     ! Source element is #concentration elem of mixed CN group

                     totmass  = pc(iz,i,iefrom)*rmass(i,ig)
                     rmasscore = pc(iz,i,icorelem(1,ig))
                     do ic=2,ncore(ig)
                        iecore = icorelem(ic,ig)
                        rmasscore = rmasscore+pc(iz,i,iecore)
                     enddo
                     fracmass = ONE_f4-rmasscore/totmass
                     elemass = fracmass*rmass(i,ig)
                     rmi(iz) = elemass
                  endif

               endif

            elseif(itype(ielem) == I_CORE2MOM) then      ! core mass^2

               i_pkern = 5   ! Use different kernel for core mass^2 production
               if(itype(ienconc(ig)) == I_INVOLATILE) then
                  rmi(iz) = rmass(i,ig)**2
               endif

            endif

            ! For each spatial grid point, sum up coagulation production
            !  contributions from each quad:

            coagpe(iz,ibin,ielem) = coagpe(iz,ibin,ielem)  & 
                 +pc(iz,i,iefrom)*pcl(iz,j,je)*rmi(iz)  &
                 *pkernel(iz,i,j,ig,jg,igrp,i_pkern)

         endif

      enddo

   endif

enddo

! Next, loop over group-bin quads for production in bin <ibin> = k from
!  bin <i> due to collision between bins <i> and <j>.
!  Production will only occur if either k != <i> or igrp != <ig>:
 
do iquad=1,npairl(igrp,ibin)

   ig = iglow(igrp,ibin,iquad)
   jg = jglow(igrp,ibin,iquad)
   i  = ilow(igrp,ibin,iquad)
   j  = jlow(igrp,ibin,iquad)

   iefrom = icoagelem(ielem,ig)         ! source element for <i> particle

   if(iefrom /= 0) then
      
      je = ienconc(jg)                     ! source element for <j> particle

      do iz=1,nz_mic

         ! Bypass calculation if few particles present:

         if(pconmax(iz,ig)*pconmax(iz,jg) > FEW_PC**2) then

            rmi(iz) = ONE_f4
            i_pkern = 2
 
            if(itype(ielem) == I_COREMASS .or.  &
                 itype(ielem) == I_VOLCORE) then          ! core mass

               i_pkern = 4     ! Use different kernel for core mass production

               if((itype(ienconc(ig)) == I_INVOLATILE .or.  &
                    itype(ienconc(ig)) == I_VOLATILE) .and.  &               
                    ig /= igrp) then     ! CN source and ig different from igrp
!?            if(itype(ienconc(ig)) .eq. I_INVOLATILE

                  if(ncore(ig) == 0) then            ! No cores in source group
                     rmi(iz) = rmass(i,ig)
                  elseif(itype(iefrom) == I_INVOLATILE .or.  &
                       itype(iefrom) == I_VOLATILE) then

                     ! Source element is #concentration elem of mixed CN group

                     totmass = pc(iz,i,iefrom)*rmass(i,ig)
                     rmasscore = pc(iz,i,icorelem(1,ig))
                     do ic=2,ncore(ig)
                        iecore = icorelem(ic,ig)
                        rmasscore = rmasscore+pc(iz,i,iecore)
                     enddo
                     fracmass = ONE_f4-rmasscore/totmass
                     elemass  = fracmass*rmass(i,ig)
                     rmi(iz) = elemass
                  endif
                  
               endif

            elseif(itype(ielem) == I_CORE2MOM) then      ! core mass^2
               
               i_pkern = 6    ! Use different kernel for core mass^2 production

               if(itype(ienconc(ig)) == I_INVOLATILE) then
                  rmi(iz) = rmass(i,ig)**2
               endif

            endif

            coagpe(iz,ibin,ielem) = coagpe(iz,ibin,ielem)  & 
                 + pc(iz,i,iefrom)*pcl(iz,j,je)*rmi(iz)  &
                 * pkernel(iz,i,j,ig,jg,igrp,i_pkern)

         endif

      enddo

   endif

enddo

end subroutine coag_production


!```````````````````````````````````````````````````````````````````
subroutine coag_solve(ibin,ielem)

!+ Calculates new particle concentrations due to coagulation microphysical
!   processes

use univ_kind_defs_mod, only: f4
use aermicro_mem_mod, only: igelem,ienconc,nz_mic,coagpe,coaglg,pc,dtime,ONE

implicit none

!=== Argument declarations:

integer, intent(IN) :: ibin,ielem

!== Local declarations:

!  The basic form from which the solution is derived is
!   ( new_value - old_value ) / dtime = source_term - loss_rate*new_value
!
!  This routine derived from psolve.f code, in which particle concentrations
!  due to coagulation were formerly included, before the relatively slow
!  coagulation calcs were separated from the other microphysical processes
!  so that time splitting could be applied to these fast & slow calcs.

integer :: k,igroup,iepart
real(f4) :: ppd,pls

!=== Executable statements:

! Define current group & particle number concentration element indices:

igroup = igelem(ielem)         ! particle group
iepart = ienconc(igroup)       ! particle number concentration element

do k=1,nz_mic          ! Visit each spatial point

   ! Compute total production rate due to coagulation:

   ppd = coagpe(k,ibin,ielem)

   ! Compute total loss rate due to coagulation:

   pls = coaglg(k,ibin,igroup)

   ! Update net particle number concentration during current timestep
   !  due to production and loss rates for coagulation:

   pc(k,ibin,ielem) = (pc(k,ibin,ielem)+dtime*ppd)/(ONE+pls*dtime)

enddo

end subroutine coag_solve


end module aemic_coag_mod
