!```````````````````````````````````````````````````````````````````
module aemic_driv_mod

!+ Module that contains aerosol microphysics driver routine(s)

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine aemic_driver(ext_n1,ext_n2,ext_n3,ext_naersclr,ext_ia,ext_iz  &
     ,ext_ja,ext_jz,ext_ka,ext_kz,ext_iv2ie_map,ext_iv2ib_map,ext_aer_sclr  &
     ,ext_maxn1,ext_maxn2,ext_maxn3,ext_aer_sclt,init_micro,ext_density  &
     ,ext_densityout,ext_theta,ext_thetaout,ext_press,ext_tempk,ext_H2Ovmc  &
     ,ext_H2Ovmcout,ext_zlev,ext_zmid,ext_rtgt,ext_dt,ext_aer_sfcrsv  &
     ,ext_aer_sclrFF,ext_aer_sclrAG,ext_tslopang,ext_grid,ext_aer_XOnuc  &
     ,ext_aer_X0pcp,ext_air_CO2_frac)

!+ Driver (but not standalone) for single column aerosol microphysics package
!   derived from CARMA (terrestrial 3D code)
!
! NOTES:
!
!  The 'ext_' prefix denotes that the variable is native to the external
!   (calling) model ONLY (i.e., not native to the microphysics code).
!
!  ext_n1 ==>  the vertical dimension of *external model*
!  ext_n2,ext_n3 ==>  the x- and y-dir dimensions, respectively
!  ext_naersclr ==>  the total number of aerosol species/bins
!  init_micro ==> logical value indicating if model should be initialized

use univ_kind_defs_mod, only: f4
use aermicro_mem_mod, only: pc,nz_mic,nz1_mic,aermicro_AD,ptc,rhoa,dkz,t,p,rmu  &
     ,thcond,gc,ngasses,fbotpart,vf,igelem,gasname,dtime,time,ftoppart  &
     ,lpcnt_col,zl,zc,dz,zc_static,zl_static,sadv_pc_tend,pc_vTX_on  &
     ,itbnd_pc,I_FLUX_SPEC,sadv_gc_tend,sadv_ptc_tend,i_curr,j_curr,grid  &
     ,do_grow,should_vTX_ie
use aemic_init_mod, only: init_general, setup_vfall, setup_grow2, setup_gkern2
use aemic_init_astruct_mod, only: init_astruct
use aemic_init_modcore_mod, only: init_modcore
use aemic_preDt_mod, only: pre_step
use aemic_predict_mod, only: predict_new_state
use aemic_postDt_mod, only: post_step

implicit none

!=== Argument declarations:

logical, intent(IN) :: init_micro
integer, intent(IN) :: ext_n1,ext_n2,ext_n3,ext_ia,ext_iz,ext_ja,ext_jz  &
     ,ext_naersclr,ext_ka,ext_kz,ext_grid,ext_maxn1,ext_maxn2,ext_maxn3
integer, dimension(ext_naersclr), intent(IN) :: ext_iv2ie_map,ext_iv2ib_map
real(f4), intent(IN) :: ext_dt
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(IN) :: ext_density,ext_press  &
     ,ext_tempk,ext_theta
real(f4), dimension(ext_n2,ext_n3), intent(IN) :: ext_rtgt,ext_tslopang
real(f4), dimension(ext_n1), intent(IN) :: ext_zlev,ext_zmid
real(f4), dimension(ext_maxn1*ext_maxn2*ext_maxn3,ext_naersclr), intent(IN) ::  &
     ext_aer_sclt
logical, dimension(ext_naersclr), intent(IN) :: ext_aer_XOnuc,ext_aer_X0pcp

real(f4), dimension(ext_n1,ext_n2,ext_n3,ext_naersclr), intent(INOUT) :: ext_aer_sclr
real(f4), dimension(ext_n2,ext_n3,ext_naersclr), intent(INOUT) :: ext_aer_sfcrsv  &
     ,ext_aer_sclrAG
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(INOUT) :: ext_air_CO2_frac,  &
     ext_H2Ovmc

real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(OUT) :: ext_thetaout  &
     ,ext_densityout,ext_H2Ovmcout
real(f4), dimension(ext_n2,ext_n3,ext_naersclr), intent(OUT) :: ext_aer_sclrFF

!== Local declarations:

integer :: i,j,iv,ie,ib,igas,k,k1, kij
real(f4) :: ftmp,zadj_fact
logical :: ltmp1

!^%
!^%real(f4) :: t1,t2  
!^%real(f4), external :: tag_CPUtime
!^%

!=== Executable statements:

if(init_micro) then     ! Initialize microphysics model
   write (*,'(/a/)') 'INFO: Initializing aerosol microphysics model ...'

   nz_mic = ext_kz-ext_ka+1  ! Set vert dim for aerosol microphysics use (NOTE:
                             !  may not be used with vertical grid nesting)
   nz1_mic = nz_mic+1

   call aermicro_AD(.TRUE.,3)   ! Allocate necessary Phase 2 memory

   ! Initialize some general control parameters:
   call init_general

   ! Initialize model grid and some atmospheric parameters:
   call init_astruct(ext_n1,ext_n2,ext_n3,ext_density,ext_theta,ext_press  &
        ,ext_tempk,ext_zlev,ext_zmid,ext_rtgt,ext_ka)

   ! Initialize core model parameters:
   call init_modcore

endif

!^%
!^%call accum_timetags_wn(0,'init',t1,t2)  ! Initialize timing accumulator
!^%
grid = ext_grid
dtime = ext_dt       ! Set model "long" timestep [s]

! Should we do some processes at all?

ltmp1 = .FALSE.
do iv=1,ext_naersclr    ! Loop over all aerosol species/bins
   if(ext_aer_XOnuc(iv)) then
      ltmp1 = .TRUE.
   endif

   ie = ext_iv2ie_map(iv)
   should_vTX_ie(ie) = ext_aer_X0pcp(iv)  ! Do vert transport for this element?
enddo
do_grow = ltmp1    ! Set nuc/evap/grow flag

do j=ext_ja,ext_jz        ! Loop over all computational horizontal grid points
   j_curr = j
   do i=ext_ia,ext_iz     !  of external model
      i_curr = i

!%%% For hydrostatic external models only:
!%      p_surf = (ext_press(ext_ka-1)+ext_press(ext_ka))*0.5
!%      p_top = ext_press(ext_kz)

      ! For every horizontal grid point of external model, adjust CARMA
      !  vertical grid quantities so that they are equal to true
      !  ("real-world") distances [cm]:

      zadj_fact = ext_rtgt(i,j)
      zl(1) = zl_static(1)*zadj_fact   ! Height AGL of column bottom [cm] 
      do k=1,nz_mic
         k1 = k+1
         zl(k1) = zl_static(k1)*zadj_fact   ! Height AGL of top of layer [cm]
         dz(k) = zl(k1)-zl(k)            ! Vertical gridspacing [cm]
         zc(k) = zc_static(k)*zadj_fact   ! Height AGL of layer midpoint [cm]
      enddo

      ! Fill atmospheric quantities using external model arrays:

      t(1:nz_mic) = ext_tempk(ext_ka:ext_kz,i,j)   ! Air temperature [K]

      p(1:nz_mic) = ext_press(ext_ka:ext_kz,i,j)     ! Air pressure [dyne/cm^2]
      rhoa(1:nz_mic) = ext_density(ext_ka:ext_kz,i,j)    ! Air density [g/cm^3]

      do k=1,nz_mic
         rmu(k) = 1.59e-6_f4*t(k)**1.5_f4/(t(k)+244.4_f4)*10._f4  ! Mars CO2 gas viscosity
         thcond(k) = (0.0445_f4*t(k)+  &       ! Thermal conductivity for CO2 gas
              5.5e5_f4*(t(k)**2)-1.5_f4)*4.18e-7_f4
      enddo

      dkz(:) = tiny(ftmp)    ! Vertical diffusion coeff OFF (effectively)

      ptc(:) = rhoa(:)*ext_theta(ext_ka:ext_kz,i,j) ! Potential temperature
                                                    !  concentration
                                                    !  [(K.g)/cm^3]
!%      sadv_ptc_tend(:) = ext_???(ext_ka:ext_kz,i,j)   ! Fill potential temp
!%                                                      !  adv tend

      do igas=1,ngasses
         if(gasname(igas)(1:1) == 'W') then   ! H2O vapor
            gc(:,igas) = ext_H2Ovmc(ext_ka:ext_kz,i,j)    ! [g/cm^3]    
         elseif(gasname(igas)(1:1) == 'C') then   ! CO2 gas
            gc(:,igas) = rhoa(:)*ext_air_CO2_frac(ext_ka:ext_kz,i,j) ! [g/cm^3]
!#            gc(:,igas) = rhoa(:)*0.9532          ! [g/cm^3]
         endif
!%         sadv_gc_tend(:,ib,ie) = ext_???(ext_ka:ext_kz,i,j) ! Fill gas
!%                                                            !  adv tend
      enddo
         
      call setup_vfall   ! Calculate current fall velocities
      call setup_grow2     ! Calculate current growth kernel, etc.
      call setup_gkern2    !

      do iv=1,ext_naersclr    ! Loop over all aerosol species/bins
         ie = ext_iv2ie_map(iv)    ! Element ID of current aerosol array
         ib = ext_iv2ib_map(iv)    ! Bin ID of current aerosol array
         pc(:,ib,ie) = ext_aer_sclr(ext_ka:ext_kz,i,j,iv)  ! Fill aerosol vals
           ! Fill with external aerosol advective/diffusive tendencies:
         do k=ext_ka,ext_kz
            kij = (j-1)*ext_n1*ext_n2+(i-1)*ext_n1+k
            sadv_pc_tend(k-ext_ka+1,ib,ie) = ext_aer_sclt(kij,iv)  
         end do
      enddo

      if(pc_vTX_on) then   ! Set selected boundary conditions for transport

         if(itbnd_pc == I_FLUX_SPEC) then    ! Need to set upper particle flux

            ! Define the particle flux through the topmost boundary (due to
            !  gravitational sedimentation, etc.):
            !  NOTE: <ftoppart> is defined as the *downward* flux from the
            !        upper boundary (and is thus negative if representing a
            !        flux *towards* the upper boundary).

            do iv=1,ext_naersclr    ! Loop over all aerosol species/bins
               ie = ext_iv2ie_map(iv)    ! Element ID of current aerosol array
               ib = ext_iv2ib_map(iv)    ! Bin ID of current aerosol array
               ! It would be more correct to use
               !     vf*(ext_aer_sclr(ext_kz)+ext_aer_sclr(ext_kz+1))*0.5 ,
               !  *BUT* then any particle growth in the topmost CARMA layer
               !  would lead to a runaway paticle concentration (due to
               !  interactions between particle growth and sedimentation) --
               !  therefore we will use the following, which avoids this
               !  problem: 
               ftoppart(ib,ie) = vf(nz1_mic,ib,igelem(ie))*  &
                    ext_aer_sclr(ext_kz+1,i,j,iv) ![#/(s.cm^2)] or [g/(s.cm^2)]
            enddo

         endif
      endif

      ! Do operations required before each time-integration step:

!^%      t1 = tag_CPUtime()
      call pre_step
!^%      t2 = tag_CPUtime()
!^%      call accum_timetags_wn(2,'pre_step',t1,t2)

      ! Step variables forward in time by one timestep:

!^%      t1 = tag_CPUtime()
      call predict_new_state
!^%      t2 = tag_CPUtime()
!^%      call accum_timetags_wn(3,'predict',t1,t2)

!^      if(lpcnt_col > 0) then
!^         print *,'$%%%%',i,j,lpcnt_col,real(lpcnt_col)/real(nz_mic)*100.
!^      endif

      ! Do operations required after each time-integration step:

!^%      t1 = tag_CPUtime()
      call post_step
!^%      t2 = tag_CPUtime()
!^%      call accum_timetags_wn(4,'post_step',t1,t2)

      ! Fill certain external model arrays with updated values:

      ext_thetaout(ext_ka:ext_kz,i,j) = ptc(:)/rhoa(:)    ! Potential temp [K]

      do igas=1,ngasses
         if(gasname(igas)(1:1) == 'W') then   ! H2O vapor
            ext_H2Ovmc(ext_ka:ext_kz,i,j) = gc(:,igas)   ! [g/cm^3]
         elseif(gasname(igas)(1:1) == 'C') then   ! CO2 gas
            ext_densityout(ext_ka:ext_kz,i,j) = gc(:,igas)+  &
                 rhoa(:)*(1.-ext_air_CO2_frac(ext_ka:ext_kz,i,j))
!#            ext_densityout(ext_ka:ext_kz,i,j) = rhoa(:)

            ext_air_CO2_frac(ext_ka:ext_kz,i,j) = gc(:,igas)/  &
                 ext_densityout(ext_ka:ext_kz,i,j)
!#            ext_air_CO2_frac(ext_ka:ext_kz,i,j) = 0.9532
         endif
      enddo

      do iv=1,ext_naersclr    ! Loop over all aerosol species/bins
         ie = ext_iv2ie_map(iv)    ! Element ID of current aerosol array
         ib = ext_iv2ib_map(iv)    ! Bin ID of current aerosol array
         ext_aer_sclr(ext_ka:ext_kz,i,j,iv) = pc(:,ib,ie)  ! Fill aerosol vals

         ! All we know for SURE about the sedimentation is the lower
         !  boundary flux (by definition) -- thus we will add an amount
         !  of aerosol that corresponds to the above flux to the surface
         !  accumulation/reservoir array(s); take into account the slope of the
         !  surface:

         ! Effective sedimentation (mass OR #) flux out of lowest layer
         !  [(# or g)/(s.cm^2)]:
         ftmp = fbotpart(ib,ie)
         ext_aer_sclrFF(i,j,iv) = ftmp  ! Flux

         ! Total amount of aerosol deposited on surface [(# or g)/cm^2] this
         !  timestep (note ground slope angle factor):
         ftmp = -ftmp*ext_dt*cos(ext_tslopang(i,j))

         ! Total (cumulative) amount of aerosol that has been deposited on 
         !  the surface in this simulation [(# or g)/cm^2]:
         ext_aer_sclrAG(i,j,iv) = ext_aer_sclrAG(i,j,iv)+ftmp

         ! Net amount of aerosol currently present on the surface in this
         !  simulation [(# or g)/cm^2]:
         ext_aer_sfcrsv(i,j,iv) = ext_aer_sfcrsv(i,j,iv)+ftmp
      enddo

   enddo
enddo

!^%call accum_timetags_wn(-22,'Display',t1,t2)

end subroutine aemic_driver


end module aemic_driv_mod
