!```````````````````````````````````````````````````````````````````
module aemic_drvint_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine aemic_get_ptd_cgs(m1,m2,m3,ia,iz,ja,jz,pp,pi0,thp,tempk,press_cgs  &
     ,density_cgs,R_air_cgs)

!+ Retrieve pressure (dyne/cm^2), temperature (K), and density (g/cm^3)
!   from PRAMS internal variables

use univ_kind_defs_mod, only: f4
use body_specific_constants_mod, only: R_air, cpi, cpor, p00
use aermicro_mem_mod, only: J_to_erg,kg_to_g,Pa_to_dyne_o_cm2

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2,m3,ia,iz,ja,jz
real(f4), dimension(m1,m2,m3), intent(IN) :: pp,pi0,thp

real(f4), dimension(m1,m2,m3), intent(INOUT) :: tempk,press_cgs,density_cgs
real(f4), intent(OUT) :: R_air_cgs

!== Local declarations:

integer :: i,j,k
real(f4) :: pi_over_cp,t_tmp,p_tmp

!=== Executable statements:

R_air_cgs = R_air*J_to_erg/kg_to_g   ! [erg/(g.K)]

do j=ja,jz
   do i=ia,iz
      do k=1,m1
         pi_over_cp = (pp(k,i,j)+pi0(k,i,j))*cpi
         t_tmp = pi_over_cp*thp(k,i,j)
         p_tmp = (p00*pi_over_cp**cpor)*Pa_to_dyne_o_cm2  
         density_cgs(k,i,j) = p_tmp/(R_air_cgs*t_tmp)     ! [g/cm^3]
         tempk(k,i,j) = t_tmp    ! [K]
         press_cgs(k,i,j) = p_tmp     ! [dyne/cm^2]
      enddo
   enddo
enddo

end subroutine aemic_get_ptd_cgs


!```````````````````````````````````````````````````````````````````
subroutine aemic2PRAMS(m1,m2,m3,ia,iz,ja,jz,thp,theta_new,density_new,rv  &
     ,rv_new,tempk,pp,pi0,tht,pt,rvt,R_air_cgs,ka,kz)

!+ Calculate theta, PI, RV tendencies (due to aerosol microphysical processes)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use body_specific_constants_mod, only: cp, p00, rocp, cp_o_cv, rocv
use main_memory_mod, only: c_gd
use main_memory_bspec_mod, only: CO2i_tog, H2Oi_tog
use aermicro_mem_mod, only: Pa_to_dyne_o_cm2

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2,m3,ia,iz,ja,jz,ka,kz
real(f4), dimension(m1,m2,m3), intent(IN) :: pp,pi0,rv
real(f4), dimension(m1,m2,m3), intent(IN) :: theta_new,density_new  &
     ,rv_new,tempk                         ! In CGS units
real(f4), intent(IN) :: R_air_cgs

real(f4), dimension(m1,m2,m3), intent(INOUT) :: tht,pt,rvt,thp

!== Local declarations:

integer :: i,j,k
real(f4) :: tendtmp,press_new,pi_new,pi_old,dti,tempk_new

!=== Executable statements:

dti = ONE_f4/c_gd%dt_dyn

! H2O and/or CO2 solid <-> gas phase change releases latent heat,
!  changing theta:

if(H2Oi_tog > 0 .or. CO2i_tog > 0) then 

   do j=ja,jz
      do i=ia,iz
         do k=ka,kz

            ! Latent heat dtheta/dt:
            tendtmp = (theta_new(k,i,j)-thp(k,i,j))*dti

            ! Update 'thp' and 'tht' (must be done HERE, before `acoustic`):
            thp(k,i,j) = thp(k,i,j)+tendtmp*c_gd%dt_dyn    ! [K]
            tht(k,i,j) = tht(k,i,j)+tendtmp     ! [K/s]

         enddo
      enddo
   enddo
   
   if(CO2i_tog > 0) then  ! CO2 solid <-> gas phase change may also change
                          !  the atmospheric mass (and thus likely pressure)

      do j=ja,jz
         do i=ia,iz
            do k=ka,kz
         
               tempk_new = (thp(k,i,j)**cp_o_cv)*  &    ! [K]
                    (R_air_cgs*density_new(k,i,j)/(p00*Pa_to_dyne_o_cm2))**rocv

               press_new = density_new(k,i,j)*R_air_cgs*tempk_new/  &
                    Pa_to_dyne_o_cm2                                   ! [Pa]
               pi_new = cp*(press_new/p00)**rocp   ! New PI value
               pi_old = pp(k,i,j)+pi0(k,i,j)

               tendtmp = (pi_new-pi_old)*dti
               pt(k,i,j) = pt(k,i,j)+tendtmp             ! PI tendency
      
            enddo
         enddo
      enddo

   endif

endif

end subroutine aemic2PRAMS


!```````````````````````````````````````````````````````````````````
subroutine aerproc_check(do_init)

!+ Manages (each timestep) model process overrides for aerosol scalars, etc.

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: n_grids, gd, current_sim_ET,  &
     enable_aerosol_microphysics
use main_memory_bspec_mod, only: dust_tog, H2Oi_tog, CO2i_tog, dust_rta,  &
     H2Oi_rta, CO2i_rta, H2Ov_tog
use vtables_mod, only: naersclr
use aermic_rad_mem_mod, only: aer_XOdiffuse,aer_XOadvect,aer_XOprecip  &
     ,aer_XOradiate,aer_XOsfcsrc,aer_XOnucleate,aer_gpbeg,aer_rvbeg  &
     ,aer_group,aer_diffuse_mask,aer_advect_mask,aer_precip_mask  &
     ,aer_radiate_mask,aer_sfcsrc_mask,aer_nucleate_mask,aer_tyet,aer_ngrp  &
     ,aer_rad_tyet,aer_rad_gpbeg,aer_inj_tyet  &
     ,aer_inj_gpbeg,aer_csr_tyet,aer_csr_gpbeg

implicit none

!=== Argument declarations:

logical, intent(IN) :: do_init

!== Local declarations:

integer :: iv,ng
real(f4) :: dt_dyn_g1
logical :: do_nucgrw

!=== Executable statements:

dt_dyn_g1 = gd(1)%dt_dyn   ! Longest timestep

aer_tyet(0) = .FALSE.
! Time for RV processes to start?
if(H2Ov_tog > 1) then   ! H2O vapor
   aer_tyet(0) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_rvbeg)
endif

! Time for each aerosol group to start?

aer_tyet(1:aer_ngrp) = .FALSE.
aer_rad_tyet(1:aer_ngrp) = .FALSE.
aer_inj_tyet(1:aer_ngrp) = .FALSE.
aer_csr_tyet(1:aer_ngrp) = .FALSE.
if(dust_tog > 0) then   ! Dust
   aer_rad_tyet(1) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_rad_gpbeg(1))
   if (dust_tog > 1) aer_tyet(1) =  &
        (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_gpbeg(1))
   if(dust_tog >= 3) then
      aer_inj_tyet(1) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_inj_gpbeg(1))
      aer_csr_tyet(1) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_csr_gpbeg(1))
   endif
endif
if(H2Oi_tog > 0) then   ! H2O_ice
   aer_tyet(2) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_gpbeg(2))
   aer_rad_tyet(2) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_rad_gpbeg(2))
endif
if(CO2i_tog > 0) then   ! CO2_ice
   aer_tyet(3) = (current_sim_ET+0.99*dt_dyn_g1 >= aer_gpbeg(3))
   aer_rad_tyet(3) = (current_sim_ET+0.99_f4*dt_dyn_g1 >= aer_rad_gpbeg(3))
endif

if(maxval(naersclr(:)) == 0) then
   return
endif

if(do_init) then     ! Only do if beginning of current *run*
   aer_XOdiffuse = .FALSE.     !
   aer_XOadvect = .FALSE.      !
   aer_XOradiate = .FALSE.     ! Initialize process toggles (assume the worst)
   aer_XOprecip = .FALSE.      !
   aer_XOsfcsrc = .FALSE.      !
   aer_XOnucleate = .FALSE.    !

   aer_diffuse_mask = .FALSE.  !
   aer_advect_mask = .FALSE.   !
   aer_radiate_mask = .FALSE.  ! Init process masks (assume all not permitted)
   aer_precip_mask = .FALSE.   !
   aer_sfcsrc_mask = .FALSE.   !
   aer_nucleate_mask = .FALSE. !

   ! Refine process masks according to model configuration (value of .FALSE.
   !  indicates a process not permitted to take place at ANY time this run):

   if ((H2Oi_tog == 0 .and. CO2i_tog == 0)) then
      do_nucgrw = .FALSE.  ! Never let nucleation/growth occur in this run 
   else
      do_nucgrw = .TRUE.  ! Nucleation/growth should occur in this run
   end if

   do ng=1,n_grids
      do iv=1,naersclr(ng)
         if (aer_group(iv) == 1) then   ! Dust group
            if (dust_rta .and. dust_tog > 0) then
               aer_radiate_mask(iv,ng) = .TRUE.
            end if
            if (dust_tog == 2 .and. enable_aerosol_microphysics) then
               aer_diffuse_mask(iv,ng) = .TRUE.  
               aer_advect_mask(iv,ng) = .TRUE.    
               aer_precip_mask(iv,ng) = .TRUE.    
               if (do_nucgrw) then
                  aer_nucleate_mask(iv,ng) = .TRUE. 
               end if
            else if (dust_tog >= 3 .and. enable_aerosol_microphysics) then
               aer_diffuse_mask(iv,ng) = .TRUE.  
               aer_advect_mask(iv,ng) = .TRUE.    
               aer_precip_mask(iv,ng) = .TRUE.    
               aer_sfcsrc_mask(iv,ng) = .TRUE.   
               if (do_nucgrw) then
                  aer_nucleate_mask(iv,ng) = .TRUE. 
               end if
            end if
         else if (aer_group(iv) == 2) then   ! H2O ice group
            if (H2Oi_rta .and. H2Oi_tog > 0) then
               aer_radiate_mask(iv,ng) = .TRUE.
            end if
            if (H2Oi_tog == 1 .and. enable_aerosol_microphysics) then
               aer_diffuse_mask(iv,ng) = .TRUE.  
               aer_advect_mask(iv,ng) = .TRUE.    
               aer_precip_mask(iv,ng) = .TRUE.
               aer_nucleate_mask(iv,ng) = .TRUE. 
            else if (H2Oi_tog == 2 .and. enable_aerosol_microphysics) then
               aer_diffuse_mask(iv,ng) = .TRUE.  
               aer_advect_mask(iv,ng) = .TRUE.
               aer_precip_mask(iv,ng) = .TRUE.    
               aer_sfcsrc_mask(iv,ng) = .TRUE.   
               aer_nucleate_mask(iv,ng) = .TRUE. 
            end if
         else if (aer_group(iv) == 3) then   ! CO2 ice group
            if (CO2i_rta .and. CO2i_tog > 0) then
               aer_radiate_mask(iv,ng) = .TRUE.
            end if
            if (CO2i_tog == 1 .and. enable_aerosol_microphysics) then
               aer_diffuse_mask(iv,ng) = .TRUE.  
               aer_advect_mask(iv,ng) = .TRUE.    
               aer_precip_mask(iv,ng) = .TRUE.   
               aer_nucleate_mask(iv,ng) = .TRUE. 
            else if(CO2i_tog == 2 .and. enable_aerosol_microphysics) then
               aer_diffuse_mask(iv,ng) = .TRUE.  
               aer_advect_mask(iv,ng) = .TRUE.
               aer_precip_mask(iv,ng) = .TRUE.    
               aer_sfcsrc_mask(iv,ng) = .TRUE.   
               aer_nucleate_mask(iv,ng) = .TRUE.
            end if
         end if
      end do
   end do
end if

do ng=1,n_grids
   do iv=1,naersclr(ng)

      if(aer_tyet(aer_group(iv))) then  ! Begin this specie's process(es) 
         if(aer_diffuse_mask(iv,ng) .and. .NOT.aer_XOdiffuse(iv,ng)) then
            aer_XOdiffuse(iv,ng) = .TRUE.
         endif
         if(aer_advect_mask(iv,ng) .and. .NOT.aer_XOadvect(iv,ng)) then
            aer_XOadvect(iv,ng) = .TRUE.
         endif
         if(aer_precip_mask(iv,ng) .and. .NOT.aer_XOprecip(iv,ng)) then
            aer_XOprecip(iv,ng) = .TRUE.
         endif
         if(aer_nucleate_mask(iv,ng) .and. .NOT.aer_XOnucleate(iv,ng) .and.  &
              ( (aer_tyet(2) .and. H2Oi_tog > 0) .or.  &
              (aer_tyet(3) .and. CO2i_tog > 0) )) then ! Don't nucleate if H2Oi
            aer_XOnucleate(iv,ng) = .TRUE.             ! or CO2i not yet active
         endif
      endif

      if(aer_sfcsrc_mask(iv,ng) .and. .NOT.aer_XOsfcsrc(iv,ng)) then
         if(aer_csr_tyet(aer_group(iv)) .or. aer_inj_tyet(aer_group(iv))) then 
            aer_XOsfcsrc(iv,ng) = .TRUE.
         endif
      endif

      if(aer_radiate_mask(iv,ng) .and. .NOT.aer_XOradiate(iv,ng)) then
         if(aer_rad_tyet(aer_group(iv))) then  ! Begin this specie's RT       
            aer_XOradiate(iv,ng) = .TRUE.
         endif
      endif

   enddo
enddo

end subroutine aerproc_check


!```````````````````````````````````````````````````````````````````
subroutine aer_setup

!+ Setup PRAMS aerosol array dimensions, descriptors, etc.

use univ_IO_mem_status_mod, only: for_allocation
use aermic_rad_mem_mod, only: aermicrad_AD,aer_group,aer_subgrp,aer_ngrp  &
     ,aer_grpbins,aer_grpelems,aer_ie,aer_ib,aer_ngasses,aer_nelements  &
     ,aer_ngroups,aer_nsolutes,aer_grpname,aer_elemname,aer_gasgrp  &
     ,aer_grpgas,aer_ig,aer_maxnbins
use main_memory_mod, only: n_grids
use main_memory_bspec_mod, only: dust_nb, H2Oi_nb, CO2i_nb, H2Ov_tog, CO2i_tog
use vtables_mod, only: naersclr,iaerschrp

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: iv,ng,ig,bcnt,ecnt,sgh,gph,igrp,iq,gcnt
integer, dimension(aer_ngrp) :: cnt_s
character(len=8) :: cname

!=== Executable statements:

! Define possible aerosol groups/species:

     ! aer_grpname(aer_group)
aer_grpname(1) = 'Dust particles (CN)'
aer_grpname(2) = 'H2O ice particles'
aer_grpname(3) = 'CO2 ice particles'

     ! aer_elemname(aer_group,aer_subgrp):
aer_elemname(1,0) = 'Dust (CN) particles [#conc]'
aer_elemname(2,0) = 'H2O ice particles [#conc]'
aer_elemname(2,1) = 'Dust mass within H2O ice particles [mconc]'
aer_elemname(3,0) = 'CO2 ice particles [#conc]'
aer_elemname(3,1) = 'Dust mass within CO2 ice particles [mconc]'
aer_elemname(3,2) = 'H2O ice mass within CO2 ice particles [mconc]'

! Calculate total number of gasses needed by configuration:

aer_ngasses = 0
if(H2Ov_tog > 0) then
   aer_ngasses = aer_ngasses+1     ! Allow for H2O vapor
endif
if(CO2i_tog > 0) then  ! Allow for CO2 (primary atm constituent on present Mars
   aer_ngasses = aer_ngasses+1
endif
!^print *,'aer_ngasses',aer_ngasses

call aermicrad_AD(for_allocation,1)     ! Allocate relevant arrays (phase 1)

! Assign proper aerosol specie subgroup and group IDs, etc.:

cnt_s = 0
do ng=1,n_grids    ! Loop over all grids
   do iv=1,naersclr(ng)     ! Loop over all aerosol scalars
      cname = iaerschrp(iv,ng)   ! Current aerosol scalar name 
      if(cname(1:4) == 'DUST') then
         aer_group(iv) = 1   ! Dust species group
         aer_subgrp(iv) = 0   ! Number concentration of dust
      elseif(cname(1:3) == 'H2O') then
         aer_group(iv) = 2   ! H2O ice species group
         if(cname(4:4) == 'I') then
            aer_subgrp(iv) = 0   ! Number concentration of H2O ice
         elseif(cname(4:4) == 'D') then
            aer_subgrp(iv) = 1   ! Mass concentration of core dust
         endif
      elseif(cname(1:3) == 'CO2') then
         aer_group(iv) = 3   ! CO2 ice species group
         if(cname(4:4) == 'I') then
            aer_subgrp(iv) = 0   ! Number concentration of CO2 ice
         elseif(cname(4:4) == 'D') then
            aer_subgrp(iv) = 1   ! Mass concentration of core dust
         elseif(cname(4:4) == 'W') then
            aer_subgrp(iv) = 2   ! Mass concentration of core H2O ice
         endif
      endif
      if(ng == 1) then   ! Only count once
         cnt_s(aer_group(iv)) = cnt_s(aer_group(iv))+1
      endif
   enddo
enddo

aer_grpbins(1) = dust_nb   !
aer_grpbins(2) = H2Oi_nb   ! Number of bins per element in each group
aer_grpbins(3) = CO2i_nb   !
!^print *,'aer_grpbins',aer_grpbins

! Create mapping arrays between aersclr var# and microphysics code element# and
!  bin#:

bcnt = 0
ecnt = 0
gcnt = 0
sgh = -1
gph = -1
do iv=1,naersclr(1) ! Use grid 1 scalar count (should be the same on all grids)
   bcnt = bcnt+1   ! Increment bin counter
   igrp = aer_group(iv)     ! PRAMS aerosol group ID#

   if(igrp /= gph) then   ! Increment group counter at group boundary
      gcnt = gcnt+1
   endif

   ! If PRAMS aerosol subgroup boundary, (re)set counters appropriately:
   if(aer_subgrp(iv) /= sgh .or. igrp /= gph) then
      sgh = aer_subgrp(iv)       
      gph = igrp
      bcnt = 1
      ecnt = ecnt+1
   endif

   aer_ie(iv) = ecnt    ! microphysics code element ID#
   aer_ib(iv) = bcnt   ! microphysics code bin ID#
   aer_ig(iv) = gcnt
!^   print *,'aer_ig,ie,ib',iv,aer_ig(iv),aer_ie(iv),aer_ib(iv)
enddo

! Assign number of elements per group; also set some microphysics code
!  dimensions (total # of groups, elements, solutes, and gasses):

ng = 0
iv = 0
aer_maxnbins = 0
do ig=1,aer_ngrp   ! Number of possible groups
   if(cnt_s(ig) > 0) then
      ng = ng+1
   endif
   if(aer_grpbins(ig) /= 0) then     ! This group used in this configuration
      aer_grpelems(ig) = cnt_s(ig)/aer_grpbins(ig)
      aer_maxnbins = max(aer_maxnbins,aer_grpbins(ig)) ! Max # of bins in a grp
   else               ! This group not used in the current configuration
      aer_grpelems(ig) = 0
   endif
   iv = iv+aer_grpelems(ig)
enddo
!^print *,'aer_grpelems',aer_grpelems

aer_ngroups = ng     ! Total number of groups needed by configuration
aer_nelements = iv   ! Total number of elements needed by configuration
aer_nsolutes = 0     ! No solutes (for now, at least)
!^print *,'aer_ngrp,ne,nsol',aer_ngroups,aer_nelements,aer_nsolutes

! Set gas types (corresponding aerosol group and vice versa):

aer_grpgas = 0      ! Assume no gasses (temporary; highlights missing gasses)

iq = 0
if(H2Ov_tog > 0) then
   iq = iq+1 
   aer_gasgrp(iq) = 2
   aer_grpgas(2) = iq
endif
if(CO2i_tog > 0) then
   iq = iq+1 
   aer_gasgrp(iq) = 3
   aer_grpgas(3) = iq
endif
!^print *,'aer_gasgrp',aer_gasgrp
!^print *,'aer_grpgas',aer_grpgas

! Set aerosol microphysics model parameters from PRAMS config, etc.:

call aer_intfc_setup

end subroutine aer_setup


!```````````````````````````````````````````````````````````````````
subroutine aer_intfc_setup

!+ Fill aerosol microphysics model parameters from PRAMS vars, etc.

use univ_kind_defs_mod, only: f4
use univ_convert_units_mod, only: um_to_m_f4, m_to_cm_f4 
use univ_math_constants_mod, only: ONE_f4, FOUR_THIRDS_f4, ONE_THIRD_f4
use univ_IO_mem_status_mod, only: for_allocation
use aermic_rad_mem_mod, only: aer_group,aer_subgrp  &
     ,aer_grpbins,aer_grpelems,aer_ie,aer_ngroups,aer_nelements  &
     ,aer_nsolutes,aer_ngasses,aer_ig,aer_grpname,aer_elemname,aer_grpgas  &
     ,aer_gasgrp,iv_is_dust,iv_ndist,iv_distID,iv_distfrac,aermicrad_AD
use main_memory_bspec_mod, only: dust_rmn, dust_mrt, H2Oi_rmn, H2Oi_mrt,  &
     CO2i_rmn, CO2i_mrt
use vtables_mod, only: naersclr
use aermicro_mem_mod, only: ngroups,ngasses,nelements,nsolutes,aermicro_AD,nbins  &
     ,igelem,ienconc,nelemg,groupname,elemname,icomp,itype,rhoelem,rmin  &
     ,rmrat,ishape,eshape,igrowgas,is_grp_ice,is_grp_mixed,inucgas,ievp2elem  &
     ,nnuc2elem,inuc2elem,if_nuc,inucproc,gasname,gwtmol,I_DUST,I_H2OICE  &
     ,I_CO2ICE,I_INVOLATILE,I_COREMASS,I_VOLATILE,I_VOLCORE,I_DROPACT  &
     ,max_nbins,sifreeze,pi_m,rmassmin  &
     ,rmass,rmassup,r,rup,rlow,dr,kg_m3_to_g_cm3,RHO_I,RHO_IC
use body_specific_constants_mod, only: rho_dust

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ie,iq,ig,iagp,iasg,iv,ie2,inuc,ib
real(f4) :: c1,vrfact

!=== Executable statements:

! Sample setup of particle types and the corresponding mapping arrays:
!
!    CN: Dust
!    Cloud hydrometeors: CO2 ice, H2O ice
!    Core masses: dust mass in H2O ice, dust mass in CO2 ice,
!                 H2O ice mass in CO2 ice
!
!    group 1   Dust                               element 1   (#/cm^3)
!    group 2   H2O ice                            element 2   (#/cm^3)
!              mass of dust in cloud crystals     element 3   (g/cm^3)
!    group 3   CO2 ice                            element 4   (#/cm^3)
!              mass of dust in cloud crystals     element 5   (g/cm^3)
!              mass of H2O ice in cloud crystals  element 6   (g/cm^3)
!
!    ngasses = 2
!        gas(1) = H2O vapor
!        gas(2) = carbon dioxide     
!
!    ngroups = 3
!    nelements = 6
!    nelemg  = (1,2,3)
!    itype   = (0,1,2,1,2,3)
!    ienconc = (1,2,4)
!    igelem  = (1,2,2,3,3,3)

ngroups = aer_ngroups   ! Total number of groups
nelements = aer_nelements   ! Total number of elements
nsolutes = aer_nsolutes     ! No solutes (for now, at least)
ngasses = aer_ngasses    ! Total number of nucleation-active gasses

call aermicro_AD(for_allocation,1)           ! Allocate Phase 1 aer_micro code memory

ie = 0
do iq=1,naersclr(1)   ! Use grid 1 scalar count (should be same on all grids)
   if(aer_ie(iq) /= ie) then    ! Next element

      iagp = aer_group(iq)
      iasg = aer_subgrp(iq)

      ie = aer_ie(iq)    ! Aerosol microphysics code indices of current var(iq)
      ig = aer_ig(iq)    !

      nbins(ig) = aer_grpbins(iagp)   ! Total number of bins per element

      igelem(ie) = ig      ! [element -> group] mapping 
      if(aer_subgrp(iq) == 0) then
         ienconc(ig) = ie    ! [group -> #conc_element] mapping 
      endif

      ! Number of elements in each group (elements in a group can include
      !  particle number concentration, mass concentrations of cores, and
      !  second moments of core mass distributions):

      nelemg(ig) = aer_grpelems(iagp)

      ! Name for each aerosol group:

      groupname(ig) = aer_grpname(iagp)   ! Group name

      ! Name for each aerosol element:

      elemname(ie) = aer_elemname(iagp,iasg)  ! Element name

      ! This array specifies the composition of each element:
      !    I_H2SO4    ==> sulfuric acid
      !    I_WATER    ==> liquid water
      !    I_H2OICE   ==> water ice
      !    I_CO2ICE   ==> carbon dioxide ice
      !    I_DUST     ==> dust (inorganic mineral composition)
      !    I_MIXEDWAT ==> mixed phase (ice/liquid) water

      if(iagp == 1 .or. iasg == 1) then
         icomp(ie) = I_DUST
      elseif(iagp == 2 .or. iasg == 2) then
         icomp(ie) = I_H2OICE
      elseif(iagp == 3) then
         icomp(ie) = I_CO2ICE
      endif

      ! This array specifies the type of each element:
      !    I_INVOLATILE ==> CN (involatile) number concentration [#/cm^3]
      !    I_VOLATILE   ==> volatile particle number concentration [#/cm^3]
      !    I_COREMASS   ==> core mass concentration [g/cm^3] of involatile core
      !    I_VOLCORE    ==> core mass concentration [g/cm^3] of a volatile core
      !    I_CORE2MOM   ==> core second moment (mass^2) [g^2/cm^3]

      if(iagp == 1) then
         itype(ie) = I_INVOLATILE
      elseif(iasg == 1) then
         itype(ie) = I_COREMASS
      elseif(iagp == 2 .and. iasg == 0) then
         itype(ie) = I_VOLATILE
      elseif(iasg == 2) then
         itype(ie) = I_VOLCORE
      elseif(iagp == 3 .and. iasg == 0) then
         itype(ie) = I_VOLATILE
      endif

      ! Mass density [g/cm^3] of each particle element (for elements of
      !  <itype> = I_VOLATILE, <rhoelem> is the density of the shell):
      
      if(iagp == 1 .or. iasg == 1) then
         rhoelem(ie) = rho_dust*kg_m3_to_g_cm3
      elseif(iagp == 2 .or. iasg == 2) then
         rhoelem(ie) = RHO_I
      elseif(iagp == 3) then
         rhoelem(ie) = RHO_IC
      endif

      ! Minimum particle radius [cm] for each group (used below to calculate
      !  <rmassmin>) AND ratio of particle mass between successive bins:

      if(iagp == 1) then
         rmin(ig) = dust_rmn*um_to_m_f4*m_to_cm_f4    ! [um -> cm]
         rmrat(ig) = dust_mrt
      elseif(iagp == 2) then
         rmin(ig) = H2Oi_rmn*um_to_m_f4*m_to_cm_f4   ! [um -> cm]  
         rmrat(ig) = H2Oi_mrt
      elseif(iagp == 3) then
         rmin(ig) = CO2i_rmn*um_to_m_f4*m_to_cm_f4   ! [um -> cm]
         rmrat(ig) = CO2i_mrt
      endif

      ! The values of <ishape> and <eshape> determine particle geometry for
      !  each aerosol group:
      !
      !    <ishape> = 1: spherical
      !    <ishape> = 2: hexagonal prisms or plates
      !    <ishape> = 3: circular disks, cylinders, or spheroids
      !
      !    <eshape> = particle length/diameter

      if(iagp == 1) then
         ishape(ig) = 1
         eshape(ig) = ONE_f4
      elseif(iagp == 2) then
         ishape(ig) = 1
         eshape(ig) = ONE_f4
      elseif(iagp == 3) then
         ishape(ig) = 1
         eshape(ig) = ONE_f4
      endif

      ! Array <igrowgas> maps a particle element to its associated gas for
      !  condensational growth/evaporation.
      !
      ! Set to zero if there is no growth specific to particle element
      !  (i.e., use zero if cores do not grow, even if they are a component
      !  of a particle group that does grow; use zero for core second moment).
      !
      !  *** Condensational growth of cores is not presently treated. ***

      if(iagp == 1) then
         igrowgas(ie) = 0
      elseif(iasg == 1) then
         igrowgas(ie) = 0
      elseif(iagp == 2 .and. iasg == 0) then
         igrowgas(ie) = aer_grpgas(iagp)
      elseif(iasg == 2) then
         igrowgas(ie) = 0
      elseif(iagp == 3 .and. iasg == 0) then
         igrowgas(ie) = aer_grpgas(iagp)
      endif

      ! If <is_grp_ice> = .TRUE. then the particle group is an ice crystal,
      !  else the particle group is liquid (or does not grow).  This array
      !  is used to select the appropriate ventilation factors in `setupgkern`.
      !
      ! If <is_grp_mixed> = .TRUE. then the particle group is a mixed
      !  ice/liquid hydrometeor. This array is used to select processes (such as
      !  core melting) that only occur in mixed particles.

      if(iagp == 1) then
         is_grp_ice(ig) = .FALSE.
         is_grp_mixed(ig) = .FALSE.
      elseif(iagp == 2) then
         is_grp_ice(ig) = .TRUE.
         is_grp_mixed(ig) = .FALSE.
      elseif(iagp == 3) then
         is_grp_ice(ig) = .TRUE.
         is_grp_mixed(ig) = .FALSE.
      endif

   endif
enddo

!^print *,'nbins',nbins
!^print *,'nelemg',nelemg
!^print *,'groupname',groupname
!^print *,'elemname',elemname
!^print *,'icomp',icomp
!^print *,'itype',itype
!^print *,'rhoelem',rhoelem
!^print *,'rmin',rmin
!^print *,'rmrat',rmrat
!^print *,'ishape',ishape
!^print *,'eshape',eshape
!^print *,'igrowgas',igrowgas
!^print *,'is_grp_ice',is_grp_ice
!^print *,'is_grp_mixed',is_grp_mixed

!  Nucleation mapping:
!
!  <nnuc2elem(ielem)> is the total number of target elements from which
!  source particles in element <ielem> are nucleated.
!
!  Nucleation transfers particle mass from element <ielem> to element
!  <inuc2elem(i,ielem)>, where <i> ranges from 0 to <nnuc2elem(ielem)>.
!
!  Set <nnuc2elem> and <inuc2elem> to zero if mass is not transferred by
!   nucleation
!
!  <if_nuc(iefrom,ieto)> is true if nucleation transfers mass from element
!   <iefrom> to element <ieto>.
!
!  <inucproc(iefrom,ieto)> specifies what nucleation process nucleates
!   particles from element <iefrom> to element <ieto>:
!     I_DROPACT:  Aerosol activation to droplets
!     I_AERFREEZE: Aerosol homogeneous freezing
!     I_DROPFREEZE: Droplet homogeneous freezing
!     I_MIXEDFREEZE: Mixed total freezing
!     I_MIXEDMELT: Mixed total melting
!   Set to zero if particles are not subject to nucleation.
!
! Total evaporation mapping: total evaporation transfers particle mass from
!  element <ielem> to element <ievp2elem(ielem)>. Set to zero if element is not
!  subject to total evaporation.
!
!  This array is not automatically derived from <inuc2elem> because multiple
!  elements can nucleate to a particular element (reverse mapping is not
!  unique).

do ie=1,nelements     ! Find dust CN element (iq)
   ig = igelem(ie)
   if(groupname(ig)(1:1) == 'D') then
      if(elemname(ie)(1:1) == 'D') then
         iq = ie
      endif
   endif
enddo

inuc2elem = 0     ! Initialize entire arrays 
nnuc2elem = 0     !
if_nuc = .FALSE.  !
inucproc = 0      !
do ie=1,nelements
   ig = igelem(ie)

   if(groupname(ig)(1:1) == 'D') then
      ievp2elem(ie) = 0
      do ie2=1,nelements
         if(elemname(ie2)(1:1) == 'D' .and. ie2 /= ie) then
            nnuc2elem(ie) = nnuc2elem(ie)+1
            inuc2elem(nnuc2elem(ie),ie) = ie2
            if_nuc(ie,ie2) = .TRUE.
            inucproc(ie,ie2) = I_DROPACT
         endif
      enddo
   elseif(groupname(ig)(1:1) == 'H') then
      if(ienconc(ig) == ie) then
         ievp2elem(ie) = iq
      elseif(elemname(ie)(1:1) == 'D') then
         ievp2elem(ie) = iq
      endif
   elseif(groupname(ig)(1:1) == 'C') then
      if(ienconc(ig) == ie) then
         ievp2elem(ie) = iq
      elseif(elemname(ie)(1:1) == 'D') then
         ievp2elem(ie) = iq
      else
         ievp2elem(ie) = 0
      endif
   endif

enddo
!^
!^print *,'nnuc2elem',nnuc2elem
!^do ie=1,nelements
!^   do ie2=1,nnuc2elem(ie)
!^      print *,'nn,ie,inuc2elem',ie2,ie,inuc2elem(ie2,ie)
!^   enddo
!^enddo
!^
!^do ie=1,nelements
!^   do ie2=1,nelements
!^      print *,'dddd',ie,ie2
!^      print *,'if_nuc',if_nuc(ie,ie2)
!^      print *,'inucproc',inucproc(ie,ie2)
!^   enddo
!^enddo
!^print *,'ievp2elem',ievp2elem
!pause
!^

! Array <inucgas> maps a particle group to its associated gas for
!  nucleation:
!
!* Nucleation from group <igroup>, nucleation path ID# <inuc>, is
!   associated with gas <inucgas(inuc,igroup)>.
!* Set to zero if particles are not subject to nucleation.

inucgas = 0     ! Initialize entire array      
do ig=1,ngroups
   ie = ienconc(ig)
   do inuc=1,nnuc2elem(ie)   ! Loop over all nucleation paths
      ie2 = inuc2elem(inuc,ie)   ! Element to transfer some of mass in <ie> to
      ! This may not hold true for all cases (???)
      inucgas(inuc,ig) = igrowgas(ienconc(igelem(ie2)))
   enddo
enddo
!^
!^do ig=1,ngroups
!^   ie = ienconc(ig)
!^   do inuc=1,nnuc2elem(ie)
!^      print *,'inuc,ig,inucgas',inuc,ig,inucgas(inuc,ig)
!^   enddo
!^enddo
!^

! Set up solute properties and mapping arrays:

! Solute name (one for each solute):

!%solname(1) = 'none'

! Solute molecular weights (one for each solute) [g/mole]:

!%solwtmol(1) = 98.

! Solute mass densities (one for each solute) [g/cm^3]:

!%rhosol(1) = 1.38

! Number of ions that solute dissociates into (one for each solute):

!%sol_ions(1) = 2.

! Solute mapping: particle element <ielem> is composed of solute
!  <isolelem(ielem)>.  Should only be non-zero for elements of
!  itype = I_INVOLATILE [involatile number concentration] or 
!  itype = I_COREMASS [core mass concentration]):

!%isolelem(1) = 1
!%isolelem(2) = 0
!%isolelem(3) = 1
!%isolelem(4) = 0
!%isolelem(5) = 0
!%isolelem(6) = 1
!%isolelem(7) = 0
!%isolelem(8) = 1

max_nbins = maxval(nbins)    ! Max number of bins in any group's element
!^print *,'max_nbins',max_nbins,nbins
!^pause

call aermicrad_AD(for_allocation,2)  ! Allocate max_bins-dep arrays (phase 2)
call aermicro_AD(for_allocation,2)   ! Allocate particle radius/mass stuff

! Calculate particle masses/radii for each bin (simplified calc from that in
!  the aerosol code itself -- but need accurate radii, etc. for radiative
!  transfer and surface process code initialization):

c1 = FOUR_THIRDS_f4*pi_m
do ig=1,ngroups
   ie = ienconc(ig)

   rmassmin(ig) = c1*rhoelem(ie)*rmin(ig)**3
   vrfact = ((1.5_f4/pi_m/(rmrat(ig)+ONE_f4))**ONE_THIRD_f4)*  &
        (rmrat(ig)**ONE_THIRD_f4-ONE_f4)

   do ib=1,nbins(ig)
           
      rmass(ib,ig) = rmassmin(ig)*rmrat(ig)**real((ib-1))     ! [g]
      rmassup(ib,ig) = 2._f4*rmrat(ig)/(rmrat(ig)+ONE_f4)*rmass(ib,ig)   ! [g]
      r(ib,ig) = (rmass(ib,ig)/rhoelem(ie)/c1)**ONE_THIRD_f4   ! [cm]
      rup(ib,ig) = (rmassup(ib,ig)/rhoelem(ie)/c1)**ONE_THIRD_f4   ! [cm] 
      dr(ib,ig) = vrfact*(rmass(ib,ig)/rhoelem(ie))**ONE_THIRD_f4   ! [cm]
      rlow(ib,ig) = rup(ib,ig)-dr(ib,ig)        ! [cm]

!^      print *,'ig,ib',ig,ib
!^      print *,'rmass..',rmass(ib,ig),rmassup(ib,ig)
!^      print *,'r(cm)...',r(ib,ig),rup(ib,ig),rlow(ib,ig),dr(ib,ig)
!^      print *,'r(um)...',r(ib,ig)*1.e4,rup(ib,ig)*1.e4,rlow(ib,ig)*1.e4  &
!^           ,dr(ib,ig)*1.e4
   enddo
enddo

! Display bin info to user:

write (*,*)
write (*,'(i2)') ngroups
do ig=1,ngroups
   write (*,'(i2)') nbins(ig)
enddo
do ig=1,ngroups
   write (*,'(a,a,i1,a)') groupname(ig)(1:len_trim(groupname(ig)))  &
        ,' (ig = ',ig,')'
enddo
write (*,'(/a)') ' ig  ib   mass [g]       r[um]       dr[um]'
do ig=1,ngroups
   do ib=1,nbins(ig)
      write (*,'(1x,i2,2x,i2,2x,e10.4,2(2x,f10.5))') ig,ib,rmass(ib,ig)  &
           ,r(ib,ig)*1.e4_f4,dr(ib,ig)*1.e4_f4
   enddo
enddo
write (*,*)

! Names and molecular weights of gas species:

do iq=1,ngasses
   iv = aer_gasgrp(iq)
   if(iv == 2) then
      gasname(iq) = 'Water vapor'
      gwtmol(iq) = 18.01_f4          ! [g/mol]
      sifreeze(iq) = 0.01_f4    ! Critical supersaturation required for nucleation
   elseif(iv == 3) then
      gasname(iq) = 'Carbon dioxide'
      gwtmol(iq) = 44.01_f4          ! [g/mol]
      sifreeze(iq) = 0.2_f4     ! Critical supersaturation required for nucleation
   endif
enddo

!^print *,'gasname',gasname
!^print *,'gwtmol',gwtmol

! Calculate aerosol sfc -> air injection / initialization parameters, etc.:

do iv=1,naersclr(1)
   ie = aer_ie(iv)
   ig = aer_ig(iv)
   if(ienconc(ig) == ie .and. elemname(ie)(1:1) == 'D') then
      iv_is_dust(iv) = .TRUE.
      iv_ndist(iv) = 1  ! # of separate distributions this aerosol scalar has 

      ! Dust "saltation" distribution (modal radius = 1.5um, rsig = 1.8):
      iv_distID(1,iv) = 1
      iv_distfrac(1,iv) = ONE_f4
      !%iv_distfrac(1,iv) = 0.5
      call aer_init_mdist_logn(1.5e-6_f4, 1.8_f4, 1, nbins(ig),ig)

      !%! Dust "other" distribution (modal radius = 0.8um, rsig = 1.6):
      !%ie_distID(2,iv) = 2
      !%iv_distfrac(2,iv) = 0.5
      !%call aer_init_mdist_logn(0.8e-6, 1.6, 2, nbins(ig),ig)
   else
      iv_is_dust(iv) = .FALSE.
      iv_ndist(iv) = 0  ! Signifies that this aerosol scalar should not be
                        !  injected or initialized
   
   endif
enddo

end subroutine aer_intfc_setup


!```````````````````````````````````````````````````````````````````
subroutine aer_injection(m1,m2,m3,m5,ia,iz,ja,jz,aer_iflux,aer_iaccum  &
     ,aer_rsv,rtgt,dn0,naerscl,dust_tmflux,air_massdensity,air_theta  &
     ,air_piocp,ustar,up,vp,sfc_tempk,thp)

!+ Parameterizes injection of aerosols into the atmosphere

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use aermic_rad_mem_mod, only: aer_ib,iv_ndist,iv_distID,iv_distfrac  &
     ,aer_mdist_bm2n,aer_mdist_bfrac,iv_is_dust,ust_tab_nent_T,ustar_t_min  &
     ,ust_tab_nent_rhof,lmflx_tab_nent_ut,lmflx_tab_nent_u,n_weibull  &
     ,intmflux,aer_inj_tyet,laminar_only
use main_memory_mod, only: richnum => vctr1, jdim, c_gd
use main_memory_bspec_mod, only: dust_tog, dust_jth, dust_jef
use body_specific_constants_mod, only: rho_dust, g_Xoid

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1,m2,m3,m5,ia,iz,ja,jz,naerscl
real(f4), dimension(m2,m3), intent(IN) :: rtgt,air_massdensity  &
     ,air_theta,air_piocp
real(f4), dimension(m1,m2,m3), intent(IN) :: dn0,up,vp,thp
real(f4), dimension(m2,m3,m5), intent(IN) :: ustar,sfc_tempk

real(f4), dimension(m2,m3,naerscl), intent(INOUT) :: aer_iflux
real(f4), dimension(m2,m3,naerscl), intent(INOUT) :: aer_iaccum,aer_rsv
real(f4), dimension(m2,m3), intent(INOUT) :: dust_tmflux

!== Local declarations:

! Aerodynamic "surface" stress threshold for dust lifting via saltation [N]:
!real(f4), parameter :: dust_saltation_threshold = 0.014_f4  &
!                            ,dust_saltation_alpha = 2.431e-5_f4  ! [s/(N.m^3)]

! Dust lookup table bounds, increments:                     
real(f4), parameter :: ust_tab_T_min = 50._f4, ust_tab_T_max = 350._f4  &
                     ,ust_tab_T_inc = ONE_f4  &  ! [K]
                     ,ust_tab_rhof_min = 1.e-4_f4, ust_tab_rhof_max = 0.2_f4  &
                     ,ust_tab_rhof_inc = 1.e-4_f4  ! [kg/m^3]

! Particle lifting flux lookup table bounds, increments:                     
real(f4), parameter :: lmflx_tab_u_min = 0.01_f4, lmflx_tab_u_max = 20.01_f4  &
                     ,lmflx_tab_u_inc = 0.1_f4  &   ! [m/s]
                     ,lmflx_tab_ut_min = 0.1_f4, lmflx_tab_ut_max = 15._f4  &
                     ,lmflx_tab_ut_inc = 0.1_f4   ! [m/s]

! Dust lifting "efficiency" (for ustar_t scheme):
!real(f4), parameter :: eff_ustar_scheme = 9.e-8_f4

real(f4), parameter :: kg_to_g = 1.e3_f4  &  ! [kg -> g] conversion factor
                      ,m_to_cm = 1.e2_f4    ! [m -> cm] conversion factor

integer :: i,j,iv,ib,iq,id,i_itp1,i_itp2,idx_gust,k
real(f4) :: dust_tmassflux,dist_massflux,aer_numflux,aer_massflux  &
     ,aer_DnumconcDt,atmp,ftmp,ftmp1,ftmp2,f_itp1,f_itp2,stress,tempk_air  &
     ,ustar_t,part_tmassflux_fact,u_avg0,v_avg0,u_avg1,v_avg1,dz,dtheta  &
     ,theta_avg,du,dv,dusq,dvsq,sfc_theta,f_itp0,part_tmassflux_fact0
logical :: gust_itp

integer, save :: ncall = 0

!=== Executable statements:

if(ncall == 0) then   ! Initialize lookup table, etc.
   if(dust_tog == 4) then    ! Ustar threshold scheme
      laminar_only = ( dust_jef < ZERO_f4)  ! Use "laminar" PDF only?
      if(laminar_only) then
         dust_jef = abs(dust_jef)
      endif

      ! Generate ustar_t lookup table for DUST particles:
      ust_tab_nent_T = nint((ust_tab_T_max-ust_tab_T_min)/ust_tab_T_inc)
      ust_tab_nent_rhof =  &
           nint((ust_tab_rhof_max-ust_tab_rhof_min)/ust_tab_rhof_inc)
      allocate(ustar_t_min(ust_tab_nent_rhof,ust_tab_nent_T))

      write (*,'(/a)') '***Using ustar threshold dust lifting scheme:'
      write (*,'(a)') 'Generating ustar_t lookup table...'
      call pflift_ustar_t_table(rho_dust,ust_tab_T_min,ust_tab_T_inc  &
           ,ust_tab_rhof_min,ust_tab_rhof_inc)

      ! Generate mass lifting flux lookup table for ALL particles:
      lmflx_tab_nent_u =  &
           nint((lmflx_tab_u_max-lmflx_tab_u_min)/lmflx_tab_u_inc)
      lmflx_tab_nent_ut =  &
           nint((lmflx_tab_ut_max-lmflx_tab_ut_min)/lmflx_tab_ut_inc)
      allocate(intmflux(lmflx_tab_nent_ut,lmflx_tab_nent_u,n_weibull))

      if(.NOT.laminar_only) then
         write (*,'(a)')   &
              'Generating mass flux factor lookup table (very gusty)...'
         call pflift_intmflux_table(lmflx_tab_u_min,lmflx_tab_u_inc  &
              ,lmflx_tab_ut_min,lmflx_tab_ut_inc,1.2_f4,1)   ! Very gusty
         write (*,'(a)')   &
              'Generating mass flux factor lookup table (some weak gusts)...'
         call pflift_intmflux_table(lmflx_tab_u_min,lmflx_tab_u_inc  &
              ,lmflx_tab_ut_min,lmflx_tab_ut_inc,4._f4,2)   ! Some weak gusts
      endif
      write (*,'(a)')   &
           'Generating mass flux factor lookup table (nearly laminar flow)...'
      call pflift_intmflux_table(lmflx_tab_u_min,lmflx_tab_u_inc  &
           ,lmflx_tab_ut_min,lmflx_tab_ut_inc,30._f4,3)   ! Nearly laminar flow
      write (*,'(a/)') 'Done.'
   endif

   ncall = 1
endif

! Calculate total dust mass lifting rate:

if(dust_tog == 3) then  ! Constant stress threshold scheme (Haberle et al 2003)

   do j=ja,jz
      do i=ia,iz

         stress = ustar(i,j,1)*ustar(i,j,1)*air_massdensity(i,j)

         if(stress > dust_jth) then
            ! Dust lifting rate [kg/(s.m^2]:
            dust_tmassflux = dust_jef*stress*stress*  &
                 (stress-dust_jth)/dust_jth 
            dust_tmflux(i,j) = dust_tmassflux*kg_to_g/(m_to_cm**2) ![g/(s.cm^2)
         else
            dust_tmflux(i,j) = ZERO_f4      ! [g/(s.cm^2)
         endif
         
      enddo
   enddo

elseif(dust_tog == 4) then    ! Ustar threshold scheme

   do j=ja,jz
      do i=ia,iz   

         ! Determine current ustar_t:

         ftmp = min(max(air_massdensity(i,j),ust_tab_rhof_min)  &
              ,ust_tab_rhof_max-ust_tab_rhof_inc*1.001_f4)
         ftmp1 = (ftmp-ust_tab_rhof_min)/ust_tab_rhof_inc
         i_itp1 = int(ftmp1)+1
         f_itp1 = ftmp1-real(i_itp1-1)

         tempk_air = air_theta(i,j)*air_piocp(i,j)
         ftmp = min(max(tempk_air,ust_tab_T_min)  &
              ,ust_tab_T_max-ust_tab_T_inc*1.001_f4)
         ftmp2 = (ftmp-ust_tab_T_min)/ust_tab_T_inc
         i_itp2 = int(ftmp2)+1
         f_itp2 = ftmp2-real(i_itp2-1)

         ftmp1 = ustar_t_min(i_itp1,i_itp2)*(ONE_f4-f_itp1)+  &
              ustar_t_min(i_itp1+1,i_itp2)*f_itp1
         ftmp2 = ustar_t_min(i_itp1,i_itp2+1)*(ONE_f4-f_itp1)+  &
              ustar_t_min(i_itp1+1,i_itp2+1)*f_itp1

         ustar_t = ftmp1*(ONE_f4-f_itp2)+ftmp2*f_itp2

         ! Determine current gustiness regime:

         if(laminar_only) then

            idx_gust = 3
            gust_itp = .FALSE.

         else
            
            k = 1
            u_avg0 = (up(k+1,i,j)+up(k+1,i-1,j))*0.5_f4    ! velocity components
            v_avg0 = (vp(k+1,i,j)+vp(k+1,i,j-jdim))*0.5_f4 ! avgd to "T"-pts [m/s]

            sfc_theta = sfc_tempk(i,j,1)/air_piocp(i,j)
            dz = c_gd%zt(k+1)*rtgt(i,j)   ! Implicitly '-0.'
            dtheta = thp(k+1,i,j)-sfc_theta
            theta_avg = 0.5_f4*(thp(k+1,i,j)+sfc_theta)
            du = u_avg0  ! Implicitly '-0.'
            dv = v_avg0  !
            dusq = max(1.e-4_f4,du*du)
            dvsq = max(1.e-4_f4,dv*dv)
            richnum(k) = g_Xoid*dz*dtheta/(theta_avg*(dusq+dvsq))  ! Ri("sfc")
            
            do while(.TRUE.)
               k = k+1
               dz = (c_gd%zt(k+1)-c_gd%zt(k))*rtgt(i,j)    ! dz of flux layer
               
               dtheta = thp(k+1,i,j)-thp(k,i,j)
               theta_avg = 0.5_f4*(thp(k+1,i,j)+thp(k,i,j))
               
               u_avg1 = (up(k+1,i,j)+up(k+1,i-1,j))*0.5_f4    ! velocity components
               v_avg1 = (vp(k+1,i,j)+vp(k+1,i,j-jdim))*0.5_f4 ! avgd to "T"-pts [m/s]
               du = u_avg1-u_avg0
               dv = v_avg1-v_avg0
               dusq = max(1.e-4_f4,du*du)
               dvsq = max(1.e-4_f4,dv*dv)
            
               richnum(k) = g_Xoid*dz*dtheta/(theta_avg*(dusq+dvsq))
               
               if(richnum(k) >= ZERO_f4 .or. k == m1-1) then
                  dz = c_gd%zm(k)*rtgt(i,j)
                  exit
               endif
            enddo
            
            if(richnum(2) < ZERO_f4) then  ! Strong to moderate gustiness
               
               if(dz >= 500._f4) then
                  idx_gust = 1
                  gust_itp = .FALSE.
               else
                  idx_gust = 1
                  f_itp0 = ONE_f4-dz/500._f4
                  gust_itp = .TRUE.
               endif

            elseif(richnum(2) <= 0.03_f4) then   ! Weak gustiness

               idx_gust = 2
               gust_itp = .FALSE.

            else     ! "Laminar"

               idx_gust = 3
               gust_itp = .FALSE.

            endif

         endif

         ! Determine current particle mass flux factor:

         ftmp = min(max(ustar_t,lmflx_tab_ut_min)  &
              ,lmflx_tab_ut_max-lmflx_tab_ut_inc*1.001_f4)
         ftmp1 = (ftmp-lmflx_tab_ut_min)/lmflx_tab_ut_inc
         i_itp1 = int(ftmp1)+1
         f_itp1 = ftmp1-real(i_itp1-1)

         ftmp = min(max(ustar(i,j,1),lmflx_tab_u_min)  &
              ,lmflx_tab_u_max-lmflx_tab_u_inc*1.001_f4)
         ftmp2 = (ftmp-lmflx_tab_u_min)/lmflx_tab_u_inc
         i_itp2 = int(ftmp2)+1
         f_itp2 = ftmp2-real(i_itp2-1)

         ftmp1 = intmflux(i_itp1,i_itp2,idx_gust)*(ONE_f4-f_itp1)+  &
              intmflux(i_itp1+1,i_itp2,idx_gust)*f_itp1
         ftmp2 = intmflux(i_itp1,i_itp2+1,idx_gust)*(ONE_f4-f_itp1)+  &
              intmflux(i_itp1+1,i_itp2+1,idx_gust)*f_itp1
         
         part_tmassflux_fact =  &
              ftmp1*(ONE_f4-f_itp2)+ftmp2*f_itp2 ! Gen mass flux factor
         
         if(gust_itp) then   ! Interpolate mass flux between regimes

            ! Determine current particle mass flux factor:

            ftmp1 = intmflux(i_itp1,i_itp2,idx_gust+1)*(ONE_f4-f_itp1)+  &
                 intmflux(i_itp1+1,i_itp2,idx_gust+1)*f_itp1
            ftmp2 = intmflux(i_itp1,i_itp2+1,idx_gust+1)*(ONE_f4-f_itp1)+  &
                 intmflux(i_itp1+1,i_itp2+1,idx_gust+1)*f_itp1
         
            part_tmassflux_fact0 =  &
                 ftmp1*(ONE_f4-f_itp2)+ftmp2*f_itp2 ! Gen mass flux factor

            ! Interpolate between 'idx_gust' and 'idx_gust+1' mass fluxes:
            part_tmassflux_fact =  &
                 part_tmassflux_fact*(ONE_f4-f_itp0)+part_tmassflux_fact0*f_itp0

         endif

         dust_tmassflux = dust_jef*  &  ! Dust lifting rate [kg/(s.m^2]
              sqrt(air_massdensity(i,j)*rho_dust)*part_tmassflux_fact

         dust_tmflux(i,j) = dust_tmassflux*kg_to_g/(m_to_cm**2) ![g/(s.cm^2)

      enddo
   enddo

endif

aer_iflux = ZERO_f4   ! Initialize entire array at once

do j=ja,jz
   do i=ia,iz

      dust_tmassflux = dust_tmflux(i,j)

      do iv=1,naerscl   ! Loop over all aerosol scalars

         if(iv_is_dust(iv)) then   ! This is a dust aerosol scalar
            ib = aer_ib(iv)
            do id=1,iv_ndist(iv) ! Loop over # of current dists for this scalar
               iq = iv_distID(id,iv)   ! ID of specific distribution to use
               
               ! Fraction of total mass flux in this distribution:
               dist_massflux = dust_tmassflux*iv_distfrac(id,iv)

               ! Mass flux of current aerosol scalar [g/(s.cm^2)]:
               aer_massflux = aer_mdist_bfrac(iq,ib)*dist_massflux

               ! Number flux of current aerosol scalar [#/(s.cm^2)]:
               aer_numflux = aer_massflux*aer_mdist_bm2n(iq,ib)

               ! Total amount of aerosol removed from surface [#/cm^2]
               !  this timestep (take diffusion code density weighting into
               !  account):
               atmp = aer_numflux*c_gd%dt_dyn*0.5_f4*(ONE_f4+dn0(1,i,j)/  &
                    dn0(2,i,j))

               aer_rsv(i,j,iv) = max(aer_rsv(i,j,iv),ZERO_f4)  ! Make sure >= 0.
               if(aer_rsv(i,j,iv) < atmp) then   ! Little or no mass to inject
                  ftmp = aer_rsv(i,j,iv)/max(atmp,1.e-20_f4)
                  atmp = ftmp*atmp                 ! Scale these to avoid 
                  aer_numflux = ftmp*aer_numflux   !  injecting "fake" mass
               endif

               ! Net amount of aerosol currently present on the surface in this
               !  simulation [#/cm^2]:
               aer_rsv(i,j,iv) = aer_rsv(i,j,iv)-atmp

               ! Total (cumulative) amount of aerosol that has been removed
               !  from the surface in this simulation [#/cm^2]:
               aer_iaccum(i,j,iv) = aer_iaccum(i,j,iv)+atmp

               if(aer_inj_tyet(1)) then   ! Only if t >= DUST_JBG
                  ! Number concentration tend for lyr k=2 of current aerosol
                  !  scalar [(#/cm^3)/s]:
                  aer_DnumconcDt = aer_numflux*c_gd%dzt(2)/(rtgt(i,j)*m_to_cm)
             
                  ! Convert to PRAMS' native "turbulent flux" (w'a') form
                  !  [(m/s)(#/cm^3)]:
                  aer_iflux(i,j,iv) = aer_DnumconcDt/c_gd%dzt(2)*rtgt(i,j)
               else
                  aer_iflux(i,j,iv) = ZERO_f4
               endif

            enddo
         endif
         
      enddo

   enddo
enddo

end subroutine aer_injection


!```````````````````````````````````````````````````````````````````
subroutine pflift_ustar_t_table(particle_massdensity,ust_tab_T_min  &
     ,ust_tab_T_inc,ust_tab_rhof_min,ust_tab_rhof_inc)

!+ Calculate minimum particle lifting threshold ustar for given fluid
!   parameters; create lookup table of values

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use aermic_rad_mem_mod, only: ustar_t_min,ust_tab_nent_T,ust_tab_nent_rhof
use body_specific_constants_mod, only: g_Xoid
use main_memory_mod, only: undef_value_f4

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: particle_massdensity,ust_tab_T_min,ust_tab_T_inc  &
     ,ust_tab_rhof_min,ust_tab_rhof_inc

!== Local declarations:

real(f4) :: massdensity_air,particle_diameter,ustar_t,tempk_air  &
     ,dyn_visc_air,kin_visc_air,ustar_t_dlast  &
     ,ustar_t_initguess
integer :: irhof,iT,ipd
logical :: dustar_pos,dustar_pos_last,iter_again

! Start with particle diameter that is quite difficult to directly lift [m]:
real(f4), parameter :: bestlift_pdiam_beg = 9.4e-5_f4,  &
                       bestlift_pdiam_binc = 1.e-6_f4,  &
                       max_pdiam_to_try = 1._f4 

!=== Executable statements:

do iT=1,ust_tab_nent_T    ! Vary air temperature [K]
   tempk_air = ust_tab_T_min+real(iT-1)*ust_tab_T_inc 

   ! Mars CO2 gas dynamic viscosity [0.1 factor is g/(cm.s) -> kg/(m.s)]: 
   dyn_visc_air = 0.1_f4*1.59e-6_f4*tempk_air**1.5_f4/(tempk_air+244.4_f4)*10._f4

   do irhof=1,ust_tab_nent_rhof    ! Vary air density [kg/m^3]
      massdensity_air = ust_tab_rhof_min+real(irhof-1)*ust_tab_rhof_inc
      kin_visc_air = dyn_visc_air/massdensity_air
      iter_again = .TRUE.
      ipd = 0
      ustar_t_dlast = epsilon(ustar_t_dlast)  ! Initialize to small value
      do while(iter_again)    ! Reached inflection point yet?
         ipd = ipd+1
         particle_diameter = bestlift_pdiam_beg+real(ipd-1)*bestlift_pdiam_binc
         
         ustar_t_initguess = ustar_t_dlast
         
         ustar_t = pflift_ustar_threshold(ustar_t_initguess,1.e-4_f4  &
              ,particle_diameter,particle_massdensity,g_Xoid,massdensity_air  &
              ,kin_visc_air)

!^
!^         aerdyn_sfc_stress = ustar_t*ustar_t*massdensity_air  ! [N]
!^
      
         if(ipd == 2) then
            dustar_pos = (ustar_t-ustar_t_dlast >= ZERO_f4)
            dustar_pos_last = dustar_pos
         elseif(ipd > 2) then
            dustar_pos = (ustar_t-ustar_t_dlast >= ZERO_f4)

            if (dustar_pos_last .neqv. dustar_pos) then   ! Detected inflection
               exit    ! Done; break out of loop
            else if(particle_diameter >= max_pdiam_to_try) then
               ustar_t = undef_value_f4  ! Failed within BCs; set to undefined value
               exit
            else
               dustar_pos_last = dustar_pos
            end if
         endif
         ustar_t_dlast = ustar_t

      enddo

      ustar_t_min(irhof,iT) = ustar_t

   enddo
enddo

end subroutine pflift_ustar_t_table


!```````````````````````````````````````````````````````````````````
subroutine pflift_intmflux_table(lmflx_tab_u_min,lmflx_tab_u_inc  &
     ,lmflx_tab_ut_min,lmflx_tab_ut_inc,c_weibull,icw)

!+ Calculate integrated particle mass flux by Weibull u* probablity
!   distribution for a given ustar_t; create lookup table of values

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use univ_exception_mod, only: exception, FATAL
use aermic_rad_mem_mod, only: intmflux,lmflx_tab_nent_u,lmflx_tab_nent_ut

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: lmflx_tab_u_min,lmflx_tab_u_inc,lmflx_tab_ut_min  &
     ,lmflx_tab_ut_inc,c_weibull
integer, intent(IN) :: icw

!== Local declarations:

real(f4) :: ustar,ustar_t,ustar_m,lmfluxint,pdfint,pdf0,pdf1  &
     ,normflux0,normflux1,ustar_int_binc,pdfint_last
integer :: iut,iu,ium
logical :: refine_intg_inc

! Start with particle diameter that is quite difficult to directly lift [m]:
real(f4), parameter :: ustar_int_beg = ZERO_f4  &   ! [m/s]
                      ,ustar_int_maxinc = 0.05_f4  ! [m/s]

!=== Executable statements:

if(ustar_int_maxinc > lmflx_tab_u_min*10._f4) then
   call exception('pflift_intmflux_table', FATAL,  &
        'Integration increment too large')
endif

do ium=1,lmflx_tab_nent_u    ! Vary "mean" ustar [m/s]
   ustar_m = lmflx_tab_u_min+real(ium-1)*lmflx_tab_u_inc 

   refine_intg_inc = .TRUE.

   do iut=1,lmflx_tab_nent_ut    ! Vary threshold ustar [m/s]
      ustar_t = lmflx_tab_ut_min+real(iut-1)*lmflx_tab_ut_inc 

      if(refine_intg_inc) then
         ustar_int_binc = ustar_int_maxinc
      endif

      do while(.TRUE.)

         ustar = ustar_int_beg
         pdf0 = (c_weibull/ustar_m)*((ustar/ustar_m)**(c_weibull-ONE_f4))*  &
              exp(-(ustar/ustar_m)**c_weibull)
 
         if(ustar >= ustar_t) then    ! Lifting particle mass
            normflux0 = ustar*ustar*ustar*  &
                 (ONE_f4-ustar_t/ustar)*(ONE_f4+ustar_t*ustar_t/(ustar*ustar))
            normflux0 = normflux0*pdf0
         else    ! Lifting threshold not met or exceeded
            normflux0 = ZERO_f4
         endif
       
         iu = 0
         lmfluxint = ZERO_f4
         pdfint = ZERO_f4
         pdfint_last = 999._f4
         do while(.TRUE.)
            iu = iu+1

            ustar = ustar_int_beg+real(iu)*ustar_int_binc

            if(log10(ustar/ustar_m)*c_weibull < 30._f4) then ! No underflow threat
               pdf1 = (c_weibull/ustar_m)*((ustar/ustar_m)**(c_weibull-ONE_f4))*  &
                    exp(-(ustar/ustar_m)**c_weibull)
            else    ! Underflow; approximate result
               pdf1 = ZERO_f4
            endif

            if(ustar >= ustar_t) then    ! Lifting particle mass
               normflux1 = ustar*ustar*ustar*  &
                    (ONE_f4-ustar_t/ustar)*(ONE_f4+ustar_t*ustar_t/(ustar*ustar))
               normflux1 = normflux1*pdf1
            else    ! Lifting threshold not met or exceeded
               normflux1 = ZERO_f4
            endif
          
            lmfluxint = lmfluxint+0.5_f4*ustar_int_binc*(normflux0+normflux1)
            pdfint = pdfint+0.5_f4*ustar_int_binc*(pdf0+pdf1)

            if(pdfint < 1.e-30_f4) then      ! Be wary of /zero conditions
               if(pdf1 <= pdf0 .and. iu > 15) then  ! Unlikely to converge any better
                  exit  ! Exit loop
               endif
            elseif(abs(ONE_f4-pdfint_last/pdfint) < 1.e-5_f4) then ! Integration done yet?
               exit  ! Exit loop
            endif

            pdfint_last = pdfint
            normflux0 = normflux1   ! Set boundary value of next interval
            pdf0 = pdf1             !
         enddo

         if(refine_intg_inc) then   ! PDF constant over iut, so refine inc once
            if(pdfint >= 0.99_f4 .and. pdfint <= 1.01_f4) then  ! Acceptable accuracy
               refine_intg_inc = .FALSE.
               exit   ! Break out of loop
            else
               ustar_int_binc = ustar_int_binc*0.2_f4  ! Lower integration dx
            endif
         else
            exit   ! Break out of loop
         endif
      enddo
      
      intmflux(iut,ium,icw) = lmfluxint

   enddo
enddo

end subroutine pflift_intmflux_table


!```````````````````````````````````````````````````````````````````
function pflift_ustar_threshold(ustar_t_initguess,ustar_t_test_epsilon  &
     ,particle_diameter,particle_massdensity,grav_accel,fluid_massdensity  &
     ,fluid_kinematic_visc)

!+ Calculate particle lifting threshold ustar for given fluid parameters
!   and particle diameter

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

real(f4) :: pflift_ustar_threshold  ! [m/s]     Procedure type

real(f4), intent(IN) :: ustar_t_initguess  &       ![m/s]
                 ,ustar_t_test_epsilon  &    !
                 ,particle_diameter  &                ! [m]
                 ,particle_massdensity  &    ! [kg/m^3]
                 ,grav_accel  &                       ! [m/s^2]
                 ,fluid_massdensity  &       ! [kg/m^3]
                 ,fluid_kinematic_visc                ! [m^2/s]

!== Local declarations:

real(f4), parameter :: Ip = 6.e-7_f4   ! Cohesion parameter [N/m^0.5]
                                !  {Lorenz et al. 1995, Newman 2002}

real(f4) :: ustar_t,ustar_t_last,Rnstar_t,A,c1,c2
integer :: cope_cnt
!^
!^integer :: iter
!^

!=== Executable statements:

! Uses semiempirical expressions from Greeley and Iversen, 1985; Also described
!  in Lorenz et al. 1995, Newman 2002

ustar_t = ustar_t_initguess    ! First guess at threshold ustar [m/s]  
ustar_t_last = 999._f4    ! [m/s]

!^
!^iter = 0
!^
cope_cnt = 0

c1 = particle_massdensity*grav_accel*particle_diameter**2.5_f4
c2 = sqrt(ONE_f4+Ip/c1)

do while(abs(ustar_t-ustar_t_last) > ustar_t_test_epsilon)
!^
!^   iter = iter+1
!^   print *,'rtt',iter,Rnstar_t,ustar_t
!^

   ! "Friction" Reynolds' number:
   Rnstar_t = ustar_t*particle_diameter/fluid_kinematic_visc
 
   if(Rnstar_t >= 0.03_f4 .and. Rnstar_t < 0.3_f4) then
      A = 0.2_f4*c2/sqrt(ONE_f4+2.5_f4*Rnstar_t)
   elseif(Rnstar_t >= 0.3_f4 .and. Rnstar_t < 10._f4) then
      A = 0.129_f4*c2/sqrt(1.928_f4*Rnstar_t**0.092_f4-ONE_f4)
   elseif(Rnstar_t >= 10.) then
      A = 0.120_f4*c2*(ONE_f4-0.0858_f4*exp(-0.0617_f4*(Rnstar_t-10._f4)))
   else    ! Rnstar_t outside of valid range
      if(cope_cnt < 10) then   ! Try "extrapolating" using "low-end" function
         A = 0.2_f4*c2/sqrt(ONE_f4+2.5_f4*Rnstar_t)
         cope_cnt  = cope_cnt+1
      else   ! Abort; algorithm not "converging" to a single answer
         call exception('pflift_ustar_threshold', FATAL,  &
              'Solution is not converging.')
      endif
   endif

   ustar_t_last = ustar_t
   ustar_t = A*sqrt(particle_massdensity*grav_accel*particle_diameter/  &
        fluid_massdensity)

enddo

pflift_ustar_threshold = ustar_t

end function pflift_ustar_threshold


!```````````````````````````````````````````````````````````````````
subroutine aer_init_mdist_logn(r0_m,rsig_m,distID,a_nbins,a_relgrp)

!+ Initialize factors for distributing a given mass of aerosol into aerosol
!   model radius bins (lognormal distribution)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: twopi_f4, ONE_f4, ZERO_f4
use aermic_rad_mem_mod, only: aer_mdist_bfrac,aer_mdist_bm2n
use aermicro_mem_mod, only: r,rlow,rup,rmass

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: r0_m  &    ! [m]       ! For *number* probablility
                 ,rsig_m   ! [none] >1.  ! 
integer, intent(IN) :: distID  &   ! ID # for current distribution
                    ,a_nbins  &   ! Number of aerosol bins in 'a_relgrp'
                    ,a_relgrp   ! Relative (that used in aer model) group ID

!== Local declarations:

! Dist total radius width [cm] (= 300um) and # of subbins to subdivide it into:
integer, parameter :: dist_subbins = 300000
real(f4), parameter :: dist_twidth = 3.e-2_f4

real(f4), parameter :: m_to_cm = 1.e2_f4    ! [m -> cm] conversion factor

real(f4) :: rsig,r0,c1,c2,bin_rad,bin_massf,subbin_dr,bin_start  &
     ,bin_end,subbin_rad,sum_bfrac,cnt_to_mass_factor
integer :: ib,isb

!=== Executable statements:

! Modal radius <r0> and deviation <rsig> of aerosol distribution:
!  (<rsig> also termed the "geometric standard deviation") 
rsig = rsig_m    ! [unitless]
cnt_to_mass_factor = exp(3._f4*log(rsig)**2)  ! # -> mass dist (Hatch-Choate eq)
r0 = r0_m*m_to_cm*cnt_to_mass_factor   ! Mass median radius [cm]*c2m_fact

! Width of a comp subbin [cm]:
subbin_dr = dist_twidth/real(dist_subbins, kind=f4)  

ib = 1   ! start with first aerosol bin
bin_rad = r(ib,a_relgrp)  ! mean radius (by volume) of current bin
bin_start = rlow(ib,a_relgrp)  ! lower radius bound of bin
bin_end = rup(ib,a_relgrp)   ! upper radius bound of bin

sum_bfrac = ZERO_f4  ! Initialize summing var

distsubstep: do isb=1,dist_subbins   ! Loop over all computational subbins

   ! Mean radius of current subbin:
   subbin_rad = (real(isb, kind=f4)-0.5_f4)*subbin_dr
   
   if(subbin_rad >= bin_start .and. subbin_rad <= bin_end) then

      c1 = ONE_f4/(sqrt(twopi_f4)*subbin_rad*log(rsig))
      c2 = -0.5_f4*(log(subbin_rad/r0)/log(rsig))**2
      
      ! Fraction of total particles that lie in current subbin (well,
      ! almost... need to multiply by a factor of dr() first): 
      bin_massf = c1*exp(c2)
      
      ! Sum to obtain fraction of total particles within radius bin
      sum_bfrac = sum_bfrac+bin_massf*subbin_dr

   elseif(subbin_rad > bin_end .or. isb == dist_subbins) then  ! Go to next bin

      ! Save finished result for current bin:
      aer_mdist_bfrac(distID,ib) = sum_bfrac 

      ! mass -> # conversion factor:
      aer_mdist_bm2n(distID,ib) = ONE_f4/rmass(ib,a_relgrp)

      sum_bfrac = ZERO_f4  ! Reset sum
      ib = ib+1  ! Increment bin index
      if(ib > a_nbins) then    ! Don't continue past last existing bin
         exit distsubstep         !
      endif                       !
      bin_rad = r(ib,a_relgrp)  ! mean radius (by volume) of current bin
      bin_start = rlow(ib,a_relgrp)  ! lower radius bound of bin
      bin_end = rup(ib,a_relgrp)   ! upper radius bound of bin

   else

      continue  ! Keep looping if not currently within a bin

   endif

enddo distsubstep

end subroutine aer_init_mdist_logn


!```````````````````````````````````````````````````````````````````
subroutine aer_ndist_logn(r0_m,rsig_m,a_nbins,a_relgrp,aer_ndist_bfrac)

!+ Initialize factors for distributing a given number of aerosol into aerosol
!   model radius bins (lognormal distribution)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: twopi_f4, ONE_f4, ZERO_f4
use aermic_rad_mem_mod, only: aer_mdist_bm2n
use aermicro_mem_mod, only: r,rlow,rup,rmass

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: r0_m  &    ! [m]       ! For *number* probablility
                 ,rsig_m   ! [none] >1.  ! 
integer, intent(IN) :: a_nbins  &   ! Number of aerosol bins in 'a_relgrp'
                    ,a_relgrp   ! Relative (that used in aer model) group ID

real(f4), dimension(a_nbins), intent(INOUT) :: aer_ndist_bfrac

!== Local declarations:

! Dist total radius width [cm] (= 300um) and # of subbins to subdivide it into:
integer, parameter :: dist_subbins = 300000
real(f4), parameter :: dist_twidth = 3.e-2_f4

real(f4), parameter :: m_to_cm = 1.e2_f4    ! [m -> cm] conversion factor

real(f4) :: rsig,r0,c1,c2,bin_rad,bin_numf,subbin_dr,bin_start  &
     ,bin_end,subbin_rad,sum_bfrac
integer :: ib,isb

!=== Executable statements:

! Modal radius <r0> and deviation <rsig> of aerosol distribution:
!  (<rsig> also termed the "geometric standard deviation") 
rsig = rsig_m    ! [unitless]
!%cnt_to_mass_factor = exp(3.*log(rsig)**2)  ! # -> mass dist (Hatch-Choate eq)
r0 = r0_m*m_to_cm   ! Count median radius [cm]

subbin_dr = dist_twidth/real(dist_subbins)  ! width of a comp subbin [cm]

ib = 1   ! start with first aerosol bin
bin_rad = r(ib,a_relgrp)  ! mean radius (by volume) of current bin
bin_start = rlow(ib,a_relgrp)  ! lower radius bound of bin
bin_end = rup(ib,a_relgrp)   ! upper radius bound of bin

sum_bfrac = ZERO_f4  ! Initialize summing var

distsubstep: do isb=1,dist_subbins   ! Loop over all computational subbins

   subbin_rad = (real(isb)-0.5_f4)*subbin_dr  ! mean radius of current subbin
   
   if(subbin_rad >= bin_start .and. subbin_rad <= bin_end) then

      c1 = ONE_f4/(sqrt(twopi_f4)*subbin_rad*log(rsig))
      c2 = -0.5_f4*(log(subbin_rad/r0)/log(rsig))**2
      
      ! Fraction of total particles that lie in current subbin (well,
      ! almost... need to multiply by a factor of dr() first): 
      bin_numf = c1*exp(c2)
      
      ! Sum to obtain fraction of total particles within radius bin
      sum_bfrac = sum_bfrac+bin_numf*subbin_dr

   elseif(subbin_rad > bin_end .or. isb == dist_subbins) then  ! Go to next bin

      ! Save finished result for current bin:
      aer_ndist_bfrac(ib) = sum_bfrac 

!%      ! mass -> # conversion factor:
!%      aer_mdist_bm2n(distID,ib) = 1./rmass(ib,a_relgrp)

      sum_bfrac = ZERO_f4  ! Reset sum
      ib = ib+1  ! Increment bin index
      if(ib > a_nbins) then    ! Don't continue past last existing bin
         exit distsubstep         !
      endif                       !
      bin_rad = r(ib,a_relgrp)  ! mean radius (by volume) of current bin
      bin_start = rlow(ib,a_relgrp)  ! lower radius bound of bin
      bin_end = rup(ib,a_relgrp)   ! upper radius bound of bin

   else

      continue  ! Keep looping if not currently within a bin

   endif

enddo distsubstep

end subroutine aer_ndist_logn


end module aemic_drvint_mod


!```````````````````````````````````````````````````````````````````
subroutine aemic_drvint(m1, m2, m3, ia, iz, ja, jz)

!+ Aerosol microphysics (derived from CARMA) routine driver/interface

use univ_kind_defs_mod, only: f4
use main_memory_mod, only: a, naerscl, n_grids, maxMzp, maxMyp, maxMxp, c_gd,  &
     i_grid
use main_memory_bspec_mod, only: H2Ov_tog, CO2i_tog
use vtables_mod, only: ipp, ipi0, ithp, ivt3da, ivt3db, ivt3dc, irv,  &
     irtgt, iaerscp, ivt3de, ivt3df, ivt3dg, itht, ipt, irvt,  &
     iaersct, iaerrs, iaerff, iaerag, itslopang, isclpn, ivt3dm
use aermic_rad_mem_mod, only: aer_active, aer_ie, aer_ib, aer_XOnucleate,  &
     aer_XOprecip
use aemic_drvint_mod, only: aemic2PRAMS, aemic_get_ptd_cgs
use grid_mem_mod, only: switch_to_Ascalar
use aemic_driv_mod, only: aemic_driver

implicit none

!=== Argument declarations:

integer, intent(IN) :: ia, iz, ja, jz, m1, m2, m3

!== Local declarations:

real(f4) :: R_air_cgs
integer :: idx_r, idx_rt
logical :: init_micro, haveH2Ov

integer, save :: ncall = 0

!=== Executable statements:

! Calculate current atmospheric temperature, pressure, and density (all
!  in *cgs* units) for aerosol microphysics code; also calcs R_air in *cgs*
!  units (R_air_cgs):  [vt3da = T, vt3db = P, vt3dc = rho]

call aemic_get_ptd_cgs(m1, m2, m3, ia, iz, ja, jz, a(ipp), a(ipi0), a(ithp),  &
     a(ivt3da), a(ivt3db), a(ivt3dc), R_air_cgs)

! Possibly set some control flags:

init_micro = .FALSE.   ! Assume that microphysics model already is initialized 

if (ncall < n_grids) then
   
   ncall = ncall+1

   if (.NOT.aer_active) then
      if (ncall == n_grids) then  ! Don't set until after last grid initialized
         aer_active = .TRUE.   ! Aerosol microphysics has now been activated
      end if
   end if

   if (ncall == 1) then   ! Only do once (first grid)
      init_micro = .TRUE.   ! microphysics model needs to be initialized 
   end if

end if

!print *,'@@#s',init_micro,grid
!pause

! Calculate aerosol microphysical tendencies (and add them carefully to aerosol
!  variables before returning):
!   ** returns new [theta = vt3de, density = vt3df, H2Ovmr = vt3dg]

if (H2Ov_tog > 0) then
   haveH2Ov = .TRUE.
   idx_r = irv
   idx_rt = irvt
else
   haveH2Ov = .FALSE.
   idx_r = ivt3dm
   idx_rt = ivt3dm
end if

call switch_to_Ascalar(i_grid, 1)   ! Fill 'iaerscp/t', etc. indices
if (CO2i_tog == 0) then
   call aemic_driver(m1, m2, m3, naerscl, ia, iz, ja, jz, 2, m1-1, aer_ie,  &
        aer_ib, a(iaerscp), maxMzp, maxMxp, maxMyp, a(iaersct), init_micro,  &
        a(ivt3dc), a(ivt3df), a(ithp), a(ivt3de), a(ivt3db), a(ivt3da),  &
        a(idx_r), a(ivt3dg), c_gd%zm, c_gd%zt, a(irtgt), c_gd%dt_dyn,  &
        a(iaerrs), a(iaerff), a(iaerag), a(itslopang), i_grid,  &
        aer_XOnucleate(:,i_grid), aer_XOprecip(:,i_grid),a(ivt3de))
else
   call aemic_driver(m1, m2, m3, naerscl, ia, iz, ja, jz, 2, m1-1, aer_ie,  &
        aer_ib, a(iaerscp), maxMzp, maxMxp, maxMyp, a(iaersct), init_micro,  &
        a(ivt3dc), a(ivt3df), a(ithp), a(ivt3de), a(ivt3db), a(ivt3da),  &
        a(idx_r), a(ivt3dg), c_gd%zm, c_gd%zt, a(irtgt), c_gd%dt_dyn,  &
        a(iaerrs), a(iaerff), a(iaerag), a(itslopang), i_grid,  &
        aer_XOnucleate(:,i_grid), aer_XOprecip(:,i_grid), a(isclpn(i_grid,1)))
end if

! Calculate PI, theta, rv microphysical tendencies:

call aemic2PRAMS(m1, m2, m3, ia, iz, ja, jz, a(ithp), a(ivt3de), a(ivt3df),  &
     a(idx_r), a(ivt3dg), a(ivt3da), a(ipp), a(ipi0), a(itht), a(ipt),  &
     a(idx_rt), R_air_cgs, 2, m1-1)

end subroutine aemic_drvint


!```````````````````````````````````````````````````````````````````
subroutine negadj(mzp, mxp, myp, ng, action)

!+ Assure that certain fields remain positive definite

use univ_math_constants_mod, only: ZERO_f4
use main_memory_mod, only: a
use main_memory_bspec_mod, only: H2Ov_tog
use aermic_rad_mem_mod, only: aer_subgrp
use vtables_mod, only: irvn, naersclr, iaerscp
use grid_mem_mod, only: switch_to_Ascalar

implicit none

!=== Argument declarations:

integer, intent(IN) :: ng, mzp, mxp, myp
character(len=*), intent(IN) :: action

!== Local declarations:

integer :: mxyzp1, vidx, iv

!=== Executable statements:

mxyzp1 = mzp*mxp*myp-1

! Make sure certain fields are positive definite:

if (H2Ov_tog >= 1) then
   vidx = irvn(ng)    ! RV (water vapor mixing ratio)
   a(vidx:vidx+mxyzp1) = merge(a(vidx:vidx+mxyzp1), ZERO_f4,  &
        a(vidx:vidx+mxyzp1) >= ZERO_f4)
end if

if (action == 'ALL') then

   do iv=1,naersclr(ng)     ! Aerosol quantities
      call switch_to_Ascalar(ng, iv)
!%      if (aer_subgrp(iv,ng) == 0) then    ! # concentration
      a(iaerscp:iaerscp+mxyzp1) = merge(a(iaerscp:iaerscp+mxyzp1), ZERO_f4,  &
           a(iaerscp:iaerscp+mxyzp1) >= ZERO_f4)
!%      else                  ! mass concentration
!%         a(iaerscp:iaerscp+nnxyzp1) = merge(a(iaerscp:iaerscp+nnxyzp1),  &
!%         ZERO_f4, a(iaerscp:iaerscp+nnxyzp1) >= ZERO_f4)
!%      end if
   end do
   
end if

end subroutine negadj
