!```````````````````````````````````````````````````````````````````
module aemic_fastproc_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine fast_aermicro

!+ Driver for the "faster" aerosol microphysics processes

use aermicro_mem_mod, only: do_grow,igelem,nelements,nbins,do_thermo  &
     ,ik,time,rnuclg,growlg,evaplg,ngroups,evappe,rnucpe,loop_again  &
     ,i_curr,j_curr
use aemic_satvap_mod, only: gas_supersat_pt
use aemic_partmphys_mod, only: nucl_loss_Mars, growevap_loss,  &
     growpe_upegtx_psolve, evappart_production, downeg_xfer,  &
     part_compdensity, apply_downeg_evap
use aemic_gasmphys_mod, only: gas_exchange, gas_solve, thetaconc_solve
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

!@integer(i4) :: ib,ie,ig

!@
!@real(f4) :: t1,t2
!@real(f4), external :: tag_CPUtime
!@

!=== Executable statements:

! Do growth microphysical processes if requested:

!#if(do_grow) then

   ! Calculate (implicit) particle loss rates for nucleation, growth,
   !  evaporation, melting, etc.:

   call zero_micarr

!@   t1 = tag_CPUtime()
   call gas_supersat_pt  
!@   t2 = tag_CPUtime()
!@   call accum_timetags_wn(6,'satvap',t1,t2)

!@   t1 = tag_CPUtime()
   call growevap_loss
!@   t2 = tag_CPUtime()
!@   call accum_timetags_wn(7,'geloss',t1,t2)

!@if(ik == 29 .and. time < 0.) then
!@   do ig=1,ngroups
!@      do ib=1,nbins(ig)
!@         print *,'growlg,evaplg',ib,ig,growlg(ib,ig),evaplg(ib,ig)
!@      enddo
!@   enddo
!@endif

!?        if( t3(ixyz) .ge. T0-40. ) call actdropl
!?        call freezaerl
!?        call freezdropl
!@   t1 = tag_CPUtime()
   call nucl_loss_Mars
!@   t2 = tag_CPUtime()
!@   call accum_timetags_wn(8,'nucmars',t1,t2)
!?        call melticel
!?        call coremelt
!?        call freezmixedl
!?        call meltmixedl

!@if(ik == 29 .and. time < 0.) then
!@   do ig=1,ngroups
!@      do ib=1,nbins(ig)
!@         print *,'rnuclg',ib,ig,rnuclg(ib,ig,1:ngroups)
!@      enddo
!@   enddo
!@endif

   ! Calculate particle production terms and solve for particle 
   !  concentrations at end of timestep (???):

!@   t1 = tag_CPUtime()
   call growpe_upegtx_psolve
!@   t2 = tag_CPUtime()
!@   call accum_timetags_wn(9,'growuppsv',t1,t2)

   ! Calculate particle production terms for evaporation:

!@   t1 = tag_CPUtime()
   call evappart_production
!@if(ik == 29 .and. time < 0.) then
!@   do ie=1,nelements
!@      do ib=1,nbins(igelem(ie))
!@         print *,'evappe_2',ib,ie,evappe(ib,ie)
!@      enddo
!@   enddo
!@endif
   call downeg_xfer
!@   t2 = tag_CPUtime()
!@   call accum_timetags_wn(10,'evapdown',t1,t2)

!@if(ik == 29 .and. time < 0.) then
!@   do ie=1,nelements
!@      do ib=1,nbins(igelem(ie))
!@         print *,'rnucpe_2,',ib,ie,rnucpe(ib,ie)
!@      enddo
!@   enddo
!@endif

   ! Calculate gas loss rates and production terms due to particle nucleation,
   !  growth, and evaporation:
!@   t1 = tag_CPUtime()
   call gas_exchange

   ! Apply evaporation production terms to particle concentrations;

   call apply_downeg_evap

   ! Solve for gas concentrations at end of time step:

   call gas_solve

!#endif

if(.NOT.loop_again) then    ! If no violation has occurred, proceed normally

   ! Update temperature if thermal processes requested:

   if(do_thermo) then
      call thetaconc_solve
   endif

   ! Update saturation ratios (note: the simulated microphysical
   !  processes do not depend on these updated values):
   
   !#too CPU costly if no output use#   call gas_supersat_pt
   
   ! Update particle densities:
   
   call part_compdensity
   !@t2 = tag_CPUtime()
   !@call accum_timetags_wn(11,'last',t1,t2)

endif

end subroutine fast_aermicro


!```````````````````````````````````````````````````````````````````
subroutine zero_micarr

!+ Zeroes the "fast" aerosol microphysics sink and source arrays

use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: rlheat,ngasses,gasprod,nelements,ngroups,nbins  &
     ,rnucpe,growpe,evappe,inucstep,growlg,evaplg,rnuclg,growlg,evaplg  &
     ,evappe,igelem

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: igas,ie,ib,ig,igto

!=== Executable statements:

! Set production terms and loss rates due to nucleation, growth,
!  and evaporation to zero.  Also set index of smallest bin nuceleated
!  during timestep equal to <nbins(ig)>:

rlheat = ZERO_f4

do igas=1,ngasses
   gasprod(igas) = ZERO_f4
enddo

do ie=1,nelements
   do ib=1,nbins(igelem(ie))
      rnucpe(ib,ie) = ZERO_f4
      growpe(ib,ie) = ZERO_f4
      evappe(ib,ie) = ZERO_f4
   enddo
enddo

do ig=1,ngroups

   inucstep(ig) = nbins(ig)

   do ib=1,nbins(ig)
      growlg(ib,ig) = ZERO_f4
      evaplg(ib,ig) = ZERO_f4
   enddo

enddo

do igto=1,ngroups
   do ig=1,ngroups
      do ib=1,nbins(ig)
         rnuclg(ib,ig,igto) = ZERO_f4
      enddo
   enddo
enddo

end subroutine zero_micarr


end module aemic_fastproc_mod
