!```````````````````````````````````````````````````````````````````
module aemic_gasmphys_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine gas_exchange

!+ Calculates the total production of gasses due to nucleation, growth, and
!   evaporation of aerosol particles

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use aermicro_mem_mod, only: rhoa,ngroups,ienconc,inucgas,nnuc2elem  &
     ,igelem,nbins,inuc2bin,rnuclg,pc,inucproc,diffmass  &
     ,I_DROPACT,I_AERFREEZE,rlhm,cp_cgs,rlheat,igrowgas,totevap,cmf,rmass  &
     ,evaplg,growlg,ik,is_grp_ice,gasprod,rlhe,inuc2elem
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! This routine calculates the total production of gases due to nucleation,
!  growth, and evaporation <gasprod> [g/x_units/y_units/z_units/s].
!  It also calculates the latent heating rate from a condensing gas
!  <rlheat> [deg_K/s]

integer :: ig,ie,inuc,igas,ienuc2,ig2,ib,i2
real(f4) :: c1,gasgain,rlh  !#,xrlh

!=== Executable statements:

c1 = ONE_f4/(cp_cgs*rhoa(ik))    ! Constant used below

! First calculate gas loss and latent heat gain rates due to nucleation:

do ig=1,ngroups

   ie = ienconc(ig)             ! element of particle number concentration

   do inuc=1,nnuc2elem(ie)     ! Loop over all nucleation paths
             
      igas = inucgas(inuc,ig)    ! condensing gas
      ienuc2 = inuc2elem(inuc,ie)   ! target particle element
      ig2 = igelem(ienuc2)    ! target particle group
       	   
      do ib=1,nbins(ig)
            
         i2 = inuc2bin(ib,ig,ig2)            ! target bin

         gasprod(igas) = gasprod(igas)-pc(ik,ib,ie)  &
              *rnuclg(ib,ig,ig2)*diffmass(i2,ig2,ib,ig)
     
      enddo

      ! Latent heating rate from condensing gas: <rlh> is latent
      !  heat of evaporation (+ fusion, for ice deposition) [erg/g]:

      if(inucproc(ie,ienuc2) == I_DROPACT) then
         rlh = rlhe(ik,igas)
      elseif(inucproc(ie,ienuc2) == I_AERFREEZE) then
         rlh = rlhe(ik,igas)+rlhm(ik,igas)
      endif
	   
      rlheat = rlheat-rlh*gasprod(igas)*c1
              
   enddo

   ! Next calculate gas lost/gained due to and heat gained/lost from 
   !  growth/evaporation:

   igas = igrowgas(ie)     ! condensing gas

   if(igas /= 0) then
      
      do ib=1,nbins(ig)-1

         ! Calculate <gasgain>, mass concentration of gas gained due to
         !  evaporation from each droplet in bin <i+1>.  First check for total
         !  evaporation:

         if(totevap(ib+1,ig)) then
            gasgain = (ONE_f4-cmf(ib+1,ig))*rmass(ib+1,ig)
         else
            gasgain = diffmass(ib+1,ig,ib,ig)
         endif

         gasprod(igas) = gasprod(igas)+gasgain*  &
              (evaplg(ib+1,ig)*pc(ik,ib+1,ie))  &
              -(growlg(ib,ig)*pc(ik,ib,ie))*diffmass(ib+1,ig,ib,ig)

      enddo

      ! Add evaporation out of smallest bin (always total evaporation):

      gasprod(igas) = gasprod(igas)+  & 
           evaplg(1,ig)*pc(ik,1,ie)*(ONE_f4-cmf(1,ig))*rmass(1,ig)
     
      ! Latent heating rate from condensing gas: <rlh> is latent heat of
      !  evaporation ( + fusion, for ice deposition ) [erg/g]:

      if(is_grp_ice(ig)) then
         rlh = rlhe(ik,igas)+rlhm(ik,igas)
      else
         rlh = rlhe(ik,igas)
      endif
	
      rlheat = rlheat-rlh*gasprod(igas)*c1
      
!?kludge to prohibit CO2 latent heating?      gasprod(2) = 1.e-50     ! ?????
   
!?      rlheat = rlheat/10.   ! ?????

!#### Need this???????? Won't this throw things out of balance (esp. for CO2)??
!#      xrlh = 0.1
!#      if(rlheat > xrlh) then
!#         rlheat = xrlh
!#      elseif(rlheat < -xrlh) then
!#         rlheat = -xrlh
!#      endif

   endif

enddo

end subroutine gas_exchange


!```````````````````````````````````````````````````````````````````
subroutine gas_solve

!+ Calculates new gas concentrations

use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: ngasses,gc,dtime,gasprod,ik,loop_again
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: igas

!=== Executable statements:

! Update each gas concentration using gas production rates:

do igas=1,ngasses

   gc(ik,igas) = gc(ik,igas)+dtime*gasprod(igas)
        
   if(gc(ik,igas) < ZERO_f4) then
      loop_again = .TRUE.    ! Violation occurred
   endif
	
enddo

end subroutine gas_solve


!```````````````````````````````````````````````````````````````````
subroutine thetaconc_solve

!+ Calculates new potential temperature concentration (and updates temperature)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use aermicro_mem_mod, only: ptc,ik,dtime,rlheat,rhoa,p,PREF,t, rkappa
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! This routine calculates new potential temperature concentration 
!  (and updates temperature) due to microphysical and radiative forcings.
!  The equation solved (the first law of thermodynamics in flux form) is
!
!    d(rhostar theta)     rhostar theta       d(qv)    1  dF
!    ---------------  = - ------------- * ( L ----- + --- -- )
!          dt                 Cp T              dt    rho dz
!
!  where
!    rhostar = scaled air density
!    theta   = potential temperature
!    t       = time
!    Cp      = specific heat (at constant pressure) of air
!    T       = air temperature
!    qv      = water vapor mixing ratio
!    L       = latent heat
!    F       = net radiative flux
!    z       = unscaled altitude
   
real(f4) :: pt  !# dTdt

!=== Executable statements:

! Solve for the new <ptc> due to latent heat exchange and radiative heating.
!  Latent and radiative heating rates are in units of [deg_K/s]:

!#norad#  ptc(ik) = ptc(ik)*(1.+dtime/t(ik)*(rlheat+radheat(ik)+dTdt))
ptc(ik) = ptc(ik)*(ONE_f4+dtime/t(ik)*(rlheat))
 
! Update <t>, using latest <rhoa> from mass continuity and <p> from hydrostasy:
 
pt = ptc(ik)/rhoa(ik)
t(ik) = pt*(p(ik)/PREF)**rkappa

end subroutine thetaconc_solve


end module aemic_gasmphys_mod
