!```````````````````````````````````````````````````````````````````
module aemic_init_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine init_general

!+ Performs initialization of "general" parameters

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL
use aermicro_mem_mod, only: dtime,do_coag,do_grow,do_thermo,itbnd_pc  &
     ,i_flux_spec,ibbnd_pc,itbnd_gc,ibbnd_gc,itbnd_ptc,ibbnd_ptc,do_varstep  &
     ,maxsubsteps,minsubsteps,conmax,dtmin,dtmax,dpctol,dgstol,do_step  &
     ,nz_mic,time,gc_vTX_on,ptc_vTX_on,pc_vTX_on,gc_extTX_on,ptc_extTX_on  &
     ,pc_extTX_on

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'init_general'

!=== Executable statements:

! Define dummy timestep size [s] (set by external model elsewhere):

dtime = 10._f4

! Define flag to control whether any coagulation is to be simulated:

do_coag = .FALSE.

! Define flag to control whether condensational growth is to be simulated
!  (evaporation and nucleation also):

do_grow = .TRUE.

! Define flag to control whether temperature is changed by latent heating:

do_thermo = .TRUE.

! Define flags to control whether vertical transport of specific quantities
!  occurs *within the aerosol model*:

gc_vTX_on = .FALSE.     ! No vert gas transport by this model
ptc_vTX_on = .FALSE.   ! No vert potential temperature transport by this model
pc_vTX_on = .TRUE.    ! Some vert particle transport by this model

! Define flags to control whether external transport of specific quantities
!  is added to those fields *within the aerosol model*:

gc_extTX_on = .FALSE.     ! Some external gas transport by this model
ptc_extTX_on = .FALSE.   ! No potential temperature transport by this model
pc_extTX_on = .TRUE.    ! Some external particle transport by this model

! Define flags to control vertical boundary conditions:
!    <itbnd_pc> = I_DIFFUSION: allow concentrations to diffuse and advect
!                              across top boundary
!               = I_FLUX_SPEC: use specified flux at top boundary
!    <ibbnd_pc>: same as <itbnd_pc>, but for bottom boundary;
!  (equivalent parameters are defined for gases and potential temperature
!   boundary conditions)

itbnd_pc = I_FLUX_SPEC
ibbnd_pc = I_FLUX_SPEC
itbnd_gc = I_FLUX_SPEC
ibbnd_gc = I_FLUX_SPEC
itbnd_ptc = I_FLUX_SPEC
ibbnd_ptc = I_FLUX_SPEC

!  Define flag to control whether a variable time-step should be used.
!   Also define min and max time-steps, maximum tolerance in particle
!   concentration changes <dpctol>, maximum tolerance in gas concentration 
!   changes <dgstol>, and minimum relative concentration to consider <conmax>
!   (i.e., for bins with concentrations less than <conmax>*max(pc), we don't
!   worry about how large the changes were):

do_varstep = .FALSE.

! Set up things that depend on variable timestepping (???):

maxsubsteps = 10000  ! 10
minsubsteps = 1   ! 1
conmax = 1.e-3_f4   ! 1.e-1_f4
if(do_varstep) then
   dtmin = 4._f4
   dtmax = 4._f4*4._f4*60._f4
   dpctol = 0.5_f4
   dgstol = 0.1_f4
   conmax = 1.e-3_f4
else
   do_step = .TRUE.     ! Advance ahead in time (always)
endif

! Check to make sure model includes at least 5 layers if vertical transport
!  is configured to be "on":

if((gc_vTX_on .or. ptc_vTX_on .or. pc_vTX_on) .and. nz_mic < 5) then
   call exception(procedure_name, FATAL,  &
        'Cannot do vertical transport with nz_mic < 5')
endif

! Initialize simulation time [s]:

time = 0.

end subroutine init_general


!```````````````````````````````````````````````````````````````````
subroutine setup_bins

!+ Evaluates derived bin mapping arrays and sets up particle size/mass bins

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ONE_THIRD_f4, FOUR_THIRDS_f4
use univ_exception_mod, only: exception, FATAL
use aermicro_mem_mod, only: ncore,sec_mom,imomelem,ngroups,nelemg,itype  &
     ,I_COREMASS,I_VOLCORE,icorelem,I_CORE2MOM, nbins,nz_mic  &
     ,rhop,rhoelem,pi_m,rmassmin,rmin,rmrat, rmass,rmassup,dm,vol,r,rup  &
     ,dr,rlow,diffmass,ienconc,dmi,small_coremconc,SMALL_PC,FIX_COREF

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

real(f4) :: c1,vrfact,smallest,better_val
integer :: ie,ig,j,ib,k,jg,jb
logical :: too_small

!=== Executable statements:

! Derived variables:

! Determine how many cores are in each group <ncore>.
!  The core elements in a group are given by <icorelem(1:ncore,igroup)>.
!
!  Also evaluate whether or not second moment is used <sec_mom> for each group.

ncore = 0          !
sec_mom = .FALSE.  ! Initialize entire array
imomelem = 0       !

ie = 0
do ig=1,ngroups
   do j=1,nelemg(ig)
      ie = ie+1
      
      if(itype(ie) == I_COREMASS .or. itype(ie) == I_VOLCORE) then
         ncore(ig) = ncore(ig)+1
         icorelem(ncore(ig),ig) = ie
      elseif(itype(ie) == I_CORE2MOM) then   
         sec_mom(ig) = .TRUE.
         imomelem(ig) = ie
      endif

   enddo
enddo
!^print *,'ncore',ncore
!^print *,'icorelem',icorelem
!^print *,'sec_mom',sec_mom
!^print *,'imomelem',imomelem

! Particle mass densities (for each group) [g/cm^3]:

do ig=1,ngroups
   ie = ienconc(ig) 
   do ib=1,nbins(ig)
      do k=1,nz_mic
         rhop(k,ib,ig) = rhoelem(ie)
      enddo
   enddo
enddo
!^print *,'rhop',rhop(1,1,1:ngroups)

! Set up the particle bins.
!  For each particle group, the mass of a particle in
!  bin j is <rmrat> times that in bin j-1
!
!    rmass(nbins(ig),ngroups)     =  bin center mass [g]
!    r(nbins(ig),ngroups)         =  bin mean (volume-weighted) radius [cm]
!    vol(nbins(ig),ngroups)       =  bin center volume [cm^3]
!    dr(nbins(ig),ngroups)        =  bin width in radius space [cm]
!    dv(nbins(ig),ngroups)        =  bin width in volume space [cm^3]
!    dm(nbins(ig),ngroups)        =  bin width in mass space [g]

c1 = FOUR_THIRDS_f4*pi_m
do ig=1,ngroups

   rmassmin(ig) = c1*rhop(1,1,ig)*rmin(ig)**3
   vrfact = ((1.5_f4/pi_m/(rmrat(ig)+ONE_f4))**ONE_THIRD_f4)*  &
        (rmrat(ig)**ONE_THIRD_f4-ONE_f4)

   do ib=1,nbins(ig)
           
      rmass(ib,ig) = rmassmin(ig)*rmrat(ig)**real((ib-1))
      rmassup(ib,ig) = 2._f4*rmrat(ig)/(rmrat(ig)+ONE_f4)*rmass(ib,ig)
      dm(ib,ig) = 2._f4*(rmrat(ig)-ONE_f4)/(rmrat(ig)+ONE_f4)*rmass(ib,ig)
      dmi(ib,ig) = ONE_f4/dm(ib,ig)
      vol(ib,ig) = rmass(ib,ig)/rhop(1,1,ig)
      r(ib,ig) = (rmass(ib,ig)/rhop(1,1,ig)/c1)**ONE_THIRD_f4
      rup(ib,ig) = (rmassup(ib,ig)/rhop(1,1,ig)/c1)**ONE_THIRD_f4
      dr(ib,ig) = vrfact*(rmass(ib,ig)/rhop(1,1,ig))**ONE_THIRD_f4
      rlow(ib,ig) = rup(ib,ig)-dr(ib,ig)

!^      print *,'ig,ib',ig,ib
!^      print *,'rmass..',rmass(ib,ig),rmassup(ib,ig),dm(ib,ig)
!^      print *,'vol',vol(ib,ig)
!^      print *,'r(cm)...',r(ib,ig)*1.e4,rup(ib,ig),rlow(ib,ig),dr(ib,ig)
!^      print *,'r(um)...',r(ib,ig)*1.e4,rup(ib,ig)*1.e4,rlow(ib,ig)*1.e4  &
!^           ,dr(ib,ig)*1.e4
   enddo
enddo

! Evaluate differences between values of <rmass> in different bins:

do ig=1,ngroups
   do jg=1,ngroups
      do ib=1,nbins(ig)
         do jb=1,nbins(jg)
            diffmass(ib,ig,jb,jg) = rmass(ib,ig)-rmass(jb,jg)
         enddo
      enddo
   enddo
enddo

! Calculate minimum permitted values of core mass concentrations (1st and 2nd
!  moments) [g/cm^3]:

smallest = tiny(c1) ! Smallest FP value permitted by current machine precision
smallest = real(int(log10(smallest), kind=f4))  ! Smallest integer FP exponent
better_val = smallest

too_small = .FALSE.

do ig=1,ngroups

   do j=1,ncore(ig)
      do ib=1,nbins(ig)
         c1 = log10(rmass(ib,ig)*FIX_COREF)-4._f4
         if(log10(SMALL_PC)+c1 <= smallest) then
            better_val = max(better_val,smallest-c1)
            too_small = .TRUE.
         else
            small_coremconc(ib,icorelem(j,ig)) = SMALL_PC*  &
                 rmass(ib,ig)*FIX_COREF
         endif
      enddo
   enddo

   if(imomelem(ig) > 0) then   ! Core second moment exists
      do ib=1,nbins(ig)
         c1 = 2._f4*log10(rmass(ib,ig)*FIX_COREF)-4._f4
         if(log10(SMALL_PC)+c1 <= smallest) then
            better_val = max(better_val,smallest-c1)
            too_small = .TRUE.
         else
            small_coremconc(ib,imomelem(ig)) =  &
                 SMALL_PC*(rmass(ib,ig)*FIX_COREF)**2
         endif
      enddo
   endif

   if(too_small) then
      write (*,'(/a,/a,/a,e15.7,//a,/a/)')  &
        'ERROR {setup_bins}: to avoid significant floating point'  &
       ,'                    underflow issues, please change SMALL_PC'  &
       ,'                    (in aermicro_mem.f90) to >'  &
       ,10._f4**better_val  &
       ,'    -OR-    modify the namelist aerosol configuration (especially'  &
       ,'            in relation to the smallest aerosol particles).' 
      call exception('setup_bins', FATAL)
   endif

enddo

end subroutine setup_bins


!```````````````````````````````````````````````````````````````````
subroutine setup_vfall

!+ Evaluates particle fall velocities and Reynolds' numbers

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4, ONE_THIRD_f4,  &
     TWO_THIRDS_f4, ONE_SIXTH_f4, ONE_NINTH_f4
use aermicro_mem_mod, only: ngroups,ishape,pi_m, eshape,nz_mic,rhoa  &
     ,R_air_cgs,t,rmu,nbins,r,powmax,bpm,vf,rhop,g_cgs,re,vol,ifall,nz1_mic  &
     ,vf_const,igridv,I_CART

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! This routine evaluates particle fall velocities, vf(k) [cm s^-1]
!  and Reynolds' numbers based on fall velocities, re(j,i,k) [dimensionless].
!  indices correspond to vertical level <k>, bin index <i>, and aerosol
!  group <j>.
!
!  Non-spherical particles are treated through shape factors <ishape>
!  and <eshape>.
!  
!  General method is to first use Stokes' flow to estimate fall
!  velocity, then calculate reynolds' number, then use "y function" 
!  (defined in Pruppacher and Klett) to reevaluate reynolds' number,
!  from which the fall velocity is finally obtained.
!
!  This routine requires that vertical profiles of temperature <t>,
!  air density <rhoa>, and viscosity <rmu> are defined (i.e., initatm.f
!  must be called before this).  The vertical profile with ix = iy = 1
!  is used.

real(f4) :: f1,f2,f3,exx,exy,xcc,xa,vg,rhoa_cgs,rmfp,r_shape,rkn,expon  &
     ,rfix,x,y,ex,b0,bb1,bb2,bb3,bxx,z
integer :: ig,k,ib

!=== Executable statements:

do ig=1,ngroups       ! Loop over all aerosol groups

   ! First evaluate factors that depend upon particle shape (used in correction
   !  factor <bpm> below):  

   if(ishape(ig) == 1) then    ! Spherical

      f1 = ONE_f4
      f2 = ONE_f4

   elseif(ishape(ig) == 2) then     ! Hexagonal

      ! Taken from Turco et al. (Planet. Space Sci. Rev. 30, 1147-1181, 1982)
      !  with diffuse reflection of air molecules assumed:

      f2 = (pi_m*ONE_NINTH_f4/tan(pi_m*ONE_SIXTH_f4))**ONE_THIRD_f4*  &
           eshape(ig)**ONE_SIXTH_f4

   elseif(ishape(ig) == 3) then    ! Spheroidal

      ! Taken FROM WHERE?:

      f2 = TWO_THIRDS_f4**ONE_THIRD_f4*eshape(ig)**ONE_SIXTH_f4

   endif

   ! Yields <f3> = 1. for <eshape(ig)> = 1 :
   f3 = 1.39_f4/sqrt((1.14_f4+0.25_f4/eshape(ig))*(0.89_f4+eshape(ig)*0.5_f4))
   f2 = f2*f3

   if(eshape(ig) > ONE_f4) then

      ! For the Stokes regime there is no separate data for hexagonal plates or
      ! columns, so we use prolate spheroids.  This is from Fuchs' book:

      exx = eshape(ig)**2-ONE_f4
      exy = sqrt(exx)
      xcc = 1.333_f4*exx/((2._f4*eshape(ig)**2-ONE_f4)*log(eshape(ig)+exy)/exy  &
           -eshape(ig))
      xa = 2.666_f4*exx/((2._f4*eshape(ig)**2-3._f4)*log(eshape(ig)+exy)/exy  &
           +eshape(ig))
      f1 = eshape(ig)**(-ONE_THIRD_f4)*(xcc+2._f4*xa)*ONE_THIRD_f4
      
   elseif(eshape(ig) < ONE_f4) then

      ! Use oblate spheroids for disks (eshape < 1.); Also from Fuchs' book:

      bxx = ONE_f4/eshape(ig)
      exx = bxx**2-ONE_f4
      exy = sqrt(exx)
      xcc = 1.333_f4*exx/(bxx*(bxx**2-2._f4)*atan(exy)/exy+bxx)
      xa = 2.666_f4*exx/(bxx*(3._f4*bxx**2-2._f4)*atan(exy)/exy-bxx)
      f1 = bxx**ONE_THIRD_f4*(xcc+2._f4*xa)*ONE_THIRD_f4

   endif

   do k=1,nz_mic      !  Loop over column

      ! This is just <rhoa> in Cartesian coordinates:

      rhoa_cgs = rhoa(k)

      ! <vg> is the mean thermal velocity of air molecules [cm/s]:

      vg = sqrt(8._f4/pi_m*R_air_cgs*t(k))

      ! <rmfp> is the mean free path of air molecules [cm]:

      rmfp = 2._f4*rmu(k)/(rhoa_cgs*vg)

      do ib=1,nbins(ig)         ! Loop over particle size bins

         ! <r_shape> is the radius of particle used to calculate <re>:

         if(ishape(ig) == 1) then    ! Spherical

            r_shape = r(ib,ig)
            
         elseif(ishape(ig) == 2) then   ! Hexagonal

            r_shape = r(ib,ig)*0.85_f4*eshape(ig)**(-ONE_THIRD_f4)

         elseif(ishape(ig) == 3) then   ! Spheroidal

            r_shape = r(ib,ig)*eshape(ig)**(-ONE_THIRD_f4)

         endif

         ! <rkn> is the Knudsen number:

         rkn = rmfp/r(ib,ig)

         ! <bpm> is correction term for particle shape and non-continuum
         !  effects. It is also used to calculate coagulation kernels and
         !  diffusion coefficients:

         expon = -0.87_f4/rkn
         expon = max(-POWMAX,expon)
         bpm(k,ib,ig) = ONE_f4+f1*f2*(1.246_f4*rkn+0.42_f4*rkn*exp(expon))

         ! These are first guesses for fall velocity and Reynolds' number; 
         !  valid for Reynolds' number < 0.01

         vf(k,ib,ig) = 2._f4*ONE_NINTH_f4*rhop(k,ib,ig)*(r(ib,ig)**2)  &
              *g_cgs*bpm(k,ib,ig)/(f1*rmu(k))
 
         re(k,ib,ig) = 2._f4*rhoa_cgs*r_shape*vf(k,ib,ig)/rmu(k)

         ! <rfix> is used in drag coefficient:

         rfix = vol(ib,ig)*rhop(k,ib,ig)*g_cgs*rhoa_cgs/rmu(k)**2

         if((re(k,ib,ig) >= 0.01_f4) .and. (re(k,ib,ig) <= 300._f4)) then

            ! This is "regime 2" in Pruppacher and Klett (chap. 10):

            if(ishape(ig) == 1) then     ! Spherical

               x = log(24._f4*re(k,ib,ig)/bpm(k,ib,ig))
               y = -3.18657_f4+x*(0.992696_f4-x*(0.00153193_f4  &
                    + x*(0.000987059_f4+x*(0.000578878_f4  &
                    - x*(8.55176e-05_f4-x*3.27815e-06_f4)))))
               if(y < -675._f4) then
                  y = -675._f4
               elseif(y >= 741._f4) then
                  y = 741._f4
               endif

               re(k,ib,ig) = exp(y)*bpm(k,ib,ig)

            elseif(eshape(ig) <= ONE_f4) then
               
               if(ishape(ig) == 2) then
                  x = log10(16._f4*rfix/(3._f4*sqrt(3._f4*ONE_f4)))
               elseif(ishape(ig) == 3) then
                  x = log10(8._f4*rfix/pi_m)
               endif

               if(eshape(ig) <= 0.2_f4) then
                  b0 = -1.33_f4
                  bb1 = 1.0217_f4
                  bb2 = -0.049018_f4
                  bb3 = ZERO_f4
               elseif(eshape(ig) <= 0.5_f4) then
                  ex = (eshape(ig)-0.2_f4)/0.3_f4
                  b0 = -1.33_f4+ex*(-1.3247_f4+1.33_f4*ONE_f4)
                  bb1 = 1.0217_f4+ex*(1.0396_f4-1.0217_f4*ONE_f4)
                  bb2 = -0.049018_f4+ex*(-0.047556_f4+0.049018_f4*ONE_f4)
                  bb3 = ZERO_f4+ex*(-0.002327_f4*ONE_f4)
               else
                  ex = (eshape(ig)-0.5_f4)/0.5_f4
                  b0 = -1.3247_f4+ex*(-1.310_f4+1.3247_f4*ONE_F4)
                  bb1 = 1.0396_f4+ex*(0.98968_f4-1.0396_f4*ONE_f4)
                  bb2 = -0.047556_f4+ex*(-0.042379_f4+0.047556_f4*ONE_f4)
                  bb3 = -0.002327_f4+ex*(0.002327_f4*ONE_F4)
               endif
               y = b0+x*(bb1+x*(bb2+x*bb3))
               
               re(k,ib,ig) = 10._f4**y*bpm(k,ib,ig)
               
            elseif(eshape(ig) > ONE_f4) then
               
               x = log10(2._f4*rfix/eshape(ig))
               
               if(eshape(ig) <= 2._f4) then
                  ex = eshape(ig)-ONE_f4
                  b0 = -1.310_f4+ex*(-1.11812_f4+1.310_f4*ONE_f4)
                  bb1 = 0.98968_f4+ex*(0.97084_f4-0.98968_f4*ONE_f4)
                  bb2 = -0.042379_f4+ex*(-0.058810_f4+0.042379_f4*ONE_F4)
                  bb3 = ZERO_f4+ex*(0.002159_f4*ONE_f4)
               elseif(eshape(ig) <= 10._f4) then
                  ex = (eshape(ig)-2._f4)*0.125_f4
                  b0 = -1.11812_f4+ex*(-0.90629_f4+1.11812_f4*ONE_f4)
                  bb1 = 0.97084_f4+ex*(0.90412_f4-0.97084_f4*ONE_f4)
                  bb2 = -0.058810_f4+ex*(-0.059312_f4+0.058810_f4*ONE_f4)
                  bb3 = 0.002159_f4+ex*(0.0029941_f4-0.002159_f4*ONE_f4)
               else
                  ex = 10._f4/eshape(ig)
                  b0 = -0.79888_f4+ex*(-0.90629_f4+0.79888_f4*ONE_f4)
                  bb1 = 0.80817_f4+ex*(0.90412_f4-0.80817_f4*ONE_f4)
                  bb2 = -0.030528_f4+ex*(-0.059312_f4+0.030528_f4*ONE_f4)
                  bb3 = ZERO_f4+ex*(0.0029941_f4*ONE_f4)
               endif
               y = b0+x*(bb1+x*(bb2+x*bb3))
               
               re(k,ib,ig) = 10._f4**y*bpm(k,ib,ig)
               
            endif

            ! Adjust <vf> for non-sphericicity:

            vf(k,ib,ig) = re(k,ib,ig)*rmu(k)/(2._f4*r_shape*rhoa_cgs)

         endif

         if(re(k,ib,ig) > 300._f4) then

            z = ((1.e6_f4*rhoa_cgs**2)/(g_cgs*rhop(k,ib,ig)*rmu(k)**4))**ONE_SIXTH_f4
            b0 = (24._f4*vf(k,ib,ig)*rmu(k))*0.01_f4
            x  = log(z*b0)
            y  = -5.00015_f4+x*(5.23778_f4-x*(2.04914_f4-x*(0.475294_f4  &
                 - x*(0.0542819_f4-x*0.00238449_f4))))
            if(y < -675._f4) then
               y = -675._f4
            elseif(y >= 741._f4) then
               y = 741._f4
            endif

            re(k,ib,ig) = z*exp(y)*bpm(k,ib,ig)
            vf(k,ib,ig) = re(k,ib,ig)*rmu(k)/(2._f4*r(ib,ig)*rhoa_cgs)

         endif

         ! Interpolate <vf> from layer mid-pts to layer boundaries.
         !  <vf(k)> is the fall velocity at the lower edge of the layer:

         if(k > 1 .and. k < nz_mic) then
            vf(k,ib,ig) = sqrt(vf(k-1,ib,ig)*vf(k,ib,ig) )
         endif

      enddo

   enddo

   do ib=1,nbins(ig)
      vf(nz_mic+1,ib,ig) = vf(nz_mic,ib,ig)
      if(nz_mic > 1) then
         vf(nz_mic,ib,ig) = sqrt(vf(nz_mic-1,ib,ig)*vf(nz_mic,ib,ig))
      endif
   enddo
          
enddo

! Constant value if <ifall> = 0:

if(ifall == 0) then
   do ig=1,ngroups
      do ib=1,nbins(ig)
         do k=1,nz1_mic
            vf(k,ib,ig) = vf_const
         enddo
      enddo
   enddo
endif

!^
!^k = 2
!^write (*,'(/,a)')  &
!^     'Fall velocities and Reynolds'' number in bottom layer (setupvfall): '
!^do ig=1,ngroups
!^   write (*,'(/,a,i3,/,a,/)') 'Particle group ',ig  &
!^        ,' bin   r [cm]       vf [cm/s]         re'
!^   do ib=1,nbins(ig)
!^      write(*,'(i3,3(1pe11.3,4x))') ib,r(ib,ig),vf(k,ib,ig),re(k,ib,ig)
!^   enddo
!^enddo
!^

! Scale Cartesian fallspeeds to the appropriate vertical coordinate system.
!  Non-Cartesian coordinates are assumed to be positive downward, but
!  vertical velocities in this model are always assumed to be positive upward:

if(igridv /= I_CART) then
   
   do k=1,nz1_mic
            
      do ig=1,ngroups
         do ib=1,nbins(ig)
            vf(k,ib,ig) = -vf(k,ib,ig)
         enddo
      enddo
      
   enddo

endif

end subroutine setup_vfall


!```````````````````````````````````````````````````````````````````
subroutine setup_grow1

!+ Calculates parameters (time-independent) used to calculate
!   condensational growth/evaporation

use univ_kind_defs_mod, only: f4
use univ_exception_mod, only: exception, FATAL
use aermicro_mem_mod, only: nelements,itype,I_CORE2MOM,igrowgas,ngasses,nbins  &
     ,ngroups,ge_pcfitconst,dm

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

!  Time-independent parameters that depend on particle radius are
!  defined in 'setupgkern'.

integer :: ie,ib,ig

!=== Executable statements:

! Check for configuration errors:

do ie=1,nelements

   if(ie > 1 .and. itype(ie) == I_CORE2MOM) then 
      if(igrowgas(ie) /= igrowgas(ie-1)) then
         call exception('setup_grow1', FATAL,  &
              'igrowgas array is inconsistent.')
      endif
   endif

   if(igrowgas(ie) > ngasses) then
      call exception('setup_grow1', FATAL,  &
           'component of igrowgas > ngasses.')
   endif

enddo

! Calculate constant parameters for use in `growevap_loss`:

do ig=1,ngroups
   do ib=2,nbins(ig)-1
      ge_pcfitconst(ib,ig,1) = dm(ib,ig)/(dm(ib-1,ig)+dm(ib,ig)+dm(ib+1,ig))
      ge_pcfitconst(ib,ig,2) = (2._f4*dm(ib-1,ig)+dm(ib,ig))/  &
           (dm(ib+1,ig)+dm(ib,ig))
      ge_pcfitconst(ib,ig,3) = (2._f4*dm(ib+1,ig)+dm(ib,ig))/  &
           (dm(ib-1,ig)+dm(ib,ig))
      ge_pcfitconst(ib,ig,4) = dm(ib,ig)/(dm(ib,ig)+dm(ib+1,ig))
      ge_pcfitconst(ib,ig,5) = 2._f4*dm(ib+1,ig)*dm(ib,ig)/(dm(ib,ig)+dm(ib+1,ig))
      ge_pcfitconst(ib,ig,6) = (dm(ib-1,ig)+dm(ib,ig))/  &
           (2._f4*dm(ib,ig)+dm(ib+1,ig))
   enddo
enddo
do ig=1,ngroups
   do ib=2,nbins(ig)-2
      ge_pcfitconst(ib,ig,7) = (dm(ib+2,ig)+dm(ib+1,ig))/  &
           ( 2._f4*dm(ib+1,ig)+dm(ib,ig))
      ge_pcfitconst(ib,ig,8) = dm(ib-1,ig)+dm(ib,ig)+dm(ib+1,ig)+dm(ib+2,ig)
   enddo
enddo

end subroutine setup_grow1


!```````````````````````````````````````````````````````````````````
subroutine setup_grow2

!+ Calculates parameters (time-dependent) used to calculate
!   condensational growth/evaporation

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: ngasses,nz_mic,gasname,diffus,p,t,rlhe,rlhm,rhoa,rmu

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! The parameters defined for each gas are 
!
!    gwtmol:   molecular weight [g/mol]
!    diffus:   diffusivity      [cm^2/s]
!    rlhe  :   latent heat of evaporation [cm^2/s^2]
!    rlhm  :   latent heat of melting [cm^2/s^2]
!
!  Time-independent parameters that depend on particle radius are
!  defined in 'setupgkern'.
!
!  This routine requires that vertical profiles of temperature <T>,
!  and pressure <p> are defined (i.e., initatm.f must be called before this).
!  The vertical profile with ix = iy = 1 is used.

integer :: k,igas
real(f4) :: rhoa_cgs

!=== Executable statements:

! Define parameters with weak time-dependence to be used in growth equation:

do k=1,nz_mic

   do igas=1,ngasses

      if(gasname(igas)(1:1) == 'W') then   ! Water vapor

         ! Diffusivity of water vapor in air from Pruppacher & Klett
         !  (eq. 13-3); units are [cm^2/s]:

         diffus(k,igas) = 0.211_f4*(1.01325e6_f4/p(k))*(t(k)/273.15_f4)**1.94_f4

         ! Latent heat of evaporation for water from Stull [erg/g]:

!         rlhe(k,igas) = (2.5_f4-0.00239_f4*(t(k)-273.16_f4)) !???? units???
         rlhe(k,igas) = (2.5e10_f4-2.344e7_f4*(t(k)-273.15_f4))   ! [erg/g]     
!@         rlhe(k,igas) = 1.e-12_f4   ! Latent heating due to H2O phase change *OFF*

         ! Latent heat of H2O ice melting from Pruppacher & Klett (eq. 4-85b);
         !  units are [erg/g ]:

!units????  rlhm(k,igas) = (79.7_f4+0.485_f4*(t(k)-273.16_f4)-2.5e-3_f4*((t(k)-273.16_f4)**2))
         rlhm(k,igas) = (3.3472e9_f4+2.5e6_f4*(t(k)-273.15_f4))   ! [erg/g]     
!@         rlhm(k,igas) = 1.e-12_f4   ! Latent heating due to H2O phase change *OFF*

      elseif(gasname(igas)(1:1) == 'C') then   ! Carbon dioxide

         ! Mars CO2; Diffusion (diffusivity??) from
         !  Kahn JGR 95, 1990 pp14,677-14,693 [cm^2/s]:
         
         diffus(k,igas) =  0.139_f4*(1.01325e6_f4/p(k))*(t(k)/273.15_f4)**1.75_f4

         ! Impose a self-diffusion limit for CO2 (???):

         rhoa_cgs = rhoa(k)
         if(diffus(k,igas) > 2._f4*rmu(k)/rhoa_cgs) then
            diffus(k,igas) = 2._f4*rmu(k)/rhoa_cgs
         endif

         ! Latent heat of CO2 ice melting [solid -> liquid] [cm^2/s^2]:

         rlhm(k,igas)= ZERO_f4    ! (this phase change doesn't occur on current Mars)

         ! Latent heat of evaporation for CO2 [solid -> gas] (from ????);
         !  [cm^2/s^2]:

!WRONG for current Mars???      rlhe(k,igas) = (6.2689e3_f4-9.8769_f4*t(k)-1.31e-1_f4*(t(k)**2)+  &
!              6.2735e-4_f4*(t(k)**3)-1.2699e-6_f4/(t(k)**4))*4.184_f4*1.e4_f4
         rlhe(k,igas) = 5.95e5_f4*1.e4_f4

      endif

   enddo
enddo

end subroutine setup_grow2


!```````````````````````````````````````````````````````````````````
subroutine setup_gkern1

!+ Defines time-independent parameters used to calculate condensational growth
!   kernel

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ONE_SIXTH_f4, TWO_THIRDS_f4,  &
     ONE_THIRD_f4
use aermicro_mem_mod, only: ngroups,is_grp_ice,ishape,pi_m,eshape,tstick,  &
     gstick,cor,phish

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ig
real(f4) :: gstickl,gsticki,esh1,a1

!=== Executable statements:

! Specify radius-independent parameters:

! Sticking coefficient for growth (also called mass accomodation coefficient):

gstickl = ONE_f4
gsticki = ONE_f4

do ig=1,ngroups    ! Loop over groups only (no radius, gas, or xy dependence)

   ! Thermal accommodation coefficient:

   tstick(ig) = ONE_f4

   ! Use gstickl or gsticki, depending on whether group is ice or not:

   if(is_grp_ice(ig)) then
      gstick(ig) = gsticki
   else
      gstick(ig) = gstickl
   endif

   ! Non-spherical corrections (need a reference for these):

   if(ishape(ig) == 1) then   ! Spheres

      cor(ig) = ONE_f4
      phish(ig) = ONE_f4

   else 

      if(ishape(ig) == 2) then     ! Hexagons

         phish(ig) = 6._f4/pi_m*tan(pi_m*ONE_SIXTH_f4)*(eshape(ig)+0.5_f4)  &
              *(pi_m/(9._f4*eshape(ig)*tan(pi_m*ONE_SIXTH_f4)))**TWO_THIRDS_f4

      elseif(ishape(ig) == 3) then    ! Spheroids

         phish(ig) = (eshape(ig)+0.5_f4)*(2._f4/(3._f4*eshape(ig)))**TWO_THIRDS_f4

      endif

      if(eshape(ig) < ONE_f4) then    ! Oblate spheroids

         esh1 = ONE_f4/eshape(ig)
         a1 = sqrt(esh1**2-ONE_f4)
         cor(ig) = a1/asin(a1/esh1)/esh1**TWO_THIRDS_f4

      else     ! Prolate spheroids

         a1 = sqrt(eshape(ig)**2-ONE_f4)
         cor(ig) = a1/log(eshape(ig)+a1)/eshape(ig)**ONE_THIRD_f4

      endif
   endif

enddo

end subroutine setup_gkern1


!```````````````````````````````````````````````````````````````````
subroutine setup_gkern2

!+ Defines radius-dependent and time-dependent parameters used to
!   calculate condensational growth kernel for particles

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4, ONE_THIRD_f4
use aermicro_mem_mod, only: ngroups,is_grp_ice,ishape,pi_m,eshape, ienconc  &
     ,nz_mic,igrowgas,rhoa,gasname,surfctwa,surfctiw,surfctia,t,akelvin  &
     ,akelvini,gwtmol,RHO_W,R_univ_cgs,diffus,thcond,cp_cgs,wtmol_air,rlhe  &
     ,rlhm,nbins,rlow,re,rmu,ft,gro,gro1,gro2,bk,avog,tstick,gstick,cor,  &
     phish,RHO_I,RHO_IC

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Growth rates are calculated at bin boundaries: the parameters calculated here 
!  ( <gro>, <gro1>, <gro2>, and <akelvin> ) 
!  are defined at lower bin boundaries through the growth rate expression
!  (for one particle) used in growevapl.f:
!
!    dm = gro*pvap*( S + 1 - Ak*As + gro1*gro2*qrad )
!    --   -------------------------------------------
!    dt               1 + gro*gro1*pvap
!
!   where 
!
!   S    = supersaturation
!   Ak   = exp(akelvin/r)
!   As   = exp(-sol_ions * solute_mass/solwtmol * gwtmol/condensate_mass)
!   pvap = saturation vapor pressure [dyne/cm^2]
!   qrad = radiative energy absorbed
!
!  This routine requires that vertical profiles of temperature <T>,
!  and pressure <p> are defined (initatm.f must be called before this).
!
!  This routine also requires that particle Reynolds' numbers are
!  defined (setupvfall.f must be called before this).

integer :: ig,ie,igas,k,ib
real(f4) :: freep,freept,rlh  &
     ,br,rknudn,rknudnt,rlam,rlamt,diffus1,thcond1,reyn_shape,schn,prnum,x1  &
     ,x2,fv,rhoa_cgs

!=== Executable statements:

do ig=1,ngroups    ! Loop over groups only (no radius, gas, or xy dependence)

   ! Evaluate growth terms only for particle elements that grow:

   ie = ienconc(ig)    ! Particle number concentration element
   igas = igrowgas(ie)     ! condensing gas is <igas>

   if(igas /= 0) then

      do k=1,nz_mic      ! Loop over vertical layers

         ! Radius-independent parameters for condensing gas:

         rhoa_cgs = rhoa(k)

         if(gasname(igas)(1:1) == 'W') then    ! Condensing gas is water vapor

            ! <surfctwa> is surface tension of water-air interface (valid
            !  from 0 to 40 C) from Pruppacher and Klett (eq. 5-12):

            surfctwa(k,igas) = 76.10_f4-0.155_f4*(t(k)-273.16_f4)
            if(surfctwa(k,igas) < ZERO_f4) then
               surfctwa(k,igas) = 1.e-6_f4
            endif
	      
            ! <surfctiw> is surface tension of water-ice interface
            !  from Pruppacher and Klett (eq. 5-48):

            surfctiw(k,igas) = 28.5_f4+0.25_f4*(t(k)-273.16_f4)
            if(surfctiw(k,igas) < ONE_f4) then
               surfctiw(k,igas) = ONE_f4
            endif

            ! <surfctia> is surface tension of ice-air interface
            !  from Hale and Plummer [J. Chem. Phys., 61, 1974]:

            surfctia(k,igas) = 141._f4+0.15_f4*t(k)

            ! <akelvin> is argument of exponential in Kelvin curvature term:

            akelvin(k,igas) = 2._f4*gwtmol(igas)*surfctwa(k,igas)  &  ! water-air
                 /(t(k)*RHO_W*R_univ_cgs)
 
            akelvini(k,igas) = 2._f4*gwtmol(igas)*surfctia(k,igas)  &  ! ice-air
                 /(t(k)*RHO_I*R_univ_cgs)
 
         elseif(gasname(igas)(1:1) == 'C') then  ! Condensing gas is CO2 (Mars)

            ! From, etc.????

!alt%            surfctiw(k,igas) = 76.10-0.155*(t(k)-273.16)
            surfctiw(k,igas) = 62._f4
            surfctia(k,igas) = surfctiw(k,igas)
            surfctwa(k,igas) = surfctiw(k,igas)

            akelvin(k,igas) = 2._f4*gwtmol(igas)*surfctwa(k,igas)  &
                 /(t(k)*RHO_IC*R_univ_cgs)

            akelvini(k,igas) = 2._f4*gwtmol(igas)*surfctia(k,igas)  &
                 /(t(k)*RHO_IC*R_univ_cgs)
     
         endif

         ! Molecular free path of condensing gas:

         freep = 3._f4*diffus(k,igas)*sqrt((pi_m*gwtmol(igas))/  &
              (8._f4*R_univ_cgs*t(k)))

         ! Thermal free path of condensing gas:

         freept = freep*thcond(k)/(diffus(k,igas)*rhoa_cgs  &
              *(cp_cgs-R_univ_cgs/(2._f4*WTMOL_AIR)))

         ! Latent heat of condensing gas:

         if(is_grp_ice(ig)) then
            rlh = rlhe(k,igas)+rlhm(k,igas)
         else
            rlh = rlhe(k,igas)
         endif

         ! Radius-dependent parameters for condensing gas:

         do ib=1,nbins(ig)    ! Loop over all bins in group
            br = rlow(ib,ig)     ! particle bin boundary radius

            ! These are Knudsen numbers:

            rknudn  = freep/br
            rknudnt = freept/br

            ! These are "lambdas" used in correction for gas kinetic effects:

            rlam = (1.33_f4*rknudn+0.71_f4)/(rknudn+ONE_f4)+(4._f4*(ONE_f4-gstick(ig)))/  &
                 (3._f4*gstick(ig))

            rlamt = (1.33_f4*rknudnt+0.71_f4)/(rknudnt+ONE_f4)+  &
                 (4._f4*(ONE_f4-tstick(ig)))/(3._f4*tstick(ig))

            ! Diffusion coefficient and thermal conductivity modified for
            !  free molecular limit and for particle shape:

            diffus1 = diffus(k,igas)*cor(ig)/(ONE_f4+rlam*rknudn*cor(ig)/phish(ig))
            thcond1 = thcond(k)*cor(ig)/(ONE_f4+rlamt*rknudnt*cor(ig)/phish(ig))

            ! Reynolds' number based on particle shape <reyn_shape>:

            if(ishape(ig) == 1) then
               reyn_shape = re(k,ib,ig)
            elseif(eshape(ig) < ONE_f4) then
               reyn_shape = re(k,ib,ig)*(ONE_f4+2._f4*eshape(ig))
            else
               reyn_shape = re(k,ib,ig)*pi_m*(ONE_f4+2._f4*eshape(ig))  &
                    /(2._f4*(ONE_f4+eshape(ig)))
            endif

            ! Particle Schmidt number:

            schn = rmu(k)/(rhoa_cgs*diffus1)

            ! Prandtl number:

            prnum = rmu(k)*cp_cgs/thcond1

            ! Ventilation factors <fv> and <ft> from Pruppacher and Klett:
  
            x1 = schn**ONE_THIRD_f4*sqrt(reyn_shape)
            x2 = prnum**ONE_THIRD_f4*sqrt(reyn_shape)

            if(is_grp_ice(ig)) then    ! Ice crystals

               if(x1 <= ONE_f4) then
                  fv = ONE_f4+0.14_f4*x1**2
               else
                  fv = 0.86_f4+0.28_f4*x1
               endif

               if(x2 <= ONE_f4) then
                  ft(k,ib,ig) = ONE_f4+0.14_f4*x2**2
               else
                  ft(k,ib,ig) = 0.86_f4+0.28_f4*x2
               endif

            else     ! Liquid drops

               if(x1 <= 1.4_f4) then
                  fv = ONE_f4+0.108_f4*x1**2
               else
                  fv = 0.78_f4+0.308_f4*x1
               endif

               if(x2 <= 1.4_f4) then
                  ft(k,ib,ig) = ONE_f4+0.108_f4*x2**2
               else
                  ft(k,ib,ig) = 0.78_f4+0.308_f4*x2
               endif

            endif

            ! Growth kernel for particle without radiation or heat conduction
            !  at radius lower boundary [(g.cm^3)/(erg.s)]:
          
            gro(k,ib,ig) = 4._f4*pi_m*br*diffus1*fv*gwtmol(igas)/(BK*t(k)*avog)   

            ! Coefficient for conduction term in growth kernel [s/g]:

            gro1(k,ib,ig) = gwtmol(igas)*rlh**2/  &
                 (R_univ_cgs*t(k)**2*ft(k,ib,ig)*thcond1)/(4._f4*pi_m*br)

            ! Coefficient for radiation term in growth kernel [g/erg]
            !  (note: no radial dependence):

            if(ib == 1) then
               gro2(k,ig) = ONE_f4/rlh
            endif
 
         enddo
      enddo
   endif
enddo

end subroutine setup_gkern2


!```````````````````````````````````````````````````````````````````
subroutine setup_nuc

!+ Evaluates derived mapping arrays and calculates the critical supersaturation
!   <scrit> used to nucleate dry particles (CN) to droplets

use univ_kind_defs_mod, only: f4
use univ_exception_mod, only: exception, FATAL
use aermicro_mem_mod, only: ngroups,nbins,inuc2bin,rmass,ienconc,ncore,icorelem  &
     ,nnucelem,nelements,if_nuc,inucelem,imomelem,igelem,nnucbin,inucbin  &
     ,ngasses,itype,I_COREMASS,inucgas,nnuc2elem,nsolutes,inuc2elem  &
     ,I_INVOLATILE,nz_mic,scrit,akelvini,R_univ_cgs,t,RHO_W,r,adelf,bdelf  &
     ,prenuc,RHO_I,rmiv,ievp2elem

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! This routine requires that array <akelvin> is defined.
!  (i.e., setupgkern.f must be called before this)

logical :: bad_val
integer :: ibfrom,igfrom,igto,ibto,ig,jcore,iecore,iefrom,ip,im  &
     ,jnucelem,ib,ie,inuc,igas,inuc2,neto,k,ieto
!^integer(i4) :: nnucb
!%integer(i4) :: isol
!%real(f4) :: bsol

!=== Executable statements:

! Bin mapping for nucleation : nucleation would transfer mass from particles
!  in <ibfrom,igfrom> into target bin <inuc2bin(ibfrom,igfrom,igto)> in group
!  <igto>.  The target bin is the smallest bin in the target size grid with
!  mass exceeding that of nucleated particle:
  
do igfrom=1,ngroups    ! nucleation source group
   do igto=1,ngroups           ! nucleation target group

      do ibfrom=1,nbins(igfrom)     ! nucleation source bin

         inuc2bin(ibfrom,igfrom,igto) = 0

         do ibto=nbins(igto),1,-1        ! nucleation target bin

            if(rmass(ibto,igto) >= rmass(ibfrom,igfrom)) then
               inuc2bin(ibfrom,igfrom,igto) = ibto
            endif

         enddo
!^         print *,'ibf,igf,igt,inuc2bin',ibfrom,igfrom,igto  &
!^              ,inuc2bin(ibfrom,igfrom,igto)
      enddo
   enddo
enddo

! Mappings for nucleation sources: 
!
!   <nnucelem(ielem)> is the number of particle elements that nucleate to
!    particle element <ielem>.
!
!   <inucelem(jefrom,ielem)> are the particle elements that
!    nucleate to particle element <ielem>, where 
!    jefrom = 1,nnucelem(ielem).
!
!   <nnucbin(igfrom,ibin,igroup)> is the number of particle bins that nucleate
!    to particles in bin <ibin,igroup> from group <igfrom>.
!
!   <inucbin(jfrom,igfrom,ibin,igto)> are the particle bins 
!    that nucleate to particles in bin <ibin,igto>, where
!    jfrom = 1,nnucbin(igfrom,ibin,igto).
  
! First enumerate and count elements that nucleate to cores:

do ig=1,ngroups

   ip = ienconc(ig)    ! target particle number concentration element

   do jcore=1,ncore(ig)

      iecore = icorelem(jcore,ig)    ! target core element 
      nnucelem(iecore) = 0

      do iefrom=1,nelements

         if(if_nuc(iefrom,iecore)) then
            nnucelem(iecore) = nnucelem(iecore)+1
            inucelem(nnucelem(iecore),iecore) = iefrom
         endif

      enddo

   enddo
enddo

! Now enumerate and count elements nucleating to particle concentration
!  (itype=I_INVOLATILE and itype=I_VOLATILE) and core second moment
!  (itype=I_COREMASS).  Elements with itype = I_VOLATILE are special because all
!  nucleation sources for core elements in same group are also sources
!  for the itype = I_VOLATILE element:

do ig=1,ngroups
   
   ip = ienconc(ig)    ! target particle number concentration element
   im = imomelem(ig)   ! target core second moment element
 
   nnucelem(ip) = 0
   if(im > 0) then
      nnucelem(im) = 0
   endif
 
   do jcore=1,ncore(ig)
 
      iecore = icorelem(jcore,ig)       ! target core mass element
 
      do jnucelem = 1,nnucelem(iecore)  ! elements nucleating to cores
 
         iefrom = inucelem(jnucelem,iecore)  ! source

         ! For particle concentration target elements, only count source
         !  elements that are also particle concentrations:

         nnucelem(ip) = nnucelem(ip)+1
         inucelem(nnucelem(ip),ip) = ienconc(igelem(iefrom))

         if(im /= 0) then
            nnucelem(im) = nnucelem(im)+1
            inucelem(nnucelem(im),im) = iefrom
         endif
 
      enddo

   enddo
enddo

! Now enumerate and count nucleating bins:

do ig=1,ngroups        ! target group
   do ib=1,nbins(ig)     ! target bin

      do igfrom=1,ngroups         ! source group
         nnucbin(igfrom,ib,ig) = 0

         do ibfrom=1,nbins(igfrom)        ! source bin
            if(inuc2bin(ibfrom,igfrom,ig) == ib) then
               nnucbin(igfrom,ib,ig) = nnucbin(igfrom,ib,ig)+1
               inucbin(nnucbin(igfrom,ib,ig),igfrom,ib,ig) = ibfrom
            endif
         enddo
      enddo

   enddo
enddo

!^
!^print *,' '
!^print *,'Nucleation mapping arrays (setupnuc):'
!^print *,' '
!^print *,'Elements mapping:'
!^do ie=1,nelements
!^   print *,' '
!^   print *,'ie,nnucelem=',ie,nnucelem(ie)
!^   if(nnucelem(ie) > 0) then
!^      do iefrom=1,nnucelem(ie)
!^         print *,'iefrom,inucelem = ',iefrom,inucelem(iefrom,ie)
!^      enddo
!^   endif
!^enddo
!^print *,' '
!^print *,'Bin mapping:'
!^do igfrom=1,ngroups
!^   do igto=1,ngroups
!^      print *,' '
!^      print *,'Groups (from, to) = ', igfrom, igto
!^     do ibto=1,nbins(igto)
!^         nnucb = nnucbin(igfrom,ibto,igto)
!^         if(nnucb == 0) print *,'  None for bin ',ibto
!^         if(nnucb > 0) then
!^            print *,'  ibin,nnucbin=',ibto,nnucb
!^            print *,'   inucbin=',inucbin(1:nnucb,igfrom,ibto,igto)
!^         endif
!^      enddo
!^   enddo
!^enddo
!^

! Check that various values are valid:

do ie=1,nelements
 
!%   if(isolelem(ie) > nsolutes) then
!%      write (*,'(/a/)') 'ERROR: component of isolelem > nsolutes'
!%      stop
!%   endif
 
   if(ievp2elem(ie) > nelements) then
      call exception('setup_nuc', FATAL,  &
           'component of ievp2elem > NELEM')
   endif

   ! Check that <isolelem> is consistent with <ievp2elem>:

!%   if(ievp2elem(ie) /= 0 .and. itype(ie) == I_COREMASS) then
!%      if(isolelem(ie) /= isolelem(ievp2elem(ie))) then
!%         write(*,'(/a/)') 'ERROR: isolelem and ievp2elem are inconsistent'
!%         stop
!%      endif
!%   endif

   ! Check that <isolelem> is consistent with <inucgas>:

!%   do igas_loop=1,ngasses      ! Loop over gasses
!%       
!%      igas = inucgas(igelem(ie),igas_loop!)
!%
!%      if(igas /= 0) then
!%         if(itype(ie) == I_COREMASS .and. isolelem(ie) == 0) then
!%            write (*,'(/a/)') 'ERROR: inucgas /= 0, but isolelem == 0'  !c
!%            stop               !c
!%         endif
!%      endif
!%   enddo

   ! Check that <inuc2elem> is consistent with <nnuc2elem>:

   if(nnuc2elem(ie) > 0) then
      do inuc2=1,nnuc2elem(ie)
         if(inuc2elem(inuc2,ie) > nelements) then
            call exception('setup_nuc', FATAL,  &
                 'component of inuc2elem > nelements')
         endif
      enddo
   endif

enddo
 
! Particle grids are incompatible if there is no target bin with enough
!  mass to accomodate nucleated particle:

bad_val = .FALSE.

do iefrom=1,nelements          ! source element
   igfrom = igelem(iefrom)
   neto = nnuc2elem(iefrom)
   if(neto > 0) then
      do inuc2=1,neto
         ieto = inuc2elem(inuc2,iefrom)
         igto = igelem(ieto) 
         do ibfrom=1,nbins(igfrom)     ! source bin
            if(inuc2bin(ibfrom,igfrom,igto) == 0) then
               write (*,'(/a,i3,a,i3,a,i3,a/)')  &
                    'WARNING: nucleation cannot occur from group',igfrom  &
                    ,'   bin',ibfrom,'   into group',igto  &
                    ,'   (<inuc2bin> is zero)'
               bad_val = .TRUE.
            endif
         enddo
      enddo
   endif
enddo

if(bad_val) then
   call exception('setup_nuc', FATAL,  &
        'incompatible grids for nucleation')
endif

! Define critical supersaturation and target bin for each (dry) particle
!  size bin that is subject to nucleation.
!  (only for CN groups subject to nucleation):

do ig=1,ngroups

   ie = ienconc(ig)             ! element of particle number concentration

   do inuc=1,nnuc2elem(ie)       ! Loop over all nucleation paths

      igas = inucgas(inuc,ig)

      if(itype(ie) == I_INVOLATILE) then

!%         isol = isolelem(ie)
 
         do ib=1,nbins(ig)

            ! This is term "B" in Pruppacher and Klett's eqn. 6-28:

!%            bsol = 3.*sol_ions(isol)*rmass(ib,ig)*gwtmol(igas)  &
!%                 /(4.*pi_m*solwtmol(isol)*RHO_W)

            ! Loop over vertical grid layers because of temperature dependence
            !  in solute term:

            do k=1,nz_mic
!NOT currently used%               scrit(k,ib,ig) = akelvini(k,igas)  &
! (water only)%                    /(R_univ_cgs*t(k)*RHO_W*r(ib,ig))

!%               scrit(k,ib,ig) = sqrt(4.*akelvin(k,igas)**3/(27.*bsol))
            enddo

         enddo
      endif
   enddo
enddo
    
! Define parameters needed for freezing nucleation calculations:

adelf = 1.29e-12_f4
bdelf = 0.05_f4
prenuc = 2.075e33_f4*RHO_W/RHO_I
rmiv = 0.6_f4

!^1 format(a,':  ',12i6)
!^2 format(/,a,':  ',i6)
!^3 format(a,a)
!^4 format(a,':  ',1pe12.3)
!^5 format(/,'Particle nucleation mapping arrays (setupnuc):')
!^6 format(i4,5x,1p2e11.3)
!^7 format(/,'Warning: nucleation cannot occur from group',i3,
!^ '   bin',i3,'   into group',i3,'   (<inuc2bin> is zero)')
!^8 format(/,'Critical supersaturations for ',a,//,
!^ '   i        r [cm]     scrit',/)
!^
!^write(LUNOPRT,5)
!^write(LUNOPRT,1) 'inucgas  ',(inucgas(1,i),i=1,NGAS)
!^write(LUNOPRT,1) 'inuc2elem',(inuc2elem(1,i),i=1,NELEM)
!^write(LUNOPRT,1) 'ievp2elem',(ievp2elem(i),i=1,NELEM)
!^write(LUNOPRT,1) 'isolelem ',(isolelem(i),i=1,NELEM)
!^
!^do isol = 1,NSOLUTE
!^
!^   write(LUNOPRT,2) 'solute number   ',isol
!^   write(LUNOPRT,3) 'solute name:    ',solname(isol)
!^   write(LUNOPRT,4) 'molecular weight',solwtmol(isol)
!^   write(LUNOPRT,4) 'mass density    ',rhosol(isol)
!^  
!^   do igroup = 1,NGROUP
!^      
!^      if( isol .eq. isolelem(ienconc(igroup)) )then
!^         
!^         write(LUNOPRT,8) groupname(igroup)
!^         
!^         write(LUNOPRT,6) (i,r(i,igroup),scrit(1,i,igroup),i=1,NBIN)
!^        
!^      endif
!^   enddo
!^enddo

end subroutine setup_nuc


end module aemic_init_mod
