!```````````````````````````````````````````````````````````````````
module aemic_init_astruct_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine init_astruct(ext_n1,ext_n2,ext_n3,ext_density,ext_theta  &
     ,ext_press,ext_tempk,ext_zlev,ext_zmid,ext_rtgt,ext_ka)

!+ Initializes model grid and some atmospheric structure parameters

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use aermicro_mem_mod, only: igridv,I_CART,I_SIG,nz_mic,nz1_mic,zc_static  &
     ,zl_static,ibbnd_pc,itbnd_pc,I_FLUX_SPEC,dkz,t,p,rhoa,rmu,thcond,w

implicit none

!=== Argument declarations:

integer, intent(IN) :: ext_n1,ext_n2,ext_n3,ext_ka
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(IN) :: ext_density  &
     ,ext_press,ext_tempk,ext_theta
real(f4), dimension(ext_n2,ext_n3), intent(IN) :: ext_rtgt
real(f4), dimension(ext_n1), intent(IN) :: ext_zlev,ext_zmid 

!== Local declarations:

!**Variables defined at vertical layer boundaries:
!
!  Geometry:
!    zl         vertical coord (may be time-dependent)
!    zl_static  vertical coord basis (time-independent)
!
!  Physical variables:
!    w          vertical velocity (NOTE: in all coordinates <w> is positive
!               in the direction of increasing height AGL; e.g., for updrafts)
!    dkz        vertical diffusion coefficient
!
!**Variables defined at vertical layer mid-points:
!
!  Geometry:
!    zc         vertical coord (may be time-dependent)
!    zc_static  vertical coord basis (time-independent)
!
!  Physical variables:
!    p          air pressure [dyne/cm^2]
!    rhoa       scaled air density [g/cm^3]
!    t          air temperature [K]
!    ptc        potential temperature concentration [(K.g)/cm^3]
!    rmu        dynamic air viscosity [g/(cm.s)]
!    thcond     thermal conductivity of dry air [erg/(cm.s.K)]

real(f4) :: wsign
integer :: k,kb,ke,idk

!=== Executable statements:

!  Define the type of grid coordinate system being used:
!   (igridv is for vertical coordinate)
!
!  Possible values for igridv:
!       I_CART    Cartesian
!       I_SIG     sigma (-pressure?)

igridv = I_CART

! Do appropriate grid-related initializations depending on grid type:

if(igridv == I_CART) then
   call g_cart(ext_n1,ext_n2,ext_n3,ext_density,ext_theta,ext_press  &
        ,ext_tempk,ext_zlev,ext_zmid,ext_rtgt,ext_ka)
elseif(igridv == I_SIG) then
   call g_sig(ext_n1,ext_n2,ext_n3,ext_density,ext_theta,ext_press  &
        ,ext_tempk,ext_zlev,ext_zmid,ext_rtgt,ext_ka)
else
   write (*,'(/a/)') 'ERROR: Unknown microphysics vertical grid.'
   stop
endif

! Hydrostatically balance initial atmospheric profiles (ONLY perform if
!  external (calling) model is hydrostatic): 

!%if(???)then
!%   call hydrostat
!%endif

! Define appropriate sign factor for `w`.  Vertical non-Cartesian coordinates
!  are assumed to be positive downward, but `w` in this model is always assumed
!  to be positive upward:

if(igridv /= I_CART) then
   wsign = -ONE_f4
else
   wsign = ONE_f4
endif

! Set boundary vertical velocities and diffusion coefficients to zero (since
!  vertical transport ignores them); change sign of <w> if needed:

do k=1,nz1_mic

   if(k == 1 .and. ibbnd_pc == I_FLUX_SPEC) then
      dkz(k) = ZERO_f4   ! [cm^2/s]
      w(k) = ZERO_f4      ! [cm/s]
   endif

   if(k == nz1_mic .and. itbnd_pc == I_FLUX_SPEC) then
      dkz(k) = ZERO_f4   ! [cm^2/s]
      w(k) = ZERO_f4      ! [cm/s]
   endif

   w(k) = wsign*w(k)

enddo

! Set vertical loop index to increment downwards (in height):

if(igridv == I_CART) then
   kb = nz_mic
   ke = 1
   idk = -1
elseif(igridv == I_SIG) then
   kb = 1
   ke = nz_mic
   idk = 1
endif

! Print initial atmospheric structure:

!^write (*,'(/a)') 'Sample atm structure for aerosol model:'
!^write(*,'(/,a3,6(2x,a11))') 'k','zc_static','p','rhoa','t','rmu','thcond'
!^do k=kb,ke,idk
!^   write (*,'(i3,1p,6(2x,e11.3)')  &
!^        k,zc_static(k),p(k),rhoa(k),t(k),rmu(k),thcond(k)
!^enddo

! Print detailed vertical structure at horizontal grid point (ix,iy)

!^write (*,'(/a)') 'Detailed vertical grid structure for aerosol model:'
!^write(*,'(/,a3,6(2x,a11))') 'k','zc_static(k)','zl_static(k)'  &
!^     ,'zl_static(k+1)','w(k)'
!^do k=kb,ke,idk
!^   write (*,'(i3,1p,6(2x,e11.3)')  &
!^        k,zc_static(k),zl_static(k),zl_static(k+1),w(k)
!^enddo

end subroutine init_astruct


!```````````````````````````````````````````````````````````````````
subroutine g_cart(ext_n1,ext_n2,ext_n3,ext_density,ext_theta,ext_press  &
     ,ext_tempk,ext_zlev,ext_zmid,ext_rtgt,ext_ka)

!+ Initializes aerosol model grid for Cartesian vertical grid coordinates

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: zl_static,zc_static,dz,t,p,rhoa,ptc,rmu,thcond,w,dkz  &
     ,p_surf,p_top,m_to_cm,nz_mic,nz1_mic

implicit none

!=== Argument declarations:

integer, intent(IN) :: ext_n1,ext_n2,ext_n3,ext_ka
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(IN) :: ext_density  &
     ,ext_press,ext_tempk
real(f4), dimension(ext_n1), intent(IN) :: ext_zlev,ext_zmid
real(f4), dimension(ext_n2,ext_n3), intent(IN) :: ext_rtgt
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(IN) :: ext_theta

!== Local declarations:

! NOTE: Vertical coordinate is Cartesian altitude, z is distance in [cm];
!  z is positive up.

real(f4) :: tmp
integer :: k,ir,jr,koff,k1

!=== Executable statements:

koff = ext_ka-1   ! Offset between external model and this model's vert levels

! Define reference point indices to use for init purposes:

ir = max(1,ext_n2/2)
jr = max(1,ext_n3/2)

! Define surface pressure [dyne/cm^2]:
!    (*NOT USED IN NONHYDROSTATIC CONFIGURATION*) 

p_surf = (ext_press(max(1,ext_ka-1),ir,jr)+ext_press(ext_ka,ir,jr))*0.5_f4

! Pressure at top of model domain [dyne/cm^2]:
!    (*NOT USED IN NONHYDROSTATIC CONFIGURATION*)

p_top = ext_press(ext_n1,ir,jr)     ! Not really top (0.5 lyr off)

! Initialize *static* model vertical grid structure [later used to calculate
!  true (and likely time-dependent) vertical grid levels]:

zl_static(1) = ZERO_f4   ! Special value for bottom level
                    !  [zl_static(1) = ext_zlev(1)*m_to_cm]
do k=1,nz_mic   ! Loop over number of model layers
   k1 = k+1

   zl_static(k1) = ext_zlev(k1)*m_to_cm    ! Height of top of layer [cm]
   zc_static(k) = ext_zmid(k+koff)*m_to_cm   ! Height of layer midpoint [cm]

   dz(k) = zl_static(k1)-zl_static(k)            ! Vertical gridspacing [cm]
enddo

! Fill model model arrays with "representative" atmospheric profile:

do k=1,nz_mic   ! Loop over number of model layers
   k1 = k+koff

   t(k) = ext_tempk(k1,ir,jr)                   ! [K]
   p(k) = ext_press(k1,ir,jr)                      ! [dyne/cm^2]
   rhoa(k) = ext_density(k1,ir,jr)               ! [g/cm^3]
   ptc(k) = rhoa(k)*ext_theta(k1,ir,jr)        ! [(g.K)/cm^3]

!@alt_formulation   rmu(k) = 5.65e-8*((t(k)**2.5)/t(k)+269.)
   rmu(k) = 1.59e-6_f4*t(k)**1.5_f4/(t(k)+244.4_f4)*10._f4 ! Viscosity (CO2 atm) [g/(cm.s)]

   thcond(k) = (0.0445_f4*t(k)+ &     ! Conductivity (CO2 atm) [erg/(cm.s.K)]
        5.5e5_f4*(t(k)**2)-1.5_f4)*4.18e-7_f4

enddo

do k=1,nz1_mic   ! Loop over number of model layer boundaries

   w(k) = ZERO_f4  ! No vertical motion at init time [cm/s]

   dkz(k) = tiny(tmp)   ! Vertical diffusion coeff (set to OFF) [cm^2/s]

enddo

end subroutine g_cart


!```````````````````````````````````````````````````````````````````
subroutine g_sig(ext_n1,ext_n2,ext_n3,ext_density,ext_theta,ext_press  &
     ,ext_tempk,ext_zlev,ext_zmid,ext_rtgt,ext_ka)

!+ Initializes aerosol model grid for sigma-type vertical grid coordinates
!     %%%%% NOT COMPLETED YET

use univ_kind_defs_mod, only: f4
use aermicro_mem_mod, only:

implicit none

!=== Argument declarations:

integer, intent(IN) :: ext_n1,ext_n2,ext_n3,ext_ka
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(IN) :: ext_density  &
     ,ext_press,ext_tempk
real(f4), dimension(ext_n1,ext_n2,ext_n3), intent(IN) :: ext_theta
real(f4), dimension(ext_n1), intent(IN) :: ext_zlev,ext_zmid
real(f4), dimension(ext_n2,ext_n3), intent(IN) :: ext_rtgt

!== Local declarations:

! NOTE: Vertical coordinate is sigma-type; z is positive up.

!  @(#) g_cart_sig.f  Ackerman  Aug-1997
!  This routine handles atmospheric grid initialization
!  for the horizontal=cartesian and vertical=sigma grid coordinates.
!
!  For this coord system and grid:
!   x,y are regular measurable distance in cm.
!   x is positive to the right (east), y is positive north. z is positive up.
!   xmet, ymet metric factors are all unity with no units.
!   z is normalized (unitless) pressure, with fixed pressure (p_top) at top of atm.
!   z=0 at top of atm, z=1 at bottom of atm (positive toward planet surface).
!   zmet metric factor makes z space into vertical distance (altitude)
!
!  Modified slightly for use with generalized coordinates.  Jul-1997, McKie.
!  (From v1.11 single geometrical coord system model's initatm.f routine.)

!%real(f4) :: rkappa,t_sfc,dlapse,rmu_0,rmu_t0,rmu_c,rmu_const
!%integer :: k

!=== Executable statements:

!  Announce entry to this routine

!%if( DEBUG ) write(LUNOPRT,'(/,a)') 'Enter g_cart_sig'

!  Define bogus values for grid selection parameters that have no meaning
!  for this coordinate system.

!%rlon0 = -999.
!%rlat0 = -999.
!%rlat1 = -999.
!%rlat2 = -999.
!%hemisph = -999.

!  Define descriptive text for type of grid

!%gridname = 'cartesian horizontal, sigma vertical'

! For this example, grid spacing is uniform.
!  Define its east-west, north-south fixed grid box size.

!%!      dxfix = 1.e4
!%!      dyfix = 1.e4


!  Define lower left (southwest) upper right (northest) horizontal
!  domain limits for the grid

!%!      dom_llx = 0.
!%!      dom_lly = 0.
!%!      dom_urx = NX * dxfix
!%!      dom_ury = NY * dyfix

!  Specify horizontal grid coordinates and grid box sizes.
!   <dx> and <dy> are the east-west and north-south grid box thicknesses.
!   <xc>, <yc> are the coord at the center of each grid box.
!   <xl> & <xu> are the lower & upper grid box x coord at y=<yc>.
!   <yl> & <yu> are the lower & upper grid box y coord at x=<xc>.
!   <rlon> & <rlat> are longitude & latitude [deg] at planet surface
!     ( <rlon>, <rlat> usually assumed constant in horiz cartesian coord )

!%!      do iy = 1,NY
!%!	do ix = 1,NX
!%!          rlon(ix,iy) = -90.
!%!          rlat(ix,iy) = 45.
!%!	  do k = 1,NZ
!%!	    dx(ix,iy,k) = dxfix
!%!           xc(ix,iy,k) = dom_llx + ( ix - .5 ) * dx(ix,iy,k)
!%!            xl(ix,iy,k) = dom_llx + ( ix - 1 ) * dx(ix,iy,k)
!%!            xu(ix,iy,k) = dom_llx + ix * dx(ix,iy,k)
!%!	    dy(ix,iy,k) = dyfix
!%!            yc(ix,iy,k) = dom_lly + ( iy - .5 ) * dy(ix,iy,k)
!%!            yl(ix,iy,k) = dom_lly + ( iy - 1 ) * dy(ix,iy,k)
!%!            yu(ix,iy,k) = dom_lly + iy * dy(ix,iy,k)
!%!          enddo
!%!        enddo
!%!      enddo

!  Define horizontal metric factors that convert coord system diffs to actual distance
!   In cartesian coordinates, the coord are directly distance.

!%do ixyz=1,NXYZ
!%   xmet3(ixyz) = 1.
!%   ymet3(ixyz) = 1.
!%enddo

!  Define vertical sigma values at grid box boundaries.
!   These could be enumerated.
!   This demo uses a simple fractional power function.
!   sigma(k)=((k-1)/NZ)**expon, where 0 < expon < 1.
!   This makes sigma intervals near ground smaller than at top of atm.
!   Use expon closer to 0 to get more dramatically changing sigma layers.

!%!      expon = .2
!%!      factor = 1. / float(NZ)
!%!      do k=1,NZP1
!%!        zl2(1,k) = ( float(k-1) * factor )**expon
!%!        do ixy=2,NXY
!%!          zl2(ixy,k) = zl2(1,k)
!%!        enddo
!%!      enddo

!  Compute sigma values at vertical midpoints of grid boxes & vert box size

!%!      do k=1,NZ
!%!        zc2(1,k) = .5 * ( zl2(1,k) + zl2(1,k+1) )
!%!        dz2(1,k) = zl2(1,k+1) - zl2(1,k)
!%!        do ixy=2,NXY
!%!          zc2(ixy,k) = zc2(1,k)
!%!          dz2(ixy,k) = dz2(1,k)
!%!        enddo
!%!      enddo 

!  Define pressures at bottom and top of model domain [dyne/cm^2]

!%!      do iy=1,NY
!%!        do ix=1,NX
!%!          p_surf(ix,iy) = 6.2 * RMB2CGS
!%!          p_top(ix,iy) =  0.1 * RMB2CGS
!%!        enddo
!%!      enddo

!  Define vertical profile of atmospheric variables that are 3-D.
!   In this demo:
!    The vertical profiles do not vary horizontally.
!    A constant potential temperature <pt> is defined throughout the atm.
!    Air temp <t> is computed as a function of <p> & <pt>.
!    Air density <rhoa> is a function of <t> & <p>, from dry air equa of state.
!    All units are cgs, deg_K.

!%rkappa = R_AIR / CP
!%t_sfc = 170.
!%dlapse = -GRAV/CP

!%do iy=1,NY
!%   do ix=1,NX
!%      t_surf(ix,iy) = t_sfc
!%!
!%!          pt_fix = t_sfc * ( PREF / p_surf(ix,iy) )**rkappa
!%!          pstar = p_surf(ix,iy) - p_top(ix,iy)
!%!          do k=1,NZ
!%!            p(ix,iy,k) = p_top(ix,iy) + zc(ix,iy,k) * pstar
!%!            t(ix,iy,k) = pt_fix * ( p(ix,iy,k) / PREF )**rkappa
!%!            rhoa(ix,iy,k) = p(ix,iy,k) / ( R_AIR * t(ix,iy,k) )
!%!            ptc(ix,iy,k) = rhoa(ix,iy,k) * pt_fix
!%!
!%!          enddo      
!%   enddo
!%enddo

!  Define <zmet>, the  metric factor that makes <dz> in
!  arbitrary coord system a true distance.
!  See Toon et al., JAS, vol 45, #15, Aug-1988, p 2125.

!%!      do iy=1,NY
!%!        do ix=1,NX
!%!          pstar = p_surf(ix,iy) - p_top(ix,iy)
!%!          do k=1,NZ
!%!            zmet(ix,iy,k) = pstar / ( GRAV * rhoa(ix,iy,k) )
!%!          enddo
!%!        enddo
!%!      enddo

!   Compute constants used in viscosity

!%rmu_0 = 1.8325e-4
!%rmu_t0 = 296.16
!%rmu_c = 120.
!%rmu_const = rmu_0 * (rmu_t0 + rmu_c)

!  Define vertical profile of atmospheric variables that are 1-D.
!   In this demo:
!    Air viscosity <rmu> is from Sutherland's equation (using Smithsonian
!     Meteorological(g_normal) Tables, in which there is a misprint -- T is deg_K, not deg_C.
!    Thermal conductivity of dry air <thcond> is from Pruppacher and Klett, Eq. 13-16.

!%do k=1,NZ
!%   rmu(k) = rmu_const / ( t(1,1,k) + rmu_c ) *  &
!%        ( t(1,1,k) / rmu_t0 )**1.5d0
!%   thcond(k) = ( 5.69 + .017*( t(1,1,k) - T0 ) )*4.18e2
	
!   Mars CO2 viscocity

!%   rmu(k) = 5.65e-8*((t(1,1,k)**2.5)  &
!%        /t(1,1,k)+269.)
     
!%   rmu(k) = 1.59e-6*t(1,1,k)**1.5  &
!%        /(t(1,1,k)+244.4)*10.

!   Conductivity for CO2 atmosphere...

!%   thcond(k) = (0.0445*t(1,1,k)+5.5e5*t(1,1,k)**2-1.5)*4.18e-7

!%enddo

!  Define vertical wind speed & diffusion coefficients [in metric cgs units]

!%do k=1,NZP1
!%   do ix=1,NX
!%      do iy=1,NY
!%!            w(ix,iy,k) =  0.
!%         dkx(ix,iy,k) = 1.e-6
!%         dky(ix,iy,k) = 1.e-6
!%         dkz(ix,iy,k) = 1.e-6
!%      enddo
!%   enddo
!%enddo

!  Define horizontal wind speed [in metric cgs units]

!%do k=1,NZ
!%   do ix=1,NX
!%      do iy=1,NY
!%         u(ix,iy,k) = 0.
!%         v(ix,iy,k) = 0.
!%      enddo
!%   enddo
!%enddo

!  Return to caller with atm profiles initialized.

end subroutine g_sig


end module aemic_init_astruct_mod
