!```````````````````````````````````````````````````````````````````
module aemic_init_modcore_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine init_modcore

!+ Initializes some core model parameters (mapping arrays, etc.)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: inucmin,if_nuc_lh,rlh_nuc,ifall,vf_const,time_nuc  &
     ,period_nuc,ngroups,nbins,nelements,groupname,igelem,ienconc,do_coag  &
     ,coaglg,coagpe,nz_mic
use aemic_init_mod, only: setup_bins, setup_vfall, setup_grow1, setup_gkern1,  &
     setup_nuc

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ig,iefrom,ieto,ie,ib,k

!=== Executable statements:

! Sample setup of particle types and the corresponding mapping arrays:
!
!    CN: Dust
!    Cloud hydrometeors: CO2 ice, H2O ice
!    Core masses: dust mass in H2O ice, dust mass in CO2 ice,
!                 H2O ice mass in CO2 ice
!
!    group 1   Dust                               element 1   (#/cm^3)
!    group 2   H2O ice                            element 2   (#/cm^3)
!              mass of dust in cloud crystals     element 3   (g/cm^3)
!    group 3   CO2 ice                            element 4   (#/cm^3)
!              mass of dust in cloud crystals     element 5   (g/cm^3)
!              mass of H2O ice in cloud crystals  element 6   (g/cm^3)
!
!    ngasses = 2
!        gas(1) = H2O vapor
!        gas(2) = carbon dioxide
!
!    ngroups = 3
!    nelements = 6
!    nelemg  = (1,2,3)
!    itype   = (0,1,2,1,2,3)
!    ienconc = (1,2,4)
!    igelem  = (1,2,2,3,3,3)

! Evaluate derived bin mapping arrays and set up the particle size bins:

call setup_bins

! Set options for particle fall velocities:
!
!    <ifall> = 0: use constant fall velocity <vf_const>, but still calculate
!                 <vf> for use in Reynolds' number <re>
!            = 1: use calculated value of <vf>

ifall = 1

!    <vf_const> is only used when <ifall> = 0

vf_const = ZERO_f4

! Evaluate fall velocities:

call setup_vfall

! Set nucleation update time [s]:

time_nuc = ZERO_f4

! Set nucleation update time interval [s]:

period_nuc = 30._f4

! Inititalize index of smallest bin in each group from which a particle
!  has nucleated:

do ig=1,ngroups
   inucmin(ig) = nbins(ig)
enddo

! Evaluate time-independent parameters and derived mapping arrays
!  used for condensational growth and evaporation:

call setup_grow1
call setup_gkern1

! <if_nuc_lh(iefrom,ieto)> is true if nucleation from element <iefrom> to
!  element <ieto> releases latent heat.
!
! <rlh_nuc(iefrom,ieto)> is the latent heat released by nucleation
!  from element <iefrom> to element <ieto> [cm^2/s^2 OR erg/g]:

if_nuc_lh = .FALSE.    ! Initialize entire arrays
rlh_nuc = ZERO_f4           !
do iefrom = 1,nelements
   if(groupname(igelem(iefrom)) == 'D') then
      do ieto = 1,nelements
         ig = igelem(ieto)
         if(groupname(ig) == 'C') then
            if(ienconc(ig) == ieto) then
               if_nuc_lh(iefrom,ieto) = .TRUE.
               rlh_nuc(iefrom,ieto) = ZERO_f4   ! ???? rlhm(1,2)????
            endif
         endif
      enddo
   endif
enddo

! Evaluate time-independent parameters and derived mapping arrays
!  used for nucleation:

call setup_nuc

! Set options for coagulation kernel:

if(do_coag) then

   ! <icoagop> = 0: use fixed coagulation kernel <ck0>
   !           = 1: calculate the coagulation kernel

!%   icoagop = 1

   ! <icollec> determines gravitational collection efficiencies
   !     = 0: use constant value <grav_e_coll0>
   !     = 1: use binwise maxima of Fuchs' and Langmuir's efficiencies
   !     = 2: use input data

!%   icollec = 2

   ! Fixed kernel value <ck0> only used when <icoagop> = 0

!%   ck0 = 1.e-5_f4

   ! <grav_e_coll0> only used when <icollec> = 0

!%   grav_e_coll0 = ONE_f4

   ! This <NGROUP> by <NGROUP> array maps aerosol groups for coagulation
   !  <icoag(i,j)> is the particle group resulting from a collision
   !  between particles in groups <i> and <j>.
   !  [This array must be diagonal, so the user need only specify
   !  elements (i=1:NGROUP,j=1:i)]:

!%   icoag(1,1) = 1
!%   icoag(1,2) = 2
!%   icoag(2,2) = 2
!%   icoag(3,1) = 0
!%   icoag(3,2) = 4
!%   icoag(3,3) = 3
!%   icoag(4,1) = 0
!%   icoag(4,2) = 0
!%   icoag(4,3) = 0
!%   icoag(4,4) = 0

   ! Evaluate derived coagulation mapping arrays and kernels.

!%   call setupckern
!%   call setupcoag

else      ! If coagulation NOT turned on

   ! Set production terms and loss rates due to slow microphysical processes
   !  (coagulation) to zero [NOTE: if (do_coag=.TRUE.), then these arrays are
   !  zeroed in `pre_step` before every timestep]:

   do ie=1,nelements
      do ib=1,nbins(igelem(ie))
         do k=1,nz_mic
            coagpe(k,ib,ie) = ZERO_f4
         enddo
      enddo
   enddo
   
   do ig=1,ngroups
      do ib=1,nbins(ig)
         do k=1,nz_mic
            coaglg(k,ib,ig) = ZERO_f4
         enddo
      enddo
   enddo

endif

end subroutine init_modcore


end module aemic_init_modcore_mod
