!```````````````````````````````````````````````````````````````````
module aemic_partmphys_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine growevap_loss

!+ Evaluates particle loss rates due to condensational growth and evaporation
!   for all condensing gasses

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4, ONE_THIRD_f4, ONE_SIXTH_f4
use aermicro_mem_mod, only: ngroups,ienconc,igrowgas,pconmax,nbins,pc  &
     ,FEW_PC,nelemg,ik,SMALL_PC,itype,I_COREMASS,rmass,gwtmol,is_grp_ice  &
     ,akelvini,POWMAX,akelvin,rup,supsati,gro,isolelem,gro1,gro2,pvapi  &
     ,supsatl,pvapl,gasname,dm,rmassup,rmrat,dtime,growlg,evaplg,ONE,ncore  &
     ,dmi,ge_pcfitconst,time,i_curr,j_curr  &
     ,otherm => fscr1  &     ! [nelements]
     ,ieother => iscr1  &    ! [nelements]
     ,dela => fscr2  &     ! [nbins(ig)]
     ,delma => fscr3  &     ! [nbins(ig)]
     ,aju => fscr4  &     ! [nbins(ig)]
     ,ar => fscr5  &     ! [nbins(ig)]
     ,al => fscr6  &     ! [nbins(ig)]
     ,a6 => fscr7  &     ! [nbins(ig)]
     ,dmdt => fscr8     ! [nbins(ig)]

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! The loss rates for each group are <growlg> and <evaplg>.
!  Units are [s^-1].

integer :: ig,iepart,igas,ieoth_rel,ieoth_abs,ib  &
     ,nother,nb
!#integer :: isol,jother
real(f4) :: condm,argsol,othermtot,expon,akas,g0,g1,g2,ss,pvap,dpc  &
     ,dpc1,dpcm1,rat1,rat2,rat3,rat4,den1,test1,test2,com2,x

!=== Executable statements:

! Calculate particle fluxes due to growth and evaporation for all
!  particle groups:

do ig=1,ngroups

   iepart = ienconc(ig)     ! element of particle number concentration 
   igas = igrowgas(iepart)      ! condensing gas
         
   if(igas /= 0) then

      ! Treat condensation of gas <igas> to/from particle group <ig>:

      ! Update maximum particle concentration for each spatial grid box
      !  (in units of cm^-3):

      pconmax(ik,ig) = SMALL_PC
      do ib=1,nbins(ig)
         pconmax(ik,ig) = max(pconmax(ik,ig),pc(ik,ib,iepart))
      enddo
      pconmax(ik,ig) = pconmax(ik,ig)

      ! Bypass calculation if few particles are present:

      if(pconmax(ik,ig) > FEW_PC) then

         do ib=1,nbins(ig)-1

            ! Consider growth of average particle at radius <rup(ibin)>;

            ! Treat solute effect first: <asol> is solute factor.
            !  Only need to treat solute effect if <nelemg(igroup)> > 1:

            if(nelemg(ig) <= 1) then
               argsol = ZERO_f4
            else

               ! <condm> is mass concentration of condensed gas <igas> in
               !  particle. <nother> is number of other elements in group
               !  having mass.  <otherm> are mass concentrations of other
               !  elements in particle group. <othermtot> is total mass
               !  concentrations of other elements in particle:

               nother = 0
               othermtot = ZERO_f4

               ! <ieoth_rel> is relative element number of other element in
               !  group;

               do ieoth_rel=2,nelemg(ig)       

                  ! <ieoth_abs> is absolute element number of other element:

                  ieoth_abs = iepart+ieoth_rel-1    

                  if(itype(ieoth_abs) == I_COREMASS) then
                     nother = nother+1
                     ieother(nother) = ieoth_abs
                     otherm(nother) = pc(ik,ib,ieoth_abs)
                     othermtot = othermtot+otherm(nother)
                  endif

               enddo

               condm = rmass(ib,ig)*pc(ik,ib,iepart)-othermtot

!###### No solutes #############
!#               if(condm <= 0.) then
!#
!#                  ! Zero mass for the condensate; <argsol> is a small
!#                  !  value << 1:
!#
               argsol = 1.e-10_f4 
!#
!#               else
!#
!#                  ! Sum over masses of other elements in group for argument of
!#                  !  solute factor:
!#
!#                  argsol = 0.
!#                  do jother=1,nother
!#                     isol = isolelem(ieother(jother))
!#                     argsol = argsol  & 
!#                         +sol_ions(isol)*otherm(jother)/solwtmol(isol)
!#                  enddo
!#                  argsol = argsol*gwtmol(igas)/condm
!#
!#               endif

            endif

            ! <akas> is combined Kelvin (curvature) and solute factors:

            ! Ignore solute factor for ice particles:

            if(is_grp_ice(ig)) then
               expon = akelvini(ik,igas)/rup(ib,ig)
            else
               expon = akelvin(ik,igas)/rup(ib,ig)-argsol 
            endif
            expon = max(-POWMAX,expon)
            akas = exp(expon)

            ! Trick for removing haze droplets from droplet bins:
            !  allows haze droplets to exist under supersaturated conditions;
            !  when below supersaturation, haze droplets will evaporate:

            ! TIM (30.07.2007): not sure if the "supsat" term in the below
            !  conditional should be 'supsati' or 'supsatl' -- at the moment,
            !  'supsatl' seems most consistent with the statement's context.

            if((.NOT.is_grp_ice(ig)) .and. (akas < ONE_f4) .and.  &
                 (supsatl(ik,igas) < ZERO_f4)) then
               akas = ONE_f4
            endif

            ! <dmdt> is growth rate in mass space [g/s]:

            g0 = gro(ik,ib+1,ig)
            g1 = gro1(ik,ib+1,ig)
            g2 = gro2(ik,ig)
 
            if(supsati(ik,igas) > ZERO_f4) then
               g1 = g1/(ONE_f4+supsati(ik,igas))
            endif
	   
            if(is_grp_ice(ig)) then
               ss = supsati(ik,igas)
               pvap = pvapi(ik,igas)
            else
               ss = supsatl(ik,igas)
               pvap = pvapl(ik,igas)
            endif
        
            if(ss > 10._f4) then
               ss = 10._f4
            endif

!?          dmdt(ibin) = pvap * ( ss + 1. - akas - 
!?     $                 qrad3(ixyz,ibin+1,igroup) * g1 * g2 ) *
!?     $                 g0 / ( 1. + g0 * g1 * pvap )

            dmdt(ib) = pvap*(ss+ONE_f4-akas)*g0/(ONE_f4+g0*g1*pvap )
!@
!@            if(dmdt(ib) > 0.) then
!@               if(pc(ik,ib,iepart) <= FEW_PC) then
!@                  dmdt(ib) = 0.
!@               endif
!@            elseif(dmdt(ib) < 0.) then
!@               if(pc(ik,ib+1,iepart) <= FEW_PC) then
!@                 dmdt(ib) = 0.
!@               endif
!@            endif
!@          

!?            if(gasname(igas)(1:1) == 'W') then
!?               dmdt(ib) = dmdt(ib)*1.e-1
!?            elseif(gasname(igas)(1:1) == 'C') then
!?               dmdt(ib) = dmdt(ib)*5.e-2
!?            endif

         enddo

         ! Now calculate condensation/evaporation production and loss rates.
         !  Use Piecewise Polynomial Method [Colela and Woodard, J. Comp. Phys.,
         !  54, 174-201, 1984]:

         ! First, use cubic fits to estimate concentration values at bin
         !  boundaries:

         do ib=2,nbins(ig)-1

            dpc = pc(ik,ib,iepart)*dmi(ib,ig)
            dpc1 = pc(ik,ib+1,iepart)*dmi(ib+1,ig)
            dpcm1 = pc(ik,ib-1,iepart)*dmi(ib-1,ig)
!@            ratt1 = dm(ib,ig)/(dm(ib-1,ig)+dm(ib,ig)+dm(ib+1,ig))
            rat1 = ge_pcfitconst(ib,ig,1)
!@            ratt2 = (2.*dm(ib-1,ig)+dm(ib,ig))/(dm(ib+1,ig)+dm(ib,ig))
            rat2 = ge_pcfitconst(ib,ig,2)
!@            ratt3 = (2.*dm(ib+1,ig)+dm(ib,ig))/(dm(ib-1,ig)+dm(ib,ig))
            rat3 = ge_pcfitconst(ib,ig,3)
            dela(ib) = rat1*(rat2*(dpc1-dpc)+rat3*(dpc-dpcm1))
            delma(ib) = ZERO_f4

            if((dpc1-dpc)*(dpc-dpcm1) > ZERO_f4) then
               delma(ib) = min(abs(dela(ib)),2._f4*abs(dpc-dpc1),  &
                    2._f4*abs(dpc-dpcm1))*abs(dela(ib))/dela(ib)
            endif

         enddo

         do ib=2,nbins(ig)-2

            dpc = pc(ik,ib,iepart)*dmi(ib,ig)
            dpc1 = pc(ik,ib+1,iepart)*dmi(ib+1,ig)
            dpcm1 = pc(ik,ib-1,iepart)*dmi(ib-1,ig)

!@            rat1 = dm(ib,ig)/(dm(ib,ig)+dm(ib+1,ig))
            rat1 = ge_pcfitconst(ib,ig,4)
!@            rat2 = 2.*dm(ib+1,ig)*dm(ib,ig)/(dm(ib,ig)+dm(ib+1,ig))
            rat2 = ge_pcfitconst(ib,ig,5)
!@            rat3 = (dm(ib-1,ig)+dm(ib,ig))/(2.*dm(ib,ig)+dm(ib+1,ig))
            rat3 = ge_pcfitconst(ib,ig,6)
!@            rat4 = (dm(ib+2,ig)+dm(ib+1,ig))/( 2.*dm(ib+1,ig)+dm(ib,ig))
            rat4 = ge_pcfitconst(ib,ig,7)
!@            den1 = dm(ib-1,ig)+dm(ib,ig)+dm(ib+1,ig)+dm(ib+2,ig)
            den1 = ge_pcfitconst(ib,ig,8)

            ! <aju(ibin)> is the estimate for concentration (dn/dm) at bin
            !  boundary <ibin>+1/2:

            aju(ib) = dpc+rat1*(dpc1-dpc)+ONE/den1 *  &
                 ( rat2*(rat3-rat4)*(dpc1-dpc) -  &
                 dm(ib,ig)*rat3*delma(ib+1) +  &
                 dm(ib+1,ig)*rat4*delma(ib))
            
         enddo

         ! Now construct polynomial functions in each bin:

         do ib=3,nbins(ig)-2
            al(ib) = aju(ib-1)
            ar(ib) = aju(ib)
         enddo

         ! Use linear functions in first two and last two bins:

         if(nbins(ig) > 1) then
            ar(2) = aju(2)
            al(2) = pc(ik,1,iepart)*dmi(1,ig) +  &
                 (rmassup(1,ig)-rmass(1,ig))/(rmass(2,ig)-rmass(1,ig)) *  &
                 (pc(ik,2,iepart)*dmi(2,ig)-pc(ik,1,iepart)*dmi(1,ig))
            ar(1) = al(2)
            al(1) = pc(ik,1,iepart)*dmi(1,ig) +  &
                 (rmassup(1,ig)/rmrat(ig)-rmass(1,ig))/  &
                 (rmass(2,ig)-rmass(1,ig))*(pc(ik,2,iepart)*dmi(2,ig)-  &
                 pc(ik,1,iepart)*dmi(1,ig))

            nb = nbins(ig)

            al(nb-1) = aju(nb-2)
            ar(nb-1) = pc(ik,nb-1,iepart)*dmi(nb-1,ig) +  &
                 (rmassup(nb-1,ig)-rmass(nb-1,ig))/  &
                 (rmass(nb,ig)-rmass(nb-1,ig))*(pc(ik,nb,iepart)*dmi(nb,ig)-  &
                 pc(ik,nb-1,iepart)*dmi(nb-1,ig))
            al(nb) = ar(nb-1)
            ar(nb) = pc(ik,nb-1,iepart)*dmi(nb-1,ig) +  &
                 (rmassup(nb,ig)-rmass(nb-1,ig))/  &
                 (rmass(nb,ig)-rmass(nb-1,ig))*(pc(ik,nb,iepart)*dmi(nb,ig)-  &
                 pc(ik,nb-1,iepart)*dmi(nb-1,ig))
         endif

         ! Next, ensure that polynomial functions do not deviate beyond the
         !  range [<al(ibin)>,<ar(ibin)>]:

         do ib=1,nbins(ig)
            dpc = pc(ik,ib,iepart)*dmi(ib,ig)
            if((ar(ib)-dpc)*(dpc-al(ib)) <= ZERO_f4) then
               al(ib) = dpc
               ar(ib) = dpc
            endif
            test1 = (ar(ib)-al(ib))*(dpc-0.5_f4*(al(ib)+ar(ib)))
            test2 = ONE_SIXTH_f4*(ar(ib)-al(ib))**2
            if(test1 > test2) then
               al(ib) = 3._f4*dpc-2._f4*ar(ib)
            elseif(test1 < -test2) then
               ar(ib) = 3._f4*dpc-2._f4*al(ib)
            endif
         enddo

         ! Lastly, calculate fluxes across each bin boundary:

         ! Use upwind advection when courant number > 1:

         do ib=1,nbins(ig)
            
            dpc = pc(ik,ib,iepart)*dmi(ib,ig)
            dela(ib) = ar(ib)-al(ib)
            a6(ib) = 6._f4*(dpc-0.5_f4*(ar(ib)+al(ib)))

         enddo

         do ib=1,nbins(ig)-1

            com2 = (dm(ib,ig)+dm(ib+1,ig))*0.5_f4
            x = abs(dmdt(ib)*dtime*dmi(ib,ig))

            if(dmdt(ib) > ZERO_f4) then

               if(x < ONE_f4) then
                  growlg(ib,ig) = dmdt(ib)/pc(ik,ib,iepart)  &
                       *(ar(ib)-0.5_f4*dela(ib)*x +(x*0.5_f4-x**2*ONE_THIRD_f4)*a6(ib))
               else
                  growlg(ib,ig) = dmdt(ib)/com2          ! upwind advection
               endif
            elseif(dmdt(ib) < ZERO_f4) then
	
               if(x < ONE_f4) then
                  evaplg(ib+1,ig) = -dmdt(ib)/pc(ik,ib+1,iepart)  &
                       *(al(ib+1)+0.5_f4*dela(ib+1)*x +(x*0.5_f4-(x**2)*ONE_THIRD_f4)*  &
                       a6(ib+1))
               else
                  evaplg(ib+1,ig) = -dmdt(ib)/com2       ! upwind advection
               endif

!if(time < 0. .and. ik == 1) then
!   print *,'evaplg',ib+1,ig,evaplg(ib+1,ig),dmdt(ib),com2,pc(ik,ib+1,iepart),(al(ib+1)+0.5*dela(ib+1)*x +(x*0.5-(x**2)*ONE_THIRD_f4)*  &
!                       a6(ib+1)),pc(ik,ib+1,iepart)  &
!                       *(al(ib+1)+0.5*dela(ib+1)*x +(x*0.5-(x**2)*ONE_THIRD_f4)*  &
!                       a6(ib+1)),x
!endif

               ! Boundary conditions: for evaporation out of first bin (with
               !  cores), use evaporation rate from second bin:

               if(ib == 1 .and. ncore(ig) > 0) then
                  evaplg(1,ig) = -dmdt(1)*dmi(1,ig)
               endif

            endif

            ! Limit growth rates to reasonable values:
!if(growlg(ib,ig) > 100.) then
!   print *,'rtss',ib,ig,growlg(ib,ig),dmdt(ib),pc(ik,ib,iepart),ar(ib),x,(ar(ib)-0.5*dela(ib)*x +(x*0.5-x**2*ONE_THIRD_f4)*a6(ib)),ss,pvap,i_curr,j_curr
!print *,'sss',1,al(1),-9999.,pc(ik,1,iepart)*dmi(1,ig)
!do isol=2,nbins(ig)
!print *,'sss',isol,al(isol),ar(isol-1),pc(ik,isol,iepart)*dmi(isol,ig)
!!print *,'pp',ik,pc(ik,1:nbins(ig),iepart)
!enddo
!print *,'sss',nbins(ig),'a',-9999.,ar(nbins(ig)),pc(ik,nbins(ig),iepart)
!   pause
!endif
            if(gasname(igas)(1:1) == 'W') then   ! H2O vapor            
               growlg(ib,ig) = min(growlg(ib,ig),1.e2_f4)
               evaplg(ib+1,ig) = min(evaplg(ib+1,ig),1.e2_f4)
            elseif(gasname(igas)(1:1) == 'C') then   ! CO2 gas            
               growlg(ib,ig) = min(growlg(ib,ig),1.e8_f4)
               evaplg(ib+1,ig) = min(evaplg(ib+1,ig),1.e8_f4)
            endif
     
         enddo

      endif
   endif
!####
!####            evaplg(1:nbins(ig),ig) = ZERO_f4
!####
enddo

end subroutine growevap_loss


!```````````````````````````````````````````````````````````````````
subroutine nucl_loss_Mars

!+ Evaluates particle loss rates due to nucleation for all condensing gasses

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4, FOUR_THIRDS_f4
use aermicro_mem_mod, only: ngroups,ngasses,ienconc,inucgas,nnuc2elem,inuc2elem  &
     ,igelem,inucproc,I_DROPACT,nbins,inuc2bin,pconmax,ik,FEW_PC,evaplg  &
     ,supsati,gasname,gwtmol,surfctia,R_univ_cgs,t,r,pi_m,surfctiw,avog,pvapi  &
     ,vol,BK,rnuclg,sifreeze,time,pc,i_curr,j_curr,grid,rhoa,p

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Evaluates particle loss rates due to nucleation <rnuclg>.
!  
!  The loss rates for all particle elements in a particle group are equal.
!
!  To avoid nucleation into an evaporating bin, this subroutine must
!  be called after nucgrowp, which evaluates evaporation loss rates <evaplg>.

logical :: evapfrom_nucto
integer :: ig,iepart,igas,inuc,ienucto,ignucto,ib,inucto 
real(f4) :: ag,ct,fx,fphi,fm1,fm1a,fm1b,fm2,fm3,fm4,fm,dfg,dfdes,dfsd  &
     ,vs,wmol,con1,c1,gk,zs,con2,pfact,energies,expon,fm2a,fm2b

!=== Executable statements:

do ig=1,ngroups       ! Loop over particle groups

   iepart = ienconc(ig)            ! particle number density element

   ! Calculate nucleation loss rates.  Do not allow nucleation into
   !  an evaporating bin.
   !
   !  <ienucto> is index of target nucleation element;
   !  <ignucto> is index of target nucleation group.

   do inuc=1,nnuc2elem(iepart)   ! Loop will not execute if current group
                                 !  does not nucleate (nnuc2elem(iepart)=0)

      ienucto = inuc2elem(inuc,iepart)
      igas = inucgas(inuc,ig)   ! condensing gas for iepart -> ienucto transfer
      ignucto = igelem(ienucto)

!@if(ik == 29 .and. time < 0.) then
!@   print *,'eeeee1',ig,ignucto,igas,iepart,ienucto
!@endif

      ! Only compute nucleation rate if <inucproc(iepart,ienucto)>= I_DROPACT:

      if(inucproc(iepart,ienucto) == I_DROPACT) then

         ! Loop over particle bins.  Loop from largest to smallest for 
         !  evaluation of index of smallest bin nucleated during timestep
         !  <inucstep>:

         do ib=nbins(ig),1,-1

            ! <inucto> is index of target nucleation bin:

            inucto = inuc2bin(ib,ig,ignucto)

!@if(ik == 29 .and. time < 0.) then
!@   print *,'eeeee2',ib,inucto
!@endif

            ! Bypass calculation if few particles are present:
                  
!@            if(pconmax(ik,ig) > FEW_PC) then  ! Only check group max conc
            if(pc(ik,ib,iepart) > FEW_PC) then   ! Check each concentration

               ! Set <evapfrom_nucto> to .TRUE. when target droplets are
               !  evaporating:
  	    
               evapfrom_nucto = (evaplg(inucto,ignucto) > ZERO_f4)
               
               ! Only calculate nucleation rate if supersaturation is near
               !  the critical supersaturation <sifreeze> [s = 0.35 for
               !  CO2<???> and s = 0.2 for H2O<???>]:

               if((supsati(ik,igas) > sifreeze(igas)) .and.  &
                    .NOT.evapfrom_nucto) then

                  if(gasname(igas)(1:1) == 'W') then   ! H2O vapor

                     ! Critical ice germ radius used in nucleation calc:
                           
                     ag = 2._f4*gwtmol(igas)*surfctia(ik,igas)/  &
                          (R_univ_cgs*t(ik)*log(supsati(ik,igas)+ONE_f4))
     
                     if(ag < 1.e-12_f4) then
                        ag = 1.e-12_f4
                     endif
                           
                     ct = 0.95_f4               ! contact parameter
                     fx = r(ib,ig)/ag   ! f-factor for spherical surface
      
                     fPhi = sqrt(ONE_f4-2._f4*ct*fx+fx*fx)
                     fm1 = ONE_f4+((ONE_f4-ct*fx)/fPhi)**3
                     fm1a = fx*fx*fx*2._f4
                     
                     fm1b = fx*fx*fx*(-3._f4*(fx-ct)/fPhi+((fx-ct)/fPhi)**3)
                     fm2 = fm1a+fm1b
                     fm3 = 3._f4*ct*fx*fx*((fx-ct)/fPhi-ONE_f4)
                     fm4 = fm1+fm2+fm3
                     fm = fm4*0.5_f4

!?	fm=(2.+ct)*(1.-ct**2)*.25    !   ..f-factor for flat surface:
!?     $            *(r(ibin,igroup)/r(NBIN,igroup))**0.5

                     ! This formulation is based on Eq 9-24 in Pruppacher
                     !  & Klett
                     !
                     !  These are the exponent energies:
                     !
                     !    dFg   = Energy of germ formation
                     !    dFdes = Desorption energy
                     !    dFsd  = Surface diffusion energy
                     
                     dFg = (16._f4*pi_m*(gwtmol(igas)**2)*  &
                          ((surfctiw(ik,igas))**3)/  &
                          (3._f4*(R_univ_cgs*t(ik)*  &
                          log(supsati(ik,igas)+ONE_f4))**2))

                     dFdes = 3.25e-21_f4*1.e7_f4

                     dFsd = dFdes*0.1_f4

                     ! These are the various terms in the prefactor:
                     !
                     !    vs = vibrational freq of the adsorbed molecule
                     !    Wmol = mass of a H2O molecule
                     !    C1 = monomer concentration
                     !    gk = number of molecules in germ cluster
                     !    Zs = Zeldovich factor
                     
                     vs = 4.9e12_f4
                     Wmol = gwtmol(igas)/avog
                     con1 = pvapi(ik,igas)/sqrt(2._f4*pi_m*Wmol*BK*t(ik))
                     C1 = con1/vs
                     gk = FOUR_THIRDS_f4*pi_m*ag**3/vol(ib,ig)
                     Zs = sqrt(dFg/(3._f4*pi_m*BK*t(ik)*gk**2))
                     con2 = con1*C1*Zs
                           
                     if(con2 > 1.e38_f4) then
                        con2 = 1.e38_f4
                     endif

                     ! And here is the prefactor <Pfact>:
                     
                     Pfact = 4._f4*(pi_m**2)*(r(ib,ig)**2)*(ag**2)*con2

                     ! And finally here is the Nucleation rate <rnuclg>:

                     energies = (2._f4*dFdes-dFsd-dFg*fm)
                     expon = energies/(BK*t(ik))
                     if(expon > 300._f4) then
                        expon = 300._f4
                     elseif(expon < -300._f4) then
                        expon = -300._f4
                     endif

                     rnuclg(ib,ig,ignucto) = Pfact*exp(expon)
                     
                     if(rnuclg(ib,ig,ignucto) > 1.e2_f4) then
                        rnuclg(ib,ig,ignucto) = 1.e2_f4
                     endif

                  elseif(gasname(igas)(1:1) == 'C') then   ! CO2 gas
	
                     ! Critical ice germ radius used in nucleation calc:
                           
                     ag = 2._f4*gwtmol(igas)*surfctiw(ik,igas)/  &
                          (R_univ_cgs*t(ik)*log(supsati(ik,igas)+ONE_f4))
     
                     if(ag < 1.e-12_f4) then
                        ag = 1.e-12_f4
                     endif

                     ct = 0.95_f4         ! contact parameter:
                     fx = r(ib,ig)/ag   ! f-factor for spherical surface
       
                     fPhi = sqrt(ONE_f4-2._f4*ct*fx+fx*fx)
                     fm1 = ONE_f4+((ONE_f4-ct*fx)/fPhi)**3
                     fm2a = fx*fx*fx*2._f4
                     fm2b = fx*fx*fx*(-3._f4*(fx-ct)/fPhi+((fx-ct)/fPhi)**3)
                     fm2 = fm2a+fm2b
                     fm3 = 3._f4*ct*fx*fx*((fx-ct)/fPhi-ONE_f4)
                     fm4 = fm1+fm2+fm3
                     fm = fm4*0.5_f4

!?   ..f-factor for flat surface:
!?
!?	fm=(2.+ct)*(1.-ct**2)*0.25
!?     $          *(r(ibin,igroup)/r(NBIN,igroup))**0.5

                     ! This formulation is based on Eq 9-24 in Pruppacher
                     !  & Klett
                     !
                     !  These are the exponent energies:
                     !
                     !    dFg   = Energy of germ formation
                     !    dFdes = Desorption energy
                     !    dFsd  = Surface diffusion energy

                     dFg = (16._f4*pi_m*(gwtmol(igas)**2)*  &
                          ((surfctiw(ik,igas))**3)/  &
                          (3._f4*(R_univ_cgs*t(ik)*  &
                          log(supsati(ik,igas)+ONE_f4))**2))*ONE_f4
     
                     dFg = abs(dFg)
                     dFdes = 3.25e-20_f4*1.e7_f4
                     dFsd = dFdes*0.1_f4
                           
                     ! These are the various terms in the prefactor:
                     !
                     !    vs = vibrational freq of the adsorbed molecule
                     !    Wmol = mass of a CO2 molecule
                     !    C1 = monomer concentration
                     !    gk = number of molecules in germ cluster
                     !    Zs = Zeldovich factor

                     vs = 2.9e12_f4
                     Wmol = gwtmol(igas)/avog
                     con1 = pvapi(ik,igas)/sqrt(2._f4*pi_m*Wmol*BK*t(ik))
                     C1 = con1/vs
                     gk = FOUR_THIRDS_f4*pi_m*ag**3/vol(ib,ig)
                     Zs = sqrt(dFg/(3._f4*pi_m*BK*t(ik)*gk**2))
                     con2 = con1*C1*Zs
                     if(con2 > 1.e38_f4) then
                        con2 = 1.e38_f4
                     endif

                     ! And here is the prefactor <Pfact>:

                     Pfact = 4._f4*(pi_m**2)*(r(ib,ig)**2)*(ag**2)*con2

                     ! And finally here is the nucleation rate <rnuclg>:

                     energies = (2._f4*dFdes-dFsd-dFg*fm)
                     expon = energies/(BK*t(ik))
                     if(expon > 300._f4) then
                        expon = 300._f4
                     elseif(expon < -300._f4) then
                        expon = -300._f4
                     endif

                     rnuclg(ib,ig,ignucto) = Pfact*exp(expon)

                     if(rnuclg(ib,ig,ignucto) > 1.e6_f4) then
                        rnuclg(ib,ig,ignucto) = 1.e6_f4
                     endif
       
                  endif
	
               endif
            endif
         enddo
      endif

   enddo
enddo

end subroutine nucl_loss_Mars


!```````````````````````````````````````````````````````````````````
subroutine growpe_upegtx_psolve

!+ Calculates particle production terms due to growth, "up-element-gradient"
!   transfer (????) processes; then solve for new particle concentrations

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use aermicro_mem_mod, only: pc,FEW_PC,ik,growlg,igelem,ienconc  &
     ,pconmax,growpe,igrowgas,nelements,nbins,rhoa,nnucelem,itype  &
     ,I_INVOLATILE,I_VOLATILE,nnucbin,inucbin,rnuclg,ncore,rmass,cp_cgs  &
     ,evappe,ngroups,evaplg,dtime,SMALL_PC,rnucpe,inucproc,I_ICEMELT  &
     ,I_DROPFREEZE,if_nuc_lh,rlheat,rlh_nuc,inucelem,icorelem,small_coremconc  &
     ,time

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ig,iepart,ib,ie,jefrom,igfrom,ifrom,jfrom,ipow,igto  &
     ,iefrom,ic,iecore
real(f4) :: rhoa_cgs,c1,elemass,totmass,rmasscore,ppd,pls,rnuclgtot  &
     ,fracmass,rnucprod

!=== Executable statements:

rhoa_cgs = rhoa(ik)   ! Calculate atmospheric density in cgs units:
c1 = cp_cgs*rhoa_cgs    ! Constant used in latent heat calc below

do ie=1,nelements

   ! Define group and particle # concentration indices for current element:

   ig = igelem(ie)      ! target particle group 
   iepart = ienconc(ig)         ! target particle number concentration element

   do ib=1,nbins(ig)

!==============================
! Calculate production terms due to condensational growth <growpe>
!  only if group to which element belongs grows:

      if(igrowgas(iepart) /= 0 .and. ib /= 1) then
      
         ! Bypass calculation if few droplets are present: 
         
         if(pconmax(ik,ig) > FEW_PC) then
            growpe(ib,ie) = pc(ik,ib-1,ie)*growlg(ib-1,ig) 
         endif

      endif

!==============================
! Calculate particle source terms <rnucpe> due to element transfer processes
!  for which the target element number is greater than the source element
!  number (otherwise, the source terms are calculated in `downeg_xfer`):

      ! Calculate production terms due to nucleation <rnucpe>:

      do jefrom=1,nnucelem(ie)  ! Loop over elements that nucleate to <ie>

         iefrom = inucelem(jefrom,ie)    ! source particle element

         ! Only calculate production rates here if <ielem> is greater than
         !  <iefrom>.  Otherwise, production is calculated in nucevapp.f:

         if(ie > iefrom) then

            igfrom = igelem(iefrom)            ! source particle group

            ! <ipow> is the power to which the source particle mass must be
            !  taken to match the type of the target element:

            if(itype(iefrom) == I_INVOLATILE .or.  &
                 itype(iefrom) == I_VOLATILE) then
               ipow = itype(ie)-1
            else
               ipow = itype(ie)-itype(iefrom) 
            endif

            ! Loop over bins that nucleate to bin <ib>:

            do jfrom=1,nnucbin(igfrom,ib,ig)

               ifrom = inucbin(jfrom,igfrom,ib,ig)    ! bin of source

               ! Bypass calculation if few source particles are present: 

               if(pconmax(ik,igfrom) > FEW_PC) then

                  if(rnuclg(ifrom,igfrom,ig) > ZERO_f4) then

                     ! First calculate mass associated with the source element
                     !  <elemass> (this is <rmass> for all source elements
                     !  except particle number concentration in a
                     !  multicomponent particle group):

                     if(ncore(igfrom) == 0 .or.  &
                          itype(iefrom) > I_VOLATILE) then
                        elemass = rmass(ifrom,igfrom)
                     else
                        totmass = pc(ik,ifrom,iefrom)*rmass(ifrom,igfrom)
                        rmasscore = pc(ik,ifrom,icorelem(1,igfrom))
                        do ic=2,ncore(igfrom)
                           iecore = icorelem(ic,igfrom)
                           rmasscore = rmasscore+pc(ik,ifrom,iecore)
                        enddo
                        fracmass = ONE_f4-rmasscore/totmass
                        elemass  = fracmass*rmass(ifrom,igfrom)
                     endif

                     rnucprod = rnuclg(ifrom,igfrom,ig)*pc(ik,ifrom,iefrom)*  &
                          elemass**ipow

                     rnucpe(ib,ie) = rnucpe(ib,ie)+rnucprod

                     ! dbg: For nucleation of mixed phase particles from ice
                     !  crystals, we need to make sure the ice core mass
                     !  fraction of the nucleated particle is nearly 1:

                     if(inucproc(iefrom,ie) == I_ICEMELT .and.  &
                          itype(ie) == I_VOLATILE) then
                        rnucpe(ib,ie+1) = rnucpe(ib,ie+1)+rnucprod*  &
                             rmass(ib,ig)*0.99_f4
                     endif
                     
                     ! dbg: For nucleation of mixed phase particles due to
                     !  droplet freezing, we need to make sure the ice core
                     !  mass fraction of the nucleated particle is very small:
                     
                     if(inucproc(iefrom,ie) == I_DROPFREEZE .and.  &
                          itype(ie) == I_VOLATILE) then
                        rnucpe(ib,ie+1) = rnucpe(ib,ie+1)+rnucprod*  &
                             rmass(ib,ig)*1.e-4_f4
                     endif
                     
                     ! Calculate latent heat associated with nucleation to
                     !  <ibin,ielem> from <ifrom,iefrom>:

                     if(if_nuc_lh(iefrom,ie)) then
                        rlheat = rlheat+rnucprod*rlh_nuc(iefrom,ie)/  &
                             c1*elemass
                     endif

                  endif
               endif
            enddo
            
         endif
      enddo

!==============================
! Calculate new particle concentrations.
!  The basic form from which the solution is derived is:
!  ( new_value - old_value ) / dtime = source_term - loss_rate*new_value

!@if(time < 0. .and. ik == 29) then
!@   print *,'rnucpe,growpe,evappe',ik,ib,ie,rnucpe(ib,ie),growpe(ib,ie),evappe(ib,ie)
!@endif

      ! Compute total production rate:

      ppd = rnucpe(ib,ie)+growpe(ib,ie)+evappe(ib,ie)  

      ! Sum up nucleation loss rates:

      rnuclgtot = ZERO_f4
      do igto=1,ngroups
         rnuclgtot = rnuclgtot+rnuclg(ib,ig,igto)
      enddo

      ! Compute total loss rate:
      
      pls = rnuclgtot+growlg(ib,ig)+evaplg(ib,ig)    

      ! Update net particle number concentration during current timestep
      !  due to production and loss rates:

      pc(ik,ib,ie) = (pc(ik,ib,ie)+dtime*ppd)/(ONE_f4+pls*dtime)

      ! Prevent particle concentrations from going to zero:

!if(pc(ik,ib,ie) < 0.) then
!   print *,'0negpc&&&&&&',ik,ib,ie,pc(ik,ib,ie)
!endif

      if(ie == iepart) then

         pc(ik,ib,ie) = max(pc(ik,ib,ie),SMALL_PC)

      else

         if(pc(ik,ib,iepart) <= SMALL_PC .or.  &
              pc(ik,ib,ie) < small_coremconc(ib,ie)) then
            pc(ik,ib,ie) = small_coremconc(ib,ie)
         endif

      endif

!==============================

   enddo
enddo

end subroutine growpe_upegtx_psolve


!```````````````````````````````````````````````````````````````````
subroutine evappart_production

!+ Calculates particle production rates due to evaporation

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use univ_numtype_constants_mod, only: epsilon_f4
use aermicro_mem_mod, only: pc,ALMOST_ZERO,nelements,igelem,nbins,evappe,ngroups  &
     ,ienconc,icorelem,I_VOLCORE,ievp2elem,ik,evaplg,ncore,rmass,rmrat  &
     ,imomelem,rmassmin,inucmin,dtime,igrowgas,itype,totevap,dm,cmf  &
     ,sec_mom,diffmass,POWMAX,nelemg,FEW_PC,time  &
     ,prob => fscr1  &     ! [nbins(ig)]
     ,evcore => fscr2        ! [nelements]

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Calculates particle source terms due to evaporation <evappe>.
!
!  Distinct evaporation of cores has not been treated.
!
!  When the core second moment is used, it is assumed that the probability
!  distribution function of core mass is a log-normal in mass space muliplied
!  by mass raised to the -3/2 power (which implies that the average core
!  mass from the probability distribution function is the same as the
!  average core mass).

! <SIG_MONO> is criterion for a monodisperse core mass distribution:
real(f4), parameter :: SIG_MONO = ALMOST_ZERO

! <CMPDF_EXP> is exponent of core mass in the probability distribution function:
real(f4), parameter :: CMPDF_EXP = -1.5

logical :: evap_mass,evap_mom,evap_mono,off_cn_grid,core_too_small,core_too_big
integer :: ie,ig,ib,iepart,iecor1,iecore,ic,iavg,ie2cn,ito,kount_s  &
     ,kount_l,ieto,igto,iesub,ielem
real(f4) :: coretot,smf,created,factor,depleted,fracmass,expon,evdrop  &
     ,coreavg,coremom,coresig,rn_norms,rn_norml,rm_norms,rm_norml,rmassto  &
     ,dmto,weightl,weights

!=== Executable statements:

! Set evaporation production rates to zero to avoid double-application
!  of rates calculated in growp.f:

do ie=1,nelements
   do ib=1,nbins(igelem(ie))
      evappe(ib,ie) = ZERO_f4
   enddo
enddo

! Calculate source terms due to evaporation:

! <ig> is source group (from which evaporation is being treated):

do ig=1,ngroups
 
   iepart = ienconc(ig)           ! source number concentration element
   iecor1 = icorelem(1,ig)        ! element of first core mass in group

   if(iecor1 /= 0 .and. igrowgas(iepart) /= 0) then
      if(itype(iecor1) /= I_VOLCORE) then
       
         ieto = ievp2elem(iecor1)      ! target number concentration element
                                      ! for total evaporation
         igto = igelem(ieto)           ! target group for total evaporation
 
         do ib=1,nbins(ig)                ! source bin for evaporation

            ! First calculate temporary evaporation source for droplets <evdrop>
            !  in bin <ib-1> assuming no total evaporation:

            evdrop = pc(ik,ib,iepart)*evaplg(ib,ig)

!@if(evdrop < 0.) then 
!@   print *,'qqqn',coretot,pc(ik,ib,iepart),pc(ik,ib,iecor1),rmass(ib,ig),ib,ig,coretot/pc(ik,ib,iepart),ik,evdrop,evaplg(ib,ig)
!@   pause
!@endif

            if(evdrop > ZERO_f4) then

               totevap(ib,ig) = .FALSE.

               ! Calculate total and average particle core mass
               !  <coretot,coreavg> and core mass fraction <cmf> by summing over
               !  all cores in group:

               coretot = pc(ik,ib,iecor1)
!$$$$$q               do ic=1,ncore(ig)
               do ic=2,ncore(ig)
                  iecore = icorelem(ic,ig)
                  coretot = coretot+pc(ik,ib,iecore)
               enddo
               coreavg = coretot/pc(ik,ib,iepart)
               coreavg = min(rmass(ib,ig),coreavg)
               cmf(ib,ig) = coreavg/rmass(ib,ig)

               ! Calculate <evcore>, the amount of the source term by number
               !  <evdrop> associated with the secondary core <ic>:

               do ic=1,ncore(ig)
                  iecore = icorelem(ic,ig)
                  evcore(ic) = evdrop*pc(ik,ib,iecore)/coretot  
               enddo

               ! Want total evaporation when droplets will be created with core
               !  mass fraction > 1 OR evaporating droplets are in bin 1:
               
               evap_mass = ((rmrat(ig)*cmf(ib,ig) >= ONE_f4) .or. (ib == 1))

               ! Test whether or not average core mass (which is totally
               !  evaporating) falls within target CN grid:

               core_too_small = (coreavg < rmass(1,igto))
               core_too_big = (coreavg >= rmass(nbins(igto),igto))
               off_cn_grid = (core_too_small .or. core_too_big)

               ! If evaporating core falls within target CN grid and core
               !  second moment is used, calculate average core second moment 
               !  <coremom>, second moment fraction <smf>, square of the
               !  logarithm of the geometric standard deviation of the assumed
               !  core mass distribution <coresig>, and index of the target
               !  CN bin of the average core mass <iavg>:

               if(sec_mom(ig) .and. .NOT.off_cn_grid) then

                  coremom = pc(ik,ib,imomelem(ig))/pc(ik,ib,iepart)
  
                  iavg = log(coreavg/rmassmin(igto))/log(rmrat(igto))+2
                  iavg = min(iavg,nbins(igto))
       
                  smf = coremom/rmass(ib,ig)**2
                  coresig = log(smf/cmf(ib,ig)**2)
                  coresig = max(SIG_MONO,coresig)

                  ! Want monodisperse total evaporation when [droplets will be
                  !  created with core second moment fraction > 1 OR core mass
                  !  fraction > 1 ] AND [ [ core mass distribution is
                  !  monodisperse ] OR [ core is smaller than the smallest CN
                  !  recently nucleated ] ]:

                  evap_mom = (rmrat(ig)**2*smf >= ONE_f4)

                  evap_mono = (evap_mass .or. evap_mom) .and.  &
                       ((coresig <= SIG_MONO) .or. (iavg <= inucmin(igto)))

               else

                  ! Either there is no core second moment or the average core
                  !  mass does not fall within target CN grid: monodisperse core
                  !  distribution is implied:

                  evap_mom = .FALSE. 
                  evap_mono = evap_mass

               endif

               ! Treat total evaporation from a monodisperse core mass 
               !  distribution:

               if(evap_mono) then

                  totevap(ib,ig) = .TRUE.

                  if(core_too_big) then

                     ! Put all of core mass into largest CN bin and print
                     !  warning (conserves mass, number increases):

                     factor = coreavg/rmass(nbins(igto),igto)

                     created = (evdrop*(factor-ONE_f4)*dtime)

!^                     if(created > FEW_PC) then
!^                        print *,  &
!^                          'WARNING: total evap core mass > rmass(nbins(ig)) --'
!^                        print *,  &
!^                         '   mass will be conserved, but number will increase:'
!^                        print *,  &
!^                        '    # created in bin #',nbins(igto),':',created
!^                        print *,'    ',ik,ib,ig,igto
!                        pause
!^                     endif

                     ! First the CN number concentration element:
 
                     evappe(nbins(igto),ieto) = evappe(nbins(igto),ieto)+  &
                          factor*evdrop

                     ! Now the CN cores:

                     do ic=2,ncore(ig)
                        iecore = icorelem(ic,ig)
                        ie2cn = ievp2elem(iecore)

                        evappe(nbins(igto),ie2cn) = evappe(nbins(igto),ie2cn)+ &
                             factor*evcore(ic)*rmass(nbins(igto),igto)
                     enddo

                  elseif(core_too_small) then

                     ! Put all of core mass into smallest CN bin and print
                     !  warning (conserves mass, number decreases):

                     factor = coreavg/rmass(1,igto)

                     depleted = (evdrop*(ONE_f4-factor)*dtime)

!^                     if(depleted > FEW_PC) then
!^                        print *,  &
!^                          'WARNING: total evap core mass > rmass(1) --'
!^                        print *,  &
!^                         '   mass will be conserved, but number will decrease:'
!^                        print *,  &
!^                       '    # depleted in bin #1:',depleted
!^                        print *,'    ',ik,ib,ig,igto,coreavg,rmass(1,igto)
!                        pause
!^                     endif

                     ! First the CN number concentration element:
 
                     evappe(1,ieto) = evappe(1,ieto)+factor*evdrop

                     ! Now the CN cores:

                     do ic=2,ncore(ig)
                        iecore = icorelem(ic,ig)
                        ie2cn = ievp2elem(iecore)

                        evappe(1,ie2cn) = evappe(1,ie2cn)+  &
                             factor*evcore(ic)*rmass(1,igto)
                     enddo
 
                  else

                     ! Partition core mass between two CN bins, conserving
                     !  total core mass and number.  The number will be
                     !  subdivided into bins <ito> and <ito>-1:

                     if(sec_mom(ig)) then
                        ito = iavg
                     else
                        ito = log(coreavg/rmassmin(igto))/log(rmrat(igto))+2
                        ito = min(ito,nbins(igto))
                     endif

                     fracmass = (rmass(ito,igto)-coreavg)/  &
                          diffmass(ito,igto,ito-1,igto)

!@                     print *,'ddd',ig,igto,ib
!@                     print *,'   a',log(coreavg/rmassmin(igto))  &
!@                          ,log(rmrat(igto))  &
!@                          ,log(coreavg/rmassmin(igto))/log(rmrat(igto))
!@                     print *,'   b',ito-1,fracmass
!@                     print *,'   c',ito,ONE-fracmass


                     ! Partition the source to the CN number concentration
                     !  element:
 
                     evappe(ito-1,ieto) = evappe(ito-1,ieto)+evdrop*fracmass
                     evappe(ito,ieto) = evappe(ito,ieto)+  &
                          evdrop*(ONE_f4-fracmass)

                     ! Partition mass from secondary cores to CN so that their
                     !  core mass fraction as target CN preserves the fraction
                     !  of core mass in the source droplet:

                     do ic=2,ncore(ig)
                        iecore = icorelem(ic,ig)
                        ie2cn = ievp2elem(iecore)

                        ! <evcore> is the fraction of the source term by number
                        !  <evdrop> that is from the secondary core <ic>:

!?                        evcore = evdrop * pc3(ixyz,i,iecore) / coretot  

                        evappe(ito-1,ie2cn) = evappe(ito-1,ie2cn)+  &
                             rmass(ito-1,igto)*evcore(ic)*fracmass
                        evappe(ito,ie2cn) = evappe(ito,ie2cn)+  &
                             rmass(ito,igto)*evcore(ic)*(ONE_f4-fracmass)

                     enddo
 
                  endif

               elseif(evap_mass .or. evap_mom) then
!                  print *,'sfsssffff'
!pause
                  ! Treat total evaporation from a polydisperse core mass
                  !  distribution:
                  !  assume a log-normal CN size distribution and conserve
                  !  number and mass as described by Turco (NASA Technical Paper
                  !  1362).  Don't put anything in CN bins with index less than
                  !  <inucmin>:

                  totevap(ib,ig) = .TRUE.

                  ! Calculate number <rn_norms,rn_norml> and mass
                  !  <rm_norms,rm_norml> normalization factors for cores smaller
                  !  and larger than <rmass(m,igto)>:

                  rn_norms = ZERO_f4
                  rn_norml = ZERO_f4
                  rm_norms = ZERO_f4
                  rm_norml = ZERO_f4
                  kount_s = 0
                  kount_l = 0

                  do ito=inucmin(igto),nbins(igto)

                     rmassto = rmass(ito,igto)           
                     dmto = dm(ito,igto)

                     ! <prob> is probability that core mass is in CN bin <ito>:

                     expon = -log(rmassto/coreavg)**2/(2._f4*coresig)
                     expon = max(-POWMAX,expon)
                     prob(ito) = rmassto**CMPDF_EXP*exp(expon)

                     if(ito <= iavg) then
                        rn_norms = rn_norms+prob(ito)*dmto
                        rm_norms = rm_norms+prob(ito)*dmto*rmassto
                        kount_s = kount_s+1
                     else
                        rn_norml = rn_norml+prob(ito)*dmto
                        rm_norml = rm_norml+prob(ito)*dmto*rmassto
                        kount_l = kount_l+1
                     endif

                  enddo

                  ! Calculate mass weighting factors <weights,weightl> for small
                  !  and large cores:

                  if(kount_s == 0) then
                     rm_norml = rm_norml/rn_norml
                     weightl = ONE_f4
                  elseif(kount_l == 0) then
                     rm_norms = rm_norms/rn_norms
                     weightl = ZERO_f4
                  else
                     rm_norms = rm_norms/rn_norms
                     rm_norml = rm_norml/rn_norml
                     weightl = (coreavg-rm_norms)/(rm_norml-rm_norms)
                     if (abs(weightl) < epsilon_f4) then
                        weightl = ZERO_f4
                     elseif((ONE_f4-weightl) <= ZERO_f4) then
                        weightl = ONE_f4
                     endif
                  endif
                  weights = ONE_f4-weightl

                  ! Renormalize probability distribution function and evaluate
                  !  the CN evaporation source term <evappe>:

                  do ito=inucmin(igto),nbins(igto)

                     if(ito <= iavg) then
                        prob(ito) = prob(ito)*weights/rn_norms
                     else
                        prob(ito) = prob(ito)*weightl/rn_norml
                     endif

                     ! First the CN number concentration element:

                     evappe(ito,ieto) = evappe(ito,ieto)+  &
                          evdrop*prob(ito)*dm(ito,igto)
                     
                     ! Now the CN core elements:
                     
                     do ic=2,ncore(ig)
                        iecore = icorelem(ic,ig)
                        ie2cn = ievp2elem(iecore)

                        evappe(ito,ie2cn) = evappe(ito,ie2cn)+  &
                             rmass(ito,igto)*evcore(ic)*prob(ito)*dm(ito,igto)
                     enddo

                  enddo

                  ! Evaluate evaporation source term <evappe> for droplet number
                  !  concentration, core mass, and core second moment:

               else

                  evappe(ib-1,iepart) = evdrop

                  do iesub=2,nelemg(ig)
                     ielem = iepart+iesub-1

                     evappe(ib-1,ielem) = evappe(ib-1,ielem)+  &
                          pc(ik,ib,ielem)*evaplg(ib,ig)

                  enddo
!if(time < 0. .and. ik == 1) then
!   print *,'hhguu',evappe(ib-1,iepart),ib,iepart,evdrop,pc(ik,ib-1,iepart),pc(ik,ib,iepart),evaplg(ib,ig)

!endif
               endif

            endif
         enddo
      
      endif

   elseif(igrowgas(iepart) /= 0) then    ! ieto == 0

      ! Treat evaporation of droplets that have no cores (for which there
      !  is no evaporation out of bin 1):

      do ib=2,nbins(ig)
         if(evaplg(ib,ig) > ZERO_f4) then

            evappe(ib-1,iepart) = evappe(ib-1,iepart)+  &
                 pc(ik,ib,iepart)*evaplg(ib,ig)

         endif
      enddo

   endif

enddo

end subroutine evappart_production


!```````````````````````````````````````````````````````````````````
subroutine downeg_xfer

!+ Calculates particle production terms due to "down-element-gradient" transfer
!   (????) processes

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use aermicro_mem_mod, only: pc,nbins,rnucpe,nelements,igelem,ienconc,nnucelem  &
     ,I_VOLATILE,I_INVOLATILE,ik,inucelem,inucbin,itype,nnucbin,pconmax  &
     ,rnuclg,rmass,icorelem,if_nuc_lh,cp_cgs,FEW_PC,rhoa,ncore,rlheat,rlh_nuc

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Calculates particle source terms <rnucpe> due to particle
!  element transfer processes for which the source element number is larger
!  than the target element number.

integer :: ie,ib,ig,iepart,jefrom,iefrom,igfrom,ipow,jfrom,ifrom,ic  &
     ,iecore
real(f4) :: rhoa_cgs,elemass,rmasscore,totmass,fracmass,rnucprod,c1

!=== Executable statements:

! Atmospheric density in cgs units:

rhoa_cgs = rhoa(ik)
c1 = cp_cgs*rhoa_cgs

! Calculate nucleation source terms for which the source element
!  number is greater than the target element number:

! Set nucleation production rates to zero to avoid double-application
!  of rates calculated in upgxfer.f:

do ie=1,nelements
   do ib=1,nbins(igelem(ie))
      rnucpe(ib,ie) = ZERO_f4
   enddo
enddo

do ie=1,nelements
   ig = igelem(ie)      ! target particle group         
   iepart = ienconc(ig)    ! target particle number concentration element

   do ib=1,nbins(ig)

      ! First calculate production terms due to nucleation <rnucpe>:

      do jefrom=1,nnucelem(ie)      ! Loop over elements that nucleate to
                                    !  element <ie>

         iefrom = inucelem(jefrom,ie)    ! source particle element

         ! Only calculate production rates here if <ielem> is less than
         !  <iefrom>.  Otherwise, production is calculated in growp.f:

         if(ie < iefrom) then

            igfrom = igelem(iefrom)            ! source particle group

            ! <ipow> is the power to which the source particle mass must be
            !  taken to match the type of the target element:

            if(itype(iefrom) == I_INVOLATILE .or.  &
                 itype(iefrom) == I_VOLATILE) then
               ipow = itype(ie)-1
            else
               ipow = itype(ie)-itype(iefrom)
            endif

            do jfrom=1,nnucbin(igfrom,ib,ig)    ! Loop over bins that
                                                !  nucleate to bin <ib>

               ifrom = inucbin(jfrom,igfrom,ib,ig)    ! bin of source

               ! Bypass calculation if few source particles are present:

               if(pconmax(ik,igfrom) > FEW_PC) then

                  if(rnuclg(ifrom,igfrom,ig) > ZERO_f4) then

                     ! First calculate mass associated with the source element
                     !  <elemass> (this is <rmass> for all source elements
                     !  except particle number concentration in a multicomponent
                     !  particle group):

                     if(ncore(igfrom) == 0 .or. itype(iefrom) > I_VOLATILE) then
                        elemass = rmass(ifrom,igfrom)
                     else
                        totmass = pc(ik,ifrom,iefrom)*rmass(ifrom,igfrom)
                        rmasscore = pc(ik,ifrom,icorelem(1,igfrom))
                        do ic=2,ncore(igfrom)
                           iecore = icorelem(ic,igfrom)
                           rmasscore = rmasscore+pc(ik,ifrom,iecore)
                        enddo
                        fracmass = ONE_f4-rmasscore/totmass
                        elemass  = fracmass*rmass(ifrom,igfrom)
                     endif

                     rnucprod = rnuclg(ifrom,igfrom,ig)*  &
                          pc(ik,ifrom,iefrom)*elemass**ipow

                     rnucpe(ib,ie) = rnucpe(ib,ie)+rnucprod

                     ! Calculate latent heat associated with nucleation
                     !  to <ibin,ielem> from <ifrom,iefrom>:

                     if(if_nuc_lh(iefrom,ie)) then
                        rlheat = rlheat+rnucprod*rlh_nuc(iefrom,ie)/c1*elemass
                     endif

                  endif

               endif
            enddo
         endif
      enddo

   enddo
enddo

end subroutine downeg_xfer


!```````````````````````````````````````````````````````````````````
subroutine apply_downeg_evap

!+ Applies evaporation and nucleation production terms to particle
!   concentrations

use aermicro_mem_mod, only: nelements,nbins,pc,ik,igelem,dtime,evappe,rnucpe,time

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: ie,ib

!=== Executable statements:

! Visit each radius bin for each element to compute particle production 
!  due to evaporation and element transfer processes for which the source
!  element number is greater than the target element number:

do ie=1,nelements
   do ib=1,nbins(igelem(ie))

      pc(ik,ib,ie) = pc(ik,ib,ie)+dtime*(evappe(ib,ie)+rnucpe(ib,ie))
!      if(time < 0. .and. ik == 1) then
!        print *,'zz',ik,ib,ie,dtime,evappe(ib,ie),rnucpe(ib,ie),pc(ik,ib,ie)  &
!              ,dtime*(evappe(ib,ie)+rnucpe(ib,ie))
!      endif
   enddo
enddo

!if(time < 0. .and. ik == 1) then
!   pause
!endif

end subroutine apply_downeg_evap


!```````````````````````````````````````````````````````````````````
subroutine part_compdensity

!+ Calculates new average particle densities (composition)

use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: ngroups,ncore,rmshell,vcore,ienconc,nbins,ik,pc  &
     ,rhoelem,rhop,SMALL_PC,icorelem,rmass

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! The particle mass density can change at each time-step due to
!  changes in the core mass fraction.

integer :: ig,iepart,ib,jcore,iecore

!=== Executable statements:

! Calculate average particle mass density for each group:

do ig = 1,ngroups

! Only need to do this if particle group has at least 1 core:

   if(ncore(ig) >= 1) then

      ! Define particle # concentration element index for current group:

      iepart = ienconc(ig)     ! element of particle number concentration 

      ! Calculate volume of cores and the mass of shell material
      !  <vcore> is the volume of core material and <rmshell> is the
      !  mass of shell material:

      do ib=1,nbins(ig)
         vcore(ik,ib) = ZERO_f4
         rmshell(ik,ib) = rmass(ib,ig)*pc(ik,ib,iepart)
      enddo

      do jcore=1,ncore(ig)
           
         iecore = icorelem(jcore,ig)    ! core element
         
         do ib=1,nbins(ig)

            vcore(ik,ib) = vcore(ik,ib)+pc(ik,ib,iecore)/rhoelem(iecore)
            rmshell(ik,ib) = rmshell(ik,ib)-pc(ik,ib,iecore)
         enddo

      enddo

      ! Calculate average density:
      
      do ib=1,nbins(ig)


         ! Do not set <rhop> to time-dependent value if the particle
         !  concentration is too small, or if the "shell" mass <rmshell>
         !  is computed to be a negative value (mostly a check for when an
         !  external model handles the aerosol advection/diffusion itself
         !  and treats them, numerically at least, unevenly, so that the
         !  effective core mass concentration becomes greater than that of
         !  particle mass concentration ???):

         if(pc(ik,ib,iepart) > SMALL_PC .and. rmshell(ik,ib) >= ZERO_f4) then
            rhop(ik,ib,ig) = rmass(ib,ig)*pc(ik,ib,iepart)/  &
                 (vcore(ik,ib)+rmshell(ik,ib)/rhoelem(iepart))
         else
            rhop(ik,ib,ig) = rhoelem(iepart)
         endif

      enddo

   endif
   
enddo

end subroutine part_compdensity


end module aemic_partmphys_mod
