!```````````````````````````````````````````````````````````````````
module aemic_predict_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine predict_new_state

!+ Step variables forward in time by one timestep

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use aermicro_mem_mod, only: nelements,nbins,igelem,fbotpart,pc  &
     ,vf,nz_mic,ptc,rhoa,t,p,PREF,ptcl,ngasses,gcl,pcl,ik  &
     ,ienconc,SMALL_PC,gc,dtime_save,dtime,ntsubsteps,minsubsteps  &
     ,rkappa,small_coremconc,time  &
     ,loop_again,loop_cnt,lpcnt_col,gc_vTX_on,ptc_vTX_on  &
     ,pc_vTX_on,ibbnd_pc,I_FLUX_SPEC,sadv_gc_tend,sadv_pc_tend,sadv_ptc_tend  &
     ,i_curr,j_curr,gc_extTX_on,ptc_extTX_on,pc_extTX_on,do_grow
use aemic_vertTX_mod, only: vert_transport
use aemic_slowproc_mod, only: slow_aermicro
use aemic_satvap_mod, only: gas_supersat_col
use aemic_fastproc_mod, only: fast_aermicro
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:
    
real(f4) :: pt,subdtni
integer :: ib,ie,k,igas,isubtim,ip,ig

!^%
!^%real(f4) :: t1,t2
!^%real(f4), external :: tag_CPUtime
!^%

!=== Executable statements:

! Calculate changes due to vertical transport (and optional externally
!  computed horizontal/other transport tendencies) of gas, potential
!  temperature, and/or aerosols:

if(pc_vTX_on) then   ! Set selected boundary conditions for transport

   if(ibbnd_pc == I_FLUX_SPEC) then    ! Need to set bottom particle flux

      ! Define the particle flux through the lowest boundary (due to
      !  gravitational sedimentation, particle entrainment from the ground
      !  surface, etc.):
      !  NOTE: <fbotpart> is defined as the *upward* flux from the lower
      !        boundary (and is thus negative if representing a flux *towards*
      !        the lower boundary).

      do ie=1,nelements
         ig = igelem(ie)
         do ib=1,nbins(ig)
            ! Particle flux towards the surface due to gravitational
            !  sedimentation [#/(s.cm^2)] or [g/(s.cm^2)]:
            fbotpart(ib,ie) = -vf(1,ib,ig)*pc(1,ib,ie)
         enddo
      enddo

   endif
endif

if(pc_vTX_on .or. ptc_vTX_on .or. gc_vTX_on) then ! Vertical transport desired
   call vert_transport       ! Calculate vertical transport; update vars
endif   

! Add any external transport tendencies to gas, potential temperature, and/or
!  aerosols:

if(ptc_extTX_on) then
   do k=1,nz_mic
      ptc(k) = ptc(k)+sadv_ptc_tend(k)*dtime  
   enddo
endif

if(gc_extTX_on) then
   do igas=1,ngasses
      do k=1,nz_mic
         gc(k,igas) = gc(k,igas)+sadv_gc_tend(k,igas)*dtime

         if(gc(k,igas) < ZERO_f4) then    ! Check for gross transport errors

            print *  &
                 ,'WARNING: gc < 0. due to transport ONLY; igas = '  &
                 ,igas,' at ',k,i_curr,j_curr,'val = ',gc(k,igas)
!            pause

            ! CLIPPING: Ensure that the remaining gas in the each model layer
            !  is >= the smallest allowable concentration (zero):

            gc(k,igas) = ZERO_f4

         endif

      enddo
   enddo
endif

if(pc_extTX_on) then
   do ie=1,nelements
      ig = igelem(ie)
      do ib=1,nbins(ig)

         ! Now add any external transport tendencies (which *may* cause 'pc'
         !  to become negative [due primarily to (possible) huge diffusion
         !  errors], but all we can do at this point is add enough aerosol
         !  afterwards to bring the concentration back to a minimum level:

         do k=1,nz_mic

            pc(k,ib,ie) = pc(k,ib,ie)+sadv_pc_tend(k,ib,ie)*dtime

!^            if(pc(k,ib,ie) < ZERO_f4) then    ! Check for gross transport errors
!^               print *  &
!^                    ,'WARNING: pc < 0. due to transport ONLY; ib,ie = '  &
!^                    ,ib,ie,' at ',k,i_curr,j_curr,'val = ',pc(k,ib,ie)
!^               pause
!^            endif

         enddo

         ! CLIPPING: Ensure that the remaining aerosol in the each model layer
         !  is >= the smallest allowable concentration:

         ip = ienconc(ig)  
         if(ie == ip) then
            do k=1,nz_mic
               pc(k,ib,ie) = max(pc(k,ib,ie),SMALL_PC)
            enddo
         else
            do k=1,nz_mic
               if(pc(k,ib,ip) <= SMALL_PC .or.  &
                    pc(k,ib,ie) < small_coremconc(ib,ie)) then
                  pc(k,ib,ie) = small_coremconc(ib,ie)
               endif
            enddo
         endif

      enddo
   enddo
endif

! Calculate changes in particle concentrations due to microphysical
!  processes, part 1.  (potentially slower microphysical calcs)
!  All spatial points are handled by one call to this routine:

call slow_aermicro

! Calculate new saturation ratios for use in `comp_nsubsteps`:

do k=1,nz_mic
   pt = ptc(k)/rhoa(k)
   t(k) = pt*(p(k)/PREF)**rkappa    ! Current temperature [K]
enddo
call gas_supersat_col

!####
gcl = gc             ! 
pcl = pc             ! Update "old" values of entire arrays (value after 
ptcl = ptc           !  external/internal transport and "slow" microphysics)
!####

if(do_grow) then

   ! Calculate changes in particle concentrations for current spatial point
   !  due to microphysical processes, part 2.  (faster microphysical calcs):

   dtime_save = dtime     ! Save current timestep used for processes so far
   lpcnt_col = 0

   do k=1,nz_mic    ! Visit each of the layer mid-points
      
      ik = k              ! Set global variable used by routines below

      ! Compute or specify number of sub-timestep intervals for current spatial
      !  point (could be same for all spatial pts, or could vary as a function
      !  of location):
      
      ntsubsteps = minsubsteps
      call comp_nsubsteps

      loop_again = .TRUE.   ! Need to loop at least once 
      loop_cnt = 0
      fast_loop: do while(loop_again)
         loop_cnt = loop_cnt+1
         
         ! Compute sub-timestep time interval for current spatial grid point:
         
         subdtni = ONE_f4/real(ntsubsteps)
         dtime = dtime_save*subdtni

         ! Do sub-timestepping for current spatial grid point:

         do isubtim=1,ntsubsteps

            loop_again = .FALSE. ! Assume that won't have to redo substep loop

            do ie=1,nelements
            
               ig = igelem(ie)
               ip = ienconc(ig)

               do ib=1,nbins(ig)
                  
                  ! Check for negative aerosol concentrations; abort if any
                  !  such values found:

                  if(pc(k,ib,ie) < ZERO_f4) then
                     write (*,'(/a,/,a,/,a)')   &
           'WARNING: a negative particle concentration has been detected --'  &
      ,'      due to the "fast" microphysical processes; will decrease subDt.'
                     print *,'ik,ib,ie,pc,pcl,dtime_save'  &
                          ,ik,ib,ie,pc(ik,ib,ie),pcl(ik,ib,ie),dtime_save
                     !#                  stop '*** Abnormal completion ***'
                     loop_again = .TRUE.
                  endif

                  ! Prevent particle concentrations from dropping below
                  !  SMALL_PC:
      
                  if(ie == ip) then
                     pc(k,ib,ie) = max(pc(k,ib,ie),SMALL_PC)             
                  else
                     if(pc(k,ib,ip) <= SMALL_PC .or.  &
                          pc(k,ib,ie) < small_coremconc(ib,ie)) then
                        pc(k,ib,ie) = small_coremconc(ib,ie)
                     endif
                  endif
                  
               enddo
            enddo

            ! Update temperature:

            pt = ptc(k)/rhoa(k)
            t(k) = pt*(p(k)/PREF)**rkappa

            call fast_aermicro   ! Call "fast" microphysical process driver

            if(loop_again) then   ! Violation has occurred [(gc or pc) < zero]
            
               ! Violated imposed minimum gas/particle concentration (< zero)
               !  due to overzealous particle growth, etc.; reduce the
               !  sub-timestep and redo the fast microphysical calculations:

               write (*,'(a,i6,a,i6,3(a,i3),a)') &
                    'WARNING: violation, substep ',isubtim,' of '  &
                    ,ntsubsteps,'; at (',k,',',i_curr,',',j_curr,')'

               gc = gcl      ! 
               pc = pcl      ! Return entire arrays to their prior values
               ptc = ptcl    !

               ntsubsteps = ntsubsteps*2   ! Reduce the effective sub-timestep

               if(ntsubsteps > 1000000) then
                  print *,'ERROR: nsubDt WAY too big',ntsubsteps,', at ',k  &
                       ,i_curr,j_curr,'isubtim = ',isubtim
                  stop '*** Abnormal completion (waybig)***'
               endif

               exit    ! Exit substep loop; redo
            endif

         enddo

      enddo fast_loop

      if(loop_cnt > 1) then
         lpcnt_col = lpcnt_col+1
      endif

   enddo

   ! Restore normal timestep:

   dtime = dtime_save

endif

! Hydrostatically update atmospheric profiles:

!%if(.not. do_parcel) then
!%   call hydrostat
!%endif

end subroutine predict_new_state


!```````````````````````````````````````````````````````````````````
subroutine comp_nsubsteps

!+ Calculates the number of sub-timesteps for the current model spatial point

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use aermicro_mem_mod, only: ik,rhoelem,rmrat,rmassmin,igrowgas,is_grp_ice,pvapi  &
     ,pvapl,supsati,supsatl,gro,gro1,ntsubsteps,dtime_save,dm,vol,ienconc  &
     ,itype,pconmax,I_VOLATILE,pc,nbins,maxsubsteps,minsubsteps,supsatiold  &
     ,supsatlold,conmax,ngroups,gc,t,i_curr,j_curr,gcl  &
     ,rmass_mode => fscr1  &   ! [ngroups]
     ,ibin_mode => iscr1  &   ! [ngroups]
     ,ibin_small => iscr2   ! [ngroups]
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

logical :: finished
integer :: ig,iepart,ib,igas
real(f4) :: dmdt,ss,pvap,g0,g1,dt_adv,pctmp
!real(f4) :: ssold
!?real(f4) :: rntot,voltot
logical, save :: already_warned = .FALSE.

!=== Executable statements:

! Find the mode mass (volume weighted) of each aerosol group
!  and find the mass bin closest to this mass;
!  ALSO, find the bin number of the smallest particle bin that contains a
!  significant number of particles:

do ig=1,ngroups

   if(pconmax(ik,ig) > conmax) then

      ibin_small(ig) = nbins(ig)-1   !
!?      rmass_mode(ig) = 0.            ! Initialize some values
!?      ibin_mode(ig) = 0              !

      iepart = ienconc(ig)     ! element of particle number concentration

      if(itype(iepart) == I_VOLATILE) then

         finished = .FALSE.
!?         voltot = 0.
!?         rntot = 0.
         do ib=1,nbins(ig)
            pctmp = pc(ik,ib,iepart)

!?            voltot = voltot+pctmp*vol(ib,ig)
!?            rntot = rntot+pctmp

            if(.NOT.finished .and. pctmp > conmax*0.1_f4) then
               ibin_small(ig) = ib    ! Smallest currently significant bin
               finished = .TRUE.
            endif
         enddo

         if(ibin_small(ig) == nbins(ig)) then
            ibin_small(ig) = nbins(ig)-1         ! Constrain
            if(.NOT.already_warned) then
               write (*,'(/a,i2,/,a/)')  &
                   'WARNING: mass/size bin distribution in at least group #'  &
                   ,ig,' likely needs broadened.'
               already_warned = .TRUE.
            endif
         endif

!?         rmass_mode(ig) = voltot/rntot*rhoelem(iepart)
!?         ibin_mode(ig) = int(log(rmass_mode(ig)/rmassmin(ig))/log(rmrat(ig)))+2
!?         ibin_mode(ig) = max(1,min(ibin_mode(ig),nbins(ig)-1))

      endif

   endif

enddo

! Calculate the growth rate of a particle with the mode radius for
!  each volatile group.  The maximum timestep to use is then the
!  mass growth rate divided by the (mass bin width)/2:

dt_adv = 1.e10_f4   ! Initialize 
do ig=1,ngroups
        
   if(pconmax(ik,ig) > conmax) then

      iepart = ienconc(ig)     ! element of particle number concentration
      igas = igrowgas(iepart)      ! condensing gas

      if(itype(iepart) == I_VOLATILE) then

         if(is_grp_ice(ig)) then
            ss = supsati(ik,igas)
            pvap = pvapi(ik,igas)
         else
            ss = supsatl(ik,igas)
            pvap = pvapl(ik,igas)
         endif
         
         g0 = gro(ik,ibin_small(ig)+1,ig)
         g1 = gro1(ik,ibin_small(ig)+1,ig)
         dmdt = max(abs(pvap*ss*g0/(ONE_f4+g0*g1*pvap)),1.e-25_f4)
         ! Calculate stable "cross-bin advection" timestep [s]:
!##%         dt_adv = min(dt_adv,0.5*dm(ibin_small(ig)+1,ig)/dmdt)
         dt_adv = min(dt_adv,0.9_f4*dm(ibin_small(ig)+1,ig)/dmdt)

      endif

   endif

enddo

!print *,'nnnn',dtime_save/dt_adv,dmdt,dmdt/((dm(ib,ig)+dm(ib+1,ig))*0.5),dt_adv,dm(ibin_small(2)+1,2),conmax

ntsubsteps = nint(min(real(maxsubsteps),dtime_save/dt_adv))
ntsubsteps = max(minsubsteps,ntsubsteps)

if(ntsubsteps == maxsubsteps) then
   print *,'ERROR: ntsubDt(dmadv) too big at ',ik,i_curr,j_curr  &
        ,dtime_save/dt_adv,maxsubsteps
   stop '*** Abnormal completion (dmadv)***'
endif

! If the supersaturation becomes greater than zero over the timestep, then
!  use <maxsubsteps>:

!#### NOTE: Is this necessary for ice??? Is a threshold of zero OK for both
!####       water and CO2 ices (or check separately)??? 

!do ig=1,ngroups
!
!   iepart = ienconc(ig)     ! element of particle number concentration
!   igas = igrowgas(iepart)      ! condensing gas
!
!   if(itype(iepart) == I_VOLATILE) then
!      
!      if(is_grp_ice(ig)) then
!         ss = supsati(ik,igas)
!         ssold = supsatiold(ik,igas)
!      else
!         ss = supsatl(ik,igas)
!         ssold = supsatlold(ik,igas)
!      endif
!
!      if((ssold < 0. .and. ss >= 0.) .or. (ssold >= 0. .and. ss < 0.)) then
!         ntsubsteps = maxsubsteps
!
!         print *,'WARNING: ntsubDt(satflop) too big at ',ik,i_curr,j_curr  &
!              ,ssold,ss,gcl(ik,igas),gc(ik,igas)
!
!      endif
!
!   endif
!
!enddo
!
!if(ik == 1 .and. i_curr == 7 .and. j_curr==4) then
!   pause
!endif

! If the ice supersaturation is large enough for homogeneous freezing
!  of sulfate aerosols, then use maximum number of substeps:

!?      if( supsati3(ixyz,1) .gt. 0.4 .and.
!?     $    t3(ixyz) .lt. 233.16 ) ntsubsteps = maxsubsteps

end subroutine comp_nsubsteps


end module aemic_predict_mod
