!```````````````````````````````````````````````````````````````````
module aemic_satvap_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine gas_supersat_col

!+ Evaluates supersaturations for all gases (in entire vertical column)

use univ_kind_defs_mod, only: f4
use aermicro_mem_mod, only: gwtmol,nz_mic,gc,t,pvapl,supsati,supsatl,pvapi  &
     ,R_univ_cgs,ngasses

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Evaluates supersaturations <supsatl> and <supsati> for all gases:

integer :: igas,k
real(f4) :: rvap,gc_cgs

!=== Executable statements:

! Calculate vapor pressures:

call vapor_press_col

do igas=1,ngasses      ! Loop over all gasses

   rvap = R_univ_cgs/gwtmol(igas)     ! Define gas constant for this gas
   do k=1,nz_mic

      gc_cgs = gc(k,igas)

      supsatl(k,igas) = (gc_cgs*rvap*t(k)-pvapl(k,igas))/pvapl(k,igas)
      supsati(k,igas) = (gc_cgs*rvap*t(k)-pvapi(k,igas))/pvapi(k,igas)

   enddo
enddo

end subroutine gas_supersat_col


!```````````````````````````````````````````````````````````````````
subroutine vapor_press_col

!+ Calculates the vapor pressure for all gases (in entire vertical column)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use aermicro_mem_mod, only: ngasses,gasname,nz_mic,t,pvapi,pvapl

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Define coefficients in Buck's formulation for saturation vapor pressures:

real(f4), parameter :: BAI = 6.1115e3_f4, BBI = 23.036_f4  &
                            ,BCI = 279.82_f4, BDI = 333.7_f4  &
                            ,BAL = 6.1121e3_f4, BBL = 18.729_f4  &
                            ,BCL = 257.87_f4, BDL = 227.3_f4

integer :: igas,k
real(f4) :: tt

!=== Executable statements:

do igas=1,ngasses      ! Loop over all gasses

   if(gasname(igas)(1:1) == 'W') then     ! H2O vapor

      ! Saturation vapor pressure over liquid water and water ice
      !  (from Buck [J. Atmos. Sci., 20, 1527, 1981]):

      do k=1,nz_mic

         tt = t(k)-273.16_f4        ! Temperature in deg_Celsius

         pvapl(k,igas) = BAL*exp((BBL-tt/BDL)*tt/(tt+BCL))

!orig%         pvapi(k,igas) = BAI*exp((BBI-tt/BDI)*tt/(tt+BCI))
         pvapi(k,igas) = 6.11_f4*exp(22.5_f4*(ONE_f4-(273.16_f4/t(k))))   ! from TC [mb]
         pvapi(k,igas) = pvapi(k,igas)*1.e3_f4         ! [mb -> dynes/cm^2]

         ! Check to see whether temperature is ouside range of validity
         !  for parameterizations (???):

         if(pvapl(k,igas) <= 1.e-10_f4) then
            pvapl(k,igas) = 1.e-10_f4
         endif

      enddo

   elseif(gasname(igas)(1:1) == 'C') then     ! carbon dioxide gas

      ! Vapor pressures for CO2 taken from Kasting  Icarus 94, 1991 (Atm):

      do k=1,nz_mic

         if(t(k) >= 216.56_f4) then
            pvapi(k,igas) = 10._f4**(3.128082_f4-(867.2124_f4/t(k))+  &
                 18.65612e-3_f4*t(k)-72.48820e-6_f4*t(k)**2+93.e-9_f4*t(k)**3)
            pvapi(k,igas) = pvapi(k,igas)*1.01325e6_f4  !??? unit convert???or??
            pvapl(k,igas) = pvapi(k,igas)
         else
            pvapi(k,igas) = 10._f4**(6.760956_f4-(1284.07_f4/(t(k)-4.718_f4))+  &
                 1.256e-4_f4*(t(k)-143.15_f4))
            pvapi(k,igas) = pvapi(k,igas)*1.01325e6_f4 !??? unit convert???or??
            pvapl(k,igas) = pvapi(k,igas)
         endif

      enddo

   endif
 
enddo

end subroutine vapor_press_col


!```````````````````````````````````````````````````````````````````
subroutine gas_supersat_pt

!+ Evaluates supersaturations for all gases (at one spatial point)

use univ_kind_defs_mod, only: f4
use aermicro_mem_mod, only: gwtmol,gc,t,pvapl,supsati,supsatl,pvapi  &
     ,R_univ_cgs,ngasses,ik

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Evaluates supersaturations <supsatl> and <supsati> for all gases:

integer :: igas
real(f4) :: rvap,gc_cgs

!=== Executable statements:

! Calculate vapor pressures:

call vapor_press_pt

do igas=1,ngasses      ! Loop over all gasses

   rvap = R_univ_cgs/gwtmol(igas)     ! Define gas constant for this gas

   gc_cgs = gc(ik,igas)
	
   supsatl(ik,igas) = (gc_cgs*rvap*t(ik)-pvapl(ik,igas))/pvapl(ik,igas)

   supsati(ik,igas) = (gc_cgs*rvap*t(ik)-pvapi(ik,igas))/pvapi(ik,igas)

enddo

end subroutine gas_supersat_pt


!```````````````````````````````````````````````````````````````````
subroutine vapor_press_pt

!+ Calculates the vapor pressure for all gases (at one spatial point)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use aermicro_mem_mod, only: ngasses,gasname,ik,t,pvapi,pvapl

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Define coefficients in Buck's formulation for saturation vapor pressures:

real(f4), parameter :: BAI = 6.1115e3_f4, BBI = 23.036_f4  &
                            ,BCI = 279.82_f4, BDI = 333.7_f4  &
                            ,BAL = 6.1121e3_f4, BBL = 18.729_f4  &
                            ,BCL = 257.87_f4, BDL = 227.3_f4

integer :: igas
real(f4) :: tt

!=== Executable statements:

do igas=1,ngasses      ! Loop over all gasses
   
   if(gasname(igas)(1:1) == 'W') then     ! H2O vapor
      
      ! Saturation vapor pressure over liquid water and water ice
      !  (from Buck [J. Atmos. Sci., 20, 1527, 1981]):

      tt = t(ik)-273.16_f4        ! Temperature in deg_Celsius
      
      pvapl(ik,igas) = BAL*exp((BBL-tt/BDL)*tt/(tt+BCL))

!orig%         pvapi(ik,igas) = BAI*exp((BBI-tt/BDI)*tt/(tt+BCI))
      pvapi(ik,igas) = 6.11_f4*exp(22.5_f4*(ONE_f4-(273.16_f4/t(ik))))   ! from TC [mb]
      pvapi(ik,igas) = pvapi(ik,igas)*1.e3_f4          ! [mb -> dynes/cm^2]

      ! Check to see whether temperature is ouside range of validity
      !  for parameterizations (???):

      if(pvapl(ik,igas) <= 1.e-10_f4) then
         pvapl(ik,igas) = 1.e-10_f4
      endif

   elseif(gasname(igas)(1:1) == 'C') then     ! carbon dioxide gas

      ! Vapor pressures for CO2 taken from Kasting  Icarus 94, 1991 (Atm):

      if(t(ik) >= 216.56_f4) then
         pvapi(ik,igas) = 10._f4**(3.128082_f4-(867.2124_f4/t(ik))+  &
              18.65612e-3_f4*t(ik)-72.48820e-6_f4*t(ik)**2+93.e-9_f4*t(ik)**3)
         pvapi(ik,igas) = pvapi(ik,igas)*1.01325e6_f4
         pvapl(ik,igas) = pvapi(ik,igas)
      else
         pvapi(ik,igas) = 10._f4**(6.760956_f4-(1284.07_f4/(t(ik)-4.718_f4))+  &
              1.256e-4_f4*(t(ik)-143.15_f4))
         pvapi(ik,igas) = pvapi(ik,igas)*1.01325e6_f4
         pvapl(ik,igas) = pvapi(ik,igas)
      endif
      
   endif
 
enddo

end subroutine vapor_press_pt


end module aemic_satvap_mod
