!```````````````````````````````````````````````````````````````````
module aemic_vertTX_mod

!+ Module that contains routines related to aerosol vertical transport

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine vert_transport

!+ Calculates spatial vertical transport on the model grid

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use aermicro_mem_mod, only: nz_mic,w,I_FLUX_SPEC,nz1_mic,vtrans,cvert,dtime  &
     ,itbnd_pc,ibbnd_pc,nelements,nbins,ftoppart,vf,drho_dt,fbotpart,ngasses  &
     ,gc,itbnd_ptc,ibbnd_ptc,pc,itbnd_gc,ibbnd_gc,ptc,rhoa,itbnd,ibbnd,ftop  &
     ,fbot,ftopgas,fbotgas,igelem,divcor,vertdifd,vertdifu,m_to_cm  &
     ,gc_vTX_on,ptc_vTX_on,pc_vTX_on,i_curr,j_curr,grid,should_vTX_ie
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: nlowlim,nuplim,k,ie,ib,ig,igas

!=== Executable statements:

! Determine limits for vertical loops: For 1-D simulations, we do not
!  want the concentration to change due to flux out of a boundary
!  grid box (i.e., upward motion for the bottom boundary and
!  downward motion for the top boundary).  This constraint ensures
!  that advection in a 1-D simulation will not change concentrations
!  at the upwind boundary:

nlowlim = 1
nuplim = nz_mic
if(w(2) > ZERO_f4) then
   nlowlim = 2
endif
if(w(nz_mic) < ZERO_f4) then
   nuplim = nz_mic-1
endif

! Treat vertical transport of air density:

if(ptc_vTX_on) then

   ! Boundary fluxes are zero:

   itbnd = I_FLUX_SPEC
   ibbnd = I_FLUX_SPEC

   ftop = ZERO_f4
   fbot = ZERO_f4

   ! Store temporary (work) values in <cvert>, 
   !  evaluate vertical velocities at layer boundaries,
   !  and set divergence correction term to zero:

   do k=1,nz1_mic
      vtrans(k) = w(k)
      if(k <= nz_mic) then
         cvert(k) = rhoa(k)
         divcor(k) = ZERO_f4
      endif
   enddo

   ! Calculate particle transport rates due to vertical advection
   !  (note: no diffusion for air density), and solve for air density
   !  at end of timestep:

   call vert_advect
   call vert_solvetx

   ! In 1D, assume assume any vertical divergence is
   !  balanced by horizontal convergence: don't allow air density
   !  to change in time, but calculate rate of change that would have
   !  resulted from advection -- this tendency is then used below for
   !  a divergence correction:

   do k=1,nz_mic
      drho_dt(k) = (rhoa(k)-cvert(k))/(rhoa(k)*dtime)
   enddo

endif

! Treat vertical transport of particles:

if(pc_vTX_on) then

   itbnd = itbnd_pc
   ibbnd = ibbnd_pc

   do ie=1,nelements         ! Loop over particle elements

      if(should_vTX_ie(ie)) then  ! If vert transport of this element wanted

         ig = igelem(ie)        ! particle group

         do ib=1,nbins(ig)          ! Loop over particle mass bins

            ! Fluxes across top and bottom of model:

            ftop = ftoppart(ib,ie)
            fbot = fbotpart(ib,ie)

            ! Store temporary (work) values in <cvert>, 
            !  evaluate vertical velocities at layer boundaries, and
            !  when 1D, assign divergence correction term:

            do k=1,nz1_mic
!@@@            vtrans(k) = w(k)-vf(k,ib,ig)
               !@@@ CLIP PARTICLE FALL SPEEDS @@@
               vtrans(k) = w(k)-min(150._f4*m_to_cm,vf(k,ib,ig))
               !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

               if(k <= nz_mic) then
                  cvert(k) = pc(k,ib,ie)
!?               if(NXY == 1) then
!?                  divcor(k) = cvert(k) * drho_dt(k)
                  divcor(k) = ZERO_f4
!?               endif
               endif
            enddo
         
            ! Calculate particle transport rates due to vertical advection and
            !  vertical diffusion, and solve for concentrations at end of
            !  timestep:

            call vert_advect
!?         call vert_diffuse
            do k=1,nz1_mic       !
               vertdifd(k) = ZERO_f4  ! Zero array values if diffusion not calc'd
               vertdifu(k) = ZERO_f4  !
            enddo                !
            call vert_solvetx

            ! Update particle concentrations:

            do k=nlowlim,nuplim
               pc(k,ib,ie) = cvert(k)
            enddo

         enddo
      endif
   enddo
endif

! Treat vertical transport of gases (for comments, see above treatment of
!  particles):

if(gc_vTX_on) then

   do k=1,nz1_mic
      vtrans(k) = w(k)
   enddo

   itbnd = itbnd_gc
   ibbnd = ibbnd_gc

   do igas=1,ngasses
         
      ftop = ftopgas(igas)
      fbot = fbotgas(igas)

      do k=1,nz1_mic
         vtrans(k) = w(k)
         if(k <= nz_mic) then
            cvert(k) = gc(k,igas)
!?            if( NXY == 1 ) divcor(k) = cvert(k) * drho_dt(k)
         endif
      enddo
         
      call vert_advect
      call vert_diffuse
      call vert_solvetx
         
      do k=nlowlim,nuplim
         gc(k,igas) = cvert(k)
      enddo
         
   enddo
	
endif

! Treat vertical transport of potential temperature (for comments, see above
!  treatment of particles):

if(ptc_vTX_on) then

   itbnd = itbnd_ptc
   ibbnd = ibbnd_ptc
      
   ftop = ZERO_f4
   fbot = ZERO_f4
      
   do k=1,nz1_mic
      vtrans(k) = w(k)
      if(k <= nz_mic) then
         cvert(k) = ptc(k)
!?         if( NXY == 1 ) divcor(k) = cvert(k) * drho_dt(k)
      endif
   enddo
      
   call vert_advect
   call vert_diffuse
   call vert_solvetx
      
   do k=nlowlim,nuplim
      ptc(k) = cvert(k)
   enddo
      
endif

end subroutine vert_transport


!```````````````````````````````````````````````````````````````````
subroutine vert_advect

!+ Calculates vertical advection rates
!
!  This routine calculates vertical advection rates using Piecewise Polynomial
!  Method [Colela and Woodard, J. Comp. Phys., 54, 174-201, 1984]
!
!  <vertadvu(k)> is upward vertical transport rate into level k
!                from level k-1 [cm/s].
!  <vertadvd(k)> is downward vert transport rate into level k-1 from level k

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4, ONE_THIRD_f4, ONE_SIXTH_f4
use univ_numtype_constants_mod, only: epsilon_f8
use aermicro_mem_mod, only: vertadvu,vertadvd,dz,cvert,vtrans,dtime,ibbnd,itbnd  &
     ,I_OPEN_FLUX,zl,zc,nz1_mic,time  &
     ,nz => nz_mic  &
     ,dela => fscr1  &      ! [nz_mic]
     ,delma => fscr2  &      ! [nz_mic]
     ,aju => fscr3  &      ! [nz_mic]
     ,ar => fscr4  &      ! [nz_mic]
     ,al => fscr5  &      ! [nz_mic]
     ,a6 => fscr6  &     ! [nz_mic]
     ,cvdz => fscr7  &  ! [nz_mic]
     ,dzkkp1 => fscr8   ! [nz_mic]

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

real(f4) :: dpc,dpc1,dpcm1,ratt1,ratt2,ratt3,rat1,rat2,rat3,rat4,den1  &
     ,com2,x,xpos,cvert0,cvertnzp1
integer :: k

!=== Executable statements:

! Initialize fluxes to zero:

do k=1,nz1_mic
   vertadvu(k) = ZERO_f4
   vertadvd(k) = ZERO_f4
enddo

! First, use cubic fits to estimate concentration values at layer boundaries:

do k=1,nz
   cvdz(k) = cvert(k)/dz(k)
enddo

do k=1,nz-1
   dzkkp1(k) = dz(k)+dz(k+1)
enddo

do k=2,nz-1
   
   dpc = cvdz(k) !cvert(k)/dz(k)
   dpc1 = cvdz(k+1) !cvert(k+1)/dz(k+1)
   dpcm1 = cvdz(k-1)  !cvert(k-1)/dz(k-1)
!   ratt1 = dz(k)/(dz(k-1)+dz(k)+dz(k+1))
   ratt1 = dz(k)/(dz(k-1)+dzkkp1(k))
!   ratt2 = (2.*dz(k-1)+dz(k))/(dz(k+1)+dz(k))
   ratt2 = (2._f4*dz(k-1)+dz(k))/dzkkp1(k)
!   ratt3 = (2.*dz(k+1)+dz(k))/(dz(k-1)+dz(k))
   ratt3 = (2._f4*dz(k+1)+dz(k))/dzkkp1(k-1)
   dela(k) = ratt1*(ratt2*(dpc1-dpc)+ratt3*(dpc-dpcm1))
   
   if ((dpc1-dpc)*(dpc-dpcm1) > ZERO_f4 .and.  &
        abs(dela(k)) > real(epsilon_f8, kind=f4)) then
      delma(k) = min(abs(dela(k)), 2._f4*abs(dpc-dpc1),  &
           2._f4*abs(dpc-dpcm1))*abs(dela(k))/dela(k)
   else
      delma(k) = ZERO_f4
   endif
   
enddo
     
do k=2,nz-2

   dpc = cvdz(k) !cvert(k)/dz(k)
   dpc1 = cvdz(k+1) !cvert(k+1)/dz(k+1)
   dpcm1 = cvdz(k-1) !cvert(k-1)/dz(k-1)
!   rat1 = dz(k)/(dz(k)+dz(k+1))
   rat1 = dz(k)/dzkkp1(k)
!   rat2 = 2.*dz(k+1)*dz(k)/(dz(k)+dz(k+1))
   rat2 = 2._f4*dz(k+1)*dz(k)/dzkkp1(k)
!   rat3 = (dz(k-1)+dz(k))/(2.*dz(k)+dz(k+1))
   rat3 = dzkkp1(k-1)/(2._f4*dz(k)+dz(k+1))
!   rat4 = (dz(k+2)+dz(k+1))/(2.*dz(k+1)+dz(k))
   rat4 = dzkkp1(k+1)/(2._f4*dz(k+1)+dz(k))
!   den1 = dz(k-1)+dz(k)+dz(k+1)+dz(k+2)
   den1 = dzkkp1(k-1)+dzkkp1(k+1)

! <aju(k)> is the estimate for concentration (dn/dz) at layer boundary <k>+1/2:

   aju(k) = dpc+rat1*(dpc1-dpc)+ONE_f4/den1*(rat2*(rat3-rat4)*(dpc1-dpc) -  &
        dz(k)*rat3*delma(k+1)+dz(k+1)*rat4*delma(k))
enddo

! Now construct polynomial functions in each layer:

do k=3,nz-2
   al(k) = aju(k-1)
   ar(k) = aju(k)
enddo

! Use linear functions in first two and last two layers:

ar(2) = aju(2)
!al(2) = cvert(1)/dz(1)+(zl(2)-zc(1))/(zc(2)-zc(1))*  &
!     (cvert(2)/dz(2)-cvert(1)/dz(1))
al(2) = cvdz(1)+(zl(2)-zc(1))/(zc(2)-zc(1))*  &
     (cvdz(2)-cvdz(1))

ar(1) = al(2)
!al(1) = cvert(1)/dz(1)-(zc(1)-zl(1))/(zc(2)-zc(1))*  &
!     (cvert(2)/dz(2)-cvert(1)/dz(1))
al(1) = cvdz(1)-(zc(1)-zl(1))/(zc(2)-zc(1))*  &
     (cvdz(2)-cvdz(1))

al(nz-1) = aju(nz-2)
!ar(nz-1) = cvert(nz-1)/dz(nz-1)+(zl(nz)-zc(nz-1))  &
!     /(zc(nz)-zc(nz-1))*(cvert(nz)/dz(nz)-cvert(nz-1)/dz(nz-1))
ar(nz-1) = cvdz(nz-1)+(zl(nz)-zc(nz-1))  &
     /(zc(nz)-zc(nz-1))*(cvdz(nz)-cvdz(nz-1))

al(nz) = ar(nz-1)
!ar(nz) = cvert(nz-1)/dz(nz-1)+(zl(nz+1)-zc(nz-1))  &
!     /(zc(nz)-zc(nz-1))*(cvert(nz)/dz(nz)-cvert(nz-1)/dz(nz-1))
ar(nz) = cvdz(nz-1)+(zl(nz+1)-zc(nz-1))  &
     /(zc(nz)-zc(nz-1))*(cvdz(nz)-cvdz(nz-1))

! Ensure that boundary values are not negative:

al(1) = max(al(1), abs(ZERO_f4))
ar(nz) = max(ar(nz), abs(ZERO_f4))

! Next, ensure that polynomial functions do not deviate beyond the
!  range [<al(k)>,<ar(k)>]:

do k=1,nz

   dpc = cvdz(k) !cvert(k)/dz(k)
   if((ar(k)-dpc)*(dpc-al(k)) <= ZERO_f4) then
      al(k) = dpc
      ar(k) = dpc
   endif

   if((ar(k)-al(k))*(dpc-0.5_f4*(al(k)+ar(k))) > ONE_SIXTH_f4*(ar(k)-al(k))**2) then
      al(k) = 3._f4*dpc-2._f4*ar(k)
   endif

   if((ar(k)-al(k))*(dpc-0.5_f4*(al(k)+ar(k))) < -ONE_SIXTH_f4*(ar(k)-al(k))**2) then
      ar(k) = 3._f4*dpc-2._f4*al(k)
   endif

enddo

! Calculate fluxes across each layer boundary:

do k=1,nz

   dpc = cvdz(k) !cvert(k)/dz(k)
   dela(k) = ar(k)-al(k)
   a6(k) = 6._f4*(dpc-0.5_f4*(ar(k)+al(k)))

enddo

do k=1,nz-1

!   com2 = (dz(k)+dz(k+1))*0.5
   com2 = dzkkp1(k)*0.5_f4
   x = vtrans(k+1)*dtime/dz(k)
   xpos = abs(x)

! Upward transport rate:

   if(vtrans(k+1) > ZERO_f4) then

      if(x < ONE_f4) then
         vertadvu(k+1) = (vtrans(k+1)*com2)*((ar(k)-0.5_f4*dela(k)*x +  &
              (x*0.5_f4-x**2*ONE_THIRD_f4)*a6(k))/cvert(k))

! If Courant # > 1, use upwind advection:

      else
         vertadvu(k+1) = vtrans(k+1)
      endif

! Downward transport rate:

   elseif(vtrans(k+1) < ZERO_f4) then

      if(x > -ONE_f4) then
         vertadvd(k+1) = (-vtrans(k+1)*com2)*((al(k+1)+0.5_f4*dela(k+1)*xpos +  &
              (xpos*0.5_f4-xpos**2*ONE_THIRD_f4)*a6(k+1))/cvert(k+1))
      else
         vertadvd(k+1) = -vtrans(k+1)
      endif

   endif

enddo

! Lower boundary transport rates:  If I_OPEN_FLUX boundary
!  condtion is selected, then use concentrations in first
!  and second layers to extrapolate concentration in layer
!  just below the model lower boundary:

if(ibbnd == I_OPEN_FLUX) then

!   com2 = (dz(1)+dz(2))*0.5
   com2 = dzkkp1(1)*0.5_f4
   x = vtrans(1)*dtime/dz(1)
   xpos = abs(x)
   cvert0 = cvert(1)*(cvert(1)/cvert(2))
   if(vtrans(1) > ZERO_f4) then

      if(x < ONE_f4) then
         vertadvu(1) = vtrans(1)/cvert0*com2*(ar(1)-0.5_f4*dela(1)*x +  &
              (x*0.5_f4-x**2*ONE_THIRD_f4)*a6(1))
      else
         vertadvu(1) = vtrans(1)
      endif

   elseif(vtrans(1) < ZERO_f4) then
      
      if(x > -ONE_f4) then
         vertadvd(1) = -vtrans(1)/cvert(1)*com2*(al(1)+0.5_f4*dela(1)*xpos +  &
              (xpos*0.5_f4-xpos**2*ONE_THIRD_f4)*a6(1))
      else
         vertadvd(1) = -vtrans(1)
      endif

   endif

endif

! Upper boundary transport rates:

if(itbnd == I_OPEN_FLUX) then

!   com2 = (dz(nz-1)+dz(nz))*0.5_f4
   com2 = dzkkp1(nz-1)*0.5_f4
   x = vtrans(nz+1)*dtime/dz(nz)
   xpos = abs(x)
   cvertnzp1 = cvert(nz)*(cvert(nz)/cvert(nz-1))
   cvertnzp1 = max(cvertnzp1,cvert(nz))

   if(vtrans(nz+1) > ZERO_f4) then

      if(x < ONE_f4) then
         vertadvu(nz+1) = vtrans(nz+1)/cvert(nz)*com2  &
              *(ar(nz)-0.5_f4*dela(nz)*x+(x*0.5_f4-x**2*ONE_THIRD_f4)*a6(nz))
      else
         vertadvu(nz+1) = vtrans(nz+1)
      endif

   elseif(vtrans(nz+1) < ZERO_f4) then

      if(x > -ONE_f4) then
         vertadvd(nz+1) = -vtrans(nz+1)/cvertnzp1*com2  &
              *(al(nz)+0.5_f4*dela(nz)*xpos+(xpos*0.5_f4-xpos**2*ONE_THIRD_f4)*a6(nz))
      else
         vertadvd(nz+1) = -vtrans(nz+1)
      endif

   endif

endif

end subroutine vert_advect


!```````````````````````````````````````````````````````````````````
subroutine vert_diffuse

!+ Calculates vertical diffusion rates

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use aermicro_mem_mod, only: nz_mic,dz,igridv,I_CART,I_SIG,rhoa,vertdifu  &
     ,vertdifd,dkz,ibbnd,itbnd,I_FLUX_SPEC,I_DIFFUSION,POWMAX,ALMOST_ONE  &
     ,ALMOST_ZERO
     
implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! Not necessarily generalized for irregular grid.
!
!  <vertdifu(k)> is upward vertical transport rate into level k from level k-1.
!  <vertdifd(k)> is downward vertical transport rate into level k-1 from level k

integer :: k
real(f4) :: dz_avg,xex,ttheta,rhofact

!=== Executable statements:

! Loop over vertical levels:

do k=2,nz_mic

   dz_avg = dz(k)                            ! layer thickness

! Check the vertical coordinate:
  
   if(igridv == I_CART) then
      
      rhofact = log(rhoa(k)/rhoa(k-1))
      xex = rhoa(k-1)/rhoa(k)
      vertdifu(k) = (rhofact*dkz(k)/dz_avg)/(1.-xex)
      vertdifd(k) = vertdifu(k)*xex

   elseif(igridv == I_SIG) then      !  ...else you're in sigma coordinates...
  
      vertdifu(k) = dkz(k)/dz_avg
      vertdifd(k) = dkz(k)/dz_avg

   endif

enddo

! Fluxes at boundaries specified by user:

if(ibbnd == I_FLUX_SPEC) then
   vertdifu(1) = ZERO_f4
   vertdifd(1) = ZERO_f4
endif

if(itbnd == I_FLUX_SPEC) then
   vertdifu(nz_mic+1) = ZERO_f4
   vertdifd(nz_mic+1) = ZERO_f4
endif

! Diffusion through boundaries:

if(ibbnd == I_DIFFUSION) then
   dz_avg = dz(1)                            ! layer thickness
   rhofact = log(rhoa(2)/rhoa(1))
   ttheta = rhofact
   if(ttheta >= ZERO_f4) then
      ttheta = min(ttheta,POWMAX)
   else
      ttheta = max(ttheta,-POWMAX)
   endif

   xex = exp(-ttheta)
   if(abs(ONE_f4-xex) < ALMOST_ZERO) then
      xex = ALMOST_ONE
   endif

   vertdifu(1) = (rhofact*dkz(1)/dz_avg)/(ONE_f4-xex)
   vertdifd(1) = vertdifu(1)*xex
endif

if(itbnd == I_DIFFUSION) then
   dz_avg = dz(nz_mic)                            ! layer thickness
   rhofact = log(rhoa(nz_mic)/rhoa(nz_mic-1))
   ttheta = rhofact
   if(ttheta >= ZERO_f4) then
      ttheta = min(ttheta,POWMAX)
   else
      ttheta = max(ttheta,-POWMAX)
   endif

   xex = exp(-ttheta)
   if(abs(ONE_f4-xex) < ALMOST_ZERO) then
      xex = ALMOST_ONE
   endif

   vertdifu(nz_mic+1) = (rhofact*dkz(nz_mic+1)/dz_avg)/(ONE_f4-xex)
   vertdifd(nz_mic+1) = vertdifu(nz_mic+1)*xex
endif

end subroutine vert_diffuse


!```````````````````````````````````````````````````````````````````
subroutine vert_solvetx

!+ Solves the vertical transport equation

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use aermicro_mem_mod, only: dz,dtime,vertdifu,vertdifd,vertadvu,vertadvd,cvert  &
     ,divcor,igridv,I_SIG,itbnd,ibbnd,I_FLUX_SPEC,ftop,fbot,nz_mic,time,grid &
     ,i_curr,j_curr  &
     ,al => fscr1  &       ! [nz_mic]
     ,bl => fscr2  &      ! [nz_mic]
     ,dl => fscr3  &      ! [nz_mic]
     ,ul => fscr4  &      ! [nz_mic]
     ,el => fscr5  &      ! [nz_mic]
     ,fl => fscr6  &      ! [nz_mic]
     ,ctempl => fscr7  &      ! [nz_mic]
     ,ctempu => fscr8      ! [nz_mic]

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! <cvert> is temporary storage for concentrations (particles,
!  gas, potential temperature) being transported.
!  New values of <cvert> are calculated.

integer :: k,nz
real(f4) :: uc,cour,denom

!=== Executable statements:

nz = nz_mic

! Determine whether transport should be solved explicitly (uc=0) or
!  implicitly (uc=1):

uc = ZERO_f4
do k=1,nz
   cour = dz(k)/dtime-(vertdifu(k+1)+vertdifd(k)+vertadvu(k+1)+vertadvd(k))
   if (cour < ZERO_f4) then
      uc = ONE_f4; exit   ! Should be solved implicitly
   end if
enddo

! Store concentrations in local variables (shifted up and down a vertical
!  level):

do k=2,nz
   ctempl(k) = cvert(k-1)
   ctempu(k-1) = cvert(k)
enddo

!?if(ibbnd == I_DIFFUSION) then
!?   ctempl(1) = ctempl(2)
!?else
ctempl(1) = ZERO_f4
!?endif
!?if(itbnd == I_DIFFUSION) then
!?   ctempu(nz) = ctempu(nz-1)
!?else
ctempu(nz) = ZERO_f4
!?endif

! Calculate coefficients of the transport equation
!  [ al(k)c(k+1) + bl(k)c(k) + ul(k)c(k-1) = dl(k) ]:

do k=1,nz
   al(k) = uc*(vertdifd(k+1)+vertadvd(k+1))
   bl(k) = -(uc*(vertdifd(k)+vertdifu(k+1)+vertadvd(k)+vertadvu(k+1))  &
        +dz(k)/dtime)
   ul(k) = uc*(vertdifu(k)+vertadvu(k))
   dl(k) = cvert(k)*((ONE_f4-uc)*(vertdifd(k)+vertdifu(k+1)+  &
        vertadvd(k)+vertadvu(k+1)) - dz(k)/dtime)-  &
        (ONE_f4-uc)*((vertdifu(k)+vertadvu(k))*ctempl(k) +  &
        (vertdifd(k+1)+vertadvd(k+1))*ctempu(k)) - divcor(k)*dz(k)
enddo

! Boundary fluxes: <ftop> is the downward flux across the
!  upper boundary; <fbot> is the upward flux across the
!  lower boundary:

if(igridv == I_SIG) then
   if(itbnd == I_FLUX_SPEC) then
      dl(1) = dl(1)-ftop
   endif
   if(ibbnd == I_FLUX_SPEC) then
      dl(nz) = dl(nz)-fbot
   endif
else
   if(itbnd == I_FLUX_SPEC) then
      dl(nz) = dl(nz)-ftop
   endif
   if(ibbnd == I_FLUX_SPEC) then
      dl(1) = dl(1)-fbot
   endif
endif

! Calculate recursion relations:

el(1) = dl(1)/bl(1)
fl(1) = al(1)/bl(1)
do k=2,nz
   denom = bl(k)-ul(k)*fl(k-1)
   el(k) = (dl(k)-ul(k)*el(k-1))/denom
   fl(k) = al(k)/denom
enddo

! Calculate new concentrations:

cvert(nz) = el(nz)
do k=nz-1,1,-1
   cvert(k) = el(k)-fl(k)*cvert(k+1)
enddo

end subroutine vert_solvetx


end module aemic_vertTX_mod
