!```````````````````````````````````````````````````````````````````
module aermicro_mem_mod

!+ Module that contains memory structures common to aerosol microphysics
!   code routines

use univ_kind_defs_mod, only: f4, f8
use univ_math_constants_mod, only: ONE_f8, pi_f4
use univ_phys_constants_mod, only: avogadnum_f4, boltzmann_f8, R_univ_gas_f8
use univ_convert_units_mod, only: deg_to_rad_f4, rad_to_deg_f4, m_to_cm_f4,  &
     kg_to_g_f4, J_to_erg_f4, kg_m3_to_g_cm3_f4, Pa_to_dyne_o_cm2_f4,  &
     J_to_erg_f8, mb_to_Pa_f8, Pa_to_dyne_o_cm2_f8
use univ_numtype_constants_mod, only: epsilon_f4
use body_specific_constants_mod, only: g_Xoid, radius, R_air, p00, cp,  &
     rho_CO2ice

implicit none

! Primary array dimensions:

integer :: nz_mic,nelements,ngroups,ngasses,nsolutes,nz1_mic
integer, allocatable, dimension(:) :: nbins    ! [ngroups]
integer :: max_nbins,max_dim

! Unit conversion factors [SI (mks) -> CGS system]:

real(f4), parameter ::  &
                   Pa_to_dyne_o_cm2 = Pa_to_dyne_o_cm2_f4  &    ! Pressure [Pa -> dyne/cm^2]
                  ,J_to_erg = J_to_erg_f4  &     ! Energy [J -> erg]
                  ,kg_to_g = kg_to_g_f4  &    ! Mass [kg -> g]
                  ,m_to_cm = m_to_cm_f4  &    ! Distance [m -> cm]
                  ,kg_m3_to_g_cm3 = kg_m3_to_g_cm3_f4  ! Density [kg/m^3 -> g/cm^3]

! Define 1.0 to be multiplied by literal constants that are embedded within
!  parentheses:

real(f4), parameter :: ONE = real(ONE_f8, kind=f4)

! Define smallest possible number such that ONE + ALMOST_ZERO > ONE:

real(f4), parameter :: ALMOST_ZERO = 1.01_f4*epsilon_f4

! Define value slightly < ONE to test for total evaporation [dimensionless]:

real(f4), parameter :: ALMOST_ONE = ONE-ALMOST_ZERO

! Define value for maximum exponential argument [dimensionless]:

real(f4), parameter :: POWMAX = real(85._f8, kind=f4)

! Define tolerance for series convergence in Mie calculations (should be 1.d-7
!  for 32 bit machines, regardless of implicit precision):

real(f4), parameter :: EPSILON_MIE = real(1.e-7_f8, kind=f4)

! Define small particle number concentration [#/(x_units*y_units*z_units)]:

real(f4), parameter :: SMALL_PC = 7.e-15_f4 !18 !-25

! Define particle number concentration [#/cm^3] used to decide whether to
!  bypass microphysical processes (set it to SMALL_PC to never bypass the
!  calculations):

real(f4), parameter :: FEW_PC = SMALL_PC*1.e5_f4

! Define core fraction (for core mass and second moment) used when particle
!  number concentrations are limited to SMALL_PC

real(f4), parameter :: FIX_COREF = ONE*1.e-4_f4 

!%! Define # components of pc() in first 4 and 5 dimensions
!%!  (used for collapsing dimensions of <pc> in some calcs)
!%
!%integer(i4), parameter :: NPC4 = NXYZ * NBIN,  &
!%                            NPC5 = NPC4 * NELEM

! Define values of flag used for vertical transport boundary conditions:

integer, parameter :: I_DIFFUSION = 0,  &
                              I_FLUX_SPEC = 1,  &
                              I_OPEN_FLUX = 2

! Define values of flag used for particle element composition specification:

integer, parameter :: I_H2SO4 = 0,  &
                              I_DUST = 1,  &
                              I_H2OICE = 2,  &
                              I_CO2ICE = 3,  &
                              I_WATER = 4,  &
                              I_MIXEDWAT = 5

! Define values of flag used for particle element type specification:

integer, parameter :: I_INVOLATILE = 0,  &
                              I_VOLATILE = 1,  &
                              I_COREMASS = 2,  &
                              I_VOLCORE = 3,  &
                              I_CORE2MOM = 4

! Define values of flag used for nucleation process specification:

integer, parameter :: I_DROPACT = 0,  &
                              I_AERFREEZE = 1,  &
                              I_DROPFREEZE = 2,  &
                              I_MIXEDFREEZE = 3,  &
                              I_MIXEDMELT = 4,  &
                              I_ICEMELT = 5

! Define values of symbols used to specify vertical grid type. Grid selection is
!  made by defining the variable `igridv` to one of grid types known to the
!  model:
!
!   Possible values for `igridv`:
!       I_CART    Cartesian
!       I_SIG     sigma

integer, parameter :: I_CART = 1,  &
                                  I_SIG = 2

! Define triple-point temperature [K]:

!%real(f4), parameter :: T0 = 273.16_f8 ! For H2O at (other conditions???)
real(f4), parameter :: T0 = real(221.0_f8, kind=f4) ! For CO2 at (other conditions???)

! Define PI for use in aerosol microphysics code [unitless]:

real(f4), parameter :: pi_m = pi_f4

! Define convenient angle measure conversion factors [unitless]:

real(f4), parameter :: deg2rad = deg_to_rad_f4,  &
                               rad2deg = rad_to_deg_f4

! Define acceleration of gravity near Mars surface [cm/s^2]:

real(f4), parameter :: g_cgs = g_Xoid*m_to_cm

! Define Mars planetary radius [cm]:

real(f4), parameter :: mrad_cgs = radius*m_to_cm

! Define Avogadro's number [#particles/mole]:

real(f4), parameter :: avog = avogadnum_f4

! Define Boltzmann's constant [erg/K]:

real(f4), parameter :: BK = real(boltzmann_f8*J_to_erg_f8, kind=f4)

! Define Loschmidt's number [mole/cm^3, @STP]

! parameter( ALOS = 2.68719e19_f4 )    ! Earth
real(f4), parameter :: ALOS = 1.956e17_f4   ! Mars

! Define universal gas constant [erg/(K.mole)]:

real(f4), parameter :: R_univ_cgs = real(R_univ_gas_f8*J_to_erg_f8, kind=f4)

! Define molecular weight of dry air [g/mole]:

real(f4), parameter :: WTMOL_AIR = R_univ_cgs/(R_air*J_to_erg/kg_to_g)
                                        !43.303646???

! Define reference pressure (e.g. for potential temp calcs) [dyne/cm^2]:

real(f4), parameter :: PREF = p00*Pa_to_dyne_o_cm2

! Define conversion factor for mb to cgs [dyne/cm^2] units:

real(f4), parameter :: RMB2CGS = real(mb_to_Pa_f8*Pa_to_dyne_o_cm2_f8, kind=f4)

! Define gas constant for dry air [erg/(K.mole)]:

real(f4), parameter :: R_air_cgs = R_univ_cgs/WTMOL_AIR    

! Define specific heat at constant pressure of dry air [cm^2/(K.s^2)]:

real(f4), parameter :: cp_cgs = cp*J_to_erg/kg_to_g

! Define constant for potential temperature <-> temperature conversion [none]:

real(f4), parameter :: rkappa = R_air_cgs/cp_cgs

! Define mass density of liquid water [g/cm^3]:

real(f4), parameter :: RHO_W = real(1._f8, kind=f4)

! Define mass density of water ice [g/cm^3]:

real(f4), parameter :: RHO_I = real(0.93_f8, kind=f4)

! Define mass density of CO2 ice [g/cm^3]:

real(f4), parameter :: RHO_IC = rho_CO2ice*kg_m3_to_g_cm3

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for model startup control
!   (These variables are defined fresh for each new run, both
!    for cold starts and restarts, and do not get dumped into
!    the output restart file at the end of a run.  They are
!    all defined at the beginning of the init routine.  They
!    control the type of initialization that will be performed,
!    as well as control things that can change from run to run
!    within a single simulation.)
!
!X   ibtime    Beginning timestep index for this run 
!X   ietime    Ending timestep index for this run 
!X   endtime   Total simulation time for this run
!X   nprint    # of timesteps between print reports (used when > 0)
!X   nhist     # of timesteps between history output (used when > 0)
!X   nrest     # of timesteps between restart output (used when > 0)
!X   pprint    time period between print reports  (used when nprint < 0)
!X   phist     time period between history outputs (used when nhist < 0)
!X   prest     time period between restart outputs (used when nrest < 0)
!X   khist     Counter for # of history timepoints output this run
!X   prtofil   Name of output print file
!X   resifil   Name of input restart file
!X   resofil   Name of output restart file
!X   hisofil   Name of output history file
!X   stepofil  Name of time-step diagnostics file
!X   radofil   Name of radiation submodel print output file
!X   do_print  .t. if print output during timestepping is desired
!X   do_hist   .t. if history output during timestepping is desired
!X   do_rest   .t. if restart output during timestepping is desired

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for model grid
!
!X   dom_llx    Domain limit, lower left x coordinate                 {initatm}
!X   dom_lly    Domain limit, lower left y coordiante                 {initatm}
!X   dom_urx    Domain limit, upper right x coordinate                {initatm}
!X   dom_ury    Domain limit, upper right y coordinate                {initatm}
!X   rlon0      center longitude for LC, PS, LL projections           {initatm}
!X   rlat0      true latitude for ME projection                       {initatm}
!X   rlat1      #1 true latitude for LC projection                    {initatm}
!X   rlat2      #2 true latitude for LC projection                    {initatm}
!X   hemisph    +1.=southern, -1.=northern hemisphere for PS, LC proj {initatm}
!*   igridv     flag to specify desired vertical grid coord system    {initatm}
!X   igridh     flag to specify desired horizontal grid coord system  {initatm}
!#   zl         Altitude at top of layer                              {initatm}
!   zlold      Altitude at top of layer at start of time step
!#   zc         Altitude at layer mid-point                           {initatm}
!   zcold      Altitude at layer mid-point at start of time step
!X   xc         Horizontal position at center of box                  {initatm}
!X   yc         Horizontal position at center of box                  {initatm}
!X   xl         Horizontal position at lower edge of box              {initatm}
!X   yl         Horizontal position at lower edge of box              {initatm}
!X   xu         Horizontal position at upper edge of box              {initatm}
!X   yu         Horizontal position at upper edge of box              {initatm}
!X   dx         Horizontal grid spacing                               {initatm}
!X   dy         Horizontal grid spacing                               {initatm}
!#   dz         Thickness of vertical layers                          {initatm}
!X   xmet       Horizontal ds/dx (ds is metric distance)              {initatm}
!X   ymet       Horizontal ds/dy (ds is metric distance)              {initatm}
!#   zmet       Vertical ds/dz (ds is metric distance)                {initatm}
!#   zmetl      Vertical ds/dz at beginning of time-step              
!X   rlon,rlat  Longitude, latitude [deg] at each horiz grid point    {initatm}
!X   gridname   Text description of horiz & vert grid coord system    {initatm}
!   iaer1      Safety marker for common block aer1

!%character(len=5) :: caer1s

integer :: ik

real(f4), allocatable, dimension(:) :: zl,zl_static    ! [nz1_mic]
real(f4), allocatable, dimension(:) :: zc,zc_static,dz    ! [nz_mic]

!%real, dimension(NXYZP1) :: zlold
!%real, dimension(NXYZ) :: zcold
!%integer :: iaer1

integer :: igridv

integer :: i_curr,j_curr
      
!%common /aer1/  &
!%     ,zlold,zcold,,,zmetl  &
!%     ,iaer1

!%common /aer1s/ caer1s

!Declare alias names for grid stuff with first 2, 3 dimensions treated linearly

!%real, dimension(NXY,NZ) :: ,  &
!%     ,zmetl2
!%real, dimension(NXYZ) ::  ,  &
!%     ,zmetl3

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for model option & control variables
!
!*   time        Simulation time at end of current timestep [s]
!*   dtime       Timestep size [s]
!*   dtmin       Minimum time-step
!*   dtmax       Maximum time-step
!*   dpctol      Maximum change in particle concentrations that will be tolerated
!*   dgstol      Maximum change in gas concentrations that will be tolerated
!*   conmax      Minumum relative concentration to consider in varstep   {prestep}
!X   itime       Timestep index at end of current timestep 
!#   igelem      Groups to which elements belong                     {setupbins}
!#   itype       Particle type specification array                   {setupbins}
!#   icomp       Particle compound specification array               {setupbins}
!#   nelemg      Number of elements in group           
!#   ncore       Number of core elements (itype = 2) in group           
!#   icorelem    Core elements (itype = 2) in group           
!#   ienconc     Particle number conc. element for group             {setupbins}
!#   ishape      Describes particle shape for group
!#   icoag       Coagulation mapping array                           {setupcoag}
!#   icoagelem   Coagulation element mapping array                   {setupcoag}
!*   ifall       Fall velocity options                               {setupvfall}
!*   icoagop     Coagulation kernel options                          {setupckern}
!*   icollec     Gravitational collection options                      {setupckern}
!*   itbnd       Top boundary condition                                {vertical}
!*   ibbnd       Bottom boundary condition                             {vertical}
!*   itbnd_pc    Top boundary condition flag for particles             {init}
!*   ibbnd_pc    Bottom boundary condition flag for particles          {init}
!*   itbnd_gc    Top boundary condition flag for gas                   {init}
!*   ibbnd_gc    Bottom boundary condition flag for gas                {init}
!*   itbnd_ptc   Top boundary condition flag for potential temp.       {init}
!*   ibbnd_ptc   Bottom boundary condition flag for potential temp.    {init}
!X   ihoradv     Specification of horizontal advection algorithm       {init}
!*   do_coag     If .true. then do coagulation                         {init}
!*   do_grow     If .true. then do condensational growth and evap.     {init}
!*   do_thermo   If .true. then do solve thermodynamics equation       {init}
!*   do_vtran    If .true. then do vertical transport                  {init}
!X   do_ew       If .true. then do east-west transport                 {init}
!X   do_ns       If .true. then do north-south transport               {init}
!   do_ccoef    If .true. then calculate coefficients for htran       {htranglk}
!*   do_varstep  If .true then use variable time-step                  {init}
!*   do_step     If .false. then step was too big, so don't advance    {varstep}
!X   do_error    If .true. then do error trapping for debugging        {init}
!X   do_netcdf   If .true. then output history in netcdf file format   {init}
!X   do_parcel   If .true. then do parcel simulation                   {init}
!   ncdf_file   Netcdf file handle integer for internal netcdf use    {outhis_ncdf}
!#   sec_mom     If .true. then core second moment (itype = 3) used    {setupgrow}
!#   igrowgas    Gas that condenses into a particle element            {setupgrow}
!#   inucgas     Gas that nucleates a particle group                   {setupnuc}
!#   if_nuc      Nucleation conditional array                          {setupaer}
!#   if_nuc_lh   Conditional array for nucleation latent heat          {setupaer}
!#   rlh_nuc     Nucleation latent heat                                {setupaer}
!#   inucproc    Nucleation conditional array                          {setupaer}
!#   nnuc2elem   Number of elements that nucleate to element           {setupnuc}
!#   inuc2elem   Nucleation transfers particles into element inuc2elem {setupnuc}
!#   ievp2elem   Total evap. transfers particles into group ievp2elem  {setupnuc}
!   ievp2bin    Total evap. transfers particles into bin ievp2bin     {setupnuc}
!#   inuc2bin    Nucleation transfers particles into bin inuc2bin      {setupnuc}
!#   isolelem    Index of solute for each particle element             {setupnuc}
!   ix          Current index for spatial grid, general east-west direction
!   iy          Current index for spatial grid, general north-south direction
!   iz          Current index for spatial grid, vertical direction
!   ixy         Current index for spatial grid, linearized 2-D ix,iy (horizontal)
!   ixyz        Current index for spatial grid, linearized 3-D ix,iy,iz 
!*   ntsubsteps  Number of time substeps for fast microphysics processes
!*   maxsubsteps Maximum number of time substeps allowed
!*   minsubsteps Maximum number of time substeps allowed
!   iaer2       Safety marker for common block aer2
!
!
!X   simtitle   Model simulation title string
!#   elemname   Names of particle elements
!#   groupname  Names of particle elements
!#   gasname    Names of gas species
!#   solname    Names of solutes
!   caer2s     Safety marker for common block aer2s 

logical :: do_coag,do_grow,do_thermo,gc_vTX_on,ptc_vTX_on,pc_vTX_on  &
     ,do_varstep,do_step,gc_extTX_on,ptc_extTX_on,pc_extTX_on

logical, allocatable, dimension(:) :: should_vTX_ie    ! [nelements]

logical :: loop_again
integer :: loop_cnt,lpcnt_col,grid



!%logical ::   &
!%           ,do_ccoef
!%character(len=5) :: caer2s

logical, allocatable, dimension(:) :: is_grp_ice,is_grp_mixed  &   ! [ngroups]
     ,sec_mom
logical, allocatable, dimension(:,:) :: if_nuc  &    ! [nelements,nelements]
     ,if_nuc_lh
character(len=30), allocatable, dimension(:) :: solname     ! [nsolutes]
character(len=30), allocatable, dimension(:) :: gasname     ! [ngasses]
character(len=30), allocatable, dimension(:) :: groupname       ! [ngroups]
character(len=60), allocatable, dimension(:) :: elemname     ! [nelements]

real(f4) :: dtime,dtmin,dtmax,time,time_nuc,period_nuc,dtime_save
real(f4) :: conmax,dpctol,dgstol

integer :: itbnd_pc,ibbnd_pc,itbnd_gc,ibbnd_gc,itbnd_ptc,ibbnd_ptc
integer :: maxsubsteps,minsubsteps,ntsubsteps
integer :: ifall,icoagop,icollec

integer,  allocatable, dimension(:) :: igelem,icomp  &   ! [nelements]
     ,itype,igrowgas,ievp2elem,isolelem
integer,  allocatable, dimension(:) :: ienconc,ishape  &    ! [ngroups]
     ,ncore,imomelem

real(f4), allocatable, dimension(:,:) :: rlh_nuc  ! [nelements,nelements]

integer :: itbnd,ibbnd

!%integer :: ix,iy,iz,ixy,ixyz,  &
!%           ncdf_file,iaer2

!%integer, dimension(NBIN,NGROUP,NGROUP) :: ,ievp2bin

integer, allocatable, dimension(:,:) ::  &
     icoagelem                            ! [nelements,ngroups]

real(f4), allocatable, dimension(:) :: sifreeze    ! [ngasses]

integer, allocatable, dimension(:,:,:) ::  & 
     inuc2bin                        ! [max_nbins,ngroups,ngroups]
integer, allocatable, dimension(:,:,:,:) ::  & 
     inucbin                   ! [max_nbins*ngroups,ngroups,max_nbins,ngroups]
integer, allocatable, dimension(:,:,:) ::  & 
     nnucbin                        ! [ngroups,max_nbins,ngroups]

integer, allocatable, dimension(:,:) ::  &
     inucelem                           ! [nelements,nelements*ngroups]     
integer, allocatable, dimension(:,:) :: icoag    ! [ngroups,ngroups]
integer, allocatable, dimension(:,:) :: icorelem ! [nelements,ngroups]
integer, allocatable, dimension(:,:) ::   &    ! [nelements,nelements]
     inucproc
integer, allocatable, dimension(:,:) :: inuc2elem     ! [10,nelements]
integer, allocatable, dimension(:,:) :: inucgas ! [nelements,ngroups]
integer, allocatable, dimension(:) :: nelemg     ! [ngroups]
integer, allocatable, dimension(:) :: nnuc2elem,nnucelem ! [nelements]

real(f4), allocatable, dimension(:,:,:) ::  & 
     ge_pcfitconst                        ! [max_nbins,ngroups,8]

     
!%      common /aer2/  &
!%      dtime_save,  &
!%        ix, iy, iz, ixy, ixyz, ntsubsteps,  &
!%       ishape,  icoag,  &
!%          itbnd,  &
!%       ibbnd,     &
!%           do_ccoef,   &
!%         ncdf_file,  &
!%       isolelem, ievp2bin, &
!%       iaer2


!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for particle grid structure
!
!#   rmin      Radius of particle in first bin [cm] 
!#   rmassmin  Mass of particle in first bin [g]
!#   rmrat     Ratio of masses of particles in consecutive bins  {setupaer}
!#   r         Radius bins [cm]
!#   rmass     Mass bins [g]
!#   vol       Particle volume [cm^3]
!#   dr        Width of bins in radius space [cm]
!#   dm        Width of bins in mass space [g]
!   dv        Width of bins in volume space [cm^3]
!#   rmassup   Upper bin boundary mass [g]
!#   rup       Upper bin boundary radius [cm]
!#   rlow      Lower bin boundary radius [cm]
!#   diffmass  Difference between <rmass> values
!#   rhop      Mass density of particle groups [g/cm^3]
!#   rhoelem   Mass density of particle elements [g/cm^3]
!#   eshape    Ratio of particle length / diameter 
!   iaer3     Safety marker for common block aer3
!

!%real, dimension(NBIN,NGROUP) :: dv
!%real, dimension(NX,NY,NZ,NBIN,NGROUP) :: rhop
!%integer :: iaer3

real(f4), allocatable, dimension(:,:,:,:) ::  &
     diffmass                 ! [max_nbins,ngroups,max_nbins,ngroups]
real(f4), allocatable, dimension(:,:) :: rmass  &    ! [max_nbins,ngroups]
     ,rmassup,dm,dmi,vol,r,dr,rup,rlow

real(f4), allocatable, dimension(:,:) ::  &
     small_coremconc                       ! [max_nbins,nelements]

real(f4), allocatable, dimension(:,:,:) ::  &
     rhop                            ! [nz_mic,max_nbins,ngroups]
real(f4), allocatable, dimension(:) :: rmin,rmrat,eshape  &   ! [ngroups]
     ,rmassmin
real(f4), allocatable, dimension(:) :: rhoelem       ! [nelements]

!%common /aer3/  &
!%      dv, &
!%     iaer3

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for primary model state variables
!
!#   pc          Particle concentration                             {initaer}
!#   gc          Gas concentration [g/x_units/y_units/z_units]      {initgas}
!#   ptc         Potential temperature concentration [g K/x_units/y_units/z_units]
!   iaer4       Safety marker for common block aer4
!

! Primary arrays:

! Aerosol particle concentrations [#/cm^3] or [g/cm^3]:
real(f4), allocatable, dimension(:,:,:) :: pc  ! [nz_mic,nbins,nelements]

real(f4), allocatable, dimension(:,:,:) ::   &
     sadv_pc_tend               ! [nz_mic,nbins,nelements]
real(f4), allocatable, dimension(:) ::   &
     sadv_ptc_tend               ! [nz_mic]
real(f4), allocatable, dimension(:,:) ::   &
     sadv_gc_tend               ! [nz_mic,ngasses]

real(f4), allocatable, dimension(:) :: ptc      ! [nz_mic]

real(f4), allocatable, dimension(:,:) :: gc  ! [nz_mic,ngasses]


!  Declare alias names for pc() and rhop() with first 2, 3, 4, 5 dimensions
!  treated linearly

!%real, dimension(NXY,NZ,NBIN,NELEM) :: pc2
!%real, dimension(NXYZ,NBIN,NELEM) ::  pc3
!%real, dimension(NPC4,NELEM) :: pc4
!%real, dimension(NPC5) :: pc5

!%equivalence( pc2, pc )
!%equivalence( pc3, pc )
!%equivalence( pc4, pc )
!%equivalence( pc5, pc )

!  Declare alias name for gc() and ptc() with first 2, 3, 4, 5 dimensions treated linearly

!%real, dimension(NXY,NZ,NGAS) :: gc2
!%real, dimension(NXYZ,NGAS) :: gc3

!%      equivalence( gc2, gc )
!%      equivalence( gc3, gc )

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for secondary model variables
!
!#   pcl         Particle concentration at beginning of time-step
!#   pcmax       Maximum concentration for each element             {prestep}
!#   pconmax     Maximum particle concentration for each grid point
!#   gcl         Gas concentration at beginning of time-step
!#   ptcl        Potential temperature concentration at beginning of time-step
!#   d_pc        Change in particle concentration due to transport
!#   d_gc        Change in gas concentration due to transport
!#   d_ptc       Change in potential temperature concentration due to transport
!#   cvert       Temporary storage for vertical transport 
!#   divcor      Correction term for vertical divergence
!   chor        Temporary storage for horizontal transport
!   dhor        Temporary storage of grid spacing for horizontal transport
!#   coaglg      Total particle loss rate due to coagulation for group
!#   coagpe      Particle production due to coagulation 
!#   rnuclg      Total particle loss rate due to nucleation for group
!#   rnucpe      Particle production due to nucleation 
!#   growlg      Total particle loss rate due to growth for group
!   growle      Partial particle loss rate due to growth for element 
!#   growpe      Particle production due to growth 
!#   evaplg      Total particle loss rate due to evaporation for group
!   evapls      Partial particle loss rate due to evaporation for element
!#   evappe      Particle production due to evaporation
!#   gasprod     Gas production term
!*   rlheat      Latent heating rate [deg_K/s]
!#   vertdifu    Upward vertical flux at grid boundary
!#   vertdifd    Downward vertical flux at grid boundary
!   ftopgas     Downward gas flux across top boundary of model
!   fbotgas     Upward gas flux across bottom boundary of model
!#   ftoppart    Downward particle flux across top boundary of model
!#   fbotpart    Upward flux particle across bottom boundary of model
!*   ftop        Downward flux across top boundary of model
!*   fbot        Upward flux across bottom boundary of model
!#   cmf         Core mass fraction in a droplet 
!#   totevap     .true. if droplets are totally evaporating to CN
!#   inucmin     Index of smallest particle nucleated; used for tot. evap.
!#   inucstep    Index of smallest particle nucleated during time step
!   iaer5       Safety marker for common block aer5
!

real(f4), allocatable, dimension(:) :: pcmax    ! [nelements]
real(f4), allocatable, dimension(:,:,:) ::  &
     pcl,coagpe,d_pc                         ! [nz_mic,max_nbins,nelements]
real(f4), allocatable, dimension(:,:) :: pconmax    ! [nz_mic,ngroups]
real(f4), allocatable, dimension(:,:) :: gcl,d_gc    ! [nz_mic,ngasses]
real(f4), allocatable, dimension(:) :: ptcl,d_ptc  &   ! [nz_mic]
     ,cvert,drho_dt,divcor
real(f4), allocatable, dimension(:,:,:) ::  &
     coaglg                         ! [nz_mic,max_nbins,ngroups]

real(f4) :: ftop,fbot,rlheat
real(f4), allocatable, dimension(:) :: ftopgas,fbotgas  ! [ngasses]

real(f4), allocatable, dimension(:,:,:) ::  &
     rnuclg                         ! [max_nbins,ngroups,ngroups]
real(f4), allocatable, dimension(:,:) ::  rnucpe  &
     ,growpe,evappe                       ! [max_nbins,nelements]
real(f4), allocatable, dimension(:,:) ::  growlg  &
     ,evaplg                          ! [max_nbins,ngroups]

real(f4), allocatable, dimension(:) :: fscr1,fscr2,fscr3,fscr4,fscr5  &
     ,fscr6,fscr7,fscr8                                     ! [max_dim]
integer, allocatable, dimension(:) :: iscr1,iscr2,iscr3    ! [max_dim]

logical, allocatable, dimension(:,:) :: totevap     ! [max_nbins,ngroups]
real(f4), allocatable, dimension(:,:) :: cmf     ! [max_nbins,ngroups]
real(f4), allocatable, dimension(:,:) :: rmshell  &
     ,vcore                                      ! [nz_mic,max_nbins]

!%real, dimension(NXORNY) :: chor,dhor
!%integer :: iaer5

real(f4), allocatable, dimension(:) :: gasprod     ! [ngasses]
real(f4), allocatable, dimension(:) :: vertdifd,vertdifu   ! [nz1_mic]
real(f4), allocatable, dimension(:,:) :: fbotpart  &
     ,ftoppart                        ! [max_nbins,nelements]
integer, allocatable, dimension(:) :: inucmin,inucstep   ! [ngroups]

!%common /aer5/  &
!%       chor,  &
!%     dhor,  &
!%     iaer5

!  Declare alias name for <evappe> to collapse into one dimension

!%real, dimension(NPC5,NGAS) :: evappe5

!%      equivalence( evappe5, evappe )

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for coagulation kernels and bin
!  pair mapping
!
!*   ck0           Constant coagulation kernel           {setupaer}
!*   grav_e_coll0  Constant value for collection effic.  {setupaer}
!#   ckernel       Coagulation kernels [/cm^3/s]         {setupckern}
!#   pkernel       Coagulation production variables      {setupcoag}
!#   volx          Coagulation subdivision variable      {setupcoag}
!#   ilow          Bin pairs for coagulation production  {setupcoag}
!#   jlow          Bin pairs for coagulation production  {setupcoag}
!#   iup           Bin pairs for coagulation production  {setupcoag}
!#   jup           Bin pairs for coagulation production  {setupcoag}
!#   npairl        Bin pair indices                      {setupcoag}
!#   npairu        Bin pair indices                      {setupcoag}
!   iaer6         Safety marker for common block aer6

real(f4) :: ck0,grav_e_coll0

!%integer :: iaer6
!%integer, dimension(NGROUP,NBIN) :: ,npair

integer, allocatable, dimension(:,:,:) :: ilow,jlow,iup  &
     ,jup             ! [ngroups,max_nbins,max_nbins*max_nbins]
integer, allocatable, dimension(:,:) :: npairl & ! [ngroups,max_nbins]
     ,npairu
real(f4), allocatable, dimension(:,:,:,:,:,:,:) ::  &
     pkernel       ! [nz_mic,max_nbins,max_nbins,ngroups,ngroups,ngroups,6]
real(f4), allocatable, dimension(:,:,:,:,:) ::  &
     ckernel                ! [nz_mic,max_nbins,max_nbins,ngroups,ngroups]
real(f4), allocatable, dimension(:,:,:,:,:) ::  &
     volx                   ! [ngroups,ngroups,ngroups,max_nbins,max_nbins]
 

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for coagulation group pair mapping
!
!   iglow      Group pairs for coagulation production  {setupcoag}
!#   iglow      Group pairs for coagulation production  {setupcoag}
!#   jglow      Group pairs for coagulation production  {setupcoag}
!#   igup       Group pairs for coagulation production  {setupcoag}
!#   jgup       Group pairs for coagulation production  {setupcoag}
!   iaer7      Safety marker for common block aer7

!%integer :: iaer7

integer, allocatable, dimension(:,:,:) :: iglow,jglow,igup  &
     ,jgup             ! [ngroups,max_nbins,max_nbins*max_nbins]

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for particle fall velocities, transport
!  rates, and coagulation kernels
!
!#   bpm       Corrects for non-sphericity and non-continuum effects {setupvfall}
!#   vf        Fall velocities at layer mid-pt                       {setupvfall}
!#   vtrans    Net vertical transport rate at layer boundary         {vertical}
!#   re        Reynolds' number based on <vfall>                     {setupvfall}
!*   vf_const  Constant vertical fall velocity when ifall=0          {setupaer}
!#   vertadvu  Upward mass transport rate due to advection	    {vertran}
!#   vertadvd  Downward mass transport rate due to advection	    {vertran}
!   htrans    Net horizontal transport rate at layer boundary       {horizont}
!   hdiff     Horizontal diffusion coefficient at layer boundary    {horizont}
!   ca        Coefficient for Galerkin horizontal transport         {glkcoef}}
!   cb        Coefficient for Galerkin horizontal transport         {glkcoef}}
!   cc        Coefficient for Galerkin horizontal transport         {glkcoef}}
!   cd        Coefficient for Galerkin horizontal transport         {glkcoef}}
!   ce        Coefficient for Galerkin horizontal transport         {glkcoef}}
!   cg        Coefficient for Galerkin horizontal transport         {glkcoef}}
!   iaer8     Safety marker for common block aer8


real(f4), allocatable, dimension(:,:,:) :: bpm  &
     ,re                             ! [nz_mic,max_nbins,ngroups]
real(f4), allocatable, dimension(:,:,:) ::  &
     vf                             ! [nz1_mic,max_nbins,ngroups]

real(f4), allocatable, dimension(:) :: vtrans  &  ! [nz1_mic]
     ,vertadvu,vertadvd

!%integer ::  iaer8

!%real, dimension(NXORNYP1) :: htrans
!%real, dimension(NXORNY) :: hdiff
!%real, dimension(2,NX,NY) :: ca,cb,cd,ce,cf,cg

real(f4) :: vf_const

!%      common /aer8/  &
!%        htrans, &
!%       hdiff, &
!%       ca, cb, cd, &
!%       ce, cf, cg, &
!%       iaer8

!  Declare alias names for <rhostar> with first 2, 3 dimensions treated linearly

!%      real :: rhostar   !????? Where is the array declaration for this??

!%      real, dimension(NXY,NZ) :: rhostar2
!%      real, dimension(NXYZ) :: rhostar3

!%      equivalence( rhostar2, rhostar )
!%      equivalence( rhostar3, rhostar )

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for atmospheric structure.
!  Note: air density, winds, and diffusion coefficients are in scaled units
!
!   zbot      height of the bottom of the model [cm]                  {initatm}
!#   p_surf    surface pressure [dyne/cm^2]                            {initatm}
!#   p_top     Atmospheric pressure at top of model domain [dyne/cm^2] {initatm}
!#   p         Atmospheric pressure at layer mid-pt [dyne/cm^2]        {initatm}
!#   rhoa      Air density at layer mid-pt [g/x_units/y_units/z_units] {initatm}
!#   t         Air temperature at layer mid-pt [deg_K]                 {initatm}
!X   t_surf    Air temperature at surface [deg_K]                     {initatm}
!#   rmu       Air viscosity at layer mid-pt [g/cm/s]                  {initatm}
!#   thcond    Thermal conductivity of dry air [erg/cm/sec/deg_K]      {initatm}
!#   w         Vertical wind speed at layer boundary [z_units/s]       {initatm}
!X   u         East-west wind speed at layer center [x_units/s]        {initatm}
!X   v         North-south wind speed at layer center [y_units/s]      {initatm}
!#   dkz       Vert diffusion coef at layer boundary [z_units^2/s]     {initatm}
!X   dkx       Horiz x diffusion coeff at layer boundary [x_units^2/s] {initatm}
!X   dky       Horiz y diffusion coeff at layer boundary [y_units^2/s] {initatm}
!#   told      Temperature at beginning of time-step
!#   pold      Pressure at beginning of time-step
!#   rhoaold   Air density at beginning of time-step
!   iaer9     Safety marker for common block aer9

!%integer ::  iaer9
!%real :: zbot
!%real, dimension(NXYZ) :: zmetold

real(f4), allocatable, dimension(:) :: dkz,w    ! [nz1_mic]
real(f4), allocatable, dimension(:) :: p,t,rhoa,rmu,thcond  &  ! [nz_mic]
     ,told,pold,rhoaold
real(f4) :: p_surf,p_top

!%      common /aer9/  &
!%       zbot,  , &
!%        zmetold, &
!%       iaer9

!  Declare alias names for atm stuff with first 2, 3 dimensions treated linearly

!%real :: zmetold2,zmetold3   !???? where are the array declarations for these?

!%      equivalence( zmetl2, zmetl )
!%      equivalence( zmetl3, zmetl )
!%      equivalence( zmetold2, zmetold )
!%      equivalence( zmetold3, zmetold )
!%      equivalence( p_top2, p_top )

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for condensational growth parameters
!
!#   gwtmol    Molecular weight for gases [g/mol]                  {setupgrow}
!#   diffus    Diffusivity of gas in air [cm^2/s]                  {setupgrow}
!#   rlhe      Latent heat of evaporation for gas [cm^2/s^2]       {setupgrow}
!#   rlhm      Latent heat of ice melting for gas [cm^2/s^2]       {setupgrow}
!   pvapl     Saturation vapor pressure over water [dyne/cm^2]    {vaporp}   
!   pvapi     Saturation vapor pressure over ice [dyne/cm^2]      {vaporp}   
!#   surfctwa  Surface tension of water-air interface              {setupgkern}
!#   surfctiw  Surface tension of water-ice interface              {setupgkern}
!#   surfctia  Surface tension of ice-air interface                {setupgkern}
!#   akelvin   Exponential arg. in curvature term for growth       {setupgkern}
!#   akelvini  Curvature term for ice                              {setupgkern}
!#   ft        Ventilation factor                                  {setupgkern}
!#   gro       Growth kernel [UNITS?]                              {setupgkern}
!#   gro1      Growth kernel conduction term [UNITS?]              {setupgkern}
!#   gro2      Growth kernel radiation term [UNITS?]               {setupgkern}
!   gvrat     For converting particle growth to gas loss [UNITS?] {setupgkern}
!#   supsatl   Supersaturation of vapor w.r.t. liquid water [dimless]
!#   supsati   Supersaturation of vapor w.r.t. ice [dimless]                  
!#   supsatlold Supersaturation (liquid) before time-step    {prestep}
!#   supsatiold Supersaturation (ice) before time-step    {prestep}
!#   scrit     Critical supersaturation for nucleation [dimless]   {setupnuc}
!#   sol_ions  Number of ions solute dissociates into              {setupnuc}
!#   solwtmol  Molecular weight of solute                          {setupnuc}
!#   rhosol    Mass density of solute                              {setupnuc}
!   iaer10   Safety marker for common block aer10

real(f4), allocatable, dimension(:) :: gwtmol     ! [ngasses]
real(f4), allocatable, dimension(:,:) :: diffus  &    ! [nz_mic,ngasses]
     ,rlhe,rlhm,surfctwa,surfctiw,surfctia,akelvin,akelvini,supsatlold  &
     ,supsatiold,supsatl,supsati,pvapl,pvapi
real(f4), allocatable, dimension(:,:,:) ::  & ! [nz_mic,max_nbins,ngroups]
     ft,gro,gro1,scrit
real(f4), allocatable, dimension(:,:) :: gro2   ! [nz_mic,ngroups]
real(f4), allocatable, dimension(:) :: gstick,tstick  &   ! [ngroups]
     ,cor,phish

!%integer :: iaer10
!%real, dimension(NBIN,NELEM,NGAS) :: gvrat
      
!%      common /aer10/  &
!%       gvrat, &
!%       iaer10

real(f4), allocatable, dimension(:) :: solwtmol,rhosol  &    ! [nsolutes]
     ,sol_ions

!  Declare alias names for pvapl(), pvapi(), supsatl(), and supsati() with
!  first 2, 3 dimensions treated linearly

!%real, dimension(NXY,NZ,NGAS) :: pvapl2,pvapi2,supsatl2,supsati2
!%real, dimension(NXYZ,NGAS) :: pvapl3,pvapi3,supsatl3,supsati3

!%      equivalence( pvapl2, pvapl )
!%      equivalence( pvapl3, pvapl )
!%     equivalence( pvapi2, pvapi )
!%      equivalence( pvapi3, pvapi )
!%      equivalence( supsatl2, supsatl )
!%      equivalence( supsatl3, supsatl )
!%      equivalence( supsati2, supsati )
!%      equivalence( supsati3, supsati )

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for nucleation parameters
!
!*   adelf    Coefficient for phase change activation energy  {freznuc}
!*   bdelf    Coefficient for phase change activation energy  {freznuc}
!*   prenuc   Pre-exponential factore for frezzing nucleation {freznuc}
!*   rmiv     Contact angle for ice/nucleus interface         {freznuc}
!   iaer11   Safety marker for common block rad1 
!

!%integer :: iaer11
real(f4) :: adelf,bdelf,prenuc,rmiv

!%      common /aer11/  &
!%        adelf, bdelf, prenuc, rmiv,  &
!%        iaer11 

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!
!  Declare global common blocks for radiative transfer parameters
!
!X   do_rad      If .true. then do radiative transfer                  {init}
!   do_solar    If .true. then do solar calculations                  {init}
!   do_ir       If .true. then do infrared calculations               {init}
!X   nrad        # of timesteps between radiation calcs (used when > 0)
!X   prad        Time period between radiation calcs (used when nrad < 0)
!   isolar_zen  =I_FIXED: fixed, =I_DIURNAL: calculated every time step
!   u0          cos( solar_zenith_angle )
!   u0_fixed    Fixed value of cos( solar_zenith_angle )
!   rad_start   solar time corresponding to <time> = 0 [s]
!   zsin,zcos   sin and cos terms for solar zenith angle precalculation
!   wave        Bin-center wavelengths [micron]
!   radheat     Radiative heating rate [deg_K/s]
!   qrad        Particle heating rate [deg_K/s]
!   alb_tomi    Spectrally-integrated albedo at top-of-model
!   alb_toai    Spectrally-integrated albedo at top-of-atmosphere
!   alb_toa     Spectrally-resolved albedo at top-of-atmosphere
!   opd         Spectrally-resolved optical depth
!   fsl_up      Solar upwelling flux [W m^-2]
!   fsl_dn      Solar downwelling flux [W m^-2]
!   fir_up      Infrared upwelling flux [W m^-2]
!   fir_dn      Infrared downwelling flux [W m^-2]
!   irad1       Safety marker for common block rad1 
!

!%logical :: do_solar,do_ir
!%integer :: isolar_zen,irad1
!%real :: u0,u0_fixed,rad_start 
!%real, dimension(NX,NY) :: zsin,zcos,alb_tomi,alb_toai
!%real, dimension(NWAVE) :: wave
!%real, dimension(NX,NY,NZ,NBIN,NGROUP) :: qrad
!%real, dimension(NX,NY,NZ) :: radheat
!%real, dimension(NX,NY,NSOL) :: alb_toa
!%real, dimension(NX,NY,NWAVE) :: opd
!%real, dimension(NX,NY,NZRADP1) :: fsl_up,fsl_dn,fir_up,fir_dn

!%      common /rad1/  &
!%       u0, u0_fixed, rad_start, &
!%       zsin, zcos, wave, &
!%       qrad, radheat,  &
!%       alb_tomi, alb_toai, &
!%       alb_toa, opd, &
!%       fsl_up, fsl_dn, &
!%       fir_up, fir_dn, &
!%        do_solar, do_ir,   isolar_zen,  &
!%       irad1

!  Declare alias names for radiative transfer parameters
!  with first 2, 3 dimensions treated linearly

!%real, dimension(NXY,NZ) :: radheat2
!%real, dimension(NXYZ) :: radheat3
!%real, dimension(NXY,NZ,NBIN,NGROUP) :: qrad2
!%real, dimension(NXYZ,NBIN,NGROUP) :: qrad3
!%real, dimension(NXY) :: alb_tomi2,alb_toai2
!%real, dimension(NXY,NSOL) :: alb_toa2
!%real, dimension(NXY,NWAVE) :: opd2
!%real, dimension(NXY,NZRADP1) :: fsl_up2,fsl_dn2,fir_up2,fir_dn2
!%real, dimension(NXYZRADP1) :: fsl_up3,fsl_dn3,fir_up3,fir_dn3

!%      equivalence( radheat2, radheat )
!%      equivalence( radheat3, radheat )
!%      equivalence( qrad2, qrad )
!%      equivalence( qrad3, qrad )
!%      equivalence( alb_tomi2, alb_tomi )
!%      equivalence( alb_toai2, alb_toai )
!%      equivalence( alb_toa2, alb_toa )
!%      equivalence( opd2, opd )
!%      equivalence( fsl_up2, fsl_up )
!%      equivalence( fsl_up3, fsl_up )
!%      equivalence( fsl_dn2, fsl_dn )
!%      equivalence( fsl_dn3, fsl_dn )
!%      equivalence( fir_up2, fir_up )
!%      equivalence( fir_up3, fir_up )
!%      equivalence( fir_dn2, fir_dn )
!%      equivalence( fir_dn3, fir_dn )


contains


!```````````````````````````````````````````````````````````````````
subroutine aermicro_AD(on_or_off,phase)

!+ Allocates or deallocates key module arrays

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

logical, intent(IN) :: on_or_off
integer, intent(IN) :: phase

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'aermicro_AD'

integer :: aerr,nz,nz1,mb,ne,ng,ns,no

!=== Executable statements:

  ne = nelements
  ng = ngroups
  ns = ngasses
  no = nsolutes
 
  if(phase == 1) then

     if(on_or_off) then

        allocate(nbins(ng),groupname(ng),nelemg(ng),ienconc(ng),rmin(ng)  &
             ,rmrat(ng),ishape(ng),eshape(ng),is_grp_ice(ng),ncore(ng)  &
             ,is_grp_mixed(ng),inucmin(ng),sec_mom(ng),imomelem(ng)  &
             ,rmassmin(ng),inucstep(ng), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD01')

        allocate(elemname(ne),igelem(ne),icomp(ne),itype(ne),rhoelem(ne)  &
             ,igrowgas(ne),nnuc2elem(ne),ievp2elem(ne),isolelem(ne)  &
             ,nnucelem(ne),pcmax(ne),should_vTX_ie(ne), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD02')

        allocate(gasname(ns),gwtmol(ns),ftopgas(ns),fbotgas(ns),gasprod(ns)  &
             ,sifreeze(ns), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD03')

        allocate(icoag(ng,ng),icorelem(ne,ng), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD04')

        allocate(inuc2elem(10,ne),if_nuc(ne,ne),inucproc(ne,ne)  &
             ,rlh_nuc(ne,ne),if_nuc_lh(ne,ne),inucelem(ne,ne*ng)  &
             ,icoagelem(ne,ng),inucgas(ne,ng), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD05')

        allocate(solname(no),solwtmol(no),rhosol(no),sol_ions(no), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD06')

     else

        deallocate(nbins,groupname,nelemg,ienconc,rmin,rmrat,ishape,eshape  &
             ,is_grp_ice,ncore,is_grp_mixed,inucmin,sec_mom,imomelem,rmassmin  &
             ,inucstep, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD01')

        deallocate(elemname,igelem,icomp,itype,rhoelem,igrowgas,nnuc2elem  &
             ,ievp2elem,isolelem,nnucelem,pcmax,should_vTX_ie, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD02')
        
        deallocate(gasname,gwtmol,ftopgas,fbotgas,gasprod,sifreeze, stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD03')

        deallocate(icoag,icorelem, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD04')

        deallocate(inuc2elem,if_nuc,inucproc,rlh_nuc,if_nuc_lh  &
             ,inucelem,icoagelem,inucgas, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD05')
     
        deallocate(solname,solwtmol,rhosol,sol_ions, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD06')

     endif

  elseif(phase == 2) then      ! Allocate some bin_dim-dependent things

     if(on_or_off) then

        mb = max_nbins

        allocate(rmass(mb,ng),rmassup(mb,ng),r(mb,ng),rup(mb,ng),dr(mb,ng)  &
             ,rlow(mb,ng), stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD99')

     else
        
        deallocate(rmass,rmassup,r,rup,dr,rlow, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD99')

     endif

  elseif(phase == 3) then   ! Allocate vertical_dim-dependent things

     if(on_or_off) then

        mb = max_nbins
        nz = nz_mic
        nz1 = nz1_mic

        max_dim = max(mb,nz1,ne,ng,ns,no)   ! Scratch array dimension
        max_dim = max_dim*max_dim*max_dim   !

        allocate(zl(nz1),zl_static(nz1),dkz(nz1),w(nz1),vtrans(nz1)  &
             ,vertadvu(nz1),vertadvd(nz1),vertdifd(nz1),vertdifu(nz1)  &
             , stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD51')

        allocate(zc(nz),zc_static(nz),dz(nz),ptc(nz),p(nz),t(nz),rhoa(nz)  &
             ,rmu(nz),thcond(nz),ptcl(nz),told(nz),pold(nz),rhoaold(nz)  &
             ,d_ptc(nz),cvert(nz),drho_dt(nz),divcor(nz),sadv_ptc_tend(nz)  &
             , stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD52')

        allocate(pc(nz,mb,ne),rhop(nz,mb,ng),bpm(nz,mb,ng),re(nz,mb,ng)  &
             ,vf(nz1,mb,ng),diffus(nz,ns),rlhe(nz,ns),rlhm(nz,ns)  &
             ,surfctwa(nz,ns),surfctiw(nz,ns),surfctia(nz,ns),akelvin(nz,ns)  &
             ,akelvini(nz,ns),ft(nz,mb,ng),gro(nz,mb,ng),gro1(nz,mb,ng)  &
             ,gro2(nz,ng),scrit(nz,mb,ng),pcl(nz,mb,ne),pconmax(nz,ng)  &
             ,gc(nz,ns),gcl(nz,ns),supsatlold(nz,ns),supsatiold(nz,ns)  &
             ,coagpe(nz,mb,ne),coaglg(nz,mb,ng),supsatl(nz,ns)  &
             ,supsati(nz,ns),d_gc(nz,ns),d_pc(nz,mb,ne),pvapl(nz,ns)  &
             ,pvapi(nz,ns),ckernel(nz,mb,mb,ng,ng)  &
             ,pkernel(nz,mb,mb,ng,ng,ng,6),rmshell(nz,mb),vcore(nz,mb)  &
             ,small_coremconc(mb,ne),sadv_pc_tend(nz,mb,ne)  &
             ,sadv_gc_tend(nz,ng),gstick(ng),tstick(ng),cor(ng),phish(ng)  &
             , stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD53')

        allocate(dm(mb,ng),vol(mb,ng),diffmass(mb,ng,mb,ng)  & 
             ,inuc2bin(mb,ng,ng),inucbin(mb*ng,ng,mb,ng),nnucbin(ng,mb,ng)  &
             ,fbotpart(mb,ne),ftoppart(mb,ne),volx(ng,ng,ng,mb,mb)  &
             ,npairu(ng,mb),npairl(ng,mb),ilow(ng,mb,mb*mb),jlow(ng,mb,mb*mb)  &
             ,iup(ng,mb,mb*mb),jup(ng,mb,mb*mb),iglow(ng,mb,mb*mb)  &
             ,jglow(ng,mb,mb*mb),igup(ng,mb,mb*mb),jgup(ng,mb,mb*mb)  &
             ,rnuclg(mb,ng,ng),rnucpe(mb,ne),growpe(mb,ne),evappe(mb,ne)  &
             ,growlg(mb,ng),evaplg(mb,ng),totevap(mb,ng),cmf(mb,ng)  &
             ,dmi(mb,ng),ge_pcfitconst(mb,ng,8), stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD54')

        allocate(fscr1(max_dim),fscr2(max_dim),fscr3(max_dim),fscr4(max_dim)  &
             ,fscr5(max_dim),fscr6(max_dim),fscr7(max_dim),iscr1(max_dim)  &
             ,iscr2(max_dim),iscr3(max_dim),fscr8(max_dim), stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
             procedure_name, 'aermicAD55')

     else

        deallocate(zl,zl_static,dkz,w,vtrans,vertadvu,vertadvd,vertdifd  &
             ,vertdifu, stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD51')

        deallocate(zc,zc_static,dz,ptc,p,t,rhoa,rmu,thcond,ptcl,told,pold  &
             ,rhoaold,d_ptc,cvert,drho_dt,divcor,sadv_ptc_tend, stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD52')

        deallocate(pc,rhop,bpm,re,vf,diffus,rlhe,rlhm,surfctwa,surfctiw  &
             ,surfctia,akelvin,akelvini,ft,gro,gro1,gro2,scrit,pcl,pconmax  &
             ,gc,gcl,supsatlold,supsatiold,coagpe,coaglg,supsatl,supsati  &
             ,d_gc,d_pc,pvapl,pvapi,ckernel,pkernel,rmshell,vcore  &
             ,small_coremconc,sadv_pc_tend,sadv_gc_tend,gstick,tstick  &
             ,cor,phish, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD53')

        deallocate(dm,vol,diffmass,inuc2bin  &
             ,inucbin,nnucbin,fbotpart,ftoppart,volx,npairu,npairl,ilow,jlow  &
             ,iup,jup,iglow,jglow,igup,jgup,rnuclg,rnucpe,growpe,evappe  &
             ,growlg,evaplg,totevap,cmf,dmi,ge_pcfitconst, stat=aerr)  
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD54')

        deallocate(fscr1,fscr2,fscr3,fscr4,fscr5,fscr6,fscr7,iscr1,iscr2  &
             ,iscr3, stat=aerr)
        if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
             procedure_name, 'aermicAD55')

     endif

  endif

  end subroutine aermicro_AD


end module aermicro_mem_mod
