!```````````````````````````````````````````````````````````````````
module near_surface_layer_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine starsl(m2, m3, m5, m6, ng, ia, iz, ja, jz, tstar, rstar, ustar,  &
     rv_air_s, theta_air_s, zpm, z0m, um, piocp_s, patch_area, sfc_tempk)

!+ Calculates surface layer parameters (ustar, tstar, rstar)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use body_specific_constants_mod, only: vonk, g_Xoid
use main_memory_mod, only: c_gd

implicit none

!=== Argument declarations:

integer, intent(IN) :: m2, m3, m5, m6, ng, ia, iz, ja, jz
real(f4), dimension(m2,m3), intent(IN) :: rv_air_s, theta_air_s, zpm, piocp_s
real(f4), dimension(m2,m3,m5), intent(IN) :: z0m, patch_area, sfc_tempk

real(f4), dimension(m2,m3), intent(INOUT) :: um
real(f4), dimension(m2,m3,m5), intent(INOUT) :: tstar, rstar, ustar

!== Local declarations:

integer :: i, j
real(f4) :: b, csm, csh, a2, c1, ri, fm, fh, c2, cm, ch, c3, d, tsfc, ubmin,  &
     ubmin_us

!=== Executable statements:

!***  Routine to compute Louis (1979) surface layer parameterization:
!
!b = 9.4
!bp = 4.7
!csm = 7.4
!csh = 5.3
!
!do j=ja,jz
!   do i=ia,iz
!      tsfc = sfc_tempk(i,j,1)/piocp_s(i,j)
!      a2 = (vonk/log(zpm(i,j)/z0m(i,j,1)))**2
!
!      if((theta_air_s(i,j)-tsfc) > 0.) then
!
!         ! *** STABLE CASE:
!
!         ubmin = 0.1
!         um(i,j) = max(ubmin,um(i,j))
!         c1 = a2*um(i,j)/0.74
!
!         ri = g_Xoid*zpm(i,j)*(theta_air_s(i,j)-tsfc)/  &
!              (0.5*(theta_air_s(i,j)+tsfc)*um(i,j)*um(i,j))
!
!         fm = 1.0/(1.0+bp*ri)**2
!         fh = fm
!      else
!
!         !*** UNSTABLE CASE:
!
!         ubmin = 1.
!         um(i,j) = max(ubmin,um(i,j))
!         c1 = a2*um(i,j)/0.74
!
!         ri = g_Xoid*zpm(i,j)*(theta_air_s(i,j)-tsfc)/  &
!              (0.5*(theta_air_s(i,j)+tsfc)*um(i,j)*um(i,j))
!
!         c2 = a2*b*sqrt(zpm(i,j)/z0m(i,j,1))
!         cm = csm*c2
!         ch = csh*c2
!         fh = (1.0-b*ri/(1.0+ch*sqrt(abs(ri))))
!         fm = (1.0-b*ri/(1.0+cm*sqrt(abs(ri))))
!      endif
!
!      ustar(i,j) = sqrt(a2*um(i,j)*um(i,j)*fm)
!      c3 = c1*fh/ustar(i,j)
!
! * no moisture *   rstar(i,j) = c3*(rv_air_s(i,j)-rv(i,j))
!      rstar(i,j) = 0.
!      tstar(i,j) = c3*(theta_air_s(i,j)-tsfc)
!   enddo
!enddo

!*** Routine to compute Louis (1981) surface layer parameterization:

b = 5._f4
csm = 7.5_f4
csh = 5._f4
d = 5._f4

if (c_gd%deltax >= 251._f4) then   !    [m] (Somewhat arbitrary, areocentric)
   ubmin_us = ONE_f4                 ! Do not use "gustiness" kludge if such
else                             !  effects are likely explicitly resolved     
   ubmin_us = 0.1_f4                !  by the current gridspacing
end if                            !

do j=ja,jz
   do i=ia,iz

!      if(patch_area(i,j,1) >= 0.009) then     !TIM -- BUT... no oceans!!!!

! a2 is the drag coefficient in neutral conditions, here same for h/m
! ri is the bulk richardson numer, eq. 3.45 in Garratt

         tsfc = sfc_tempk(i,j,1)/piocp_s(i,j)
         a2 = (vonk/log(zpm(i,j)/z0m(i,j,1)))**2
         
         if ((theta_air_s(i,j)-tsfc) > ZERO_f4) then

            !*** STABLE CASE:
            ubmin = 0.1_f4
            um(i,j) = max(ubmin, um(i,j))
            c1 = a2*um(i,j)
            
            ri = g_Xoid*zpm(i,j)*(theta_air_s(i,j)-tsfc)/  &
                 (0.5_f4*(theta_air_s(i,j)+tsfc)*um(i,j)*um(i,j))

            fm = ONE_f4/(ONE_f4+(2._f4*b*ri/sqrt(1.+d*ri)))
            fh = ONE_f4/(ONE_f4+(3._f4*b*ri*sqrt(1.+d*ri)))
         else

            !*** UNSTABLE CASE:

!            ubmin = 1.
            um(i,j) = max(ubmin_us, um(i,j))
            c1 = a2*um(i,j)
            
            ri = g_Xoid*zpm(i,j)*(theta_air_s(i,j)-tsfc)/  &
                 (0.5_f4*(theta_air_s(i,j)+tsfc)*um(i,j)*um(i,j))

            c2 = b*a2*sqrt(zpm(i,j)/z0m(i,j,1)*(abs(ri)))
            cm = csm*c2
            ch = csh*c2
            fm = (ONE_f4-2._f4*b*ri/(ONE_f4+2._f4*cm))
            fh = (ONE_f4-3._f4*b*ri/(ONE_f4+3._f4*ch))

         end if

         ustar(i,j,1) = sqrt(c1*um(i,j)*fm)
         c3 = c1*fh/ustar(i,j,1)

         !* no moisture *      rstar(i,j,1) = c3*(rv_air_s(i,j)-rv(i,j))
         rstar(i,j,1) = ZERO_f4
         tstar(i,j,1) = c3*(theta_air_s(i,j)-tsfc)

!      endif
   end do
end do

end subroutine starsl


!```````````````````````````````````````````````````````````````````
subroutine sfclmcvr(m2, m3, m5, ia, iz, ja, jz, ubar, uslmn, vslmn, quant1_s,  &
     vt2da, cosine1, sine1, pct, ustar, tstar, rstar, upwp, vpwp, wpwp, wptp,  &
     wprp, vtscr)

!+ Calculates surface layer turbulent fluxes, etc.

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use body_specific_constants_mod, only: vonk

implicit none

!=== Argument declarations:

integer, intent(IN) :: m2, m3, m5, ia, iz, ja, jz
real(f4), dimension(m2,m3), intent(IN) :: ubar, uslmn, vslmn, quant1_s
real(f4), dimension(m2,m3,m5), intent(IN) :: ustar, tstar, rstar, pct

real(f4), dimension(m2,m3), intent(INOUT) :: cosine1, sine1, vtscr, vt2da
real(f4), dimension(m2,m3), intent(INOUT) :: upwp, vpwp, wpwp, wptp, wprp

!== Local declarations:

! This routine computes the turbulent fluxes of momentum,      
! heat and moisture from the surface layer using the           
! Manton-Cotton algebraic surface layer equations.      

integer(f4) :: i, j
real(f4) :: cc, wtol, psin
     
data cc/4.7_f4/, wtol/1e-20_f4/
   
!=== Executable statements:

do j=ja,jz
   do i=ia,iz

      cosine1(i,j) = uslmn(i,j)/ubar(i,j)
      sine1(i,j) = vslmn(i,j)/ubar(i,j)

      !*** Lower boundary condition fluxes:

      vtscr(i,j) = ustar(i,j,1)*pct(i,j,1)
      upwp(i,j) = upwp(i,j)-cosine1(i,j)*ustar(i,j,1)*vtscr(i,j)
      vpwp(i,j) = vpwp(i,j)-sine1(i,j)*ustar(i,j,1)*vtscr(i,j)
      wptp(i,j) = wptp(i,j)-tstar(i,j,1)*vtscr(i,j)
        ! Must have strange (includes megagrams=Mg) units of
        !  [(Mg/kg_air)(m/s)]:
      wprp(i,j) = ZERO_f4     ! assume no "moisture"

      vt2da(i,j) = quant1_s(i,j)*vonk*tstar(i,j,1)/(ustar(i,j,1)*ustar(i,j,1))
      
      if (vt2da(i,j) < ZERO_f4) then
         vt2da(i,j) = vt2da(i,j)*sqrt(sqrt(ONE_f4-15._f4*vt2da(i,j)))
      else
         vt2da(i,j) = vt2da(i,j)/(ONE_f4+cc*vt2da(i,j))
      end if

      psin = sqrt((ONE_f4-2.86_f4*vt2da(i,j))  &
           /(ONE_f4+vt2da(i,j)*(-5.39_f4+vt2da(i,j)*6.998_f4)))

      wpwp(i,j) = wpwp(i,j)+(0.27_f4*  &
           max(6.25_f4*(ONE_f4-vt2da(i,j))*psin, wtol)  &
           -1.18_f4*vt2da(i,j)*psin)*ustar(i,j,1)*vtscr(i,j)

   end do
end do

end subroutine sfclmcvr


!```````````````````````````````````````````````````````````````````
subroutine sfc_stress(m1, m2, m3, m5, ia, iz, ja, jz, ustar, density_s, stress)

!+ Calculates aerodynamic shear stress at "surface", given u* and density

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, ia, iz, ja, jz, m5
real(f4), dimension(m2,m3), intent(IN) :: density_s   ! [kg/m^3]
real(f4), dimension(m2,m3,m5), intent(IN) :: ustar   ! [m/s]

real(f4), dimension(m2,m3), intent(INOUT) :: stress     ! [N]

!== Local declarations:

integer :: i, j

!=== Executable statements:

do j=ja,jz
   do i=ia,iz
      stress(i,j) = ustar(i,j,1)*ustar(i,j,1)*density_s(i,j)
   end do
end do

end subroutine sfc_stress


end module near_surface_layer_mod


!```````````````````````````````````````````````````````````````````
subroutine sfclyr(mzp, mxp, myp, ia, iz, ja, jz, ibcon)

!+ Topmost driver for surface layer / SBPS model routines

use main_memory_mod, only: a, c_gd, npatch, nzs, nzg
use vtables_mod, only: iup, ivp, itheta, ipp, ipt, ipi0, irtgt, ivt3dj,  &
     ivt3dk, itht

implicit none

!=== Argument declarations:

integer, intent(IN) :: mzp, mxp, myp, ia, iz, ja, jz, ibcon

!== Local declarations:

integer :: mxyp

!=== Executable statements:

if (c_gd%nstbot == 0) return   ! Return if nest does not reach surface

mxyp = mxp*myp
call SBPS_driver(mzp, mxp, myp, nzg+nzs, npatch, nzg, nzs, ia, iz, ja, jz,  &
     a(iup),a(ivp), a(itheta), a(itht), a(ipp), a(ipt), a(ipi0), a(irtgt),  &
     a(ivt3dj+0*mxyp), a(ivt3dj+1*mxyp), a(ivt3dj+2*mxyp), a(ivt3dj+3*mxyp),  &
     a(ivt3dj+4*mxyp), a(ivt3dj+5*mxyp), a(ivt3dj+6*mxyp), a(ivt3dj+7*mxyp),  &
     a(ivt3dk+0*mxyp), a(ivt3dk+1*mxyp), a(ivt3dk+2*mxyp))

!*** [LES only] Perturb lowest level theta at specified time after start:
!
!if(time >= 11000. .and. time <= 11001.) then
!   call LES_iperturb(nnzp(ngrid),nnxp(ngrid),nnyp(ngrid),a(ithp))
!endif

end subroutine sfclyr


!```````````````````````````````````````````````````````````````````
subroutine SBPS_driver(m1, m2, m3, m4, m5, m6, m7, ia, iz, ja, jz, up, vp,  &
     theta, tht, pp, pt, pi0, rtgt, theta_s, rv_s, piocp_s, press_s,  &
     density_s, u_avg_s, v_avg_s, wspd_s, quant1_s, zt_2, net_atm_dmass)

!+ Sub/super surface parameterization driver

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use body_specific_constants_mod, only: cpi, p00, cpor, cvor, R_air, cp,  &
     rocp, g_Xoid
use main_memory_mod, only: jdim, c_gd, a, vctr1, naerscl,  &
     i_grid, enable_turb_diffusion, prognose_SBPS, vctr2, vctr3, vctr4
use main_memory_bspec_mod, only: slz
use vtables_mod, only: iustar, itstar, irstar, ialbdstat, ialbedt, isfctempk,  &
     isbpsdens, isbpstmcp, isbpstmcd, isbpslthk, isbpsq, iz0net, ipatchfcv,  &
     nv2Ds, nv3Ds, ind2Ds, ind3Ds, ivtab2Ds, ivtab3Ds, irshort, irlong,  &
     irlongup, ivt2da, ivt2db, ivt2dc, ivt2dd, itfz, ivw, iuw, iwfz,  &
     iqfz, iaerfi, iaeraj, iaerrs, irtgt, idn0, iup, ivp, ithp
use aermic_rad_mem_mod, only: aer_XOsfcsrc
use PRAMS_SBPS_mod, only: SBPS_gndice
use near_surface_layer_mod, only: starsl, sfclmcvr
use grid_mem_mod, only: switch_to_Ascalar
use aemic_drvint_mod, only: aer_injection

implicit none

!=== Argument declarations:

integer, intent(IN) :: m1, m2, m3, m4, m5, m6, m7, ia, iz, ja, jz
real(f4), dimension(m1,m2,m3), intent(IN) :: up, vp, theta, pp, pi0
real(f4), dimension(m2,m3), intent(IN) :: rtgt

real(f4), dimension(m2,m3), intent(INOUT) :: theta_s, rv_s,  & ! named scr arrs
     piocp_s, press_s, density_s, u_avg_s, v_avg_s, wspd_s, quant1_s, zt_2,  &
     net_atm_dmass
real(f4), dimension(m1,m2,m3), intent(INOUT) :: pt, tht

!== Local declarations:

integer :: i, j, iter, idx, iv
real(f4) :: dz_2, pi2_o, p2_o, p2_n, t2, m2_o, m2_n, pi2_n, th2_n, tmp_f4
logical :: inj_on

!=== Executable statements:

! Compute various quantities that are static during SBPS sub-timestepping: 

do j=ja,jz
   do i=ia,iz

      theta_s(i,j) = theta(2,i,j)   ! Guess at the "sfc" potential temp [K]
      rv_s(i,j) = ZERO_f4    ! Guess at "sfc" water vapor; zero for now [??]

      piocp_s(i,j) = (pp(1,i,j)+pp(2,i,j)+pi0(1,i,j)+pi0(2,i,j))*  &
           0.5_f4*cpi                     ! "sfc" value of (full_PI/cp) [none]
      press_s(i,j) = p00*piocp_s(i,j)**cpor      ! "sfc" pressure [Pa]

      density_s(i,j) = p00*((piocp_s(i,j))**cvor)/  &
           (R_air*theta_s(i,j))               ! "sfc" air density [kg/m^3]

      u_avg_s(i,j) = (up(2,i,j)+up(2,i-1,j))*0.5_f4    ! "sfc" velocity components
      v_avg_s(i,j) = (vp(2,i,j)+vp(2,i,j-jdim))*0.5_f4 !  avgd to "T"-pts [m/s]
      
      ! Horizontal "sfc" wind magnitude [m/s]; later altered according to
      !  stability in `starsl` below:
      wspd_s(i,j) = sqrt(u_avg_s(i,j)*u_avg_s(i,j)+v_avg_s(i,j)*v_avg_s(i,j))

      zt_2(i,j) = c_gd%zt(2)*rtgt(i,j)     ! Actual height AGL [m] of k=2 level
      quant1_s(i,j) = g_Xoid*zt_2(i,j)/theta_s(i,j)  ! Quantity used by `sfclmcvr`

   end do
end do

net_atm_dmass = ZERO_f4    ! Initialize entire CO2 mass gain/loss array

do iter=1,c_gd%n_iter_SBPS      ! SBPS model substep iteration loop

   ! Compute SBPS thermal fluxes, etc.:

   if (prognose_SBPS) then
      call SBPS_gndice(i_grid, m1, m2, m3, m4, m5, m6, m7, ia, iz, ja, jz,  &
           c_gd%dt_SBPS, a(irshort), a(irlong), a(irlongup), a(iustar),  &
           a(itstar), a(irstar), a(ialbdstat), piocp_s, density_s, vctr1,  &
           vctr2, vctr3, vctr4, slz, a(itfz), a(ialbedt), a(isfctempk),  &
           a(isbpsdens), a(isbpstmcp), a(isbpstmcd), a(isbpslthk), a(isbpsq),  &
           net_atm_dmass, press_s, iter, rtgt, theta_s, theta, pp, pi0,  &
           c_gd%bottom_other_sflux_down, c_gd%bottom_other_sflux_up,  &
           c_gd%emisst, c_gd%emissivity_static, c_gd%bottom_SWflux_down,  &
           c_gd%bottom_SWflux_up, c_gd%topo_factor_sflux)
   end if

   ! Zero out turbulent flux arrays:

   a(iuw:iuw+m2*m3-1) = ZERO_f4; a(ivw:ivw+m2*m3-1) = ZERO_f4
   a(iwfz:iwfz+m2*m3-1) = ZERO_f4; a(itfz:itfz+m2*m3-1) = ZERO_f4
   a(iqfz:iqfz+m2*m3-1) = ZERO_f4

   if (enable_turb_diffusion) then
      ! Calculate atmospheric surface layer quantities:
      call starsl(m2, m3, m5, m6, i_grid, ia, iz, ja, jz, a(itstar),  &
           a(irstar), a(iustar), rv_s, theta_s, zt_2, a(iz0net), wspd_s,  &
           piocp_s, a(ipatchfcv), a(isfctempk))
     
      ! Calculate surface turbulent fluxes:
      call sfclmcvr(m2, m3, m5, ia, iz, ja, jz, wspd_s, u_avg_s, v_avg_s,  &
           quant1_s, a(ivt2da), a(ivt2db), a(ivt2dc), a(ipatchfcv),  &
           a(iustar), a(itstar), a(irstar), a(iuw), a(ivw), a(iwfz), a(itfz),  &
           a(iqfz), a(ivt2dd))
   end if

end do

! Calculate PI and theta changes induced by CO2 phase change:

do j=ja,jz
   do i=ia,iz
      if (abs(net_atm_dmass(i,j)) > 1.e-9_f4) then

         dz_2 = rtgt(i,j)/c_gd%dzt(2)    ! delta-z of layer k=2 
         pi2_o = pp(2,i,j)+pi0(2,i,j)   ! "old" full_PI value at k=2

         p2_o = p00*(pi2_o*cpi)**cpor   ! "old" pressure [Pa] at k=2
         t2 = theta(2,i,j)*pi2_o*cpi   ! "old" temperature [K] at k=2
         m2_o = p2_o/(R_air*t2)*dz_2   ! "old" mass of layer k=2
         
         m2_n = m2_o+net_atm_dmass(i,j)  ! "new" mass of layer k=2
         p2_n = m2_n*R_air*t2/dz_2  ! "new" pressure [Pa] at k=2
         pi2_n = cp*(p2_n/p00)**rocp  ! "new" PI value at k=2

         pt(2,i,j) = pt(2,i,j)+(pi2_n-pi2_o)/c_gd%dt_dyn  ! PI tendency

         th2_n = t2*(p00/p2_n)**rocp   ! "new" theta value at k=2
         tht(2,i,j) = tht(2,i,j)+(th2_n-theta(2,i,j))/c_gd%dt_dyn  ! theta tendency

      end if
   end do
end do
     
! Check if Qp energy needs emptied into Q:

!#call SBPS_Qprcadj(m2,m3,m4,m5,ia,iz,ja,jz,a(isbpsq),Qp)

! Calculate aerosol sfc -> air turbulent fluxes [w'aer'(ib)]:
!   NOTE: `sfc_stress_cgsrho` returns stress [N] in vt2da

call switch_to_Ascalar(i_grid, 1)  ! Fill 'iaerscp/t', etc. indices
inj_on =.FALSE.
do iv=1,naerscl
   if (aer_XOsfcsrc(iv,i_grid)) then
      inj_on = .TRUE.
   end if
end do

if (inj_on) then
   call aer_injection(m1, m2, m3, m5, ia, iz, ja, jz, a(iaerfi), a(iaeraj),  &
        a(iaerrs), a(irtgt), a(idn0), naerscl, a(ivt2db), density_s, theta_s,  &
        piocp_s, a(iustar), a(iup), a(ivp), a(isfctempk), a(ithp))
end if

! Apply lateral boundary conditions to SBPS arrays:

do iv=1,nv2Ds(i_grid)
   idx = ind2Ds(ivtab2Ds(iv,i_grid))
   call sfc2Dbc(m2, m3, m5, ia, iz, ja, jz, jdim, a(idx))
end do

do iv=1,nv3Ds(i_grid)
   idx = ind3Ds(ivtab3Ds(iv,i_grid))
   call sfc3Dbc(m4, m2, m3, m5, ia, iz, ja, jz, jdim, a(idx))
end do

! Calculate an estimate for what the next SBPS timestep should be:
   ! Based on a 1 mm thick layer, with k=0.6 (~800 tiu), rho~1500., cs=711.
   !  [SI units] -- dt < rho*cs*dz*dz/(k*2.).  A more efficient (clever?) way to
   !  do this would require a different timestep for each column/point.
tmp_f4 = maxval(a(isbpstmcd:isbpstmcd+m2*m3*m4*m5-1),  &   ! Current max thermal
     mask=a(isbpstmcd:isbpstmcd+m2*m3*m4*m5-1) < 1.e5_f4)  !  conductivity
c_gd%dt_SBPS_estimate = 0.75_f4*0.6_f4/tmp_f4
c_gd%dt_SBPS_estimate = min(0.75_f4, c_gd%dt_SBPS_estimate)

end subroutine SBPS_driver
