module ARCnRT_laginterp_mod

implicit none


contains


      subroutine ARCnRT_laginterp(pgref,pint,co2i,co2v,fzeroi,fzerov)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

!  Lagrange interpolation (linear in log pressure) of the CO2 
!  k-coefficients in the pressure domain.  Subsequent use of these
!  values will use a simple linear interpolation in pressure.

      use ARCnRT_grid_h
      use ARCnRT_radinc_h
use univ_std_fileIO_mod, only: find_file_IOunit
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use aermic_rad_mem_mod, only: dataset_path_bspec
use ARCnRT_lagrange_mod, only: ARCnRT_lagrange

      implicit none

!  PGREF   - Pressure grid on which k-coefficients were computed.
!  PINT    - Interpolated pressure grid (finer grid spacing than 
!            PGREF)
!  CO2I    - CO2 IR k-coefficients on interpolated grid
!  CO2V    - CO2 visible k-coefficients on interpolated grid
!  FZEROI  - Fraction of zeros in each IR spectral interval
!  FZEROV  - Fraction of zeros in each visible spectral interval
!
!----------------------------------------------------------------------!

      real(f8), intent(IN) :: pgref(L_NPREF)
      real(f8), intent(OUT) :: pint(L_PINT)
      real(f8), intent(INOUT) :: co2i(L_NTREF,L_PINT,L_REFH2O,L_NSPECTI,L_NGAUSS)      
      real(f8), intent(INOUT) :: co2v(L_NTREF,L_PINT,L_REFH2O,L_NSPECTV,L_NGAUSS)
      real(f8), intent(OUT) :: fzeroi(L_NSPECTI)
      real(f8), intent(OUT) :: fzerov(L_NSPECTV)

      integer :: n, nt, np, nh, ng, nw, m, i
      real(f8) :: co2i8(L_NTREF,L_NPREF,L_REFH2O,L_NSPECTI,L_NGAUSS)
      real(f8) :: co2v8(L_NTREF,L_NPREF,L_REFH2O,L_NSPECTV,L_NGAUSS)
 
      real(f8) :: x, xi(4), yi(4), ans
      real(f8) :: pref(L_NPREF) !TIM, p

      real(f8) :: pin(L_PINT) =  [                                      &
                           -6._f8, -5.8_f8, -5.6_f8, -5.4_f8, -5.2_f8,     &
                           -5._f8, -4.8_f8, -4.6_f8, -4.4_f8, -4.2_f8,     &
                           -4._f8, -3.8_f8, -3.6_f8, -3.4_f8, -3.2_f8,     &
                           -3._f8, -2.8_f8, -2.6_f8, -2.4_f8, -2.2_f8,     &
                           -2._f8, -1.8_f8, -1.6_f8, -1.4_f8, -1.2_f8,     &
                           -1._f8, -0.8_f8, -0.6_f8, -0.4_f8, -0.2_f8,     &
                            0._f8,  0.2_f8,  0.4_f8,  0.6_f8,  0.8_f8,     &
                            1._f8,  1.2_f8,  1.4_f8,  1.6_f8,  1.8_f8,     &
                            2._f8,  2.2_f8,  2.4_f8,  2.6_f8,  2.8_f8,     &
                            3._f8,  3.2_f8,  3.4_f8,  3.6_f8,  3.8_f8,     &
                            4._f8                                  ]

integer :: IO_unit
character(len=MAX_FILE_PATH_LEN) :: fn_tmp

!======================================================================!

!  Fill pint for output from this subroutine

      do n=1,L_PINT
        PINT(n) = PIN(n)
      end do

!  Take log of the reference pressures

      do n=1,L_NPREF
        pref(n) = LOG10(PGREF(n))
      end do

!     Get CO2 k coefficients

IO_unit = find_file_IOunit()
fn_tmp = trim(dataset_path_bspec)//'/CO2H2O_V_12_95_INTEL'
      open(IO_unit, file=trim(fn_tmp), form='unformatted')
      read(IO_unit) co2v8
      read(IO_unit) fzerov
      close(IO_unit)

fn_tmp = trim(dataset_path_bspec)//'/CO2H2O_IR_12_95_INTEL'
      open(IO_unit, file=trim(fn_tmp), form='unformatted')
      read(IO_unit) co2i8
      read(IO_unit) fzeroi
      close(IO_unit)

!  Take Log10 of the values - we interpolate the log10 of the values,
!  not the values themselves.   Smallest value is 1.0E-200.

      do nt=1,L_NTREF
        do np=1,L_NPREF
          do nh=1,L_REFH2O
            do ng = 1,L_NGAUSS

               if (nt == 1 .and. np == 1 .and. nh == 1 .and. ng == 1)  &
                    print *,'ARCnRT_laginterp: kludge to trick vectorizer?'

              do nw=1,L_NSPECTV
                if(co2v8(nt,np,nh,nw,ng).gt.1.e-200_f8) then
                  co2v8(nt,np,nh,nw,ng) = log10(co2v8(nt,np,nh,nw,ng))
                else
                  co2v8(nt,np,nh,nw,ng) = -200._f8
                end if
              end do

              do nw=1,L_NSPECTI
                if(co2i8(nt,np,nh,nw,ng).gt.1.e-200_f8) then
                  co2i8(nt,np,nh,nw,ng) = log10(co2i8(nt,np,nh,nw,ng))
                else
                  co2i8(nt,np,nh,nw,ng) = -200._f8
                end if
              end do
      
            end do
          end do
        end do
      end do

!  Interpolate the values:  first the IR

      do nt=1,L_NTREF
        do nh=1,L_REFH2O
        do nw=1,L_NSPECTI
          do ng=1,L_NGAUSS

!  First, the initial interval (P=1e-6 to 1e-5)

            n = 1 
            do m=1,5
              x     = pint(m)
              xi(1) = pref(n)
              xi(2) = pref(n+1)
              xi(3) = pref(n+2)
              xi(4) = pref(n+3)
              yi(1) = co2i8(nt,n,nh,nw,ng)
              yi(2) = co2i8(nt,n+1,nh,nw,ng)
              yi(3) = co2i8(nt,n+2,nh,nw,ng)
              yi(4) = co2i8(nt,n+3,nh,nw,ng)
              call ARCnRT_lagrange(x,xi,yi,ans)
              co2i(nt,m,nh,nw,ng) = 10._f8**ans
            end do 
 
            do n=2,L_NPREF-2
              do m=1,5
                i     = (n-1)*5+m
                x     = pint(i)
                xi(1) = pref(n-1)
                xi(2) = pref(n)
                xi(3) = pref(n+1)
                xi(4) = pref(n+2)
                yi(1) = co2i8(nt,n-1,nh,nw,ng)
                yi(2) = co2i8(nt,n,nh,nw,ng)
                yi(3) = co2i8(nt,n+1,nh,nw,ng)
                yi(4) = co2i8(nt,n+2,nh,nw,ng)
                call ARCnRT_lagrange(x,xi,yi,ans)
                co2i(nt,i,nh,nw,ng) = 10._f8**ans
              end do 
            end do

!  Now, get the last interval (P=1e+3 to 1e+4)

            n = L_NPREF-1
      
            do m=1,5
              i     = (n-1)*5+m
              x     = pint(i)
              xi(1) = pref(n-2)
              xi(2) = pref(n-1)
              xi(3) = pref(n)
              xi(4) = pref(n+1)
              yi(1) = co2i8(nt,n-2,nh,nw,ng)
              yi(2) = co2i8(nt,n-1,nh,nw,ng)
              yi(3) = co2i8(nt,n,nh,nw,ng)
              yi(4) = co2i8(nt,n+1,nh,nw,ng)
              call ARCnRT_lagrange(x,xi,yi,ans)
              co2i(nt,i,nh,nw,ng) = 10._f8**ans
            end do  

!  Fill the last pressure point

            co2i(nt,L_PINT,nh,nw,ng) = 10._f8**co2i8(nt,L_NPREF,nh,nw,ng)

          end do
        end do
        end do
      end do

!  Interpolate the values:  now the visible

      do nt=1,L_NTREF
        do nh=1,L_REFH2O
        do nw=1,L_NSPECTV
          do ng=1,L_NGAUSS

!  First, the initial interval (P=1e-6 to 1e-5)

            n = 1 
            do m=1,5
              x     = pint(m)
              xi(1) = pref(n)
              xi(2) = pref(n+1)
              xi(3) = pref(n+2)
              xi(4) = pref(n+3)
              yi(1) = co2v8(nt,n,nh,nw,ng)
              yi(2) = co2v8(nt,n+1,nh,nw,ng)
              yi(3) = co2v8(nt,n+2,nh,nw,ng)
              yi(4) = co2v8(nt,n+3,nh,nw,ng)
              call ARCnRT_lagrange(x,xi,yi,ans)
              co2v(nt,m,nh,nw,ng) = 10._f8**ans
            end do 
 
            do n=2,L_NPREF-2
              do m=1,5
                i     = (n-1)*5+m
                x     = pint(i)
                xi(1) = pref(n-1)
                xi(2) = pref(n)
                xi(3) = pref(n+1)
                xi(4) = pref(n+2)
                yi(1) = co2v8(nt,n-1,nh,nw,ng)
                yi(2) = co2v8(nt,n,nh,nw,ng)
                yi(3) = co2v8(nt,n+1,nh,nw,ng)
                yi(4) = co2v8(nt,n+2,nh,nw,ng)
                call ARCnRT_lagrange(x,xi,yi,ans)
                co2v(nt,i,nh,nw,ng) = 10._f8**ans
              end do 
            end do

!  Now, get the last interval (P=1e+3 to 1e+4)

            n = L_NPREF-1
      
            do m=1,5
              i     = (n-1)*5+m
              x     = pint(i)
              xi(1) = pref(n-2)
              xi(2) = pref(n-1)
              xi(3) = pref(n)
              xi(4) = pref(n+1)
              yi(1) = co2v8(nt,n-2,nh,nw,ng)
              yi(2) = co2v8(nt,n-1,nh,nw,ng)
              yi(3) = co2v8(nt,n,nh,nw,ng)
              yi(4) = co2v8(nt,n+1,nh,nw,ng)
              call ARCnRT_lagrange(x,xi,yi,ans)
              co2v(nt,i,nh,nw,ng) = 10._f8**ans
            end do  

!  Fill the last pressure point

            co2v(nt,L_PINT,nh,nw,ng) = 10._f8**co2v8(nt,L_NPREF,nh,nw,ng)
            
          end do
        end do
        end do
      end do

      return
      end subroutine ARCnRT_laginterp


end module ARCnRT_laginterp_mod
