!```````````````````````````````````````````````````````````````````
module ARCnRT_memory_mod

!+ Module that contains memory structures common to the ARCnRT code routines

use univ_kind_defs_mod, only: f8, f4
!use univ_math_constants_mod, only: pi_f8, twopi_f8
use body_specific_constants_mod, only: g_Xoid, cp
use ARCnRT_radinc_h, only: L_NSPECTV, L_NGAUSS, L_NSPECTI 

implicit none

!  PLEV & TLEV are the pressure and temperatures at the GCM levels
!  while PMID & TMID are the pressure and temperatures at the GCM
!  midpoint.

real(f8), allocatable, dimension(:) :: PLEV, TLEV   ! (L_LEVELS)
real(f8), allocatable, dimension(:) :: TMID, PMID  ! (L_LEVELS)

!  DIFFVT is the total diffuse visible flux for a given spectral 
!  interval

real(f8) :: DIFFVT

!  VISUAL

real(f8), allocatable, dimension(:,:,:) :: DTAUV !(L_NLAYRAD,L_NSPECTV,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: TAUV !(L_NLEVRAD,L_NSPECTV,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: TAUCUMV !(L_LEVELS,L_NSPECTV,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: COSBV !(L_NLAYRAD,L_NSPECTV,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: WBARV !(L_NLAYRAD,L_NSPECTV,L_NGAUSS)
real(f8) :: taugsurf(L_NSPECTV,L_NGAUSS-1)
real(f8) :: taucump(L_NSPECTV,L_NGAUSS)
integer :: ngwv(L_NSPECTV)
real(f8) ::  detau(L_NSPECTV,L_NGAUSS)

!  IR

real(f8), allocatable, dimension(:,:,:) :: DTAUI !(L_NLAYRAD,L_NSPECTI,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: TAUCUMI !(L_LEVELS,L_NSPECTI,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: COSBI !(L_NLAYRAD,L_NSPECTI,L_NGAUSS)
real(f8), allocatable, dimension(:,:,:) :: WBARI !(L_NLAYRAD,L_NSPECTI,L_NGAUSS)
real(f8) :: taugsurfi(L_NSPECTI,L_NGAUSS-1)
integer :: ngwi(L_NSPECTI)

!  Water mixing

real(f8), allocatable, dimension(:) :: QH2O !(L_LEVELS)

real(f8) :: scaleht
real(f8) :: SOL(L_NSPECTV)

integer :: gcmlayers
integer :: NLAYRAD, NLEVRAD, NSPECTI, NSPECTV, NPREF, NTREF
integer :: K, L, NW, nn, nnn
real(f8) ::  albi, albv
real(f8) ::  acosz
real(f8) ::  ans
!%real(f8) ::  heatingv(L_NLAYRAD), heatingir(L_NLAYRAD)
!%real(f8) ::  total(L_NLAYRAD)
real(f8) ::  fdmax, fdmin
real(f8) ::  firmax, firmin, fvmax, fvmin, df
real(f4) ::  tstrato, psfo, gto, coszo, alspo, tautoto
real(f8) ::  tstratd, gtd

real(f8), allocatable, dimension(:) :: FMNETI, FMNETV, net_dTdt !(L_NLAYRAD)
real(f8), allocatable, dimension(:) :: fluxupi, fluxdni !(L_NLAYRAD)
real(f8) :: NFLUXTOPI
real(f8), allocatable, dimension(:) :: fluxupv, fluxdnv !(L_NLAYRAD)
real(f8) :: NFLUXTOPV
real(f8), allocatable, dimension(:) :: fluxdv, fluxdi !(L_NLAYRAD)

real(f8), allocatable, dimension(:,:) :: up_fluxes,  &
     dn_fluxes                             ! {L_NLAYRAD,L_NSPECTI+L_NSPECTV}

!  Clouds

integer :: nlev
real(f8) :: pcld, tautotcld

real(f8) :: dust_tautot_ref_pr, dust_tau_ref_pr_min, cos_mu
logical :: want_rtflux_output, sw_on, part_in_tshadow, use_gas_opacities
integer :: i_curr, j_curr, swcut


real(f8), parameter :: grav = g_Xoid  ! gravitational acceleration [m/s^2]

real(f8), parameter :: cp_air = cp  ! specific heat of "air" [J/(K.kg)] 

real(f4) :: solar_sw   ! Solar flux (@ 1AU) within this RT code's "shortwave"
                       !  spectral range [W/m^2]


contains


!```````````````````````````````````````````````````````````````````
subroutine ARCnRT_mem_AD(on_or_off)

!+ Allocates or deallocates key module arrays

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use ARCnRT_radcommon_h, only: tauref
use ARCnRT_fccsave_h, only: TAUREFCLD, QEXTREFCLD, QXVCLD, QSVCLD, GVCLD,  &
     QXICLD, QSICLD, GICLD, QEXTREFDST, QXVDST, QSVDST, GVDST, QXIDST,  &
     QSIDST, GIDST
use ARCnRT_standard_h, only: TAUCUM
use ARCnRT_radinc_h, only: L_NSPECTV, L_NSPECTI, L_NLAYRAD, L_NGAUSS,  &
     L_NLEVRAD
use ARCnRT_grid_h, only: L_LEVELS

implicit none

!=== Argument declarations:

logical, intent(IN) :: on_or_off

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'ARCnRT_mem_AD'

integer :: e

!=== Executable statements:
 
if (on_or_off) then

   allocate(tauref(l_levels+1), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1')

   allocate(TAUREFCLD(L_LEVELS+1), QEXTREFCLD(L_LEVELS+1),  &
        QXVCLD(L_LEVELS+1,L_NSPECTV), QSVCLD(L_LEVELS+1,L_NSPECTV),  &
        GVCLD(L_LEVELS+1,L_NSPECTV), QXICLD(L_LEVELS+1,L_NSPECTI),  &
        QSICLD(L_LEVELS+1,L_NSPECTI), GICLD(L_LEVELS+1,L_NSPECTI),  &
        QEXTREFDST(L_LEVELS+1), QXVDST(L_LEVELS+1,L_NSPECTV),  &
        QSVDST(L_LEVELS+1,L_NSPECTV), GVDST(L_LEVELS+1,L_NSPECTV),  &
        QXIDST(L_LEVELS+1,L_NSPECTI), QSIDST(L_LEVELS+1,L_NSPECTI),  &
        GIDST(L_LEVELS+1,L_NSPECTI), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2')

   allocate(taucum(l_levels), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '3')

   allocate(plev(l_levels), tlev(l_levels), pmid(l_levels), tmid(l_levels),  &
        DTAUV(L_NLAYRAD,L_NSPECTV,L_NGAUSS),  &
        TAUV(L_NLEVRAD,L_NSPECTV,L_NGAUSS),  &
        TAUCUMV(L_LEVELS,L_NSPECTV,L_NGAUSS),  &
        COSBV(L_NLAYRAD,L_NSPECTV,L_NGAUSS),  &
        WBARV(L_NLAYRAD,L_NSPECTV,L_NGAUSS),  &
        DTAUI(L_NLAYRAD,L_NSPECTI,L_NGAUSS),  &
        TAUCUMI(L_LEVELS,L_NSPECTI,L_NGAUSS),  &
        COSBI(L_NLAYRAD,L_NSPECTI,L_NGAUSS),  &
        WBARI(L_NLAYRAD,L_NSPECTI,L_NGAUSS), net_dTdt(L_NLAYRAD),  &
        QH2O(L_LEVELS), FMNETI(L_NLAYRAD), FMNETV(L_NLAYRAD),  &
        fluxupi(L_NLAYRAD), fluxdni(L_NLAYRAD), fluxupv(L_NLAYRAD),  &
        fluxdnv(L_NLAYRAD), fluxdv(L_NLAYRAD), fluxdi(L_NLAYRAD),  &
        up_fluxes(L_NLAYRAD,L_NSPECTI+L_NSPECTV),  &
        dn_fluxes(L_NLAYRAD,L_NSPECTI+L_NSPECTV), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '4')

else

   deallocate(tauref, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

   deallocate(TAUREFCLD, QEXTREFCLD, QXVCLD, QSVCLD, GVCLD, QXICLD, QSICLD,  &
        GICLD, QEXTREFDST, QXVDST, QSVDST, GVDST, QXIDST, QSIDST,  &
        GIDST, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '2')

   deallocate(taucum, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '3')

   deallocate(plev, tlev, pmid, tmid, DTAUV, TAUV, TAUCUMV, COSBV, WBARV,  &
        DTAUI, TAUCUMI, COSBI, WBARI, QH2O, FMNETI, FMNETV, fluxupi,  &
        fluxdni, fluxupv, fluxdnv, fluxdv, fluxdi, net_dTdt, up_fluxes,  &
        dn_fluxes, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '4')

end if

end subroutine ARCnRT_mem_AD


!```````````````````````````````````````````````````````````````````
function get_ntot_wavlnrng()  result(answer)

!+ Inquiry method

!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use ARCnRT_radinc_h, only: L_NSPECTV, L_NSPECTI 

implicit none

!=== Argument declarations:

integer :: answer     ! Procedure result

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'get_ntot_wavlnrng'

!=== Executable statements:

answer = L_NSPECTI+L_NSPECTV

end function get_ntot_wavlnrng


end module ARCnRT_memory_mod
