module ARCnRT_optci_mod

implicit none


contains


      SUBROUTINE ARCnRT_OPTCI(DTAUI,TAUCUMI,CO2I,PLEV,PFGASREF,TGASREF,       &
                       QREFV,QXIDST,QSIDST,GIDST,COSBI,WBARI,TAUREF,   &
                       TMID,PMID,TAUGSURF,QH2O,WREFH2O,                &
                       Qextrefcld,TAUREFCLD,Qxicld,Qsicld,gicld)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

! THIS SUBROUTINE SETS THE OPTICAL CONSTANTS IN THE INFRARED
! IT CALCUALTES FOR EACH LAYER, FOR EACH SPECRAL INTERVAL IN THE IR
! LAYER: WBAR, DTAU, COSBAR
! LEVEL: TAU
!
! Qrefv is the extinction coefficient at the reference (visible) 
! wavelength - 0.67 microns.
!
! TAUI(L,LW) is the cumulative optical depth at level L (or alternatively
! at the *bottom* of layer L), LW is the spectral wavelength interval.
!
!     TLEV(L) - Temperature at the layer boundary (i.e. level)
!     PLEV(L) - Pressure at the layer boundary (i.e. level)
!     CO2_KI(NT,NP,NW,NG) - IR CO2 k-coefficients 
!                           CO2_K(temp,Pres,Waveln,gauss)
!                           currently: CO2_K(7,11,5,17)
!
!  DTAUI      - IR opacity of layer L.
!  TAUI       - IR opacity from top of the atmosphere to bottom
!               of layer L.
!  TAUCUMI    - Cumulative optical depth, top of atmosphere to
!               the bottom of each layer.
!  PLEV(L)    - Pressure at the layer boundary (i.e. level)
!  CO2I       - IR CO2 k-coefficients 
!  PLEV       - Pressures at each level
!  PFGASREF   - Fine grid pressure grid 
!  TGASREF    - Temperature grid for copacities
!  QXIDST     - Qext for dust
!  QSIDST     - Qscat for dust
!  GIDST      - g (scattering asymmetry) for dust
!  WBARI      - total scattering albedo
!  COSBI      - total scattering asymmetry
!  TAUREF     - Dust optical depth in each layer, at the reference 
!               wavelength (0.67 microns)
!  TMID       - Temperature at the layer mid-point
!  PMID       - Pressure at the layer mid-point
!  TAUGSURF   - Gas optical depth (from top of atmosphere to the
!               surface)
!  QH2O       - Water mixing ratio
!  WREFH2O    - Grid of water amounts
!  QEXTREFCLD - Qext for clouds at the reference wavelength
!  TAUREFCLD  - Cloud optical depth at the reference wavelength
!  QXICLD     - Qext for water ice clouds
!  QSICLD     - Qscat for water ice clouds
!  GICLD      - g (scattering asymmetry) for clouds
!
!----------------------------------------------------------------------C

use univ_numtype_constants_mod, only: epsilon_f8
      use ARCnRT_grid_h
      use ARCnRT_constants_h, only: Cmk
      use ARCnRT_radinc_h
use ARCnRT_memory_mod, only: use_gas_opacities
use ARCnRT_tpindex_mod, only: ARCnRT_tpindex

      implicit none

      real(f8), intent(INOUT) :: DTAUI(L_NLAYRAD,L_NSPECTI,L_NGAUSS)
      real(f8), intent(INOUT) :: TAUCUMI(L_LEVELS,L_NSPECTI,L_NGAUSS)
      real(f8), intent(INOUT) :: COSBI(L_NLAYRAD,L_NSPECTI,L_NGAUSS)
      real(f8), intent(INOUT) :: WBARI(L_NLAYRAD,L_NSPECTI,L_NGAUSS)
      real(f8), intent(INOUT) :: TAUREF(L_LEVELS+1)
      real(f8), intent(INOUT) :: TAUREFCLD(L_LEVELS+1)
      real(f8), intent(INOUT) :: taugsurf(L_NSPECTI,L_NGAUSS-1)
      real(f8), intent(IN) :: CO2I(L_NTREF,L_PINT,L_REFH2O,L_NSPECTI,L_NGAUSS)
      real(f8), intent(IN) :: PLEV(L_LEVELS), TGASREF(L_NTREF),  &
           PFGASREF(L_PINT), QREFV(L_LEVELS+1), Qxicld(L_LEVELS+1,L_NSPECTI),  &
           QXIDST(L_LEVELS+1,L_NSPECTI), QSIDST(L_LEVELS+1,L_NSPECTI),  &
           Qsicld(L_LEVELS+1,L_NSPECTI), gicld(L_LEVELS+1,L_NSPECTI),  &
           GIDST(L_LEVELS+1,L_NSPECTI), TMID(L_LEVELS), PMID(L_LEVELS),  &
           QH2O(L_LEVELS), WREFH2O(L_REFH2O), Qextrefcld(L_LEVELS+1)

      real(f8) :: DTAUKI(L_LEVELS+1,L_NSPECTI,L_NGAUSS)
      real(f8) :: TAUI(L_NLEVRAD,L_NSPECTI,L_NGAUSS)
      real(f8) :: TAUGAS

      integer :: L, NW, NG, K, LK
      integer :: MT(L_LEVELS), MP(L_LEVELS) !TIM, NP(L_LEVELS)
      real(f8) :: ANS !TIM, TAUREFL
      real(f8) :: TAEROS(L_LEVELS,L_NSPECTI)
      real(f8) :: DPR(L_LEVELS), U(L_LEVELS), TAUAC
      real(f8) :: LCOEF(4), LKCOEF(L_LEVELS,4)

!     For dust
      real(f8) :: TAUREF_save(L_LEVELS+1)

!     For clouds

      real(f8) :: TCLOUD(L_LEVELS,L_NSPECTI) !TIM,TAUREFLCLD

      real(f8) :: TAUREFLK(L_LEVELS+1,L_NSPECTI)
      real(f8) :: TAUCLDK(L_LEVELS+1,L_NSPECTI)

! fraction of zeros in each spectral interval, as a function of T, P

!TIM      real(f8) :: dt, tt

!  Water mixing ratio variables

      real(f8) :: WRATIO(L_LEVELS)
      real(f8) :: KCOEF(4)
      integer :: NH2O(L_LEVELS)

!======================================================================C

      do nw=1,L_NSPECTI
        do ng=1,L_NGAUSS
          DTAUKI(L_LEVELS+1,nw,ng)   = 0._f8
        end do
        TAUREFLK(L_LEVELS+1,nw) = 0._f8
        TAUCLDK(L_LEVELS+1,nw)  = 0._f8
      end do

!  Save old tauref values

      do k=1,L_LEVELS+1
        tauref_save(k) = tauref(k)
      end do

!  Determine the total gas opacity throughout the column, for each
!  spectral interval, NW, and each Gauss point, NG.

      DO NG=1,L_NGAUSS-1
        do NW=1,L_NSPECTI
          TAUGSURF(NW,NG) = 0._f8
        end do
      end do

      do K=2,L_LEVELS
        DPR(k) = PLEV(K)-PLEV(K-1)
        U(k)   = Cmk*DPR(k)

        call ARCnRT_tpindex(PMID(K),TMID(K),QH2O(K),pfgasref,tgasref,WREFH2O, &
                     LCOEF,MT(K),MP(K),NH2O(K),WRATIO(K))

        do LK=1,4
          LKCOEF(K,LK) = LCOEF(LK)
        end do

        TAUREF(K)    = TAUREF(K)    / Qrefv(K)
        TAUREFCLD(K) = TAUREFCLD(K) / Qextrefcld(K)

        DO NW=1,L_NSPECTI
          TAEROS(K,NW) = TAUREF(K)    * Qxidst(K,NW)
          TCLOUD(K,NW) = TAUREFCLD(K) * Qxicld(K,NW)
        END DO
      end do

if (.NOT.use_gas_opacities) then
   u = 0._f8
end if

      do K=2,L_LEVELS
        do nw=1,L_NSPECTI
          do ng=1,L_NGAUSS-1

!           NOW COMPUTE TAUGAS

!  Interpolate between water mixing ratios
!  WRATIO = 0.0 if the requested water amount is equal to, or outside the
!  the range of water amount data.

            KCOEF(1) = CO2I(MT(K),MP(K),NH2O(K),NW,NG) + WRATIO(K)*    &
                      (CO2I(MT(K),MP(K),NH2O(K)+1,NW,NG) -             &
                       CO2I(MT(K),MP(K),NH2O(K),NW,NG))

            KCOEF(2) = CO2I(MT(K),MP(K)+1,NH2O(K),NW,NG) + WRATIO(K)*  &
                      (CO2I(MT(K),MP(K)+1,NH2O(K)+1,NW,NG) -           &
                       CO2I(MT(K),MP(K)+1,NH2O(K),NW,NG))

            KCOEF(3) = CO2I(MT(K)+1,MP(K)+1,NH2O(K),NW,NG) + WRATIO(K)*&
                      (CO2I(MT(K)+1,MP(K)+1,NH2O(K)+1,NW,NG) -         &
                       CO2I(MT(K)+1,MP(K)+1,NH2O(K),NW,NG))

            KCOEF(4) = CO2I(MT(K)+1,MP(K),NH2O(K),NW,NG) + WRATIO(K)*  &
                      (CO2I(MT(K)+1,MP(K),NH2O(K)+1,NW,NG) -           &
                       CO2I(MT(K)+1,MP(K),NH2O(K),NW,NG))

!  Interpolate the CO2 k-coefficients to the requested T,P


            ANS = LKCOEF(K,1)*KCOEF(1) + LKCOEF(K,2)*KCOEF(2) +        &
                  LKCOEF(K,3)*KCOEF(3) + LKCOEF(K,4)*KCOEF(4)

            TAUGAS          = U(k)*ANS
            TAUGSURF(NW,NG) = TAUGSURF(NW,NG) + TAUGAS
            DTAUKI(K,nw,ng) = TAUGAS+TAEROS(K,NW)+TCLOUD(K,NW)
          end do

!  Now fill in the "clear" part of the spectrum (NG = L_NGAUSS)
!  Which holds continuum opacity only

          NG              = L_NGAUSS
          DTAUKI(K,nw,ng) = TAEROS(K,NW)+TCLOUD(K,NW)
        end do
      end do

!  Now the full treatment for the layers, where besides the opacity
!  we need to calculate the scattering albedo and asymmetry factors
!  for each layer

      DO NW=1,L_NSPECTI
        DO K=2,L_LEVELS+1
          TAUREFLK(K,NW) = TAUREF(K)    * QSIDST(K,NW)
          TAUCLDK(K,NW)  = TAUREFCLD(K) * QSICLD(K,NW)
        ENDDO
      ENDDO

      DO NW=1,L_NSPECTI

!  First, the special "clear" channel

        NG = L_NGAUSS

        DO L=1,L_NLAYRAD
          K              = 2*L+1
          DTAUI(L,nw,ng) = DTAUKI(K,NW,NG) + DTAUKI(K+1,NW,NG) + 1.e-50_f8
          if(DTAUI(L,NW,NG) .GT. 1.E-9_f8) then
            WBARI(L,nw,ng) = min((TAUREFLK(K,NW)+ TAUREFLK(K+1,NW) +       &
                              TAUCLDK(K,NW) + TAUCLDK(K+1,NW)   ) /    &
                              DTAUI(L,NW,NG), 1._f8-epsilon_f8)
          else
            WBARI(L,nw,ng) = 0._f8
            DTAUI(L,NW,NG) = 1.E-9_f8
          endif

          TAUAC = TAUREFLK(K,NW)+ TAUREFLK(K+1,NW) + TAUCLDK(K,NW) +   &
                  TAUCLDK(K+1,NW)

          if(TAUAC .GT. 0._f8) then
            cosbi(L,NW,NG) = ( GIDST(K,NW)   * TAUREFLK(K,NW) +        &
                               GIDST(K+1,NW) * TAUREFLK(K+1,NW) +      &
                               GICLD(K,NW)   * TAUCLDK(K,NW) +         &
                               GICLD(K+1,NW) * TAUCLDK(K+1,NW) ) /     &
                              (TAUREFLK(K,NW)+ TAUREFLK(K+1,NW) +      &
                               TAUCLDK(K,NW) + TAUCLDK(K+1,NW)   )
            cosbi(L,NW,NG) = min(max(cosbi(L,NW,NG), -1._f8+epsilon_f8),  &
                 1._f8-epsilon_f8)
          else
            cosbi(L,NW,NG) = 0._f8
          end if
        END DO

!  . . .Now the other Gauss points, if needed.

        DO NG=1,L_NGAUSS-1

          DO L=1,L_NLAYRAD
            K              = 2*L+1
            DTAUI(L,nw,ng) = DTAUKI(K,NW,NG)+DTAUKI(K+1,NW,NG)+1.e-50_f8
            if(DTAUI(L,NW,NG) .GT. 1.E-9_f8) then
              WBARI(L,nw,ng) = min((TAUREFLK(K,NW)+ TAUREFLK(K+1,NW) +     &
                                TAUCLDK(K,NW) + TAUCLDK(K+1,NW)   ) /  &
                                DTAUI(L,NW,NG), 1._f8-epsilon_f8)
            else
              WBARI(L,nw,ng) = 0._f8
              DTAUI(L,NW,NG) = 1.E-9_f8
            endif

            cosbi(L,NW,NG) = cosbi(L,NW,L_NGAUSS)
          END DO
        END DO

      END DO     ! NW spectral loop

!     TOTAL EXTINCTION OPTICAL DEPTHS

      DO NW=1,L_NSPECTI
        NG = L_NGAUSS
        TAUI(1,NW,NG) = 0._f8
        DO L=1,L_NLAYRAD
          TAUI(L+1,NW,NG) = TAUI(L,NW,NG)+DTAUI(L,NW,NG)
        END DO

        TAUCUMI(1,NW,NG)=0._f8
        DO K=2,L_LEVELS
          TAUCUMI(K,NW,NG)=TAUCUMI(K-1,NW,NG)+DTAUKI(K,NW,NG)
        END DO

        DO NG=1,L_NGAUSS-1
          TAUI(1,NW,NG)=0._f8
          DO L=1,L_NLAYRAD
            TAUI(L+1,NW,NG)=TAUI(L,NW,NG)+DTAUI(L,NW,NG)
          END DO

          TAUCUMI(1,NW,NG)=0._f8
          DO K=2,L_LEVELS
            TAUCUMI(K,NW,NG)=TAUCUMI(K-1,NW,NG)+DTAUKI(K,NW,NG)
          END DO
        END DO
      END DO

!  Restore old tauref values

      do k=1,L_LEVELS+1
        tauref(k) = tauref_save(k)
      end do

      RETURN
      end subroutine ARCnRT_optci


end module ARCnRT_optci_mod
