module ARCnRT_optcv_mod

implicit none


contains


      SUBROUTINE ARCnRT_OPTCV(DTAUV,TAUV,TAUCUMV,CO2V,PLEV,PFGASREF,          &
                       TGASREF,QXVDST,QSVDST,GVDST,WBARV,COSBV,        &
                       TAURAY,TAUREF,TMID,PMID,TAUGSURF,QH2O,WREFH2O,  &
                       Qextrefcld,taurefcld,Qxvcld,Qsvcld,gvcld)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

! THIS SUBROUTINE SETS THE OPTICAL CONSTANTS IN THE VISIBLE 
! IT CALCUALTES FOR EACH LAYER, FOR EACH SPECRAL INTERVAL IN THE VISIBLE
! LAYER: WBAR, DTAU, COSBAR
! LEVEL: TAU
!
! TAUV(L,NW,NG) is the cumulative optical depth at the top of radiation code
! layer L. NW is spectral wavelength interval, ng the Gauss point index.
!
!  DTAUV      - Visible opacity of layer L.
!  TAUV       - Visible opacity from top of the atmosphere to bottom
!               of layer L.
!  TLEV(L)    - Temperature at the layer boundary
!  PLEV(L)    - Pressure at the layer boundary (i.e. level)
!  CO2V       - Visible CO2 k-coefficients 
!  PLEV       - Pressures at each level
!  PFGASREF   - Fine grid pressure grid 
!  TGASREF    - Temperature grid for copacities
!  QXVDST     - Qext for dust
!  QSVDST     - Qscat for dust
!  GVDST      - g (scattering asymmetry) for dust
!  WBARV      - total scattering albedo
!  COSBV      - total scattering asymmetry
!  TAURAY     - Rayleigh scattering optical depth
!  TAUREF     - Dust optical depth in each layer, at the reference 
!               wavelength (0.67 microns)
!  TMID       - Temperature at the layer mid-point
!  PMID       - Pressure at the layer mid-point
!  TAUGSURF   - Gas optical depth (from top of atmosphere to the surface)
!  QH2O       - Water mixing ratio
!  WREFH2O    - Grid of water amounts
!  QEXTREFCLD - Qext for clouds at the reference wavelength
!  TAUREFCLD  - Cloud optical depth at the reference wavelength
!  QXVCLD     - Qext for water ice clouds
!  QSVCLD     - Qscat for water ice clouds
!  GVCLD      - g (scattering asymmetry) for clouds
!
!----------------------------------------------------------------------C

use univ_numtype_constants_mod, only: epsilon_f8
      use ARCnRT_grid_h
      use ARCnRT_constants_h, only: Cmk
      use ARCnRT_radinc_h
use ARCnRT_memory_mod, only: use_gas_opacities
use ARCnRT_tpindex_mod, only: ARCnRT_tpindex

      implicit none

      real(f8), intent(INOUT) :: DTAUV(L_NLAYRAD,L_NSPECTV,L_NGAUSS)
      real(f8)  :: DTAUKV(L_LEVELS+1,L_NSPECTV,L_NGAUSS)
      real(f8), intent(INOUT) :: TAUV(L_NLEVRAD,L_NSPECTV,L_NGAUSS)
      real(f8), intent(INOUT) :: TAUCUMV(L_LEVELS,L_NSPECTV,L_NGAUSS)
      real(f8), intent(IN) :: PLEV(L_LEVELS), TGASREF(L_NTREF),  &
           PFGASREF(L_PINT), QXVDST(L_LEVELS+1,L_NSPECTV),  &
           QSVDST(L_LEVELS+1,L_NSPECTV), GVDST(L_LEVELS+1,L_NSPECTV),  &
           TAURAY(L_NSPECTV), TMID(L_LEVELS), PMID(L_LEVELS), QH2O(L_LEVELS),  &
           WREFH2O(L_REFH2O), Qextrefcld(L_LEVELS+1),  &
           Qxvcld(L_LEVELS+1,L_NSPECTV), Qsvcld(L_LEVELS+1,L_NSPECTV),  &
           gvcld(L_LEVELS+1,L_NSPECTV)
      real(f8), intent(INOUT) :: taugsurf(L_NSPECTV,L_NGAUSS-1)
      real(f8), intent(INOUT) :: TAUREFCLD(L_LEVELS+1)
      real(f8), intent(IN) :: CO2V(L_NTREF,L_PINT,L_REFH2O,L_NSPECTV,L_NGAUSS)
      real(f8), intent(INOUT) :: COSBV(L_NLAYRAD,L_NSPECTV,L_NGAUSS)
      real(f8), intent(INOUT) :: WBARV(L_NLAYRAD,L_NSPECTV,L_NGAUSS)
      real(f8), intent(INOUT) :: TAUREF(L_LEVELS+1)


!     For dust
      real(f8) :: Qextref(L_LEVELS+1)
      real(f8) :: TAUREF_save(L_LEVELS+1)

!     For clouds

      real(f8) :: TCLOUD(L_LEVELS,L_NSPECTV)

      real(f8) :: TAUREFLK(L_LEVELS+1,L_NSPECTV)
      real(f8) :: TAUCLDK(L_LEVELS+1,L_NSPECTV)

!TIM      integer :: L, NW, NG, K, NG1(L_NSPECTV), LK
      integer :: L, NW, NG, K, LK
      integer :: MT(L_LEVELS), MP(L_LEVELS) !TIM, NP(L_LEVELS)
      real(f8) :: ANS, TAUGAS
      real(f8) :: TRAY(L_LEVELS,L_NSPECTV)
      real(f8) :: TAEROS(L_LEVELS,L_NSPECTV)
      real(f8) :: DPR(L_LEVELS), U(L_LEVELS)
      real(f8) :: LCOEF(4), LKCOEF(L_LEVELS,4)

      real(f8) :: TRAYAER

!  Reference wavelength is (now) bin #2 - put into qextref
!      real(f8) QextREF

!  Water mixing ratio stuff

      real(f8) :: WRATIO(L_LEVELS)
      real(f8) :: KCOEF(4)
      integer :: nh2o(L_LEVELS)

!======================================================================C

!  Save old tauref values

      do k=1,L_LEVELS+1
        tauref_save(k) = tauref(k)
      end do

!  Determine the total gas opacity throughout the column, for each
!  spectral interval, NW, and each Gauss point, NG.
!  Calculate the continuum opacities, i.e., those that do not depend on
!  NG, the Gauss index.

      DO NG=1,L_NGAUSS-1
        do NW=1,L_NSPECTV
          TAUGSURF(NW,NG) = 0._f8
        end do
      end do
     
      do K=2,L_LEVELS
        DPR(k) = PLEV(K)-PLEV(K-1)
        U(k)   = Cmk*DPR(k)
   
        call ARCnRT_tpindex(PMID(K),TMID(K),QH2O(K),pfgasref,tgasref,WREFH2O, &
                     LCOEF,MT(K),MP(K),NH2O(K),WRATIO(K))

        do LK=1,4
          LKCOEF(K,LK) = LCOEF(LK)
        end do

        Qextref(K) = QXVDST(K,L_NREFV)

        TAUREF(K)    = TAUREF(K) / Qextref(K)
        TAUREFCLD(K) = TAUREFCLD(K) / Qextrefcld(K)

        DO NW=1,L_NSPECTV
          TRAY(K,NW)   = TAURAY(NW)*DPR(K)
          TAEROS(K,NW) = TAUREF(K)    * QXVDST(K,NW)
          TCLOUD(K,NW) = TAUREFCLD(K) * QXVCLD(K,NW)
        END DO
      end do

if (.NOT.use_gas_opacities) then
   tray = 0._f8
   u = 0._f8
end if

!  TRAYAER is Tau RAYleigh scattering, plus AERosol opacity

      do K=2,L_LEVELS
 
        do NW=1,L_NSPECTV

          TRAYAER = TRAY(K,NW) + TAEROS(K,NW) + TCLOUD(K,NW)

          do NG=1,L_NGAUSS-1

!           NOW COMPUTE TAUGAS

!  Interpolate between water mixing ratios
!  WRATIO = 0.0 if the requested water amount is equal to, or outside the
!  the range of water amount data.

            KCOEF(1) = CO2V(MT(K),MP(K),NH2O(K),NW,NG) + WRATIO(K)*    &
                      (CO2V(MT(K),MP(K),NH2O(K)+1,NW,NG) -             &
                       CO2V(MT(K),MP(K),NH2O(K),NW,NG))

            KCOEF(2) = CO2V(MT(K),MP(K)+1,NH2O(K),NW,NG) + WRATIO(K)*  &
                      (CO2V(MT(K),MP(K)+1,NH2O(K)+1,NW,NG) -           &
                       CO2V(MT(K),MP(K)+1,NH2O(K),NW,NG))

            KCOEF(3) = CO2V(MT(K)+1,MP(K)+1,NH2O(K),NW,NG) + WRATIO(K)*&
                      (CO2V(MT(K)+1,MP(K)+1,NH2O(K)+1,NW,NG) -         &
                       CO2V(MT(K)+1,MP(K)+1,NH2O(K),NW,NG))

            KCOEF(4) = CO2V(MT(K)+1,MP(K),NH2O(K),NW,NG) + WRATIO(K)*  &
                      (CO2V(MT(K)+1,MP(K),NH2O(K)+1,NW,NG) -           &
                       CO2V(MT(K)+1,MP(K),NH2O(K),NW,NG))

!  Interpolate the CO2 k-coefficients to the requested T,P

      
            ANS = LKCOEF(K,1)*KCOEF(1) + LKCOEF(K,2)*KCOEF(2) +        &
                  LKCOEF(K,3)*KCOEF(3) + LKCOEF(K,4)*KCOEF(4)

            TAUGAS          = U(k)*ANS
            TAUGSURF(NW,NG) = TAUGSURF(NW,NG) + TAUGAS
            DTAUKV(K,nw,ng) = TAUGAS + TRAYAER 
          end do

!  Now fill in the "clear" part of the spectrum (NG = L_NGAUSS)
!  Which holds continuum opacity only

          NG = L_NGAUSS
          DTAUKV(K,nw,ng) = TAEROS(K,NW)+TRAY(K,NW)+TCLOUD(K,NW)

        end do
      end do

!  Now the full treatment for the layers, where besides the opacity
!  we need to calculate the scattering albedo and asymmetry factors
!  for each layer

      DO NW=1,L_NSPECTV
        DO K=2,L_LEVELS
          TAUREFLK(K,NW) = TAUREF(K)    * QSVDST(K,NW)
          TAUCLDK(K,NW)  = TAUREFCLD(K) * QSVCLD(K,NW)
        ENDDO
      ENDDO

      DO NW=1,L_NSPECTV

!  First, the special "clear" channel
 
        NG = L_NGAUSS
        DO L=1,L_LAYERS
          K              = 2*L+1
          DTAUV(L,nw,ng) = DTAUKV(K,NW,NG)+DTAUKV(K+1,NW,NG)
          COSBV(L,NW,NG) = ( GVDST(K,NW)  * TAUREFLK(K,NW) +           &
                             GVDST(K+1,NW)* TAUREFLK(K+1,NW) +         &
                             GVCLD(K,NW)  * TAUCLDK(K,NW)  +           &
                             GVCLD(K+1,NW)* TAUCLDK(K+1,NW) ) /        &
                           ( TRAY(K,NW)     + TRAY(K+1,NW) +           &
                             TAUREFLK(K,NW) + TAUREFLK(K+1,NW) +       &
                             TAUCLDK(K,NW)  + TAUCLDK(K+1,NW) )
          cosbv(L,NW,NG) = min(max(cosbv(L,NW,NG), -1._f8+epsilon_f8),  &
               1._f8-epsilon_f8)

          WBARV(L,nw,ng) = min(( TAUREFLK(K,NW) + TAUREFLK(K+1,NW) +       &
                             TAUCLDK(K,NW)  + TAUCLDK(K+1,NW)  +       &
                            (TRAY(K,NW)+TRAY(K+1,NW))*0.9999_f8)/         &
                             DTAUV(L,nw,ng), 1._f8-epsilon_f8)
        END DO

!  Special bottom layer

        L              = L_NLAYRAD
        K              = 2*L+1
        DTAUV(L,nw,ng) = DTAUKV(K,NW,NG)
        COSBV(L,NW,NG) = ( GVDST(K,NW) * TAUREFLK(K,NW) +              &
                           GVCLD(K,NW) * TAUCLDK(K,NW) ) /             &
                         ( TRAY(K,NW)  + TAUREFLK(K,NW) +              &
                           TAUCLDK(K,NW) )
        cosbv(L,NW,NG) = min(max(cosbv(L,NW,NG), -1._f8+epsilon_f8),  &
             1._f8-epsilon_f8)

        WBARV(L,nw,ng) = min((TAUREFLK(K,NW) + TAUCLDK(K,NW) +             &
                          TRAY(K,NW)*0.9999_f8)/DTAUV(L,nw,ng), 1._f8-epsilon_f8)

!  . . .Now the other Gauss points, if needed.

        DO NG=1,L_NGAUSS-1
          DO L=1,L_LAYERS
            K              = 2*L+1
            DTAUV(L,nw,ng) = DTAUKV(K,NW,NG)+DTAUKV(K+1,NW,NG)
            COSBV(L,NW,NG) = COSBV(L,NW,L_NGAUSS)
            WBARV(L,nw,ng) = min(( TAUREFLK(K,NW) + TAUREFLK(K+1,NW) +     &
                               TAUCLDK(K,NW)  + TAUCLDK(K+1,NW) +      &
                              (TRAY(K,NW)+TRAY(K+1,NW))*0.9999_f8)/       &
                               DTAUV(L,nw,ng), 1._f8-epsilon_f8)
          END DO

!  Special bottom layer

          L              = L_NLAYRAD
          K              = 2*L+1
          DTAUV(L,nw,ng) = DTAUKV(K,NW,NG)
          COSBV(L,NW,NG) = COSBV(L,NW,L_NGAUSS)
          WBARV(L,nw,ng) = min(( TAUREFLK(K,NW) + TAUCLDK(K,NW) +          &
                             TRAY(K,NW)*0.9999_f8 ) / DTAUV(L,nw,ng), 1._f8-epsilon_f8)
        END DO

      END DO     ! NW spectral loop

!     TOTAL EXTINCTION OPTICAL DEPTHS

      DO NW=1,L_NSPECTV
        NG = L_NGAUSS
        TAUV(1,NW,NG) = 0._f8
        DO L=1,L_NLAYRAD
          TAUV(L+1,NW,NG) = TAUV(L,NW,NG)+DTAUV(L,NW,NG)
        END DO

        TAUCUMV(1,NW,NG)=0._f8
        DO K=2,L_LEVELS
          TAUCUMV(K,NW,NG)=TAUCUMV(K-1,NW,NG)+DTAUKV(K,NW,NG)
        END DO
  
        DO NG=1,L_NGAUSS-1
          TAUV(1,NW,NG)=0._f8
          DO L=1,L_NLAYRAD
            TAUV(L+1,NW,NG)=TAUV(L,NW,NG)+DTAUV(L,NW,NG)
          END DO

          TAUCUMV(1,NW,NG)=0._f8
          DO K=2,L_LEVELS
            TAUCUMV(K,NW,NG)=TAUCUMV(K-1,NW,NG)+DTAUKV(K,NW,NG)
          END DO
        END DO
      END DO

!  Restore old tauref values

      do k=1,L_LEVELS+1
        tauref(k) = tauref_save(k)
      end do

      RETURN
      end subroutine ARCnRT_optcv


end module ARCnRT_optcv_mod
