module ARCnRT_setrad_mod

implicit none


contains


      subroutine ARCnRT_setrad(TGASREF,PFGASREF,CO2V,CO2I,QEXTV,QSCATV,WV,GV, &
                        QEXTI,QSCATI,WI,GI,QEXTVc,QSCATVc,WVc,GVc,     &
                        QEXTIc,QSCATIc,WIc,GIc,FZEROI,FZEROV)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

!     PURPOSE:
!        Set up values used by the radiation code, such as the CO2 gas
!     absorption coefficients.  True constants are defined, and the 
!     time-independent quantities used by the radiation code are 
!     calculated. 
!
!     TGASREF        - Temperatures of the opacity grid
!     PFGASREF       - Pressures (on fine mesh) of the opacity grid
!     CO2V           - Visible CO2 k-coefficients (CO2 gas opacity)
!     CO2I           - IR CO2 k-coefficients (CO2 gas opacity)
!     FZEROV         - Fraction of zeros in the visible (k-coefficient
!                      off-line calculation)
!     FZEROI         - Fraction of zeros in the IR (k-coefficient
!                      off-line calculation)
!
!     AEROSOL RADIATIVE OPTICAL CONSTANTS
!     Values are at the wavelenght interval center
!
!  Dust
!
!     MIE SCATTERING - Size distribution weighted
!     Qextv    - Extinction efficiency - in the visible.
!     Qscatv   - Scattering efficiency - in the visible.
!     WV       - Single scattering albedo - in the visible.
!     GV       - Asymmetry parameter - in the visible.
!
!     Qexti    - Extinction efficiency - in the infrared.
!     Qscati   - Scattering efficiency - in the infrared.
!     WI       - Single scattering albedo - in the infrared.
!     GI       - Asymmetry parameter - in the infrared.
!     
!  Water ice cloud constants
!
!     MIE SCATTERING - Size distribution weighted
!     Qextvc   - Extinction efficiency - in the visible.
!     Qscatvc  - Scattering efficiency - in the visible.
!     WVc      - Single scattering albedo - in the visible.
!     GVc      - Asymmetry parameter - in the visible.
!
!     Qextic   - Extinction efficiency - in the infrared.
!     Qscatic  - Scattering efficiency - in the infrared.
!     WIc      - Single scattering albedo - in the infrared.
!     GIc      - Asymmetry parameter - in the infrared.
!     
!----------------------------------------------------------------------C

      use ARCnRT_grid_h
      use ARCnRT_radinc_h
use univ_std_fileIO_mod, only: find_file_IOunit
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use aermic_rad_mem_mod, only: dataset_path_bspec
use ARCnRT_laginterp_mod, only: ARCnRT_laginterp

      implicit none

      real(f8), intent(INOUT) :: TGASREF(L_NTREF), PFGASREF(L_PINT),  &
           qextv(L_NSPECTV), qscatv(L_NSPECTV),  &
           wv(L_NSPECTV), gv(L_NSPECTV), qexti(L_NSPECTI), qscati(L_NSPECTI),  &
           wi(L_NSPECTI), gi(L_NSPECTI), fzeroi(L_NSPECTI),  &
           qextvc(L_NSPECTV), qscatvc(L_NSPECTV), wvc(L_NSPECTV),  &
           gvc(L_NSPECTV), qextic(L_NSPECTI), qscatic(L_NSPECTI),  &
           wic(L_NSPECTI), gic(L_NSPECTI), fzerov(L_NSPECTV)
      real(f8), intent(INOUT) :: CO2I(L_NTREF,L_PINT,L_REFH2O,L_NSPECTI,L_NGAUSS)
      real(f8), intent(INOUT) :: CO2V(L_NTREF,L_PINT,L_REFH2O,L_NSPECTV,L_NGAUSS)

!TIM      integer :: N, NS, ios, nd
      integer :: N, ios, nd

      real(f8) :: PGASREF(L_NPREF)

!     real(f8) kvis, kir
!TIM      integer :: nt, np, nw, ng

integer :: IO_unit
character(len=MAX_FILE_PATH_LEN) :: fn_tmp
 
!=======================================================================

!  Read in the dust and water ice optical parameters:  Qext, Qscat,
!  w0, and g

!  Dust
IO_unit = find_file_IOunit()
fn_tmp = trim(dataset_path_bspec)//'/QEXT_DUST'
      open(IO_unit, file=trim(fn_tmp), status='old', iostat=ios)
      if(ios.ne.0) then
        write(6,'("ARCnRT_setrad.f90:  Could not open dust Qext file")')
        stop
      end if

!  Visible

      do n=1,3
        read(IO_unit,*) !header
      end do

      do n=1,L_NSPECTV 
        read(IO_unit,*) nd, Qextv(n), Qscatv(n), wv(n), gv(n)
      end do

!  IR

      do n=1,3
        read(IO_unit,*) !header
      end do

      do n=1,L_NSPECTI 
        read(IO_unit,*) nd, Qexti(n), Qscati(n), wi(n), gi(n)
      end do

      close(IO_unit)

!  Water ice cloud

IO_unit = find_file_IOunit()
fn_tmp = trim(dataset_path_bspec)//'/QEXT_WATER'
      open(IO_unit, file=trim(fn_tmp), status='old', iostat=ios)
      if(ios.ne.0) then
        write(6,'("ARCnRT_setrad.f90:  Could not open H2O Qext file")')
        stop
      end if

!  Visible

      do n=1,3
        read(IO_unit,*) !header
      end do

      do n=1,L_NSPECTV 
        read(IO_unit,*) nd, Qextvc(n), Qscatvc(n), wvc(n), gvc(n)
      end do

!  IR

      do n=1,3
        read(IO_unit,*) !header
      end do

      do n=1,L_NSPECTI 
        read(IO_unit,*) nd, Qextic(n), Qscatic(n), wic(n), gic(n)
      end do

      close(IO_unit)
      
!     Set the reference pressure and temperature arrays.  These are
!     the pressures and temperatures at which we have k-coefficients.

      pgasref( 1) = 1.E-6_f8
      pgasref( 2) = 1.E-5_f8
      pgasref( 3) = 1.E-4_f8
      pgasref( 4) = 1.E-3_f8
      pgasref( 5) = 1.E-2_f8
      pgasref( 6) = 1.E-1_f8
      pgasref( 7) = 1._f8
      pgasref( 8) = 1.E+1_f8
      pgasref( 9) = 1.E+2_f8
      pgasref(10) = 1.E+3_f8
      pgasref(11) = 1.E+4_f8

      tgasref(1)  =  50._f8
      tgasref(2)  = 100._f8
      tgasref(3)  = 150._f8
      tgasref(4)  = 200._f8
      tgasref(5)  = 250._f8
      tgasref(6)  = 300._f8
      tgasref(7)  = 350._f8
 
!     Insure that w0 < 1

      DO N=1,L_NSPECTV
        IF(wv(n).ge.0.9999_f8) then
          Qscatv(n) = 0.9999_f8*Qextv(n)
          wv(n)     = 0.9999_f8
        END IF
      END DO

      DO N=1,L_NSPECTI
        IF(wi(n).ge.0.9999_f8) then
          Qscati(n) = 0.9999_f8*Qexti(n)
          wi(n)     = 0.9999_f8
        END IF
      END DO

!  Fill the water ice cloud variables

!     Insure that w0 < 1

      DO N=1,L_NSPECTV
        IF(wvc(n).ge.0.9999_f8) then
          Qscatvc(n) = 0.9999_f8*Qextvc(n)
          wvc(n)     = 0.9999_f8
        END IF
      END DO

      DO N=1,L_NSPECTI
        IF(wic(n).ge.0.9999_f8) then
          Qscatic(n) = 0.9999_f8*Qextic(n)
          wic(n)     = 0.9999_f8
        END IF
      END DO

!     Interpolate CO2 k coefficients to the finer pressure grid.

      call ARCnRT_laginterp(PGASREF,PFGASREF,CO2I,CO2V,FZEROI,FZEROV)

      return
      end subroutine ARCnRT_setrad


end module ARCnRT_setrad_mod
