module ARCnRT_setspi_mod

implicit none


contains


      subroutine ARCnRT_setspi(WNOI,DWNI,WAVEI)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

!     PURPOSE:
!        Set up the spectral intervals in the infrared.  Based on
!     Chris McKay's SETSPI code.
!
!     INPUT PARAMETERS
!     L_NSPECTI  - Number of spectral intervals in the INFRARED
!
!     OUTPUT PARAMETERS
!     WNOI       - Array of wavenumbers at the spectral interval
!                  centers for the infrared.  Array is NSPECTI
!                  elements long.
!     DWNI       - Array of "delta wavenumber", i.e., the width,
!                  in wavenumbers (cm^-1) of each IR spectral
!                  interval.  NSPECTI elements long.
!     WAVEI      - Array (NSPECTI elements long) of the wavelenght
!                  (in microns) at the center of each IR spectral
!                  interval.
!     
!**********************************************************************C

      use ARCnRT_grid_h
      use ARCnRT_constants_h, only: PI
      use ARCnRT_radinc_h
      use ARCnRT_radcommon_h, only: planckir

      implicit none

!     BWNI - Bin wavenumber of the edges of the IR spectral bins
!     units are inverse centimeters.  Dimension needs to be changed
!     if the number of IR bins changes.

      real(f8), intent(OUT) :: WNOI(L_NSPECTI), DWNI(L_NSPECTI),  &
           WAVEI(L_NSPECTI)

      REAL(f8) :: BWNI(L_NSPECTI+1)

      real(f8) :: a, b, ans, y, bpa, bma, T
!TIM      real(f8) :: wn1, wn2
!TIM      integer :: n, nw, nt, m
      integer :: nw, nt, m

!  C1 and C2 values from Goody and Yung (2nd edition)  MKS units
!  These values lead to a "sigma" (sigma*T^4) of 5.67032E-8 W m^-2 K^-4

      real(f8) :: c1 = 3.741832e-16_f8      ! W m^-2
      real(f8) :: c2 = 1.438786e-2_f8       ! m K
      
      real(f8) :: x(12) = [ -0.981560634246719_f8,  -0.904117256370475_f8, &
                          -0.769902674194305_f8,  -0.587317954286617_f8, &
                          -0.367831498998180_f8,  -0.125233408511469_f8, &
                           0.125233408511469_f8,   0.367831498998180_f8, &
                           0.587317954286617_f8,   0.769902674194305_f8, &
                           0.904117256370475_f8,   0.981560634246719_f8 ]

      real(f8) :: w(12) = [  0.047175336386512_f8,   0.106939325995318_f8, &
                           0.160078328543346_f8,   0.203167426723066_f8, &
                           0.233492536538355_f8,   0.249147045813403_f8, &
                           0.249147045813403_f8,   0.233492536538355_f8, &
                           0.203167426723066_f8,   0.160078328543346_f8, &
                           0.106939325995318_f8,   0.047175336386512_f8 ]

!======================================================================C

!     Bin wavenumber - wavenumber [cm^(-1)] at the edges of the IR
!     spectral bins.

      BWNI( 1) =   10._f8       ! 1000.0 microns
      BWNI( 2) =  166.667_f8       !   60.0 microns
      BWNI( 3) =  416.667_f8       !   24.0 microns
      BWNI( 4) =  833.333_f8       !   12.0 microns
      BWNI( 5) = 1250._f8       !    8.0 microns
      BWNI( 6) = 2222.222_f8       !    4.5 microns

!     Set up mean wavenumbers and wavenumber deltas.  Units of 
!     wavenumbers is cm^(-1); units of wavelengths is microns.

      do M=1,L_NSPECTI
        WNOI(M)  = 0.5_f8*(BWNI(M+1)+BWNI(M))
        DWNI(M)  = BWNI(M+1)-BWNI(M)
        WAVEI(M) = 1.e+4_f8/WNOI(M)
      end do

!  For each IR wavelength interval, compute the integral of B(T), the
!  Planck function, divided by the wavelength interval, in cm-1.  The
!  integration is in MKS units, the final answer is the same as the
!  original planck.f; W m^-2 wavenumber^-1, where wavenumber is in CM^-1.

      DO NW=1,L_NSPECTI
        a = 1.e-2_f8/BWNI(NW+1)
        b = 1.e-2_f8/BWNI(NW)
        bpa = (b+a)/2._f8
        bma = (b-a)/2._f8
        do nt=500,9000
          T   = dble(NT)/1.e+1_f8
          ans = 0._f8
          do m=1,12
            y    = bma*x(m)+bpa
            ans  = ans + w(m)*c1/(y**5*(exp(c2/(y*T))-1._f8))
          end do
          planckir(NW,nt-499) = ans*bma/(PI*DWNI(NW))
        end do
      END DO

      return
      end subroutine ARCnRT_setspi


end module ARCnRT_setspi_mod
