module ARCnRT_setspv_mod

implicit none


contains


      SUBROUTINE ARCnRT_SETSPV(WNOV,DWNV,WAVEV,SOLARF,TAURAY)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

!     PURPOSE:
!        Set up the spectral intervals in the visible (solar).  Based
!     on Chris McKay's SETSPV code.
!
!     INPUT PARAMETERS
!     L_NSPECTV  - Number of spectral intervals in the visible
!
!     OUTPUT PARAMETERS
!     WNOV       - Array of wavenumbers at the spectral interval
!                  center for the visible.  Array is NSPECTV
!                  elements long.
!     DWNV       - Array of "delta wavenumber", i.e., the width,
!                  in wavenumbers (cm^-1) of each visible spectral
!                  interval.  NSPECTV elements long.
!     WAVEV      - Array (NSPECTV elements long) of the wavelenght
!                  (in microns) at the center of each visible spectral
!                  interval.
!     SOLARF     - Array (NSPECTV elements) of solar flux (W/M^2) in
!                  each spectral interval.  Values are for 1 AU, and
!                  are scaled to the Mars distance elsewhere.
!     TAURAY     - Array (NSPECTV elements) of the wavelength independent
!                  part of Rayleigh Scattering.  The pressure dependent 
!                  part is computed elsewhere (OPTCV).
!
!**********************************************************************C

      use ARCnRT_grid_h
      use ARCnRT_constants_h, only: SCALEP
      use ARCnRT_radinc_h
use ARCnRT_memory_mod, only: grav, solar_sw

      implicit none

!     BWNV - Bin wavenumber of the edges of the visible spectral bins
!     units are inverse centimeters.  Dimension needs to be changed
!     if the number of visible bins changes.

      real(f8), intent(OUT) :: WNOV(L_NSPECTV), DWNV(L_NSPECTV),  &
           WAVEV(L_NSPECTV), SOLARF(L_NSPECTV), TAURAY(L_NSPECTV)

      REAL(f8) :: SUMm, WL
      INTEGER :: N, M

!     P0      - Rayleigh scattering reference pressure in pascals.
!     GRAV    - Acceleration due to gravity (g) - MKS

      real(f8) :: P0 = 9.423e+6_f8

!     Bin wavenumber - wavenumber [cm^(-1)] at the edges of the visible
!     spectral bins.  Go from smaller to larger wavenumbers, the same as
!     in the IR.

!     2222.22D0    ->   4.50 microns
!     3087.37D0    ->   3.24 microns
!     4030.63D0    ->   2.48 microns
!     5370.57D0    ->   1.86 microns
!     7651.11D0    ->   1.31 microns
!     12500.00D0   ->   0.80 microns
!     25000.00D0   ->   0.40 microns            
!     41666.67D0   ->   0.24 microns

      REAL(f8) :: BWNV(L_NSPECTV+1) = [ 2222.22_f8, 3087.37_f8, 4030.63_f8, &
                                     5370.57_f8, 7651.11_f8, 12500._f8, &
                                     25000._f8, 41666.67_f8 ]

!     Solar flux within each spectral interval, at 1AU (W/M^2)
!     Sum equals 1356 W/m^2 (values from Wehrli, 1985)

      real(f8) :: SOLAR(L_NSPECTV) = [ 12.7_f8, 24.2_f8, 54.6_f8, 145.9_f8, 354.9_f8,   &
                                     657.5_f8, 106.3_f8 ]

!======================================================================C

solar_sw = sum(SOLAR)  ! Solar flux (@ 1AU) within this RT code's "shortwave"
                       !  spectral range [W/m^2]

!     Set up mean wavenumbers and wavenumber deltas.  Units of 
!     wavenumbers is cm^(-1); units of wavelengths is microns.

      do M=1,L_NSPECTV
        WNOV(M)  = 0.5_f8*(BWNV(M+1)+BWNV(M))
        DWNV(M)  = BWNV(M+1)-BWNV(M)
        WAVEV(M) = 1.E+4_f8/WNOV(M)
      end do

!     Sum the solar flux, and write out the result.  

      summ = 0._f8
      do N=1,L_NSPECTV
        SOLARF(N) = SOLAR(N)
        summ       = summ+SOLARF(N)
      end do
      write(6,'("Solar flux at 1AU = ",f7.2," W/M^2")') summ

!     Set up the wavelength independent part of Rayleigh Scattering.
!     The pressure dependent part will be computed elsewhere (OPTCV).
!     WAVEV is in microns.  There is no Rayleigh scattering in the IR.

      do N=1,L_NSPECTV
        WL        = WAVEV(N)
        TAURAY(N) = (8.7_f8/grav)*(1.527_f8*(1._f8+0.013_f8/wl**2)/wl**4)*        &
                     scalep/P0
      end do

      RETURN
      end subroutine ARCnRT_setspv


end module ARCnRT_setspv_mod
