module ARCnRT_sfluxi_mod

implicit none


contains


      SUBROUTINE ARCnRT_SFLUXI(PLEV,TLEV,DTAUI,TAUCUMI,UBARI,RSFI,DWNI,  &
                        COSBI,WBARI,GWEIGHT,NFLUXTOPI,FMNETI,          &
                        fluxupi,fluxdni,FZEROI,TAUGSURF)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

      use ARCnRT_grid_h
      use ARCnRT_radinc_h
      use ARCnRT_radcommon_h, only: planckir, tlimiti
use ARCnRT_memory_mod, only: want_rtflux_output, up_fluxes, dn_fluxes
use ARCnRT_gfluxi_mod, only: ARCnRT_gfluxi

      implicit none

!  PLEV      - Pressures at levels (i.e. layer boundaries and 
!              mid-point)
!  TLEV      - Temperatures at levels
!  DTAUI     - IR opacity of layer L.
!  TAUCUMI   - IR opacity from top of atmosphere to the bottom
!              of level K.
!  RSFI      - Surface albedo in the IR.
!  DWNI      - Width of each IR spectral interval
!  WBARI     - Scattering parameter
!  COSBI     - Scattering asymetry parameter
!  UBARI     - Cosine of the incidence angle
!  GWEIGHT   - Gauss weight
!  NFLUXTOPI - Net flux at the top of the atmosphere 
!  FMNETI    - Net flux at the bottom of layer L
!  FLUXUPI   - Upward flux at layer L
!  FLUXDNI   - Downward flux at layer L
!  FZEROI    - Fraction of zeros (gas opacity k-coefficient off-line
!              computations) in each spectral interval.
!  TAUGSURF  - Gas optical depth (top of atmosphere to the surface).
!
!----------------------------------------------------------------------!

      real(f8), intent(IN) :: TLEV(L_LEVELS), PLEV(L_LEVELS),  &
           DTAUI(L_NLAYRAD,L_NSPECTI,L_NGAUSS), UBARI, RSFI,  &
           TAUCUMI(L_LEVELS,L_NSPECTI,L_NGAUSS),  &
           DWNI(L_NSPECTI), COSBI(L_NLAYRAD,L_NSPECTI,L_NGAUSS),  &
           WBARI(L_NLAYRAD,L_NSPECTI,L_NGAUSS), GWEIGHT(L_NGAUSS),  &
           FZEROI(L_NSPECTI), taugsurf(L_NSPECTI,L_NGAUSS-1)
      real(f8), intent(OUT) :: NFLUXTOPI, FMNETI(L_NLAYRAD),  &
           fluxupi(L_NLAYRAD), fluxdni(L_NLAYRAD)

      integer :: NLEVRAD, L, NW, NG, NTS, NTT
      real(f8) :: FMUPI(L_NLEVRAD), FMDI(L_NLEVRAD)
      real(f8) :: FTOPUP

      real(f8) :: TSURF, BSURF, TTOP, BTOP, TAUTOP
!TIM      real(f8) :: PLANCK, PLTOP
      real(f8) :: PLTOP
      real(f8) :: fzero

!======================================================================C
 
      NLEVRAD = L_NLEVRAD
 
!     ZERO THE NET FLUXES
    
      NFLUXTOPI = 0._f8
 
      DO L=1,L_NLAYRAD
        FMNETI(L)  = 0._f8
        FLUXUPI(L) = 0._f8
        FLUXDNI(L) = 0._f8
      END DO
 
!     WE NOW ENTER A MAJOR LOOP OVER SPECTRAL INTERVALS IN THE INFRARED
!     TO CALCULATE THE NET FLUX IN EACH SPECTRAL INTERVAL

      TTOP  = TLEV(2)
      TSURF = TLEV(L_LEVELS)

!TIM      NTS   = TSURF*10._f8-499
!TIM      NTT   = TTOP *10._f8-499
      NTS   = int(TSURF*10._f8)-499
      NTT   = int(TTOP *10._f8)-499

      DO 501 NW=1,L_NSPECTI

!       SURFACE EMISSIONS - INDEPENDENT OF GAUSS POINTS

        BSURF = (1._f8-RSFI)*PLANCKIR(NW,NTS)
        PLTOP = PLANCKIR(NW,NTT)

!  If FZEROI(NW) = 1, then the k-coefficients are zero - skip to the
!  special Gauss point at the end.
 
        FZERO = FZEROI(NW)
        IF(FZERO.ge.0.99_f8) goto 40
 
        DO NG=1,L_NGAUSS-1
         
          if(TAUGSURF(NW,NG).lt. TLIMITI) then
            fzero = fzero + (1._f8-FZEROI(NW))*GWEIGHT(NG)
            goto 30
          end if

!         SET UP THE UPPER AND LOWER BOUNDARY CONDITIONS ON THE IR
!         CALCULATE THE DOWNWELLING RADIATION AT THE TOP OF THE MODEL
!         OR THE TOP LAYER WILL COOL TO SPACE UNPHYSICALLY
 
          TAUTOP = DTAUI(1,NW,NG)*PLEV(2)/(PLEV(4)-PLEV(2))
          BTOP   = (1._f8-EXP(-TAUTOP/UBARI))*PLTOP
 
!         WE CAN NOW SOLVE FOR THE COEFFICIENTS OF THE TWO STREAM
!         CALL A SUBROUTINE THAT SOLVES  FOR THE FLUX TERMS
!         WITHIN EACH INTERVAL AT THE MIDPOINT WAVENUMBER 
       
          CALL ARCnRT_GFLUXI(NLEVRAD,TLEV,NW,DWNI(NW),DTAUI(:,NW,NG),         &
                      TAUCUMI(:,NW,NG),                                &
                      WBARI(:,NW,NG),COSBI(:,NW,NG),UBARI,RSFI,BTOP,   &
                      BSURF,FTOPUP,FMUPI,FMDI)

!         NOW CALCULATE THE CUMULATIVE IR NET FLUX

          NFLUXTOPI = NFLUXTOPI+FTOPUP*DWNI(NW)*GWEIGHT(NG)*           &
                                 (1._f8-FZEROI(NW))

          DO L=1,L_NLEVRAD-1

!           CORRECT FOR THE WAVENUMBER INTERVALS

            FMNETI(L)  = FMNETI(L)+(FMUPI(L)-FMDI(L))*DWNI(NW)*        &
                                    GWEIGHT(NG)*(1._f8-FZEROI(NW))
            FLUXUPI(L) = FLUXUPI(L) + FMUPI(L)*DWNI(NW)*GWEIGHT(NG)*   &
                                      (1._f8-FZEROI(NW))
            FLUXDNI(L) = FLUXDNI(L) + FMDI(L)*DWNI(NW)*GWEIGHT(NG)*    &
                                      (1._f8-FZEROI(NW))
          END DO

if (want_rtflux_output) then  ! Calc/save 2-stream fluxes [f(wavelength)]
   do l=1,l_nlevrad-1
      up_fluxes(l,nw) = up_fluxes(l,nw)+FMUPI(L)*DWNI(NW)*GWEIGHT(NG)*  &
           (1._f8-FZEROI(NW))
      dn_fluxes(l,nw) = dn_fluxes(l,nw)+FMDI(L)*DWNI(NW)*GWEIGHT(NG)*  &
           (1._f8-FZEROI(NW))
   end do
end if

   30     CONTINUE

       END DO       !End NGAUSS LOOP

   40  CONTINUE

!      SPECIAL 17th Gauss point

       NG     = L_NGAUSS

       TAUTOP = DTAUI(1,NW,NG)*PLEV(2)/(PLEV(4)-PLEV(2))
       BTOP   = (1._f8-EXP(-TAUTOP/UBARI))*PLTOP

!      WE CAN NOW SOLVE FOR THE COEFFICIENTS OF THE TWO STREAM
!      CALL A SUBROUTINE THAT SOLVES  FOR THE FLUX TERMS
!      WITHIN EACH INTERVAL AT THE MIDPOINT WAVENUMBER 

       CALL ARCnRT_GFLUXI(NLEVRAD,TLEV,NW,DWNI(NW),DTAUI(:,NW,NG),            &
                      TAUCUMI(:,NW,NG),                                &
                      WBARI(:,NW,NG),COSBI(:,NW,NG),UBARI,RSFI,BTOP,   &
                      BSURF,FTOPUP,FMUPI,FMDI)
 
!      NOW CALCULATE THE CUMULATIVE IR NET FLUX

       NFLUXTOPI = NFLUXTOPI+FTOPUP*DWNI(NW)*FZERO

       DO L=1,L_NLEVRAD-1

!        CORRECT FOR THE WAVENUMBER INTERVALS

         FMNETI(L)  = FMNETI(L)+(FMUPI(L)-FMDI(L))*DWNI(NW)*FZERO
         FLUXUPI(L) = FLUXUPI(L) + FMUPI(L)*DWNI(NW)*FZERO
         FLUXDNI(L) = FLUXDNI(L) + FMDI(L)*DWNI(NW)*FZERO
       END DO

if (want_rtflux_output) then  ! Calc/save 2-stream fluxes [f(wavelength)]
   do l=1,l_nlevrad-1
      up_fluxes(l,nw) = up_fluxes(l,nw)+FMUPI(L)*DWNI(NW)*FZERO
      dn_fluxes(l,nw) = dn_fluxes(l,nw)+FMDI(L)*DWNI(NW)*FZERO
   end do
end if
  
  501 CONTINUE      !End Spectral Interval LOOP

! *** END OF MAJOR SPECTRAL INTERVAL LOOP IN THE INFRARED****

      RETURN
      end subroutine ARCnRT_sfluxi


end module ARCnRT_sfluxi_mod
