module ARCnRT_sfluxv_mod

implicit none


contains


      SUBROUTINE ARCnRT_SFLUXV(DTAUV,TAUV,TAUCUMV,RSFV,WBARV,COSBV,      &
                        UBAR0,SOL,GWEIGHT,NFLUXTOPV,FMNETV,            &
                        FLUXUPV,FLUXDNV,DIFFVT,FZEROV,taugsurf,        &
                        detau)

!     GCM v23   2010
!     Ames Mars GCM group
!     Jeffery Hollingsworth, PI
!     NASA Ames Research Center

      use ARCnRT_grid_h
      use ARCnRT_radinc_h
      use ARCnRT_radcommon_h, only: tlimits
use ARCnRT_memory_mod, only: want_rtflux_output, up_fluxes, dn_fluxes
use ARCnRT_getdetau_mod, only: ARCnRT_getdetau
use ARCnRT_gfluxv_mod, only: ARCnRT_gfluxv

      implicit none

!  DTAUV     - Visible opacity of layer L.
!  TAUV      - Visible opacity from top of the atmosphere to bottom
!              of layer L.
!  TAUCUMV   - Visible opacity from top of atmosphere to the bottom
!              of level K.
!  RSFV      - Surface albedo in the visible.
!  WBARV     - Scattering parameter
!  COSBV     - Scattering asymetry parameter
!  UBARO     - Cosine of the incidence angle
!  SOL       - Solar flux in each spectral band, at the top of the
!              atmosphere
!  GWEIGHT   - Gauss weight
!  NFLUXTOPV - Net flux at the top of the atmosphere 
!  FMNETV    - Net flux at the bottom of layer L
!  FLUXUPV   - Upward flux at layer L
!  FLUXDNV   - Downward flux at layer L
!  DIFFVT    - THE DIFFUSE COMPONENT OF THE DOWNWARD SOLAR FLUX
!  FZEROV    - Fraction of zeros (gas opacity k-coefficient off-line
!              computations) in each spectral interval.
!  TAUGSURF  - Gas optical depth (top of atmosphere to the surface).
!  DETAU     - Scaled ( delta-Eddington) optical depth at the surface.
!
!----------------------------------------------------------------------!

      real(f8), intent(IN) :: DTAUV(L_NLAYRAD,L_NSPECTV,L_NGAUSS),  &
           TAUV(L_NLEVRAD,L_NSPECTV,L_NGAUSS), rsfv, ubar0,  &
           TAUCUMV(L_LEVELS,L_NSPECTV,L_NGAUSS),  &
           WBARV(L_NLAYRAD,L_NSPECTV,L_NGAUSS), SOL(L_NSPECTV),  &
           COSBV(L_NLAYRAD,L_NSPECTV,L_NGAUSS), GWEIGHT(L_NGAUSS),  &
           FZEROV(L_NSPECTV), taugsurf(L_NSPECTV,L_NGAUSS-1)
      real(f8), intent(OUT) :: NFLUXTOPV, FMNETV(L_NLAYRAD), DIFFVT,  &
           FLUXUPV(L_NLAYRAD), FLUXDNV(L_NLAYRAD)
      real(f8), intent(INOUT) :: detau(L_NSPECTV,L_NGAUSS)

      real(f8) :: FMUPV(L_NLAYRAD), FMDV(L_NLAYRAD)
      real(f8) :: FLUXUP, FLUXDN
 
!  delta-Eddington surface tau

      integer :: L, NG, NW !TIM, NG1
      real(f8) :: f0pi, btop, bsurf, taumax, eterm

      real(f8) :: DIFFV

      real(f8) :: fzero

!======================================================================C

      TAUMAX = L_TAUMAX

!     ZERO THE NET FLUXES

      NFLUXTOPV = 0._f8

      DO L=1,L_NLAYRAD
        FMNETV(L)  = 0._f8
        FLUXUPV(L) = 0._f8
        FLUXDNV(L) = 0._f8
      END DO

      DIFFVT = 0._f8

!     WE NOW ENTER A MAJOR LOOP OVER SPECTRAL INTERVALS IN THE VISIBLE
!     TO CALCULATE THE NET FLUX IN EACH SPECTRAL INTERVAL

      DO 500 NW=1,L_NSPECTV
      
        F0PI = SOL(NW)

        FZERO = FZEROV(NW)
        IF(FZERO.ge.0.99_f8) goto 40
        DO NG=1,L_NGAUSS-1

          call ARCnRT_GETDETAU(DTAUV(:,NW,NG),TAUV(:,NW,NG),                &
                        TAUCUMV(:,NW,NG),WBARV(:,NW,NG),             &
                        COSBV(:,NW,NG),UBAR0,detau(NW,NG))

          if(TAUGSURF(NW,NG) .lt. TLIMITS) then
            fzero = fzero + (1._f8-FZEROV(NW))*GWEIGHT(NG)
            goto 30
          end if

!         SET UP THE UPPER AND LOWER BOUNDARY CONDITIONS ON THE VISIBLE

          BTOP = 0._f8

!         LOOP OVER THE NTERMS BEGINNING HERE

!  detau(NW,NG) is the scaled optical depth at the surface

          ETERM = MIN(detau(NW,NG)/UBAR0,MAXEXP)
          BSURF = RSFV*UBAR0*SOL(NW)*EXP(-ETERM)

!         WE CAN NOW SOLVE FOR THE COEFFICIENTS OF THE TWO STREAM
!         CALL A SUBROUTINE THAT SOLVES  FOR THE FLUX TERMS
!         WITHIN EACH INTERVAL AT THE MIDPOINT WAVENUMBER
! 
!         FUW AND FDW ARE WORKING FLUX ARRAYS THAT WILL BE USED TO 
!         RETURN FLUXES FOR A GIVEN NT

          CALL ARCnRT_GFLUXV(DTAUV(:,NW,NG),TAUV(:,NW,NG),TAUCUMV(:,NW,NG),   &
                      WBARV(:,NW,NG),COSBV(:,NW,NG),UBAR0,F0PI,RSFV,   &
                      BTOP,BSURF,FMUPV,FMDV,DIFFV,FLUXUP,FLUXDN,       &
                      detau(nw,ng))
 
!         NOW CALCULATE THE CUMULATIVE VISIBLE NET FLUX 

          NFLUXTOPV = NFLUXTOPV+(FLUXUP-FLUXDN)*GWEIGHT(NG)*           &
                                (1._f8-FZEROV(NW))
          DO L=1,L_NLAYRAD
            FMNETV(L)=FMNETV(L)+( FMUPV(L)-FMDV(L) )*                  &
                                 GWEIGHT(NG)*(1._f8-FZEROV(NW))
            FLUXUPV(L) = FLUXUPV(L) + FMUPV(L)*GWEIGHT(NG)*            &
                         (1._f8-FZEROV(NW))
            FLUXDNV(L) = FLUXDNV(L) + FMDV(L)*GWEIGHT(NG)*             &
                         (1._f8-FZEROV(NW))
          END DO

if (want_rtflux_output) then  ! Calc/save 2-stream fluxes [f(wavelength)]
   do l=1,l_nlevrad-1
      up_fluxes(l,nw+L_NSPECTI) = up_fluxes(l,nw+L_NSPECTI)+  &
           FMUPV(L)*GWEIGHT(NG)*(1._f8-FZEROV(NW))
      dn_fluxes(l,nw+L_NSPECTI) = dn_fluxes(l,nw+L_NSPECTI)+  &
           FMDV(L)*GWEIGHT(NG)*(1._f8-FZEROV(NW))
   end do
end if

!         THE DIFFUSE COMPONENT OF THE DOWNWARD SOLAR FLUX

          DIFFVT = DIFFVT + DIFFV*GWEIGHT(NG)*(1._f8-FZEROV(NW))

   30     CONTINUE 

        END DO   ! the Gauss loop 

   40   continue 
!       Special 17th Gauss point

        NG = L_NGAUSS

!       SET UP THE UPPER AND LOWER BOUNDARY CONDITIONS ON THE VISIBLE
 
        BTOP = 0._f8

        call ARCnRT_GETDETAU(DTAUV(:,NW,NG),TAUV(:,NW,NG),                &
                      TAUCUMV(:,NW,NG),WBARV(:,NW,NG),             &
                      COSBV(:,NW,NG),UBAR0,detau(NW,NG))

!       LOOP OVER THE NTERMS BEGINNING HERE
 
        ETERM = MIN(detau(NW,NG)/UBAR0,MAXEXP)
        BSURF = RSFV*UBAR0*SOL(NW)*EXP(-ETERM)

!       WE CAN NOW SOLVE FOR THE COEFFICIENTS OF THE TWO STREAM
!       CALL A SUBROUTINE THAT SOLVES  FOR THE FLUX TERMS
!       WITHIN EACH INTERVAL AT THE MIDPOINT WAVENUMBER
! 
!       FUW AND FDW ARE WORKING FLUX ARRAYS THAT WILL BE USED TO 
!       RETURN FLUXES FOR A GIVEN NT

        CALL ARCnRT_GFLUXV(DTAUV(:,NW,NG),TAUV(:,NW,NG),TAUCUMV(:,NW,NG),     &
                    WBARV(:,NW,NG),COSBV(:,NW,NG),UBAR0,F0PI,RSFV,     &
                    BTOP,BSURF,FMUPV,FMDV,DIFFV,FLUXUP,FLUXDN,         &
                    detau(nw,ng))
 
!       NOW CALCULATE THE CUMULATIVE VISIBLE NET FLUX 

        NFLUXTOPV = NFLUXTOPV+(FLUXUP-FLUXDN)*FZERO
        DO L=1,L_NLAYRAD
          FMNETV(L)=FMNETV(L)+( FMUPV(L)-FMDV(L) )*FZERO
          FLUXUPV(L) = FLUXUPV(L) + FMUPV(L)*FZERO
          FLUXDNV(L) = FLUXDNV(L) + FMDV(L)*FZERO
        END DO

if (want_rtflux_output) then  ! Calc/save 2-stream fluxes [f(wavelength)]
   do l=1,l_nlevrad-1
      up_fluxes(l,nw+L_NSPECTI) = up_fluxes(l,nw+L_NSPECTI)+FMUPV(L)*FZERO
      dn_fluxes(l,nw+L_NSPECTI) = dn_fluxes(l,nw+L_NSPECTI)+FMDV(L)*FZERO
   end do
end if

!       THE DIFFUSE COMPONENT OF THE DOWNWARD SOLAR FLUX

        DIFFVT = DIFFVT + DIFFV*FZERO

  500 CONTINUE

!     *** END OF MAJOR SPECTRAL INTERVAL LOOP IN THE VISIBLE*****
 
      RETURN 
      end subroutine ARCnRT_sfluxv


end module ARCnRT_sfluxv_mod
