!```````````````````````````````````````````````````````````````````
module aerad_gasopd_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine calc_gasopd

!+ Computes gas opacities (mixed CO2 and H2O_vapor) of all model layers

use univ_kind_defs_mod, only: f8, f4
use aertxrad_mem_mod, only: lkcoef,air_kc_lw,p,p_lev,t,qH2O  &
     ,nz1_rad,lw_iwbeg,lw_iwend,sw_iwbeg,sw_iwend,ngauss_pts,tau_gas  &
     ,air_kc_sw,air_kc_lw,sw_on,gas_odc,grav,dzc,rho_air  &
     ,ptmp => fscr1  &   ! [nz1_rad]
     ,u => dscr1  &   ! [nz1_rad]
     ,wratio => dscr2  &   ! [nz1_rad]
     ,nH2O => iscr1  &   ! [nz1_rad]
     ,mt => iscr2  &   ! [nz1_rad]
     ,mp => iscr3     ! [nz1_rad]

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

real(f8), dimension(4) :: kcoef
integer :: k,nw,ng,iw
real(f8) :: ans,w,c1
real(f4) :: dp

!=== Executable statements:

! Obtain coefficients/factors needed for opacity calculation:

do k=2,nz1_rad       ! Loop over most of the layer MIDPOINTS
   ! The estimated # of gas molecules in current atm layer [actually calculated
   !  as # of CO2 molecules; but currently used as proxy "real" molecule count
   !  even when other radiatively active molecules (e.g., H2O) are present]: 
   !  *NOTE*: a conversion factor of 1.e-4 [cm^2 -> m^2] is present in this
   !          value as well, to convert the CGS k-coeff units to SI units
   dp = p_lev(k)-p_lev(k-1)
!@   u(k) = dble(3.51e22*dp*1.e-2) 
   u(k) = real(gas_odc*dzc(k)*rho_air(k), kind=f8)                                      
   ptmp(k) = p(k)*1.e-2_f4   ! Convert units [Pa -> mb] 
enddo

k = 1   ! Model top "layer"
dp = p_lev(k)    ! Actually p_lev(1)-0.
!@u(k) = dble(3.51e22*dp*1.e-2)    ! Convert units of dpr(k) [Pa -> mb] 
u(k) = real(gas_odc/grav*dp, kind=f8)    ! Assume this layer is hydrostatic
ptmp(k) = 0.5_f4*p_lev(k)*1.e-2_f4   ! Convert units [Pa -> mb] 

call tpindex_col(nz1_rad,1,nz1_rad,ptmp,t,qH2O,lkcoef,mt,mp,nH2O,wratio)

! Determine the total gas opacity throughout the column -- for each
!  wavelength interval (NW) and each Gauss point (NG):

! "Longwave" region

do k=1,nz1_rad      ! Loop over all layer MIDPOINTS
   iw = 0
   do nw=lw_iwbeg,lw_iwend
      iw = iw+1
      do ng=1,ngauss_pts

         !  Interpolate between water mixing ratios (WRATIO=0.0 if the
         !   requested water amount is equal to, or outside the the range of
         !   water amount data reference values):

         w = wratio(k)

         c1 = air_kc_lw(MT(K),MP(K),NH2O(K),iW,NG)
         kcoef(1) = c1+w*(air_kc_lw(MT(K),MP(K),NH2O(K)+1,iW,NG)-c1)
        
         c1 = air_kc_lw(MT(K),MP(K)+1,NH2O(K),iW,NG)
         kcoef(2) = c1+w*(air_kc_lw(MT(K),MP(K)+1,NH2O(K)+1,iW,NG)-c1)
         
         c1 = air_kc_lw(MT(K)+1,MP(K)+1,NH2O(K),iW,NG)
         kcoef(3) = c1+w*(air_kc_lw(MT(K)+1,MP(K)+1,NH2O(K)+1,iW,NG)-c1)
         
         c1 = air_kc_lw(MT(K)+1,MP(K),NH2O(K),iW,NG)
         kcoef(4) = c1+w*(air_kc_lw(MT(K)+1,MP(K),NH2O(K)+1,iW,NG)-c1)

         ! Interpolate the CO2 k-coefficients to the requested T,P:

         ans = LKCOEF(K,1)*KCOEF(1) + LKCOEF(K,2)*KCOEF(2) +  &
               LKCOEF(K,3)*KCOEF(3) + LKCOEF(K,4)*KCOEF(4)

         tau_gas(ng,nw,k) = real(u(k)*ans)    ! Gas opacity
	    
      enddo
      
   enddo

enddo

! "Shortwave" region

if(sw_on) then   ! *ONLY* calculate if needed

   do k=1,nz1_rad      ! Loop over all layer MIDPOINTS

      iw = 0
      do nw=sw_iwbeg,sw_iwend
         iw = iw+1
         do ng=1,ngauss_pts

            !  Interpolate between water mixing ratios (WRATIO=0.0 if the
            !   requested water amount is equal to, or outside the the range of
            !   water amount data reference values):

            w = wratio(k)
            
            c1 = air_kc_sw(MT(K),MP(K),NH2O(K),iW,NG)
            kcoef(1) = c1+w*(air_kc_sw(mt(k),mp(k),nH2O(k)+1,iw,ng)-c1)
         
            c1 = air_kc_sw(MT(K),MP(K)+1,NH2O(K),iW,NG)
            kcoef(2) = c1+w*(air_kc_sw(MT(K),MP(K)+1,NH2O(K)+1,iW,NG)-c1)
         
            c1 = air_kc_sw(MT(K)+1,MP(K)+1,NH2O(K),iW,NG)
            kcoef(3) = c1+w*(air_kc_sw(MT(K)+1,MP(K)+1,NH2O(K)+1,iW,NG)-c1)
              
            c1 = air_kc_sw(MT(K)+1,MP(K),NH2O(K),iW,NG)
            kcoef(4) = c1+w*(air_kc_sw(MT(K)+1,MP(K),NH2O(K)+1,iW,NG)-c1)

            ! Interpolate the CO2 k-coefficients to the requested T,P:

            ans = LKCOEF(K,1)*KCOEF(1) + LKCOEF(K,2)*KCOEF(2) +  &
                 LKCOEF(K,3)*KCOEF(3) + LKCOEF(K,4)*KCOEF(4)

            tau_gas(ng,nw,k) = real(u(k)*ans)    ! Gas opacity
	    
         enddo
	  
      enddo
   
   enddo

endif

end subroutine calc_gasopd


!```````````````````````````````````````````````````````````````````
subroutine tpindex_col(nk,ka,kz,pw,tw,qH2O,lkcoef,mt,mp,nH2O,wratio)

!+ Calculate interpolation indices for "air" k-coefficient interpolation (calc
!   for entire column each call)

use univ_kind_defs_mod, only: f8, f4
use univ_math_constants_mod, only: ONE_f8, ZERO_f8
use aertxrad_mem_mod, only: air_kc_nwref,air_kc_nfpref,air_kc_nTref,tref_air  &
     ,fpref_air_log10,H2Oref_air

implicit none

!=== Argument declarations:

integer, intent(IN) :: nk,ka,kz
real(f4),dimension(nk), intent(IN) :: pw,tw,qH2O

real(f8), dimension(nk,4), intent(OUT) :: lkcoef
integer,dimension(nk), intent(OUT) :: mt,mp,nH2O
real(f8),dimension(nk), intent(OUT) :: wratio

!== Local declarations:

! from GCM2.0, Feb 2003
!
!  PURPOSE
!    Get the TI, UI values for a 2-dimensional interpolation
!    based on the following (The interpolation is done in interpco2):
!    Interpolate the CO2 K-coefficients to the current P,T values.
!    The CO2 coefficients are given on a P,T grid:
!    P = {1E-6, 1E-5, 1E-4, 1E-3, 1E-2, 1E-1, 1, 1E+1, 1E+2, 1E+3, 1E+4},
!    T = {50, 100, 150, 200, 250, 300, 350}.
!
!    The interpolation is the usual interpolation in 2-dimensions given
!    in "Numerical Recipes", where the "X" are P, the "Y" are
!    T, and the F(X,Y) are the CO2 K-coefficients.
!
!     The interpolating box is designated as follows:
!
!           (PL,TU)                        (PRR,TU)
!
!                          (TW,PW)
!
!           
!           (PL,TL)                        (PRR,TL)
!
!     PL  - Pressure left
!     PRR - Pressure right
!     TL  - Temperature lower
!     TU  - Temperature upper
!     PW  - Pressure wanted
!     TW  - Temperature wanted
!
!
!  AUTHOR
!      JIM SCHAEFFER     STERLING SOFTWARE  TASK 404  MAR 1998
!
!  FOR
!      BOB HABERLE       PART OF GCM II/RADTRAN UPDATES
!
!  ENVIRONMENT
!      SUN ULTRA-2       SOLARIS 2.5.1   FORTRAN 77
!
!  REVISION HISTORY
!     3-98  JRS  ORIGINAL
!
!  INPUT PARAMETERS
!    PW                 - The pressure to interpolate to
!    TW                 - The temperature to interpolate to
!    Pref(NP)           - The pressure grid array.
!    Tref(NT)           - The temperature grid array.
!    
!  OUTPUT PARAMETERS
!    TI                 - Interpolation term (pressure)
!    UI                 - Interpolation term (temperature)
!    MT                 - Temperature index (bottom left Temperature)
!                         of bounding box
!    MP                 - Pressure index (bottom left pressure)
!                         of bounding box
!
!  CALLED BY
!    SETRAD 
!
!  SUBROUTINES CALLED
!      NONE

integer :: n,k,mt0,mp0
real(f8) :: t,u
real(f4) :: pwl,tmp

!=== Executable statements:

do k=ka,kz   ! Loop over all requested layers

   ! Get the upper and lower Temperature-grid indices that bound the
   !  requested temperature.  If the requested temperature is outside
   !  the T-grid, set up to extrapolate from the appropriate end:
   
   tmp = tw(k)
   mt0 = 0
   if(tmp <= tref_air(1)) then
      mt0 = 1
   else
      do n=1,air_kc_nTref-1
         if(tmp > tref_air(n) .and. tmp <= tref_air(n+1)) then
            mt0 = n
            exit         
         endif
      enddo
      if(mt0 == 0) then
         mt0 = air_kc_nTref-1
      endif
   endif

   u = dble((tmp-tref_air(mt0))/(tref_air(mt0+1)-tref_air(mt0)))
   mt(k) = mt0

   ! Get the upper and lower Pressure-grid indicies that bound the
   !  requested pressure.  If the requested pressure is outside
   !  the P-grid, set up to extrapolate from the appropiate end:

   pwl = log10(pw(k))

   mp0 = 0
   do n=2,air_kc_nfpref-1
      if(pwl <= fpref_air_log10(n)) then
         MP0 = n-1
         exit
      endif
   enddo
   if(mp0 == 0) then
      MP0 = air_kc_nfpref-1
   endif

   t = real((pwl-fpref_air_log10(mp0))/  &
        (fpref_air_log10(mp0+1)-fpref_air_log10(mp0)), kind=f8)
   mp(k) = mp0

   ! Fill the interpolation coeficients:

   lkcoef(k,1) = (ONE_f8-T)*(ONE_f8-U)
   lkcoef(k,2) = T*(ONE_f8-U)
   lkcoef(k,3) = T*U
   lkcoef(k,4) = (ONE_f8-T)*U

   ! Get the indicies for water abundance.  There are 10 sets of 
   !  k-coefficients with differing amounts of water vs. CO2:
   
   tmp = qH2O(k)
   IF(tmp <= H2Oref_air(1)) then
      NH2O(k)   = 1
      WRATIO(k) = ZERO_f8
   ELSEIF(tmp >= H2Oref_air(air_kc_nwref)) then
      NH2O(k)   = air_kc_nwref
      WRATIO(k) = ZERO_f8
   ELSE
      DO N=2,air_kc_nwref
         IF(tmp >= H2Oref_air(N-1) .and. tmp < H2Oref_air(N)) then
            NH2O(k)   = N-1
            WRATIO(k) = dble((tmp-H2Oref_air(N-1))/  &
                 (H2Oref_air(N)-H2Oref_air(N-1)))
            exit   ! Leave DO-loop
         ENDIF
      ENDDO
   ENDIF
   
enddo

end subroutine tpindex_col


end module aerad_gasopd_mod
