!```````````````````````````````````````````````````````````````````
module aerad_init_mod

!+

use univ_kind_defs_mod, only: f4

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine init_ref(ext_naersclr, ext_aer_ngrp, ext_aer_grpbins,  &
     ext_aer_group, ext_aer_subgrp, ext_aer_ig, ext_XOrad_mask)

!+ Initializes several reference value arrays

use univ_kind_defs_mod, only: f4, f8
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL
use aertxrad_mem_mod, only: lw_iwbeg,lw_iwend,sw_iwbeg,sw_iwend,nlw_wavlnrng  &
     ,wavenum_edge_sw,wavenum_edge_lw,ntot_wavlnrng,spc_ig,gw_solflux  &
     ,spc_nbins,spc_begiv,spc_endiv,spc_ctype,solflux,lw_ipbeg,lw_ipend  &
     ,nlw_gpt,sw_ipbeg,sw_ipend,ntot_gpt,u1i,u1s,u1i_2,sqrt3,twopi  &
     ,tpft_beg,tpft_end,ngweights,tpft_invfinc,ptemp_itp,g_weight,sbk,pi  &
     ,naerspc_rad, solar_sw
use aermic_rad_mem_mod, only: faux_dust_rtxinfo_needed
use aerad_solflux_mod, only: solarflux
use aerad_precalc_mod, only: planckf

implicit none

!=== Argument declarations:

integer, intent(IN) :: ext_naersclr, ext_aer_ngrp
integer, dimension(ext_aer_ngrp), intent(IN) :: ext_aer_grpbins
integer, dimension(ext_naersclr), optional, intent(IN) :: ext_aer_group,  &
     ext_aer_subgrp, ext_aer_ig
logical, dimension(ext_naersclr), optional, intent(IN) :: ext_XOrad_mask

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'init_ref'

integer :: iq,iw,ip,ig,iv,it
real(f4) :: wn1,wn2
real(f8) :: w1,w2,tempk
logical :: found

!=== Executable statements:

! Assign beginning and end wavelength range and point indices for each
!  wavelength group (currently 2):

lw_iwbeg = 1                ! "longwave" group   
lw_iwend = nlw_wavlnrng     !
sw_iwbeg = lw_iwend+1      ! "shortwave" group
sw_iwend = ntot_wavlnrng   !

lw_ipbeg = 1                ! "longwave" group   
lw_ipend = nlw_gpt          !
sw_ipbeg = lw_ipend+1      ! "shortwave" group
sw_ipend = ntot_gpt        !

! Calculate the "average" wavenumber (1/wavelength) <wavenum()> for each
!  wavelength interval in each wavelength group [1/cm]:

iq = 0
ip = lw_ipbeg-1
do iw=lw_iwbeg,lw_iwend  ! "avg" wavenumber over each "longwave" interval
   iq = iq+1
   wn1 = wavenum_edge_lw(iq)    ! [cm^-1]
   wn2 = wavenum_edge_lw(iq+1)  ! 

   ! Integrated solar flux for each wavelength range:
   solflux(iw) = real(solarflux(wn1,wn2), kind=f8)

   do ig=1,ngweights
      ip = ip+1
      ! Gauss-weighted solar flux in each probability interval:
      gw_solflux(ip) = solflux(iw)*g_weight(ig)
   enddo
enddo

iq = 0
ip = sw_ipbeg-1
do iw=sw_iwbeg,sw_iwend  ! "avg" wavenumber over each "shortwave" interval
   iq = iq+1
   wn1 = wavenum_edge_sw(iq)    ! [cm^-1]
   wn2 = wavenum_edge_sw(iq+1)  ! 

   ! Integrated solar flux for each wavelength range:
   solflux(iw) = real(solarflux(wn1,wn2), kind=f8)
  
   do ig=1,ngweights
      ip = ip+1
      ! Gauss-weighted solar flux in each probability interval:
      gw_solflux(ip) = solflux(iw)*g_weight(ig)
   enddo
enddo

solar_sw = sum(solflux(sw_iwbeg:sw_iwend))  ! Solar flux (@ 1AU) within this RT
                                            !  code's "shortwave" range [W/m^2]

! Set up reference arrays for dealing with aerosol quantities:

ip = 0
if (ext_naersclr > 0) then
   if (.not.(present(ext_aer_group) .and. present(ext_aer_subgrp) .and.  &
        present(ext_aer_ig) .and. present(ext_XOrad_mask))) call exception(  &
        procedure_name, FATAL,  &
        'In this use case, all optional arguments must be provided.')

   ig = ext_aer_ig(1)
   found = .FALSE.
   do iv=1,ext_naersclr  ! Loop over all aerosol scalars   
      if (ext_XOrad_mask(iv)) then   ! Calc radiative transfer for this scalar
         if (ig /= ext_aer_ig(iv)) then   ! Changing groups
            ig = ext_aer_ig(iv)   ! Relative group ID#
            found = .FALSE.
         end if
         if (ext_aer_subgrp(iv) == 0 .and. .NOT.found) then  ! Uncounted # conc
            ip = ip+1    ! ID# of this rad-active specie
            found = .TRUE.
            it = ext_aer_group(iv)  ! Absolute group ID#
            spc_nbins(ip) = ext_aer_grpbins(it)  ! # bins rad-active specie has
            spc_begiv(ip) = iv    ! Starting aersclr # of this rad-active specie
            spc_endiv(ip) = iv+spc_nbins(ip)-1  ! Ending aersclr # of this specie
            spc_ig(ip) = ig  ! Relative group ID#
            spc_ctype(ip) = ext_aer_group(iv)    ! Specie compositional type
         end if
      end if
   end do

   if (faux_dust_rtxinfo_needed) then
      ip = naerspc_rad        ! Last specie
      it = ext_aer_group(1)  ! Assume this is dust (dust must be "ON")
      spc_ctype(ip) = it   
      spc_nbins(ip) = ext_aer_grpbins(it) 
      spc_begiv(ip) = -999    ! Not a part of actual aerosol arrays
      spc_endiv(ip) = -999    !
      spc_ig(ip) = ext_aer_ig(1)    ! Assume this is dust (dust must be "ON")
   end if
end if

! Get CO2/H2Ov "air" k-coefficients, and interpolate them to a finer grid of
!  reference pressures:

call air_kc_fpinterp

! Calculate Planck function quantities as a function of wavelength point and
!  temperature:

ip = lw_ipbeg-1
do iw=1,nlw_wavlnrng   ! Loop over all "longwave" wavelength intervals

   ! NOTE: Set these so that difference below is
   !    f(longer_wavlen)-f(shorter_wavlen),
   !    w1 = longer_wavlen, w2 = shorter_wavlen:
   w1 = real(1.439_f4*wavenum_edge_lw(iw), kind=f8)  ! 1.439 ~ ((h*c)/k)*1.e2 ,
   w2 = real(1.439_f4*wavenum_edge_lw(iw+1), kind=f8)    !  1.e2 factor is m->cm
      
   do ig=1,ngweights
      ip = ip+1

      do it=tpft_beg,tpft_end
         tempk = tpft_invfinc*real(it, kind=f8)   ! Temperature [K]
      
         ptemp_itp(it,ip) = (PLANCKf(w1,tempk)-PLANCKf(w2,tempk))*  &
              g_weight(ig)*sbk/pi
      enddo
      ptemp_itp(1:tpft_beg,ip) = ptemp_itp(tpft_beg,ip)

   enddo
enddo

do ip=sw_ipbeg,sw_ipend                         
   u1i(ip) = sqrt3        ! For "shortwave" points
   u1i_2(ip) = 0.5_f8*sqrt3
enddo     
                                                   
do ip=lw_ipbeg,lw_ipend 
   u1i(ip) = 2._f8         ! For "longwave" points                 
   u1i_2(ip) = 1._f8   ! 0.5_f8*2._f8
enddo

do ip=1,ntot_gpt
   u1s(ip) = twopi/u1i(ip)
enddo

end subroutine init_ref


!```````````````````````````````````````````````````````````````````
subroutine init_optprop(ext_aerradius, ext_dust_op_case)

!+ Initializes several optical property arrays

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4, pi_f4
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
use univ_num_search_mod, only: locidx_1D_mono_type02, error_val_i4
use univ_pointer_utils_mod, only: ptr_release
use univ_convert_units_mod, only: um_to_m_f4, cm_to_m_f4, m_to_um_f4
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_PROC_NAME_LEN
use manage_NetCDF_classic_mod, only: mNc_file_metadata_obj, mNc_dim_val,  &
     mNc_recycle_f_md_obj, mNc_populate_f_md_obj_r
use manage_NetCDF_classic_rv_mod, only: mNc_read_var
use aermic_rad_mem_mod, only: dataset_path_bspec
use aertxrad_mem_mod, only: lw_iwbeg, lw_iwend, spc_nbins, spc_ctype,  &
     naerspc_rad, spc_xsarea_fact, spc_ig, aerad_ngrp, aeradID_dust,  &
     aeradID_H2Oi, aeradID_CO2i, nlw_wavlnrng, nsw_wavlnrng, sw_iwbeg,  &
     sw_iwend, abp_optprop_obj, lw_abp_optprop, sw_abp_optprop

implicit none

!=== Argument declarations:

integer, intent(IN) :: ext_dust_op_case
real(f4), dimension(:,:), intent(IN) :: ext_aerradius

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'init_optprop'

real(f4), parameter :: um2_to_m2 = um_to_m_f4**2,  &  ! um^2 -> m^2 conversion
                       cm_to_um = cm_to_m_f4*m_to_um_f4  ! cm -> um conversion
integer :: ia, e, ip, ig, iw, iwa, ib, ir, ifr, ifb, n_fr, n_fb
logical :: e_check
real(f4) :: ft, ft1, r_bin
character(len=MAX_FILE_PATH_LEN) :: data_fpath
character(len=MAX_FILE_PATH_LEN) :: tmp_str

type(mNc_file_metadata_obj) :: data_file
type(abp_optprop_obj), dimension(aerad_ngrp) :: ia_abp_optprop

!=== Executable statements:

!********* "Longwave" spectral intervals **************************************

! Allocate some memory:
allocate(lw_abp_optprop(naerspc_rad), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1lw')

! Set wavelength-, radius-, and specie-dependent airborne particle optical
!  properties:
do ip=1,naerspc_rad ! Loop over all possible RT-active airborne particle species
   ig = spc_ig(ip)  ! Relative group ID#
   ia = spc_ctype(ip)   ! Airborne particle type

   if (.NOT.associated(ia_abp_optprop(ia)%p_radius)) then ! Initialize this type
      ! Determine data file name:
      select case (ia)
      case (aeradID_dust)
         select case (ext_dust_op_case)
         case (1)
            data_fpath = trim(dataset_path_bspec)//  &
                 '/palagonite_C1_opcf_sphere_lw.nc'
         case (2)
            data_fpath = trim(dataset_path_bspec)//  &
                 '/palagonite_C2_opcf_sphere_lw.nc'
         case (3)
            data_fpath = trim(dataset_path_bspec)//  &
                 '/Mars_dust_W2009_opcf_sphere_lw.nc'
         end select
      case (aeradID_H2Oi)
         data_fpath = trim(dataset_path_bspec)//'/H2Oi_opcf_sphere_lw.nc'
      case (aeradID_CO2i)
         data_fpath = trim(dataset_path_bspec)//'/CO2i_opcf_sphere_lw.nc'
      end select

      ! Open data file; read metadata:
      call mNc_populate_f_md_obj_r(data_fpath, data_file)
   
      ! Sanity check on number of spectral intervals:
      if (mNc_dim_val(data_file, 4) /= nlw_wavlnrng) then
         write (tmp_str, '(3a,i3,a,i3,a)')  &
              '# of spectral intervals in selected RT code and datafile (',  &
              trim(data_fpath), ') must be equal.  Currently ',  &
              nlw_wavlnrng, ' (RT code) vs. ', mNc_dim_val(data_file, 4),  &
              ' (datafile).'
         call exception(procedure_name, FATAL, tmp_str)
      end if

      ! Read selected data from file:
      call mNc_read_var(data_file, 'p_radius', ia_abp_optprop(ia)%p_radius)
      call mNc_read_var(data_file, 'flux_vs_wvl_slope_r',  &
           ia_abp_optprop(ia)%fvws_r)
      call mNc_read_var(data_file, 'flux_vs_wvl_slope_b',  &
           ia_abp_optprop(ia)%fvws_b)
      call mNc_read_var(data_file, 'Q_ext', ia_abp_optprop(ia)%Q_ext)
      call mNc_read_var(data_file, 'Q_sca', ia_abp_optprop(ia)%Q_sca)
      call mNc_read_var(data_file, 'gQsca', ia_abp_optprop(ia)%gQsca)

      ! Release object memory and close file:
      call mNc_recycle_f_md_obj(data_file)
   end if

   ! Allocate some module (global) variables:
   n_fr = size(ia_abp_optprop(ia)%fvws_r)
   n_fb = size(ia_abp_optprop(ia)%fvws_b)
   allocate(lw_abp_optprop(ip)%fvws_r(n_fr), lw_abp_optprop(ip)%fvws_b(n_fb),  &
        lw_abp_optprop(ip)%d_fvws_r(n_fr-1),  &
        lw_abp_optprop(ip)%d_fvws_b(n_fb-1),  &
        lw_abp_optprop(ip)%Q_ext(n_fr,n_fb,spc_nbins(ip),nlw_wavlnrng),  &
        lw_abp_optprop(ip)%Q_sca(n_fr,n_fb,spc_nbins(ip),nlw_wavlnrng),  &
        lw_abp_optprop(ip)%gQsca(n_fr,n_fb,spc_nbins(ip),nlw_wavlnrng), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2lw')

   ! Copy reference arrays, compute some additional reference parameters:
   lw_abp_optprop(ip)%fvws_r = ia_abp_optprop(ia)%fvws_r
   lw_abp_optprop(ip)%fvws_b = ia_abp_optprop(ia)%fvws_b
   lw_abp_optprop(ip)%n_fvws_r = n_fr; lw_abp_optprop(ip)%n_fvws_b = n_fb
   lw_abp_optprop(ip)%search_beg_idx_fvws_r = n_fr/2+1
   lw_abp_optprop(ip)%search_beg_idx_fvws_b = n_fb/2+1
   lw_abp_optprop(ip)%max_fvws_r = maxval(lw_abp_optprop(ip)%fvws_r)
   lw_abp_optprop(ip)%min_fvws_r = minval(lw_abp_optprop(ip)%fvws_r)
   lw_abp_optprop(ip)%max_fvws_b = maxval(lw_abp_optprop(ip)%fvws_b)
   lw_abp_optprop(ip)%min_fvws_b = minval(lw_abp_optprop(ip)%fvws_b)
   do ifr=1,n_fr-1
      lw_abp_optprop(ip)%d_fvws_r(ifr) =  &
           lw_abp_optprop(ip)%fvws_r(ifr+1)-lw_abp_optprop(ip)%fvws_r(ifr)
   end do
   do ifb=1,n_fb-1
      lw_abp_optprop(ip)%d_fvws_b(ifb) =  &
           lw_abp_optprop(ip)%fvws_b(ifb+1)-lw_abp_optprop(ip)%fvws_b(ifb)
   end do

   iw = 0
   do iwa=lw_iwbeg,lw_iwend    ! Loop over all "longwave" wavelength intervals
      iw = iw+1   ! Relative (to data file) spectral interval
      ir = 1  ! Initial value
      do ib=1,spc_nbins(ip)  ! Loop over # of airborne particle radius bins
         r_bin = ext_aerradius(ib,ig)*cm_to_um   ! Avg radius in this bin [um]
         spc_xsarea_fact(ib,ip) = real(pi_f4)*r_bin*r_bin*um2_to_m2  ! [m^2]

         ! Search for target particle radius; check if out of bounds:
         ir = locidx_1D_mono_type02(ia_abp_optprop(ia)%p_radius, r_bin, ir)
         if (ir == error_val_i4) then
            write (*, '(/a)') 'ERROR: optical property radius bin not found.'
            write (*, '(a,i3,a,i3,a,e10.3,2(a,e10.3),a)')  &
                 'airborne particle type:', ia, ', bin #', ib,  &
                 ', target radius [um] = ', r_bin,  &
                 ' ; (min,max) radii in file [um] = (',  &
                 minval(ia_abp_optprop(ia)%p_radius), ', ',  &
                 maxval(ia_abp_optprop(ia)%p_radius), ')'
            call exception(procedure_name, FATAL)
         end if

         ! Calculate interpolation factors:
         ft = min(ONE_f4, (r_bin-ia_abp_optprop(ia)%p_radius(ir))/  &
              (ia_abp_optprop(ia)%p_radius(ir+1)-  &
              ia_abp_optprop(ia)%p_radius(ir)))
         ft1 = max(ZERO_f4, ONE_f4-ft)

         ! Interpolate optical properties to target radii:
         lw_abp_optprop(ip)%Q_ext(:,:,ib,iw) =  &
              ia_abp_optprop(ia)%Q_ext(:,:,ir,iw)*ft1+  &
              ia_abp_optprop(ia)%Q_ext(:,:,ir+1,iw)*ft
         lw_abp_optprop(ip)%Q_sca(:,:,ib,iw) =  &
              ia_abp_optprop(ia)%Q_sca(:,:,ir,iw)*ft1+  &
              ia_abp_optprop(ia)%Q_sca(:,:,ir+1,iw)*ft
         lw_abp_optprop(ip)%gQsca(:,:,ib,iw) =  &
              ia_abp_optprop(ia)%gQsca(:,:,ir,iw)*ft1+  &
              ia_abp_optprop(ia)%gQsca(:,:,ir+1,iw)*ft
      end do   ! end: ib, ir

      ! Display some statistics about the optical properties:
      write (*, '(2a,2(1x,i4))') trim(data_fpath), ', specie ctype, ip:', ia, ip
      write (*, '(a,i2)') 'Longwave spectral interval #', iw
      write (*, '(a,3(1x,e12.4))') 'Q_ext (min,avg,max): ',  &
           minval(lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           sum(lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))/  &
           size(lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           maxval(lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))
      write (*, '(a,3(1x,e12.4))') 'ss_albedo (min,avg,max): ',  &
           minval(lw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           sum(lw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))/  &
           size(lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           maxval(lw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))
      write (*, '(a,3(1x,e12.4))') 'asym_factor (min,avg,max): ',  &
           minval(lw_abp_optprop(ip)%gQsca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           lw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)),  &
           sum(lw_abp_optprop(ip)%gQsca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           lw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw))/  &
           size(lw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           maxval(lw_abp_optprop(ip)%gQsca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           lw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw))
   end do   ! end: iwa, iw
end do  ! end: ip, ig, ia

! Deallocate/nullify some pointers:
do ia=1,aerad_ngrp
   if (associated(ia_abp_optprop(ia)%p_radius)) then
      e_check = .FALSE.
      call ptr_release(ia_abp_optprop(ia)%p_radius, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%fvws_r, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%fvws_b, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%Q_ext, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%Q_sca, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%gQsca, e)
      e_check = e_check .or. (e /= 0)
      if (e_check) call exception(procedure_name, FATAL, 'dealloc_2lw')
   end if
end do

!********* "Shortwave" spectral intervals **************************************

! Allocate some memory:
allocate(sw_abp_optprop(naerspc_rad), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1sw')

! Set wavelength-, radius-, and specie-dependent airborne particle optical
!  properties:
do ip=1,naerspc_rad ! Loop over all possible RT-active airborne particle species
   ig = spc_ig(ip)  ! Relative group ID#
   ia = spc_ctype(ip)   ! Airborne particle type

   if (.NOT.associated(ia_abp_optprop(ia)%p_radius)) then ! Initialize this type
      ! Determine data file name:
      select case (ia)
      case (aeradID_dust)
         select case (ext_dust_op_case)
         case (1)
            data_fpath = trim(dataset_path_bspec)//  &
                 '/palagonite_C1_opcf_sphere_sw.nc'
         case (2)
            data_fpath = trim(dataset_path_bspec)//  &
                 '/palagonite_C2_opcf_sphere_sw.nc'
         case (3)
            data_fpath = trim(dataset_path_bspec)//  &
                 '/Mars_dust_W2009_opcf_sphere_sw.nc'
         end select
      case (aeradID_H2Oi)
         data_fpath = trim(dataset_path_bspec)//'/H2Oi_opcf_sphere_sw.nc'
      case (aeradID_CO2i)
         data_fpath = trim(dataset_path_bspec)//'/CO2i_opcf_sphere_sw.nc'
      end select

      ! Open data file; read metadata:
      call mNc_populate_f_md_obj_r(data_fpath, data_file)
   
      ! Sanity check on number of spectral intervals:
      if (mNc_dim_val(data_file, 4) /= nsw_wavlnrng) then
         write (tmp_str, '(3a,i3,a,i3,a)')  &
              '# of spectral intervals in selected RT code and datafile (',  &
              trim(data_fpath), ') must be equal.  Currently ',  &
              nsw_wavlnrng, ' (RT code) vs. ', mNc_dim_val(data_file, 4),  &
              ' (datafile).'
         call exception(procedure_name, FATAL, tmp_str)
      end if

      ! Read selected data from file:
      call mNc_read_var(data_file, 'p_radius', ia_abp_optprop(ia)%p_radius)
      call mNc_read_var(data_file, 'flux_vs_wvl_slope_r',  &
           ia_abp_optprop(ia)%fvws_r)
      call mNc_read_var(data_file, 'flux_vs_wvl_slope_b',  &
           ia_abp_optprop(ia)%fvws_b)
      call mNc_read_var(data_file, 'Q_ext', ia_abp_optprop(ia)%Q_ext)
      call mNc_read_var(data_file, 'Q_sca', ia_abp_optprop(ia)%Q_sca)
      call mNc_read_var(data_file, 'gQsca', ia_abp_optprop(ia)%gQsca)

      ! Release object memory and close file:
      call mNc_recycle_f_md_obj(data_file)
   end if

   ! Allocate some module (global) variables:
   n_fr = size(ia_abp_optprop(ia)%fvws_r)
   n_fb = size(ia_abp_optprop(ia)%fvws_b)
   allocate(sw_abp_optprop(ip)%fvws_r(n_fr), sw_abp_optprop(ip)%fvws_b(n_fb),  &
        sw_abp_optprop(ip)%d_fvws_r(n_fr-1),  &
        sw_abp_optprop(ip)%d_fvws_b(n_fb-1),  &
        sw_abp_optprop(ip)%Q_ext(n_fr,n_fb,spc_nbins(ip),nsw_wavlnrng),  &
        sw_abp_optprop(ip)%Q_sca(n_fr,n_fb,spc_nbins(ip),nsw_wavlnrng),  &
        sw_abp_optprop(ip)%gQsca(n_fr,n_fb,spc_nbins(ip),nsw_wavlnrng), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2sw')

   ! Copy reference arrays, compute some additional reference parameters:
   sw_abp_optprop(ip)%fvws_r = ia_abp_optprop(ia)%fvws_r
   sw_abp_optprop(ip)%fvws_b = ia_abp_optprop(ia)%fvws_b
   sw_abp_optprop(ip)%n_fvws_r = n_fr; sw_abp_optprop(ip)%n_fvws_b = n_fb
   sw_abp_optprop(ip)%search_beg_idx_fvws_r = n_fr/2+1
   sw_abp_optprop(ip)%search_beg_idx_fvws_b = n_fb/2+1
   sw_abp_optprop(ip)%max_fvws_r = maxval(sw_abp_optprop(ip)%fvws_r)
   sw_abp_optprop(ip)%min_fvws_r = minval(sw_abp_optprop(ip)%fvws_r)
   sw_abp_optprop(ip)%max_fvws_b = maxval(sw_abp_optprop(ip)%fvws_b)
   sw_abp_optprop(ip)%min_fvws_b = minval(sw_abp_optprop(ip)%fvws_b)
   do ifr=1,n_fr-1
      sw_abp_optprop(ip)%d_fvws_r(ifr) =  &
           sw_abp_optprop(ip)%fvws_r(ifr+1)-sw_abp_optprop(ip)%fvws_r(ifr)
   end do
   do ifb=1,n_fb-1
      sw_abp_optprop(ip)%d_fvws_b(ifb) =  &
           sw_abp_optprop(ip)%fvws_b(ifb+1)-sw_abp_optprop(ip)%fvws_b(ifb)
   end do

   ! Copy reference arrays:
   sw_abp_optprop(ip)%fvws_r = ia_abp_optprop(ia)%fvws_r
   sw_abp_optprop(ip)%fvws_b = ia_abp_optprop(ia)%fvws_b

   iw = 0
   do iwa=sw_iwbeg,sw_iwend    ! Loop over all "longwave" wavelength intervals
      iw = iw+1   ! Relative (to data file) spectral interval
      ir = 1  ! Initial value
      do ib=1,spc_nbins(ip)  ! Loop over # of airborne particle radius bins
         r_bin = ext_aerradius(ib,ig)*cm_to_um   ! Avg radius in this bin [um]
         spc_xsarea_fact(ib,ip) = real(pi_f4)*r_bin*r_bin*um2_to_m2  ! [m^2]

         ! Search for target particle radius; check if out of bounds:
         ir = locidx_1D_mono_type02(ia_abp_optprop(ia)%p_radius, r_bin, ir)
         if (ir == error_val_i4) then
            write (*, '(/a)') 'ERROR: optical property radius bin not found.'
            write (*, '(a,i3,a,i3,a,e10.3,2(a,e10.3),a)')  &
                 'airborne particle type:', ia, ', bin #', ib,  &
                 ', target radius [um] = ', r_bin,  &
                 ' ; (min,max) radii in file [um] = (',  &
                 minval(ia_abp_optprop(ia)%p_radius), ', ',  &
                 maxval(ia_abp_optprop(ia)%p_radius), ')'
            call exception(procedure_name, FATAL)
         end if

         ! Calculate interpolation factors:
         ft = min(ONE_f4, (r_bin-ia_abp_optprop(ia)%p_radius(ir))/  &
              (ia_abp_optprop(ia)%p_radius(ir+1)-  &
              ia_abp_optprop(ia)%p_radius(ir)))
         ft1 = max(ZERO_f4, ONE_f4-ft)

         ! Interpolate optical properties to target radii:
         sw_abp_optprop(ip)%Q_ext(:,:,ib,iw) =  &
              ia_abp_optprop(ia)%Q_ext(:,:,ir,iw)*ft1+  &
              ia_abp_optprop(ia)%Q_ext(:,:,ir+1,iw)*ft
         sw_abp_optprop(ip)%Q_sca(:,:,ib,iw) =  &
              ia_abp_optprop(ia)%Q_sca(:,:,ir,iw)*ft1+  &
              ia_abp_optprop(ia)%Q_sca(:,:,ir+1,iw)*ft
         sw_abp_optprop(ip)%gQsca(:,:,ib,iw) =  &
              ia_abp_optprop(ia)%gQsca(:,:,ir,iw)*ft1+  &
              ia_abp_optprop(ia)%gQsca(:,:,ir+1,iw)*ft
      end do   ! end: ib, ir

      ! Display some statistics about the optical properties:
      write (*, '(2a,2(1x,i4))') trim(data_fpath), ', specie ctype, ip:', ia, ip
      write (*, '(a,i2)') 'Shortwave spectral interval #', iw
      write (*, '(a,3(1x,e12.4))') 'Q_ext (min,avg,max): ',  &
           minval(sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           sum(sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))/  &
           size(sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           maxval(sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))
      write (*, '(a,3(1x,e12.4))') 'ss_albedo (min,avg,max): ',  &
           minval(sw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           sum(sw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))/  &
           size(sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           maxval(sw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw))
      write (*, '(a,3(1x,e12.4))') 'asym_factor (min,avg,max): ',  &
           minval(sw_abp_optprop(ip)%gQsca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           sw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw)),  &
           sum(sw_abp_optprop(ip)%gQsca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           sw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw))/  &
           size(sw_abp_optprop(ip)%Q_ext(n_fr/2+1,n_fb/2+1,:,iw)),  &
           maxval(sw_abp_optprop(ip)%gQsca(n_fr/2+1,n_fb/2+1,:,iw)/  &
           sw_abp_optprop(ip)%Q_sca(n_fr/2+1,n_fb/2+1,:,iw))
   end do   ! end: iwa, iw
end do  ! end: ip, ig, ia

! Deallocate/nullify some pointers:
do ia=1,aerad_ngrp
   if (associated(ia_abp_optprop(ia)%p_radius)) then
      e_check = .FALSE.
      call ptr_release(ia_abp_optprop(ia)%p_radius, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%fvws_r, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%fvws_b, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%Q_ext, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%Q_sca, e)
      e_check = e_check .or. (e /= 0)
      call ptr_release(ia_abp_optprop(ia)%gQsca, e)
      e_check = e_check .or. (e /= 0)
      if (e_check) call exception(procedure_name, FATAL, 'dealloc_2sw')
   end if
end do

end subroutine init_optprop


!```````````````````````````````````````````````````````````````````
subroutine air_kc_fpinterp

!+ Interpolates k-coefficients to a finer pressure grid

use univ_kind_defs_mod, only: f8
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use aermic_rad_mem_mod, only: dataset_path_bspec
use aertxrad_mem_mod, only: air_kc_nTref,air_kc_npref,air_kc_nwref,nlw_wavlnrng  &
     ,nsw_wavlnrng,ngauss_pts,fpref_air_log10,pref_air_log10,air_kc_lw  &
     ,air_kc_sw,air_kc_nfpref

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

real(f8), dimension(air_kc_nTref,air_kc_npref,air_kc_nwref  &
     ,nlw_wavlnrng,ngauss_pts+1) :: air_kc_lw_coarse
real(f8), dimension(air_kc_nTref,air_kc_npref,air_kc_nwref  &
     ,nsw_wavlnrng,ngauss_pts+1) :: air_kc_sw_coarse
integer :: n,m,nt,np,nh,ng,nw,i
real(f8) :: x,ans
real(f8), dimension(4) :: xi,yi

character(len=MAX_FILE_PATH_LEN) :: str_tmp

!=== Executable statements:

! from GCM2.0, Feb 2003
!
! Lagrange interpolation (linear in log pressure) of the "air" 
!  k-coefficients in the pressure domain.  Subsequent use of these
!  values will use a simple linear interpolation in pressure.

! Read in "air" k-coefficient tables from files:

str_tmp = trim(dataset_path_bspec)//'/CO2H2O_V_12_95-LE'
open(20, file=trim(str_tmp), status='old', form='unformatted')
read(20) air_kc_sw_coarse       ! "shortwave" wavelength group k-coeffs
close(20)

str_tmp = trim(dataset_path_bspec)//'/CO2H2O_IR_12_95-LE'
open(20, file=trim(str_tmp), status='old', form='unformatted')
read(20) air_kc_lw_coarse       ! "longwave" wavelength group k-coeffs
close(20)

! Take log(base 10) of the "air" k-coeff values (interpolate the log10 of the
!  values, not the values themselves); Limit minimum value to 1.d-200:

do nt=1,air_kc_nTref
   do np=1,air_kc_npref
      do nh=1,air_kc_nwref
         do ng=1,ngauss_pts
            
            do nw=1,nsw_wavlnrng
               if(air_kc_sw_coarse(nt,np,nh,nw,ng) > 1.e-200_f8) then
                  air_kc_sw_coarse(nt,np,nh,nw,ng) =  &
                       log10(air_kc_sw_coarse(nt,np,nh,nw,ng))
               else
                  air_kc_sw_coarse(nt,np,nh,nw,ng) = -200._f8
               endif
            enddo
            
            do nw=1,nlw_wavlnrng
               if(air_kc_lw_coarse(nt,np,nh,nw,ng) > 1.e-200_f8) then
                  air_kc_lw_coarse(nt,np,nh,nw,ng) =  &
                       log10(air_kc_lw_coarse(nt,np,nh,nw,ng))
               else
                  air_kc_lw_coarse(nt,np,nh,nw,ng) = -200._f8
               endif
            enddo
            
         enddo
      enddo
   enddo
enddo

! Interpolate the "air" k-coeff values to a finer reference pressure grid:

! for the "longwave" wavelength group:

do nt=1,air_kc_nTref
   do nh=1,air_kc_nwref
      do nw=1,nlw_wavlnrng
         do ng=1,ngauss_pts

            ! The initial interval (p = 1e-6 -> 1e-5):

            n = 1 
            do m=1,5
               x     = fpref_air_log10(m)
               xi(1) = pref_air_log10(n)
               xi(2) = pref_air_log10(n+1)
               xi(3) = pref_air_log10(n+2)
               xi(4) = pref_air_log10(n+3)
               yi(1) = air_kc_lw_coarse(nt,n,nh,nw,ng)
               yi(2) = air_kc_lw_coarse(nt,n+1,nh,nw,ng)
               yi(3) = air_kc_lw_coarse(nt,n+2,nh,nw,ng)
               yi(4) = air_kc_lw_coarse(nt,n+3,nh,nw,ng)
               call lagrange(x,xi,yi,ans)
               air_kc_lw(nt,m,nh,nw,ng) = 10._f8**ans
            enddo
            
            ! The majority of the pressure intervals:

            do n=2,air_kc_npref-2
               do m=1,5
                  i     = (n-1)*5+m
                  x     = fpref_air_log10(i)
                  xi(1) = pref_air_log10(n-1)
                  xi(2) = pref_air_log10(n)
                  xi(3) = pref_air_log10(n+1)
                  xi(4) = pref_air_log10(n+2)
                  yi(1) = air_kc_lw_coarse(nt,n-1,nh,nw,ng)
                  yi(2) = air_kc_lw_coarse(nt,n,nh,nw,ng)
                  yi(3) = air_kc_lw_coarse(nt,n+1,nh,nw,ng)
                  yi(4) = air_kc_lw_coarse(nt,n+2,nh,nw,ng)
                  call lagrange(x,xi,yi,ans)
                  air_kc_lw(nt,i,nh,nw,ng) = 10._f8**ans
               enddo
            enddo
            
            ! The final interval (p = 1e+3 -> ~1e+4):

            n = air_kc_npref-1
            do m=1,5
               i     = (n-1)*5+m
               x     = fpref_air_log10(i)
               xi(1) = pref_air_log10(n-2)
               xi(2) = pref_air_log10(n-1)
               xi(3) = pref_air_log10(n)
               xi(4) = pref_air_log10(n+1)
               yi(1) = air_kc_lw_coarse(nt,n-2,nh,nw,ng)
               yi(2) = air_kc_lw_coarse(nt,n-1,nh,nw,ng)
               yi(3) = air_kc_lw_coarse(nt,n,nh,nw,ng)
               yi(4) = air_kc_lw_coarse(nt,n+1,nh,nw,ng)
               call lagrange(x,xi,yi,ans)
               air_kc_lw(nt,i,nh,nw,ng) = 10._f8**ans
            enddo
            
            !  Fill the last pressure point (p = 1e+4):

            air_kc_lw(nt,air_kc_nfpref,nh,nw,ng) =  &
                 10._f8**air_kc_lw_coarse(nt,air_kc_npref,nh,nw,ng)

         enddo
      enddo
   enddo
enddo

! for the "shortwave" wavelength group:

do nt=1,air_kc_nTref
   do nh=1,air_kc_nwref
      do nw=1,nsw_wavlnrng
         do ng=1,ngauss_pts
            
            ! The initial interval (p = 1e-6 -> 1e-5):

            n = 1 
            do m=1,5
               x     = fpref_air_log10(m)
               xi(1) = pref_air_log10(n)
               xi(2) = pref_air_log10(n+1)
               xi(3) = pref_air_log10(n+2)
               xi(4) = pref_air_log10(n+3)
               yi(1) = air_kc_sw_coarse(nt,n,nh,nw,ng)
               yi(2) = air_kc_sw_coarse(nt,n+1,nh,nw,ng)
               yi(3) = air_kc_sw_coarse(nt,n+2,nh,nw,ng)
               yi(4) = air_kc_sw_coarse(nt,n+3,nh,nw,ng)
               call lagrange(x,xi,yi,ans)
               air_kc_sw(nt,m,nh,nw,ng) = 10._f8**ans
            enddo

            ! The majority of the pressure intervals:
            
            do n=2,air_kc_npref-2
               do m=1,5
                  i     = (n-1)*5+m
                  x     = fpref_air_log10(i)
                  xi(1) = pref_air_log10(n-1)
                  xi(2) = pref_air_log10(n)
                  xi(3) = pref_air_log10(n+1)
                  xi(4) = pref_air_log10(n+2)
                  yi(1) = air_kc_sw_coarse(nt,n-1,nh,nw,ng)
                  yi(2) = air_kc_sw_coarse(nt,n,nh,nw,ng)
                  yi(3) = air_kc_sw_coarse(nt,n+1,nh,nw,ng)
                  yi(4) = air_kc_sw_coarse(nt,n+2,nh,nw,ng)
                  call lagrange(x,xi,yi,ans)
                  air_kc_sw(nt,i,nh,nw,ng) = 10._f8**ans
               enddo
            enddo

            ! The final interval (p = 1e+3 -> ~1e+4):

            n = air_kc_npref-1
            do m=1,5
               i     = (n-1)*5+m
               x     = fpref_air_log10(i)
               xi(1) = pref_air_log10(n-2)
               xi(2) = pref_air_log10(n-1)
               xi(3) = pref_air_log10(n)
               xi(4) = pref_air_log10(n+1)
               yi(1) = air_kc_sw_coarse(nt,n-2,nh,nw,ng)
               yi(2) = air_kc_sw_coarse(nt,n-1,nh,nw,ng)
               yi(3) = air_kc_sw_coarse(nt,n,nh,nw,ng)
               yi(4) = air_kc_sw_coarse(nt,n+1,nh,nw,ng)
               call lagrange(x,xi,yi,ans)
               air_kc_sw(nt,i,nh,nw,ng) = 10._f8**ans
            enddo

            !  Fill the last pressure point (p = 1e+4):

            air_kc_sw(nt,air_kc_nfpref,nh,nw,ng) =  &
                 10._f8**air_kc_sw_coarse(nt,air_kc_npref,nh,nw,ng)
                
         enddo
      enddo
   enddo
enddo

end subroutine air_kc_fpinterp


!```````````````````````````````````````````````````````````````````
subroutine lagrange(x,xi,yi,ans)

!+ Performs Lagrange interpolation

use univ_kind_defs_mod, only: f8

implicit none

!=== Argument declarations:

real(f8), intent(IN) :: x
real(f8), dimension(4), intent(IN) :: xi,yi

real(f8), intent(OUT) :: ans

!== Local declarations:

! from GCM2.0, Feb 2003
!
! Lagrange interpolation - Polynomial interpolation at point x 
!  xi(1) <= x <= xi(4).  Yi(n) is the functional value at XI(n).
 
real(f8) :: fm1,fm2,fm3,fm4

!=== Executable statements

fm1 = x-xi(1)
fm2 = x-xi(2)
fm3 = x-xi(3)
fm4 = x-xi(4)

! Get the "answer" at the requested X:
 
ans = fm2*fm3*fm4*YI(1)/((XI(1)-XI(2))*(XI(1)-XI(3))*(XI(1)-XI(4))) +  &
      fm1*fm3*fm4*YI(2)/((XI(2)-XI(1))*(XI(2)-XI(3))*(XI(2)-XI(4))) +  &
      fm1*fm2*fm4*YI(3)/((XI(3)-XI(1))*(XI(3)-XI(2))*(XI(3)-XI(4))) +  &
      fm1*fm2*fm3*YI(4)/((XI(4)-XI(1))*(XI(4)-XI(2))*(XI(4)-XI(3)))

end subroutine lagrange


end module aerad_init_mod
