!```````````````````````````````````````````````````````````````````
module aerad_precalc_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine rad_precalc

!+ Calculates quantities needed before the main radiative transfer calcs can
!   be performed

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ONE_f8, ZERO_f8
use aertxrad_mem_mod, only: nz1_rad,ngweights,tau_gas,emis,sfc_emiss,taul,y3  &
     ,sfc_albedo,sol,sflux_frac,rsfx,w0,g0,opd  &
     ,tau_aer,singscat_albd,asym_fact,epsilon,epsilon_s,ngangles,g_angle  &
     ,naerspc_rad,ip_ibeg,ip_iend,iw_ibeg,iw_iend,gw_solflux,spc_radtx  &
     ,i_curr,j_curr,swcut,part_in_tshadow,sw_on,sw_ipbeg,sw_iwbeg,sw_iwend

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

real(f8) :: den,x,f0_ig,w0_ig,g0_ig,taul_ig,tmp
integer :: it,ig,k,ip,iw,k_1,ia

!=== Executable statements:

!   The vertical structure used by this radiation code is as follows:
!
!   Rad Index = Layer Boundaries
!   Atmos Index = Layer Centers
!
!   Layer                       Rad Index   Atmos Index   
!
!    top -----------------------  j = 1
!         - - - - - - - - - - -   j = 2       k = 1
!        -----------------------
!         - - - - - - - - - - -               k = 2
!        -----------------------
!         - - - - - - - - - - -
!        -----------------------
!         - - - - - - - - - - -               k = NZ-1
!        -----------------------  
!         - - - - - - - - - - -   j = 2*NZ-1  k = NZ
! bottom -----------------------  j = 2*NZ
!============================================================


! Map/average important parameters to gauss points (it):

it = ip_ibeg-1
do iw=iw_ibeg,iw_iend    ! Only necessary wavelength bands
   do ig=1,ngweights
      it = it+1
         
      emis(it) = sfc_emiss(iw)
      rsfx(it) = sfc_albedo(iw)
   enddo
enddo
   
if(sw_on) then
   if(part_in_tshadow) then   ! Eliminate direct "shortwave" flux within shadow
!%      it = ip_ibeg-1
      it = sw_ipbeg-1
!%      do iw=iw_ibeg,iw_iend    ! Only necessary wavelength bands
      do iw=sw_iwbeg,sw_iwend  ! Solar IR direct flux unused (currently)
         do ig=1,ngweights
            it = it+1
            sol(it,1:swcut-1) = gw_solflux(it)*sflux_frac
            sol(it,swcut:nz1_rad) = ZERO_f8
         enddo
      enddo
   else      ! No topographic shadowing in current column
!%      it = ip_ibeg-1
      it = sw_ipbeg-1
!%      do iw=iw_ibeg,iw_iend    ! Only necessary wavelength bands
      do iw=sw_iwbeg,sw_iwend  ! Solar IR direct flux unused (currently)
         do ig=1,ngweights
            it = it+1
            sol(it,1:nz1_rad) = gw_solflux(it)*sflux_frac
         enddo
      enddo
   endif
endif

opd(ip_ibeg:ip_iend,:) = ZERO_f8     ! Initialize necessary array portions in 
taul(ip_ibeg:ip_iend,:) = ZERO_f8    !  anticipation of summing below

do k=1,nz1_rad     ! Loop over all layer BOUNDARIES 
   k_1 = max(1,k-1)   ! index for level above (except for k=1, of course)
        
   it = ip_ibeg-1
   do iw=iw_ibeg,iw_iend      ! Only necessary wavelength bands
      do ig=1,ngweights
         it = it+1

         ! Combine the aerosol and gas optical parameters to get the total
         !  "effective" layer parameters:

         taul_ig = tau_gas(ig,iw,k)   !
         w0_ig = ZERO_f8                   !  Start with gas optical params 
         g0_ig = ZERO_f8                   !      

         do ip=1,naerspc_rad   ! Loop over all rad active aerosol species
            if(spc_radtx(ip)) then  ! Do only if specie currently rad_active

               taul_ig = taul_ig+tau_aer(ip,iw,k)  ! Add aerosol opacity
               tmp = singscat_albd(ip,iw,k)*tau_aer(ip,iw,k)
               w0_ig = w0_ig+tmp
               g0_ig = g0_ig+asym_fact(ip,iw,k)*tmp

            endif
         enddo

         if(taul_ig < epsilon) then   ! Clip if optical depth too small
            taul_ig = epsilon
         endif

         w0_ig = w0_ig/taul_ig        ! "total" weighted average singscat_albd
         w0_ig = min(ONE_f8-epsilon,w0_ig)  !

         if(w0_ig > epsilon) then                                    
            g0_ig = g0_ig/(w0_ig*taul_ig)  ! "total" weighted average asym_fact
         else
!?any point to this?            wsum = epsilon_s   !? w0_ig = epsilon
            g0_ig = ZERO_f8           ! "total" weighted average asym_fact
         endif

!if(i_curr == 75 .and. j_curr == 80 .and. k == 20) then
!   write(46,'(a)') '####################  Original      ##################'
!   write(46,'(a,3i4,3e10.2)') 'ortau',iw,ig,it,tau_gas(ig,iw,k),tau_aer(1,iw,k),tau_aer(2,iw,k)
!   write(46,'(a,3i4,3e10.2)') 'orssa',iw,ig,it,0.,singscat_albd(1,iw,k),singscat_albd(2,iw,k)
!   write(46,'(a,3i4,3e10.2)') 'orasf',iw,ig,it,0.,asym_fact(1,iw,k),asym_fact(2,iw,k)
!   write(46,'(a,3i4,f10.4)') 'solfr',iw,ig,it,sol(it,k)
!   write(46,'(a)') '####################  Layer-Weighted  ##################'
!   write(46,'(a,3i4,3e10.2)') 'lyrwt',iw,ig,it,taul_ig,w0_ig,g0_ig
!endif
  
         ! Apply delta-Eddington scaling to truncate the forward scattering
         !  lobe of the scattering phase function (effectively leave the
         !  energy within the truncated portion in the direct beam), as
         !  oringinally described in Joseph et al. (1976):
   
         f0_ig = g0_ig*g0_ig
         den = ONE_f8-w0_ig*f0_ig                                     
         taul(it,k) = taul_ig*den     ! "delta-scaled" total opt_depth
         w0(it,k) = (ONE_f8-f0_ig)*w0_ig/den ! "delta-scaled" total singscat_albd
         g0(it,k) = g0_ig/(ONE_f8+g0_ig)  ! "delta-scaled" total asym_fact
                                        ! Actually '(g0_ig-f0_ig)/(1._f8-f0_ig)'
         
         opd(it,k) = opd(it,k_1)+taul(it,k)   ! Cumulative (from top down)
                                              !  "delta-scaled" opt_depth
      enddo
      
   enddo

   do ia=1,ngangles
      do it=ip_ibeg,ip_iend
         x = taul(it,k)/g_angle(ia)  
         if(x <= 400._f8) then                                    
            y3(it,ia,k) = min(ONE_f8,exp(-x))
         else                                                   
            y3(it,ia,k) = ZERO_f8    ! would be effectively zero anyway
         endif
      enddo
   enddo

enddo

! Calculate Planck function stuff for current temperatures:

call dplanck
   
end subroutine rad_precalc


!```````````````````````````````````````````````````````````````````
subroutine dplanck

!+ Calculate Planck function and and its derivative at ground
!   and at all altitudes

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ONE_f8, ZERO_f8 
use aertxrad_mem_mod, only: t_lev,ptempg,sfc_tempk,nz1_rad,ptemp,epsilon,slope &
     ,taul,lw_ipbeg,tpft_finc,ptemp_itp,lw_ipend

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

real(f8) :: ft, taul_sfc
integer :: ip,k,k_1,it

!=== Executable statements:
                                                                       
! Calculate stuff based on the wavelength-dependent Planck function at the
!  ground surface and for all vertical levels:

do ip=lw_ipbeg,lw_ipend   ! Loop over all "longwave" points

   it = int(sfc_tempk*tpft_finc)
   ft = real(sfc_tempk*tpft_finc-real(it), kind=f8)

   ! Interpolate between table values:
   ptempg(ip) = ptemp_itp(it,ip)*(ONE_f8-ft)+ptemp_itp(it+1,ip)*ft

enddo

   ! Surface taul (even wafer-thin layer) is *very* large at wavelengths <= IR: 
taul_sfc = 1.e6_f8

do k=1,nz1_rad      ! Loop over all layer BOUNDARIES
   k_1 = max(1,k-1)

   do ip=lw_ipbeg,lw_ipend   ! Loop over all "longwave" points

      it = int(t_lev(k)*tpft_finc)
      ft = dble(t_lev(k)*tpft_finc-real(it))

      ! Interpolate between table values:
      ptemp(ip,k) = ptemp_itp(it,ip)*(ONE_f8-ft)+ptemp_itp(it+1,ip)*ft

      if(taul(ip,k) > epsilon) then
         if (k < nz1_rad) then
            slope(ip,k) = (ptemp(ip,k)-ptemp(ip,k_1))/taul(ip,k)
         else   ! "Include" some ground opacity
            slope(ip,k) = (ptemp(ip,k)-ptemp(ip,k_1))/(taul_sfc-taul(ip,k))
         end if
      else
         slope(ip,k) = ZERO_f8
      endif

   enddo

enddo

end subroutine dplanck


!```````````````````````````````````````````````````````````````````
function PLANCKf(e,t1) result(integrated_rad)

!+ Computes integral of the Planck function between zero and a given wavelength
                           
use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ONE_f8, ZERO_f8, ONE_THIRD_f8

implicit none

!=== Argument declarations:

real(f8) :: integrated_rad

real(f8), intent(IN) :: e,t1

!== Local declarations:                                                                     
!     ******************************************************            
!     *  Purpose             :  Calculate Planck Function  *            
!     *  Subroutines Called  :  None                       *            
!     *  Input               :  WAVE, NCOUNT               *            
!     *  Output              :  PLANK                      *            
!     * ****************************************************            
!                                                                       
!  THIS SUBROUTINE COMPUTES THE INTEGRAL OF THE PLANCK FUNCTION BETWEEN 
!  ZERO AND THE SPECIFIED VALUE OF LAMBDA.  THUS (USING XL AS LAMBDA)   
!  WE WANT TO INTEGRATE                                                 
!  R = INTEGRAL(XL=0 TO XL=XLSPEC) ( C1*XL**-5* / (EXP(C2/XL*T)-1) )*
!  SUBSTITUTING U=C2/(XL*T), THE INTEGRAL BECOMES                       
!  R = A CONSTANT TIMES INTEGRAL (USPEC TO INFINITY) OF                 
!            ( U**3 / (EXP(U) - 1) )*DU                                 
!  THE APPROXIMATIONS SHOWN HERE ARE ON PAGE 998 OF ABRAMOWITZ AND 
!  UNDER THE HEADING OF DEBYE FUNCTIONS.  C2 IS THE PRODUCT OF PLANCK'S 
!  CONSTANT AND THE SPEED OF LIGHT DIVIDED BY BOLTZMANN'S CONSTANT.     
!  C2 = 14390 WHEN LAMBDA IS IN MICRONS.                                
!  THE FACTOR 0.15399 IS THE RECIPROCAL OF SIX TIMES                    
!  THE SUM OF (1/N**2) FOR ALL N FROM ONE TO INFINITY.  IT IS CHOSEN TO 
!  NORMALIZE THE INTEGRAL TO A MAXIMUM VALUE OF UNITY.                  
!  RADIATION IN REAL UNITS IS OBTAINED BY MULTIPLYING THE INTEGRAL BY   
!  THE STEFAN-BOLTZMANN CONSTANT TIMES T**4.                            
                                                                       
real(f8), dimension(5) :: am
real(f8) :: d,v1,a
integer :: m

!=== Executable statements:
                                                                       
d = ZERO_f8 
v1 = e/t1                                            
                                                                       
if(v1 <= ONE_f8) then                                              
   d = ONE_f8-0.15399_f8*V1**3 *  &                           
        (ONE_THIRD_f8-v1/8._f8+v1**2/60._f8-v1**4/5040._f8+  &          
        v1**6/272160._f8-V1**8/13305600._f8)          
endif
                                                                      
if(v1 > ONE_f8 .and. v1 <= 50._f8) then                           
   do m=1,5                                              
      a = real(m, kind=f8)*v1                                      
      am(m) = 0.15399_f8*exp(-a)/m**4*(((a+3._f8)*a+6._f8)*a+6._f8)            
   enddo
                                                                
   d = am(1)+am(2)+am(3)+am(4)+am(5)                    
endif
                                                                       
integrated_rad = d*t1**4

end function PLANCKf


end module aerad_precalc_mod
