!```````````````````````````````````````````````````````````````````
module aerad_solflux_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
function solarflux(wn_lw,wn_up)

!+ Computes integrated solar intensity (flux) over given wavenumber range

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_std_fileIO_mod, only: find_file_IOunit
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use aermic_rad_mem_mod, only: dataset_path

implicit none

!=== Argument declarations:

real(f4) :: solarflux   ! Procedure type

real(f4), intent(IN) :: wn_lw,wn_up     ! wavenumber [cm^-1]

!== Local declarations:

integer, parameter ::  &
           modtran_dat_dim = 49951  &   ! # points in datafile 
          ,modtran_dat_dwn = 1  &    ! Delta wavenum (cm^-1) between pts
          ,modtran_dat_toplim = 50000  &   ! Wavenum (cm^-1) of iw=1
          ,modtran_dat_botlim = 50   ! Wavenum (cm^-1) of iw=modtran_dat_dim

real(f4), dimension(modtran_dat_dim), save :: solar_irrad,dat_wavlen
integer, dimension(modtran_dat_dim), save :: dat_wavnum
integer, save :: ncall = 0

real(f4) :: ftmp
integer :: iw, itop, ibot, IO_unit

character(len=MAX_FILE_PATH_LEN) :: fn_tmp

!=== Executable statements:

if (ncall == 0) then    ! Read in dataset (only do once)
   ! Read in data from file:
   !  NOTE: there are several datasets (from MODTRAN) in this file; we will use
   !        the Chance and Kurucz dataset (50 to 50000 [1/cm]; used in
   !        MODTRAN4?)
   !
   !  dat_wavnum -> [cm^-1],  dat_wavlen -> [nm],  solar_irrad -> [W/(m^2.nm)]

   IO_unit = find_file_IOunit()

   fn_tmp = trim(dataset_path)//'/common/solar_rad_MODTRAN.txt'
   open(IO_unit, file=trim(fn_tmp), status='old', form='formatted')
   read (IO_unit, *)   ! Skip header line
   do iw=1,modtran_dat_dim
      read (IO_unit, *) dat_wavnum(iw),dat_wavlen(iw),ftmp,solar_irrad(iw)
   enddo
   close(IO_unit)

   ncall = 1
end if

! itop  itop-1     ibot+1  ibot 
!   |     | .......... |     |
!    wn_lw              wn_up 
!          ----> wavenumber cm^-1
!          <---- data array index, wavelength           

itop = modtran_dat_toplim-int(wn_lw)+1    ! Index in data array (top index
                                          !  of the two bracketing indices)

ibot = modtran_dat_toplim-int(wn_up)      ! Index in data array (bottom index
                                          !  of the two bracketing indices)

if(ibot < 1) then
   ibot = 1       ! Don't try to use data we don't have
elseif(ibot > modtran_dat_dim) then
   solarflux = ZERO_f4
   return       ! Incorrect parameter
endif

if(itop > modtran_dat_dim-2) then
   itop =  modtran_dat_dim-2   ! Don't use iw=modtran_dat_dim irradiance
elseif(itop < 1) then
   solarflux = ZERO_f4
   return       ! Incorrect parameter
endif

iw = ibot    ! Add bottom fractional portion
ftmp = (solar_irrad(iw)+solar_irrad(iw+1))*0.5_f4*(dat_wavlen(iw+1)-1.e7_f4/wn_up)
do iw=ibot+1,itop-2    ! Integrate over bulk of requested interval
   ftmp = ftmp+(solar_irrad(iw)+solar_irrad(iw+1))*  &
        0.5_f4*(dat_wavlen(iw+1)-dat_wavlen(iw))
enddo
iw = itop-1       ! Add top fractional portion
ftmp = ftmp+(solar_irrad(iw)+solar_irrad(iw+1))*  &
     0.5_f4*(1.e7_f4/wn_lw-dat_wavlen(iw))

solarflux = ftmp      ! [W/m^2]

end function solarflux


end module aerad_solflux_mod
