!```````````````````````````````````````````````````````````````````
module aertxrad_mem_mod

!+ Module that contains memory structures common to the radiative transfer code
!   routines

use univ_kind_defs_mod, only: f8, f4
use univ_math_constants_mod, only: pi_f8, twopi_f8
use body_specific_constants_mod, only: g_Xoid, cp

implicit none

logical, parameter :: refine_lw_fluxes = .TRUE.

! Define double precision 1.0 to be multiplied by literal constants
!  that are embedded within parentheses:

real(f8), public, parameter :: ONE = 1._f8

real(f8), public, parameter :: avognum = 6.02252e23_f8  ! Avogadro's number [#/mol]

real(f8), public, parameter :: bk = 1.3806503e-23_f8  ! Boltzmann constant [J/K]

! Stefan-Boltzmann constant [J/(K^4.m^2.s)] OR [W/(K^4.m^2)]      

real(f8), public, parameter :: sbk = 5.670400e-8_f8

! Universal gas constant [J/(K.mol)]

real(f8), public, parameter :: R_univ = 8.3144_f8

real(f4), public, parameter :: CO2_atm_mr = 0.9532_f4  & ! CO2 mixing ratio of "air"
                            ,CO2n_amu = 44.01_f4 ! Atomic mass units per "natural"
                                              !  (isotopic mix) CO2 molecule 
real(f4), public, parameter :: amu = 1.6605e-27_f4   ! One atomic mass unit [kg]

! Constant for gas opacity calc (1.e-4 factor is cm^2 -> m^2 conversion):
real(f4), public, parameter :: gas_odc = 1.e-4_f4*CO2_atm_mr/(CO2n_amu*amu)

! Approximate smallest double precision floating point difference:
real(f8), public, parameter :: epsilon = 1.0e-12_f8
real(f4), public, parameter :: epsilon_s = 1.0e-12_f4

real(f8), public, parameter :: pi = pi_f8  &   ! PI
                            ,twopi = twopi_f8    ! 2.*PI

real(f4), public, parameter :: grav = g_Xoid  ! gravitational acceleration [m/s^2]
real(f4), public, parameter :: cp_air = cp  ! specific heat of "air" [J/(K.kg)] 

real(f8), public, parameter :: sqrt3 = 1.732050808_f8   ! square root of 3

! Define number of wavelength intervals and gauss points for calcs (these are
!  also some dimensions of the current air k-coeff dataset):

integer, public, parameter :: nsw_wavlnrng = 7  &  ! # shortwave wavln ranges
                               ,nlw_wavlnrng = 5  &  ! # infrared wavln ranges
                               ,ngauss_pts = 16  ! # of Gauss quadrature points
                                                 !  used per wavelength range

! Define constants for Gauss quadrature calculations:

integer, public, parameter :: ngangles = 8  &  ! # of Gauss angles to use
                               ,ngweights = 16   ! # of Gauss k-coeff weights
                                                 !  to use

! Further dimensions of current air (right now a CO2/H2Ov mixture?) k-coeff
!  dataset:

integer, public, parameter :: air_kc_npref = 11  & ! # of reference pressures
                               ,air_kc_nTref = 7  &  ! # of reference temps
                               ,air_kc_nwref = 10   ! # of ref H2Ov contents

! Number of "fine" reference pressure levels to interpolate air k-coeff
!  dataset to (see routine `laginterp`):

integer, public, parameter :: air_kc_nfpref = (air_kc_npref-2)*5+6

! Some dimensions derived from those above:

integer, public, parameter ::   &  ! Total # of wavelength ranges to be used
     ntot_wavlnrng = nsw_wavlnrng+nlw_wavlnrng

integer, public, parameter ::  &    ! Total # of shortwave probability ranges
     nsw_gpt =  nsw_wavlnrng*ngauss_pts
integer, public, parameter ::  &    ! Total # of infrared probability ranges
     nlw_gpt =  nlw_wavlnrng*ngauss_pts
integer, public, parameter ::  &    ! Total # of probability ranges
     ntot_gpt =  nsw_gpt+nlw_gpt

! <wavenum_edge_x> refers to the wavenumbers (1/wavelength) at the wavelength
!  interval edges (note that there are 'ntot_wavlnrng+1' edges for each
!  wavelength group) [cm^-1]:

real(f4), public, dimension(nlw_wavlnrng+1) :: wavenum_edge_lw
real(f4), public, dimension(nsw_wavlnrng+1) :: wavenum_edge_sw

   ! Overlaps slightly with the sw bands to capture the full ~4.7um CO2 gas
   !  spectral feature using the longwave methodology:
data wavenum_edge_lw /  &      ! "longwave" edges
     10._f4, 166.667_f4, 416.667_f4, 833.333_f4, 1250._f4, 2500._f4 /!2222.22_f4

data wavenum_edge_sw /  &      ! "shortwave" edges
     2222.22_f4, 3087.37_f4, 4030.63_f4, 5370.57_f4, 7651.11_f4, 12500._f4, 25000._f4, 41666.67_f4 /

!*** These are for the Gauss-split 0.95 case (????):

real(f8), public, dimension(ngauss_pts) :: g_xpos,g_weight
real(f8), public, dimension(ngangles) :: g_ang_weight,g_angle

! "x"-positions for Gaussian quadrature within each wavelength band [none]
!  (there are 'ngauss_pts' of these):
                                             
data g_xpos / 0.0446339553_f8, 0.15371455_f8, 0.30272520_f8, 0.47500000_f8  &
     ,0.64727480_f8, 0.79628545_f8, 0.90191644_f8, 0.95000000_f8, 0.95253071_f8  &
     ,0.95809021_f8, 0.96593287_f8, 0.97499997_f8, 0.98406706_f8, 0.99190975_f8  &
     ,0.99746927_f8, 0.99999999_f8  /

! Weights for Gaussian quadrature within each wavelength band [none] (there are
!  'ngauss_pts' of these):

data g_weight / 4.8083554740e-02_f8, 1.0563099137e-01_f8, 1.4901065679e-01_f8  &
     ,1.7227479710e-01_f8, 1.7227479710e-01_f8, 1.4901065679e-01_f8, 1.0563099137e-01_f8  &
     ,4.8083554740e-02_f8, 2.5307134073e-03_f8, 5.5595258613e-03_f8, 7.8426661469e-03_f8  &
     ,9.0670945845e-03_f8, 9.0670945845e-03_f8, 7.8426661469e-03_f8, 5.5595258613e-03_f8  &
     ,2.5307134073e-03_f8 /

! Weights for Gaussian quadrature (for each probability interval) over a
!  hemisphere subdivided into 'ngangles' equal angles  [none] (there are
!  'ngangles' of these):

! For ngangles = 8:
data g_angle / 0.0446339553_f8, 0.1443662570_f8, 0.2868247571_f8, 0.4548133152_f8  &
     , 0.6280678354_f8, 0.7856915206_f8, 0.9086763921_f8, 0.9822200849_f8 /
data g_ang_weight / 0.0032951914_f8, 0.0178429027_f8, 0.0454393195_f8  &
     ,0.0791995995_f8, 0.1060473594_f8, 0.1125057995_f8, 0.0911190236_f8  &
     ,0.0445508044_f8 / 
! For ngangles = 3:
!@data g_angle / 0.2123405382_f8, 0.5905331356_f8, 0.9114120405_f8 /
!@data g_ang_weight / 0.0698269799_f8, 0.2292411064_f8, 0.2009319137_f8 / 

! Planck function interpolation table temperatures, etc. [K]:

integer, public, parameter :: tpft_inc = 1000  & ! Increment [#pts/K]
                               ,tpft_beg = 1*tpft_inc  &  ! Start temp (1 K)
                               ,tpft_end = 400*tpft_inc  &  ! End temp (400 K)
                               ,tpft_nt = tpft_end  ! table temp dimension
real(f8), public, parameter :: tpft_invfinc = 1.e-3_f8  & ! Factor dble(1/tpft_inc)
                            ,tpft_dinc = 1.e3_f8   ! Factor dble(tpft_inc)
real(f4), public, parameter :: tpft_finc = 1.e3_f4   ! Factor real(tpft_inc)

real(f8), public, allocatable, dimension(:,:) ::   &
     ptemp_itp      ! Table [tpft_nt,ntot_gpt]

! These pressure, temperature, and water vapor (???) reference points are for
!  the CO2+H2Ov "air" k-coefficients (NASA Ames circa 2002???):

real(f4), public, dimension(air_kc_npref) :: pref_air_log10
real(f4), public, dimension(air_kc_nfpref) :: fpref_air_log10
real(f4), public, dimension(air_kc_ntref) :: tref_air
real(f4), public, dimension(air_kc_nwref) :: H2Oref_air

! log10(pressure) [mb] -- there are 'air_kc_npref' of these:
data pref_air_log10 / -6._f4, -5._f4, -4._f4, -3._f4, -2._f4, -1._f4, 0._f4 ,  &
     1._f4, 2._f4, 3._f4, 4._f4 /

! Temperature [K] -- there are 'air_kc_nTref' of these:
data Tref_air / 50._f4, 100._f4, 150._f4, 200._f4, 250._f4, 300._f4, 350._f4 /

! Water vapor [units???] -- there are 'air_kc_nwref' of these:
data H2Oref_air / 1.e-7_f4, 1.e-6_f4, 1.e-5_f4, 1.e-4_f4, 1.e-3_f4, 1.e-2_f4,  &
     1.e-1_f4, 2.e-1_f4, 3.e-1_f4, 4.e-1_f4 /

! log10(pressure) [mb] -- there are 'air_kc_nfpref' of these:

data fpref_air_log10 / -6._f4, -5.8_f4, -5.6_f4, -5.4_f4, -5.2_f4, -5._f4  &
     ,-4.8_f4, -4.6_f4, -4.4_f4, -4.2_f4, -4._f4, -3.8_f4, -3.6_f4, -3.4_f4, -3.2_f4  &
     ,-3._f4, -2.8_f4, -2.6_f4, -2.4_f4, -2.2_f4, -2._f4, -1.8_f4, -1.6_f4, -1.4_f4  &
     ,-1.2_f4, -1._f4, -0.8_f4, -0.6_f4, -0.4_f4, -0.2_f4, 0._f4, 0.2_f4, 0.4_f4  &
     ,0.6_f4, 0.8_f4, 1._f4, 1.2_f4, 1.4_f4, 1.6_f4, 1.8_f4, 2._f4, 2.2_f4, 2.4_f4  &
     ,2.6_f4, 2.8_f4, 3._f4, 3.2_f4, 3.4_f4, 3.6_f4, 3.8_f4, 4._f4 /

! Total # of *possible* radiatively active aerosol species; type IDs:
integer, public, parameter :: aerad_ngrp = 3
integer, public, parameter :: aeradID_dust = 1, aeradID_H2Oi = 2  &
                               ,aeradID_CO2i = 3

!------------------------------------------------------------------------------
! Radiative transfer model variable/array declarations (** USER SHOULD NOT NEED
!  TO CHANGE ANYTHING BELOW THIS **) :

type abp_optprop_obj
   real(f4), pointer, dimension(:) :: p_radius => NULL()
   real(f4), pointer, dimension(:,:,:,:) :: Q_ext => NULL(),  &
        Q_sca => NULL(), gQsca => NULL()
   real(f4), pointer, dimension(:) :: fvws_r => NULL(), fvws_b => NULL(),  &
        d_fvws_r => NULL(), d_fvws_b => NULL()
   real(f4) :: max_fvws_r, min_fvws_r, max_fvws_b, min_fvws_b
   integer :: n_fvws_r, n_fvws_b, search_beg_idx_fvws_r, search_beg_idx_fvws_b
end type abp_optprop_obj

type(abp_optprop_obj), allocatable, dimension(:) :: lw_abp_optprop,  &
     sw_abp_optprop              ! {naerspc_rad}

integer, public :: nz_rad,nz1_rad  &   ! vertical column (layer/level) dims 
     ,two_nz1_rad,scr_dim

integer, public :: naerspc_rad   ! # of radiatively active aerosol (not gas)
                                   !  species

integer, public :: max_aerbins,i_curr,j_curr,swcut

integer, public :: lw_iwbeg,lw_iwend,sw_iwbeg,sw_iwend,lw_ipbeg,lw_ipend  &
     ,sw_ipbeg,sw_ipend

real(f8), public :: cos_mu,sflux_frac

real(f4), public :: sw_sfc_dnflux,lw_sfc_upflux,lw_sfc_dnflux,sfc_tempk

integer, public :: ip_ibeg,ip_iend,iw_ibeg,iw_iend
logical, public :: sw_on,part_in_tshadow

real(f8), public, dimension(air_kc_npref) :: pref_log10_air

real(f8), public, dimension(air_kc_nTref,air_kc_nfpref,air_kc_nwref  &
     ,nsw_wavlnrng,ngauss_pts) :: air_kc_sw
real(f8), public, dimension(air_kc_nTref,air_kc_nfpref,air_kc_nwref  &
     ,nlw_wavlnrng,ngauss_pts) :: air_kc_lw

real(f8), public, dimension(ntot_gpt) :: emis,rsfx,ptempg,u1s,u1i,u1i_2  &
     ,gw_solflux

real(f4), public, dimension(ntot_wavlnrng) :: sfc_albedo,sfc_emiss
real(f8), public, dimension(ntot_wavlnrng) :: solflux

real(f8), public, allocatable, dimension(:,:) :: w0,g0  &   ! [ntot_gpt,nz1_rad]
     ,direct,opd,taul,direc,fnet,directu,slope,ptemp,acon,ak,bcon,b1,b2,ee1  &
     ,em1,em2,el1,el2,gami,b3,ck1,ck2,cpp,cpb,cm,cmb,ee3,el3,tmi,a1,a2,a3,a4  &
     ,a7,y5,sol

real(f8), public, allocatable, dimension(:,:,:) :: uintent  &
     ,dintent,y3,y1,y2,y4,y8                ! [ntot_gpt,ngangles,nz1_rad]

real(f8), public, allocatable, dimension(:,:) :: af,bf,ef,as,df  &
     ,ds,xk                      ! [ntot_gpt,two_nz1_rad]

real(f4), public, allocatable, dimension(:,:,:) ::  singscat_albd,asym_fact  &
     ,tau_aer           ! [naerspc_rad,ntot_wavlnrng,nz1_rad]

real(f4), public, allocatable, dimension(:,:) ::  &
     spc_xsarea_fact      ! [max_aerbins,naerspc_rad]

real(f4), public, allocatable, dimension(:,:) ::  &
     bdst_Nconc          ! {nz1_rad,max_aerbins}

real(f4), public, allocatable, dimension(:,:) ::  &
     bdst_N_scr                      ! {nz1_rad+10,max_aerbins}
real(f4), public, allocatable, dimension(:) :: bdst_dz_scr   ! {nz1_rad+10}

real(f4), public, allocatable, dimension(:) :: bdst_ndist_bfrac  ! {max_aerbins}

integer, public, allocatable, dimension(:) :: spc_nbins  &   ! [naerspc_rad]
     ,spc_begiv,spc_endiv,spc_ctype,spc_ig
logical, public, allocatable, dimension(:) :: spc_radtx     ! [naerspc_rad]

real(f4), public, allocatable, dimension(:) :: p, t, qH2O,  &      ! [nz1_rad]
     sw_dTdt, lw_dTdt, net_dTdt, rho_air, dzc

real(f8), public, allocatable, dimension(:) :: z_flux_from_RT, z_flux_for_dTdt

real(f4), public, allocatable, dimension(:) :: p_lev, t_lev, qH2O_lev,  &
     dz_dTdt  ! [nz1_rad]

real(f8), public, allocatable, dimension(:) :: sw_upflux,  &   ! [nz1_rad]
     sw_dnflux, lw_upflux, lw_dnflux, sw_upflux_final, sw_dnflux_final,  &
     lw_upflux_final, lw_dnflux_final

real(f8), public, allocatable, dimension(:,:) :: up_fluxes,  &
     dn_fluxes                             ! {nz1_rad,ntot_wavlnrng}

real(f8), public, allocatable, dimension(:,:) :: lkcoef    ! [nz1_rad,4]

real(f4), public, allocatable, dimension(:,:,:) ::  &
     tau_gas               ! [ngauss_pts,ntot_wavlnrng,nz1_rad)

real(f8), public, allocatable, dimension(:) :: dscr1,dscr2,dscr3     ! [scr_dim]
real(f4), public, allocatable, dimension(:) :: fscr1,fscr2,fscr3     ! [scr_dim]

integer, public, allocatable, dimension(:) :: iscr1,iscr2,iscr3  ! [scr_dim]

logical, public :: want_rtflux_output

real(f4) :: solar_sw   ! Solar flux (@ 1AU) within this RT code's "shortwave"
                       !  spectral range [W/m^2]


contains


!```````````````````````````````````````````````````````````````````
subroutine aertxrad_AD(on_or_off)

!+ Allocates or deallocates key module arrays

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

logical, intent(IN) :: on_or_off

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'aertxrad_AD'

integer :: aerr,ntgp,nz1,tnz1,ngp,mb,na,ntwv,nga

!=== Executable statements:
 
ntgp = ntot_gpt
nz1 = nz1_rad
tnz1 = two_nz1_rad
ngp = ngauss_pts
nga = ngangles
mb = max_aerbins
na = naerspc_rad
ntwv = ntot_wavlnrng

if(on_or_off) then

   allocate(w0(ntgp,nz1),g0(ntgp,nz1),direct(ntgp,nz1),opd(ntgp,nz1)  &
        ,taul(ntgp,nz1),direc(ntgp,nz1),fnet(ntgp,nz1),directu(ntgp,nz1)  &
        ,slope(ntgp,nz1),ptemp(ntgp,nz1),acon(ntgp,nz1),ak(ntgp,nz1)  &
        ,bcon(ntgp,nz1),b1(ntgp,nz1),b2(ntgp,nz1),ee1(ntgp,nz1)  &
        ,em1(ntgp,nz1),em2(ntgp,nz1),el1(ntgp,nz1),el2(ntgp,nz1)  &
        ,gami(ntgp,nz1),b3(ntgp,nz1),ck1(ntgp,nz1),ck2(ntgp,nz1)  &
        ,cpp(ntgp,nz1),cpb(ntgp,nz1),cm(ntgp,nz1),cmb(ntgp,nz1)  &
        ,ee3(ntgp,nz1),el3(ntgp,nz1),tmi(ntgp,nz1),a1(ntgp,nz1)  &
        ,a2(ntgp,nz1),a3(ntgp,nz1),a4(ntgp,nz1),a7(ntgp,nz1),y5(ntgp,nz1)  &
        ,tau_gas(ngp,ntwv,nz1),sol(ntgp,nz1), stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD01')
   
   allocate(uintent(ntgp,nga,nz1),dintent(ntgp,nga,nz1),y3(ntgp,nga,nz1)  &
        ,y1(ntgp,nga,nz1),y2(ntgp,nga,nz1),y4(ntgp,nga,nz1)  &
        ,y8(ntgp,nga,nz1),ptemp_itp(tpft_nt,ntgp), stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD02')

   allocate(af(ntgp,tnz1),bf(ntgp,tnz1),ef(ntgp,tnz1),as(ntgp,tnz1)  &
        ,df(ntgp,tnz1),ds(ntgp,tnz1),xk(ntgp,tnz1), bdst_dz_scr(nz1+10),  &
        stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD03')

   if (mb > 0) then
      allocate(bdst_Nconc(nz1,mb), bdst_ndist_bfrac(mb),  &
           bdst_N_scr(nz1+10,mb), stat=aerr)
      if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'aerradAD04a')

      if (na > 0) then
         allocate(spc_xsarea_fact(mb,na), stat=aerr)
         if (aerr /= 0) call illegal_alloc_stop(for_allocation,  &
              procedure_name, 'aerradAD04b')
      end if
   end if

   if (na > 0) then
      allocate(tau_aer(na,ntwv,nz1), singscat_albd(na,ntwv,nz1),  &
           asym_fact(na,ntwv,nz1), stat=aerr)
      if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'aerradAD05a')

      allocate(spc_nbins(na),spc_begiv(na),spc_endiv(na),spc_ctype(na)  &
           ,spc_ig(na),spc_radtx(na), stat=aerr)
      if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'aerradAD05b')
   end if
   
   allocate(p(nz1),t(nz1),qH2O(nz1),sw_dTdt(nz1),lw_dTdt(nz1)  &
        ,net_dTdt(nz1),rho_air(nz1),dzc(nz1), z_flux_from_RT(nz1),  &
        z_flux_for_dTdt(nz1), stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD06')

   allocate(p_lev(nz1), t_lev(nz1), qH2O_lev(nz1), lkcoef(nz1,4),  &
        sw_upflux(nz1), sw_dnflux(nz1), lw_upflux(nz1), lw_dnflux(nz1),  &
        sw_upflux_final(nz1), sw_dnflux_final(nz1), lw_upflux_final(nz1),  &
        lw_dnflux_final(nz1), dz_dTdt(nz1), stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD07')

   allocate(dscr1(scr_dim),dscr2(scr_dim),dscr3(scr_dim),fscr1(scr_dim)  &
        ,fscr2(scr_dim),fscr3(scr_dim),iscr1(scr_dim),iscr2(scr_dim)  &
        ,iscr3(scr_dim), stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD08')

   allocate(up_fluxes(nz1,ntot_wavlnrng), dn_fluxes(nz1,ntot_wavlnrng),  &
        stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        'aerradAD09')

else

   deallocate(w0,g0,direct,opd,taul,direc,fnet,directu,slope,ptemp,acon  &
        ,ak,bcon,b1,b2,ee1,em1,em2,el1,el2,gami,b3,ck1,ck2,cpp,cpb,cm,cmb  &
        ,ee3,el3,tmi,a1,a2,a3,a4,a7,y5,tau_gas,sol, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD01')
   
   deallocate(uintent,dintent,y3,y1,y2,y4,y8,ptemp_itp, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD02')
   
   deallocate(af,bf,ef,as,df,ds,xk, bdst_dz_scr, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD03')

   if (mb > 0) then
      deallocate(bdst_Nconc, bdst_ndist_bfrac, bdst_N_scr, stat=aerr)
      if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aerradAD04a')

      if (na > 0) then
         deallocate(spc_xsarea_fact, stat=aerr)
         if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
              procedure_name, 'aerradAD04b')
      end if
   end if

   if (na > 0) then
      deallocate(tau_aer, singscat_albd, asym_fact, stat=aerr)
      if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aerradAD05a')

      deallocate(spc_nbins, spc_begiv, spc_endiv, spc_ctype, spc_ig,  &
           spc_radtx, stat=aerr)
      if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
           procedure_name, 'aerradAD05b')
   end if

   deallocate(p,t,qH2O,sw_dTdt,lw_dTdt,net_dTdt,rho_air,dzc, z_flux_from_RT,  &
        z_flux_for_dTdt, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD06')

   deallocate(p_lev, t_lev, qH2O_lev, lkcoef, sw_upflux, sw_dnflux,  &
        lw_upflux, lw_dnflux, sw_upflux_final, sw_dnflux_final,  &
        lw_upflux_final, lw_dnflux_final, dz_dTdt, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD07')

   deallocate(dscr1,dscr2,dscr3,fscr1,fscr2,fscr3,iscr1,iscr2,iscr3  &
        , stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD08')

   deallocate(up_fluxes, dn_fluxes, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'aerradAD09')

endif

end subroutine aertxrad_AD


!```````````````````````````````````````````````````````````````````
function get_ntot_wavlnrng()  result(answer)

!+ Inquiry method

!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

integer :: answer     ! Procedure result

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'get_ntot_wavlnrng'

!=== Executable statements:

answer = ntot_wavlnrng

end function get_ntot_wavlnrng


end module aertxrad_mem_mod
