!```````````````````````````````````````````````````````````````````
module no_atm_RT_driver_mod

!+

implicit none

integer, parameter :: n_spectral_bands = 2

integer, parameter :: LW = 1, SW = 2


contains


!```````````````````````````````````````````````````````````````````
subroutine no_atm_RT_driver(ext_n1, ext_n2, ext_n3, ext_n5, ext_ia, ext_iz,  &
     ext_ja, ext_jz, ext_sfcT, ext_cosZ, ext_msdist, ext_albedo,  &
     ext_emissivity,  &
     ext_rlong, ext_stellar_azm_ang, ext_tazm_ang, ext_tslope_ang,  &
     ext_tslas_tog, ext_tshadow_tog, ext_nazm_tshadow, ext_cosz_horizon,  &
     ext_TCx_obstruct, ext_TCz_obstruct, init_rad, ext_stellar_ET_flux,  &
     ext_flxout_imask, ext_bottom_other_sflux_down, ext_bottom_SWflux_down,  &
     ext_topo_factor_sflux, ext_fluxo_np, ext_up_fluxes,  &
     ext_dn_fluxes, ext_dT_dt, ext_stellar_sw)

!+ Modular radiative transfer code driver (RT assuming no atmosphere)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: twopi_f4, ZERO_f4, ONE_f4
use univ_phys_constants_mod, only: stfnbltz_f4
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use body_specific_constants_mod, only: solar_mean_broadband

implicit none

!=== Argument declarations:

integer, intent(IN) :: ext_n1, ext_n2, ext_n3, ext_n5, ext_ia, ext_iz,  &
     ext_ja, ext_jz, ext_tslas_tog, ext_tshadow_tog, ext_nazm_tshadow
real(f4), intent(IN) :: ext_msdist, ext_stellar_ET_flux
real(f4), dimension(ext_n2,ext_n3,ext_n5), intent(IN) :: ext_sfcT
real(f4), dimension(ext_n2,ext_n3), intent(IN) :: ext_albedo, ext_cosZ,  &
     ext_stellar_azm_ang, ext_tazm_ang, ext_tslope_ang, ext_emissivity
real(f4), dimension(ext_n2,ext_n3,ext_nazm_tshadow), intent(IN) ::  &
     ext_cosz_horizon, ext_TCx_obstruct, ext_TCz_obstruct
logical, intent(IN) :: init_rad
integer, dimension(ext_n2,ext_n3), intent(IN) :: ext_flxout_imask
integer, intent(IN) :: ext_fluxo_np

real(f4), dimension(ext_n2,ext_n3), intent(INOUT) :: ext_rlong,  &
     ext_bottom_other_sflux_down, ext_bottom_SWflux_down, ext_topo_factor_sflux

real(f4), dimension(ext_n1,n_spectral_bands,ext_fluxo_np), optional,  &
     intent(INOUT) :: ext_up_fluxes, ext_dn_fluxes
real(f4), dimension(ext_n1,ext_fluxo_np), optional, intent(INOUT) :: ext_dT_dt
real(f4), optional, intent(OUT) :: ext_stellar_sw

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'no_atm_RT_driver'
    
!!real(f4) :: tmp,sinz,cosai,atmp,aint,cosz_h,x_obst,z_obst,sw_cutoff
!!integer :: iv,ig,ik,k,i,j,iw,ip,ia0,ia1, ix_fo, iy_fo
!!logical :: horizon_extension,hextn_active
!%logical :: found

logical :: horizon_extension, hextn_active, part_in_tshadow, sw_on,  &
     want_rtflux_output
integer :: i, j, ia0, ia1, iw, ip
real(f4), dimension(n_spectral_bands) :: flux_up, flux_dn, sfc_albedo,  &
     sfc_emissivity
real(f4) :: sinz, cosai, atmp, aint, cosz_h, x_obst, z_obst, sw_cutoff,  &
     sflux_frac, sfc_tempk, cos_mu

!=== Executable statements:

!@if (init_rad) then     ! Initialize radiative transfer model (do only once)
!@
!@end if

! Check whether optional arguments are required:
if (maxval(ext_flxout_imask) > 0) then
   if (.not. (present(ext_up_fluxes) .and. present(ext_dn_fluxes) .and.  &
        present(ext_dT_dt) .and. present(ext_stellar_sw)))  &
        call exception(procedure_name, FATAL,  &
        'Optional arguments not provided, but are required.')
end if

! Fraction of the interplanetary stellar flux (at 1 AU) at top of atmosphere:
sflux_frac = ONE_f4/ext_msdist    ! [1/AU^2]

do j=ext_ja,ext_jz        ! Loop over all *computational* horizontal grid
   do i=ext_ia,ext_iz     !  points of external model

      flux_dn(:) = ZERO_f4; flux_up(:) = ZERO_f4

      ! Set ground (surface) temperature:
      sfc_tempk = ext_sfcT(i,j,1)   ! [K]   !@@ FIX! Integrate over patches!

      sfc_emissivity(LW) = ext_emissivity(i,j)
      sfc_albedo(LW) = ONE_f4-sfc_emissivity(LW)
      flux_up(LW) = sfc_emissivity(LW)*stfnbltz_f4*(sfc_tempk**4)

      !*** Set cosine of incident stellar angle (zenith angle); take account of
      !     (crudely) any horizon extension:

      cos_mu = ext_cosZ(i,j)     ! [none]

      hextn_active = .FALSE.
      if (ext_tshadow_tog == 1) then   ! Need to check for horizon extension
      
         ! Calculate cosz_horizon array location/interpolative indices (using
         !  stellar azimuth values):
         atmp = ext_stellar_azm_ang(i,j)/  &
              (twopi_f4/real(ext_nazm_tshadow, kind=f4))
         ia0 = int(atmp)
         aint = atmp-real(ia0, kind=f4)  ! Location crucial to avoid ia0 issues
         
         if (ia0 == 0) then
            ia0 = ext_nazm_tshadow; ia1 = 1
         else if (ia0 == ext_nazm_tshadow) then
            ia1 = 1
         else
            ia1 = ia0+1
         end if

         ! Calc horizon cosz, etc. for current stellar azimuth via interpolation:
         z_obst = (ONE_f4-aint)*ext_TCz_obstruct(i,j,ia0)+  &
              aint*ext_TCz_obstruct(i,j,ia1)
         if (z_obst < epsilon(z_obst)) then   ! Horizon extension
            horizon_extension = .TRUE.
            cosz_h = (ONE_f4-aint)*ext_cosz_horizon(i,j,ia0)+  &
                 aint*ext_cosz_horizon(i,j,ia1)
            if (cos_mu > cosz_h .and. cos_mu < 4.e-2_f4) then
               ! Sun is above extended horizon, but still low in sky;
               !  cosZ=0.04 =~ 87.71deg
               cos_mu = min(cos_mu-cosz_h, 4.e-2_f4)
               hextn_active = .TRUE.
            end if
         else
            horizon_extension = .FALSE.
         end if
      end if

      ! Only calculate "shortwave" stuff if sun is sufficiently above horizon;
      !  set probability point interval indices for use in radiative transfer
      !  calcs:

      sw_on = ( cos_mu > 1.e-6_f4 )

      if (sw_on) then

         ! Set "shortwave" wavelength group sfc optical properties:      
         sfc_albedo(SW) = ext_albedo(i,j)       ! [none]

         ! Topographic slope/aspect correction:

         if (ext_tslas_tog == 1) then
            sinz = sin(acos(cos_mu))
            
            ! If cosai < 0., point not sunlit
            cosai = (cos_mu*cos(ext_tslope_ang(i,j)))+  &
                 (sinz*sin(ext_tslope_ang(i,j))*  &
                 cos(ext_stellar_azm_ang(i,j)-ext_tazm_ang(i,j)))
         end if

         ! Topographic shadowing:

         part_in_tshadow  = .FALSE.   ! Initialize
         if (ext_tshadow_tog == 1 .and.  &     ! Possible shadowing
              .NOT.horizon_extension) then
            cosz_h = (ONE_f4-aint)*ext_cosz_horizon(i,j,ia0)+  &
                 aint*ext_cosz_horizon(i,j,ia1)
            x_obst = (ONE_f4-aint)*ext_TCx_obstruct(i,j,ia0)+  &
                 aint*ext_TCx_obstruct(i,j,ia1)
!Calc'd          z_obst = (1.-aint)*ext_TCz_obstruct(i,j,ia0)+  &
!above                 aint*ext_TCz_obstruct(i,j,ia1)

            sw_cutoff = z_obst-(x_obst*tan(asin(real(cos_mu)))) 

            part_in_tshadow = ( sw_cutoff > ZERO_f4 ) ! Part of column shadowed?
         end if

         ! Stellar beam adjusted for stellocentric distance and stellar
         !  incidence angle:
         if (.not.part_in_tshadow) then ! No direct "shortwave" flux in shadow
            flux_dn(SW) = ext_stellar_ET_flux*sflux_frac*cos_mu
            flux_up(SW) = flux_dn(SW)*sfc_albedo(SW)
         end if

      end if    

      !================================================================

      ! Calc/save 2-stream fluxes?
      want_rtflux_output = ( ext_flxout_imask(i,j) > 0 )

      !================================================================

      ! *Incident* sfc "shortwave" flux [W/m^2]:
      if (sw_on) then
         ! The portion of the stellar flux (at the body's surface) that *IS*
         !  treated by the "shortwave" atmospheric RT calculations:
         ext_bottom_SWflux_down(i,j) = flux_dn(SW)

         ! The portion of the stellar flux (at the body's surface) that *IS NOT*
         !  treated by the "shortwave" atmospheric RT calculations:
         ext_bottom_other_sflux_down(i,j) =  &
              (solar_mean_broadband-ext_stellar_ET_flux)/ext_msdist*cos_mu

         if (ext_tslas_tog == 1 .and. .NOT.hextn_active) then
            ! Multiply by cosai to take both topography orientation AND stellar
            !  incidence angle into account (after dividing by cos_mu to obtain
            !  the non-angle-corrected incident stellar flux):
            ext_topo_factor_sflux(i,j) = cosai/cos_mu
         else
            ! Take ONLY cos(Z) into account:
            ext_topo_factor_sflux(i,j) = ONE_f4
         end if
      else
         ext_bottom_SWflux_down(i,j) = ZERO_f4
         ext_bottom_other_sflux_down(i,j) = ZERO_f4
      end if

      ext_rlong(i,j) = flux_dn(LW)  ! Surface "longwave" down flux [W/m^2]

      if (want_rtflux_output) then        ! Calc/save 2-stream fluxes
         ip = ext_flxout_imask(i,j)
         do iw=1,n_spectral_bands
            ext_up_fluxes(:,iw,ip) = flux_up(iw)
            ext_dn_fluxes(:,iw,ip) = flux_dn(iw)
         end do

         ext_dT_dt(:,ip) = ZERO_f4

         ! The portion of the stellar flux @1 AU that *IS* treated by the
         !  "shortwave" atmospheric RT calculations:
         ext_stellar_sw = ext_stellar_ET_flux
      end if
 
   end do
end do

end subroutine no_atm_RT_driver


end module no_atm_RT_driver_mod
