!```````````````````````````````````````````````````````````````````
module radn_CO2fit_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine CO2fit(ABSARY,W,PAVGIN,OUTCOEF)

!+ Interpolate CO2 emissivity data (for 8-byte floating-point values)

use univ_kind_defs_mod, only: f4, f8
use univ_math_constants_mod, only: ZERO_f4
use radn_CO2ea_mem_mod, only: DIM_CO2_data_pr,DIM_CO2_data_cn,DIM_CO2_data_tm  &
     ,wr,ncon,prln,tr
use search_mod, only: srchgt_dblinp
use spline_mod, only: calc_spline

implicit none

!=== Argument declarations:

real(f8), dimension(DIM_CO2_data_pr,DIM_CO2_data_cn,DIM_CO2_data_tm), intent(IN) :: &
     absary
real(f8), intent(IN) :: w, pavgin

real(f4), dimension(3), intent(INOUT) :: outcoef

!== Local declarations:

!  PURPOSE
!      TO INTERPOLATE AND OTHERWISE ABUSE THE CO2 DATA.
!      CO2FIT DOES A CUBIC SPLINE INTERPOLATION OF THE ABS15
!      AND ABSSOL CO2 ABSORPTION DATA FOUND IN BLOCK DATA
!      FITCO2.  AT A GIVEN GAS CONCENTRATION, THE INTERPOLATION
!      OF THE (NATURAL) LOG ABSORPTION AT THE (NATURAL) LOG OF
!      THE PRESSURE GRID.  A QUADRADIC IS THEN FIT THROUGH
!      THESE VALUES AT THE THREE TEMPERATURE GRID POINTS, AND
!      THE COEFFICIENTS OF THE FIT ARE RETURNED.
!
!  INPUT VARIABLES:
!      ABSARY  - THE CO2 ABSORPTION ARRAY TO BE INTERPOLATED
!                (EITHER ABS15 OR ABSSOL)
!      W       - THE GAS CONCENTRATION
!      PAVGIN  - THE AVERAGED PRESSURE
!                USED AS PAVG = LOG(PAVGIN)
!
!  OUTPUT VARIABLES:
!      OUTCOEF - COEFFICIENTS OF THE QUADRATIC FIT (OF THE FORM:
!        Y = OUTCOEF(1) + X*OUTCOEF(2) + X*X*OUTCOEF(3) )
!
!  AUTHOR
!      J.R. SCHAEFFER    INFORMATICS    TASK 904    JULY 86
!
!  FOR
!      JIM POLLACK     ( SEE NOTES OF 8-8-86 AND 8-28-86 )
!
!  ENVIRONMENT
!      Cray Y-MP         UNICOS 4.0     FORTRAN 77
!
!  REVISION HISTORY
!       IMSL routines ICSCCU and ICSEVU were replaced.  The new
!       IMSL library renamed all routines.
!       SPLINE and SPEVAL replace IMSL routines.
!
!  CALLED BY:
!      EMISS
!
!  ROUTINES CALLED:
!      SPLINE, SPEVAL

real(f8), dimension(DIM_CO2_data_pr) :: ELN, YOUT
real(f8), dimension(3) :: EWP
real(f8) :: pavg,ans,wrln1,wrln2
real(f4) :: tmp21,tmp31,tmp32
integer :: nlpres,nstar,nstar1,j,m

!=== Executable statements:

nlpres = DIM_CO2_data_pr-1

! IF GAS CONCENTRATIONS ARE UNREALISTICALLY LOW (OR ZERO)
! SET COEFFICIENTS TO ZERO AND EXIT THIS MESS.

if(w.lt.1.e10_f8) then
   outcoef(1) = ZERO_f4
   outcoef(2) = ZERO_f4
   outcoef(3) = ZERO_f4
   return
endif

! CONVERT INPUT PRESSURE TO LN PRESSURE

pavg = log(pavgin)

! DETERMINE GAS CONCENTRATION INDICES

if(w.gt.wr(2)) then
   nstar = min(srchgt_dblinp(ncon-1,real(wr, kind=f8),1,w)-1,ncon-1)
else
   nstar = 1
endif

nstar1 = nstar+1

! FIT THE DATA

wrln1 = LOG(W/WR(NSTAR))
wrln2 = LOG(WR(NSTAR1)/WR(NSTAR))

do J=1,DIM_CO2_data_tm
   do M=1,DIM_CO2_data_pr

! LINEAR (IN LN SPACE) INTERPOLATION BETWEEN GAS
! CONCENTRATIONS ACCESSED BY NSTAR AND NSTAR+1

      ELN(M) = LOG(ABSARY(M,NSTAR,J))+(wrln1*  &
           LOG(ABSARY(M,NSTAR1,J)/ABSARY(M,NSTAR,J))/wrln2)
   enddo

   if(pavg.gt.prln(1)) then
! DO A CUBIC SPLINE FIT IN THE PRESSURE DIRECTION

      CALL calc_SPLINE(PRLN,ELN,NLPRES,YOUT)
      CALL SPEVAL(PRLN,ELN,YOUT,NLPRES,PAVG,ANS,1)

   else

! PRESSURE IS LOWER THAN PRLN(1) - DO A LINEAR FIT FROM
! THE LOWEST TWO PRESSURE POINTS OF THE GRID.

      ANS = ELN(1)+(PAVG-PRLN(1))*(ELN(2)-ELN(1))/(PRLN(2)-PRLN(1))
   endif

! INTERPOLATED VALUE AT A GIVEN TEMPERATURE

   EWP(J) = EXP(ANS)

enddo

! DETERMINE THE QUADRADIC-FIT COEFFICIENTS OVER TEMPERATURE
! GRID POINTS

tmp21 = tr(2)-tr(1)
tmp32 = tr(3)-tr(2)
tmp31 = tr(3)-tr(1)

OUTCOEF(3) = ewp(1)/(tmp31*tmp21)-ewp(2)/(tmp32*tmp21)+ewp(3)/(tmp32*tmp31)
OUTCOEF(2) = (ewp(2)-ewp(1))/tmp21-outcoef(3)*(tr(2)+tr(1))
OUTCOEF(1) = ewp(1)-outcoef(2)*tr(1)-outcoef(3)*tr(1)*tr(1)

end subroutine CO2fit


!```````````````````````````````````````````````````````````````````
subroutine co2fit4(absary,w,pavgin,outcoef,DIM_CO2_data_pr,DIM_CO2_data_cn  &
     ,DIM_CO2_data_tm,wr,tr,prln,ncon)

!+ Interpolate CO2 emissivity data (for 4-byte floating-point values)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use search_mod, only: jsrchgt
use spline_mod, only: calc_spline

implicit none

!=== Argument declarations:

integer, intent(IN) :: DIM_CO2_data_pr,DIM_CO2_data_cn,DIM_CO2_data_tm,ncon
real(f4), dimension(DIM_CO2_data_pr,DIM_CO2_data_cn,DIM_CO2_data_tm), intent(IN) :: &
     absary
real(f4), dimension(DIM_CO2_data_cn), intent(IN) :: wr
real(f4), dimension(DIM_CO2_data_tm), intent(IN):: tr
real(f4), dimension(DIM_CO2_data_pr), intent(IN) :: prln
real(f4), intent(IN) :: w,pavgin

real(f4), dimension(3), intent(INOUT) :: outcoef

!== Local declarations:

!  PURPOSE
!      TO INTERPOLATE AND OTHERWISE ABUSE THE CO2 DATA.
!      CO2FIT DOES A CUBIC SPLINE INTERPOLATION OF THE ABS15
!      AND ABSSOL CO2 ABSORPTION DATA FOUND IN BLOCK DATA
!      FITCO2.  AT A GIVEN GAS CONCENTRATION, THE INTERPOLATION
!      OF THE (NATURAL) LOG ABSORPTION AT THE (NATURAL) LOG OF
!      THE PRESSURE GRID.  A QUADRADIC IS THEN FIT THROUGH
!      THESE VALUES AT THE THREE TEMPERATURE GRID POINTS, AND
!      THE COEFFICIENTS OF THE FIT ARE RETURNED.
!
!  INPUT VARIABLES:
!      ABSARY  - THE CO2 ABSORPTION ARRAY TO BE INTERPOLATED
!                (EITHER ABS15 OR ABSSOL)
!      W       - THE GAS CONCENTRATION
!      PAVGIN  - THE AVERAGED PRESSURE
!                USED AS PAVG = LOG(PAVGIN)
!
!  OUTPUT VARIABLES:
!      OUTCOEF - COEFFICIENTS OF THE QUADRATIC FIT (OF THE FORM:
!        Y = OUTCOEF(1) + X*OUTCOEF(2) + X*X*OUTCOEF(3) )
!
!  AUTHOR
!      J.R. SCHAEFFER    INFORMATICS    TASK 904    JULY 86
!
!  FOR
!      JIM POLLACK     ( SEE NOTES OF 8-8-86 AND 8-28-86 )
!
!  ENVIRONMENT
!      Cray Y-MP         UNICOS 4.0     FORTRAN 77
!
!  REVISION HISTORY
!       IMSL routines ICSCCU and ICSEVU were replaced.  The new
!       IMSL library renamed all routines.
!       SPLINE and SPEVAL replace IMSL routines.
!
!  CALLED BY:
!      EMISS
!
!  ROUTINES CALLED:
!      SPLINE, SPEVAL

real(f4), dimension(DIM_CO2_data_pr) :: eln, yout
real(f4), dimension(3) :: ewp
real(f4) :: pavg,ans,wrln1,wrln2,tmp21,tmp31,tmp32
integer :: nlpres,nstar,nstar1,j,m

!=== Executable statements:

nlpres = DIM_CO2_data_pr-1

!*** If CO2 gas concentrations are unrealistically low (or zero), set
!     coefficients to zero and return:

if(w < 1.e10_f4) then
   outcoef(1) = ZERO_f4
   outcoef(2) = ZERO_f4
   outcoef(3) = ZERO_f4
   return
endif

!*** Convert input pressure to natural log of pressure:

pavg = log(pavgin)

!*** CO2 gas concentration index:

if(w > wr(2)) then
   nstar = min(jsrchgt(ncon-1,wr,1,w)-1,ncon-1)
else
   nstar = 1
endif
nstar1 = nstar+1

wrln1 = log(w/wr(nstar))
wrln2 = log(wr(nstar1)/wr(nstar))

do j=1,DIM_CO2_data_tm
   do m=1,DIM_CO2_data_pr

!*** Linear (in log space) intrpolation between CO2 gas concentrations at
!     indices nstar and nstar+1:

      eln(m) = log(absary(m,nstar,j))+(wrln1*  &
           log(absary(m,nstar1,j)/absary(m,nstar,j))/wrln2)
   enddo

   if(pavg > prln(1)) then

      !*** Cubic spline fit in pressure direction:

      call calc_spline(real(prln, kind=f4),eln,nlpres,yout)
      call speval4(prln,eln,yout,nlpres,pavg,ans,1)

   else

      !*** Pressure is lower than prln(1); linearly interpolate from lowest two
      !     pressure points:

      ans = eln(1)+(pavg-prln(1))*(eln(2)-eln(1))/(prln(2)-prln(1))

   endif

   !*** Interpolated value at each temperature value:

   ewp(j) = exp(ans)

enddo

!*** Determine the quadratic-fit coefficients (T is independent variable):

tmp21 = tr(2)-tr(1)
tmp32 = tr(3)-tr(2)
tmp31 = tr(3)-tr(1)

outcoef(3) = ewp(1)/(tmp31*tmp21)-ewp(2)/(tmp32*tmp21)+ewp(3)/(tmp32*tmp31)
outcoef(2) = (ewp(2)-ewp(1))/tmp21-outcoef(3)*(tr(2)+tr(1))
outcoef(1) = ewp(1)-outcoef(2)*tr(1)-outcoef(3)*tr(1)*tr(1)

end subroutine co2fit4


end module radn_CO2fit_mod
