!```````````````````````````````````````````````````````````````````
module radn_IRin_mod

!~ 22.09.2008 (last modified; TIM)

!+

implicit none

private

public :: radn_IRin

contains


!```````````````````````````````````````````````````````````````````
subroutine radn_IRin(flxc,LB_end,LM_end)

!~ 19.11.2008 (last modified; TIM)

!+ Calculates up/down IR flux in the 15um CO2 band

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use radn_main_mem_mod, only: tg,pl,tl2,ir15dn,ir15up,ir15net,ir15
use radn_CO2ea_mem_mod, only: cem,abs15,CO2_emiss_pr_incr,DIM_CO2_data_pr  &
     ,DIM_CO2_emiss_pr,lw_CO2_emiss,DIM_CO2_data_tm,DIM_CO2_data_cn,tr,wr  &
     ,prln,ncon
use radn_dust_mem_mod, only: emisdst,tdincr,mdst,nmdst,emd,e15dst,DUST_taucum  &
     ,mmaxed1,tdmin
use radn_BBf_mem_mod, only: bbg,bbplck,bbp,rdelnu,eg15gnd,mtmax
use radn_CO2fit_mod, only: co2fit4

implicit none

!=== Argument declarations:

integer, intent(IN) :: LB_end,LM_end
real(f4), intent(IN) :: flxc

!== Local declarations:

!  PURPOSE
!      IN15IR CALCULATES FOR A GIVEN 'PI' GRID POINT THE UPWARD AND
!      DOWNWARD FLUXES OF 15 MICRON (INSIDE THE CO2 ABSORPTION BAND)
!      INFRARED RADIATION AT EACH ATMOSPHERIC LAYER BOUNDARY. THE IR NET
!      FLUXES AND IR ABSORPTION FOR THIS BAND ARE ALSO CALCULATED.
!      THE EXCHANGE TERMS IN THE TRANSFER EQUATION ARE EVALUATED BY
!      IMPOSING A FINE-MESH STRUCTURE OF SEVEN LEVELS ON ADJACENT
!      LAYERS (SINCE THE BULK OF THE EXCHANGE COMES FROM THEM) TO
!      OBTAIN TEMPERATURE PROFILES THAT MORE PRECISELY ESTIMATE
!      STATIC STABILITY, WHICH IN TURN AFFECTS CONVECTION, INTENSITY
!      OF MEAN MERIDIONAL CIRCULATION, AND GROWTH RATES OF
!      BAROCLINICALLY UNSTABLE WAVES.
!  AUTHOR
!      STEVE POHORSKY    INFORMATICS     TASK 605    JUL 82
!
!  FOR
!      JIM POLLACK      (SEE NOTE OF 5-17-82)
!
!  ENVIRONMENT
!      Cray-2            UNICOS 3.0      FORTRAN
!
!  REVISION HISTORY
!      7-82  SP  NAME OF THIS SUBROUTINE CHANGED FROM CO2IR TO IN15IR.
!       OCT 84  JB WHITE  TASK 904  FINE-MESH STRUCTURE INCORPORATED
!                                   (SEE NOTE OF 10 SEPTEMBER 1984)
!      JR SCHAEFFER     TASK 904           DEC 86
!      MODIFICATIONS TO CO2 LINE-BY-LINE EMISSIVITY CALCULATIONS
!                                             (SEE NOTE 9/30/86)
!      JR SCHAEFFER        TASK 904          JAN 87
!      TAYLOR SERIES APPROXIMATION TO LOG(PRESSURE) INCLUDED
!                             (SEE NOTE OF 1/13/87)
!      JR SCHAEFFER        TASK 904          FEB 87
!      INTERPOLATION TO GET BOUNDARY LAYER TEMPERATURES
!                                       (SEE NOTE OF 1/30/87)
!      JR SCHAEFFER        TASK 904          FEB 87
!      INCORPORATED VARIABLE DUST  ( SEE NOTE OF 1/19/87 ).
!
!  INPUT PARAMETERS
!      JCMN & ICMN          - THE J AND I COORDINATES OF THE 'PI' POINT.
!      TG                   - GROUND TEMPERATURE.
!      GT(JCMN,ICMN)        - GROUND TEMPERATURE OR (-1) TIMES CO2 ICE
!                             MASS AT THE 'PI' POINT. (SEE TEMPGR.)
!      TAUCUM(K) ARRAY      - CUMULATIVE DUST OPTICAL DEPTH FROM TOP OF
!                             ATMOSPHERE TO LEVEL K. (K = 1
!                             TO NLEVELS BY 2S.)
!      TAUICET(K) ARRAY     - CUMULATIVE ICE OPTICAL DEPTH FROM TOP OF
!                             ATMOSPHERE TO LEVEL K. (K = 1
!                             TO NLEVELS BY 2S.)
!      TL(K) ARRAY          - TEMPERATURE AT EACH LEVEL FOR THE 'PI'
!                             POINT. (K = 2 TO NLEVSM1 BY 2S.)
!      PL(NLEVELS)          - SURFACE PRESSURE FOR THE 'PI' POINT.
!     ARRAY ELEMENT
!
!       FMEM(K,L,2,MP) ARRAY  EMISSIVITY BETWEEN LAYER K AND FINE-MESH
!                             LEVEL L FOR DOWN AND UP FLUXES AND WITH
!                             RESPECT TO SURFACE PRESSURE DEFINED BY MP;
!                             COMPUTED IN EMISS AND PASSED THROUGH
!                             COMMON /FMESH0/ .
!
!  OUTPUT PARAMETERS
!      IR15DN(K) ARRAY    - THE DOWNWARD FLUX OF 15-MICRON (INSIDE THE
!                           CO2 ABSORPTION BAND) INFRARED RADIATION AT
!                           EACH ATMOSPHERIC LAYER BOUNDARY EXCEPT THE
!                           TROPOPAUSE. ( K = 1,5,7,...NLEVELS.)
!      IR15UP(K) ARRAY    - THE UPWARD FLUX OF 15-MICRON (INSIDE THE
!                           CO2 ABSORPTION BAND) INFRARED RADIATION AT
!                           EACH ATMOSPHERIC LAYER BOUNDARY EXCEPT THE
!                           TROPOPAUSE. ( K = 1,5,7,...NLEVELS.)
!      IR15NET(K) ARRAY   - THE NET FLUX OF 15-MICRON (INSIDE THE
!                           CO2 ABSORPTION BAND) INFRARED RADIATION AT
!                           EACH ATMOSPHERIC LAYER BOUNDARY EXCEPT THE
!                           TROPOPAUSE. ( K = 1,5,7,...NLEVELS.)
!                           THE CONVENTION USED HERE IS  IR15NET > 0 IF
!                           THE NET FLUX IS IN AN UPWARD DIRECTION.
!      IR15(K) ARRAY      - IR15(K) IS THE AMOUNT OF 15-MICRON
!                           (INSIDE THE CO2 ABSORPTION BAND) INFRARED
!                           ENERGY ABSORBED IN THE ATMOSPHERIC LAYER
!                           WHOSE BOTTOM IS AT LEVEL K.
!                           ( K = 5 TO NLEVELS BY 2S.)
!      BBG                - THE 15-MICRON BLACK BODY PLANCK FUNCTION
!                           VALUE AT THE TEMPERATURE TG.
!      BBP(K) ARRAY       - BBP(K) = THE 15-MICRON BLACK BODY PLANCK
!                           FUNCTION VALUE AT THE TEMPERATURE TL(K).
!                           ( K = 2 TO NLEVSM1 BY 2S.)
!      EMG15              - GROUND EMISSIVITY INSIDE 15 MICRON BAND.
!
!        BFM(K,L.2) ARRAY    FINE-MESH PLANCK FUNCTIONS, AND DUST AND IC
!       TDFM(K,L,2)   '      OPTICAL DEPTHS COMPUTED AND USED HERE AND
!       TIFM(K,L,2)   '      PASSED TO OUT15IR THROUGH COMMON /FMESH/ TO
!                            AVOID RECOMPUTING THEM THERE.
!
!  CALLED BY
!      COMP3

integer, external :: jsrchgt

!real(f4), dimension(3,DIM_mubar,0:DIM_CO2_emiss_pr) :: sw_CO2_emiss
!real(f4), dimension(3,0:DIM_CO2_emiss_pr,0:DIM_CO2_emiss_pr) :: lw_CO2_emiss
!real(f4), dimension(0:DIM_CO2_emiss_pr) :: CO2_emiss_pr
!common /CO2_emissivity/ sw_CO2_emiss,lw_CO2_emiss,CO2_emiss_pr

real(f4), dimension(3) :: emco
real(f4), dimension(LB_end,LB_end) :: tbi,emisco2,e15comb
real(f4), dimension(LB_end) :: pints,pints1
integer, dimension(LB_end) :: iptmp,iptmp1

integer :: mt,ik,ik2,ib,m,nm,ic,ibp,itmp,itmp1
real(f4) :: taudkkp,tdtable,tsum,psum,fsum,ptmp,tint,ttmp,qtmp1,qtmp2  &
     ,etmp,etmp1,w,pavg,emg15,qirfact

!=== Executable statements:

!*** Interpolate in temperature to obtain the black body Planck function
!      values:

!*** At the ground:

mt = int(tg-49._f4)        ! Blackbody table starts at 50K

if(mt < 1) then       !
   mt = 1             ! If `mt` exceeds blackbody table limits,
endif                 !   set to upper or lower limit
if(mt > mtmax) then   !
   mt = mtmax         ! 
endif                 !

tint = tg-real(mt+49)     ! Interpolative fraction used in below interpolation

!*** Surface BB Planck function value * (pi*deltanu):

bbg = flxc*((ONE_f4-tint)*bbplck(mt)+tint*bbplck(mt+1))

!*** and at all OL:

do ik=1,LM_end    ! Loop over all OL
   mt = int(tl2(ik)-49._f4)            ! Blackbody table starts at 50K

   if(mt < 1) then       !
      mt = 1             ! If `mt` exceeds blackbody table limits,
   endif                 !   set to upper or lower limit
   if(mt > mtmax) then   !
      mt = mtmax         ! 
   endif                 !

   tint = tl2(ik)-real(mt+49)       ! Interpolative fraction

   ! Layer BB Planck f. value * (pi*deltanu):

   bbp(ik) = flxc*((ONE_f4-tint)*bbplck(mt)+tint*bbplck(mt+1))  

enddo

! Set ground emissivity value:
! NOTE: (executes faster than if `eg15gnd` is used directly):

emg15 = eg15gnd

! Compute dust and CO2 emissivities between atmospheric levels:

! Generate first part of emissivity matrices -- diagonal elements are zero:
!    (*NEEDED* for correct execution of flux sum loop)

do ik=1,LB_end          ! Loop over ALL CL
   e15comb(ik,ik) = ZERO_f4
   emisdst(ik,ik) = ZERO_f4
enddo

! Generate second and final portion of emissivity matrices:

qirfact = 0.5_f4  ! Factor used to mute the IR response of dust (to account for
               !   2:1 solar vs. IR dust response that was not accounted for 
               !   in the creation of the coefficient tables)

! Dust emissivity matrix:

ttmp = epsilon(ttmp)   ! Smallest value that can be subtracted/added

do ik=2,LB_end     ! Loop over CL
   do ib=1,ik-1     ! Loop from top CL to level above current `ik` CL

      ! Dust optical depth between levels:

      taudkkp = abs(DUST_taucum(ik)-DUST_taucum(ib))*qirfact
    
      if(taudkkp >= ttmp) then           ! do if atmosphere is dusty (enough)
         if(taudkkp < tdmin(2)) then       !
            m = 1                          !
         else                              ! Set index appropriately
            m = 2                          !
         endif                             !

         nm = 1+int((taudkkp-tdmin(m))/tdincr(m))  ! Second index

         if(nm > mmaxed1) then     ! If index is greater than limit, act
            nm = mmaxed1            ! appropriately
            taudkkp = tdmin(m)+tdincr(m)*(mmaxed1-1)
         endif

         tdtable = tdmin(m)+tdincr(m)*(nm-1)
         mdst(ik,ib) = m         !  Store indices for radn_IRout
         nmdst(ik,ib) = nm       !
         emd(ik,ib) = (taudkkp-tdtable)/tdincr(m)   ! Store for radn_IRout
         emisdst(ik,ib) = e15dst(m,nm)+(e15dst(m,nm+1)-  &   ! Dust emissivity
              e15dst(m,nm))*emd(ik,ib)
      else    ! If this atm layer is effectively clear
         m = 1
         nm = 1                 ! Assume that we already know that tdmin(1)=0.
         tdtable = ZERO_f4           ! 
         mdst(ik,ib) = m          !  Store indices for radn_IRout
         nmdst(ik,ib) = nm        !
         emd(ik,ib) = ZERO_f4       ! Store for radn_IRout
         emisdst(ik,ib) = e15dst(m,nm)            ! Dust emissivity
      endif

      emisdst(ib,ik) = emisdst(ik,ib)     ! Make matrix symmetric

   enddo
enddo

! CO2 emissivity matrix:

! Calculate pressure-weighted temperature averages:

tbi(1,1) = ZERO_f4        ! Set first element to zero

do ik=2,LB_end       ! Loop over CL
   ik2 = ik-1
   do ib=1,ik2    ! Loop from top CL to CL above current `ik` CL
      tsum = ZERO_f4          !
      psum = ZERO_f4          ! Initialize summation variables

      do ic=ib,ik2     ! Loop from `ib` CL to the CL above current `ik` CL
         tsum = tsum+tl2(ic)*(pl(ic+1)-pl(ic))
         psum = psum+(pl(ic+1)-pl(ic))
      enddo
      
      tbi(ik,ib) = tsum/psum            ! Pressure-weighted temp average
   enddo
enddo

! Calculate indices and interpolative fractions for CO2 emissivity calcs:

do ik=1,LB_end    ! Loop over all CL
   ptmp = pl(ik)/CO2_emiss_pr_incr  ! Find stock press. closest to curr. press
   iptmp(ik) = int(ptmp)        ! Closest pressure index in stock emiss matrix
   iptmp1(ik) = iptmp(ik)+1
   pints(ik) = ptmp-real(iptmp(ik))    ! Interpolative fraction for above index
   pints1(ik) = ONE_f4-pints(ik)
enddo

! Down flux:

do ik=2,LB_end     ! Loop over CL

   itmp = iptmp(ik)     ! Set constant indices 
   etmp = pints(ik)     !
   itmp1 = iptmp1(ik)    !
   etmp1 = pints1(ik)   !
   
   do ib=1,ik-1     ! Loop from top CL to CL above current `ik` CL

      if((itmp-iptmp(ib)) <= 1 .or. itmp1 > DIM_CO2_emiss_pr) then 

         w = cem*abs(pl(ik)-pl(ib))               
         pavg = 0.5_f4*(pl(ik)+pl(ib))
         call co2fit4(abs15,w,pavg,emco,DIM_CO2_data_pr,DIM_CO2_data_cn  &
              ,DIM_CO2_data_tm,wr,tr,real(prln, kind=f4),ncon)   ! Use CO2FIT if dp too small

      else
         
         do ic=1,3      ! Otherwise, interpolate CO2 emissivity coefficients:

            !@@@ Equivalent to below?
            !?            qtmp1 = etmp1*lw_CO2_emiss(ic,itmp,iptmp(ib))+  &
            !?                 etmp*lw_CO2_emiss(ic,itmp1,iptmp(ib))
            !?            qtmp2 = etmp1*lw_CO2_emiss(ic,itmp,iptmp1(ib))+  &
            !?                 etmp*lw_CO2_emiss(ic,itmp1,iptmp1(ib))
     
            qtmp1 = etmp1*lw_CO2_emiss(ic,iptmp(ib),itmp)+  &
                 etmp*lw_CO2_emiss(ic,iptmp(ib),itmp1)
            qtmp2 = etmp1*lw_CO2_emiss(ic,iptmp1(ib),itmp)+  &
                 etmp*lw_CO2_emiss(ic,iptmp1(ib),itmp1)

            emco(ic) = pints1(ib)*qtmp1+pints(ib)*qtmp2
               
         enddo
      endif

      ttmp = tbi(ik,ib)           ! Quadratic fit for CO2 emissivity
      emisco2(ik,ib) = rdelnu*(emco(1)+emco(2)*ttmp+emco(3)*ttmp*ttmp)

   enddo
enddo

! Up flux:

do ik=1,LB_end      ! Loop over all CL
   do ib=ik+1,LB_end     ! Loop from CL below `ik' to last CL 
      emisco2(ik,ib) = emisco2(ib,ik)   ! Make CO2 emissivity matrix symmetric
   enddo
enddo

do ik=2,LM_end
   do ib=1,LB_end-ik   ! Loop over CL from top to the 'ik' CL from surface

      ibp = ib+ik

      if(emisco2(ib,ibp) < emisco2(ib,ibp-1)) then  ! Check for what?????
         emisco2(ib,ibp) = emisco2(ib,ibp-1)        !
      endif                                         !
      if(emisco2(ib,ibp) < emisco2(ib+1,ibp)) then  !
         emisco2(ib,ibp) = emisco2(ib+1,ibp)        !
      endif

      emisco2(ibp,ib) = emisco2(ib,ibp)     ! Provide for symmetry
   enddo
enddo

! Compute the combined 15um emissivity:

! Down flux:

do ik=2,LB_end    ! Loop over CL
   do ib=1,ik-1      ! Loop from top CL to CL above current `ik` CL 
      e15comb(ik,ib) = ONE_f4-(ONE_f4-emisdst(ik,ib))*(ONE_f4-emisco2(ik,ib))
   enddo
enddo

! Up flux:

do ik=1,LB_end-1    ! Loop over CL
   do ib=ik+1,LB_end      ! Loop from CL below `ik' to last CL
      e15comb(ik,ib) = e15comb(ib,ik)     ! Symmetric matrix
   enddo
enddo

! EVALUATE 15-MICRON FLUXES AT CL:

! Downward: (starting at atmosphere top and proceeding downward)

ir15dn(1) = ZERO_f4       ! Zero flux at top of atmosphere

do ik=2,LB_end       ! Loop over CL

! Accumulate partial fluxes into flux sum:

   fsum = ZERO_f4       ! Initialize flux sum

   do ib=1,ik-1
      fsum = fsum+(bbp(ib))*abs(e15comb(ik,ib+1)-e15comb(ik,ib))
   enddo

! Downward flux at level `ik`:

   ir15dn(ik) = fsum
enddo

! Upward: (starting at the surface and proceeding upward)

! Flux at surface is from ground and reflection by the ground:

ir15up(LB_end) = emg15*bbg+(ONE_f4-emg15)*ir15dn(LB_end)

do ik=LB_end-1,1,-1     ! Loop from first CL above surface to top

! Accumulate partial fluxes into flux sum:

   fsum = ZERO_f4       ! Initialize flux sum
   do ib=LM_end,ik,-1
      fsum = fsum+(bbp(ib))*abs(e15comb(ik,ib+1)-e15comb(ik,ib))
   enddo

! Upward flux at level `ik`:
! (The atmospheric flux sum + the transmitted portion of ground IR flux)
   
   ir15up(ik) = fsum+(ONE_f4-e15comb(ik,LB_end))*ir15up(LB_end)

enddo

! IR NET FLUX AND ABSORPTION INSIDE 15-MICRON BAND:

ir15net(1) = ir15up(1)          ! =IR15UP(1)-IR15DN(1)

do ik=2,LB_end
   ir15net(ik) = ir15up(ik)-ir15dn(ik)       ! Net IR 15um flux
enddo

do ik=2,LB_end
   ir15(ik-1) = ir15net(ik)-ir15net(ik-1)  ! Net flux divergence of 15um IR
                                           !  (ir15 is filled on OL)
enddo

end subroutine radn_IRin


!```````````````````````````````````````````````````````````````````
function limits(arg,offset,ilow,iup)

!~ 19.11.2008 (last modified; TIM)

!+

use univ_kind_defs_mod, only: f4

!=== Argument declarations:

integer :: limits    ! Procedure type

real(f4), intent(IN) :: arg,offset
integer, intent(IN) :: ilow,iup

limits = min(max(int(arg-offset),ilow),iup)

end function limits


end module radn_IRin_mod
