!```````````````````````````````````````````````````````````````````
module radn_IRout_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine radn_IRout(LB_end, LM_end)

!+ Calculates up/down IR flux outside the 15um CO2 band

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_phys_constants_mod, only: stfnbltz_f4
use radn_main_mem_mod, only: iroutdn,tl2,iroutup,ironet,tg
use radn_BBf_mem_mod, only: egognd,bbp,bbg
use radn_dust_mem_mod, only: emisdst,mdst,nmdst,eodtab,emd

implicit none

!=== Argument declarations:

integer, intent(IN) :: LB_end, LM_end

!== Local declarations:

!  PURPOSE
!      OUT15IR CALCULATES FOR A GIVEN 'PI' GRID POINT THE UPWARD AND
!      DOWNWARD FLUXES OF NON-15 MICRON (OUTSIDE OF CO2 ABSORPTION BAND)
!      INFRARED RADIATION AT EACH ATMOSPHERIC LAYER BOUNDARY. THE IR NET
!      FLUXES AND IR ABSORPTION OUTSIDE THIS BAND ARE ALSO CALCULATED.
!      THE FINE-MESH STRUCTURE DESCRIBED FOR IN15IR IS APPLIED AS WELL
!      HERE EVEN THOUGH THE OPTICAL DEPTHS IN THIS SPECTRAL REGION ARE
!      GENERALLY SMALL, BECAUSE LARGE OPTICAL DEPTHS MAY BE ENCOUNTERED
!      DURING TRANSPORT EXPERIMENTS AND TO MAKE THE TREATMENT OF THE
!      INFRARED UNIFORMLY CONSISTENT.
!
!  AUTHOR
!      STEVE POHORSKY    INFORMATICS     TASK 605    MAY 82
!
!  FOR
!      JIM POLLACK      (SEE NOTE OF 3-30-82)
!
!  ENVIRONMENT
!      Cray-2            UNICOS 3.0      FORTRAN
!
!  REVISION HISTORY
!      7-82 SP   NAME OF THIS SUBROUTINE CHANGED FROM DUSTIR TO OUT15IR.
!      7-82 SP   INFRARED ABSORPTION CALCULATION ADDED TO END OF ROUTINE
!      7-82 SP   CALCULATION OF BBG & BBP REMOVED SINCE THEY ARE
!                CALCTULATED IN IN15IR.
!       OCT 84  JB WHITE  TASK 904  FINE-MESH STRUCTURE INCORPORATED
!                                   (SEE NOTE OF 10 SEPTEMBER 1984)
!       JR SCHAEFFER     TASK 904           DEC 86
!       INCORPORATE CO2 LINE-BY-LINE EMISSIVITY (SEE NOTE OF 10/6/86).
!  INPUT PARAMETERS
!      JCMN & ICMN          - THE J AND I COORDINATES OF THE 'PI' POINT.
!      TG                   - GROUND TEMPERATURE.
!      GT(JCMN,ICMN)        - GROUND TEMPERATURE OR (-1) TIMES CO2 ICE
!                             MASS AT THE 'PI' POINT.
!      TAUCUM(K) ARRAY      - CUMULATIVE DUST OPTICAL DEPTH FROM TOP OF
!                             ATMOSPHERE TO LEVEL K. (K = 1
!                             TO NLEVELS BY 2S.)
!      TAUICET(K) ARRAY     - CUMULATIVE ICE OPTICAL DEPTH FROM TOP OF
!                             ATMOSPHERE TO LEVEL K. (K = 1
!                             TO NLEVELS BY 2S.)
!      TL(K) ARRAY          - TEMPERATURE AT EACH LEVEL FOR THE 'PI'
!                             POINT. (K = 2 TO NLEVSM1 BY 2S.)
!      BBG                  - THE 15-MICRON BLACK BODY PLANCK FUNCTION
!                             VALUE AT THE TEMPERATURE TG.
!      BBP(K) ARRAY         - BBP(K) = THE 15-MICRON BLACK BODY PLANCK
!                             FUNCTION VALUE AT THE TEMPERATURE TL(K).
!                             ( K = 2 TO NLEVSM1 BY 2S.)
!
!       BFM(K,L,2) ARRAY  FINE-MESH PLANCK FUNCTIONS, AND DUST AND ICE
!
!  OUTPUT PARAMETERS
!      IROUTDN(K) ARRAY   - THE DOWNWARD FLUX OF NON-15-MICRON (OUTSIDE
!                           CO2 ABSORPTION BAND) INFRARED RADIATION AT
!                           EACH ATMOSPHERIC LAYER BOUNDARY EXCEPT THE
!                           TROPOPAUSE. ( K = 1,5,7,...NLEVELS.)
!      IROUTUP(K) ARRAY   - THE UPWARD FLUX OF NON-15-MICRON (OUTSIDE
!                           CO2 ABSORPTION BAND) INFRARED RADIATION AT
!                           EACH ATMOSPHERIC LAYER BOUNDARY EXCEPT THE
!                           TROPOPAUSE. ( K = 1,5,7,...NLEVELS.)
!      IRONET(K) ARRAY    - THE NET FLUX OF NON-15-MICRON (OUTSIDE
!                           CO2 ABSORPTION BAND) INFRARED RADIATION AT
!                           EACH ATMOSPHERIC LAYER BOUNDARY EXCEPT THE
!                           TROPOPAUSE. ( K = 1,5,7,...NLEVELS.)
!                           THE CONVENTION USED HERE IS  IRONET > 0 IF
!                           THE NET FLUX IS IN AN UPWARD DIRECTION.
!      IROUT(K) ARRAY     - IROUT(K) IS THE AMOUNT OF NON-15-MICRON
!                           (OUTSIDE CO2 ABSORPTION BAND) INFRARED
!                           ENERGY ABSORBED IN THE ATMOSPHERIC LAYER
!                           WHOSE BOTTOM IS AT LEVEL K.
!                           ( K = 5 TO NLEVELS BY 2S.)
!      EMGOUT             - GROUND EMISSIVITY OUTSIDE 15 MICRON BAND.
!
!  CALLED BY
!      COMP3

integer :: ik,ib,m,nm
real(f4) :: fsum,ttmp,emgout
real(f4), dimension(LB_end,LB_end) :: eoutcom

!=== Executable statements:

! Compute dust emissivities between atmospheric levels:

emgout = egognd             ! Emissivity of ground

! Generate first part of emissivity array; diagonal elements are zero:
!    (*NEEDED* for correct execution of flux sum loop)

do ik=1,LB_end
   eoutcom(ik,ik) = ZERO_f4
enddo

! Generate second part of dust emissivity array:

if(emisdst(LB_end,LB_end-1) > ZERO_f4) then    ! If atmosphere is dusty, then:

   do ik=2,LB_end      ! Loop over CL
      do ib=1,ik-1
         m = mdst(ik,ib)
         nm = nmdst(ik,ib)
         emisdst(ik,ib) = eodtab(m,nm)+(eodtab(m,nm+1)-  &
              eodtab(m,nm))*emd(ik,ib)     ! Dust optical depth between layers
         eoutcom(ik,ib) = ONE_f4-(ONE_f4-emisdst(ik,ib))   ! Combined dust emissivity
      enddo
   enddo

else                               ! If atmosphere is clear, then:

   do ik=2,LB_end    ! Loop over CL
      do ib=1,ik-1
         eoutcom(ik,ib) = ZERO_f4        ! Compute combined dust emissivity
      enddo
   enddo

endif

do ik=1,LB_end-1      ! Loop over CL
   do ib=ik+1,LB_end
      eoutcom(ik,ib) = eoutcom(ib,ik)      ! Make array symmetric
   enddo
enddo

! EVALUATE NON-15-MICRON FLUXES AT CL:

! Downward: (starting at atmosphere top and proceeding downward)

iroutdn(1) = ZERO_f4      ! Zero flux at top of atmosphere

do ik=2,LB_end      ! Loop over CL
 
! Accumulate partial fluxes into flux sum: 

   fsum = ZERO_f4       ! Initialize flux sum
   do ib=1,ik-1
      ttmp = tl2(ib)
      fsum = fsum+(stfnbltz_f4*ttmp*ttmp*ttmp*ttmp-bbp(ib))*  &
           (eoutcom(ik,ib)-eoutcom(ik,ib+1))
   enddo

! Downward flux at level `ik`:

   iroutdn(ik) = fsum
enddo

! Upward: (starting at the surface and proceeding upward)

! Flux at surface is from ground and reflection by the ground:

iroutup(LB_end) = emgout*(stfnbltz_f4*tg*tg*tg*tg-bbg)+  &
     (ONE_f4-emgout)*iroutdn(LB_end)

do ik=LB_end-1,1,-1     ! Loop from first CL above surface to top

   ! Accumulate partial fluxes into flux sum:
   
   fsum = ZERO_f4       ! Initialize flux sum
   
   do ib=LM_end,ik,-1
      ttmp = tl2(ib)
      fsum = fsum+(stfnbltz_f4*ttmp*ttmp*ttmp*ttmp-bbp(ib))*  &
           (eoutcom(ik,ib+1)-eoutcom(ik,ib))
   enddo

! Upward flux at level `ik`:
! (The atmospheric flux sum + the transmitted portion of ground IR flux)
 
   iroutup(ik) = fsum+(ONE_f4-eoutcom(ik,LB_end))*iroutup(LB_end)

enddo

! IR NET FLUXES AND ABSORPTION OUTSIDE 15-MICRON BAND:

ironet(1) = iroutup(1)      ! =IROUTUP(1)-IROUTDN(1)

do ik=2,LB_end
   ironet(ik) = iroutup(ik)-iroutdn(ik)    ! Net IR non-15um flux 
enddo

!%do ik=2,LB_end
!%   irout(ik-1) = ironet(ik)-ironet(ik-1) ! Net flux divergence of non-15um IR
!%                                          !  (irout filled on OL)
!%enddo

end subroutine radn_IRout


end module radn_IRout_mod
