!```````````````````````````````````````````````````````````````````
module radn_gsflux_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine getflux(u0val,albval,tautval,taucval,fluxin,fluxval,LB_end)

!+ Adjusts net solar flux for radiative absorption by dust

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use radn_netflux_mem_mod, only: mmaxtau,mmaxtot,mmaxu0,albedos,tautots  &
     ,taucum0,u0vals
use search_mod, only: jsrchge
use spline_mod, only: calc_spline

implicit none

!=== Argument declarations:

integer, intent(IN) :: LB_end
real(f4), intent(IN) :: albval,tautval,fluxin,u0val
real(f4), dimension(LB_end), intent(IN) :: taucval

real(f4), dimension(LB_end), intent(INOUT) :: fluxval

!== Local declarations:
      
!  PURPOSE
!      GETFLUX CALCULATES THE NET FLUX OF SOLAR ENERGY AT THE
!      BOTTOM OF THE SPECIFIED LAYERS ( LEVELS KSTART TO KEND IN
!      INCREMENTS OF 2 ).
!
!  AUTHOR
!      STEVE POHORSKY    INFORMATICS     TASK 605    OCT 81
!
!  FOR
!      JIM POLLACK     ( SEE NOTES OF 8-20-81, 3-8-82 )
!
!  ENVIRONMENT
!      Cray Y-MP         UNICOS 4.0      FORTRAN 77
!
!  REVISION HISTORY
!     1-82  SP   FLUXIN PARAMETER ADDED. (SEE UNDER DUSTSUN.)
!     3-82  SP   CORRECTED INDICES FOR TAUTVAL INTERPOLATION.
!     5-82  SP   CHANGED TO ALLOW THE PROPER TABLE ENTRIES TO BE USED IN
!                THE TAUTOT EXTRAPOLATION IN GETFLUX IN THE CASE OF
!                TAUTVAL = TAUTOTS(M) FOR SOME M. (SEE GETINDX.)
!        JR SCHAEFFER     INFORMATICS     TASK 904     JUNE 1986
!        CODE REWRITE - SEE NOTES OF MAY 2, 1986 AND MAY 26,1986.
!
!  INPUT PARAMETERS
!             1) ALBVAL - SURFACE ALBEDO
!             2) U0VAL - COSINE OF SOLAR ZENITH
!             3) TAUTVAL - TOTAL OPTICAL DEPTH TO SURFACE
!             4) KSTART & KEND - STARTING & ENDING LEVEL INDICES OF THE
!                                RANGE OF LEVELS AT WHICH THE NET FLUX
!                                IS TO BE CALCULATED. BY OUR INDEXING
!                                CONVENTION, NET FLUX AND CUMULATIVE
!                                OPTICAL DEPTH ARE DEFINED ONLY FOR ODD
!                                LEVEL INDICES ( BOTTOMS OF LAYERS ).
!                                3 <= KSTART <= KEND <= NLEVELS.
!             5) TAUCVAL(K) ARRAY ELEMENTS - CUMULATIVE OPTICAL DEPTH
!                          FROM TOP OF ATMOSPHERE TO LEVEL K, FOR THE
!                          SPECIFIED LEVELS.
!             6) FLUXIN  - THE INCIDENT SOLAR FLUX AT THE TROPOPAUSE
!                          (LEVEL 3). ( THE VALUES IN THE NET FLUX
!                          TABLE ARE BASED ON A UNIT INCIDENT FLUX
!                          AT THE TROPOPAUSE. WE NEED TO MULTIPLY THE
!                          RESULTS FROM THE NET FLUX TABLE BY
!                          FLUXIN TO GET THE CORRECT NET FLUX
!                          QUANTITIES.
!
!  OUTPUT PARAMETERS
!      FLUXVAL(K) ARRAY ELEMENTS - ( K = KSTART TO KEND BY TWOS.) NET
!                            SOLAR FLUX AT THE SPECIFIED LEVELS.
!
!  CALLED BY
!      DUSTSUN, TEMPGR
!  SUBROUTINES CALLED
!        SPLINE
!
!     NET FLUX IS A FUNCTION
!     OF FOUR VARIABLES:        1) SURFACE ALBEDO - ALBVAL
!                               2) COSINE OF SOLAR ZENITH - U0VAL
!                               3) CUMULATIVE OPTICAL DEPTH FROM TOP
!                                  OF ATMOSPHERE TO LEVEL K - TAUCVAL
!                               4) TOTAL OPTICAL DEPTH TO SURFACE -
!                                  TAUTVAL
!
!     THE TABLE OF NET FLUXES GIVES THE NET FLUX FOR A FOUR-DIMENSIONAL
!     GRID OF POINTS IN THE ABOVE FOUR VARIABLES. THE GRID IS NOT
!     ENTIRELY THE TYPICAL RECTANGULAR GRID BECAUSE OF THE STRONG
!     CONNECTION BETWEEN THE TAUCUM (TAUCVAL) AND TAUTOT (TAUTVAL)
!     VARIABLES. THE GRID IS FULLY RECTANGULAR IN THREE OF THE
!     VARIABLES, TAUTOT, U0VAL, AND ALBVAL. THE SET OF TAUCUM
!     COORDINATES IS NOT CONSTANT THROUGHOUT THE GRID THOUGH.  SINCE
!     TAUCUM ( BY PHYSICS ) CAN NEVER EXCEED TAUTOT, THE TAUCUM VALUES
!     ARE CHOSEN (SCALED) AS A FUNCTION OF TAUTOT. FOR A GIVEN TAUTOT
!     VALUE, THE TAUCUM VALUES ARE WELL SPACED IN THE INTERVAL FROM 0
!     TO TAUTOT.  THE NET FLUX TABLE IS REFERENCED VIA THE NETFLUX
!     FUNCTION WHICH TRANSFORMS THE INTERNAL THREE-DIMENSIONAL
!     REPRESENTATION OF THE NET FLUX DATA INTO THE FOUR-DIMENSIONAL
!     LOGICAL REPRESENTATION.

real(f4), dimension(mmaxtau) :: fl, yout
real(f4) :: apart,u0part,tpart,ans,u0t1,amu1,amu2,amu3,amu4
integer :: ik,ma,mu,mt,map1,mup1,mtp1

!=== Executable statements:

!*** Albedo index:

if(albval <= albedos(2)) then
   ma = 1
else
   ma = 2
endif
map1 = ma+1

apart = (albval-albedos(ma))/(albedos(map1)-albedos(ma)) ! Albedo interp frac

!*** MU0 (cosZ) index:

if(u0val > u0vals(2)) then
   mu = min(jsrchge(mmaxu0-1,u0vals,1,u0val)-1,mmaxu0-1)
else
   mu = 1
endif
mup1 = mu+1

u0part = (u0val-u0vals(mu))/(u0vals(mup1)-u0vals(mu))      ! cosZ interp frac

!*** Dusty or not?

if(tautval > 1.e-12_f4) then       ! Dusty atmosphere

   !*** Tautot index (mt):

   if(tautval > tautots(2)) then
      mt = min(jsrchge(mmaxtot-1,tautots,1,tautval)-1,mmaxtot-1) 
   else
      mt = 1
   endif
   mtp1 = mt+1

   tpart = (tautval-tautots(mt))/(tautots(mtp1)-tautots(mt)) ! tau interp frac

   !*** Linearly interpolate (in albedo, mu0, and tautot spaces) each taucum0
   !     grid point, then fit to a cubic spline to obtain the final taucum0
   !     values:

   do ik=1,mmaxtau
 
      !*** Interpolate the albedo value at mu, mt, mu+1, and mt+1
      !     between albedos(ma) and albedos(ma+1):

      amu1 = netflux(ik,mt,mu,ma)+apart*  &
           (netflux(ik,mt,mu,map1)-netflux(ik,mt,mu,ma))
      
      amu2 = netflux(ik,mt,mup1,ma)+apart*  &
           (netflux(ik,mt,mup1,map1)-netflux(ik,mt,mup1,ma))
      
      amu3 = netflux(ik,mtp1,mu,ma)+apart*  &
           (netflux(ik,mtp1,mu,map1)-netflux(ik,mtp1,mu,ma))
      
      amu4 = netflux(ik,mtp1,mup1,ma)+apart*  &
           (netflux(ik,mtp1,mup1,map1)-netflux(ik,mtp1,mup1,ma))

      !*** Interpolate the mu0 value at mt and mt+1
      !     between u0vals(mu) and u0vals(mu+1):

      u0t1 = amu1+(amu2-amu1)*u0part
!@      u0t2 = amu3+(amu4-amu3)*u0part    ! Combined with FL calc below

      !*** Interpolate the tautot value between tautots(mt) and tautots(mt+1):

      fl(ik) = u0t1+((amu3+(amu4-amu3)*u0part)-u0t1)*tpart

   enddo

   !*** Interpolate using cubic splines: 

   call calc_spline(taucum0,fl,mmaxtau,yout)

   !*** Obtain shortwave flux values for each CLevel:

   do ik=2,LB_end

      call speval4(taucum0,fl,yout,mmaxtau,(taucval(ik)/tautval),ans,1)

      fluxval(ik) = ans*fluxin   ! Flux = ans * (net SW flux at atm top)
      
   enddo

else             ! Perfectly clear atmosphere

   !*** For this case, tautot and taucum are known, need to calc the below:

   u0t1 = ONE_f4-albval

   !*** Calculate shortwave flux values for each CLevel:

   do ik=2,LB_end
      fluxval(ik) = u0t1*fluxin
   enddo

endif

end subroutine getflux


!```````````````````````````````````````````````````````````````````
function netflux(ntaucum,ntautot,nu0,nalbedo)

!+

use univ_kind_defs_mod, only: f4
use radn_netflux_mem_mod, only: mmaxtau, fluxnet

implicit none

!=== Argument declarations:

real(f4) :: netflux   ! Procedure type

integer, intent(IN) :: ntaucum,ntautot,nu0,nalbedo

!== Local declarations:

! NONE

!=== Executable statements:

netflux = fluxnet(ntaucum+mmaxtau*(ntautot-1),nu0,nalbedo)

end function netflux


end module radn_gsflux_mod
