!```````````````````````````````````````````````````````````````````
module radn_init2_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine radn_init2

!+ Radiation initialization routine (2 of 2)

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use univ_numtype_constants_mod, only: tiny_f4
use univ_convert_units_mod, only: Pa_to_hPa_f4
use radn_CO2ea_mem_mod, only: CO2_emiss_pr_incr
use radn_dust_mem_mod, only: DIM_dust_levs, DUST_tau_pr, DUST_tau,  &
     DUST_tautot, DUST_taucum, DUST_conrnu
use radn_main_mem_mod, only: DUST_tautot_ref_pr
use main_memory_bspec_mod, only: dust_ctau_refp

implicit none
      
!=== Argument declarations:

! NONE

!== Local declarations:

real(f4), dimension(DIM_DUST_levs) :: DUST_mixratio
real(f4) :: ref_pr_ratio, DUST_mixratio_0, DUST_avg_pr, pdif1, pdif2, dust_sum
integer :: id, id1

!=== Executable statements:

!*** Initialize constant values of cumulative optical depth vector (CLevels):

DUST_taucum(1) = ZERO_f4
DUST_taucum(2) = ZERO_f4

if (DUST_tautot < -tiny_f4) return  ! Don't bother executing any further, but
                                    !  'DUST_tau' also remains uninitialized

!*** CALCULATION OF DUST MIXING RATIO AND CUMULATIVE DUST OPTICAL
!     DEPTH AT A SEQUENCE OF REFERENCE PRESSURE LEVELS:

!MGCM_COMMENT
! Includes Bob Haberle's updates (of 9/17/99) 
! Reference the dust optical depth to RPTAU, and modify the way
! the dust mixing ratio is calculated to more accurately reflect the
! pressure-optical depth relationship.
!MGCM_COMMENT

! Set the dust column opacity reference pressure to a namelist parameter:
DUST_tautot_ref_pr = dust_ctau_refp*Pa_to_hPa_f4     ! [Pa -> hPa]

!*** Calculate geometric pressure stretching ratio 'ref_pr_ratio':

ref_pr_ratio = (5._f4*DUST_tautot_ref_pr/CO2_emiss_pr_incr)**  &
     (ONE_f4/(real(DIM_DUST_levs-1)))

!*** Create reference pressure profile for dust optical depths:

DUST_tau_pr(1) = CO2_emiss_pr_incr  ! Lowest nonzero pressure in `CO2_emiss_pr`
do id=2,DIM_DUST_levs
   DUST_tau_pr(id) = ref_pr_ratio*DUST_tau_pr(id-1)    ! At "layer" boundaries 
enddo

!*** Calculate dust mixing ratio at the reference pressure level:

dust_sum = ZERO_f4
loop01: do id=2,DIM_DUST_levs
   id1 = id-1
   if(DUST_tau_pr(id) < DUST_tautot_ref_pr) then
      DUST_avg_pr = 0.5_f4*(DUST_tau_pr(id1)+DUST_tau_pr(id)) ! At "layer" midpoints
      dust_sum = dust_sum+exp(DUST_conrnu*  &
           (ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*  &
           (DUST_tau_pr(id)-DUST_tau_pr(id1))
   endif
   if(DUST_tau_pr(id) >= DUST_tautot_ref_pr) then
      DUST_avg_pr = 0.5_f4*(DUST_tautot_ref_pr+DUST_tau_pr(id1))
      dust_sum = dust_sum+exp(DUST_conrnu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*  &
           (DUST_tautot_ref_pr-DUST_tau_pr(id1))
      DUST_mixratio_0 = DUST_tautot/dust_sum
      exit loop01
   endif
enddo loop01

! Calculate dust mixing ratio for all other levels:

do id=1,DIM_DUST_levs-1

   ! Region 1: Mixing ratio changes continuously through the layer:

   if(DUST_tautot_ref_pr > DUST_tau_pr(id+1)) then
      DUST_avg_pr = 0.5_f4*(DUST_tau_pr(id+1)+DUST_tau_pr(id))
      DUST_mixratio(id) = DUST_mixratio_0*  &
           exp(DUST_conrnu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))
   endif

   ! Region 2: Reference pressure level within this layer:
   
   if(DUST_tautot_ref_pr <= DUST_tau_pr(id+1) .and.  &
        DUST_tautot_ref_pr >= DUST_tau_pr(id)) then
      DUST_avg_pr = 0.5_f4*(DUST_tau_pr(id)+DUST_tautot_ref_pr)
      pdif1 = DUST_tautot_ref_pr-DUST_tau_pr(id)
      pdif2 = DUST_tau_pr(id+1)-DUST_tautot_ref_pr
      DUST_mixratio(id) = DUST_mixratio_0*  &
           (exp(DUST_conrnu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*pdif1+  &
           pdif2)/(DUST_tau_pr(id+1)-DUST_tau_pr(id))
   endif

   ! Region 3: Mixing ratio constant:

   if (DUST_tautot_ref_pr < DUST_tau_pr(id)) then
      DUST_mixratio(id) = DUST_mixratio_0
   endif

enddo

!*** Calculate dust optical depth at reference pressure levels:

DUST_tau(1) = ZERO_f4
do id=2,DIM_DUST_levs
   id1 = id-1
   DUST_tau(id) = DUST_tau(id1)+DUST_mixratio(id1)*  &
        (DUST_tau_pr(id)-DUST_tau_pr(id1))
enddo

end subroutine radn_init2


!```````````````````````````````````````````````````````````````````
subroutine radn_mvdust_col(LB_end,tautot_ij,conrath_nu)

!+ Calculates/sets dust optical depths for a single column, given parameters

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use radn_main_mem_mod, only: pl, DUST_tautot_ref_pr
use radn_dust_mem_mod, only: DIM_dust_levs,DUST_tau

implicit none
      
!=== Argument declarations:

integer, intent(IN) :: LB_end
real(f4), intent(IN) :: tautot_ij, conrath_nu

!== Local declarations:

real(f4), dimension(DIM_DUST_levs) :: DUST_mixratio
real(f4) :: DUST_mixratio_0,DUST_avg_pr,pdif1,pdif2 &
     ,dust_sum,ptmp_old,ptmp
integer :: id,id1

!=== Executable statements:

!*** CALCULATION OF DUST MIXING RATIO AND CUMULATIVE DUST OPTICAL
!     DEPTH AT A SEQUENCE OF REFERENCE PRESSURE LEVELS:

!MGCM_COMMENT
! Includes Bob Haberle's updates (of 9/17/99) 
! Reference the dust optical depth to RPTAU, and modify the way
! the dust mixing ratio is calculated to more accurately reflect the
! pressure-optical depth relationship.
!MGCM_COMMENT

!*** Calculate dust mixing ratio at the reference pressure level:

dust_sum = ZERO_f4
loop01: do id=2,LB_end
   id1 = id-1
   if(pl(id) < DUST_tautot_ref_pr) then
      DUST_avg_pr = 0.5_f4*(pl(id1)+pl(id)) ! At "layer" midpoints
      dust_sum = dust_sum+exp(conrath_nu*  &
           (ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*  &
           (pl(id)-pl(id1))
   endif

   if(pl(id) >= DUST_tautot_ref_pr) then
      DUST_avg_pr = 0.5_f4*(DUST_tautot_ref_pr+pl(id1))
      dust_sum = dust_sum+  &
           exp(conrath_nu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*  &
           (DUST_tautot_ref_pr-pl(id1))
      DUST_mixratio_0 = tautot_ij/dust_sum  
      exit loop01
   endif

   if(id == LB_end) then   ! If sfc_press < 'DUST_tautot_ref_pr'; make up atm
      ptmp_old = pl(id)
      ptmp = ptmp_old+(pl(id)-pl(id1))
      do while(.TRUE.)
         if(ptmp < DUST_tautot_ref_pr) then
            DUST_avg_pr = 0.5_f4*(ptmp_old+ptmp) ! At "layer" midpoints
            dust_sum = dust_sum+exp(conrath_nu*  &
                 (ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*  &
                 (ptmp-ptmp_old)
            ptmp_old = ptmp
            ptmp = ptmp_old+(pl(id)-pl(id1))
         endif
         if(ptmp >= DUST_tautot_ref_pr) then  ! Time to wrap things up
            DUST_avg_pr = 0.5_f4*(DUST_tautot_ref_pr+ptmp_old)
            dust_sum = dust_sum+  &
                 exp(conrath_nu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*  &
                 (DUST_tautot_ref_pr-ptmp_old)
            DUST_mixratio_0 = tautot_ij/dust_sum
            exit loop01
         endif
      enddo
   endif

enddo loop01

! Calculate dust mixing ratio for all other levels:

do id=1,LB_end-1

   ! Region 1: Mixing ratio changes continuously through the layer:

   if(DUST_tautot_ref_pr > pl(id+1)) then
      DUST_avg_pr = 0.5_f4*(pl(id+1)+pl(id))
      DUST_mixratio(id) = DUST_mixratio_0*  &
           exp(conrath_nu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))
   endif

   ! Region 2: Reference pressure level within this layer:
   
   if(DUST_tautot_ref_pr <= pl(id+1) .and.  &
        DUST_tautot_ref_pr >= pl(id)) then
      DUST_avg_pr = 0.5_f4*(pl(id)+DUST_tautot_ref_pr)
      pdif1 = DUST_tautot_ref_pr-pl(id)
      pdif2 = pl(id+1)-DUST_tautot_ref_pr
      DUST_mixratio(id) = DUST_mixratio_0*  &
           (exp(conrath_nu*(ONE_f4-(DUST_tautot_ref_pr/DUST_avg_pr)))*pdif1+  &
           pdif2)/(pl(id+1)-pl(id))
   endif

   ! Region 3: Mixing ratio constant:

   if(DUST_tautot_ref_pr < pl(id)) then
      DUST_mixratio(id) = DUST_mixratio_0
   endif

enddo

!*** Calculate dust optical depth at reference pressure levels:

DUST_tau(1) = ZERO_f4
do id=2,LB_end
   id1 = id-1
   DUST_tau(id) = DUST_tau(id1)+DUST_mixratio(id1)*  &
        (pl(id)-pl(id1))
enddo

end subroutine radn_mvdust_col


end module radn_init2_mod
