!```````````````````````````````````````````````````````````````````
module radn_pt_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine radn_pt(itgt,jtgt,n1,n2,n3,p_PRAMS,t_PRAMS,LM_end)

!+ Interpolates atmospheric pressure and temperature profiles

use univ_kind_defs_mod, only: f4
use radn_main_mem_mod, only: yp,yt,x,y2,xs,pst,psp,idx02,pl,pl2,tl,tl2,rdlev_t
use spline_mod, only: calc_spline

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1,n2,n3,itgt,jtgt,LM_end
real(f4), dimension(n1,n2,n3), intent(IN) :: p_PRAMS,t_PRAMS

!== Local declarations:

integer :: ik,lrz,nq1,nq2
real(f4) :: slopet,delta,deltat

!=== Executable statements:

!*** Fill PL(*)/TL(*) with PRAMS/interpolated column values:

!*** Note:  OL = output levels   (in pl2, tl2)   
!           CL = calculation levels   (in pl, tl)

! Fill actual levels via cubic spline:
! Note: mass function will be discontinuous if not used; incidentally, linear
!       interpolation is good enough for emissivity calcs 

! Prepare work arrays for spline routine:

! Put PRAMS temperature and pressure values in arrays (switch ordering):

nq1 = n1+1
do lrz=1,n1
   nq2 = nq1-lrz
   yp(nq2) = p_PRAMS(lrz,itgt,jtgt)*0.01_f4      ! Pa to mb
   yt(nq2) = t_PRAMS(lrz,itgt,jtgt)
   x(nq2) = nq2
enddo
lrz = lrz-1     ! Need to do this because of extra DO loop count
deltat = yt(n1-1)-yt(n1-2)             !
slopet = deltat-(yt(n1-2)-yt(n1-3))    ! Fill lowest temperature level
deltat = deltat+slopet                 ! (by adaptive linear extrapolation)
yt(n1) = yt(n1-1)+deltat               !

! Interpolate to column level values calculated at the beginning of the run:

call calc_spline(x,yt,lrz,y2)                   ! For temperature
call speval4(x,yt,y2,lrz,xs,pst,idx02)
call calc_spline(x,yp,lrz,y2)    ! Calculate cubic spline function for pressure
call speval4(x,yp,y2,lrz,xs,psp,idx02)  ! Find interpolated values

! Fill most PL(*), TL(*) from working arrays:

do ik=3,idx02,2     ! psp(1) is spline-extrapolated; no good, so won't use
   pl(ik/2+1+rdlev_t/2) = psp(ik)
   tl(ik/2+1+rdlev_t/2) = pst(ik)
enddo
do ik=2,idx02-1,2
   pl2(ik/2+rdlev_t/2) = psp(ik)
   tl2(ik/2+rdlev_t/2) = pst(ik)
enddo

!*** FILL "FAKE" UPPERMOST LEVEL VALUES 
!     [pl,tl(1:rdlev_t-1), pl2,tl2(1:rdlev_t-2)]:

! Fill pressures at top:

pl(1) = 1.e-10_f4            ! Set top level to very small pressure value (mb)

! Fill temperature at top:

tl(1) = 101._f4     ! Fake temp at top of atmosphere (>100K for the fun of it)

! Extrapolate upwards for last point that greatly affects heating rate of
!  topmost (k=n1) PRAMS level [pl/tl(rdlev_t-1)]:
!      [uses ln(press) extrapolation]

ik = rdlev_t/2+2     !% rdlev_t/2+1

! ln[pl2(ik)]-ln[pl(ik+1)] = ln[X]-ln[pl2(ik)]
! ==> ln[pl2(ik)/pl(ik+1)] = ln[X/pl2(ik)] ==> solve for X (which is pl(ik))
pl(ik)=pl2(ik)*pl2(ik)/pl(ik+1)  ! Extrapolated pressure

slopet = (tl2(ik)-tl(ik+1))/log(pl2(ik)/pl(ik+1))
deltat = slopet*log(pl(ik)/pl2(ik))
tl(ik) = tl2(ik)+deltat    ! Extrapolated temperature

! Extrapolate upwards (making sure profile smoothly trends towards pl(1), tl(1)
!  values):

delta = log(pl(1)/pl(ik))/real(rdlev_t+2)    !% rdlev_t)
slopet = (tl(1)-tl(ik))/(delta*real(rdlev_t+2))     !% rdlev_t))
do ik=rdlev_t+2,2,-1         !% rdlev_t,2,-1

   if(mod(ik,2) /= 0) then   ! pl,tl
      pl(ik/2+1) = exp(log(pl2(ik/2+1))+delta)
      tl(ik/2+1) = tl2(ik/2+1)+slopet*delta
   else               ! pl2,tl2
      pl2(ik/2) = exp(log(pl(ik/2+1))+delta)
      tl2(ik/2) = tl(ik/2+1)+slopet*delta
   endif

enddo

!^
!{ 1 } Write pressures and temperatures used in radiation calcs to file:
!open(33,file='radn_pt.dat',status='unknown')
!do ik=1,LM_end+1
!   write (*,'(i4,5f13.6)') ik,pl(ik),tl(ik),pl2(ik),tl2(ik),pl2(ik+1)-pl2(ik)
!enddo
!close(33)
!^

end subroutine radn_pt


!```````````````````````````````````````````````````````````````````
subroutine column_init(n1,qfact,inidel)

!+ Calculates dimensions for column storage

use univ_kind_defs_mod, only: f4
use radn_main_mem_mod, only: fmcut,fmcnt

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1
real(f4), intent(IN) :: inidel

real(f4), intent(OUT) :: qfact

!== Local declarations:

integer :: qcnt,qcut
real(f4) :: qlev,qdel,qlevp,qfactp,qfactmp,botmin,botmax,qdmin,qdfmax
logical :: qdone

!=== Executable statements:

botmin = inidel-0.001_f4     ! Define what is "close enough to inidel"
botmax = inidel+0.001_f4     !
qdmin = 0.125_f4       !  Minimum fine mesh spacing is 1/8 coarse spacing
qdfmax = 1._f4/6._f4          ! Maximum fine mesh spacing is 1/6 coarse spacing

qcut = n1-fmcut+1        ! PRAMS level where fine mesh begins
qfact = 0.95_f4         ! Starting value for stretching factor
qdone = .FALSE.     ! Initialize so that loop below will execute

do while(.not.qdone)
   qdone = .TRUE.         ! Assume that output will be exactly what we want
   qcnt = 0
   qlev = qcut       ! Initialize to PRAMS level where fine mesh should start
   qdel = inidel      ! Start with a grid spacing of half the PRAMS level
   qlevp = 0._f4
   qcalc : do while(qlev <= real(n1)+botmin-1)
      qcnt = qcnt+1
      qdel = qdel*qfact     ! Stretch the spacing
      if(qdel < qdmin) then       ! If spacing too small:
         qfactmp = qfact
         if(qlev < qlevp) then    ! We overshot on the last attempt
            qfact = (qfact+qfactp)*0.5_f4     ! Try a middle value
         else        ! qfact is too small
            qfact = qfact+0.001_f4        ! Increase qfact
         endif
         qlevp = qlev
         qfactp = qfactmp
         qdone = .FALSE.
         exit qcalc
      endif
      qlev = qlev+qdel      ! Level value (in fractional PRAMS levels)
   enddo qcalc
   if(qdel > qdfmax) then      ! If spacing too large:
      qfactp = qfact
      qfact = qfact-0.001_f4           ! Decrease qfact
      qdone = .FALSE.
   endif

   if(qdone) then      ! If everything else has gone well:
      if(qlev > real(n1)+botmax-1) then    ! If bottom value not close enough:
         qfactp = qfact
         qfact = qfact-0.00001_f4         ! Decrease qfact
         qdone = .FALSE.
      elseif(mod(qcnt,2) /= 1) then    ! Check if number of levels is odd
         qfactmp = qfact
         qfact = (qfact+qfactp)*0.5_f4      ! Try a middle value of qfact
         qdone = .FALSE.            ! Number of levels must be odd
         qfactp = qfactmp
      endif
   endif
enddo
fmcnt = qcnt

write (*,'(/a,5x,f9.7,3x,i4/)') 'Fine mesh solution found:',qfact,qcnt

end subroutine column_init


!```````````````````````````````````````````````````````````````````
subroutine pt_init(n1,qfact,inidel)

!+ Calculates variable fine mesh spacings

use univ_kind_defs_mod, only: f4
use radn_main_mem_mod, only: fmcut,xs,fmcnt

implicit none

!=== Argument declarations:

integer, intent(IN) :: n1
real(f4), intent(IN) :: qfact,inidel

!== Local declarations:

real(f4) :: qdel
integer :: ikcut,ik

!=== Executable statements:

do ik=1,(n1-fmcut+1)*2
   xs(ik) = real(ik)*0.5_f4
enddo
ikcut = ik-1

qdel = inidel
do ik=1,fmcnt
   qdel = qdel*qfact
   xs(ik+ikcut) = xs(ik+ikcut-1)+qdel
enddo

!do ik=n1-fmcut+1,n1-2
!   obl1 = jsrchgt((n1-fmcut+1)*2+fmcnt,xs,1,real(ik)-inidel)-1
!   obl2 = jsrchgt((n1-fmcut+1)*2+fmcnt,xs,1,real(ik)+inidel)-1
!
!   if(mod(obl1,2) == 1) then
!      obf1 = (xs(obl1+2)-(real(ik)-inidel))/(xs(obl1+2)-xs(obl1))
!      obl3 = obl1+1
!   else
!      obf1 = (xs(obl1+1)-(real(ik)-inidel))/(xs(obl1+1)-xs(obl1-1))
!      obl3 = obl1
!   endif
!   if(mod(obl2,2) == 1) then
!      obf2 = ((real(ik)+inidel)-xs(obl2))/(xs(obl2+2)-xs(obl2))
!      obl4 = obl2+1
!   else
!      obf2 = ((real(ik)+inidel)-xs(obl2-1))/(xs(obl2+1)-xs(obl2-1))
!      obl4 = obl2
!   endif
!
!   obe(ik) = (obl4-obl3)/2-1
!   oblt(ik) = (obl3+rdlev_t)/2
!   oblb(ik) = (obl4+rdlev_t)/2
!   obft(ik) = obf1
!   obfb(ik) = obf2

!enddo

!ik = n1-1
!obl1 = jsrchgt((n1-fmcut+1)*2+fmcnt,xs,1,real(ik)-inidel/3.)-1
!obl2 = jsrchgt((n1-fmcut+1)*2+fmcnt,xs,1,real(ik)+inidel/3.)-1
!if(mod(obl1,2) == 1) then
!   obf1 = (xs(obl1+2)-(real(ik)-inidel/3.))/(xs(obl1+2)-xs(obl1))
!   obl3 = obl1+1
!else
!   obf1 = (xs(obl1+1)-(real(ik)-inidel/3.))/(xs(obl1+1)-xs(obl1-1))
!   obl3 = obl1
!endif
!if(mod(obl2,2) == 1) then
!   obf2 = ((real(ik)+inidel/3.)-xs(obl2))/(xs(obl2+2)-xs(obl2))
!   obl4 = obl2+1
!else
!   obf2 = ((real(ik)+inidel/3.)-xs(obl2-1))/(xs(obl2+1)-xs(obl2-1))
!   obl4 = obl2
!endif

!obe(ik) = (obl4-obl3)/2-1
!oblt(ik) = (obl3+rdlev_t)/2
!oblb(ik) = (obl4+rdlev_t)/2
!obft(ik) = obf1
!obfb(ik) = obf2

end subroutine pt_init


end module radn_pt_mod
