!```````````````````````````````````````````````````````````````````
module radn_sCO2_mod

!~ 24.09.2008 (last modified; TIM)

!+

implicit none

private

public :: radn_sCO2

contains


!```````````````````````````````````````````````````````````````````
subroutine radn_sCO2(cosz_h,fcosz,LB_end,LM_end,swcut)

!~ 19.11.2008 (last modified; TIM)

!+ Calculates radiative absorption of shortwave rad. by CO2

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4, ONE_f4
use radn_main_mem_mod, only: acosz,sco2p,tl2,pl,scosz
use radn_CO2ea_mem_mod, only: mubar,DIM_mubar,DIM_CO2_emiss_pr,CO2_emiss_pr_incr  &
     ,sw_CO2_emiss
use search_mod, only: jsrchge

implicit none

!=== Argument declarations:

integer, intent(IN) :: LB_end,LM_end,swcut
real(f4), intent(IN) :: cosz_h

real(f4), intent(INOUT) :: fcosz

!== Local declarations:

!  PURPOSE
!      CO2SUN CALCULATES FOR A GIVEN 'PI' POINT THE AMOUNT OF SOLAR
!      ENERGY ABSORBED BY CO2 IN EACH ATMOSPHERIC LAYER.
!
!  AUTHOR
!      STEVE POHORSKY    INFORMATICS     TASK 605    OCT 81
!
!  FOR
!      JIM POLLACK      (SEE NOTES OF 8-20-81,3-8-82)
!
!  ENVIRONMENT
!      Cray-2            UNICOS 3.0      FORTRAN
!
!  REVISION HISTORY
!    3-82   SP   TBARP CALCULATION CHANGED AS IN NOTE OF 3-8-82.
!    4-85   JBW   PROVISION MADE TO COMPUTE STRATOSPHERIC
!       TEMPERATURES; CODE COMPLETELY STRUCTURED; VECTORIZATION
!       ENHANCED.
!
!      Re-written September 1993.
!      Removed references to all commons and created an argument list
!      to pass all variables.
!  INPUT PARAMETERS
!      PL(K) ARRAY          - PRESSURE AT EACH LEVEL FOR THE 'PI' POINT.
!                             (K = 3 TO NLEVELS.)
!      TL(K) ARRAY          - TEMPERATURE AT EACH LEVEL FOR THE 'PI'
!                             POINT. (K = 3 TO NLEVELS.)
!      COSZ                 - COSINE OF THE SOLAR ZENITH ANGLE.
!      SCOSZ                - AMOUNT OF SUNLIGHT INCIDENT AT THE TOP
!                             OF THE ATMOSPHERE.
!
!  OUTPUT PARAMETERS
!      SCO2P(K) ARRAY       - TOTAL, FOR ALL LAYERS ABOVE LEVEL K, OF
!                             SOLAR ENERGY ABSORBED BY CO2 (K=3,
!                             NLEVELS,2)
!      SCO2(K) ARRAY        - SCO2(K) IS THE AMOUNT OF SOLAR ENERGY
!                             ABSORBED BY CO2 IN THE ATMOSPHERIC
!                             LAYER WHOSE BOTTOM IS AT LEVEL K=3,
!                             NLEVELS,2)
!  CALLED BY
!      COMP3

!real(f_normal), dimension(3,DIM_mubar,0:DIM_CO2_emiss_pr) :: sw_CO2_emiss
!real(f_normal), dimension(3,0:DIM_CO2_emiss_pr,0:DIM_CO2_emiss_pr) :: lw_CO2_emiss
!real(f_normal), dimension(0:DIM_CO2_emiss_pr) :: CO2_emiss_pr
!common /CO2_emissivity/ sw_CO2_emiss,lw_CO2_emiss,CO2_emiss_pr

real(f4), dimension(3) :: abstarp
real(f4), dimension(LB_end) :: tb
real(f4) :: ptmp,cosint,cosint1,pint
integer :: ik,ikm1,mempr,mempr1,nt,nwstar,nwstar1

!=== Executable statements:

!*** Calculate the amount of solar energy absorbed by CO2 in each
!      atmospheric layer:

if(acosz <= 1.e-6_f4) then

   if(cosz_h > 1.e-6_f4 .or. acosz <= cosz_h) then    ! No sun (below horizon)

      do ik=1,LB_end   ! Loop over all OL
         sco2p(ik) = ZERO_f4    ! No solar flux absorbed if sun is down or too low
      enddo
      fcosz = ZERO_f4
      return
      
   else           ! Horizon extension
      
      acosz = 0.04_f4  ! Calcs junk if ACOSZ<.04
      fcosz = 0.04_f4

   endif

else
  
   if(cosz_h > 1.e-6_f4 .and. acosz <= cosz_h) then      ! Horizon obstruction

      fcosz = 2._f4  ! Flag to trigger albedo adjustment/etc. in DUSTSUN

   else           ! Regular horizon case or high sun elevation
   
      if(acosz < 0.04_f4) then
         acosz = 0.04_f4       ! Calcs junk if ACOSZ<.04
      endif
      fcosz = acosz

   endif

endif

!*** Interpolate line by line solar absorption coefficients in pressure,
!     solar zenith angle (acosz), and temperature:

!*** Calculate pressure weighted temperatures (on CL):

tb(2) = tl2(1)*pl(2)
do ik=3,LB_end
   tb(ik) = tb(ik-1)+tl2(ik-1)*(pl(ik)-pl(ik-1))
enddo

do ik=2,LB_end
   tb(ik) = tb(ik)/pl(ik)
enddo

if(acosz > mubar(1)) then   ! Set zenith angle interpolative indices
   nwstar = min(jsrchge(DIM_mubar-1,mubar,1,acosz)-1,DIM_mubar-1)
   cosint = (acosz-mubar(nwstar))/(mubar(nwstar+1)-mubar(nwstar))
else
   nwstar = 1
   cosint = ZERO_f4
endif
nwstar1 = nwstar+1
cosint1 = ONE_f4-cosint

sco2p(1) = ZERO_f4        ! No solar absorption at top of atmosphere

do ik=2,LB_end   ! Loop over CL
   ptmp = min(real(DIM_CO2_emiss_pr),pl(ik)/CO2_emiss_pr_incr) ! Pressure
   mempr = min(DIM_CO2_emiss_pr-1,int(ptmp))                   ! interpolative
   pint = ptmp-real(mempr)                                     ! indices
   mempr1 = mempr+1                                            !
   do nt=1,3     ! Interpolation of solar absorption coefficients:
      abstarp(nt) = (ONE_f4-pint)*(cosint1*sw_CO2_emiss(nt,nwstar,mempr)+  &
           cosint*sw_CO2_emiss(nt,nwstar1,mempr))+  &
           pint*(cosint1*sw_CO2_emiss(nt,nwstar,mempr1)+  &
           cosint*sw_CO2_emiss(nt,nwstar1,mempr1))
   enddo

   sco2p(ik) = scosz*(abstarp(1)+abstarp(2)*tb(ik)+abstarp(3)*  &
        tb(ik)*tb(ik))      ! Quadratic fit to layer solar absorption

   ikm1 = ik-1
   if(sco2p(ik) < sco2p(ikm1)) then
      sco2p(ik) = sco2p(ikm1) ! Make "monotonic"
   endif

!%   sco2(ikm1) = sco2p(ik)-sco2p(ikm1)  ! Net flux divergence over the layer
      
enddo

if(fcosz > 1.1_f4) then
   do ik=swcut,LB_end     
      sco2p(ik) = ZERO_f4      ! If shadowed region of column, no flux
   enddo
endif

end subroutine radn_sCO2


end module radn_sCO2_mod
