!```````````````````````````````````````````````````````````````````
module radn_sdust_mod

!+

implicit none


contains


!```````````````````````````````````````````````````````````````````
subroutine radn_sdust(als,cosz_h,fcosz,LB_end,LM_end,swcut,cosai,tautot_ij  &
     ,conrath_nu)

!+ Calculates radiative absorption of shortwave radiation by dust

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ZERO_f4
use univ_numtype_constants_mod, only: tiny_f4
use radn_main_mem_mod, only: pl,fluxes,scosz,sco2p,acosz
use radn_dust_mem_mod, only: DUST_tau_pr,DUST_taucum,DUST_tau,DIM_dust_levs  &
     ,DUST_tautot
use search_mod, only: jsrchgt
use radn_gsflux_mod, only: getflux
use radn_init2_mod, only: radn_mvdust_col

implicit none

!=== Argument declarations:

integer, intent(IN) :: LB_end,LM_end,swcut
real(f4), intent(IN) :: als,cosz_h,fcosz,cosai

real(f4), intent(INOUT) :: tautot_ij,conrath_nu

!== Local declarations:

!  PURPOSE
!      DUSTSUN CALCULATES FOR A GIVEN 'PI' POINT THE AMOUNT OF SOLAR
!      ENERGY ABSORBED BY DUST IN EACH ATMOSPHERIC LAYER. DUSTSUN TAKES
!      INTO ACCOUNT THE EFFECT OF A WATER ICE CLOUD , IF PRESENT, BY
!      COMBINING THE CLOUD ALBEDO WITH THE GROUND ALBEDO.
!
!  AUTHOR
!      STEVE POHORSKY    INFORMATICS     TASK 605    OCT 81
!
!  FOR
!      JIM POLLACK      (SEE NOTE OF 8-20-81)
!
!  ENVIRONMENT
!      Cray-2            UNICOS 3.0      FORTRAN
!
!  REVISION HISTORY
!     1-82  SP   PREVIOUSLY THE INCIDENT (DOWNWARD) SOLAR FLUX AT THE
!                TROPOPAUSE WAS USED AS THE NET FLUX AT THE TROPOPAUSE.
!                THE CHANGE IS TO MAKE THE INCIDENT FLUX (FLUXIN) A
!                SCALING PARAMETER TO GETFLUX AND HAVE GETFLUX COMPUTE
!                THE NET FLUX ( FLUXES(3) ) AT THE TROPOPAUSE.
!     4-82  SP   CHANGED TO USE THE NET FLUX TABLE TO COMPUTE FLUXES FOR
!                THE CASE OF NO DUST PRESENT.
!     4-82  SP   CHANGED SO FLUXES ARRAY IS AN OUTPUT PARAMETER IN ALL
!                CASES.
!     6-82  SP   TAUCUM(1) ADDED AS AN OUTPUT PARAMETER.
!
!      Re-written September 1993.
!      Removed references to all commons and created an argument list
!      to pass all variables.
!  INPUT PARAMETERS
!      JCMN & ICMN          - THE J AND I COORDINATES OF THE 'PI' POINT.
!      P(JCMN,ICMN)         - THE CURRENT 'PI' VALUE (SURFACE PRESSURE
!                             MINUS TROPOPAUSE PRESSURE) AT THE 'PI'
!                             POINT.
!      GT(JCMN,ICMN)        - GROUND TEMPERATURE OR (-1) TIMES CO2 ICE
!                             MASS AT THE 'PI' POINT.
!      ALSP(JCMN,ICMN)      - THE LOCAL SURFACE ALBEDO FOR THIS 'PI'
!                             POINT ASSUMING NO CO2 ICE IS PRESENT.
!      QDUST(JCMN,ICMN,K)   - THE MIXING RATIO OF DUST TO AIR IN THE KTH
!                             LAYER. (K = 1 TO NLAY.)
!      COSZ                 - COSINE OF THE SOLAR ZENITH ANGLE.
!      SCO2P(NLEVELS)       - TOTAL, FOR ALL LAYERS , OF SOLAR ENERGY
!                             ABSORBED BY CO2.
!
!  OUTPUT PARAMETERS
!      FLUXES(K) ARRAY      - NET SOLAR FLUX AT LEVEL K, EXCLUDING
!                             SUNLIGHT ABSORBED BY CO2. (K = 3
!                             TO NLEVELS BY 2S.)
!      SDUST(K) ARRAY       - SDUST(K) IS THE AMOUNT OF SOLAR ENERGY
!                             ABSORBED BY DUST IN THE ATMOSPHERIC
!                             LAYER WHOSE BOTTOM IS AT LEVEL K. (K = 5
!                             TO NLEVELS BY 2S.)
!      ALS                  - COMBINED SURFACE ALBEDO WHICH INCLUDES THE
!                             EFFECTS OF BOTH ICE CLOUD AND THE GROUND.
!      TAUCUM(K) ARRAY      - CUMULATIVE OPTICAL DEPTH FROM TOP OF
!                             ATMOSPHERE TO LEVEL K. (K = 1
!                             TO NLEVELS BY 2S.)
!      TAUDUST(K) ARRAY     - OPTICAL DEPTH OF THE LAYER WHOSE BOTTOM
!                             SURFACE IS LEVEL K. (K = 5 TO NLEVELS
!                             BY 2S.)
!  CALLED BY
!      COMP3
!
!  SUBROUTINES CALLED
!      GETFLUX

integer :: ik,nstar,dim1
real(f4) :: pstar1,fluxin,newalb,tmp

!=== Executable statements:

!*** Calculates the amount of solar energy absorbed by dust in each atm layer:

!*** Interpolate dust optical depths to current conditions:

if (DUST_tautot >= -tiny_f4) then    ! Use constant "tautot" and "conrath_nu"

   dim1 = DIM_DUST_levs-1

   do ik=LB_end,2,-1   ! Loop over CL
      pstar1 = max(pl(ik),DUST_tau_pr(1))
      nstar = min(jsrchgt(dim1,DUST_tau_pr,1,pstar1)-1,dim1)
      DUST_taucum(ik) = DUST_tau(nstar)+(pstar1-DUST_tau_pr(nstar))*  &
           (DUST_tau(nstar+1)-DUST_tau(nstar))/(DUST_tau_pr(nstar+1)-  &
           DUST_tau_pr(nstar))
      
      ! If dust opacity too small, set to very small constant (to avoid
      !  problems in IR routines?)
      tmp = tiny_f4*10._f4   ! Smallest value representable by data type (*10)
      if(DUST_taucum(ik) < tmp) then
         DUST_taucum(ik) = tmp
      endif

   enddo

else       ! using "more variable" dust loading scheme

   call radn_mvdust_col(LB_end,tautot_ij,conrath_nu)

   do ik=LB_end,2,-1   ! Loop over CL
      DUST_taucum(ik) = DUST_tau(ik)
      
      ! If dust opacity too small, set to very small constant (to avoid
      !  problems in IR routines?)
      tmp = tiny_f4*10._f4  ! Smallest value representable by data type (*10)
      if(DUST_taucum(ik) < tmp) then
         DUST_taucum(ik) = tmp
      endif
   enddo

endif

!*** No solar heating if sun is down or too low:

if(fcosz < 1.e-6_f4) then     ! Sun below horizon or too low
   do ik=1,LB_end    ! Loop over OL
      fluxes(ik) = ZERO_f4
   enddo
   return
elseif(fcosz > 1.1_f4 .or. cosai < -tiny_f4) then    ! Shadowed case
   newalb = 0.1_f4    ! Lowest albedo in netflux table (would like to use 0.)
else                     ! Normal sunlit case
   newalb = als   ! Use real albedo
endif

fluxes(1) = ZERO_f4       ! Set to zero at top of atmosphere

!*** Amount of solar energy (per unit area parallel to the surface) incident
!      at the top of the atmosphere, excluding the amount absorbed by CO2:

fluxin = scosz-sco2p(LB_end)

!*** Calculate net flux (FLUXES) at CL by linear interpolation:
!     (GETFLUX introduces the effects of albedo and dust)

call getflux(acosz,newalb,DUST_taucum(LB_end),DUST_taucum,fluxin,fluxes,LB_end)

!*** The solar absorption by dust in each layer is the net flux at the top
!      of the layer minus the net flux at the bottom of the layer:

!%do ik=rdlev_t/2,LM_end    ! Loop over OL
!%   sdust(ik) = fluxes(ik)-fluxes(ik+1)
!%enddo

if(fcosz > 1.1_f4) then
   do ik=swcut,LB_end     
      fluxes(ik) = ZERO_f4      ! If shadowed region of column, no flux
   enddo
endif

if(cosai < ZERO_f4) then
   fluxes(LB_end) = ZERO_f4      ! If ground tilted too much, no flux
endif

end subroutine radn_sdust


end module radn_sdust_mod
