!```````````````````````````````````````````````````````````````````
module simple_astron_scheme_mod

!+ Module to support orbital/time routines' memory and constant requirements

use univ_kind_defs_mod, only: f4, f8

implicit none

private

!*** Orbital table variables ---------------------:

type sas_table_obj
   private

   real(f8), public, pointer, dimension(:) :: L_s => NULL(),  &
                                              stellar_declination => NULL(),  &
                                              EOT_adjustment  => NULL(),  &
                                              stellocentric_radius  => NULL()

   integer, public :: n_t_entries,  &   ! Number of orbital table entries
                      t_entry_current,   &    ! Current table entry in use
                      t_entry_start     ! Starting table entry
   real(f8), public :: t_entry_inc   ! Time increment for table entries [s]

   ! Offset [s] between t_entry_start time and actual decimal start time:
   real(f8), public :: start_offset

   real(f8) :: mUT_start   ! Mean UT of start time

   ! Number of wrapped orbital table entries (usually >= 2, since this method
   !  needs to account for any tiny asymmetries in the table's L_s values):
   integer, public :: wrap_pts

   real(f8) :: mn_troporb_period   ! Tropical orbital period [s]; should be
                                    !  evenly divisible by the value of
                                    ! 't_entry_inc'

   real(f8) :: dp_len   ! Length of body's diurnal period [s]

   real(f8) ::  mn_semimajor_axis, AU_to_m_knob, body_sysmass,  &
        massratio_focus_vs_body, mn_eccentricity,  &
        mn_Ls_of_peristellon, mn_body_obliquity

   logical :: components_are_allocated = .FALSE.
end type sas_table_obj

public :: sas_table_obj

public :: sas_t_obj_recycle, populate_sas_table, set_sloc_sas_table,  &
     get_current_sas_vals

contains


!```````````````````````````````````````````````````````````````````
subroutine sas_t_obj_init(obj, n_t_entries)

!+ 

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation

!=== Argument declarations:

integer, intent(IN) :: n_t_entries

type(sas_table_obj), intent(INOUT) :: obj

!== Local declarations:

integer :: aerr

!=== Executable statements:

if (obj%components_are_allocated) call sas_t_obj_recycle(obj)

allocate(obj%L_s(n_t_entries), obj%stellar_declination(n_t_entries),  &
     obj%EOT_adjustment(n_t_entries), obj%stellocentric_radius(n_t_entries),  &
     stat=aerr)
if (aerr /= 0) call illegal_alloc_stop(for_allocation, 'sas_t_obj_init', 'A')

obj%n_t_entries = n_t_entries
obj%components_are_allocated = .TRUE.

end subroutine sas_t_obj_init


!```````````````````````````````````````````````````````````````````
subroutine sas_t_obj_recycle(obj)

!+ 

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_deallocation

!=== Argument declarations:

type(sas_table_obj), intent(INOUT) :: obj

!== Local declarations:

integer :: aerr

!=== Executable statements:

if (obj%components_are_allocated) then
   deallocate(obj%L_s, obj%stellar_declination, obj%EOT_adjustment,  &
        obj%stellocentric_radius, stat=aerr)
   if (aerr /= 0) call illegal_alloc_stop(for_deallocation,  &
        'sas_obj_recycle', '01')
   obj%L_s => NULL(); obj%stellar_declination => NULL()
   obj%EOT_adjustment => NULL(); obj%stellocentric_radius => NULL()
   obj%components_are_allocated = .FALSE.
end if

end subroutine sas_t_obj_recycle


!```````````````````````````````````````````````````````````````````
subroutine populate_sas_table(obj, mn_troporb_period, t_entry_inc,  &
     mn_semimajor_axis, AU_to_m_knob, body_sysmass, massratio_focus_vs_body,  &
     mn_eccentricity, mn_Ls_of_peristellon, mn_body_obliquity)

!+ Tabulate a variety of orbital parameters, etc. for a body's full year

use univ_kind_defs_mod, only: f8
use univ_convert_units_mod, only: deg_to_rad_f8, AU_to_m_f8, rad_to_deg_f8
use univ_exception_mod, only: exception, WARN
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN
use univ_math_constants_mod, only: twopi_f8, ZERO_f8, ONE_f8
use univ_phys_constants_mod, only: grav_const_f8

implicit none

!=== Argument declarations:

real(f8), intent(IN) :: mn_troporb_period, t_entry_inc, mn_semimajor_axis,  &
     AU_to_m_knob, body_sysmass, massratio_focus_vs_body, mn_eccentricity,  &
     mn_Ls_of_peristellon, mn_body_obliquity
type(sas_table_obj), intent(INOUT) :: obj

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'populate_sas_table'

! Some formulae are from 
!     Allison and McEwen, 2000: Planetary and Space Science, 48, 215-235

! Number of wrapped orbital table entries (usually >= 2, since this method
!  needs to account for any tiny asymmetries in the table's L_s values):
integer, parameter :: wrap_pts = 2

character(len=MAX_MESSAGE_LEN) :: msg
integer :: i
real(f8) :: mn_orbital_angular_vel, ct, mean_anomaly, true_anomaly,  &
     eccentric_anomaly, a, tmp, as_minus_Ls, lstmp

!=== Executable statements:

obj%mn_troporb_period = mn_troporb_period  ! [s]
obj%t_entry_inc = t_entry_inc  ! [s]
obj%wrap_pts = wrap_pts

! Number of orbital table entries in a body's year (wraps cyclically);
!  initialize table object:

i = nint(obj%mn_troporb_period/obj%t_entry_inc)+obj%wrap_pts
call sas_t_obj_init(obj, i)

! Calculate mean orbital velocity [rad/s] using Kepler's 3rd Law and simple
!  Newtonian two-body gravitation:

obj%mn_semimajor_axis = mn_semimajor_axis ! [AU]
obj%AU_to_m_knob = AU_to_m_knob ! [none]
obj%body_sysmass = body_sysmass ! [kg]
obj%massratio_focus_vs_body = massratio_focus_vs_body   ! [none]
obj%mn_eccentricity = mn_eccentricity  ! [none]
obj%mn_Ls_of_peristellon = mn_Ls_of_peristellon  ! [deg]
obj%mn_body_obliquity = mn_body_obliquity  ! [deg]

a = obj%mn_semimajor_axis*AU_to_m_f8*obj%AU_to_m_knob   ! [AU -> m]

mn_orbital_angular_vel = sqrt(grav_const_f8*obj%body_sysmass*  &
     (obj%massratio_focus_vs_body+ONE_f8)/(a*a*a))   ! [rad/s]

ct = ZERO_f8  ! Initialize time

!^open(76,file='orb4.dat',status='unknown',form='formatted')

do i=1,obj%n_t_entries    ! Loop through all entries in table

   ct = real(i, kind=f8)*obj%t_entry_inc   ! Current time [s]
   mean_anomaly = mn_orbital_angular_vel*ct   ! Current mean anomaly [rad]
   
   if (i == obj%n_t_entries-obj%wrap_pts) then  ! Last value close to the first?
      if (abs(mean_anomaly-twopi_f8) > (0.01_f8/t_entry_inc)) then
         write (msg, '(a,e15.7,a,e15.7,a)')  &
              'orbital table not cyclic enough (',  &
              abs(mean_anomaly-twopi_f8), 'must be <= ', 0.01_f8/t_entry_inc,  &
              '; change AUtoM and/or morbpd values *slightly*?).'
         call exception(procedure_name, WARN, trim(msg))
      end if
   end if

   ! Calculate other (true, eccentric) anomalies:
   call calc_te_anomalies(mean_anomaly, true_anomaly, eccentric_anomaly,  &
        obj%mn_eccentricity)

   ! Current stellocentric radius:
   obj%stellocentric_radius(i) = obj%mn_semimajor_axis*  &  ! [AU]
        (ONE_f8-obj%mn_eccentricity*cos(eccentric_anomaly))

   lstmp = true_anomaly+obj%mn_Ls_of_peristellon*deg_to_rad_f8  ! Current Ls
   if (lstmp >= twopi_f8) then
      lstmp = lstmp-twopi_f8
   end if
   obj%L_s(i) = lstmp*rad_to_deg_f8  ! [degrees]

   tmp = asin(sin(obj%mn_body_obliquity*deg_to_rad_f8)*sin(lstmp)) 
   obj%stellar_declination(i) = tmp         ! Stellar declination [radians]

   ! (True stellar right ascension - Ls):
   tmp = tan(obj%mn_body_obliquity*deg_to_rad_f8*0.5_f8)
   as_minus_Ls = (-tmp*tmp*sin(2._f8*lstmp)+  &
        0.5_f8*tmp*tmp*tmp*tmp*sin(4._f8*lstmp)-  &
        0.333333333_f8*tmp*tmp*tmp*tmp*tmp*tmp*sin(6._f8*lstmp))

   tmp = -((true_anomaly-mean_anomaly)+(as_minus_Ls))  ! Equation of time
   if (tmp >= twopi_f8) tmp = tmp-twopi_f8
!  obj%EOT_adjustment(i) = tmp/twopi_d*24._f8    ! [body-hours]
   obj%EOT_adjustment(i) = tmp   ! [radians]

!^   write (76, '(i10,6(g18.10))') i, ct, mean_anomaly,  &
!^        obj%stellocentric_radius(i), obj%L_s(i),  &
!^        obj%stellar_declination(i), obj%EOT_adjustment(i)

end do

!^close(76)

end subroutine populate_sas_table


!```````````````````````````````````````````````````````````````````
subroutine calc_te_anomalies(mean_anomaly, true_anomaly, eccentric_anomaly,  &
     eccentricity)

!+ Calculates the true and eccentric (orbital) anomalies

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: twopi_f8, ONE_f8, ZERO_f8

implicit none

!=== Argument declarations:

real(f8), intent(IN) :: mean_anomaly, eccentricity

real(f8), intent(OUT) :: true_anomaly, eccentric_anomaly

!== Local declarations:

! Iteratively solves Kepler's equation (given a mean anomaly and eccentricity)
!   for the corresponding eccentric anomaly; then computes and returns the
!   true anomaly.
!
!      mean_anom -- the mean anomaly in radians
!      true_anom -- the true anomaly in radians
!
!      eccn_anom -- the eccentric anomaly in radians
!      ecc       -- the eccentricity of the orbit

real(f8) :: ea, delta, ta, f_ea, deriv_f_ea

!=== Executable statements:

ea = mean_anomaly  ! Start by setting the eccentric anomaly to the mean anomaly
delta = ONE_f8  ! Initialize delta to huge number (relatively)

do
   f_ea = ea-eccentricity*sin(ea)-mean_anomaly
   deriv_f_ea = ONE_f8-eccentricity*cos(ea)
   delta = f_ea/deriv_f_ea
   if (abs(delta) <= 1.e-8_f8) exit  ! Iterate until desired precision reached
   ea = ea-delta
end do

!@ This single statement is equivalent to the below two full lines, but may
!@  require more computational time:
!@ta = 2._f8*atan2(sqrt(ONE_f8+eccentricity)*sin(0.5_f8*ea),  &
!@     sqrt(ONE_f8-eccentricity)*cos(0.5_f8*ea))
ta = 2._f8*atan(sqrt((ONE_f8+eccentricity)/  &
     (ONE_f8-eccentricity))*tan(0.5_f8*ea))   ! Calculate true anomaly
if (ta < ZERO_f8) ta = ta+twopi_f8    ! Check if in correct quadrant

true_anomaly = ta; eccentric_anomaly = ea

end subroutine calc_te_anomalies


!```````````````````````````````````````````````````````````````````
subroutine set_sloc_sas_table(obj, mUT_start, dp_len, target_Ls)

!+ Find and set starting timestep in orbital info table

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ZERO_f8, ONE_f8
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

! Universal Time (LMST at body's longitude=0) [decimal body-hours]:
real(f8), intent(IN) :: mUT_start

real(f8), intent(IN) :: dp_len   ! Length of body's diurnal period [s]

type(sas_table_obj), intent(INOUT) :: obj
real(f8), intent(INOUT) :: target_Ls

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'set_sloc_sas_table'

integer :: i, tloc, itest
real(f8) :: ls_tb, test1, test2, test3, test4, tgttmp, ttmp, start_ts, start_tm

!=== Executable statements:

obj%dp_len = dp_len

! Adjust target Ls for search:

ls_tb = obj%L_s(1)            !
if (target_Ls < ls_tb) then   !
   tgttmp = target_Ls+360._f8 ! Create continuous "function" [ values range from
else                          !  L_s(1) to almost L_s(1)+360.]
   tgttmp = target_Ls         !
end if                        !

tloc = 0    ! Initialize table location
do i=1,obj%n_t_entries-1    ! Loop over entire orbital info table
   test2 = obj%L_s(i+1)
   if (test2 < ls_tb .or. (i+1) == obj%n_t_entries) then
      test2 = test2+360._f8   ! Force to be continuous "function"
   end if

   if (test2 > tgttmp) then    ! Timestep with target_Ls found
      tloc = i; exit    ! Break out of do loop
   end if
end do
if (tloc == 0)  &
     call exception(procedure_name, FATAL,  &
     'Ls search failed (TARGET_LS not 0.-360.? Orbital calculations failed?)')

! Find datafile timestep mean UT near target mean UT:

obj%mUT_start = mUT_start

! Number of mean diurnal periods from the starting location in the table:
ttmp = real(tloc, kind=f8)*real(obj%t_entry_inc/dp_len, kind=f8)
! "Hours" elapsed in the current diurnal period:
ttmp = (ttmp-real(int(ttmp), kind=f8))*24._f8

test1 = obj%mUT_start-ttmp
test2 = 24._f8-abs(test1)

if (abs(test1) > test2) then
   itest = -int(sign(ONE_f8, test1)*1.1_f8)  ! -1 or 1; backwards or forwards
   test4 = test2       ! change mean UT by up to this many body-hours
else
   itest = int(sign(ONE_f8, test1)*1.1_f8)  ! -1 or 1; backwards or forwards
   test4 = abs(test1)  ! change mean UT by up to this many body-hours
end if

i = tloc
test1 = ZERO_f8
do while(test1 < test4)  ! Loop until mean UT changed by enough

   if (i >= obj%n_t_entries-1 .and. itest > 0) then
      i = i-obj%n_t_entries-2           ! Deal with table wrapping issues
   else if (i < 1) then                 !
      i = i+obj%n_t_entries-2           ! 
   end if                                

   test2 = real(i, kind=f8)*real(obj%t_entry_inc/dp_len, kind=f8)
   test2 = (test2-real(int(test2), kind=f8))*24._f8
   
   ttmp = real(i+1, kind=f8)*real(obj%t_entry_inc/dp_len, kind=f8)
   ttmp = (ttmp-real(int(ttmp), kind=f8))*24._f8

   ! Adjust for day changes between timesteps:

   if (ttmp < test2) then
      test3 = ttmp+24._f8
   else
      test3 = ttmp
   end if

   test1 = test1+abs(test3-test2)
   i = i+itest

end do

if (itest < 0) then
   obj%t_entry_start = i   ! Final datafile timestep to use as a starting place
else
   obj%t_entry_start = i-itest
end if

obj%t_entry_current = obj%t_entry_start

!*** Note: the following time calculation MUST be done with 8-byte floats. ***

! Time of start timestep (seconds from beginning of table):
start_ts = real(obj%t_entry_start, kind=f8)*obj%t_entry_inc
! Time of model start (seconds from beginning of table):
start_tm  = (real(int(start_ts/dp_len), kind=f8)+  &
     real(obj%mUT_start/24._f8, kind=f8))*dp_len
if (start_tm < start_ts) start_tm = start_tm+dp_len  ! Adjust for dp-wrapping

! Offset of start time from start timestep:
obj%start_offset = real(start_tm-start_ts, kind=f8)
if (abs(obj%start_offset) >= real(obj%t_entry_inc, kind=f8))  &
     call exception(procedure_name, FATAL,  &
     'abs(start_offset) >= t_entry_inc (BUG!; FIX ME!')

! Set Ls of model start correctly:
ttmp = obj%start_offset/obj%t_entry_inc
target_Ls = (ONE_f8-ttmp)*obj%L_s(obj%t_entry_start)+  &
     ttmp*obj%L_s(obj%t_entry_start+1)

end subroutine set_sloc_sas_table


!```````````````````````````````````````````````````````````````````
subroutine get_current_sas_vals(obj, current_elapsed_time, current_mUT,  &
     current_Ls, current_EOTadj, current_sdec, current_srad)

!+ Calculates interpolated orbital/time info given current
!     time; increments orbtime module timestep as needed 

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ONE_f8

implicit none

!=== Argument declarations:

real(f8), intent(IN) :: current_elapsed_time

type(sas_table_obj), intent(INOUT) :: obj

real(f8), intent(OUT) :: current_mUT, current_Ls, current_EOTadj,  &
     current_sdec, current_srad

!== Local declarations:

real(f8) :: ctime, centm, ctdiff
integer :: centry, ce1

!=== Executable statements:

! Current mean UT:

current_mUT = current_elapsed_time*(24._f8/obj%dp_len)+obj%mUT_start
current_mUT = mod(current_mUT, 24._f8)

! Current time in table (seconds):
ctime = real(obj%t_entry_start, kind=f8)*obj%t_entry_inc+  &
     current_elapsed_time+obj%start_offset
centry = int(ctime/obj%t_entry_inc)  ! Current table entry based on above
centm = real(centry, kind=f8)*obj%t_entry_inc    ! Time of centry table entry
ctdiff = (ctime-centm)/obj%t_entry_inc   ! Interpolation ratio

do while (centry >= obj%n_t_entries-1)    ! Table wrapping; multiple years
   centry = centry-obj%n_t_entries+2    !   table idx [(ntabent-1) = 1]
end do                           !
ce1 = centry+1
obj%t_entry_current = centry

if (obj%L_s(centry) <= obj%L_s(ce1)) then
   current_Ls = (ONE_f8-ctdiff)*obj%L_s(centry)+ctdiff*obj%L_s(ce1)
else     ! Evade 360/0. degree boundary issue
   current_Ls = (ONE_f8-ctdiff)*obj%L_s(centry)+ctdiff*(obj%L_s(ce1)+360._f8)
   current_Ls = mod(current_Ls, 360._f8)
end if
current_EOTadj = (ONE_f8-ctdiff)*obj%EOT_adjustment(centry)+  &
     ctdiff*obj%EOT_adjustment(ce1)
current_sdec = (ONE_f8-ctdiff)*obj%stellar_declination(centry)+  &
     ctdiff*obj%stellar_declination(ce1)
current_srad = (ONE_f8-ctdiff)*obj%stellocentric_radius(centry)+  &
     ctdiff*obj%stellocentric_radius(ce1)

end subroutine get_current_sas_vals


end module simple_astron_scheme_mod
