!```````````````````````````````````````````````````````````````````
module za_vardust_spec_mod

!+ Module to specify zonally-averaged *Mars* dust amount (proxied by total
!   column radiative transfer opacity) and vertical distribution.

use univ_kind_defs_mod, only: f4

implicit none

private

! Determine (in a portable fashion) some useful datatype KINDs:
!%integer, parameter :: i4 = selected_int_kind(9),  &    ! can hold +/- 10^9
!%     f4 = selected_real_kind(6,30),  & ! >= 6 dec_dig prec, dec_exp of +/- 30
!%     f8 = selected_real_kind(12,200)   ! >= 12 dec_dig prec, dec_exp of +/- 200

real(f4), allocatable, dimension(:,:,:) :: spline_coeff

integer :: nlat_orig, nLs_wrap, opac_idx, htop_idx, lw_idx, up_idx
real(f4), allocatable, dimension(:) :: lat_orig, Ls_wrap
real(f4), allocatable, dimension(:,:) :: opac_wrap, htop_wrap

logical :: init_completed = .FALSE.

public :: za_vardust_spec_init, za_vardust_spec, za_vardust_spec_end

contains


!```````````````````````````````````````````````````````````````````
subroutine za_vardust_spec_init(fn_path)

!+ Initialize some module parameters

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use netcdf, only: nf90_open, nf90_inq_dimid, nf90_inquire_dimension,  &
     nf90_inq_varid, nf90_get_var, nf90_nowrite, nf90_close
use spline_mod, only: calc_spline

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: fn_path

!== Local declarations:

character(len=MAX_FILE_PATH_LEN) :: fname
integer :: ncid, lat_dimid, rec_dimid, itmp_0D, varid_tmp, n, nLs_orig
real(f4), allocatable, dimension(:) :: tmp1_orig
real(f4), allocatable, dimension(:,:) :: tmp2_orig

!=== Executable statements:

if (init_completed) return   ! Nothing to do here

!*** Read in fields from file(s) and precalculate spline coefficients:

! Open the NetCDF file:
fname = fn_path(1:len_trim(fn_path))//'/'//  &
     'dust_9um_opacity-haze_top_TIM-zavg_5deg_Mars.nc'
call nc_check(nf90_open(fname, nf90_nowrite, ncid))

! Read in its contents:

call nc_check(nf90_inq_dimid(ncid, 'lat', lat_dimid))
call nc_check(nf90_inq_dimid(ncid, 'Ls', rec_dimid))
   
call nc_check(nf90_inquire_dimension(ncid, lat_dimid, len=itmp_0D))
nlat_orig = itmp_0D

call nc_check(nf90_inquire_dimension(ncid, rec_dimid, len=itmp_0D))
nLs_orig = itmp_0D; nLs_wrap = itmp_0D+2

allocate(lat_orig(nlat_orig), tmp1_orig(nLs_orig),  &
     tmp2_orig(nlat_orig,nLs_orig), Ls_wrap(nLs_wrap),  &
     opac_wrap(nlat_orig,nLs_wrap), htop_wrap(nlat_orig,nLs_wrap))

call nc_check(nf90_inq_varid(ncid, 'lat', varid_tmp))
call nc_check(nf90_get_var(ncid, varid_tmp, lat_orig))

call nc_check(nf90_inq_varid(ncid, 'Ls', varid_tmp))
call nc_check(nf90_get_var(ncid, varid_tmp, tmp1_orig))
Ls_wrap(2:nLs_orig+1) = tmp1_orig
Ls_wrap(1) = Ls_wrap(nLs_orig+1)-360._f4
Ls_wrap(nLs_wrap) = Ls_wrap(2)+360._f4

call nc_check(nf90_inq_varid(ncid, 'opacity', varid_tmp))
call nc_check(nf90_get_var(ncid, varid_tmp, tmp2_orig))
opac_wrap(:,2:nLs_orig+1) = tmp2_orig
opac_wrap(:,1) = opac_wrap(:,nLs_orig+1)
opac_wrap(:,nLs_wrap) = opac_wrap(:,2)

call nc_check(nf90_inq_varid(ncid, 'haze_top', varid_tmp))
call nc_check(nf90_get_var(ncid, varid_tmp, tmp2_orig))
htop_wrap(:,2:nLs_orig+1) = tmp2_orig
htop_wrap(:,1) = htop_wrap(:,nLs_orig+1)
htop_wrap(:,nLs_wrap) = htop_wrap(:,2)

! Close file:
call nc_check(nf90_close(ncid))

! Calculate (and save) natural spline coefficients:

allocate(spline_coeff(nlat_orig,nLs_wrap,2))

opac_idx = 1; htop_idx = 2

do n=1,nLs_wrap
   call calc_spline(lat_orig,opac_wrap(:,n),nlat_orig,  &
        spline_coeff(:,n,opac_idx))
   call calc_spline(lat_orig,htop_wrap(:,n),nlat_orig,  &
        spline_coeff(:,n,htop_idx))
enddo

lw_idx = 1; up_idx = min(nLs_wrap,2)    ! First guess for Ls location

deallocate(tmp1_orig, tmp2_orig)

!*** Module initialization complete:

init_completed = .TRUE.

end subroutine za_vardust_spec_init


!```````````````````````````````````````````````````````````````````
subroutine za_vardust_spec(n_tgt,Ls_tgt,lat_tgt,opacity,haze_top)

!+ Calculate interpolated opacity and haze_top values, given target latitude(s)
!   and Ls.

use univ_kind_defs_mod, only: f4
use univ_math_constants_mod, only: ONE_f4
use spline_mod, only: spline_interpolate

implicit none

!=== Argument declarations:

integer, intent(IN) :: n_tgt
real(f4), intent(IN) :: Ls_tgt
real(f4), dimension(n_tgt), intent(IN) :: lat_tgt

real(f4), dimension(n_tgt), intent(OUT) :: opacity, haze_top

!== Local declarations:

integer :: i
real(f4), dimension(n_tgt) :: lw_val, up_val
real(f4) :: lw_fitp, up_fitp

!=== Executable statements:

if(.NOT.init_completed) then
   write (*,'(/a/)') 'ERROR {za_vardust_spec}: Must call za_vardust_spec_init first' 
   stop '** Abnormal Completion **'
endif

! If necessary, perform a bisection search for the appropriate location in
!  the Ls range:

if(.NOT. (Ls_wrap(lw_idx) <= Ls_tgt .and. Ls_wrap(up_idx) > Ls_tgt)) then
   lw_idx = min(lw_idx+1, nLs_wrap); up_idx = min(up_idx+1, nLs_wrap)
   if(Ls_wrap(lw_idx) <= Ls_tgt .and. Ls_wrap(up_idx) > Ls_tgt) then
      continue
   else
      up_idx = nLs_wrap; lw_idx = 1
      do while(up_idx-lw_idx > 1)
         i = (up_idx+lw_idx)/2
         if(Ls_wrap(i) > Ls_tgt) then
            up_idx = i
         else
            lw_idx = i
         endif
      enddo
   endif
endif

up_fitp = (Ls_tgt-Ls_wrap(lw_idx))/(Ls_wrap(up_idx)-Ls_wrap(lw_idx))
lw_fitp = ONE_f4-up_fitp

! Interpolate:

call spline_interpolate(lat_orig,opac_wrap(:,lw_idx),  &
     spline_coeff(:,lw_idx,opac_idx),nlat_orig,lat_tgt,lw_val,n_tgt)
call spline_interpolate(lat_orig,opac_wrap(:,up_idx),  &
     spline_coeff(:,up_idx,opac_idx),nlat_orig,lat_tgt,up_val,n_tgt)
opacity = lw_fitp*lw_val+up_fitp*up_val

call spline_interpolate(lat_orig,htop_wrap(:,lw_idx),  &
     spline_coeff(:,lw_idx,htop_idx),nlat_orig,lat_tgt,lw_val,n_tgt)
call spline_interpolate(lat_orig,htop_wrap(:,up_idx),  &
     spline_coeff(:,up_idx,htop_idx),nlat_orig,lat_tgt,up_val,n_tgt)
haze_top = lw_fitp*lw_val+up_fitp*up_val

end subroutine za_vardust_spec


!```````````````````````````````````````````````````````````````````
subroutine za_vardust_spec_end

!+ Initialize some module parameters

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

if(init_completed) then
   deallocate(lat_orig, Ls_wrap, opac_wrap, htop_wrap)
   deallocate(spline_coeff)

   init_completed = .FALSE.
endif

end subroutine za_vardust_spec_end


!```````````````````````````````````````````````````````````````````
subroutine nc_check(status)

!+ Checks NetCDF procedure status
  
use netcdf, only: nf90_noerr, nf90_strerror

implicit none

!=== Argument declarations:

integer, intent(IN) :: status

!== Local declarations:

! NONE

!=== Executable statements:

if(status /= nf90_noerr) then 
   write (*,'(/a/)') trim(nf90_strerror(status))
   stop
endif

end subroutine nc_check

end module za_vardust_spec_mod


!program test
!
!use za_vardust_spec_mod
!
!implicit none
!
!! Determine (in a portable fashion) some useful datatype KINDs:
!integer, parameter :: i4 = selected_int_kind(9),  &    ! can hold +/- 10^9
!     f4 = selected_real_kind(6,30),  & ! >= 6 dec_dig prec, dec_exp of +/- 30
!     f8 = selected_real_kind(12,200)   ! >= 12 dec_dig prec, dec_exp of +/- 200
!
!integer, parameter :: n = 37
!integer :: j
!real(f4) :: dlat
!real(f4), dimension(n) :: lat_tgt, opacity, haze_top
!
!dlat = 180._f4/real(n-1,kind=f4)
!do j=1,n
!   lat_tgt(j) = -90._f4+real(j-1, kind=f4)*dlat
!enddo
!!lat_tgt = (/ 61._f4, 0._f4, 73.1_f4 /)
!
!call za_vardust_spec_init('.')
!call za_vardust_spec(n,5._f4,lat_tgt,opacity,haze_top)
!call za_vardust_spec_end
!
!do j=1,n
!   print *,j,lat_tgt(j),opacity(j),haze_top(j)
!enddo
!
!end program test
