!```````````````````````````````````````````````````````````````````
module ARL_packed_fmt_mod

!+ Routines and storage that implements a utility that writes Air Resources
!   Laboratory packed-format meterological data files.  Based on circa
!   1996-2001 Air Resources Laboratory code.

use univ_kind_defs_mod, only: f4

implicit none

! packed data set array limits
integer, parameter :: mlvl = 1000,  &      ! number of vertical levels
                      mvar = 35,  &      ! number of different variables 
                      mlen = 60000,  &      ! maximum length of header record
                      mgrd = 3      ! number of simultaneous grids

integer, parameter :: label_len = 50,  &  ! Each data record's label byte-length
                      vstr_len = 4   ! Length of the ID string for each var

type mfiles
   sequence
   real(f4), dimension(mlvl) :: height      ! height of each level
   real(f4), dimension(12) :: grids      ! grid definition array
   integer, dimension(mlvl) :: nvar,  &    ! variables each level
                               nrec      ! record offset from index
   integer, dimension(mvar,mlvl) :: chks    ! rotating checksum
   character(len=vstr_len), dimension(mvar,mlvl) :: varb  ! variable ID strings
   character(len=4) :: model                 ! model identification
   logical :: newt                  ! new time flag
   integer :: ig,  &                    ! unique grid number
              ksys,  &                  ! vertical coordinate system
              nxg, nyg,  &               ! horizontal grid points
              nlvl,  &              ! number of vertical levels
              mrec,  &                  ! record number of index record
              nrpt,  &                  ! records per time period
              iy0, im0, id0,  &           ! initial date for record
              ih0, mn0,  &               ! initial hour minute
              ic0, icx,  &               ! fcst hour, extended fcst hour
              kunit,  &                 ! output unit number
              lrec,  &                  ! bytes in data part of record
              lenh,  &  ! bytes in extended header (108 + nlvl * ( 8 + nvar*8 ))
              nhrec,  &                 ! number of extended header recs
              nhbyt                 ! bytes last extend header rec
end type mfiles

type(mfiles), dimension(mgrd) :: gv    ! define for each meteorological grid
integer :: gg


contains


!```````````````````````````````````````````````````````````````````
subroutine pakset(lunit, fname, krec1, nxp, nyp, nzp, reset_ng)

!+ Packing setup initialize packing routines pakrec and pakndx
!   one entry per data file to pack.  multiple files can be packed at the
!   same time (up to mgrd - see structure). a unique unit number must
!   be defined for each output file.  the file name contains the meteo
!   structure information. after this routine the output file should
!   opened to the specified unit.

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_MESSAGE_LEN
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

integer, intent(IN) :: lunit,  &     ! output unit number
                       krec1     ! position of index record at time-1
character(*), intent(INOUT) :: fname     ! file name (default: METDATA.CFG)
logical, optional, intent(IN) :: reset_ng
integer, intent(OUT) :: nxp, nyp,  &  ! horizontal grid dimensions
                        nzp       ! vertical grid dimension (incl sfc)

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'pakset'

logical :: ftest   ! file existence test
integer :: ntot, krec, k, i, nlvl, nv, l, nl, nvar
character(len=MAX_MESSAGE_LEN) :: msg
integer, save :: ng = 0  ! grid number counter

!=== Executable statements:

if (present(reset_ng)) then
   if (reset_ng) ng = 0
end if
ng = ng+1
if (ng > mgrd) then
   write (msg, *) 'trying to initialize grid number ', ng,  &
        ': exceeds maximum number of grids (', mgrd, ')'
   call exception(procedure_name, FATAL, msg)
end if
gg = ng

!==>check for output format configuration file

inquire(file=fname, exist=ftest)
if (.not.ftest) then
   fname = 'METDATA.CFG'
   inquire(file=fname, exist=ftest)
   if (.not.ftest) then
      write (msg, *) 'unable to find data configuration file: ', trim(fname)
      call exception(procedure_name, FATAL, msg)
   end if
end if

open (lunit, file=fname)
! character field meteo model identification
read (lunit, '(20x,a4)') gv(ng)%model

! integer grid number and vertical coordinate system
! ksys - 1:sigma  2: pressure  3: z-terrain  4: hybrid
read (lunit, '(20x,i4)') gv(ng)%ig, gv(ng)%ksys

! read 12 grid mapping variables
read (lunit, '(20x,f10.0)') (gv(ng)%grids(i),i=1,12)

! grid size in x,y,z
read (lunit, '(20x,i4)') gv(ng)%nxg, gv(ng)%nyg, gv(ng)%nlvl

! level heights, variables per level, variable id
nlvl = gv(ng)%nlvl
do nl=1,nlvl
   read (lunit, '(20x,f6.0,i3,99(1x,a4))') gv(ng)%height(nl), nvar,  &
        (gv(ng)%varb(nv,nl),nv=1,nvar)
   gv(ng)%nvar(nl) = nvar
end do
close (lunit)

!==>compute extended header length
gv(ng)%lenh = 108
nlvl = gv(ng)%nlvl
do l=1,nlvl
   gv(ng)%lenh = gv(ng)%lenh+8
   nvar = gv(ng)%nvar(l)
   do k=1,nvar
      gv(ng)%lenh = gv(ng)%lenh+8
   end do
end do

!==>check for multiple extended header records
gv(ng)%lrec = (gv(ng)%nxg*gv(ng)%nyg)
! check limits
if ((gv(ng)%lenh) > mlen .or. (gv(ng)%lrec) < 108) then
   write (msg, *)  &
        'extended header exceeds limits... maximum header length: ', mlen,  &
        ', required header length:', gv(ng)%lenh, ', available (nx*xy): ',  &
        gv(ng)%lrec
   call exception(procedure_name, FATAL, msg)
end if

! number of header records
gv(ng)%nhrec = gv(ng)%lenh/gv(ng)%lrec+1
! bytes in last header record
gv(ng)%nhbyt = gv(ng)%lenh-(gv(ng)%nhrec-1)*gv(ng)%lrec

!==>compute record count offset from index record for each level
ntot = gv(ng)%nhrec
gv(ng)%nrec(1) = ntot
nlvl = gv(ng)%nlvl
do k=2,nlvl
   ntot = ntot+gv(ng)%nvar(k-1)
   gv(ng)%nrec(k) = ntot
end do

! records per time period
ntot = ntot+gv(ng)%nvar(nlvl)
gv(ng)%nrpt= ntot

! check validity of index record
krec = max0(krec1, 1)
if (mod(krec-1, ntot) /= 0) then
   write (msg, *) 'position record not an even multiple; ',  &
       'record number in argument: ', krec, '; records per time period: ', ntot
   call exception(procedure_name, FATAL, msg)
else
   write (*, *) trim(procedure_name)//' notice:'
   write (*, *) '  number of index records = ', gv(ng)%nhrec
   write (*, *) '  number of records /time = ', ntot
end if

!==>set remaining variables
gv(ng)%mrec = krec-ntot     ! internal record counter points to index record
gv(ng)%newt = .true.        ! set flag for first time group for initialization
gv(ng)%kunit = lunit        ! unit number

!==>return sizes for variable dimensions
nxp = gv(ng)%nxg
nyp = gv(ng)%nyg
nzp = gv(ng)%nlvl

end subroutine pakset


!```````````````````````````````````````````````````````````````````
subroutine pakrec(lunit, rvar, cvar, nx, ny, nxy, kvar, iy, im, id, ih, mn,  &
     ic, ll, kini)

!+ pack a record writes a specific record of data field to unit kun
!   previously opened for direct unformatted access

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_MESSAGE_LEN
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

integer, intent(IN) :: lunit,  &       ! output unit number
                       nx, ny,  &       ! dimensions of rvar
                       nxy,  &         ! dimensions of cvar
                       iy, im, id,  &    ! date identification
                       ih, mn,  &       ! time identification (mn-minutes)
                       ic,  &          ! forecast hour, icx hour for >99
                       ll,  &          ! level indicator
                       kini        ! initialization (0-no 1-yes)
real(f4), dimension(nx,ny), intent(IN) :: rvar  ! input data to be packed
character(len=1), dimension(nxy), intent(OUT) :: cvar   ! packed data array
character(len=vstr_len), intent(IN) :: kvar    ! descriptor of variable written

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'pakrec'

character(len=label_len) :: label                  ! standard record label
real(f4) :: prec, var1 
integer :: nvar, nxyg, nexp, jrec, ksum, k, kk, kg, nv, il, icw
character(len=MAX_MESSAGE_LEN) :: msg

!=== Executable statements:

!==>check if requested unit number properly opened
kg = 0
do kk=1,gg
   if (lunit == gv(kk)%kunit) kg = kk
end do
if (kg == 0) then
   write (msg, *)  &
        'uninitialized unit; must call pakset first for unit: ', lunit
   call exception(procedure_name, FATAL, msg)
end if

!==>test grid dimensions for consistency with initialization
nxyg = gv(kg)%nxg*gv(kg)%nyg
if (gv(kg)%nxg /= nx .or. gv(kg)%nyg /= ny .or. nxyg /= nxy) then
   write (msg, *) 'file dimensions do not match; initialization: ',  &
        gv(kg)%nxg, gv(kg)%nyg, nxyg, ', argument list: ', nx, ny, nxy
   call exception(procedure_name, FATAL, msg)
end if

!==>standard forecast hour to write cannot exceed two digits
icw = min(ic, 99)

!==>set all base variables with first entry at each new time
if (gv(kg)%newt) then

   !    increment internal counter to next index record
   gv(kg)%mrec = gv(kg)%mrec+gv(kg)%nrpt

   !    extended forecast hour (3 digit)
   gv(kg)%icx = ic

   !    save initial times for headers
   gv(kg)%iy0 = iy
   gv(kg)%im0 = im
   gv(kg)%id0 = id
   gv(kg)%ih0 = ih
   gv(kg)%mn0 = mn
   gv(kg)%ic0 = icw

   !    set switch, reset by pakndx
   gv(kg)%newt = .false.

   !    initialize all records in this time group to null
   if (kini == 1) call pakini(kg, cvar, nxy)

else

   !==>check current if current record consistent with first
   if (iy /= gv(kg)%iy0 .or. im /= gv(kg)%im0 .or.  &
        id /= gv(kg)%id0 .or. ih /= gv(kg)%ih0) then
      write (msg, *) 'at index: ', gv(kg)%mrec, '; argument list times: ',  &
           iy, im, id, ih, ' do not match initial times: ', gv(kg)%iy0,  &
           gv(kg)%im0, gv(kg)%id0, gv(kg)%ih0
      call exception(procedure_name, FATAL, msg)
   end if

end if

!==>when no data is supplied just do a normal return
!   normally used in conjunction with the initialization flag
if (kvar == 'null') return

!==>check vertical index
if (ll < 1 .or. ll > gv(kg)%nlvl) then
   write (msg, *) 'level indicator out of range; argument level: ', ll,  &
        ', valid range: 1 to ', gv(kg)%nlvl
   call exception(procedure_name, FATAL, msg)
else
   !    level indicator should =0 at the surface
   il = ll-1
end if

!==>compute the record offset based upon variable match
nv = 0
nvar = gv(kg)%nvar(ll)
do k=1,nvar
   if (kvar == gv(kg)%varb(k,ll)) nv = k
end do

if (nv == 0) then
   write (msg, *) 'variable not in cfg file; argument list variable: ', kvar
   call exception(procedure_name, FATAL, msg)
end if

!==>pack data and write

! convert real to packed character
call pakout(rvar, cvar, nx, ny, nxy, prec, nexp, var1, ksum)

! save checksum in table
gv(kg)%chks(nv,ll) = ksum

! write index portion of record
write (label, '(5i2,2i2,a4,i4,2e14.7)') iy, im, id, ih, icw, il, gv(kg)%ig,  &
     kvar, nexp, prec, var1

! compute record based upon variable offset
jrec = gv(kg)%mrec+gv(kg)%nrec(ll)+nv-1
if (jrec <= 1) then
   write (msg, *) 'output record <=1; index record: ', gv(kg)%mrec,  &
        ', level offset: ', gv(kg)%nrec(ll), ', varbl offset: ', nv
   call exception(procedure_name, FATAL, msg)
else
   write (gv(kg)%kunit, rec=jrec) label, cvar
end if

end subroutine pakrec


!```````````````````````````````````````````````````````````````````
subroutine pakini(kg, cvar, nxy)

!+ packed data initialization - writes null variables fields to all
!   the records of an output file for a particular time period

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_MESSAGE_LEN
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

integer, intent(IN)  :: kg,  &         ! current active grid number
                        nxy        ! dimension of packed data
character(len=1), dimension(nxy), intent(OUT) :: cvar  ! packed data array

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'pakini'

character(len=label_len) :: label   
integer :: k, kk, iy, im, ih, ig, mrec, nhrec, nexp, nlvl, nvar, ic, nv, nl,  &
     il, id
real(f4) :: prec, var1
character(len=MAX_MESSAGE_LEN) :: msg

!=== Executable statements:

ic = -1  ! forecast field defaults to missing (-1)

! packing values all default to zero
nexp = 0
prec = 0._f4
var1 = 0._f4

! record number of index record
mrec = gv(kg)%mrec
if (mrec < 1) then
   write (msg, *) 'index record <1 = ', mrec
   call exception(procedure_name, FATAL, msg)
else
   write (*, *) trim(procedure_name)//' notice: start initialization rec = ',  &
        mrec
end if

! initialize packed data array
do k=1,nxy
   cvar(k) = ' '
end do
cvar(nxy) = char(13)

! set dates
iy = gv(kg)%iy0
im = gv(kg)%im0
id = gv(kg)%id0
ih = gv(kg)%ih0

! grid index
ig = gv(kg)%ig

! index record
write (label, '(7i2,a4,i4,2e14.7)') iy, im, id, ih, ic, 0, ig, 'indx', nexp,  &
     prec, var1
mrec = mrec-1
nhrec = gv(kg)%nhrec
do kk=1,nhrec
   mrec = mrec+1
   write (gv(kg)%kunit, rec=mrec) label, cvar
end do

! number of level loop
nlvl = gv(kg)%nlvl
do nl=1,nlvl
   !    level indicator (0=surface)
   il = nl-1

   !    number of variables per level
   nvar = gv(kg)%nvar(nl)
   do nv=1,nvar

      write (label, '(7i2,a4,i4,2e14.7)') iy, im, id, ih, ic, il, ig, 'null',  &
           nexp, prec, var1
      mrec = mrec+1
      write (gv(kg)%kunit, rec=mrec) label, cvar

   end do
end do
write (*, *) trim(procedure_name)//' notice: end initialization rec = ', mrec

end subroutine pakini


!```````````````````````````````````````````````````````````````````
subroutine pakout(rvar, cvar, nx, ny, nxy, prec, nexp, var1, ksum)

!+ Pack output converts a real array to character(len=1) packed array

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: nx, ny, nxy   ! dimension limits
real(f4), dimension(nx,ny), intent(IN) :: rvar  ! data array to be packed  
character(len=1), dimension(nxy), intent(OUT) :: cvar  ! packed output array
real(f4), intent(OUT) :: prec,  &        ! precision of packed data array
                         var1        ! value of real array at position (1,1)
integer, intent(OUT) :: nexp,  &        ! packing scaling exponent
                        ksum        ! rotating checksum of packed data 

!== Local declarations:

integer :: icval, i, j, k
real :: scale, rcol, sexp, rmax, rold 

!=== Executable statements:

var1 = rvar(1,1)

rold = var1
rmax = 0._f4

! find the maximum difference between adjacent elements
do j=1,ny
   do i=1,nx
      !       compute max difference between elements along row
      rmax = max(abs(rvar(i,j)-rold), rmax)
      rold = rvar(i,j)
   end do
   !    row element 1 difference always from previous row
   rold = rvar(1,j)
end do

sexp = 0._f4
! compute the required scaling exponent
if (abs(rmax) > tiny(rmax)*10._f4) sexp = log(rmax)/log(2._f4)
nexp = int(sexp)
! positive or whole number scaling round up for lower precision
if (sexp >= 0._f4 .or. mod(sexp, 1._f4) < tiny(sexp)*10._f4) nexp = nexp+1
! precision range is -127 to 127 or 254
prec = (2._f4**nexp)/254._f4
scale = 2._f4**(7-nexp)

! initialize checksum
ksum = 0
! set column1 value
rcol = var1

k = 0
! pack the array from low to high
do j=1,ny
   rold = rcol
   do i=1,nx
      k = k+1

      !       packed integer at element
      icval = int((rvar(i,j)-rold)*scale+127.5_f4)
      !       previous element as it would appear unpacked
      rold = (icval-127._f4)/scale+rold
      !       save the first column element for next row
      if (i == 1) rcol = rold
      !       convert to character
      cvar(k) = char(icval)

      !       maintain rotating checksum
      ksum = ksum+icval
      !       if sum carries over the eighth bit add one
      if (ksum >= 256) ksum = ksum-255

   end do
end do

end subroutine pakout


!```````````````````````````````````````````````````````````````````
subroutine pakndx(lunit)

!+ pack index - after all the records for a particular time
!   period have been written to a file, this routine writes the
!   index record for that time group.  the index record is always
!   the first record of the time group.  it includes grid definition
!   variables, and checksum information.

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

integer, intent(IN) :: lunit

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'pakndx'

integer :: nvar, nlvl, nrec, jrec, nhl1, nhl2, i, j, k, l, kk, kg, kol 
real(f4) :: zl 
character(len=label_len) :: label   ! standard record label
character(len=mlen) :: header  ! extender header

!=== Executable statements:

!==>determine which grid
kg = 0
do kk=1,gg
   if (lunit == gv(kk)%kunit) kg = kk
end do
if (kg == 0) call exception(procedure_name, FATAL,  &
     'requesting uninitialized unit (must call pakset first)')

!==>conventional 50 byte label
write (label, '(5i2,2i2,a4,i4,2e14.7)') gv(kg)%iy0, gv(kg)%im0, gv(kg)%id0,  &
     gv(kg)%ih0, gv(kg)%ic0, 0, gv(kg)%ig, 'indx', 0, 0._f4, 0._f4

!==>first part of header: 1 -> 108
write (header(1:108), '(a4,i3,i2,12f7.2,3i3,i2,i4)') gv(kg)%model,  &
     gv(kg)%icx, gv(kg)%mn0, gv(kg)%grids, gv(kg)%nxg, gv(kg)%nyg,  &
     gv(kg)%nlvl, gv(kg)%ksys, gv(kg)%lenh

!==>loop through remainder of the extended header
kol = 109
nlvl = gv(kg)%nlvl

do l=1,nlvl
   zl = gv(kg)%height(l)

!    precision depends upon the height coordinate
   if (zl >= 10000._f4) then
      write (header(kol:kol+7), '(f6.0,i2)') zl, gv(kg)%nvar(l)
   else if (zl >= 1000._f4) then
      write (header(kol:kol+7), '(f6.1,i2)') zl, gv(kg)%nvar(l)
   else if (zl >= 100._f4 .and. zl < 1000._f4) then
      write (header(kol:kol+7), '(f6.2,i2)') zl, gv(kg)%nvar(l)
   else if (zl >= 10._f4 .and. zl < 100._f4) then
      write (header(kol:kol+7), '(f6.3,i2)') zl, gv(kg)%nvar(l)
   else if (zl >= 1._f4 .and. zl < 10._f4) then
      write (header(kol:kol+7), '(f6.4,i2)') zl, gv(kg)%nvar(l)
   else
      write (header(kol:kol+7), '(f6.5,i2)') zl, gv(kg)%nvar(l)
   end if

   !    add variable id's and checksums
   kol = kol+8
   nvar = gv(kg)%nvar(l)
   do k=1,nvar
      write (header(kol:kol+7), '(a4,i3)') gv(kg)%varb(k,l), gv(kg)%chks(k,l)
      kol = kol+8
   end do
end do

!==>write extended header to disk

nhl1 = 1
! number of index records
nrec = gv(kg)%nhrec
! point to first index record
jrec = gv(kg)%mrec

! test for previous setup
if (jrec < 1) call exception(procedure_name, FATAL, 'no prior calls to pakrec')

do k=1,nrec
   !    byte count for each index
   nhl2 = nhl1+gv(kg)%lrec-1
   if (k == nrec) nhl2 = nhl1+gv(kg)%nhbyt-1

   write (gv(kg)%kunit, rec=jrec) label, header(nhl1:nhl2)
   jrec = jrec+1
   nhl1 = nhl2+1
end do

!==>clear flags

! checksum table
do j=1,mlvl
   do i=1,mvar
      gv(kg)%chks(i,j) = 0
   end do
end do

! new time flag
gv(kg)%newt = .true.

end subroutine pakndx


!```````````````````````````````````````````````````````````````````
subroutine pakinp(rvar, cvar, nx, ny, nx1, ny1, lx, ly, prec, nexp, var1, ksum)

!+ pack input does the conversion of char(len=1) packed array to
!   a real(f4) data array

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

real(f4), dimension(:,:), intent(OUT) :: rvar     ! real data unpacked
character(len=1), dimension(:), intent(IN) :: cvar     ! packed input of nx*ny
integer, intent(IN) :: nx, ny,  &          ! size of input array  
                       nx1, ny1,  &        ! optional sub-grid left edge 
                       lx, ly,  &          ! length of sub-grid
                       nexp           ! packing scaling exponent
real(f4), intent(IN) :: prec,  &           ! precision of packed data 
                        var1           ! value of array at (1,1)
integer, intent(INOUT) :: ksum           ! rotating checksum 

!== Local declarations:

real(f4) :: scale, rold, rnew
integer :: i, ii, j, jj, k

!=== Executable statements:

scale = 1._f4/2._f4**(7-nexp)  ! scaling exponent

!-------------------------------------------------------------------------------
! unpack initial value for each row of column 1
!-------------------------------------------------------------------------------

rold = var1
do j=1,ny
   k = (j-1)*nx+1                              ! position at column 1
   rnew = ((ichar(cvar(k))-127._f4)*scale)+rold   ! value from value of prev row
   rold = rnew
   jj = j-ny1+1                                ! index in output sub-grid
   if (jj >= 1 .and. jj <= ly) rvar(1,jj) = rnew ! case of sub-grid at left edge
end do

!-------------------------------------------------------------------------------
! only unpack within j-subgrid
!-------------------------------------------------------------------------------

do j=ny1,(ny1+ly-1)
   jj=j-ny1+1                             ! sub-grid array (1 to ly)

   rold = rvar(1,jj)
   do i=2,(nx1+lx-1)                      ! unpack i from 1 to i sub-grid max
      k = (j-1)*nx+i
      rnew = ((ichar(cvar(k))-127._f4)*scale)+rold
      rold = rnew
      ii = i-nx1+1                          ! sub-grid array element (1 to lx)
      if (abs(rnew) < prec) rnew = 0._f4       ! check precision for true zero
      if (ii >= 1 .and. ii <= lx) rvar(ii,jj) = rnew
   end do
end do

!-------------------------------------------------------------------------------
! only do full-grid checksum when ksum=0
!-------------------------------------------------------------------------------

if (ksum /= 0) return

do k=1,nx*ny
   ksum = ksum+ichar(cvar(k))
   if (ksum >= 256) ksum = ksum-255    ! sum carries over the eighth bit add one
end do

end subroutine pakinp


end module ARL_packed_fmt_mod
