#!/usr/bin/env python
"""
Sets up and executes a PRAMS postprocessor run.

This program requires either Python version 2.6+ or 3.2+, and is importable as a
python module.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import sys
import os
import errno
import subprocess
import getopt
import string
import random
import time
import datetime
import io
import reformat_PRAMS_namelist

#------------------------------------------------------------------------------
def mkdir_p(path):
    """Emulates 'mkdir -p' functionality."""
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def run_postp_instance(run_root_dir_full, exe_file_full, generated_nml_fn,
                       nml_file_full, generated_nml_str, init_delay,
                       use_debugger):
    """Set up and execute a postp instance (can run as a separate thread)."""

    # Create configured run directory (and its parent dirs), if needed:
    mkdir_p(run_root_dir_full)

    # Change to configured run directory:
    os.chdir(run_root_dir_full)

    # Open new output namelist file; write the Fortran namelist contents:
    try:
        text_to_write = unicode(generated_nml_str, "ascii")  # v2.6+
    except:
        text_to_write = generated_nml_str  # v3.2+
    with io.open(generated_nml_fn, "wt", encoding="ascii") as output_f:
        output_f.write(text_to_write)

    # Generate '&postp_args' Fortran namelist block, and append it to the
    #  namelist file produced above:
    text_to_write = \
                    "\n&postp_args\n" \
                    " namelist_fn = \'"+nml_file_full+"\' \n" \
                    "/\n"
    with io.open(generated_nml_fn, "at", encoding="ascii") as output_f:
        output_f.write(text_to_write)

    # Construct command-line sequence:
    cmd_line_seq = []
    if use_debugger:
        cmd_line_seq.append("gdb")
    cmd_line_seq.append(exe_file_full)    

    # Delay for a specified amount of time (possibly to allow time_staggering
    #  of multiple IO-intensive processes):
    time.sleep(init_delay)

    # Try to execute the completed command-line sequence; return the command,
    #  its exit status, and any exception raised:
    try:
        p = subprocess.Popen(cmd_line_seq, universal_newlines=True)
        return ' '.join(cmd_line_seq), p.wait(), None
    except Exception as e:
        return ' '.join(cmd_line_seq), None, str(e)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def main(nml_file, use_debugger, exe_file_pfx, run_root_dir):
    """Driver routine."""

    # Define some useful parameters:

    nml_file_full = os.path.abspath(nml_file)
    run_root_dir_full = os.path.abspath(run_root_dir)
    exe_file_pfx_full = os.path.abspath(exe_file_pfx)

    generated_nml_fn_pfx = "auto_gen-postp.nml"

    generated_nml_fn = generated_nml_fn_pfx
    exe_file_full = exe_file_pfx_full

    # Reformat a more human-readable namelist into a more computer-friendly
    #  one; store it in a (complex) string:
    generated_nml_str0 = reformat_PRAMS_namelist.rfmt_Ps_nml(nml_file_full,
                                                             False)

    today = datetime.date.today()

    # Create "unique" run_root directory path for this process:
    run_root_dir_full_wID = os.path.join(run_root_dir_full,
                                         "autogen_temporary-postp",
                                         str(today)+'-'+ \
                   ''.join(random.choice(string.ascii_uppercase + string.digits)
                      for _ in range(8)))

    # Do the configured work:
    run_postp_instance(run_root_dir_full_wID, exe_file_full, generated_nml_fn,
                       nml_file_full, generated_nml_str0, 0., use_debugger)
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
def usage():
    usage_msg = """
usage: run_postp.py [-h] [-f NML_FILE] [-d] [--exe-file-prefix PFX]
                    [--run-root_dir DIR]

Sets up and executes a PRAMS postprocessor run.

optional arguments:
  -h, --help   Show this help message and exit.
  -f NML_FILE, --nml-file NML_FILE   The namelist filename/path to use
                                      (default: 'POSTP_IN')
  -d, --debug   Run postprocessor inside a debugger utility (gdb)
                 (default: False)
  --exe-file-prefix PFX   PRAMS postprocessor executable location/name prefix
                           (default: './exe/postp')
  --run-root-dir DIR   Directory where the postp executable will/should be run
                        from (it and its parents will be created if needed
                        (default: '.')
"""
    print(usage_msg)
#------------------------------------------------------------------------------


if __name__ == "__main__":
    # Process arguments:
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:d", ["help", "nml-file=",
                                  "debug", "exe-file-prefix=", "run-root-dir="])
    except getopt.GetoptError as err:
        print(err)
        usage()
        sys.exit(1)
      # Default option values:
    nml_file = "POSTP_IN"
    use_debugger = False
    exe_file_pfx = os.path.join("exe", "postp")
    run_root_dir = '.'
      # Act according to the options given:
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(1)
        elif o in ("-f", "--nml-file"):
            nml_file = a
        elif o in ("-d", "--debug"):
            use_debugger = True
        elif o in ("--exe-file-prefix"):
            exe_file_pfx = a
        elif o in ("--run-root-dir"):
            run_root_dir = a
        else:
            assert False, "unhandled option"
      # Act according to the arguments given:
    if (len(args) > 0 or len(args) < 0):
        print("An incorrect number of arguments were provided.")
        usage()
        sys.exit(1)

    # Run driver:
    main(nml_file, use_debugger, exe_file_pfx, run_root_dir)
