!```````````````````````````````````````````````````````````````````
module postp_ARL_mod

!+

use univ_kind_defs_mod, only: f4

implicit none

! Fictitious Earth date:
integer, parameter :: i_Eyear = 2007, i_Emonth = 1, i_Eday = 1
real(f4), parameter :: Eday_s = 86400._f4, Ehr_s = 3600._f4, Emin_s = 60._f4


contains


!```````````````````````````````````````````````````````````````````
subroutine postp_ARL_driv

!+ Post-processing driver for ARL format output

use univ_kind_defs_mod, only: f4, f8
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_convert_units_mod, only: m_to_km_f4, Pa_to_hPa_f4
use univ_integer_time_mod, only: time_obj, fp_seconds_eqv_Time
use univ_exception_mod, only: exception, FATAL
use univ_std_fileIO_mod, only: find_file_IOunit, mkdir_p, determine_IO_reclen
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use univ_namelist_utils_mod, only: Time_to_gen_time_strrep
use body_specific_constants_mod, only: mnsdy
use postp_main_memory_mod, only: cfg_f, cfg_f_inc, output_pp_file_pfx,  &
     ztrnsfm, cfg_n_ov, state_file_name, model_sim_ET, cfg_vname, nxp, nyp,  &
     nvar_read, what_vtype, cfg_vtype, cfg_vlevs, cfg_vdesc, cfg_vunits,  &
     what_vlevs, what_vdesc, what_vunits, n_state_files, grid_idx_list,  &
     cfg_n_g, cfg_spatial, zt, PRAMS_vdata, use_full_az_scr, use_full_xy_scr,  &
     vl_scr1_t2, vctr1_az, use_full_azxy_scr, use_full_sz_scr,  &
     full_az_scr_needed, vl_scr2_t2, full_xy_scr_needed, nzp, nvar_want,  &
     full_azxy_scr_needed, full_sz_scr_needed, xt, plat, plon,  &
     undef_value_f4, var_want_to_read
use postp_varlib_mod, only: postp_varlib
use postp_driv_mod, only: manage_PRAMS_vdata, manage_cfg_v_memory
use postp_getdata_mod, only: count_vars_to_read, fill_PRAMS_data, get_PRAMS_data
use main_memory_mod, only: module_tps, start_UT, mnsdy_Time, morbpd_Time
use ARL_packed_fmt_mod, only: label_len, vstr_len, pakset, pakrec, pakndx

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'postp_ARL_driv'

integer :: igd, iv, ifn, lnoutp, lnstp, IO_unit, rfcnt, i, e, IO_reclen, ic, k
character(len=MAX_FILE_PATH_LEN) :: s_dfn
character(len=7) :: cgrid_a
character(len=40) :: gen_time_strrep
type(time_obj) :: tmp_Time
real(f4), allocatable, dimension(:,:) :: reclen_test
real(f4) :: UT
real(f8) :: tinc
logical :: err

character(len=vstr_len), allocatable, dimension(:) :: ARL_fieldtype
real(f4), allocatable, dimension(:) :: ARL_fieldfac
real(f4), allocatable, dimension(:,:) :: ARL_rvar
character(len=1), allocatable, dimension(:) :: ARL_ivari
character(len=MAX_FILE_PATH_LEN) :: ARL_cfgnam, ARL_datnam
integer, dimension(12) :: ARL_imnt
integer :: ARL_navrb, IOu_ARLcfg, IOu_ARLdat, ARL_krec, ARL_n1, ARL_n2,  &
     ARL_n3, ARL_n12, nimn_tinc, ARL_im, ARL_iday, ARL_iyr, ARL_ihr,  &
     ARL_fhr, ARL_imin, ARL_kini, ARL_ll, ARL_n3p1
real(f4) :: ARL_dxkm, ARL_synclat, ARL_synclon

!=== Executable statements:

! Check for some obvious confguration problems:

if (ztrnsfm == 2) call exception(procedure_name, FATAL,  &
     'ZTRNSFM = 2 can not be used with ARL-packed-format output files')

do i=1,cfg_n_g  ! Loop over all desired grids
   igd = grid_idx_list(i)
   
   if (cfg_spatial(i)%n_X < 2)  &  ! 'ARL_dxkm' calc below needs this
        call exception(procedure_name, FATAL,  &
        'Must have >= 2 X-direction points configured for output.')

   ifn = cfg_f(1)

   err = .FALSE.
   err = err .or. ( cfg_spatial(i)%i_X(1) < 2 .or.  &
        cfg_spatial(i)%i_X(cfg_spatial(i)%n_X) > nxp(igd,ifn)-1 )
   err = err .or. ( cfg_spatial(i)%i_Y(1) < 2 .or.  &
        cfg_spatial(i)%i_Y(cfg_spatial(i)%n_Y) > nyp(igd,ifn)-1 )
   err = err .or. ( cfg_spatial(i)%i_AZ(1) < 2 .or.  &
        cfg_spatial(i)%i_AZ(cfg_spatial(i)%n_AZ) > nzp(igd,ifn)-1 )
   if (err) call exception(procedure_name, FATAL,  &
        'For ARL, only computational gridpoints may be output; '//  &
        'the use of "COM" in DCFG (namelist) is recommmended.')
end do

ARL_navrb = cfg_n_ov  ! Assume that all output fields are relevant

allocate(ARL_fieldtype(cfg_n_ov), ARL_fieldfac(cfg_n_ov), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, 'ARL_fld')

ARL_fieldfac = undef_value_f4  ! Initialize; assumes no extra factor is needed

! Earth-month lengths:
ARL_imnt = (/ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /)

! Make initial pass through all grids, files, and variables; record what
!  variables need to be read in and where they may be found:

nvar_read = 0   ! Start out by assuming that no vars need read in
use_full_az_scr = .FALSE.; use_full_xy_scr = .FALSE.
use_full_azxy_scr = .FALSE.; use_full_sz_scr = .FALSE.

do i=1,cfg_n_g  ! Loop over all desired grids
   igd = grid_idx_list(i)
   do ifn=cfg_f(1),cfg_f(2),cfg_f_inc   ! Loop over desired files
      do iv=1,cfg_n_ov    ! Loop over # of variables to process

         ! What state variable(s) does this variable require? 
         call postp_varlib(igd, i, ifn, cfg_vname(iv), .TRUE.)
         use_full_az_scr(i) = ( use_full_az_scr(i) .or. full_az_scr_needed )
         use_full_xy_scr(i) = ( use_full_xy_scr(i) .or. full_xy_scr_needed )
         use_full_azxy_scr(i) = (use_full_azxy_scr(i) .or. full_azxy_scr_needed)
         use_full_sz_scr(i) = ( use_full_sz_scr(i) .or. full_sz_scr_needed )

         ! Need to configure variable(s) for reading?
         call count_vars_to_read(igd, ifn)

      end do

      ! Need lat/lon values for 'ARL_synclat'/'ARL_synclon' parameters below: 
      nvar_want = 2
      var_want_to_read(1:nvar_want) = (/'GLAT','GLON'/)
      call count_vars_to_read(igd, ifn)
   end do
end do

! Allocate some data storage:
allocate(PRAMS_vdata(nvar_read), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '1')

! Main pass; read in data, calculate and output POSTP variable values:

i = index(output_pp_file_pfx, '/', back=.TRUE.)-1  ! Determine dir name length
call mkdir_p(output_pp_file_pfx(1:i))  ! Create any needed directories

lnoutp = len_trim(output_pp_file_pfx)  ! Length of string

do i=1,cfg_n_g   ! Loop over all desired grids
   igd = grid_idx_list(i)

   call manage_PRAMS_vdata(igd)    ! Initialize PRAMS_vdata for this grid
   call manage_cfg_v_memory(igd, i) ! Init cfg-dimensioned multi-use storage

   write (cgrid_a, '(a1,i2.2,a4)') 'g', igd, '.vfm'  ! State file grid suffix

   rfcnt = 0
   do ifn=cfg_f(1),cfg_f(2),cfg_f_inc   ! Loop over desired files
      rfcnt = rfcnt+1      ! File number (relative) counter

      tmp_Time = fp_seconds_eqv_Time(model_sim_ET(ifn), module_tps)
      call Time_to_gen_time_strrep(tmp_Time, gen_time_strrep,  &
           len(gen_time_strrep), mnsdy_Time, morbpd_Time,  &
           short_unit_style=.TRUE.)
      UT = mod(real(model_sim_ET(ifn)/mnsdy*24._f8)+start_UT, 24._f4)
      write (*, '(a,i5,a,i2,3a,f6.2,a)') 'Postprocessing < ', rfcnt,  &
           ' > grid #', igd, ', time = [', trim(gen_time_strrep), '] [',  &
           UT, ' UT] ...'

      lnstp = len_trim(state_file_name(ifn))-7
      s_dfn = state_file_name(ifn)(1:lnstp)//cgrid_a    ! State file name

      IO_unit = find_file_IOunit()

      allocate(reclen_test(nxp(igd,ifn),nyp(igd,ifn)), stat=e)
      if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
           'reclen')
      IO_reclen = determine_IO_reclen(reclen_test)
      deallocate(reclen_test, stat=e)
      if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
           'reclen')

      ! Open state data file:
      open(IO_unit, file=s_dfn(1:lnstp+7), status='old', form='unformatted',  &
           access='direct', recl=IO_reclen)

      ! Read in all *NEEDED* PRAMS data for this grid/file; fill data array:
      call fill_PRAMS_data(IO_unit, ifn, igd, nxp(igd,ifn)*nyp(igd,ifn))

      close(IO_unit)   ! Close state file

      ! Compute values of configured variables; output only select data:

      if (ifn == cfg_f(1)) then

         ! Perform an initial first pass without any file output, in order to
         !  determine output var metadata:
         do iv=1,cfg_n_ov    ! Loop over # of variables to process
            ! Read/compute var data:
            call postp_varlib(igd, i, ifn, cfg_vname(iv), .FALSE.)

            ! Store/calculate variable descriptors:
            cfg_vtype(iv) = what_vtype    ! Store var types, etc.
            cfg_vlevs(iv) = what_vlevs    !
            cfg_vdesc(iv) = what_vdesc    !
            cfg_vunits(iv) = what_vunits  !

            ! Check output variables for compatibility and any needed conversion
            !  factor:
            select case (trim(cfg_vname(iv)))
             ! "surface" fields:
            case ('topo')
               ARL_fieldtype(iv) = 'SHGT'  ! [m]
            case ('sfc_press')
               ARL_fieldtype(iv) = 'PRSS'  ! [hPa]
               ARL_fieldfac(iv) = Pa_to_hPa_f4
            case ('tempk_2m')
               ARL_fieldtype(iv) = 'T02M'  ! [K]
            case ('shf_sfclyr')
               ARL_fieldtype(iv) = 'SHTF'  ! [W/m^2]
            case ('umomf_sfclyr')
               ARL_fieldtype(iv) = 'UMOF'  ! [N/m^2]
            case ('vmomf_sfclyr')
               ARL_fieldtype(iv) = 'VMOF'  ! [N/m^2]
            case ('ustar')
               ARL_fieldtype(iv) = 'USTR'  ! [m/s]
            case ('rshort')
               ARL_fieldtype(iv) = 'DSWF'  ! [W/m^2]
             ! "upper-air" fields:
            case ('u_avg')
               ARL_fieldtype(iv) = 'UWND'  ! [m/s]
            case ('v_avg')
               ARL_fieldtype(iv) = 'VWND'  ! [m/s]
            case ('w_avg')
               ARL_fieldtype(iv) = 'DZDT'  ! [m/s]
            case ('tempk')
               ARL_fieldtype(iv) = 'TEMP'  ! [K]
            case ('press')
               ARL_fieldtype(iv) = 'PRES'  ! [hPa]
               ARL_fieldfac(iv) = Pa_to_hPa_f4
            case ('H2Ov_mr')
               ARL_fieldtype(iv) = 'SPHU'  ! [kg/kg_air]
            case ('sgs_tke')
               ARL_fieldtype(iv) = 'TKEN'  ! [m^2/s^2]
            case default
               call exception(procedure_name, FATAL,  &
                    'Corresponding ARL field name is unknown for '//  &
                    trim(cfg_vname(iv)))
            end select
         end do

         ! Determine file units for ARL-format output files:
         IOu_ARLdat = find_file_IOunit(IO_unit+1)
         IOu_ARLcfg = find_file_IOunit(IOu_ARLdat+1)

         ! Reduce grid dimensions to omit boundary points:
         ARL_n1 = cfg_spatial(i)%n_X
         ARL_n2 = cfg_spatial(i)%n_Y
         ARL_n12 = ARL_n1*ARL_n2
         ARL_n3 = cfg_spatial(i)%n_AZ   ! Vertical dim of "upper-level" fields

         ! Compute time-independent date/time information needed for `pakrec`:

         if (n_state_files > cfg_f_inc) then   ! If enough files
            tinc = model_sim_ET(cfg_f(1)+cfg_f_inc)-model_sim_ET(cfg_f(1)) ! [s]
         else
            tinc = model_sim_ET(cfg_f(1))   ! Best possible right now [s]
         end if
         nimn_tinc = nint(tinc/Emin_s) ! Closest integer # of [Earth-minutes]

         ARL_im = i_Emonth; ARL_iday = i_Eday
           ! ARL-PACKED 2-digit year and Y2K fix:
         ARL_iyr = i_Eyear-1900
         if (ARL_iyr >= 100) ARL_iyr = ARL_iyr-100
           ! Leap year fix:
         if (mod(ARL_iyr, 4) == 0) ARL_imnt(2) = 29

         ARL_ihr = int(UT)   ! Integer # of body-hours, UT
         ARL_imin = nint(mod(UT, 1._f4)*60._f4)  ! Closest # of body-minutes, UT

         write (ARL_datnam, '(2a,i2.2,a)') output_pp_file_pfx(1:lnoutp),  &
              '_g', igd, '.bin'
         write (ARL_cfgnam, '(2a,i2.2,a)') output_pp_file_pfx(1:lnoutp),  &
              '_g', igd, '.cfg'

         ARL_dxkm = (xt(cfg_spatial(i)%i_X(2),igd,ifn)-  &
              xt(cfg_spatial(i)%i_X(1),igd,ifn))*m_to_km_f4  ! Delta-x [km]

           ! Obtain this grid's latitude and longitude fields:
         call get_PRAMS_data(vl_scr1_t2, 'GLAT', igd, i, ifn)
         call get_PRAMS_data(vl_scr2_t2, 'GLON', igd, i, ifn)

         ARL_synclat = vl_scr1_t2%p2D(1,1); ARL_synclon = vl_scr2_t2%p2D(1,1)

           ! Vertical layer heights to be output:
         do ic=1,cfg_spatial(i)%n_AZ
            vctr1_az%p1D(ic) = zt(cfg_spatial(i)%i_AZ(ic),igd,ifn)
         end do

         call write_ARLcfg(plat(igd,ifn), plon(igd,ifn), ARL_dxkm, 1._f4,  &
              1._f4, 99, ARL_n1, ARL_n2, ARL_n3, vctr1_az%p1D, ARL_cfgnam,  &
              ARL_fieldtype, IOu_ARLcfg, ARL_synclat, ARL_synclon, ARL_navrb,  &
              cfg_vlevs)

         ! `pakset` will alter values of 'ARL_n1' and 'ARLn2', but their values
         !  should not change; 'ARL_n3p1' is the of total ARL levels (i.e.,
         !  "upper-air"+"surface"):
         ARL_krec = 1   ! starting record number to write output
         call pakset(IOu_ARLdat, ARL_cfgnam, ARL_krec, ARL_n1, ARL_n2,  &
              ARL_n3p1, .TRUE.)

         allocate (ARL_rvar(ARL_n1,ARL_n2), ARL_ivari(ARL_n12), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'ARL_*var*')

         ! Open ARL packed data file after `pakset` call:
         allocate(reclen_test(ARL_n12+label_len,1), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              'reclen_ARL')
         IO_reclen = determine_IO_reclen(reclen_test)
         deallocate(reclen_test, stat=e)
         if (e /= 0) call illegal_alloc_stop(for_deallocation,  &
              procedure_name, 'reclen_ARL')
         open(unit=IOu_ARLdat, file=ARL_datnam, form='unformatted', &
              access='direct', recl=IO_reclen)

         ARL_kini = 0

      end if

      ! Compute time-dependent date/time information needed for `pakrec`:

      if (rfcnt > 1) then
         ARL_imin = ARL_imin+nimn_tinc  ! [Earth-minutes]
         ARL_ihr = ARL_ihr+ARL_imin/60  ! [Earth-hours]
         ARL_imin = mod(ARL_imin, 60) ! [Earth-minutes]
      end if

      ! Closest forecast [Earth-hour]:
      ARL_fhr = min(99, nint(real((rfcnt-1)*nimn_tinc, kind=f4)/60._f4))

      ! If needed, increment date/time [Earth- hours, days, months, years]:
      do while (ARL_ihr >= 24)
         ARL_ihr = ARL_ihr-24; ARL_iday = ARL_iday+1
         if (ARL_imnt(ARL_im) < ARL_iday) then
            ARL_im = ARL_im+1; ARL_iday = 1
         end if
      end do
      if (ARL_im > 12) then 
         ARL_im = 1; ARL_iyr = ARL_iyr+1
      end if

     !=== Write "surface" fields:

      ARL_ll = 1

      do iv=1,cfg_n_ov    ! Loop over # of variables to process
         if (cfg_vlevs(iv) == 0) then
              ! Read/compute var data:
            call postp_varlib(igd, i, ifn, cfg_vname(iv), .FALSE.)
              ! Copy postprocessed field into 'ARL_rvar':
            call postp_ARL_varout(i, ARL_rvar, ARL_fieldfac(iv))
              ! Write the contents of 'ARL_rvar' to an ARL -format file:
            call pakrec(IOu_ARLdat, ARL_rvar, ARL_ivari, ARL_n1, ARL_n2,  &
                 ARL_n12, ARL_fieldtype(iv), ARL_iyr, ARL_im, ARL_iday,  &
                 ARL_ihr, ARL_imin, ARL_fhr, ARL_ll, ARL_kini)
         end if
      end do

     !=== Write "upper-air" fields:

      do k=1,ARL_n3
         ARL_ll = ARL_ll+1

         do iv=1,cfg_n_ov    ! Loop over # of variables to process
            if (cfg_vlevs(iv) /= 0) then
                 ! Read/compute var data:
               call postp_varlib(igd, i, ifn, cfg_vname(iv), .FALSE.)
                 ! Copy a slice of the postprocessed field into 'ARL_rvar':
               call postp_ARL_varout(i, ARL_rvar, ARL_fieldfac(iv), k)
                 ! Write the contents of 'ARL_rvar' to an ARL-format file:
               call pakrec(IOu_ARLdat, ARL_rvar, ARL_ivari, ARL_n1, ARL_n2,  &
                    ARL_n12, ARL_fieldtype(iv), ARL_iyr, ARL_im, ARL_iday,  &
                    ARL_ihr, ARL_imin, ARL_fhr, ARL_ll, ARL_kini)
            end if
         end do
      end do

      call pakndx(IOu_ARLdat)

   end do

   deallocate (ARL_rvar, ARL_ivari, stat=e)
   if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name,  &
        'ARL_*var*')

   close (IOu_ARLdat)

   write (*, '(/,2a)') 'Wrote ARL-packed-format header file:  ',  &
        trim(ARL_cfgnam)
   write (*, '(2a,/)') 'Wrote ARL-packed-format data file:  ',  &
        trim(ARL_datnam)

end do

! Recycle data structures:
call manage_PRAMS_vdata(-1)
call manage_cfg_v_memory(-1, -1) 

! Deallocate work arrays:
deallocate(ARL_fieldtype, ARL_fieldfac, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, 'ARL_fld')
deallocate(PRAMS_vdata, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

end subroutine postp_ARL_driv


!```````````````````````````````````````````````````````````````````
subroutine postp_ARL_varout(cfg_igd, ARL_rvar, field_factor, az_idx)

!+ Copies a slice of postprocessed variable data to an output array

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_GrADS_format_mod, only: write_GrADS_dfile
use univ_exception_mod, only: exception, FATAL
use postp_main_memory_mod, only: what_vtype, vl_reorder_t3, vl_result_t2,  &
     vl_result_t3, vl_result_t4, cfg_spatial, check_undef_val_f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: cfg_igd
integer, optional, intent(IN) :: az_idx
real(f4), intent(IN) :: field_factor

real(f4), dimension(:,:), intent(OUT) :: ARL_rvar

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'postp_ARL_varout'

integer :: nx, ny, naz, ip

!=== Executable statements:

which_vtype: select case(what_vtype)
case (2)  ! "regular" 2D variable

   ARL_rvar = vl_result_t2%p2D

case (3)  ! "regular" 3D variable

   naz = cfg_spatial(cfg_igd)%n_AZ
   nx = cfg_spatial(cfg_igd)%n_X; ny = cfg_spatial(cfg_igd)%n_Y
   vl_reorder_t3%p3D = reshape(vl_result_t3%p3D, (/nx,ny,naz/), order=(/3,1,2/))

   if (present(az_idx)) then
      ARL_rvar = vl_reorder_t3%p3D(:,:,az_idx)
   else
      call exception(procedure_name, FATAL,  &
           'must supply the az_idx argument for an atmospheric 3D variable')
   end if

case (4)  ! SBPS 2D variable

   do ip=1,cfg_spatial(cfg_igd)%n_P
      ARL_rvar = vl_result_t4%p3D(:,:,ip)
   end do

case default

   write (*, '(/a,i1,a)') 'ERROR: unknown or unsupported variable type ( = ',  &
        what_vtype, ' ).'
   call exception(procedure_name, FATAL)

end select which_vtype

! If needed, scale the field:
if (field_factor < check_undef_val_f4) ARL_rvar = ARL_rvar*field_factor

end subroutine postp_ARL_varout


!```````````````````````````````````````````````````````````````````
subroutine write_ARLcfg(polelat, polelon, dxkm, xtan, ytan, igrdn, n1, n2,  &
     kqz, zout, cfgnam, ftype, IOu_cfg, slat, slon, navrb, vlevs)

!+ This routine creates the *.cfg file needed for ARL-format packing

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN
use ARL_packed_fmt_mod, only: vstr_len

implicit none

!=== Argument declarations:

real(f4), intent(IN) :: polelat, polelon, dxkm, xtan, ytan, slat, slon
integer, intent(IN) :: igrdn, n1, n2, kqz, IOu_cfg, navrb
real(f4), dimension(:), intent(IN) :: zout
character(len=*), intent(IN) :: cfgnam
character(len=vstr_len), dimension(:), intent(IN) :: ftype
integer, dimension(:), intent(IN) :: vlevs

!== Local declarations:

integer :: ivcord, nav3d, ll, nlevels, k, nasfc, flist_nc, iv
real(f4) :: orien, coneang, zsfc
character(len=MAX_MESSAGE_LEN) :: field_list_str

!=== Executable statements:

ivcord = 3  ! Terrain-sigma vertical coordinates
orien = 0._f4; coneang = 90._f4; zsfc = 0._f4

nasfc = count(vlevs == 0)
nav3d = navrb-nasfc   ! Number of 3D variables (i.e., not "surface" vars)

open(IOu_cfg, file=trim(cfgnam), status='unknown', form='formatted')

!---> Create Standard ARLPACKED config file 
   
write (*, *) 'OPENING CFG FILE: ', trim(cfgnam)

write (IOu_cfg, '(a,9x,a)') 'MODEL TYPE:', 'PRAM'  ! Max name length is 4-chars
write (IOu_cfg, '(a,9x,i4)') 'GRID NUMB: ', igrdn   ! Not used for HYSPLIT4+
write (IOu_cfg, '(a,9x,i4)') 'VERT COORD:', ivcord

!---> Tangent lat,lon
write (IOu_cfg, '(a,9x,f10.2)') 'POLE LAT:  ', polelat
write (IOu_cfg, '(a,9x,f10.2)') 'POLE LON:  ', polelon

!---> Reference lat,lon
write (IOu_cfg, '(a,9x,f10.2)') 'REF  LAT:  ', polelat
write (IOu_cfg, '(a,9x,f10.2)') 'REF  LON:  ', polelon
write (IOu_cfg, '(a,9x,f10.2)') 'REF GRID:  ', dxkm   
write (IOu_cfg, '(a,9x,f10.2)') 'ORIENTATION', orien
write (IOu_cfg, '(a,9x,f10.2)') 'CONE ANGLE:', coneang

!---> Sync X,Y, LAT,LON
write (IOu_cfg, '(a,9x,f10.2)') 'SYNC X:    ', xtan
write (IOu_cfg, '(a,9x,f10.2)') 'SYNC Y:    ', ytan

! Sync x,y is determined anywhere on the grid, instead of polelat/lon
write (IOu_cfg, '(a,9x,f10.2)') 'SYNC LAT:  ', slat
write (IOu_cfg, '(a,9x,f10.2)') 'SYNC LON:  ', slon

write (IOu_cfg, '(a)') 'SPECIAL:           '
write (IOu_cfg, '(a,9x,i4)') 'NUMB X:    ', n1
write (IOu_cfg, '(a,9x,i4)') 'NUMB Y:    ', n2

nlevels = kqz+1  ! Include a (near-)surface level
write (IOu_cfg, '(a,9x,i4)') 'NUMB LEVELS', nlevels

!=== Write "surface" level info:

! Generate a string of relevant field IDs:
flist_nc = 0
do iv=1,navrb
   if (vlevs(iv) == 0) then
      flist_nc = flist_nc+1; field_list_str(flist_nc:flist_nc) = ' '
      field_list_str(flist_nc+1:flist_nc+vstr_len) = ftype(iv)
      flist_nc = flist_nc+vstr_len
   end if
end do

! Write the configuration info line to the output file:
ll = 1
write (IOu_cfg, '(a,i3.3,12x,f6.0,i3,a)') 'LEVEL', ll, zsfc, nasfc,  &
     field_list_str(1:flist_nc)

!=== Write "upper-air" level info:

! Generate a string of relevant field IDs:
flist_nc = 0
do iv=1,navrb
   if (vlevs(iv) /= 0) then
      flist_nc = flist_nc+1; field_list_str(flist_nc:flist_nc) = ' '
      field_list_str(flist_nc+1:flist_nc+vstr_len) = ftype(iv)
      flist_nc = flist_nc+vstr_len
   end if
end do

! Write the configuration info line(s) to the output file:
do k=2,nlevels
   ll = ll+1
   write (IOu_cfg, '(a,i3.3,12x,f6.0,i3,a)') 'LEVEL', ll, zout(k-1),  &
        nav3d, field_list_str(1:flist_nc)
end do

close(IOu_cfg)

end subroutine write_ARLcfg


end module postp_ARL_mod
