!```````````````````````````````````````````````````````````````````
module postp_init_mod

!+

implicit none

integer, parameter :: absmax_var_per_sf = 10000


contains


!```````````````````````````````````````````````````````````````````
subroutine postp_init

!+ Primary initialization tasks (that depend on file data) for postp

use univ_kind_defs_mod, only: f8
use univ_math_constants_mod, only: ZERO_f8
use univ_filelist_mod, only: filelist_count, filelist
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_deallocation
use univ_exception_mod, only: exception, FATAL
use univ_std_fileIO_mod, only: find_file_IOunit
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use state_header_for_postp_mod, only: state_table_for_postp
use postp_main_memory_mod, only: input_state_file_pfx, n_state_files,  &
     n_state_f_vars, n_state_f_grids, state_file_name, manage_misc_memory,  &
     var_brec, var_erec, var_type, var_grid, var_name, var_nlen, max_n_sf_v,  &
     max_n_sf_g, cfg_f, cfg_n_f, model_sim_ET, time1, time2,  &
     cfg_t, nxp, nyp, nzp, nzg, nzs, cfg_f_inc, maxnpa,  &
     maxnxp, maxnyp, maxnzg, maxnzp, maxnzs, min_n_sf_g, npatch

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'postp_init'

character(len=MAX_FILE_PATH_LEN) :: fn_tmp
character(len=1) c1_tmp
integer :: IO_unit, ifn, e, iv, nv

!=== Executable statements:

IO_unit = find_file_IOunit() 

!*** Search for filenames with a given prefix:

fn_tmp = trim(input_state_file_pfx)//'-S-*-hdr.vfm'

! Find, count, and store all filenames that match the prefix search string:

n_state_files = 0
call filelist_count(fn_tmp, n_state_files)
if (n_state_files == 0) then
   write (*, '(/,a,/,6x,a)') 'ERROR: No state file header found with form:',  &
        trim(fn_tmp)
   call exception(procedure_name, FATAL)
end if

call manage_misc_memory(dim_n_state_files=.TRUE.)
call filelist(state_file_name, fn_tmp, n_state_files)

write (*, '(/,a,2x,i5,/)') 'Number of matching files found: ', n_state_files

! First pass through sequential binary header files; determine # of grids,
! # of variables, and simulation elapsed time for all input state files (list
! determined by `filelist` call above):

allocate(state_table_for_postp(absmax_var_per_sf), stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

do ifn=1,n_state_files
   open(IO_unit, file=trim(state_file_name(ifn)), form='unformatted',  &
        status='old')
   read (IO_unit) n_state_f_vars(ifn)   ! Only part of the info in this record
   if (n_state_f_vars(ifn) > absmax_var_per_sf) call exception(  &
        procedure_name, FATAL,  & ! Bad things will happen in this case
        'Increase the value of a hard-coded parameter (absmax_var_per_sf).')
   read (IO_unit) state_table_for_postp(1:n_state_f_vars(ifn))  ! Read state file info table
   read (IO_unit) c1_tmp     ! Only part of the info in this record
   read (IO_unit) c1_tmp     ! Only part of the info in this record
   read (IO_unit) n_state_f_grids(ifn)  ! Only part of the info in this record
   read (IO_unit) model_sim_ET(ifn)
   close(IO_unit)   ! Close header file
end do

! Find range of files desired to process:

cfg_t(1) = time1
cfg_t(2) = time2

cfg_f(1) = 0
do ifn=1,n_state_files
   if (model_sim_ET(ifn) >= cfg_t(1)) then
      cfg_f(1) = ifn; exit
   end if
end do
if (cfg_f(1) == 0) call exception(procedure_name, FATAL,  &
     'No state files found within requested time range.')

if (cfg_t(2) < ZERO_f8) then     ! Special case
   cfg_f(2) = n_state_files
else             ! Usual case
   ifn = cfg_f(1)
   do
      if (ifn > n_state_files .or. model_sim_ET(ifn) > cfg_t(2)+1.e-5_f8) exit
      ifn = ifn+1
   end do

   if (ifn == cfg_f(1)) then
      cfg_f(2) = ifn
   else
      cfg_f(2) = ifn-1
   end if
end if

cfg_n_f = 0
do ifn=cfg_f(1),cfg_f(2),cfg_f_inc
   cfg_n_f = cfg_n_f+1      ! Count number of files that should be processed 
end do

! Some max/min values (only for subset of files to be processed):
max_n_sf_v = maxval(n_state_f_vars(cfg_f(1):cfg_f(2):cfg_f_inc)) ! max vars
max_n_sf_g = maxval(n_state_f_grids(cfg_f(1):cfg_f(2):cfg_f_inc)) ! max grids
min_n_sf_g = minval(n_state_f_grids(cfg_f(1):cfg_f(2):cfg_f_inc)) ! min grids

! Allocate file/variable info tables:
call manage_misc_memory(dim_max_n_sf_v=.TRUE.)
call manage_misc_memory(dim_max_n_sf_g=.TRUE.)

! Second pass through sequential binary header files; only extract metadata from
! the subset of files that will be processed:
do ifn=cfg_f(1),cfg_f(2),cfg_f_inc

   nv = n_state_f_vars(ifn)
   
   open(IO_unit, file=trim(state_file_name(ifn)), form='unformatted',  &
        status='old')    ! Open header file
   read (IO_unit) e     ! Ignore this
   read (IO_unit) state_table_for_postp(1:nv)  ! Read state file info table

   ! Fill variable info arrays:

   do iv=1,nv
      var_name(iv,ifn) = state_table_for_postp(iv)%var_name
      var_nlen(iv,ifn) = len_trim(state_table_for_postp(iv)%var_name)
      var_brec(iv,ifn) = state_table_for_postp(iv)%beg_rec
      var_erec(iv,ifn) = state_table_for_postp(iv)%end_rec
      var_type(iv,ifn) = state_table_for_postp(iv)%var_type
      var_grid(iv,ifn) = state_table_for_postp(iv)%grid
   end do

   call postp_mine_state_f_md(ifn, IO_unit, 0) ! "Mine" some selected meta_data
   
   close(IO_unit)   ! Close header file

end do

! Find maximum state file spatial data dimensions; allocate relevant arrays:
maxnxp = maxval(nxp(:,cfg_f(1):cfg_f(2):cfg_f_inc))    ! # X points
maxnyp = maxval(nyp(:,cfg_f(1):cfg_f(2):cfg_f_inc))    ! # Y points
maxnzp = maxval(nzp(:,cfg_f(1):cfg_f(2):cfg_f_inc))    ! # Z points
maxnzg = maxval(nzg(cfg_f(1):cfg_f(2):cfg_f_inc))    ! # SBS points
maxnzs = maxval(nzs(cfg_f(1):cfg_f(2):cfg_f_inc))    ! # SPS points 
maxnpa = maxval(npatch(cfg_f(1):cfg_f(2):cfg_f_inc)) ! # patches
call manage_misc_memory(dim_max_spatial=.TRUE.)

! Third and final pass through sequential binary header files; only extract
! metadata from the subset of files that will be processed:
do ifn=cfg_f(1),cfg_f(2),cfg_f_inc

   nv = n_state_f_vars(ifn)
   
   open(IO_unit, file=trim(state_file_name(ifn)), form='unformatted',  &
        status='old')    ! Open header file
   read (IO_unit) e     ! Ignore this
   read (IO_unit) state_table_for_postp(1:nv)  ! Read state file info table

   call postp_mine_state_f_md(ifn, IO_unit, 1) ! "Mine" more selected metadata
   
   close(IO_unit)   ! Close header file

end do

! Deallocate data structure that is no longer necessary:
deallocate(state_table_for_postp, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '1')

end subroutine postp_init


!```````````````````````````````````````````````````````````````````
subroutine postp_init_dcfg

!+ Primary dimensional configuration initialization tasks for postp

use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation,  &
     for_deallocation
use univ_numtype_constants_mod, only: huge_i4
use univ_num_sort_mod, only: qi_sort
use univ_exception_mod, only: exception, FATAL, WARN
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN, MAX_FILE_PATH_LEN
use postp_main_memory_mod, only: manage_misc_memory, max_n_sf_g,  &
     cfg_f, cfg_n_g, grid_idx_list, nxp, nyp, nzp, nzg, nzs,  &
     cfg_f_inc, min_n_sf_g, npatch, dcfg, n_dcfg, cfg_spatial, have_full_az,  &
     have_full_sz, have_full_azxy, have_full_xy

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'postp_init_dcfg'

character(len=MAX_FILE_PATH_LEN) :: fn_tmp
integer :: ifn, e, i, nc_abs, ic_start, ic_beg, ic_end, i_dcfg, igd, iq,  &
     itmp_min, itmp_max, idm
integer, allocatable, dimension(:) :: idx_list_tmp, tag_list_tmp, grid_tag_list

!=== Executable statements:

!=== Parse/check "spatial" dimension configuration (dcfg) string:

! First, parse dcfg to obtain the quantity and identity of the desired grids:

allocate(idx_list_tmp(max_n_sf_g), tag_list_tmp(max_n_sf_g),  &
     grid_tag_list(max_n_sf_g), stat=e)
if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name, '2')

idx_list_tmp(:) = -1; tag_list_tmp(:) = -1; grid_idx_list(:) = -1  ! Initialize
do i_dcfg=1,n_dcfg
   nc_abs = len_trim(dcfg(i_dcfg))
   if (nc_abs < 3) call exception(procedure_name, FATAL,  &
        'dcfg string too short')
   if (dcfg(i_dcfg)(1:1) /= 'G') call exception(procedure_name, FATAL,  &
        'dcfg string G-segment must begin with "G"')

   ic_start = 2
   ic_beg = index(dcfg(i_dcfg)(ic_start:), '=')
   ic_end = index(dcfg(i_dcfg)(ic_start:), '|')-2
   fn_tmp = adjustl(dcfg(i_dcfg)(ic_beg+ic_start:ic_end+ic_start))
   
   if (trim(fn_tmp) == 'ALL') then
      do igd=1,max_n_sf_g
         idx_list_tmp(igd) = igd; tag_list_tmp(igd) = i_dcfg
      end do
   else
      call process_dim_str('G-segment', fn_tmp, idx_list_tmp,  &
           t_list=tag_list_tmp, tag=i_dcfg)
   end if
end do

if (maxval(idx_list_tmp) > min_n_sf_g) call exception(procedure_name, WARN,  &
  'Number of grids not constant over the configured time range (using minimum)')
where (idx_list_tmp > min_n_sf_g) idx_list_tmp = -1 
cfg_n_g = count(idx_list_tmp > 0)
i = 0
do igd=1,max_n_sf_g
   if (idx_list_tmp(igd) > 0) then
      i = i+1
      grid_idx_list(i) = idx_list_tmp(igd)
      grid_tag_list(i) = tag_list_tmp(igd)
   end if
end do

! Allocate more arrays:
call manage_misc_memory(dim_cfg_n_g=.TRUE.)

! Now, parse dcfg to obtain the quantity and identity of the spatial
!  dimensions:

do i_dcfg=1,n_dcfg
   nc_abs = len_trim(dcfg(i_dcfg))

! X-segment:

   ic_start = index(dcfg(i_dcfg), 'X')+1  ! Skip to X-segment
   if (ic_start == 1) call exception(procedure_name, FATAL,  &
        'dcfg string X-segment was not found')
   ic_beg = index(dcfg(i_dcfg)(ic_start:), '=')
   ic_end = index(dcfg(i_dcfg)(ic_start:), '|')-2
   fn_tmp = adjustl(dcfg(i_dcfg)(ic_beg+ic_start:ic_end+ic_start))

   do i=1,cfg_n_g
      igd = grid_idx_list(i)
      if (grid_tag_list(i) == i_dcfg) then  ! Set values using this dcfg entry
         ! Determine if this dimension varies between the configured files;
         !  if so, raise a fatal exception:
         itmp_min = huge_i4; itmp_max = 0
         do ifn=cfg_f(1),cfg_f(2),cfg_f_inc
            itmp_min = min(itmp_min, nxp(igd,ifn))
            itmp_max = max(itmp_max, nxp(igd,ifn))
         end do
         if (itmp_min /= itmp_max) call exception(procedure_name, FATAL,  &
              'X dim varies over configured time range')

         ! Set/allocate dimension index parameters:
         cfg_spatial(i)%n_X = itmp_min
         allocate(cfg_spatial(i)%i_X(itmp_min), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              '3X')

         ! Parse dcfg entry and apply the results to this dimension/grid:
         cfg_spatial(i)%i_X = huge_i4
         if (trim(fn_tmp) == 'ALL') then
            do iq=1,cfg_spatial(i)%n_X
               cfg_spatial(i)%i_X(iq) = iq
            end do
         else if (trim(fn_tmp) == 'COM') then
            idm = 0
            do iq=2,cfg_spatial(i)%n_X-1
               idm = idm+1
               cfg_spatial(i)%i_X(idm) = iq
            end do
         else
            call process_dim_str('X-segment', fn_tmp, cfg_spatial(i)%i_X)
         end if

         ! Sort and count dimension indices so they are ready for use:
         call qi_sort(cfg_spatial(i)%i_X, cfg_spatial(i)%n_X)
         cfg_spatial(i)%n_X = count(cfg_spatial(i)%i_X <= cfg_spatial(i)%n_X)
         if (cfg_spatial(i)%n_X == 0) call exception(procedure_name, FATAL,  &
              'The configured X dim has a size of 0.')
      end if
   end do

! Y-segment:

   ic_start = index(dcfg(i_dcfg), 'Y')+1  ! Skip to Y-segment
   if (ic_start == 1) call exception(procedure_name, FATAL,  &
        'dcfg string Y-segment was not found')
   ic_beg = index(dcfg(i_dcfg)(ic_start:), '=')
   ic_end = index(dcfg(i_dcfg)(ic_start:), '|')-2
   fn_tmp = adjustl(dcfg(i_dcfg)(ic_beg+ic_start:ic_end+ic_start))

   do i=1,cfg_n_g
      igd = grid_idx_list(i)
      if (grid_tag_list(i) == i_dcfg) then  ! Set values using this dcfg entry
         ! Determine if this dimension varies between the configured files;
         !  if so, raise a fatal exception:
         itmp_min = huge_i4; itmp_max = 0
         do ifn=cfg_f(1),cfg_f(2),cfg_f_inc
            itmp_min = min(itmp_min, nyp(igd,ifn))
            itmp_max = max(itmp_max, nyp(igd,ifn))
         end do
         if (itmp_min /= itmp_max) call exception(procedure_name, FATAL,  &
              'Y dim varies over configured time range')

         ! Set/allocate dimension index parameters:
         cfg_spatial(i)%n_Y = itmp_min
         allocate(cfg_spatial(i)%i_Y(itmp_min), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              '3Y')

         ! Parse dcfg entry and apply the results to this dimension/grid:
         cfg_spatial(i)%i_Y = huge_i4
         if (trim(fn_tmp) == 'ALL') then
            do iq=1,cfg_spatial(i)%n_Y
               cfg_spatial(i)%i_Y(iq) = iq
            end do
         else if (trim(fn_tmp) == 'COM') then
            idm = 0
            do iq=2,cfg_spatial(i)%n_Y-1
               idm = idm+1
               cfg_spatial(i)%i_Y(idm) = iq
            end do
         else
            call process_dim_str('Y-segment', fn_tmp, cfg_spatial(i)%i_Y)
         end if

         ! Sort and count dimension indices so they are ready for use:
         call qi_sort(cfg_spatial(i)%i_Y, cfg_spatial(i)%n_Y)
         cfg_spatial(i)%n_Y = count(cfg_spatial(i)%i_Y <= cfg_spatial(i)%n_Y)
         if (cfg_spatial(i)%n_Y == 0) call exception(procedure_name, FATAL,  &
              'The configured Y dim has a size of 0.')

         have_full_xy(i) = ( cfg_spatial(i)%n_X == nxp(igd,cfg_f(1)) .and.  &
              cfg_spatial(i)%n_Y == nyp(igd,cfg_f(1)) )
      end if
   end do

! AZ-segment:

   ic_start = index(dcfg(i_dcfg), 'AZ')+1  ! Skip to AZ-segment
   if (ic_start == 1) call exception(procedure_name, FATAL,  &
        'dcfg string AZ-segment was not found')
   ic_beg = index(dcfg(i_dcfg)(ic_start:), '=')
   ic_end = index(dcfg(i_dcfg)(ic_start:), '|')-2
   fn_tmp = adjustl(dcfg(i_dcfg)(ic_beg+ic_start:ic_end+ic_start))

   do i=1,cfg_n_g
      igd = grid_idx_list(i)
      if (grid_tag_list(i) == i_dcfg) then  ! Set values using this dcfg entry
         ! Determine if this dimension varies between the configured files;
         !  if so, raise a fatal exception:
         itmp_min = huge_i4; itmp_max = 0
         do ifn=cfg_f(1),cfg_f(2),cfg_f_inc
            itmp_min = min(itmp_min, nzp(igd,ifn))
            itmp_max = max(itmp_max, nzp(igd,ifn))
         end do
         if (itmp_min /= itmp_max) call exception(procedure_name, FATAL,  &
              'AZ dim varies over configured time range')

         ! Set/allocate dimension index parameters:
         cfg_spatial(i)%n_AZ = itmp_min
         allocate(cfg_spatial(i)%i_AZ(itmp_min), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              '3AZ')

         ! Parse dcfg entry and apply the results to this dimension/grid:
         cfg_spatial(i)%i_AZ = huge_i4
         if (trim(fn_tmp) == 'ALL') then
            do iq=1,cfg_spatial(i)%n_AZ
               cfg_spatial(i)%i_AZ(iq) = iq
            end do
         else if (trim(fn_tmp) == 'COM') then
            idm = 0
            do iq=2,cfg_spatial(i)%n_AZ-1
               idm = idm+1
               cfg_spatial(i)%i_AZ(idm) = iq
            end do
         else
            call process_dim_str('AZ-segment', fn_tmp, cfg_spatial(i)%i_AZ)
         end if

         ! Sort and count dimension indices so they are ready for use:
         call qi_sort(cfg_spatial(i)%i_AZ, cfg_spatial(i)%n_AZ)
         cfg_spatial(i)%n_AZ = count(cfg_spatial(i)%i_AZ <= cfg_spatial(i)%n_AZ)
         if (cfg_spatial(i)%n_AZ == 0) call exception(procedure_name, FATAL,  &
              'The configured AZ dim has a size of 0.')

         have_full_az(i) = ( cfg_spatial(i)%n_AZ == nzp(igd,cfg_f(1)) )
         have_full_azxy(i) = ( have_full_az(i) .and. have_full_xy(i) )
      end if
   end do

! SZ-segment:

   ic_start = index(dcfg(i_dcfg), 'SZ')+1  ! Skip to SZ-segment
   if (ic_start == 1) call exception(procedure_name, FATAL,  &
        'dcfg string SZ-segment was not found')
   ic_beg = index(dcfg(i_dcfg)(ic_start:), '=')
   ic_end = index(dcfg(i_dcfg)(ic_start:), '|')-2
   fn_tmp = adjustl(dcfg(i_dcfg)(ic_beg+ic_start:ic_end+ic_start))

   do i=1,cfg_n_g
      igd = grid_idx_list(i)
      if (grid_tag_list(i) == i_dcfg) then  ! Set values using this dcfg entry
         ! Determine if this dimension varies between the configured files;
         !  if so, raise a fatal exception:
         itmp_min = huge_i4; itmp_max = 0
         do ifn=cfg_f(1),cfg_f(2),cfg_f_inc
            itmp_min = min(itmp_min, nzg(ifn)+nzs(ifn))
            itmp_max = max(itmp_max, nzg(ifn)+nzs(ifn))
         end do
         if (itmp_min /= itmp_max) call exception(procedure_name, FATAL,  &
              'SZ dim varies over configured time range')

         ! Set/allocate dimension index parameters:
         cfg_spatial(i)%n_SZ = itmp_min
         allocate(cfg_spatial(i)%i_SZ(itmp_min), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              '3SZ')

         ! Parse dcfg entry and apply the results to this dimension/grid:
         cfg_spatial(i)%i_SZ = huge_i4
         if (trim(fn_tmp) == 'ALL') then
            do iq=1,cfg_spatial(i)%n_SZ
               cfg_spatial(i)%i_SZ(iq) = iq
            end do
         else
            call process_dim_str('SZ-segment', fn_tmp, cfg_spatial(i)%i_SZ)
         end if

         ! Sort and count dimension indices so they are ready for use:
         call qi_sort(cfg_spatial(i)%i_SZ, cfg_spatial(i)%n_SZ)
         cfg_spatial(i)%n_SZ = count(cfg_spatial(i)%i_SZ <= cfg_spatial(i)%n_SZ)
         if (cfg_spatial(i)%n_SZ == 0) call exception(procedure_name, FATAL,  &
              'The configured SZ dim has a size of 0.')

         have_full_sz(i) = ( cfg_spatial(i)%n_SZ == nzg(cfg_f(1))+nzs(cfg_f(1)) )
      end if
   end do

! P-segment:

   ic_start = index(dcfg(i_dcfg), 'P')+1  ! Skip to P-segment
   if (ic_start == 1) call exception(procedure_name, FATAL,  &
        'dcfg string P-segment was not found')
   ic_beg = index(dcfg(i_dcfg)(ic_start:), '=')
   ic_end = index(dcfg(i_dcfg)(ic_start:), '|')-2
   fn_tmp = adjustl(dcfg(i_dcfg)(ic_beg+ic_start:ic_end+ic_start))

   do i=1,cfg_n_g
      igd = grid_idx_list(i)
      if (grid_tag_list(i) == i_dcfg) then  ! Set values using this dcfg entry
         ! Determine if this dimension varies between the configured files;
         !  if so, raise a fatal exception:
         itmp_min = huge_i4; itmp_max = 0
         do ifn=cfg_f(1),cfg_f(2),cfg_f_inc
            itmp_min = min(itmp_min, npatch(ifn))
            itmp_max = max(itmp_max, npatch(ifn))
         end do
         if (itmp_min /= itmp_max) call exception(procedure_name, FATAL,  &
              'P dim varies over configured time range')

         ! Set/allocate dimension index parameters:
         cfg_spatial(i)%n_P = itmp_min
         allocate(cfg_spatial(i)%i_P(itmp_min), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              '3P')

         ! Parse dcfg entry and apply the results to this dimension/grid:
         cfg_spatial(i)%i_P = huge_i4
         if (trim(fn_tmp) == 'ALL') then
            do iq=1,cfg_spatial(i)%n_P
               cfg_spatial(i)%i_P(iq) = iq
            end do
         else
            call process_dim_str('P-segment', fn_tmp, cfg_spatial(i)%i_P)
         end if
      
         ! Sort and count dimension indices so they are ready for use:
         call qi_sort(cfg_spatial(i)%i_P, cfg_spatial(i)%n_P)
         cfg_spatial(i)%n_P = count(cfg_spatial(i)%i_P <= cfg_spatial(i)%n_P)
         if (cfg_spatial(i)%n_P == 0) call exception(procedure_name, FATAL,  &
              'The configured P dim has a size of 0.')
      end if
   end do

end do

! Allocate and fill special index arrays for "full" (ignoring configuration)
!  dimension ranges:
do i=1,cfg_n_g

  ! X-segment:
   itmp_min = nxp(grid_idx_list(i),cfg_f(1))
   allocate(cfg_spatial(i)%i_fX(itmp_min), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        '4X')
   do iq=1,itmp_min
      cfg_spatial(i)%i_fX(iq) = iq
   end do

  ! Y-segment:
   itmp_min = nyp(grid_idx_list(i),cfg_f(1))
   allocate(cfg_spatial(i)%i_fY(itmp_min), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        '4Y')
   do iq=1,itmp_min
      cfg_spatial(i)%i_fY(iq) = iq
   end do

  ! AZ-segment:
   itmp_min = nzp(grid_idx_list(i),cfg_f(1))
   allocate(cfg_spatial(i)%i_fAZ(itmp_min), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        '4AZ')
   do iq=1,itmp_min
      cfg_spatial(i)%i_fAZ(iq) = iq
   end do

  ! SZ-segment:
   itmp_min = nzg(cfg_f(1))+nzs(cfg_f(1))
   allocate(cfg_spatial(i)%i_fSZ(itmp_min), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        '4SZ')
   do iq=1,itmp_min
      cfg_spatial(i)%i_fSZ(iq) = iq
   end do

  ! P-segment:
   itmp_min = npatch(cfg_f(1))
   allocate(cfg_spatial(i)%i_fP(itmp_min), stat=e)
   if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
        '4P')
   do iq=1,itmp_min
      cfg_spatial(i)%i_fP(iq) = iq
   end do

end do

deallocate(idx_list_tmp, tag_list_tmp, grid_tag_list, stat=e)
if (e /= 0) call illegal_alloc_stop(for_deallocation, procedure_name, '2')

end subroutine postp_init_dcfg


!```````````````````````````````````````````````````````````````````
subroutine postp_mine_state_f_md(ifn, IO_unit, phase)

!+ Grabs relevant info from state file metadata

use univ_kind_defs_mod, only: f4, f8
use univ_IO_mem_status_mod, only: illegal_alloc_stop, for_allocation
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use postp_main_memory_mod, only: n_state_f_grids, nzg, nzs, naddsc, ztop,  &
     npatch, nxp, nyp, nzp, dzm, dzt, manage_misc_memory, zm, zt, maxnzp, xm,  &
     xt, ym, yt, plon, plat, SBPS_tlev, map_projection, r, rup, rlow, rmass,  &
     nelements, ngroups, max_nbins, nbins, groupname, elemname, rhoelem,  &
     ienconc, dust_ig, dustnc_ie, H2Oi_ig, H2Oinc_ie, CO2i_ig, CO2inc_ie,  &
     dust_present, H2Oi_present, CO2i_present, rhop, vol, Ls_of_periapse,  &
     orbital_eccentricity, axial_obliquity, AU_to_m_knob, Ls_curr, UT_curr,  &
     EOT_curr, sdec_curr, sdist_curr, RTc_tog, dust_op_case
use main_memory_mod, only: PRAMS_version, simulation_description, target_Ls,  &
     start_UT
use aertxrad_mem_mod, only: lw_iwbeg, lw_iwend, sw_iwbeg, sw_iwend,  &
     nlw_wavlnrng, ntot_wavlnrng, spc_nbins, spc_ctype, naerspc_rad,  &
     spc_ig, aeradID_dust, aeradID_H2Oi, aeradID_CO2i, spc_xsarea_fact
use aermic_rad_mem_mod, only: dataset_path_bspec
use namelist_bspec_mod, only: default_input_data_dir_bspec

implicit none

!=== Argument declarations:

integer, intent(IN) :: ifn, IO_unit, phase

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'postp_mine_state_f_md'

integer :: junki, ig, n1, n2, n3, ie, ib, ip, ng, k, metadata_version, e, igd
real(f8) :: junk_f8
real(f4) :: junkf
character(len=4) :: tmp_str

!=== Executable statements:

read (IO_unit) PRAMS_version, simulation_description
read (IO_unit) tmp_str
read (tmp_str, *) metadata_version

read (IO_unit) junki, nzg(ifn), nzs(ifn), npatch(ifn), naddsc(ifn)
read (IO_unit) junk_f8  ! Skip this record (already read in the useful part)
if (metadata_version == 1) then
   read (IO_unit) target_Ls, start_UT
else
   read (IO_unit) target_Ls, start_UT, Ls_curr(ifn), UT_curr(ifn),  &
        EOT_curr(ifn), sdec_curr(ifn), sdist_curr(ifn)
end if
read (IO_unit) junki, junki, map_projection
read (IO_unit) junkf    ! Skip this record
read (IO_unit) ztop, junki
read (IO_unit) junki    ! Skip this record
read (IO_unit) junki    ! Skip this record

ng = n_state_f_grids(ifn)

nxp(:,ifn) = 0; nyp(:,ifn) = 0; nzp(:,ifn) = 0  ! Initialize
read (IO_unit) nxp(1:ng,ifn), nyp(1:ng,ifn), nzp(1:ng,ifn)

read (IO_unit) junki    ! Skip this record
read (IO_unit) junki    ! Skip this record
read (IO_unit) junki    ! Skip this record
read (IO_unit) junki    ! Skip this record
read (IO_unit) junkf    ! Skip this record

plat(:,ifn) = -999._f4; plon(:,ifn) = -999._f4  ! Initialize
read (IO_unit) plat(1:ng,ifn), plon(1:ng,ifn) 

read (IO_unit) junkf    ! Skip this record
read (IO_unit) junkf    ! Skip this record

if (phase == 1) then

   read (IO_unit) junkf    ! Skip this record
   read (IO_unit) junkf    ! Skip this record
   read (IO_unit) SBPS_Tlev(1:(nzg(ifn)+nzs(ifn)),ifn)

   xm(:,:,ifn) = 0._f4; xt(:,:,ifn) = 0._f4    !
   ym(:,:,ifn) = 0._f4; yt(:,:,ifn) = 0._f4    ! Initialize
   zm(:,:,ifn) = 0._f4; zt(:,:,ifn) = 0._f4    !
   dzm(:,:,ifn) = 0._f4; dzt(:,:,ifn) = 0._f4  !
   do igd=1,n_state_f_grids(ifn)
      n1 = nzp(igd,ifn); n2 = nxp(igd,ifn); n3 = nyp(igd,ifn)
      read (IO_unit) xm(1:n2,igd,ifn), xt(1:n2,igd,ifn)
      read (IO_unit) ym(1:n3,igd,ifn), yt(1:n3,igd,ifn)
      read (IO_unit) zm(1:n1,igd,ifn), zt(1:n1,igd,ifn)
      read (IO_unit) dzm(1:n1,igd,ifn), dzt(1:n1,igd,ifn)
      read (IO_unit) junkf    ! Skip this record
      read (IO_unit) junkf    ! Skip this record
      read (IO_unit) junkf    ! Skip this record
   end do

   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junki    ! Skip this record

   read (IO_unit) junki    ! Skip this record

   read (IO_unit) junkf    ! Skip this record
   read (IO_unit) junkf    ! Skip this record
   read (IO_unit) junkf    ! Skip this record

      ! By default, assume these are not present:
   dust_present = .FALSE.; H2Oi_present = .FALSE.; CO2i_present = .FALSE.
   read (IO_unit) junki  ! iaertog
   if (junki > 0) then   ! Only read if run configured for aerosol microphysics
      if (metadata_version < 3) then
         read (IO_unit) nelements, ngroups, max_nbins
         RTc_tog = 2; dust_op_case = 3  ! Default values
      else
         read (IO_unit) nelements, ngroups, max_nbins, RTc_tog, dust_op_case
      end if
      naerspc_rad = ngroups  ! Assume all present groups may be rad_active

      if (.not. allocated(nbins)) call manage_misc_memory(dim_aerosols=.TRUE.)

      read (IO_unit) nbins(1:ngroups)
      read (IO_unit) groupname(1:ngroups)
      read (IO_unit) ienconc(1:ngroups)
      read (IO_unit) rhoelem(1:nelements)
      read (IO_unit) elemname(1:nelements)
      read (IO_unit) rmass(1:max_nbins,1:ngroups)
      read (IO_unit) r(1:max_nbins,1:ngroups)
      read (IO_unit) rup(1:max_nbins,1:ngroups)
      read (IO_unit) rlow(1:max_nbins,1:ngroups)
      !... rhop() and vol() are computed elsewhere, as needed

!^print *,'rmass-dust',rmass(1:nbins(1),1)
!^print *,'rmass-H2Oi',rmass(1:nbins(2),2)
!^print *,'r-dust',r(1:nbins(1),1)
!^print *,'r-H2Oi',r(1:nbins(2),2)
!^print *,'rhoelem',rhoelem(1:nelements)

      ! Process this config data:

      lw_iwbeg = 1                ! "longwave" group   
      lw_iwend = nlw_wavlnrng     !
      sw_iwbeg = lw_iwend+1      ! "shortwave" group
      sw_iwend = ntot_wavlnrng   !

      dataset_path_bspec = default_input_data_dir_bspec

      if (.not.allocated(spc_ctype)) then
         allocate(spc_nbins(naerspc_rad), spc_ctype(naerspc_rad),  &
              spc_ig(naerspc_rad),  &
              spc_xsarea_fact(maxval(nbins(1:ngroups)),naerspc_rad), stat=e)
         if (e /= 0) call illegal_alloc_stop(for_allocation, procedure_name,  &
              '1')
      end if

      do ip=1,naerspc_rad
         ig = ip
         spc_ig(ip) = ig
         spc_nbins(ip) = nbins(ig)
         if (groupname(ig)(1:1) == 'D') then 
            dust_present = .TRUE.
            spc_ctype(ip) = aeradID_dust
            dust_ig = ig
            dustnc_ie = ienconc(ig)
         else if (groupname(ig)(1:1) == 'H') then 
            H2Oi_present = .TRUE.
            spc_ctype(ip) = aeradID_H2Oi
            H2Oi_ig = ig
            H2Oinc_ie = ienconc(ig)
         else if (groupname(ig)(1:1) == 'C') then 
            CO2i_present = .TRUE.
            spc_ctype(ip) = aeradID_CO2i
            CO2i_ig = ig            
            CO2inc_ie = ienconc(ig)
         end if
      end do

      ! Particle mass densities (for each group) [g/cm^3]:
      do ig=1,ngroups
         ie = ienconc(ig) 
         do ib=1,nbins(ig)
            do k=1,maxnzp
               rhop(k,ib,ig) = rhoelem(ie)
               vol(ib,ig) = rmass(ib,ig)/rhop(1,ib,ig)
            end do
         end do
      end do

   end if

   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junkf    ! Skip this record
   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junki    ! Skip this record
   read (IO_unit) junki    ! Skip this record
   do ig=1,n_state_f_grids(ifn)
      read (IO_unit) junkf    ! Skip this record
   end do

   read (IO_unit) Ls_of_periapse, orbital_eccentricity, axial_obliquity,  &
        AU_to_m_knob

end if

end subroutine postp_mine_state_f_md


!```````````````````````````````````````````````````````````````````
subroutine process_dim_str(ID_str, dim_str, i_list, t_list, tag)

!+ Processes dimension configuration string

use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL
use postp_namelist_mod, only: parse_dim_str

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: dim_str, ID_str
integer, dimension(:), intent(INOUT) :: i_list
integer, optional, intent(IN) :: tag
integer, optional, dimension(:), intent(INOUT) :: t_list

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'process_dim_str'

integer :: ic_end, ic_beg
logical :: tagging

!=== Executable statements:

if (verify(dim_str, '0123456789,: ') /= 0) call exception(procedure_name,  &
     FATAL, 'Invalid '//trim(ID_str)//' list and/or range syntax')

tagging = ( present(tag) .and. present(t_list) )
if ((present(tag) .or. present(t_list)) .and. (.not. tagging))  &
     call exception(procedure_name, FATAL,  &
     'Optional arguments tag and t_list must both be provided.')

ic_beg = 1; ic_end = len_trim(dim_str)
if (tagging) then
   call parse_dim_str(dim_str(ic_beg:ic_end), i_list,  &
        tag_list=t_list, tag=tag)
else
   call parse_dim_str(dim_str(ic_beg:ic_end), i_list)
end if

end subroutine process_dim_str


end module postp_init_mod
