!```````````````````````````````````````````````````````````````````
module postp_namelist_mod

!+ Module that contains routines related to the postp namelist

use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN
use univ_integer_time_mod, only: time_obj
use namelist_maxdims_mod, only: nml__strtoklen_max, nml_fn_strtoklen_max,  &
     nml_max_n_sounding, nml_max_n_grids
use postp_main_memory_mod, only: pp_description, pp_action,  &
     cfg_f_inc, input_state_file_pfx, output_pp_file_pfx, ztrnsfm,  &
     pfact_xa, pfact_ya, pfact_za, pfact_xb, pfact_yb, pfact_zb, ttag_fmt,  &
     default_input_data_dir

implicit none

character(len=18), parameter :: postp_nml_fn = 'auto_gen-postp.nml'

!== &namelist_info
integer, parameter :: n_chars_nml_version = 4, n_chars_nml_is_for = 19
character(len=n_chars_nml_is_for) :: namelist_is_for
character(len=n_chars_nml_version) :: namelist_version_common
character(len=MAX_MESSAGE_LEN) :: namelist_notes_common
namelist /namelist_info/ namelist_is_for, namelist_version_common,  &
     namelist_notes_common

!== &pp_basic_control
character(len=nml__strtoklen_max) :: time1_, time2_
character(len=nml_fn_strtoklen_max), dimension(nml_max_n_grids) :: dcfg_
type(time_obj) :: time1_Time, time2_Time
namelist /pp_basic_control/ pp_description, pp_action, input_state_file_pfx,  &
     output_pp_file_pfx, dcfg_, time1_, time2_, cfg_f_inc,  &
     default_input_data_dir

!== &pp_vars
character(len=nml__strtoklen_max), dimension(nml_max_n_sounding) :: vars_
namelist /pp_vars/ vars_

!== &pp_spatial_config
namelist /pp_spatial_config/ ztrnsfm

!== &pp_OPENDX_config
namelist /pp_OPENDX_config/ pfact_xa, pfact_ya, pfact_za, pfact_xb, pfact_yb,  &
     pfact_zb, ttag_fmt


contains


!```````````````````````````````````````````````````````````````````
subroutine nml_info_check(namelist_fh)

!+ Check '&namelist_info' namelist block for errors 

use univ_std_fileIO_mod, only: find_file_IOunit
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh    ! File I/O unit of open namelist file

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'nml_info_check'

character(len=n_chars_nml_is_for) :: base_app_to_match = 'postp_for_PRAMS'
character(len=n_chars_nml_version) :: base_version_to_match = '0004'

integer :: cn, IO_status
character(len=MAX_MESSAGE_LEN) :: msg

!=== Executable statements:

!== Input '&namelist_info' namelist block; check for errors:

read(namelist_fh, nml=namelist_info, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &namelist_info not found or is damaged.')

cn = max(1, len_trim(namelist_is_for))
if (namelist_is_for(1:cn) /= base_app_to_match) then
   write(msg, '(5a)')  'Input namelist application (', namelist_is_for(1:cn),  &
        ') is incompatible with the necessary application (',  &
        base_app_to_match, ').'
   call exception(procedure_name, FATAL, trim(msg))
end if
cn = max(1, len_trim(namelist_version_common))
if (namelist_version_common(1:cn) /= base_version_to_match) then
   write(msg, '(5a)')  'Input namelist version (',  &
        namelist_version_common(1:cn),  &
        ') is incompatible with the necessary version (',  &
        base_version_to_match, ').'
   call exception(procedure_name, FATAL, trim(msg))
end if

end subroutine nml_info_check


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_pp_basic_control(namelist_fh)

!+ Processing related to items from namelist &pp_basic_control

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_math_constants_mod, only: ONE_f8
use univ_integer_time_mod, only: time_eqv_fp_seconds, fp_seconds_eqv_time,  &
     set_time
use univ_namelist_utils_mod, only: gen_time_strrep_to_Time, nml_num_value_check
use main_memory_mod, only: module_tps, zero_Time, morbpd_Time, mnsdy_Time
use postp_main_memory_mod, only: manage_misc_memory,  &
     time1, time2, n_dcfg, dcfg  ! and others already used by this module
use body_specific_constants_mod, only: morbpd, mnsdy

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'proc_nml_pp_basic_control'

integer :: IO_status
logical :: error
logical, dimension(size(dcfg_, 1)) :: mask

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

pp_description = 'No description provided.'
pp_action = '__NULL__'
input_state_file_pfx = '__NULL__'
output_pp_file_pfx = '__NULL__'
dcfg_ = '__NULL__'
time1_ = '__NULL__'; time2_ = '__NULL__'
cfg_f_inc = -999
default_input_data_dir = '__NULL__'

!== Attempt to read the namelist:

read(namelist_fh, nml=pp_basic_control, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &pp_basic_control not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

   ! Compute/set some important time parameters:
zero_Time = set_time(0, 0, module_tps)
mnsdy_Time = fp_seconds_eqv_time(mnsdy, module_tps)
morbpd_Time = fp_seconds_eqv_time(morbpd, module_tps)

!... 'pp_description' may contain nearly anything

select case (pp_action)
case ('NETCDF', 'GRADS', 'OPENDX', 'FLD_EXT', 'ARL')
   continue
case default
   call exception(procedure_name, FATAL,  &
        'Improper value for namelist variable PP_ACTION')
end select

if (trim(input_state_file_pfx) == '__NULL__') call exception(procedure_name,  &
     FATAL, 'Improper value for namelist variable INPUT_STATE_FILE_PFX')
if (trim(output_pp_file_pfx) == '__NULL__') call exception(procedure_name,  &
     FATAL, 'Improper value for namelist variable OUTPUT_PP_FILE_PFX')

mask = ( dcfg_ /= '__NULL__' ); n_dcfg = count(mask)
if (n_dcfg == 0) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable DCFG_')
call manage_misc_memory(dim_n_dcfg=.TRUE.)
dcfg = dcfg_(1:n_dcfg)

call gen_time_strrep_to_Time(trim(time1_), time1_Time, error, mnsdy_Time,  &
     morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable TIME1_')
time1 = time_eqv_fp_seconds(time1_Time)

if (trim(time2_) == 'END') then
   time2 = -ONE_f8   ! Special case
else   ! Usual case
   call gen_time_strrep_to_Time(trim(time2_), time2_Time, error, mnsdy_Time,  &
        morbpd_Time, module_tps)
   if (error) call exception(procedure_name, FATAL,  &
        'Improper value for namelist variable TIME2_')
   time2 = time_eqv_fp_seconds(time2_Time)
end if

call nml_num_value_check(cfg_f_inc, 'CFG_F_INC', ge=0)

if (trim(default_input_data_dir) == '__NULL__') call exception(  &
     procedure_name, FATAL,  &
     'Improper value for namelist variable DEFAULT_INPUT_DATA_DIR')

end subroutine proc_nml_pp_basic_control


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_pp_vars(namelist_fh)

!+ Processing related to items from namelist &pp_vars

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use postp_main_memory_mod, only:  &
     cfg_n_ov, cfg_vname,  &  ! and others already used by this module
     manage_misc_memory

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'proc_nml_pp_vars'

integer :: IO_status
logical, dimension(size(vars_, 1)) :: mask

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

vars_ = '__NULL__'

!== Attempt to read the namelist:

read(namelist_fh, nml=pp_vars, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &pp_vars not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

mask = ( vars_ /= '__NULL__' ); cfg_n_ov = count(mask)
if (cfg_n_ov == 0) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable VARS_')
call manage_misc_memory(dim_cfg_n_ov=.TRUE.)

cfg_vname(1:cfg_n_ov) = vars_(1:cfg_n_ov)

end subroutine proc_nml_pp_vars


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_pp_spatial_config(namelist_fh)

!+ Processing related to items from namelist &pp_spatial_config

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_namelist_utils_mod, only: nml_num_value_check
!use postp_main_memory_mod, only:  &
      ! and others already used by this module

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'proc_nml_pp_spatial_config'

integer :: IO_status

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

ztrnsfm = -1

!== Attempt to read the namelist:

read(namelist_fh, nml=pp_spatial_config, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &pp_spatial_config not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

call nml_num_value_check(ztrnsfm, 'ZTRNSFM', 'and', ge=1, le=2)

end subroutine proc_nml_pp_spatial_config


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_pp_OPENDX_config(namelist_fh)

!+ Processing related to items from namelist &pp_OPENDX_config

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_numtype_constants_mod, only: huge_f4
use univ_namelist_utils_mod, only: nml_num_value_check
!use postp_main_memory_mod, only:  &
      ! and others already used by this module

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'proc_nml_pp_OPENDX_config'

integer :: IO_status

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

pfact_xa = huge_f4; pfact_ya = huge_f4; pfact_za = huge_f4
pfact_xb = huge_f4; pfact_yb = huge_f4; pfact_zb = huge_f4
ttag_fmt = '__NULL__'

!== Attempt to read the namelist:

read(namelist_fh, nml=pp_OPENDX_config, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &pp_OPENDX_config not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

call nml_num_value_check(pfact_xa, 'PFACT_XA', 'and', gt=-huge_f4, lt=huge_f4)
call nml_num_value_check(pfact_ya, 'PFACT_YA', 'and', gt=-huge_f4, lt=huge_f4)
call nml_num_value_check(pfact_za, 'PFACT_ZA', 'and', gt=-huge_f4, lt=huge_f4)
call nml_num_value_check(pfact_xb, 'PFACT_XB', 'and', gt=-huge_f4, lt=huge_f4)
call nml_num_value_check(pfact_yb, 'PFACT_YB', 'and', gt=-huge_f4, lt=huge_f4)
call nml_num_value_check(pfact_zb, 'PFACT_ZB', 'and', gt=-huge_f4, lt=huge_f4)

select case (ttag_fmt)
case ('sec', 'b_m', 'b_h', 'b_d')
   continue
case default
   call exception(procedure_name, FATAL,  &
        'Improper value for namelist variable TTAG_FMT')
end select

end subroutine proc_nml_pp_OPENDX_config


!```````````````````````````````````````````````````````````````````
subroutine parse_dim_str(dim_str, idx_list, tag_list, tag)

!+ Parses config string; assigns parsed values

use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN
use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: dim_str
integer, dimension(:), intent(INOUT) :: idx_list
integer, optional, intent(IN) :: tag
integer, dimension(:), optional, intent(INOUT) :: tag_list

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'parse_dim_str'

character(len=MAX_MESSAGE_LEN) :: err_msg
integer :: ios, ic_end, ic_beg, ic_absend, ic_delim, idx, n_idx_max, ic_le_b,  &
     ic_le_e, ic_comma
integer, dimension(3) :: i_range
logical :: tagging

!=== Executable statements:

tagging = ( present(tag_list) .and. present(tag) )

ic_absend = len_trim(dim_str)
n_idx_max = size(idx_list, 1)

ic_le_b = 1; ic_le_e = ic_absend  ! At first, assume a single "list entry"

ic_comma = index(dim_str(ic_le_b:ic_absend), ',')
if (ic_comma > 0) ic_le_e = ic_le_b+ic_comma-2
do
   err_msg = 'Improper range format for namelist string ('//  &
        trim(dim_str(ic_le_b:ic_le_e))//')'
   i_range(3) = 1  ! Set to default value

   ic_delim = index(dim_str(ic_le_b:ic_le_e), ':')   
   if (ic_delim > 0) then    ! Seems to contain a "range"
      ic_beg = ic_le_b; ic_end = ic_le_b+ic_delim-2
      read (dim_str(ic_beg:ic_end), *, iostat=ios) i_range(1)
      if (ios /= 0) call exception(procedure_name, FATAL, err_msg)

      ic_beg = ic_end+2
      ic_delim = index(dim_str(ic_beg:ic_le_e), ':')
      if (ic_delim > 0) then
         read (dim_str(ic_beg+ic_delim:ic_le_e), *, iostat=ios) i_range(3)
         if (ios /= 0) call exception(procedure_name, FATAL, err_msg)
         ic_end = ic_beg+ic_delim-2
      else
         ic_end = ic_le_e
      end if

      read (dim_str(ic_beg:ic_end), *, iostat=ios) i_range(2)
      if (ios /= 0) call exception(procedure_name, FATAL, err_msg)
   else       ! Possibly a unary index?
      ic_beg = ic_le_b; ic_end = ic_le_e
      read (dim_str(ic_beg:ic_end), *, iostat=ios) i_range(1)
      if (ios /= 0) call exception(procedure_name, FATAL, err_msg)
      i_range(2) = i_range(1)
   end if

   if (minval(i_range(1:3)) < 1) then
      err_msg = trim(err_msg)//'; no values < 1 permitted.'
      call exception(procedure_name, FATAL, err_msg)
   end if

   do idx=i_range(1),min(n_idx_max, i_range(2)),i_range(3)
      idx_list(idx) = idx
      if (tagging) tag_list(idx) = tag
   end do

   if (ic_comma < 1) exit   ! No further substring(s) to process

   ic_le_b = min(ic_le_b+ic_comma, ic_absend); ic_le_e = ic_absend
   ic_comma = index(dim_str(ic_le_b:ic_le_e), ',')
   if (ic_comma > 0) ic_le_e = ic_le_b+ic_comma-2
end do

end subroutine parse_dim_str


end module postp_namelist_mod
