!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
module polar_intrp_extrap_mod

!+ Module to facilitate the calculation (interpolation and/or extrapolation) of
!   "missing" polar values in a given GCM field

!~ 06.05.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4

implicit none

private

interface calc_polarvals_mpt
   module procedure calc_polarvals_mpt_j, calc_polarvals_mpt_ij,  &
        calc_polarvals_mpt_ijm, calc_polarvals_mpt_ijmn,  &
        calc_polarvals_mpt_ijmno
end interface

public :: calc_polarvals_mpt


contains


!``````````````````````````````````````````````````````````````````
subroutine calc_polarvals_mpt_j(field)

!+ Calculate extrapolated and averaged field value at the poles (1D j array)

!~ 06.05.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

real(f4), dimension(:), intent(INOUT) :: field

!== Local declarations:

integer :: n_lat
real(f8) :: sumN, sumS

!=== Executable statements:

n_lat = size(field, 1)

sumS = field(2)+(field(2)-field(3))
sumN = field(n_lat-1)+(field(n_lat-1)-field(n_lat-2))

field(1) = real(sumS, kind=f4)
field(n_lat) = real(sumN, kind=f4)

end subroutine calc_polarvals_mpt_j


!``````````````````````````````````````````````````````````````````
subroutine calc_polarvals_mpt_ij(field)

!+ Calculate extrapolated and averaged field value at the poles (2D i,j array)

!~ 06.05.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

real(f4), dimension(:,:), intent(INOUT) :: field

!== Local declarations:

integer :: i, n_lat, n_lon
real(f8) :: sumN, sumS

!=== Executable statements:

n_lon = size(field, 1); n_lat = size(field, 2)

sumN = 0._f8; sumS = 0._f8
do i=1,n_lon
   sumS = sumS + field(i,2)+(field(i,2)-field(i,3))
   sumN = sumN + field(i,n_lat-1)+(field(i,n_lat-1)-field(i,n_lat-2))
end do

field(:,1) = real(sumS/real(n_lon, kind=f8), kind=f4)
field(:,n_lat) = real(sumN/real(n_lon, kind=f8), kind=f4)

end subroutine calc_polarvals_mpt_ij


!``````````````````````````````````````````````````````````````````
subroutine calc_polarvals_mpt_ijm(field)

!+ Calculate extrapolated and averaged field value at the poles (3D i,j,m array)

!~ 06.05.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

real(f4), dimension(:,:,:), intent(INOUT) :: field

!== Local declarations:

integer :: i, n_lat, n_lon, n_m, m
real(f8) :: sumN, sumS

!=== Executable statements:

n_lon = size(field, 1); n_lat = size(field, 2); n_m = size(field, 3)

do m=1,n_m
   sumN = 0._f8; sumS = 0._f8
   do i=1,n_lon
      sumS = sumS + field(i,2,m)+(field(i,2,m)-field(i,3,m))
      sumN = sumN + field(i,n_lat-1,m)+(field(i,n_lat-1,m)-field(i,n_lat-2,m))
   end do

   field(:,1,m) = real(sumS/real(n_lon, kind=f8), kind=f4)
   field(:,n_lat,m) = real(sumN/real(n_lon, kind=f8), kind=f4)
end do

end subroutine calc_polarvals_mpt_ijm


!``````````````````````````````````````````````````````````````````
subroutine calc_polarvals_mpt_ijmn(field)

!+ Calculate extrapolated and averaged field value at poles (4D i,j,m,n array)

!~ 06.05.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

real(f4), dimension(:,:,:,:), intent(INOUT) :: field

!== Local declarations:

integer :: i, n_lat, n_lon, n_m, m, n, n_n
real(f8) :: sumN, sumS

!=== Executable statements:

n_lon = size(field, 1); n_lat = size(field, 2); n_m = size(field, 3)
n_n = size(field, 4)

do n=1,n_n
   do m=1,n_m
      sumN = 0._f8; sumS = 0._f8
      do i=1,n_lon
         sumS = sumS + field(i,2,m,n)+(field(i,2,m,n)-field(i,3,m,n))
         sumN = sumN + field(i,n_lat-1,m,n)+(field(i,n_lat-1,m,n)-  &
              field(i,n_lat-2,m,n))
      end do

      field(:,1,m,n) = real(sumS/real(n_lon, kind=f8), kind=f4)
      field(:,n_lat,m,n) = real(sumN/real(n_lon, kind=f8), kind=f4)
   end do
end do

end subroutine calc_polarvals_mpt_ijmn


!``````````````````````````````````````````````````````````````````
subroutine calc_polarvals_mpt_ijmno(field)

!+ Calculate extrapolated and averaged field value at poles (5D i,j,m,n,o array)

!~ 07.05.2012 (last modified; TIM)

use univ_kind_defs_mod, only: f4, f8

implicit none

!=== Argument declarations:

real(f4), dimension(:,:,:,:,:), intent(INOUT) :: field

!== Local declarations:

integer :: i, n_lat, n_lon, n_m, m, n, n_n, o, n_o
real(f8) :: sumN, sumS

!=== Executable statements:

n_lon = size(field, 1); n_lat = size(field, 2); n_m = size(field, 3)
n_n = size(field, 4); n_o = size(field, 5)

do o=1,n_o
   do n=1,n_n
      do m=1,n_m
         sumN = 0._f8; sumS = 0._f8
         do i=1,n_lon
            sumS = sumS + field(i,2,m,n,o)+(field(i,2,m,n,o)-field(i,3,m,n,o))
            sumN = sumN + field(i,n_lat-1,m,n,o)+(field(i,n_lat-1,m,n,o)-  &
                 field(i,n_lat-2,m,n,o))
         end do

         field(:,1,m,n,o) = real(sumS/real(n_lon, kind=f8), kind=f4)
         field(:,n_lat,m,n,o) = real(sumN/real(n_lon, kind=f8), kind=f4)
      end do
   end do
end do

end subroutine calc_polarvals_mpt_ijmno


end module polar_intrp_extrap_mod
